Package: mosaic
Type: Package
Title: Project MOSAIC Statistics and Mathematics Teaching Utilities
Version: 1.10.2
Description: Data sets and utilities from Project MOSAIC (<http://www.mosaic-web.org>) used
    to teach mathematics, statistics, computation and modeling.  Funded by the
    NSF, Project MOSAIC is a community of educators working to tie together
    aspects of quantitative work that students in science, technology,
    engineering and mathematics will need in their professional lives, but
    which are usually taught in isolation, if at all.
Authors@R: c(
    person(given = "Randall",  family = "Pruim", email = "rpruim@gmail.com", role = c("aut", "cre")), 
    person(given = "Daniel T.",   family = "Kaplan", email = "dtkaplan@macalester.edu", role = "aut"), 
    person(given = "Nicholas J.", family = "Horton", email = "nhorton@amherst.edu", role = "aut")
    )
Depends: R (>= 4.1),
Imports: dplyr, tibble, lattice (>= 0.20-21), ggformula, mosaicData,
        Matrix, mosaicCore (>= 0.7.0), ggplot2, rlang (>= 0.4.7),
        purrr, MASS, grid, tidyr, methods, utils
Suggests: ggstance, ggridges, vdiffr, lubridate, magrittr, NHANES,
        RCurl, sp, vcd, testthat (>= 3.0.0), knitr, tools, parallel,
        mapproj, rgl, rmarkdown, covr, formatR, palmerpenguins,
        ggrepel, readr, ggdendro, gridExtra, splines, latticeExtra,
        glue, broom, leaflet
Enhances: manipulate
VignetteBuilder: knitr
License: GPL (>= 2)
LazyLoad: yes
LazyData: yes
URL: https://github.com/ProjectMOSAIC/mosaic,
        https://www.mosaic-web.org/mosaic/
BugReports: https://github.com/ProjectMOSAIC/mosaic/issues
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-13 01:53:14 UTC; rpruim
Author: Randall Pruim [aut, cre],
  Daniel T. Kaplan [aut],
  Nicholas J. Horton [aut]
Maintainer: Randall Pruim <rpruim@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 07:30:21 UTC
Built: R 4.5.2; ; 2026-05-13 08:16:56 UTC; unix

Package: mosaic
Type: Package
Title: Project MOSAIC Statistics and Mathematics Teaching Utilities
Version: 1.10.2
Description: Data sets and utilities from Project MOSAIC (<http://www.mosaic-web.org>) used
    to teach mathematics, statistics, computation and modeling.  Funded by the
    NSF, Project MOSAIC is a community of educators working to tie together
    aspects of quantitative work that students in science, technology,
    engineering and mathematics will need in their professional lives, but
    which are usually taught in isolation, if at all.
Authors@R: c(
    person(given = "Randall",  family = "Pruim", email = "rpruim@gmail.com", role = c("aut", "cre")), 
    person(given = "Daniel T.",   family = "Kaplan", email = "dtkaplan@macalester.edu", role = "aut"), 
    person(given = "Nicholas J.", family = "Horton", email = "nhorton@amherst.edu", role = "aut")
    )
Depends: R (>= 4.1),
Imports: dplyr, tibble, lattice (>= 0.20-21), ggformula, mosaicData,
        Matrix, mosaicCore (>= 0.7.0), ggplot2, rlang (>= 0.4.7),
        purrr, MASS, grid, tidyr, methods, utils
Suggests: ggstance, ggridges, vdiffr, lubridate, magrittr, NHANES,
        RCurl, sp, vcd, testthat (>= 3.0.0), knitr, tools, parallel,
        mapproj, rgl, rmarkdown, covr, formatR, palmerpenguins,
        ggrepel, readr, ggdendro, gridExtra, splines, latticeExtra,
        glue, broom, leaflet
Enhances: manipulate
VignetteBuilder: knitr
License: GPL (>= 2)
LazyLoad: yes
LazyData: yes
URL: https://github.com/ProjectMOSAIC/mosaic,
        https://www.mosaic-web.org/mosaic/
BugReports: https://github.com/ProjectMOSAIC/mosaic/issues
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-13 01:53:14 UTC; rpruim
Author: Randall Pruim [aut, cre],
  Daniel T. Kaplan [aut],
  Nicholas J. Horton [aut]
Maintainer: Randall Pruim <rpruim@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 07:30:21 UTC
Built: R 4.6.0; ; 2026-05-13 08:56:43 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(TukeyHSD,formula)
S3method(TukeyHSD,lm)
S3method(as.xtabs,data.frame)
S3method(as.xtabs,matrix)
S3method(boott,do.data.frame)
S3method(chisq,default)
S3method(chisq,htest)
S3method(chisq,table)
S3method(coef,nlsfunction)
S3method(confint,data.frame)
S3method(confint,do.data.frame)
S3method(confint,do.tbl_df)
S3method(confint,htest)
S3method(confint,numeric)
S3method(confint,summary.lm)
S3method(cor_test,default)
S3method(cor_test,formula)
S3method(cull_for_do,aggregated.stat)
S3method(cull_for_do,anova)
S3method(cull_for_do,aov)
S3method(cull_for_do,cointoss)
S3method(cull_for_do,default)
S3method(cull_for_do,fitdistr)
S3method(cull_for_do,htest)
S3method(cull_for_do,lm)
S3method(cull_for_do,lme)
S3method(cull_for_do,matrix)
S3method(cull_for_do,table)
S3method(do,default)
S3method(do,numeric)
S3method(factorize,character)
S3method(factorize,data.frame)
S3method(factorize,default)
S3method(factorize,numeric)
S3method(fortify,SpatialPolygonsDataFrame)
S3method(fortify,TukeyHSD)
S3method(fortify,hclust)
S3method(fortify,summary.glm)
S3method(fortify,summary.lm)
S3method(model,nlsfunction)
S3method(mplot,TukeyHSD)
S3method(mplot,data.frame)
S3method(mplot,default)
S3method(mplot,hclust)
S3method(mplot,lm)
S3method(mplot,summary.glm)
S3method(mplot,summary.lm)
S3method(plot,freqpolygon)
S3method(plotCumfreq,default)
S3method(plotCumfreq,formula)
S3method(plotModel,default)
S3method(plotModel,parsedModel)
S3method(print,cointoss)
S3method(print,oddsRatio)
S3method(print,relrisk)
S3method(print,repeater)
S3method(prop_test,character)
S3method(prop_test,default)
S3method(prop_test,factor)
S3method(prop_test,formula)
S3method(prop_test,logical)
S3method(prop_test,numeric)
S3method(pval,htest)
S3method(rescale,character)
S3method(rescale,data.frame)
S3method(rescale,default)
S3method(rescale,factor)
S3method(rescale,numeric)
S3method(sample,data.frame)
S3method(sample,default)
S3method(sample,factor)
S3method(sample,lm)
S3method(sample,matrix)
S3method(solve,formula)
S3method(stat,htest)
S3method(stat,uneval)
S3method(summary,nlsfunction)
S3method(summary,oddsRatio)
S3method(summary,relrisk)
S3method(t_test,default)
S3method(t_test,formula)
S3method(value,default)
S3method(value,integrate)
export(Broyden)
export(CIAdata)
export(CIsim)
export(Do)
export(IQR)
export(MAD)
export(MAD_)
export(SAD)
export(SAD_)
export(adapt_seq)
export(aggregatingFunction1)
export(aggregatingFunction1or2)
export(aggregatingFunction2)
export(as.xtabs)
export(ashplot)
export(bargraph)
export(binom.test)
export(cdata)
export(cdata_f)
export(cdata_v)
export(cdist)
export(chisq)
export(cnorm)
export(col.mosaic)
export(compare)
export(compareMean)
export(compareProportion)
export(condition)
export(connector)
export(cor)
export(cor.test)
export(cor_test)
export(count)
export(counts)
export(cov)
export(cross)
export(ct)
export(cull_for_do)
export(ddata)
export(ddata_f)
export(ddata_v)
export(deal)
export(deg2rad)
export(deltaMethod)
export(derivedFactor)
export(derivedVariable)
export(design_plot)
export(dfapply)
export(diffmean)
export(diffprop)
export(do)
export(docFile)
export(dot)
export(dotPlot)
export(dpqrdist)
export(ediff)
export(expandFun)
export(factorise)
export(factorize)
export(fav_stats)
export(favstats)
export(fetchData)
export(fetchGapminder)
export(fetchGapminder1)
export(fetchGoogle)
export(findZeros)
export(findZerosMult)
export(fitModel)
export(fitSpline)
export(fitdistr)
export(fivenum)
export(fractions)
export(freqpoly)
export(freqpolygon)
export(getVarFormula)
export(googleMap)
export(gwm)
export(hist2freqpolygon)
export(ilogit)
export(inferArgs)
export(inspect)
export(iqr)
export(is.wholenumber)
export(ladd)
export(latlon2xyz)
export(leaflet_map)
export(lhs)
export(linearModel)
export(logit)
export(lonlat2xyz)
export(mMap)
export(mPlot)
export(mScatter)
export(mUSMap)
export(mUniplot)
export(mWorldMap)
export(maggregate)
export(makeColorscheme)
export(makeFun)
export(makeMap)
export(mat)
export(max)
export(mean)
export(mean_)
export(median)
export(mid)
export(min)
export(mm)
export(model)
export(mosaic.getOption)
export(mosaic.options)
export(mosaic.par.get)
export(mosaic.par.set)
export(mosaicLatticeOptions)
export(mplot)
export(msummary)
export(n_missing)
export(nflip)
export(ntiles)
export(oddsRatio)
export(orrr)
export(panel.ashplot)
export(panel.cumfreq)
export(panel.dotPlot)
export(panel.freqpolygon)
export(panel.levelcontourplot)
export(panel.lmbands)
export(panel.plotFun)
export(panel.plotFun1)
export(panel.xhistogram)
export(panel.xqqmath)
export(pdata)
export(pdata_f)
export(pdata_v)
export(pdist)
export(perc)
export(percs)
export(perctable)
export(plotCumfreq)
export(plotDist)
export(plotFun)
export(plotModel)
export(plotPoints)
export(prepanel.cumfreq)
export(prepanel.default.ashplot)
export(prepanel.default.freqpolygon)
export(prepanel.xhistogram)
export(prod)
export(project)
export(prop)
export(prop.test)
export(prop1)
export(prop_test)
export(props)
export(proptable)
export(pval)
export(qdata)
export(qdata_f)
export(qdata_v)
export(qdist)
export(quantile)
export(r.squared)
export(rad2deg)
export(rand)
export(range)
export(rdata)
export(rdata_f)
export(rdata_v)
export(read.file)
export(relm)
export(relrisk)
export(resample)
export(rescale)
export(restoreLatticeOptions)
export(rflip)
export(rfun)
export(rgeo)
export(rgeo2)
export(rhs)
export(rlatlon)
export(rlonlat)
export(rpoly2)
export(rspin)
export(rsquared)
export(rstudio_is_available)
export(sample)
export(sd)
export(set.rseed)
export(shuffle)
export(singvals)
export(smoother)
export(sp2df)
export(spliner)
export(standardCountry)
export(standardName)
export(standardState)
export(stat)
export(statTally)
export(sum)
export(surround)
export(swap)
export(t.test)
export(t_test)
export(tally)
export(theme.mosaic)
export(theme_map)
export(update_ci)
export(value)
export(var)
export(vlength)
export(xcbeta)
export(xcbinom)
export(xcchisq)
export(xcf)
export(xcgamma)
export(xcgeom)
export(xchisq.test)
export(xcnbinom)
export(xcnorm)
export(xcpois)
export(xct)
export(xhistogram)
export(xhistogramBreaks)
export(xpbeta)
export(xpbinom)
export(xpchisq)
export(xpf)
export(xpgamma)
export(xpgeom)
export(xpnbinom)
export(xpnorm)
export(xppois)
export(xpt)
export(xqbeta)
export(xqbinom)
export(xqchisq)
export(xqf)
export(xqgamma)
export(xqgeom)
export(xqnbinom)
export(xqnorm)
export(xqpois)
export(xqqmath)
export(xqt)
export(xyz2latlon)
export(zscore)
exportClasses(repeater)
exportMethods("*")
exportMethods(project)
import(ggformula)
import(ggplot2)
import(lattice)
import(methods)
import(mosaicCore)
import(mosaicData)
import(rlang)
importFrom(MASS,fitdistr)
importFrom(MASS,fractions)
importFrom(Matrix,mean)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,do)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,fortify)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,gray)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rgb)
importFrom(grDevices,topo.colors)
importFrom(graphics,hist)
importFrom(graphics,plot)
importFrom(graphics,stem)
importFrom(grid,gpar)
importFrom(grid,grid.layout)
importFrom(grid,grid.lines)
importFrom(grid,grid.points)
importFrom(grid,grid.polyline)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,unit.c)
importFrom(mosaicCore,compare)
importFrom(mosaicCore,condition)
importFrom(mosaicCore,count)
importFrom(mosaicCore,counts)
importFrom(mosaicCore,dfapply)
importFrom(mosaicCore,ediff)
importFrom(mosaicCore,ilogit)
importFrom(mosaicCore,inspect)
importFrom(mosaicCore,lhs)
importFrom(mosaicCore,logit)
importFrom(mosaicCore,makeFun)
importFrom(mosaicCore,mosaic_formula)
importFrom(mosaicCore,mosaic_formula_q)
importFrom(mosaicCore,msummary)
importFrom(mosaicCore,n_missing)
importFrom(mosaicCore,named)
importFrom(mosaicCore,named_among)
importFrom(mosaicCore,nice_names)
importFrom(mosaicCore,perc)
importFrom(mosaicCore,percs)
importFrom(mosaicCore,prop)
importFrom(mosaicCore,prop1)
importFrom(mosaicCore,props)
importFrom(mosaicCore,rhs)
importFrom(mosaicCore,tally)
importFrom(mosaicCore,unnamed)
importFrom(mosaicCore,vector2df)
importFrom(purrr,map_dbl)
importFrom(purrr,possibly)
importFrom(stats,TukeyHSD)
importFrom(stats,addmargins)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,cutree)
importFrom(stats,dbeta)
importFrom(stats,dcauchy)
importFrom(stats,dchisq)
importFrom(stats,dexp)
importFrom(stats,df)
importFrom(stats,dgamma)
importFrom(stats,dist)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,dt)
importFrom(stats,dweibull)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,lsfit)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,nls)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,qqnorm)
importFrom(stats,qt)
importFrom(stats,rbinom)
importFrom(stats,reorder)
importFrom(stats,resid)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,splinefun)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stats,wilcox.test)
importFrom(stats,xtabs)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,str)
importFrom(utils,tail)

This release of NCSA Mosaic includes native support for browsing
data files in NCSA HDF format.  For more information on HDF, see
ftp.ncsa.uiuc.edu in /HDF or URL:
        
  http://www.ncsa.uiuc.edu/SDG/Software/HDF/HDFIntro.html

If you compile Mosaic from source, link to HDF 3.3r1 (or the most
recently announced HDF 3.3 revision at the time you compile) from
ftp.ncsa.uiuc.edu in /HDF.  Note that HDF 3.3 supports UCAR's netCDF
data format transparently.

There are some new X resources for Mosaic 2.0 that customize the HDF
browsing capabilities:

hdfMaxImageDimension
  Maximum size for an inlined HDF image.  If an image dimension
  is larger than this, the image will be scaled down for display
  and that fact (and the original dimensions) will be noted in the
  text description next to the inlined image.  Default 400.
hdfMaxDisplayedDatasets
  Maximum number of datasets that will be displayed on a toplevel
  page (additional datasets will be placed on another page).
  Default 15.
hdfMaxDisplayedAttributes
  Maximum number of attributes that will be displayed on a toplevel
  page (additional attributes will be placed on another page).
  Default 10.
hdfPowerUser
  If true, less verbose text will be generated.  Default false.

For more information on the HDF browsing support in general, see:

  http://www.ncsa.uiuc.edu/SDG/Software/Mosaic/Docs/hdf-browsing.html

Some example HDF files suitable for browsing are referenced
near the end of the Mosaic demo document.

To talk to human beings about the HDF browsing support...

  The HDF side of things:
    Chris Houck
    chouck@ncsa.uiuc.edu

  The Mosaic side of things:
    Marc Andreessen
    marca@ncsa.uiuc.edu

Please let us know if you are using this HDF support, and also please
let us know of any problems or comments you might have.

---
title: The mosaic package
---

<!-- [![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/mosaic)](http://cran.r-project.org/package=mosaic) -->
<!-- [![Travis-CI Build Status](https://travis-ci.org/ProjectMOSAIC/mosaic.svg?branch=master)](https://app.travis-ci.com/ProjectMOSAIC/mosaic/) -->

## Purpose and design

The mosaic package is designed to facilitate the use of R in statistics and calculus
instruction by providing a number of functions that (a) make many common tasks fit
into a common template, and (b) simplify some tasks that would otherwise be too
complicated for beginners.

## Installation

You install from CRAN using 

```r
install.packages("mosaic")
```
or from github with

```r
devtools::install_github("ProjectMOSAIC/mosaic", build_vignettes = TRUE)
```
If you want to try out our developmental code (the beta branch), use

```r
devtools::install_github("ProjectMOSAIC/mosaic", ref="beta")
```

Some features of the mosaic package have been split off into auxiliary packages. These include:

* mosaicModel -- implements high-level systems for working with statistical models: effect-size calculation, bootstrapped confidence intervals, prediction error, graphics for models with multiple inputs. The package contains an introductory vignette.
* mosaicCalc -- provides the calculus components of mosaic, including integration, differentiation, and differential equation solving. 

<!-- See *[Modeling-based calculus with -->
<!-- R/mosaic](http://www.mosaic-web.org/wp-content/uploads/2019/01/UMAP-mosaic-calculus.pdf)* -->
<!-- for an instructor-oriented introduction and *[Start R in -->
<!-- Calculus](http://project-mosaic-books.com/?page_id=22)* for a student-facing -->
<!-- guide. -->

Install these packages using `install.packages(c("mosaicCalc", "mosaicModel"))` or from GitHub as described above. 


Updates to the master github repository are more frequent than CRAN updates.
Our beta branch is where we implement bug fixes most quickly and develop new features.
We try to keep it pretty stable, but there may be a few rough edges, missing documentation, etc. while things are in progress.  

If you discover a problem with any version of the package, be sure to let us know so that we can address it.  Post an issue on github or send email to `Rpkgs@mosaic-web.org`.

## Getting Started with mosaic

The package includes several vignettes to help you get started.  One of these 
vignettes 
([*Resources Related to the mosaic package*](https://www.mosaic-web.org/mosaic/articles/mosaic-resources.html)) includes a list of many resources, both within the
package and external to it.  That's a good place to start.

## Getting Help

Need help?  Try posting a question on Stack Overflow using the tag [r-mosaic](https://stackoverflow.com/questions/tagged/r-mosaic).


## Project MOSAIC

Project MOSAIC is a community of educators working to develop a new way to introduce mathematics, statistics, computation and modeling to students in colleges and universities.

Our goal: Provide a broader approach to quantitative studies that provides better support for work in science and technology. The focus of the project is to tie together better diverse aspects of quantitative work that students in science, technology, and engineering will need in their professional lives, but which are today usually taught in isolation, if at all.

* Modeling. The ability to create, manipulate and investigate useful and informative mathematical representations of a real-world situations.
* Statistics. The analysis of variability that draws on our ability to quantify uncertainty and to draw logical inferences from observations and experiment.
* Computation. The capacity to think algorithmically, to manage data on large scales, to visualize and interact with models, and to automate tasks for efficiency, accuracy, and reproducibility.
* Calculus. The traditional mathematical entry point for college and university students and a subject that still has the potential to provide important insights to today's students.

The name MOSAIC reflects the first letters --- M, S, C, C --- of these important components of a quantitative education. Project MOSAIC is motivated by a vision of quantitative education as a mosaic where the basic materials come together to form a complete and compelling picture.

Find out more about Project MOSAIC at [http://www.mosaic-web.org].

NCSA Mosaic for the X Window System Version 2.4
===============================================

Welcome to NCSA Mosaic for the X Window System 2.4!

This README details installation steps.  

README-2.0 details major differences and new features in the 2.x
versions of Mosaic and should be read by anyone migrating from Mosaic
1.x.

More complete information and documentation on NCSA Mosaic is
available online, via NCSA Mosaic.


Binaries
--------

NCSA Mosaic is known to compile on the following platforms:

  SGI (IRIX 4.0.2) 
  SGI (IRIX 5.1.1) 
  IBM (AIX 3.2.4 with IBM X11R5)
  Sun 4 (SunOS 4.1.3 with stock X11R4 and Motif 1.1).
  DECstation 5000-200 (Ultrix).
  DEC Alpha (OSF/1).
  Hewlett Packard HP/UX 7.x, 8.x, 9.x.

Binaries for these platforms (and possibly others) are available on
ftp.ncsa.uiuc.edu in /Mosaic/Mosaic-binaries.

Rumor has it that Mosaic will compile on systems like:

  Dell SVR4
  Sun Solaris 2.x
  SCO Open Desktop
  ...lots others...
  
If you have to make nontrivial changes to NCSA Mosaic to get it to
compile on a particular platform, please send a set of context diffs
(e.g., 'diff -c oldfile newfile') to mosaic-x@ncsa.uiuc.edu.


Installation Instructions
-------------------------

Simply examine the toplevel Makefile, change the appropriate
customizable options, and type 'make'.  

The final result is a single independent executable, src/Mosaic.

(The Makefile.[sun,dec,ibm,alpha,etc.] files are the Makefiles we use
locally for compilation on various platforms; they will almost
certainly NOT WORK for you without modification.  We recommend you
start with the stock Makefile and make modifications as necessary to
avoid confusion.)

There is one tricky thing:

-- You have the option of compiling in support for NCSA HDF, a
   platform-independent hierarchical scientific data format, and NCSA
   DTM, a network-based message-passing protocol useful for exchanging
   scientific data between applications.  If you compile one of them
   in, you should compile both of them in.  

   If you don't already know what HDF and DTM are and want to compile
   Mosaic quickly, forget about them for the time being -- you can
   always recompile later.

   The DTM library is in subdirectory libdtm.  The HDF library must be
   obtained separately from ftp.ncsa.uiuc.edu in /HDF; get version
   3.3r1 or later.  Set the various options in the Makefile to point
   to all the right places, and you should be set.


After You Have Compiled
-----------------------

If you do not normally run Motif on your system (e.g., if you run Sun
OpenWindows instead), then you may get a whole bunch of run-time
errors about translations when you start Mosaic.

If this happens, copy the file XKeysymDB (included in this directory)
to /usr/lib/X11.  (If you compile Mosaic yourself, you may need to
place this file elsewhere, depending on your X configuration.)  See
the FAQ list online for more information.


X Defaults
----------

NCSA Mosaic includes sets of fallback X resources that provide
reasonable screen display properties for three configurations: color,
monochrome, and color SGI.  (Color SGI has its own configuration since
SGI systems commonly use a gamma correction factor of 1.7, which makes
their screens brighter than usual.)

If you compile NCSA Mosaic out of the box, or if you download a binary
from ftp.ncsa.uiuc.edu, the default resources will be for a color
display (or, if you compile on an SGI, the default will be for color
SGI).  See the Makefile for information on how to have monochrome
resources by default.  On the command line, the flags '-mono' and
'-color' allow you to switch resource configurations at runtime.

For your convenience, three corresponding X app-defaults files are
included in this distribution: app-defaults.color,
app-defaults.color-sgi, and app-defaults.mono.


Bug Reports and Comments
------------------------

Bug reports and other comments can be sent to mosaic-x@ncsa.uiuc.edu.

If you find NCSA Mosaic useful or particularly interesting, please
also send us a note -- continued development of this project partially
depends on user feedback and support.


Cheers,

Marc Andreessen and Eric Bina
Software Development Group
National Center for Supercomputing Applications
University of Illinois at Urbana-Champaign
marca@ncsa.uiuc.edu, ebina@ncsa.uiuc.edu

## ----setup,echo=FALSE,message=FALSE,include=FALSE-------------------
#source('setup.R')
require(mosaic)
require(parallel)
require(ggformula)
options(digits=4)
theme_set(theme_bw())
trellis.par.set(theme=col.mosaic())
set.seed(123)
#knit_hooks$set(inline = function(x) {
#	if (is.numeric(x)) return(knitr:::format_sci(x, 'latex'))
#	x = as.character(x)
#	h = knitr:::hilight_source(x, 'latex', list(prompt=FALSE, size='normalsize'))
#	h = gsub("([_#$%&])", "\\\\\\1", h)
#	h = gsub('(["\'])', '\\1{}', h)
#	gsub('^\\\\begin\\{alltt\\}\\s*|\\\\end\\{alltt\\}\\s*$', '', h)
#})
knitr::opts_chunk$set(
	dev="pdf",
	eval=FALSE,
	tidy=FALSE,
	fig.align='center',
	fig.show='hold',
	message=FALSE
	)

## -------------------------------------------------------------------
# apropos()
# ?
# ??
# example()

## -------------------------------------------------------------------
# # basic ops: + - * / ^ ( )
# log(); exp(); sqrt()

## ----highlight=FALSE------------------------------------------------
# log10(); abs(); choose()

## -------------------------------------------------------------------
# goal(y ~ x | z, data = mydata, ...)

## -------------------------------------------------------------------
# favstats()   # mosaic
# tally()      # mosaic
# mean()       # mosaic augmented
# median()     # mosaic augmented
# sd()         # mosaic augmented
# var()        # mosaic augmented
# diffmean()   # mosaic

## ----highlight=FALSE------------------------------------------------
# quantile()   # mosaic augmented
# prop()       # mosaic
# perc()       # mosaic
# rank()
# IQR()        # mosaic augmented
# min(); max() # mosaic augmented

## -------------------------------------------------------------------
# gf_boxplot()      # ggformula
# gf_point()        # ggformula
# gf_histogram()    # ggformula
# gf_density()      # ggformula
# gf_dens()         # ggformula
# gf_freqpol()      # ggformula
# gf_qq()           # ggformula
# gf_fun()          # ggformula
# makeFun()         # mosaic

## ----highlight=FALSE------------------------------------------------
# gf_dotplot()      # ggformula
# gf_bar()          # ggformula
# gf_col()          # ggformula

## ----eval=FALSE-----------------------------------------------------
# mplot(HELPrct)

## -------------------------------------------------------------------
# rflip()     # mosaic
# do()        # mosaic
# sample()    # mosaic augmented
# resample()  # with replacement
# shuffle()   # mosaic

## ----highlight=FALSE------------------------------------------------
# rbinom()
# rnorm()     # etc, if needed

## -------------------------------------------------------------------
# gf_dist()    # ggformula
# # plain
# pbinom(); pnorm();
# # mosaic augmented
# xpnorm(); xpchisq(); xpt()
# xqbinom(); xqnorm();
# xqchisq(); xqt()

## -------------------------------------------------------------------
# t.test()       # mosaic augmented
# binom.test()   # mosaic augmented
# prop.test()    # mosaic augmented
# xchisq.test()            # mosaic
# fisher.test()
# pval()                   # mosaic
# model <- lm()     # linear models
# summary(model)
# coef(model)
# confint(model) # mosaic augmented
# anova(model)
# makeFun(model)           # mosaic
# resid(model); fitted(model)
# gf_model(model)       # ggformula

## ----highlight=FALSE------------------------------------------------
# mplot(TukeyHSD(model))
# model <- glm() # logistic reg.

## -------------------------------------------------------------------
# nrow(); ncol(); dim()
# inspect()            # mosaic
# names()
# head(); tail()
# factor()

## ----highlight=FALSE------------------------------------------------
# read.file()          # mosaic
# with()
# summary()
# glimpse()            # dplyr
# ntiles()             # mosaic
# cut()
# c()
# cbind(); rbind()
# colnames()
# rownames()
# relevel()
# reorder()

## ----highlight=FALSE------------------------------------------------
# rep()
# seq()
# sort()
# rank()

## ----highlight=FALSE------------------------------------------------
# select()             # dplyr
# mutate()             # dplyr
# filter()             # dplyr
# arrange()            # dplyr
# summarise()          # dplyr
# group_by()           # dplyr
# left_join()          # dplyr
# inner_join()         # dplyr

## ----more-hooks,eval=TRUE,echo=FALSE--------------------------------
knitr::opts_chunk$set(
	eval=TRUE,
  size='small',
	fig.width=4,
	fig.height=1.9,
	fig.align="center",
	out.width=".25\\textwidth",
	out.height=".125\\textwidth",
	tidy=TRUE,
	comment=NA
)

## ----echo=FALSE-----------------------
options(width=40)
options(show.signif.stars=FALSE)

## ----coins,fig.keep="last"------------
rflip(6)
do(2) * rflip(6)
coins <- do(1000)* rflip(6)
tally(~ heads, data=coins)

## -------------------------------------
tally(~ heads, data=coins, format="perc")
tally(~ (heads>=5 | heads<=1) , data=coins)

## ----coins-hist,fig.keep="last"-------
gf_histogram(~ heads, data = coins, binwidth = 1,
            fill = ~ (heads >=5 | heads <= 1))

## ----tally----------------------------
tally(sex ~ substance, data=HELPrct)
mean(age ~ sex, data = HELPrct)
diffmean(age ~ sex, data = HELPrct)
favstats(age ~ sex, data = HELPrct)

## ----densityplot,fig.height=2.4, tidy = FALSE----
gf_dens(~ age | sex, data = HELPrct,
                color = ~ substance)

## ----bwplot---------------------------
gf_boxplot(age ~ substance | sex, data = HELPrct)

## ----message=FALSE--------------------
pval(binom.test(~ sex, data = HELPrct))
confint(t.test(~ age, data = HELPrct))

## ----tidy=FALSE-----------------------
model <-
  lm(age ~ sex + substance, data = HELPrct)
anova(model)

## ----tidy=FALSE-----------------------
gf_point(Sepal.Length ~ Sepal.Width,
        color = ~Species, data = iris)

## ----fig.keep="last", tidy=FALSE, fig.height=2.3----
model <-
  lm(length ~ width + sex, data = KidsFeet)
l.hat <- makeFun(model)
l.hat(width = 8.25, sex = "B")
gf_point(length ~ width, data = KidsFeet,
         color = ~ sex) %>%
  gf_fun(l.hat(w, sex = "B") ~ w, color = ~"B") %>%
  gf_fun(l.hat(w, sex = "G") ~ w, color = ~"G")

## ----fig.height=1.75------------------
gf_dist("chisq", df=4)

## ----include=FALSE--------------------
tally(homeless ~ sex, data = HELPrct)

## ----include=FALSE--------------------
chisq.test(tally(homeless ~ sex, data = HELPrct))
prop.test(homeless ~ sex, data = HELPrct)

library(shiny)
library(mosaic)
options(digits = 4)

# Define server logic for random distribution application
function(input, output) {
  
  # Reactive expression to generate the requested distribution.
  # This is called whenever the inputs change. The output
  # functions defined below then all use the value computed from
  # this expression
  data <- reactive({
    dist <- switch(input$dist,
                   norm = rnorm,
                   unif = runif,
                   lnorm = rlnorm,
                   exp = rexp,
                   rnorm)
    
    data.frame(x = dist(input$n))
  })
  
  # Generate a plot of the data. Also uses the inputs to build
  # the plot label. Note that the dependencies on both the inputs
  # and the data reactive expression are both tracked, and
  # all expressions are called in the sequence implied by the
  # dependency graph
  output$plot <- renderPlot({
    dist <- input$dist
    n <- input$n
    
    histogram( ~x, data = data(), 
         main=paste('r', dist, '(', n, ')', sep=''))
  })
  
  # Generate a summary of the data
  output$summary <- renderPrint({
    favstats(~x, data = data())
  })
  
  # Generate an HTML table view of the data
  output$table <- renderTable({
    data()
  })
  
}

plotFun(sin(x) ~x, xlim=c(-2*pi, 2*pi))
plotFun(sin, xlim=c(-2*pi, 2*pi))
plotFun( cos(x) ~ x, col="red", add=TRUE)
plotFun( sin(x) ~ x, xlim=c(-2*pi, 2*pi), col="navy" )
plotFun(D(sin(x) ~ x), add=TRUE, col="red")

library(shiny)

# Define UI for random distribution application 
fluidPage(
  
  # Application title
  titlePanel("Tabsets"),
  
  # Sidebar with controls to select the random distribution type
  # and number of observations to generate. Note the use of the
  # br() element to introduce extra vertical spacing
  sidebarLayout(
    sidebarPanel(
      radioButtons("dist", "Distribution type:",
                   c("Normal" = "norm",
                     "Uniform" = "unif",
                     "Log-normal" = "lnorm",
                     "Exponential" = "exp")),
      br(),
      
      sliderInput("n", 
                  "Number of observations:", 
                  value = 500,
                  min = 1, 
                  max = 1000)
    ),
    
    # Show a tabset that includes a plot, summary, and table view
    # of the generated distribution
    mainPanel(
      tabsetPanel(type = "tabs", 
                  tabPanel("Plot", plotOutput("plot")), 
                  tabPanel("Summary", verbatimTextOutput("summary")), 
                  tabPanel("Table", tableOutput("table"))
      )
    )
  )
)

x <- rep(1:6, each=5) + runif(6 * 5, -.2, .2)
y <-  1*x + 3 + rnorm(30,3)
lsline <- function( x, y, a=NULL, b=0, ... ) {
  if (is.null(a)) {
    a <- mean(y) - b * mean(x) 
  }
  xyplot(y ~ x, 
    panel=function(x,y,...) {
      resid <- y - a - b*x
      grid::grid.rect( x, y, width=resid, height=resid, default.units="native",
        gp=grid::gpar(fill='red',col='red',alpha=0.1),
		    hjust = ifelse (resid < 0, 1, 0),
        vjust = ifelse (resid < 0, 1, 0) 
      )
      panel.xyplot(x,y,...)
      panel.abline(a,b)
      grid::pushViewport(grid::viewport(clip="off"))
      grid::grid.text( x=.95, y=1.01, just=c('right','bottom'), format(sum(resid^2)), 
        gp=grid::gpar(col='red')
      )
      grid::upViewport()
    },
    ...
  )
}

if (require(manipulate)) {
  manipulate( lsline(x,y,a=NULL, b=slope, 
  		type=if (show.regress) c('p','r') else 'p'),
    slope = slider(-5,10, initial=0, step=0.05),
    show.regress = checkbox(FALSE, label="Show regression line")
  )
} else {
  xpnorm( 750, 500, 100 )
}

C                         NCSA DTM version 2.0
C                               June 10, 1991
C
C NCSA DTM Version 2.0 source code and documentation are in the public
C domain.  Specifically, we give to the public domain all rights for future
C licensing of the source code, all resale rights, and all publishing rights.
C
C We ask, but do not require, that the following message be included in all
C derived works:
C
C Portions developed at the National Center for Supercomputing Applications at
C the University of Illinois at Urbana-Champaign.
C
C THE UNIVERSITY OF ILLINOIS GIVES NO WARRANTY, EXPRESSED OR IMPLIED, FOR THE
C SOFTWARE AND/OR DOCUMENTATION PROVIDED, INCLUDING, WITHOUT LIMITATION,
C WARRANTY OF MERCHANTABILITY AND WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE
C

C	$Header: /X11/marca/cvsroot/xmosaic2/libdtm/crayinit.f,v 1.1.1.1 1993/07/04 00:03:09 marca Exp $

C	$Log: crayinit.f,v $
c Revision 1.1.1.1  1993/07/04  00:03:09  marca
c Mosaic for X version 2 distribution
c
c Revision 1.1  1993/01/18  21:50:07  marca
c I think I got it now.
c
c Revision 1.2  1993/01/18  21:46:45  marca
c Plugging DTM in...
c
c Revision 1.3  91/06/13  21:23:57  jefft
c added type of function to prevent returning of a float
c 
c Revision 1.2  1991/06/11  15:21:54  sreedhar
c disclaimer added
c
c Revision 1.1  1990/11/08  16:29:43  jefft
c Initial revision
c

       integer function DTMINIT()
       integer i, tmp, pcnt
       integer DTMMIP, DTMMOP
       character*64 portname

       pcnt = 0

       do 10 i = 1, iargc()

         call getarg(i, portname)
         if (index(portname, '-DTMIN') .NE. 0) then
           call getarg(i+1, portname)
           tmp = DTMMIP(portname)
           pcnt = pcnt + 1
         elseif (index(portname, '-DTMOUT') .NE. 0) then
           call getarg(i+1, portname)
           tmp = DTMMOP(portname)
           pcnt = pcnt + 1
         elseif (index(portname, '-DTM') .NE. 0) then
           call getarg(i+1, portname)
           tmp = DTMMOP(portname)
           pcnt = pcnt + 1
         endif

 10    continue

       DTMINIT = pcnt
       return
       end

tryCatch(utils::globalVariables(c('slider','picker','button','checkbox','rot','elev','slider','manipulate','colorList')), 
         error=function(e) message('Looks like you should update R.'))

#' Plotting mathematical expressions
#'
#' Plots mathematical expressions in one and two variables.  
#'
#' @rdname plotFun
#' @name plotFun
#' @aliases plotFun
#'
#' @param object a mathematical expression or a function "of one variable" which will
#' converted to something intuitively equivalent to `object(x) ~ x`. (See examples)
#' @param plot a trellis object; by default, the most recently created trellis plot.  
#' When `add` is `TRUE`, the new function will
#' be plotted into a layer added to this object.
#' @param add if `TRUE`, then add a layer to an existing plot rather than creating a new plot.  
#' If `NULL`, this will be determined by the value of `under`.
#' @param under if `TRUE`, then new layer is added beneath existing layers
#' @param xlim limits for x axis (or use variable names, see examples)
#' @param ylim limits for y axis (or use variable names, see examples)
#' @param npts number of points for plotting. 
#' @param xlab label for x axis
#' @param ylab label for y axis
#' @param zlab label for z axis (when in surface-plot mode)
#' @param col vector of colors for line graphs and contours
#' @param lwd vector of line widths for line graphs
#' @param lty vector of line types for line graphs
#' @param filled fill with color between the contours (`TRUE` by default)
#' @param levels levels at which to draw contours
#' @param nlevels number of contours to draw (if `levels` not specified)
#' @param labels if `FALSE`, don't label contours
#' @param surface draw a surface plot rather than a contour plot
#' @param col.regions  a vector of colors or a function (`topo.colors` by default) for generating such
#' @param type type of plot (`"l"` by default)
#' @param alpha number from 0 (transparent) to 1 (opaque) for the fill colors 
#' @param groups grouping argument ala lattice graphics
#' @param discontinuities a vector of input values at which a function is 
#'   discontinuous or `NULL` to use
#'   a heuristic to auto-detect.
#' @param discontinuity a positive number determining how sensitive the plot is to 
#'   potential discontinuity.  Larger values result in less sensitivity.  The default is 1. 
#'   Use `discontinuity = Inf` to disable discontinuity detection.  Discontinuity detection
#'   uses a crude numerical heuristic and may not give the desired results in all cases.
#' @param interactive a logical indicating whether the surface plot should be 
#'   interactive.
#' @param ... additional parameters, typically processed by `lattice` functions such as 
#' [lattice::xyplot()], [lattice::levelplot()] or their panel functions.  
#' Frequently used parameters include 
#' \describe{ 
#' \item{`main`}{main title for plot }
#' \item{`sub`}{subtitle for plot }
#' \item{`lwd`}{line width }
#' \item{`lty`}{line type }
#' \item{`col`}{a color or a (small) integer indicating which color in the current
#' color scheme is desired.} 
#' }
#' Additionally, these arguments can be used to specify parameters for the function being 
#' plotted and to specify the plotting window with natural names.  See the examples for such usage.
#'
#' @return a `trellis` object
#'
#' @details
#' makes plots of mathematical expressions using the formula syntax.  Will
#' draw both line plots and contour/surface plots (for functions of two variables).
#' In RStudio, the surface plot comes with sliders to set orientation.
#' If the colors in filled surface plots are too blocky, increase `npts` 
#' beyond the default of 50, though `npts=300` is as much as you're likely to ever need.
#' See examples for overplotting a constraint function on an objective function.
#' 
#' @examples
#' plotFun( a*sin(x^2)~x, xlim=range(-5,5), a=2 )  # setting parameter value
#' plotFun( u^2 ~ u, ulim=c(-4,4) )                # limits in terms of u
#' # Note roles of ylim and y.lim in this example
#' plotFun( y^2 ~ y, ylim=c(-2,20), y.lim=c(-4,4) )    
#' # Combining plot elements to show the solution to an inequality
#' plotFun( x^2 -3 ~ x, xlim=c(-4,4), grid=TRUE )
#' ladd( panel.abline(h=0,v=0,col='gray50') )
#' plotFun( (x^2 -3) * (x^2 > 3) ~ x, type='h', alpha=.1, lwd=4, col='lightblue', add=TRUE )
#' plotFun( sin(x) ~ x, 
#'    groups=cut(x, findZeros(sin(x) ~ x, within=10)$x), 
#'    col=c('blue','green'), lty=2, lwd=3, xlim=c(-10,10) )
#' plotFun( sin(x) ~ x, 
#'    groups=cut(x, findZeros(sin(x) ~ x, within=10)$x), 
#'    col=c(1,2), lty=2, lwd=3, xlim=c(-10,10) )
#' ## plotFun( sin(2*pi*x/P)*exp(-k*t)~x+t, k=2, P=.3)
#' f <- rfun( ~ u & v )
#' plotFun( f(u=u,v=v) ~ u & v, u.lim=range(-3,3), v.lim=range(-3,3) )
#' plotFun( u^2 + v < 3 ~ u & v, add=TRUE, npts=200 )
#' if (require(mosaicData)) {
#' # display a linear model using a formula interface
#' model <- lm(wage ~ poly(exper,degree=2), data=CPS85)
#' fit <- makeFun(model)
#' xyplot(wage ~ exper, data=CPS85)
#' plotFun(fit(exper) ~ exper, add=TRUE, lwd=3, col="red")
#' # Can also just give fit since it is a "function of one variable"
#' plotFun(fit, add=TRUE, lwd=2, col='white')
#' }
#' # Attempts to find sensible axis limits by default
#' plotFun( sin(k*x)~x, k=0.01 )
#' # Plotting a linear model with multiple predictors.
#' mod <- lm(length ~ width * sex, data=KidsFeet)
#' fitted.length <- makeFun(mod)
#' xyplot(length ~ width, groups=sex, data=KidsFeet, auto.key=TRUE)
#' plotFun(fitted.length(width, sex="B") ~ width, add=TRUE, col=1)
#' plotFun(fitted.length(width, sex="G") ~ width, add=TRUE, col=2)
#' @export

plotFun <- function(object, ..., 
                    plot=trellis.last.object(),
                    add=NULL,
                    under=FALSE,
                    xlim=NULL, ylim=NULL, npts=NULL,
                    ylab=NULL, xlab=NULL, zlab=NULL, 
                    filled=TRUE, 
                    levels=NULL, nlevels=10,labels=TRUE,
                    surface=FALSE,
                    groups=NULL,
                    col = trellis.par.get('superpose.line')$col,
                    col.regions=topo.colors, 
                    type="l", 
                    lwd = trellis.par.get("superpose.line")$lwd,
                    lty = trellis.par.get("superpose.line")$lty,
                    alpha=NULL,
                    discontinuities = NULL,
                    discontinuity = 1,
                    interactive = rstudio_is_available()
){
  
  if ( is.function(object) ) { 
    formula <- f(x) ~ x 
    formula[[2]] <- as.call( list(substitute(object), quote(x)))
    object <- formula
  }
  
  if(is.null(add)) add <- under
  
  if (discontinuity <= 0 | length(discontinuity) != 1) 
    stop("discontinuity should be a positive number or Inf.")
  
  dots <- list(...)
  # dots[['type']] <- type
  # dots[['alpha']] <- alpha
  
  ..f.. <- do.call( "makeFun", c(object, dots, strict.declaration=FALSE), envir= parent.frame()) 
  
  if ( is.vector(col.regions ) ) col.regions  <- makeColorscheme(col.regions )
  
  if (length(unique(levels))==1) levels = c(levels,Inf) # make sure there's a range
  
  # funny names (like ..f..) are to avoid names that might be used by the user
  # not sure whether this precaution is necessary in current implementation
  
  vars <- formals(..f..)
  # print(formals(..f..))
  # print(..f..)
  rhsVars <- all.vars(rhs(object))
  otherVars <- setdiff(names(vars), rhsVars)
  ndims <- length(rhsVars)
  cleanDots <- dots
  for (v in otherVars) {
    cleanDots[[v]] <- NULL
  }
  
  pgArgs <- list()
  # print(otherVars)
  otherGroups <- if (length(otherVars) > 0 ) paste(otherVars,"",sep="") else c()
  if (length( setdiff( otherGroups, names(dots) ) ) > 0 ) {
    # print(list(otherGroups=otherGroups, dots=names(dots)) )
    
    stop(paste("Cannot plot with free parameters; try setting", 
               paste( setdiff(otherGroups, names(dots)), collapse=", " ) ))
  }
  for (param in otherVars) pgArgs[[param]] <- dots[[paste(param,"",sep="")]]
  paramGrid <- do.call( "expand.grid", pgArgs )
  
  fList <- if (nrow(paramGrid) < 1) list(..f..) else list()
  for (r in rows(paramGrid) ) {
    rowAsList <- as.list( paramGrid[r,])
    names(rowAsList) <- otherVars
    # print( c( object, c(dots, rowAsList, strict.declaration=FALSE) ) )
    fList <- c( fList, do.call ( "makeFun", 
                                 c( object, c(dots, rowAsList,
                                              strict.declaration=FALSE, envir= parent.frame()) ) 
    ))
  }
  
  
  if (add) { 
    if (ndims==1) {
      rlang::check_installed('latticeExtra')  
      return( 
        plot + 
          latticeExtra::layer( 
            do.call(panel.plotFun1, 
                    c( list(..f..=fList,  
                            npts=npts, 
                            discontinuity = discontinuity,
                            discontinuities = discontinuities,
                            filled=filled, levels=levels, 
                            nlevels=nlevels, surface=surface, 
                            col.regions=col.regions, 
                            type=type, alpha=alpha, col=col,
                            lty = lty, lwd = lwd), 
                       dots )), 
            data=as.list(environment()), 
            ...,
            under=under )
      ) 
    }
    # message("2-D adding temporarily un-discontinued.")
    check_installed('latticeExtra')
    return( plot + latticeExtra::layer(
      do.call( panel.plotFun,
               c(list( object=object, npts=npts, 
                       discontinuity = discontinuity, discontinuities = discontinuities,
                       filled=filled, levels=levels, nlevels=nlevels, suface=surface,
                       col.regions=col.regions, type=type, alpha=alpha,
                       lwd = lwd, lty = lty),
                 dots) ),
      data=as.list(environment()),
      under=under )
    )
    
    # 		return( ladd( panel.plotFun( object, npts=npts, # lwd=lwd, #col=col, 
    # 		                     filled=filled, levels=levels, nlevels=nlevels, surface=surface, 
    # 		                     col.regions=col.regions, type=type, alpha=alpha, ...)) )
    # 		# return(invisible(NULL))
  }   # end if(add)
  
  limits <- inferArgs( dots=dots, vars=rhsVars, defaults=list(xlim=xlim, ylim=ylim) )
  
  if( ndims == 1 ){
    # message("One Dimension.")
    
    npts <- ifelse( is.null(npts), 500, npts)
    
    # Set the axis labels
    # deparse needed for lattice (not originally for plot)
    
    if( is.null(ylab) ) ylab <- deparse( lhs(object) ) # deparse(..f..$sexpr)
    if( is.null(xlab) ) xlab <- rhsVars
    
    # need to handle range detection for list of functions
    if (is.null(limits$xlim) || length(limits$xlim) < 2 ) {
      zeros <- c() #empty
      tryCatch( zeros <- findZeros( object, nearest=6, ... )[[1]], 
                error=function(e){e},warning=function(e){} )
      limits$xlim <- switch(as.character(length(zeros)), 
                            "0" = c(0,1),
                            "1" = c(-1.5,1.5) * (zeros+ifelse(zeros==0,1,0)),
                            c(-.1,.1) * diff(range(zeros)) + range(zeros)
      )
    } 
    
    limits$xlim <- range(limits$xlim)
    
    if (length(limits$xlim) !=2) stop ("Invalid limits.")
    
    # Evaluate the function on appropriate inputs.
    .f. <- fList[[1]]
    .xvals <- 
      if ('h' %in% type)  
        seq(base::min(limits$xlim), base::max(limits$xlim), length.out=npts)
    else 
      adapt_seq(base::min(limits$xlim), base::max(limits$xlim), 
                f=function(xxqq){ .f.(xxqq) }, 
                length.out=npts,
                quiet=TRUE)
    
    .yvals <- c()
    for ( .f. in fList ) .yvals <- c( .yvals, sapply( .xvals, .f. ) )
    localData <- data.frame(x = .xvals, y = .yvals, component = contintuous_components(.xvals, .yvals, adjust=discontinuity))
    localData$alone <- ediff(localData$component) * ediff(localData$component, pad="tail") != 0
    # don't let points that are not connected to neighbors affect default plot widnow.
    localData <- localData[!localData$alone, ]
    localData <- data.frame(x = range(localData$x, na.rm=TRUE, fintie=TRUE), 
                            y = range(localData$y, na.rm=TRUE, finite=TRUE))
    
    # note: passing ... through to the lattice functions currently conflicts with
    # using ... to set values of "co-variates" of the function.
    
    if( length(limits$ylim) != 2 ) {
      
      thePlot <- do.call(lattice::xyplot,
                         c( list(y ~ x, 
                                 ..f.. = fList,
                                 data=localData,
                                 # groups=eval(substitute(groups),localData),
                                 groups=substitute(groups),
                                 xlim=limits$xlim, 
                                 xlab=xlab, ylab=ylab,
                                 panel="panel.plotFun1",
                                 npts = npts, 
                                 discontinuity = discontinuity,
                                 discontinuities = discontinuities,
                                 col=col,
                                 lwd = lwd, lty = lty),
                            cleanDots
                         )
      )
    } else { 
      thePlot <- do.call(lattice::xyplot, c(list(
        y ~ x,
        ..f.. = fList,
        data=localData,
        # groups=eval(substitute(groups),localData),
        groups=substitute(groups),
        xlim=limits$xlim, 
        ylim=limits$ylim, 
        xlab=xlab,ylab=ylab,
        panel="panel.plotFun1",
        npts=npts, 
        discontinuity = discontinuity,
        discontinuities = discontinuities,
        col=col, lty = lty, lwd = lwd, alpha = alpha),
        cleanDots)
      )
    }
    return((thePlot))
  }  # end ndims == 1
  
  if (ndims == 2 ) {
    # message("Two Dimensions.")
    npts <- ifelse( is.null(npts), 40, npts)
    if( length(ylab) == 0 ) ylab <- rhsVars[2]
    if( length(xlab) == 0 ) xlab <- rhsVars[1]
    if( length(zlab) == 0 ) zlab <- deparse(lhs(object))
    
    if (is.null(limits$xlim) || length(limits$xlim) < 2 ) 
      limits$xlim <- c(0,1)  # temporary default
    else 
      limits$xlim <- range(limits$xlim)
    
    if (is.null(limits$ylim) || length(limits$ylim) < 2 ) 
      limits$ylim <- c(0,1)  # temporary default
    else 
      limits$ylim <- range(limits$ylim)
    
    .xvals <- seq(base::min(limits$xlim),base::max(limits$xlim),length=npts)
    .yvals <- seq(base::min(limits$ylim),base::max(limits$ylim),length=npts)
    zvals <- tryCatch(
      outer(.xvals, .yvals, function(x,y){..f..(x,y)} ),
      warning = function(w) {} )
    grid <- expand.grid( .xvals, .yvals )
    grid$height <- c(zvals)
    localData <- grid
    names(localData) <- c(rhsVars, '.height')
    
    if( surface ) { 
      if (add) {
        stop('Should not get here, but no add option for surface plots yet anyway.')
        return(invisible(NULL))
      }
      zcuts = pretty(grid$height, 50)
      zcolors = col.regions (length(zcuts),alpha=.5*alpha)
      if(interactive && rstudio_is_available()) {
        return(manipulate::manipulate(
          wireframe(height ~ Var1 + Var2, 
                    xlab = xlab,ylab = ylab,
                    zlab = list(zlab, rot = 90),
                    data = grid,
                    groups = eval(substitute(groups), localData),
                    drape = filled,
                    shade = FALSE, colorkey = FALSE,
                    scales = list(arrows = FALSE),
                    screen = c(z = rot,x = elev - 90),
                    distance = dist,
                    at = zcuts, #col=rgb(1,1,1,0),
                    col.regions = zcolors
                    #...
          ),
          rot=manipulate::slider(min=-180,max=180,step=5,initial=35,label="Rotation"),
          elev=manipulate::slider(min=-90,max=90,step=5,initial=30,label="Elevation"),
          dist=manipulate::slider(min=0,max=1,step=.01,initial=.2,label="Distance")
        ))
      } else {  # without manipulate
        return((wireframe(height ~ Var1 + Var2, 
                          xlab = xlab, ylab = ylab, zlab = list(zlab, rot = 90),
                          data = grid,
                          groups = eval(substitute(groups), localData),
                          drape = filled, shade=FALSE, colorkey=FALSE,
                          # scales = list(arrows = FALSE),
                          col.regions = zcolors,
                          # col=rgb(1,1,1,0),
                          at = zcuts,
                          ...
        ) ) )
      }
      
    } else {  # i.e., surface==FALSE
      # a convenience wrapper around levelplot when a contour plot
      # is being drawn de novo
      funPlot.draw.contour <- function(x,y,z,ncontours=6,at=NULL,
                                       filled=TRUE, col.regions=topo.colors,labels=TRUE,
                                       showlabels=TRUE, contours=TRUE, groups=NULL, label=TRUE,
                                       xlab="",ylab="", ...){
        if (is.null(at)) at = pretty(z,ncontours)
        argsToPass <- list(z~x*y,at=at, xlab=xlab,ylab=ylab,
                           panel=panel.levelcontourplot,
                           groups=substitute(groups),col.regions=col.regions(60),
                           contour=contours, labels=labels, colorkey=FALSE,
                           retion=TRUE, filled=filled, ...)
        # kill off arguments we don't want going to levelplot:
        argsToPass[["k"]] <- NULL
        return(( do.call( levelplot, argsToPass) ))                                                           
      }
      
      if( is.null(alpha) ) alpha <- 1
      fillcolors <- col.regions (length(levels)+2, alpha=alpha)
      
      if( all (is.logical(zvals) ) ){  # it's a constraint function
        fillcolors <- col.regions (4, alpha=alpha)
        nlevels <- 2
      }
      if (add) {
        return( ladd( 
          panel.levelcontourplot(grid$Var1, grid$Var2, grid$height, subscripts=1, 
                                 at = levels, labels = labels, 
                                 filled=filled, 
                                 groups=eval(substitute(groups), localData),
                                 col.regions = col.regions ,
                                 contour = TRUE, 
                                 region = FALSE, 
                                 ...)
        ) )
      }
      return((funPlot.draw.contour(grid$Var1, grid$Var2, grid$height, 
                                   xlab=xlab, ylab=ylab, at=levels,
                                   filled=filled, labels=labels,
                                   groups=eval(substitute(groups),localData),
                                   col.regions = col.regions ,
                                   #col=col, 
                                   ...) ) )
    }
  }
  stop("Bug alert: You should not get here.  Please report.")
}

guess_discontinuities <- function( f, xlim, ..., resolution = 10000, adjust = 1) {
  x <- seq(xlim[1], xlim[2], length.out = resolution)
  F <- Vectorize(f)
  y <- F(x, ...)
  discontinuity_at(x, y, adjust = adjust)
}

discontinuity_at <- function(x, y, adjust = 1) {
  y[!is.finite(y)] <- NA
  y_scaled <- (y - base::min(y, na.rm = TRUE)) / diff(range(y[is.finite(y)]))
  x_scaled <- (x - base::min(x, na.rm = TRUE)) / diff(range(x[is.finite(x)]))
  slope  <- diff(y_scaled) / diff(x_scaled)
  jump <- diff(y_scaled)
  left_jump <- c(0, jump)
  right_jump <- c(jump, 0)
  left_slope  <- c(slope[1], slope) 
  right_slope  <- c(slope, slope[length(slope)]) 
  x[which( is.na(y) | 
             !is.finite(y) | 
             ( 
               (abs(atan(left_slope) - atan(right_slope)) > adjust / 2) & 
                 (abs(atan(left_slope)) > adjust/2 | abs(atan(right_slope)) > adjust/2)
             )
  )]
}

contintuous_components <- function(x, y, adjust = 1) {
  y[!is.finite(y)] <- NA
  y_scaled <- (y - base::min(y, na.rm = TRUE)) / diff(range(y[is.finite(y)]))
  x_scaled <- (x - base::min(x, na.rm = TRUE)) / diff(range(x[is.finite(x)]))
  slope  <- diff(y_scaled) / diff(x_scaled)
  jump <- diff(y_scaled)
  left_jump <- c(0, jump)
  right_jump <- c(jump, 0)
  left_slope  <- c(slope[1], slope) 
  right_slope  <- c(slope, slope[length(slope)]) 
  discontinuities <- x[which( is.na(y) | 
                                !is.finite(y) | 
                                ( 
                                  (abs(atan(left_slope) - atan(right_slope)) > adjust /2) & 
                                    (abs(atan(left_slope)) > adjust /2 | abs(atan(right_slope)) > adjust /2)
                                )
  )]
  j <- sapply(x, function(x) sum(x > discontinuities))
  k <- sapply(x, function(x) sum(x >= discontinuities))
  j + k
}

# lengths of points on one continuous branch of a function.

branch_lengths <- function(x, y, discontinuities = NULL, discontinuity = 1) {
  if (is.null(discontinuities)) discontinuities <- discontinuity_at(x, y, adjust = discontinuity) 
  if (length(discontinuities) < 1L) return( length(x) )
  # check <  and <= in case a grid point is exactly a discontinuity.
  j <- sapply(x, function(x) sum(x > discontinuities))
  k <- sapply(x, function(x) sum(x >= discontinuities))
  diff( c( 0, which(diff(j+k) > 0), length(x) ) )
}

#' Panel function for plotting functions
#'
#' @seealso plotFun
#' @param ..f.. an object (e.g., a formula) describing a function
#' @param x,y ignored, but there for compatibility with other lattice panel functions
#' @param col a vector of colors
#' @param lwd width of the line
#' @param lty line type
#' @param npts an integer giving the number of points (in each dimension) to sample the function
#' @param zlab label for z axis (when in surface-plot mode)
#' @param filled fill with color between the contours (`TRUE` by default)
#' @param levels levels at which to draw contours
#' @param nlevels number of contours to draw (if `levels` not specified)
#' @param surface a logical indicating whether to draw a surface plot rather than a contour plot
#' @param type type of plot (`"l"` by default)
#' @param alpha number from 0 (transparent) to 1 (opaque) for the fill colors 
#' @param discontinuities a vector of input values at which a function is 
#'   discontinuous or `NULL` to use
#'   a heuristic to auto-detect.
#' @param discontinuity a positive number determining how sensitive the plot is to 
#'   potential discontinuity.  Larger values result in less sensitivity.  The default is 1. 
#'   Use `discontinuity = Inf` to disable discontinuity detection.  Discontinuity detection
#'   uses a crude numerical heuristic and may not give the desired results in all cases.
#' @param ... additional arguments, typically processed by `lattice` panel functions
#'        such as [lattice::panel.xyplot()] or [lattice::panel.levelplot()].
#'        Frequently used arguments include
#'        \describe{
#'          \item{`lwd`}{line width}
#'        	\item{`lty`}{line type}
#'        	\item{`col`}{a color}
#'        }
#'
#' @examples
#' x <- runif(30,0,2*pi) 
#' d <- data.frame( x = x,  y = sin(x) + rnorm(30,sd=.2) )
#' xyplot( y ~ x, data=d )
#' ladd(panel.plotFun1( sin, col='red' ) )
#' xyplot( y ~ x | rbinom(30,1,.5), data=d )
#' ladd(panel.plotFun1( sin, col='red', lty=2 ) )    # plots sin(x) in each panel
#' @export

panel.plotFun1 <- function( ..f.., ...,
                            x, y,
                            type="l", 
                            lwd = trellis.par.get("superpose.line")$lwd,
                            lty = trellis.par.get("superpose.line")$lty,
                            col = trellis.par.get('superpose.line')$col,
                            npts=NULL,
                            zlab=NULL, 
                            filled=TRUE, 
                            levels=NULL, 
                            nlevels=10,
                            surface=FALSE,
                            alpha=NULL,
                            discontinuity = NULL,
                            discontinuities = NULL) { 
  dots <- list(...)
  
  if (is.function(..f..) ) ..f.. <- list(..f..)
  if (! is.list(..f..) || length(..f..) < 1) {
    print(str(..f..))
    stop("Empty or malformed list of functions.")
  }
  
  if (is.numeric(col)) {
    message('converting numerical color value into a color using lattice settings')
    col <- trellis.par.get('superpose.line')$col[col]
  }
  
  
  # funny names (like ..f..) are to avoid names that might be used by the user
  # not sure whether this precaution is necessary in current implementation
  
  # perhaps use environment(object)?
  # ..f.. <- do.call( "makeFun", c(object, dots, strict.declaration=FALSE), envir= parent.frame())  
  idx <- 0
  
  for ( .f. in ..f..) {  
    idx <- idx + 1
    # print(.f.)
    vars <- formals(.f.)
    #  rhsVars <- all.vars(rhs(object))
    #  ndims <- length(rhsVars)
    
    parent.xlim <- current.panel.limits()$xlim
    parent.ylim <- current.panel.limits()$ylim
    
    npts <- ifelse( is.null(npts), 200, npts)
    
    if (! missing(x) && !missing(y) && length(x) >= npts && length(y) == length(x)) {
      .xvals <- x
      .yvals <- y
    } else {
      # Evaluate the function on appropriate inputs to help figure out y limits.
      .xvals <-  if ('h' %in% type)  
        seq(base::min(parent.xlim), base::max(parent.xlim), length.out=npts)
      else 
        adapt_seq(base::min(parent.xlim), base::max(parent.xlim), 
                  f=function(xxqq){ .f.(xxqq) }, 
                  length.out=npts,
                  quiet=TRUE)
      
      .yvals <- suppressWarnings( sapply( .xvals, .f. ) )
    }
    
    # need to strip out any components of ... that are in the object so they
    # don't get passed to the panel function.
    cleandots = list(...)
    # cleandots[ intersect(names(cleandots), all.vars(object)) ] <- NULL
    cleandots[c('x','y','type','alpha','col')] <- NULL
    # use do.call to call the panel function so that the cleandots can be put back in
    
    if (type == "l") {
      if (is.null(discontinuities)) 
        discontinuities <- discontinuity_at(.xvals, .yvals, adjust = discontinuity)
      do.call(
        grid::grid.polyline,
        c(list(x=.xvals, y=.yvals, default.units = "native",
               gp=do.call(
                 grid::gpar, 
                 c(list(alpha=alpha, col=.getColor(idx, col), 
                   lty = lty, lwd = lwd),
                   cleandots)
               ), 
               id.lengths = branch_lengths(.xvals, .yvals, discontinuities)  
        ))
      )
    } else {
      do.call(
        panel.xyplot,
        c(list(x=.xvals, y=.yvals, type=type,  alpha=alpha, 
               lwd = lwd, lty = lty, col=.getColor(idx,col)),  cleandots)
      )
    }
  }
}

panel.plotFun1a <- function( ..f.., ...,
                             x, y,
                             type="l", 
                             col = trellis.par.get('superpose.line')$col,
                             lwd = trellis.par.get("superpose.line")$lwd,
                             lty = trellis.par.get("superpose.line")$lty,
                             npts=NULL,
                             zlab=NULL, 
                             filled=TRUE, 
                             levels=NULL, 
                             nlevels=10,
                             surface=FALSE,
                             alpha=NULL,
                             discontinuity = NULL,
                             discontinuities = NULL) { 
  dots <- list(...)
  
  if (is.function(..f..) ) ..f.. <- list(..f..)
  if (! is.list(..f..) || length(..f..) < 1) {
    print(str(..f..))
    stop("Empty or malformed list of functions.")
  }
  
  if (is.numeric(col)) {
    message('converting numerical color value into a color using lattice settings')
    col <- trellis.par.get('superpose.line')$col[col]
  }
  
  
  # funny names (like ..f..) are to avoid names that might be used by the user
  # not sure whether this precaution is necessary in current implementation
  
  # perhaps use environment(object)?
  # ..f.. <- do.call( "makeFun", c(object, dots, strict.declaration=FALSE), envir= parent.frame())  
  
  parent.xlim <- current.panel.limits()$xlim
  parent.ylim <- current.panel.limits()$ylim
  points_data <- data.frame() 
  groups_data <- data.frame()
  id_lengths <- integer(0)
  idx <- 0L
  
  for (.f. in ..f..) {
    idx <- idx + 1
    # print(.f.)
    vars <- formals(.f.)
    #  rhsVars <- all.vars(rhs(object))
    #  ndims <- length(rhsVars)
    
    npts <- ifelse( is.null(npts), 200, npts)
    
    if (! missing(x) && !missing(y) && length(x) >= npts && length(y) == length(x)) {
      .xvals <- x
      .yvals <- y
    } else {
      # Evaluate the function on appropriate inputs to get y values.
      .xvals <-  if ('h' %in% type)  
        seq(base::min(parent.xlim), base::max(parent.xlim), length.out=npts)
      else 
        adapt_seq(base::min(parent.xlim), base::max(parent.xlim), 
                  f=function(xxqq){ .f.(xxqq) }, 
                  length.out=npts,
                  quiet=TRUE)
      
      tryCatch(.yvals <- sapply( .xvals, .f. ), warning=function(w) {} )
    }
    
    if (is.null(discontinuities)) 
      discontinuities <- discontinuity_at(.xvals, .yvals, adjust = discontinuity)
    groups_data <-
      dplyr::bind_rows(
        groups_data, 
        data.frame(
          length = branch_lengths(.xvals, .yvals, discontinuities),
          id = idx
          )
      )
    
    points_data <- 
      dplyr::bind_rows(points_data, 
                       data.frame(x = .xvals, y = .yvals, id=idx))
    
  }
  
  # points_data is now a data frame with x, y, and an id showing which function
  # is being described.
  
  # need to strip out any components of ... that are in the object so they
  # don't get passed to the panel function.
  cleandots = list(...)
  # cleandots[ intersect(names(cleandots), all.vars(object)) ] <- NULL
  cleandots[c('x','y','type','alpha','col')] <- NULL
  # use do.call to call the panel function so that the cleandots can be put back in
  
  if (type == "l") {
    do.call(
      grid::grid.polyline,
      c(list(x=points_data$x, y=points_data$y, default.units = "native",
             gp=do.call(
               grid::gpar, 
               c(list(alpha=alpha, col=.getColor(groups_data$id, col),
                      lwd = lwd, lty = lty), cleandots)
             ), 
             id.lengths = groups_data$length
      ))
    )
  } else {
    do.call(
      panel.xyplot,
      c(list(x=points_data$x, y=points_data$y, type=type,  
             lwd = lwd, lty = lty,
             alpha=alpha, col=.getColor(points_data$id, col)),  cleandots)
    )
  }
}


#' Panel function for plotting functions
#'
#' @seealso plotFun
#' @param object an object (e.g., a formula) describing a function
#' @param npts an integer giving the number of points (in each dimension) to sample the function
#' @param zlab label for z axis (when in surface-plot mode)
#' @param lwd width of the line
#' @param lty line type
#' @param filled fill with color between the contours (`TRUE` by default)
#' @param levels levels at which to draw contours
#' @param nlevels number of contours to draw (if `levels` not specified)
#' @param surface a logical indicating whether to draw a surface plot rather than a contour plot
#' @param col.regions  a vector of colors or a function (`topo.colors` by default) for generating such
#' @param type type of plot (`"l"` by default)
#' @param alpha number from 0 (transparent) to 1 (opaque) for the fill colors 
#' @param discontinuities a vector of input values at which a function is 
#'   discontinuous or `NULL` to use
#'   a heuristic to auto-detect.
#' @param discontinuity a positive number determining how sensitive the plot is to 
#'   potential discontinuity.  Larger values result in less sensitivity.  The default is 1. 
#'   Use `discontinuity = Inf` to disable discontinuity detection.  Discontinuity detection
#'   uses a crude numerical heuristic and may not give the desired results in all cases.
#' @param ... additional arguments, typically processed by `lattice` panel functions
#'        such as [lattice::panel.xyplot()] or [lattice::panel.levelplot()].
#'        Frequently used arguments include
#'        \describe{
#'          \item{`lwd`}{line width}
#'        	\item{`lty`}{line type}
#'        	\item{`col`}{a color}
#'        }
#'
#' @examples
#' x <- runif(30,0,2*pi) 
#' d <- data.frame( x = x,  y = sin(x) + rnorm(30,sd=.2) )
#' xyplot( y ~ x, data=d )
#' ladd(panel.plotFun( sin(x) ~ x, col='red' ) )
#' xyplot( y ~ x | rbinom(30,1,.5), data=d )
#' ladd(panel.plotFun( sin(x) ~ x, col='red', lty=2 ) )    # plots sin(x) in each panel
#' @export

panel.plotFun <- function( object, ..., 
                           type="l", 
                           npts=NULL,
                           zlab=NULL, 
                           filled=TRUE, 
                           levels=NULL, 
                           nlevels=10,
                           surface=FALSE,
                           col.regions =topo.colors, 
                           lwd = trellis.par.get("superpose.line")$lwd,
                           lty = trellis.par.get("superpose.line")$lty,
                           alpha=NULL,
                           discontinuity = NULL,
                           discontinuities = NULL ) { 
  dots <- list(...)
  if ( is.function(object) ) { 
    formula <- f(x) ~ x 
    formula[[2]] <- as.call( list(substitute(object), quote(x)))
    object <- formula
  }
  
  if ( is.vector(col.regions ) ) col.regions  <- makeColorscheme(col.regions )
  
  plot.line <- trellis.par.get('plot.line')
  superpose.line <- trellis.par.get('superpose.line')
  
  # funny names (like ..f..) are to avoid names that might be used by the user
  # not sure whether this precaution is necessary in current implementation
  
  # perhaps use environment(object)?
  ..f.. <- do.call( "makeFun", c(object, dots, strict.declaration=FALSE), envir= parent.frame())  
  
  vars <- formals(..f..)
  rhsVars <- all.vars(rhs(object))
  ndims <- length(rhsVars)
  
  parent.xlim <- current.panel.limits()$xlim
  parent.ylim <- current.panel.limits()$ylim
  
  if( ndims > 2 || ndims < 1 ) 
    stop("Formula must provide 1 or 2 independent variables (right hand side).")
  
  if( ndims == 1 ){
    npts <- ifelse( is.null(npts), 200, npts)
    
    
    # Evaluate the function on appropriate inputs.
    .xvals <- 
      if ('h' %in% type)  
        seq(base::min(parent.xlim), base::max(parent.xlim), length.out=npts)
    else 
      adapt_seq(base::min(parent.xlim), base::max(parent.xlim), 
                f=function(xxqq){ ..f..(xxqq) }, 
                length.out=npts,
                quiet=TRUE)
    .yvals <- tryCatch( sapply( .xvals, ..f.. ), 
                        warning=function(w) {} )
    
    
    # need to strip out any components of ... that are in the object so they
    # don't get passed to the panel function.
    cleandots = list(...)
    cleandots[ names(cleandots) %in% all.vars(object) ] <- NULL
    # use do.call to call the panel function so that the cleandots can be put back in
    if (type == "l") {
      idx <- 1
      return(
        do.call(
          grid::grid.polyline,
          c(list(x=.xvals, y=.yvals, default.units = "native",
                 gp=do.call(
                   grid::gpar, 
                   c(list(alpha=alpha, col=.getColor(idx, col),
                          lty = lty, lwd = lwd), cleandots)
                 ), 
                 id.lengths = branch_lengths(.xvals, .yvals, discontinuities) 
          ))
        )
      )
    } else {
      return(do.call(
        panel.xyplot,
        c(list(x=.xvals, y=.yvals, type=type, alpha=alpha, lty = lty, lwd = lwd), 
          cleandots)))
    }
  }
  
  if (ndims == 2 ) {
    if( surface ) { 
      stop('no add option for surface plots yet.')
      return(NULL)
    }
    # if we get here, surface == FALSE & ndims=2
    npts <- ifelse( is.null(npts), 40, npts)
    # create a function of those two variables
    
    if( length(zlab) == 0 ) zlab <- deparse(lhs(object) )
    
    .xvals <- seq(base::min(parent.xlim),base::max(parent.xlim),length=npts)
    .yvals <- seq(base::min(parent.ylim),base::max(parent.ylim),length=npts)
    zvals <- tryCatch( 
      outer(.xvals, .yvals, function(x,y){..f..(x,y)} ),
      warning=function(w) {} )
    grid <- expand.grid( .xvals, .yvals )
    grid$height <- c(zvals)
    
    zcuts <- pretty(grid$height,50)
    zcolors <- col.regions (length(zcuts),alpha=.5)
    # print(zcolors)
    if( is.null(alpha) ) alpha<-.4
    
    if( all(is.logical(zvals)) ) {  # it's a constraint function
      #nlevels <- 2
      levels <- c(0.0,Inf) 
    }
    fillcolors <- col.regions (length(levels) + 2, alpha=alpha)
    if(is.null(levels)) levels=pretty(grid$height, nlevels)
    
    return( panel.levelcontourplot(x = grid$Var1, y = grid$Var2, z = grid$height,
                                   subscripts = 1:nrow(grid),
                                   at = levels,
                                   col.regions = fillcolors,
                                   filled=filled, 
                                   ...
                                   #col=col, lwd = lwd, lty = 1,
    )
    )
  }
  stop("Bug alert: You should not get here.  Please report.")
}



.getColor <- function( n=1, col=trellis.par.get('superpose.line')$col) {
  if (length(col) < n) col <- rep(col, length.out=n)
  col[n]
}
#' Infer arguments 
#' 
#' The primary purpose is for inferring argument settings from names derived from variables
#' occurring in a formula.  For example, the default use is to infer limits for variables
#' without having to call them `xlim` and `ylim` when the variables in the formula
#' have other names.  Other uses could easily be devised by specifying different `variants`.
#' 
#' @param vars a vector of variable names to look for
#' @param dots a named list of argument values
#' @param defaults named list or alist of default values for limits
#' @param variants a vector of optional postfixes for limit-specifying variable names
#' @return a named list or alist of limits.  The names are determined by the names in `defaults`.
#'
#' If multiple `variants` are matched, the first is used.
#' @examples
#' inferArgs(c('x','u','t'), list(t=c(1,3), x.lim=c(1,10), u=c(1,3), u.lim=c(2,4)))
#' inferArgs(c('x','u'), list(u=c(1,3)), defaults=list(xlim=c(0,1), ylim=NULL)) 
#' @export

inferArgs <- function( vars, dots, defaults=alist(xlim=, ylim=, zlim=), variants=c('.lim','lim') ) {
  limNames <- names(defaults)
  if (length(vars) > length(limNames)) 
    stop( paste("Can only infer", 
                paste(limNames, sep=""), 
                "; but you provided", 
                length(vars)),
          " variables to search for.", 
          sep="")
  result <-defaults
  
  for (slot in 1:length(vars)) {
    for (variant in rev(variants)){
      var.lim <- paste(vars[slot],variant,sep="")
      if (! is.null(dots[[var.lim]]) ) {
        result[[limNames[slot]]] <- dots[[var.lim]]
      }
    }
  }
  return(result)
}

# =============================
# Create an environment for storing axis limits, etc.
.plotFun.envir = new.env(parent=baseenv())
# =====
#' 
#' Lattice plot that draws a filled contour plot
#' 
#' Used within plotFun
#'
#' @param x x on a grid
#' @param y y on a grid
#' @param z zvalues for the x and y
#' @param subscripts which points to plot
#' @param at cuts for the contours
#' @param shrink what does this do?
#' @param labels draw the contour labels
#' @param label.style where to put the labels
#' @param contour logical draw the contours
#' @param region logical color the regions
#' @param col color for contours
#' @param lty type for contours
#' @param lwd width for contour
#' @param border type of border
#' @param ... dots additional arguments
#' @param col.regions  a vector of colors or a function (`topo.colors` by default) for generating such
#' @param filled whether to fill the contours with color
#' @param alpha.regions transparency of regions
#' @export

panel.levelcontourplot <- function(x, y, z, subscripts=1, 
                                   at, shrink, labels = TRUE, 
                                   label.style = c("mixed","flat","align"), 
                                   contour = FALSE, 
                                   region = TRUE,
                                   col = add.line$col, 
                                   lty = add.line$lty,
                                   lwd = add.line$lwd, 
                                   border = "transparent", ...,
                                   col.regions = regions$col,
                                   filled=TRUE, 
                                   alpha.regions = regions$alpha
)
{
  add.line <- trellis.par.get('add.line')
  regions <- trellis.par.get('regions')
  
  if(filled) panel.levelplot(x, y, z, subscripts, 
                             at = pretty(z,5*length(at)), shrink, 
                             labels = FALSE, 
                             label.style = label.style, 
                             contour = FALSE, 
                             region = TRUE,  
                             border = border, ..., 
                             col.regions = col.regions #, 
                             #                  alpha.regions = regions$alpha
  )
  if( all(is.logical(z)) ) {  # it's a constraint function
    at <- c(0,1) 
    labels <- FALSE
  }
  panel.levelplot(x, y, z, subscripts, 
                  at = at, shrink, labels = labels, 
                  label.style = label.style, 
                  contour = TRUE, 
                  region = FALSE, lty=lty, 
                  col = col, lwd=lwd,
                  border = border, ...)
}

#' Create a color generating function from a vector of colors
#'
#' @param col a vector of colors
#' @return a function that generates a vector of colors interpolated among the colors in `col`
#'
#' @examples
#' cs <- makeColorscheme( c('red','white','blue') )
#' cs(10)
#' cs(10, alpha=.5)
#' @export

makeColorscheme <- function(col) {
  result <- function(n, alpha=1, ...)  {
    idx <-  0.5 + (0:n)/(n+0.001) * (length(colorList))  
    return( apply(col2rgb(col[ round(idx) ]), 2, 
                  function(x) { rgb(x[1],x[2],x[3], alpha=round(alpha*255), maxColorValue=255) } ) )
  }
  e <- new.env()
  e[['colorList']] <- col
  environment(result) <- e
  return(result)
}

# context("plotPoints()")
require(mosaicData)
testthat::test_that("plotPoints works", {
  
  wrapped_expect_doppelganger("plotPoints1", plotPoints( width ~ length, data=KidsFeet, groups=sex, pch=20))
  
  f <- makeFun( lm( width ~ length * sex, data=KidsFeet))
  wrapped_expect_doppelganger("plotPoints2", plotFun( f(length=length,sex="G")~length, add=TRUE, col="pink"))
  wrapped_expect_doppelganger("plotPoints3", plotFun( f(length=length,sex="B")~length, add=TRUE))
  
})

# context("fitSpline()")

testthat::test_that("fitSpline works", {
  
  f <- fitSpline( weight ~ height, data = women, df = 5 )
  g <<- fitSpline( length ~ width, data = KidsFeet, type = 'natural', df = 5 )
  h <<- fitSpline( length ~ width, data = KidsFeet, type = 'linear', df = 5 )
  
  wrapped_expect_doppelganger("fitspline1", xyplot( weight ~ height, data = women ))
  wrapped_expect_doppelganger("fitspline2", plotFun(f(height) ~ height, add = TRUE))
  
  # xyplot( length ~ width, data = KidsFeet, col = 'gray70', pch = 16)
  
  wrapped_expect_doppelganger("fitspline3", plotFun(g, add = TRUE, col = 'navy'))
  wrapped_expect_doppelganger("fitspline4", plotFun(h, add = TRUE, col = 'red'))
  
  
})

# It is unclear why the CRAN checks are finding a problem with u.
# this masks the problem, but it would be better to find and remove
# the issue.

utils::globalVariables(c('u'))

#' Projections
#' 
#' Compute projections onto the span of a vector or a model space, dot products, and
#' vector lengths in Euclidean space.
#'
#' @rdname project-methods
#' @docType methods
#' @param x a numeric vector (all functions) or a formula (only for `project`).  
#' Left-hand sides of formulas should be a single quantity
#' @param u a numeric vector 
#' @param v a numeric vector
#' @param data a data frame.
#' @param type one of `"length"` or `"vector"` determining the type of the 
#' returned value
#' @param coefficients For `project(y ~ x)` indicates whether the projection
#' coefficents should be returned or the projection vector.
#' @param ... additional arguments

#' @details
#' `project` (preferably pronounced "pro-JECT" as in "projection") 
#' does either of two related things:
#' (1) Given two vectors as arguments, it will project the first onto the 
#' second, returning the point in the subspace of the second that is as
#' close as possible to the first vector.  (2) Given a formula as an argument,
#' will work very much like `lm()`, constructing a model matrix from
#' the right-hand side of the formula and projecting the vector on the 
#' left-hand side onto the subspace of that model matrix.  
#' 
#' In (2), rather than 
#' returning the projected vector, `project()` returns the coefficients
#' on each of the vectors in the model matrix.
#' UNLIKE `lm()`, the intercept vector is NOT included by default.  If 
#' you want an intercept vector, include `+1` in your formula.
#' @export

setGeneric( 
	"project", 
	function(x, ... )  {
		standardGeneric('project')
	}
)
##################
#' @rdname project-methods
#' @return `project` returns the projection of `x` onto `u` 
#' (or its length if `u` and `v` are numeric vectors and `type == "length"`)
#'
#' @examples
#' x1 <- c(1,0,0); x2 <- c(1,2,3); y1 <- c(3,4,5); y2 <- rnorm(3)
#' # projection onto the 1 vector gives the mean vector
#' mean(y2)            
#' project(y2, 1)
#' # return the length of the vector, rather than the vector itself
#' project(y2, 1, type='length')
#' project(y1 ~ x1 + x2) -> pr; pr
#' # recover the projected vector 
#' cbind(x1,x2) %*% pr -> v; v
#' project( y1 ~ x1 + x2, coefficients=FALSE )
#' dot( y1 - v, v ) # left over should be orthogonal to projection, so this should be ~ 0
#' if (require(mosaicData)) {
#' project(width~length+sex, data=KidsFeet)
#' }
#' @export

setMethod(
		  'project',
		  signature=c('formula'),
		  function( x, u=NULL, data=parent.frame(2), coefficients=TRUE, ...) {
			  # x is the formula
			  # u is just a placeholder
			  foo <- model.frame( x, data=data )
			  u <- model.response(foo)
			  M <- mat( x, data=data )
			  n <- svd(M);
			  zeros <- n$d < .0000001;
			  recip <- n$d;
			  recip[!zeros] <- 1/recip[!zeros];
			  if (length(recip)==1) recip = matrix(recip)
			  else recip = diag(recip)
			  coefs <- n$v %*% recip %*% t(n$u) %*% u
			  rownames(coefs) <- colnames(M)
        # convert to a vector
        vcoefs <- as.vector(coefs)
        names(vcoefs) <- colnames(M)
        if (coefficients) 
          return(vcoefs)
        else {
          return(as.vector( M %*% vcoefs ) )
        }
    
		  }
		  )
##################

# This is used in fastR and should not go away.


#' @rdname project-methods
#' @export

setMethod(
	'project',
	signature=c('numeric'),
	function (x, u = rep(1, length(x)), type = c("vector", "length", "coef"), ...) {
		type <- match.arg(type)
		u <- rep(u, length.out=length(x))
		
		if(dot(u,u) <= 0) {
		  return(
		    switch(type, 
		       vector = rep(0, length(u)),
		       length = 0,
		       coef   = 0
		       )
		  )
		}
		
		switch(type, 
		       vector = u * (dot(x, u)/dot(u, u)), 
		       length = abs(dot(x, u))/sqrt(dot(u, u)), 
		       coef   =  dot(x, u)/dot(u, u) 
		       )
	}
)

#' @rdname project-methods
#' @export

setMethod(
		  'project',
		  signature=c('matrix'),
		  function(x, u, data=parent.frame()) {
			  A <- x; b <- u
			  b <- cbind(b)
			  # See if A is just a single 1 --- if so, turn it into a vector of ones
			  # This would happen automatically if mat is used to combine 1 with
			  # another vector, but there is the possibility that someone might try
			  # project(1,b)
			  if( length(A[])==1 ) {
				  A <- mat(cbind(1+0*b))
			  }
			  foo <- dim(A);
			  goo <- dim(b);
			  if (goo[1] !=1 & goo[2] != 1){
				  stop('u must be a vector');
			  }
			  if (foo[1] != length(b)) {
				  stop('x must contain vectors of the same dimension as u');
			  }
			  if (foo[2] <= foo[1]) {
				  return(  cbind(lsfit(A,b,intercept=FALSE)$coef ) );
			  }
			  n <- svd(A);
			  zeros <- n$d < .0000001;
			  recip <- n$d;
			  recip[!zeros] <- 1/recip[!zeros];
			  n$v%*%diag(recip)%*%t(n$u)%*%b
		  }
		  )
 
#' @rdname project-methods
#' @return `vlength` returns the length of the vector 
#' (i.e., the square root of the sum of the squares of the components)
#' @seealso \code{link{project}}
#' @examples
#' vlength(rep(1,4))
#' if (require(mosaicData)) {
#' m <- lm( length ~ width, data=KidsFeet )
#' # These should be the same
#' vlength( m$effects )  
#' vlength( KidsFeet$length)
#' # So should these
#' vlength( tail(m$effects, -2) )
#' sqrt(sum(resid(m)^2))
#' }
#' v <- c(1,1,1); w <- c(1,2,3)
#' u <- v / vlength(v)  # make a unit vector
#' # The following should be the same:
#' project(w,v, type="coef") * v 
#' project(w,v)
#' # The following are equivalent
#' abs(dot( w, u ))
#' vlength( project( w, u) )
#' vlength( project( w, v) )
#' project( w, v, type='length' )
#' @export

vlength <- function(x, ...) {
   sqrt(sum(x^2))
}

#' @rdname project-methods
#' @return `dot` returns the dot product of `u` and `v`
#' @export

dot <- function(u, v){
  return( sum(u*v) )
}

# context('Linear Algebra')


test_that("dot works for vectors", {
  x <- c(1,2,3)
  u <- c(1,1,1)
  expect_equal(ignore_attr = TRUE,  dot(x,u), 6 )
})

test_that("project works for vectors", {
  x <- c(1,2,3)
  u <- c(1,1,1)
  n <- 10
  y <- rnorm(n)
  one <- rep(1,n)
  expect_equal(ignore_attr = TRUE,  project(x, c(1,0,0)), c(1,0,0) ) 
  expect_equal(ignore_attr = TRUE,  project(x, c(0,1,0)), c(0,2,0) )
  expect_equal(ignore_attr = TRUE,  project(x, c(0,0,1)), c(0,0,3) )
  expect_equal(ignore_attr = TRUE,  project(x, u),  c(2,2,2) ) 
  expect_equal(ignore_attr = TRUE,  project(y, one), rep(mean(y), n) )
})

test_that("1 vector works", {
  x <- c(1,2,3)
  u <- c(1,1,1)
  y <- rnorm(10)
  expect_equal(ignore_attr = TRUE,  project(x,1), c(2,2,2) )
  expect_equal(ignore_attr = TRUE,  project(y,1),  rep(mean(y),length(y)) ) 
})

test_that("formula interface to project works in present environment",{
  x <- c(1,2,3)
  b <- c(5,4,3)
  expect_equal( round(project(b~x)[1],4), 1.5714, ignore_attr = TRUE)
  expect_equal( project(b~x+1)[1], 6, ignore_attr = TRUE)
})

test_that("formula interface works with data frame",{
  expect_equal( 
    round(project( wage ~ educ, data=mosaicData::CPS85)[1],3), 
    0.695, ignore_attr = TRUE)
  expect_equal( 
    round(project( wage ~ educ+1, data=mosaicData::CPS85)[1],3), 
    -0.746, ignore_attr = TRUE)
})

#' Dotplots
#' 
#' A high level function and panel function for producing
#' 	a variant of a histogram called a dotplot.
#'
#' @param x  a vector of values or a formula 
#'
#' @param nint  the number of intervals to use 
#'
#' @param panel  a panel function 
#'
#' @param breaks,equal.widths,groups,pch,col,lty,lwd,col.line,type,alpha  
#'     as in [histogram()] 
#'
#' @param cex  a ratio by which to increase or decrease the dot size
#'
#' @param \dots  additional arguments 
#' 
#' @return a trellis object
#'
#' 
#' @seealso [histogram()]
#' 
#' 
#' @examples
#' if (require(mosaicData)) {
#' dotPlot( ~ age, data = HELPrct)
#' dotPlot( ~ age, nint=42, data = HELPrct)
#' dotPlot( ~ height | voice.part, data = singer, nint = 17,
#'           endpoints = c(59.5, 76.5), layout = c(4,2), aspect = 1,
#'           xlab = "Height (inches)")
#' }
#' @keywords graphics 
#' @export

dotPlot <-
function (x, breaks, ..., panel = panel.dotPlot) 
{
  histogram(x, type = "count", panel = panel, breaks = breaks, ...)
}

#' @rdname dotPlot
#' @export

panel.dotPlot <-
  function (x, breaks, equal.widths = TRUE, groups = NULL, 
            nint = if (is.factor(x)) nlevels(x) else round(1.3* log2(length(x)) + 4),
            pch,
            col,
            lty = trellis.par.get("dot.line")$lty, 
            lwd = trellis.par.get("dot.line")$lwd, 
            col.line = trellis.par.get("dot.line")$col, 
            alpha = trellis.par.get("dot.symbol")$alpha, 
            cex=1, 
            type = "count", 
            ...) 
{
  if (is.function(breaks)) {
      breaks <- do.call(breaks, c(list(x=x, nint=nint), list(...)))
  }
      
  if (missing(pch)) {
    pch <-  if (is.null(groups)) trellis.par.get("dot.symbol")$pch 
              else trellis.par.get("superpose.symbol")$pch 
  }
  if (missing(col)) {
    col <- if (is.null(groups)) trellis.par.get("dot.symbol")$col 
            else trellis.par.get("superpose.symbol")$col 
  }
    
	if (is.null(nint)) nint <- if (is.factor(x)) nlevels(x) else round(1.3* log2(length(x)) + 4)
    dot.line <- trellis.par.get("dot.line")
    dot.symbol <- trellis.par.get("dot.symbol")
    sup.symbol <- trellis.par.get("superpose.symbol")
    x <- as.numeric(x)
	if ( !is.null(groups) ) {
		groups <- factor(groups)
		col <- rep(col, length.out=length(levels(groups)))
		pch <- rep(pch, length.out=length(levels(groups)))
		col <- col[as.numeric(groups)]
		pch <- pch[as.numeric(groups)]
		col <- col[order(x)]
		pch <- pch[order(x)]
	} else {
		pch <- rep(pch[1], length.out <- length(x))
		col <- rep(col[1], length.out <- length(x))
	}

    if (length(x) > 0) {
        if (missing(breaks)) {
            breaks <- if (equal.widths) 
                do.breaks(range(x, finite = TRUE), nint)
            else quantile(x, (0:nint)/nint, na.rm = TRUE)
        }
        h <- hist(x, breaks = breaks, plot = FALSE, warn.unused=FALSE, ...)
        cumcounts <- c(0,cumsum(h$counts))
        nb <- length(breaks)

        if (length(h$counts) != nb - 1) {
            stop("problem with hist computations")
        }
        if (nb > 1) {
            for (bin in 1:(nb - 1)) {
                if (h$counts[bin] <= 0) {
                  next
                }
                yvals <- 1:(h$counts[bin])
                xvals <- rep( h$mids[bin], h$counts[bin] )
                grid.points( 
                  size = cex * grid::unit(0.9 / current.panel.limits()$ylim[2], 
                                          "snpc"),  
                  pch = pch, 
                  gp = gpar(fill = col, alpha = alpha, col = col, 
                            lty = lty, lwd = lwd), 
				  		    x = xvals,
                  y = yvals - 0.5, 
						default.units = "native")
				pch <- pch[ -(1:h$counts[bin]) ]
				col <- col[ -(1:h$counts[bin]) ]
            }
        }
    }
}

# context("dotPlot()")

testthat::test_that("Dot Plot works", {
  require(mosaicData)
  
  wrapped_expect_doppelganger("dotplot1", dotPlot( ~ age, data = HELPrct))
  wrapped_expect_doppelganger("dotplot2", dotPlot( ~ age, nint=42, data = HELPrct))
  wrapped_expect_doppelganger("dotplot3", dotPlot( ~ height | voice.part, data = singer, nint = 17,
                                                   endpoints = c(59.5, 76.5), layout = c(4,2), aspect = 1,
                                                   xlab = "Height (inches)"))
  
  
})

#' Adaptively generate sequences in an interval
#'
#' `adapt_seq` is similar to `seq` except that instead of 
#' selecting points equally spaced along an interval, it selects points
#' such that the values of a function applied at those points are 
#' (very) roughly equally spaced.  This can be useful for sampling 
#' a function in such a way that it can be plotted more smoothly, 
#' for example.
#'
#'
#' @param from start of interval
#' @param to end of interval
#' @param length.out desired length of sequence
#' @param f a function
#' @param args arguments passed to `f`
#' @param quiet suppress warnings about NaNs, etc.
#' @return a numerical vector
#' @examples
#' adapt_seq(0, pi, 25, sin)
#'
#' @export
adapt_seq <-function(from, to, 
                     length.out=200, 
                     f=function(x,...){ 1 }, 
                     args=list(),
                     quiet=FALSE
) 
{
  n <- round(sqrt(length.out))
  n <- max(n,10)
  s <- seq(from, to, length.out=n)
  iteration <- 0
  keepers <- s
  
  repeat{
    iteration <- iteration + 1
    # clean up: remove bad s values
    if (quiet) {
      suppressWarnings(y <- do.call(f, args=c(list(s), args)))
    } else {
      y <- do.call(f, args=c(list(s), args))
    }
    w <- which(is.finite(y))
    a <- min(w) - 1
    a <- max(a,1)
    b <- max(w) + 1
    b <- min(b,length(s))
    w <- sort(unique(c(w, 1, length(s), a, b )))
    s <- s[w]; y <- y[w]
    
    # quit if we have enough or reach edge condition
    # print(list(i=iteration, a=s[a], b=s[b], start=head(s,4), end=tail(s,4)))
    if ((length(s) < 2) || (iteration > 20) || (length(s) > length.out)) 
      return(s[is.finite(y)])
    
    # keep best points + previous keepers
    ds <- ediff(s)
    dy <- ediff(y)
    angle <- atan(dy/ds)
    # score <- abs(ediff(dy/ds, pad="tail"))
    score <- abs(ediff(angle, pad="tail"))
    wsum <- cumsum(is.finite(y))
    a <- which.min( wsum== 1) - 1  # last non-finite
    a <- max(a,1)
    b <- which.min( wsum == max(wsum) ) # first of last run of non-finites
    b < min(b,length(s))
    score[c(a,b)] <- Inf     # keep a and b
    
    w <- which( s %in% keepers | score > quantile(score, probs=0.2, na.rm=TRUE))
    # be sure to keep left and right ends
    w <- sort(unique(c(w, 1, length(s))))
    s <- s[w]
    keepers <- s
    
    # create some more s values at midpoints
    ds <- diff(s)
    mid.s <- s[-1] - .5 * ds
    s <- sort( c(s, mid.s) )
    if (quiet) {
      suppressWarnings(y <- do.call(f, args=c(list(s), args)))
    } else {
      y <- do.call(f, args=c(list(s), args))
    }
  }

#    print(list(i=iteration, s=c(head(s), tail(s))))
  return(s[is.finite(y)])
}

#' Extract summary statistics
#' 
#' Extract confidence intervals, test statistics or p-values from an 
#'   `htest` object.
#' @rdname interval
#' @param object a fitted model object or an htest object.
#' @param parm a specification of which parameters are to be given confidence intervals, 
#'   either a vector of numbers or a vector of names. If missing, all parameters are considered.
#' @param level the confidence level required.
#' @param x An object of class `htest`.
#' @param \dots Additional arguments.
#' @return the extracted p-value, confidence interval, or test statistic
#' 
#' @examples
#' confint(t.test(rnorm(100)))
#' pval(t.test(rnorm(100)))
#' stat(t.test(rnorm(100)))
#' confint(var.test(rnorm(10,sd=1), rnorm(20, sd=2)))
#' pval(var.test(rnorm(10,sd=1), rnorm(20, sd=2)))
#' if (require(mosaicData)) {
#' data(HELPrct)
#' stat(t.test (age ~ shuffle(sex), data=HELPrct))
#' # Compare to test statistic computed with permuted values of sex.
#' do(10) * stat(t.test (age ~ shuffle(sex), data=HELPrct))
#' }
#' @keywords stats 
#' @keywords inference 

# remove inverval alias to avoid conflict with lubridate::interval
# interval <- confint

#' @param verbose a logical
#' @export

confint.htest <- function (object, parm, level, ...){
  if (! missing( parm ) || !missing( level ) ) { warning("parm and level are ignored.") }
  int <- object$conf.int
  lev <- attr(int, "conf.level")
  verbose <- list(...)[['verbose']]
  if (is.null(verbose)) verbose <- FALSE
  if ( verbose ) {
	  cat('\nMethod: ')
	  cat(object$method)
	  cat('\nEstimate: ')
	  cat( format(object$estimate, getOption('digits',3)) )
	  cat( paste("\n", lev * 100, "% confidence interval: ", sep = "") )
	  cat( paste(format(as.vector(int), getOption('digits',3)), collapse=" ") )
  }
  interv <- as.vector(int) 
  names(interv) <- c('lower','upper')
  level <- c(level=lev)
  int <- c(object$estimate, interv, level )
  int <- 
    as.data.frame(
      matrix(nrow = 1, c(object$estimate, lower = interv[1], upper = interv[2], level = level)))
  names(int) <- c(names(object$estimate), "lower", "upper", "level")
  if (verbose) 
    invisible(int) 
  else 
    int
}

#' @rdname interval
#' @keywords stats 
#' @keywords inference 
#' @export

pval <- function(x, ...){UseMethod("pval", x)}

#' @rdname interval
#' @param digits number of digits to display in verbose output
#' @export

pval.htest <- function (x, digits=4, verbose=FALSE, ...){
  pval <- x$p.value
  if (!verbose) {
    return( c(p.value=pval) )
  }
  # verbose stuff below
  stat <- x$statistic
  param <- x$parameter
  method <- x$method
  cat('\n')
  cat( paste('Method: ', method, "\n", sep="") )
  
  tryCatch( {
    alt <- x$alternative
    direction <- switch(alt, 
                        'less' = ' < ',
                        'greater' = ' > ',
                        'two.sided' = ' <> ',
                        ' <> '
    )
    null <- x$null.value
    cat('\n\n')
    cat(paste(
      'Null Hypothesis: ', 
      names(null), 
      " = ", 
      null,
      sep="") 
    )  
    cat('\n')
    cat(paste(
      'Alt. Hypothesis: ', 
      names(null), 
      direction, 
      null,
      sep="") 
    )  
    estimate <- x$estimate
    cat('\n\n')
    cat(paste(names(stat), " = ", 
              signif(stat,digits=digits),
              sep="") )  
    cat('  (')
    cat( paste( 
      names(param), " = ", 
      signif(param,digits=digits), 
      sep="",
      collapse=', ') )  
    cat(')\n\n')
  }, error=function(e) {}
  )
  cat( paste("p-value = ", signif(pval,digits), sep="") ) 
  cat('\n\n')
  return(invisible(c(p.value=pval)))
}

#' @rdname interval
#' @keywords stats 
#' @keywords inference 
#' @export

stat <- function(x,...) { UseMethod("stat", x)}

#' @rdname interval
#' @export

stat.htest <- function(x,...) {
	x $ statistic
}

# This is added for compatibility with ggplot2 version 2.3

#' @rdname interval
#' @importFrom utils packageVersion
#' @export
#' 
stat.uneval <- function(x, ...) {
  if (utils::packageVersion("ggplot2") <= "2.2.1") {
    stop("ggplot2 > 2.2.1 required for ggplot2::stat().")
  } else {
	  ggplot2::stat(x, ...)
  }
}


#' Extract r-squared value
#' 
#' Attempts to extract an r-squared value from a model or model-like object.
#' @param x an object
#' @param \dots additional arguments
#' @export

rsquared <- function(x, ...) {
  NULLFUN <- function(e) NULL 
  result <- tryCatch( x$r.squared, error=NULLFUN)
  if (is.null(result)) 
    result <- tryCatch( summary(x, ...)$r.squared, error=NULLFUN )
  return(result)
}

#' @rdname defunct
#' @export

r.squared <- function(...) {
  .Defunct("rsquared")
}

#' Mustang Prices
#' 
#' Mustang Prices
#' 
#' #' @docType data
#' @name Mustangs
#' @usage data(Mustangs)
#' @format  A data frame with 25 observations on the following 3 variables.
#' \describe{
#'    \item{\code{Age }}{age of vehicle in years}
#'    \item{\code{Miles }}{1000s of miles driven}
#'    \item{\code{Price }}{selling price in 1000s USD}
#' }
#' @source These data were used in a "resampling bake-off" hosted by Robin Lock.
#' 
#' @details 
#' A student collected data on the selling prices for a sample of used Mustang
#' cars being offered for sale at an internet website. 
#'
"Mustangs"

#' @importFrom stats reorder
NA

#' Additional interfaces to TukeyHSD
#' 
#' [TukeyHSD()] requires use of [aov()].  
#' Since this is a hindrance for beginners, wrappers
#' have been provided to remove this need.
#' @rdname TukeyHSD.lm
#' @param x an object, for example of class `lm` or `formula`
#' @param data a data frame.  NB: This does not come second in the argument list.
#' @param which,ordered,conf.level,\dots just as in [TukeyHSD()] from the `base` package
#' 
#' @export
TukeyHSD.lm <- function(x, which, ordered = FALSE, conf.level=0.95, ...) {
	stats::TukeyHSD( aov(x), which = which, ordered = ordered, conf.level = conf.level, ...)
}

#' @rdname TukeyHSD.lm
#' @examples
#' ## These should all give the same results
#' if (require(mosaicData)) {
#'   model <- lm(age ~ substance, data=HELPrct)
#'   TukeyHSD(model)
#'   TukeyHSD( age ~ substance, data=HELPrct)
#'   TukeyHSD(aov(age ~ substance, data=HELPrct))
#' }
#' @export
TukeyHSD.formula <- function(x, which, ordered = FALSE, conf.level=0.95, data=parent.frame(), ...) {
  TukeyHSD( lm(x, data=data, ...), which = which, ordered = ordered, conf.level=conf.level, ...)
}

#' Convert objects to xtabs format
#' 
#' Convert a data frame or a matrix into an `xtabs` object.
#' 
#' The intended use is to convert a two-way contingency table stored in a data
#' frame or a matrix into an `xtabs` object.
#' 
#' @aliases as.xtabs as.xtabs.data.frame as.xtabs.matrix
#' @param x object (typically a data frame) to be converted to `xtabs`
#' format
#' @param ... additional arguments to be passed to or from methods.
#' @return An `xtabs` object.
#' @keywords manipulate
#' @rdname as-xtabs
#' @examples
#' # example from example(fisher.test)
#' df <- data.frame( X=c('Tea','Milk'), Tea=c(3,1), Milk=c(1,3) )
#' xt <- as.xtabs(df, rowvar="Guess", colvar="Truth"); xt
#' if (require(vcd)) { mosaic(xt) }
#' @export

as.xtabs <- function(x, ...) { UseMethod('as.xtabs') }

#' @rdname as-xtabs
#' @param rowvar name of the row variable as character string
#' @param colvar name of the column variable as character string
#' @param labels column of data frame that contains the labels of the row
#' variable.
#' @export
as.xtabs.data.frame <- function(x, rowvar=NULL, colvar=NULL, labels=1, ...) {

  if (labels >= 1) {
  	cnames <- names(x)[-1]
  	m <- as.matrix(x[,-c(labels)])
  } else {
  	cnames <- names(x)
	m <- as.matrix(x)
  }

  rnames <- x[,labels]
  rownames(m) <- rnames
  if (! is.character(rowvar) ) { rowvar <- "variable.1" }
  if (! is.character(colvar) ) { colvar <- "variable.2" }
  
  dn <- list( rnames, cnames)
  names(dn) <- c(rowvar, colvar)
  attr(m,'dimnames') <- dn
  class(m) <- c('xtabs', 'table')
  return(m)
}

#' @rdname as-xtabs
#' @export
as.xtabs.matrix <- function(x, rowvar=NULL, colvar=NULL, ...)  {
  rnames <- rownames(x)
  cnames <- colnames(x)
  rownames(m) <- rnames
  if (! is.character(rowvar) ) { rowvar <- "variable.1" }
  if (! is.character(colvar) ) { colvar <- "variable.2" }
  
  dn <- list( rnames, cnames)
  names(dn) <- c(rowvar, colvar)
  attr(m,'dimnames') <- dn
  class(m) <- c('xtabs', 'table')
  return(m)
}

# context("as.xtabs()")

df <- data.frame( X=c('Tea','Milk'), Tea=c(3,1), Milk=c(1,3) )
xt <- as.xtabs(df, rowvar="Guess", colvar="Truth")
xt

testthat::test_that("as.xtabs works", {
  example <- structure(c(3, 1, 1, 3), .Dim = c(2L, 2L), .Dimnames = list(Guess = c("Tea", 
                                                                                   "Milk"), Truth = c("Tea", "Milk")), class = c("xtabs", "table"
                                                                                   ))
  df <- data.frame( X=c('Tea','Milk'), Tea=c(3,1), Milk=c(1,3) )
  xt <- as.xtabs(df, rowvar="Guess", colvar="Truth")
  xt
  expect_equal(example, xt)
})

#' The Data Distribution
#' 
#' Density, distribution function, quantile function, and random generation
#' from data.

#' @rdname pqrdata
#' @keywords distribution 
#' @param p a vector of probabilities
#' @param q a vector of quantiles
#' @param formula a formula or a vector
#' @param data a data frame in which to evaluate `formula`
#' @param \dots additional arguments passed to `quantile` or `sample`
#' @return For `qdata`, a vector of quantiles
#' @examples
#' data(penguins, package = "palmerpenguins")
#' qdata(flipper_length_mm ~ species, 0.5, data = penguins)
#' qdata( ~ flipper_length_mm, p = 0.5, groups = species, data = penguins)
#' qdata(penguins$flipper_length_mm, p = 0.5)
#' qdata( ~ flipper_length_mm, p = 0.5, data = penguins)
#' qdata( ~ flipper_length_mm, p = 0.5, groups = species, data = penguins)
#' @export

qdata <- function( formula, p = seq(0, 1, 0.25), data = NULL, ...) { 
  vals_call <- substitute(formula)
  if (! is.numeric(p) || any(p < 0) || any(p > 1)) {
    stop("Invalid values of p.  Are your arguments in the wrong order?")
  }
  if (is.null(data)) { data <- parent.frame() }
  args <- eval(substitute(alist(vals_call, ...)))
  args [["p"]] <- p 
  args [["data"]] <- data
  do.call( "qdata_f", args )
}


#' The Data Distribution
#' 
#' Utility functions for density, distribution function, quantile function, 
#' and random generation from data.
#' 
#' @rdname pqrdata2
#' @seealso  [ddata()], [pdata()], [qdata()],  
#' [rdata()], [cdata()]
#' @param p a vector of probabilities
#' @param q a vector of quantiles
#' @param x a vector containing the data
#' @param data a data frame in which to evaluate `formula`
#' @param n number of values to sample
#' @param replace  a logical indicating whether to sample with replacement
#' @param groups a grouping variable, typically the name of a variable in `data`
#' @param \dots additional arguments passed to `quantile` or `sample`
#' @param na.rm a logical indicating whether `NA`s should be removed before computing.
#' @param log  a logical indicating whether the result should be log transformed
#' @export
 
qdata_v <- function( x, p = seq(0, 1, 0.25), na.rm = TRUE, ... ) {
  .check_for_quant_input(x)
  if (any(p > 1) | any(p < 0)) 
    stop("Prob outside of range 0 to 1.  Do you perhaps want pdata?")
  qs <- quantile(x, probs = p, na.rm = na.rm, ...)
  return(qs)
  
  if (length(p) == 1) {
    # result <- setNames( c(p, qs), c("p","quantile") )
    result <- data.frame(quantile = qs, p = p)
  } else {
    result <- data.frame(quantile = qs, p = p)
  }
  result
}

#' @rdname pqrdata2
#' @export

qdata_f <- aggregatingFunction1(qdata_v, output.multiple = TRUE, na.rm = TRUE)

#' @rdname pqrdata
#' @return for `cdata`, a data frame giving
#' upper and lower limits and the central proportion requested
#' @examples
#' data(penguins, package = 'palmerpenguins')
#' cdata(penguins$flipper_length_mm, 0.5)
#' cdata( ~ flipper_length_mm, 0.5, data = penguins)
#' cdata( ~ flipper_length_mm, 0.5, data = penguins)
#' cdata( ~ flipper_length_mm | species, data = penguins, p = .5)
#' @export

cdata <- function( formula, p = 0.95, data = NULL, ...) { 
  vals_call <- substitute(formula)
  if (! is.numeric(p) || any(p < 0) || any(p > 1)) {
    stop("Invalid values of p.  Are your arguments in the wrong order?")
  }
  if (is.null(data)) { data <- parent.frame() }
  
  args <- eval(substitute(alist(vals_call, ...)))
  args [["p"]] <- p # substitute(p, parent.frame())
  args [["data"]] <- data
  do.call( "cdata_f", args )
}
#' @rdname pqrdata2
#' @export

cdata_v <- function( x, p = .95, na.rm = TRUE, ... ) {
  lo_p <- (1-p)/2
  hi_p <- 1 - lo_p
  lo <- quantile( x, lo_p, na.rm = na.rm, ... )
  hi <- quantile( x, hi_p, na.rm = na.rm, ... )
  if (length(p) == 1) {
    # result <- setNames( c(lo, hi, p), c("lower", "upper", "central.p") )
    result <- data.frame(lower = lo, upper = hi, central.p = p)
  } else {
    result <- data.frame(lower = lo, upper = hi, central.p = p)
  }
  return(result)
}

# xcdata <- function( formula, p = 0.95, data = NULL, ...) { 
#   vals_call <- substitute(formula)
#   if (! is.numeric(p) || any(p < 0) || any(p > 1)) {
#     stop("Invalid values of p.  Are your arguments in the wrong order?")
#   }
#   
#   args <- eval(substitute(alist(vals_call, ...)))
#   args [["p"]] <- p # substitute(p, parent.frame())
#   args [["data"]] <- data
#   cd <- do.call( "cdata_f", args )
#   data |>
#     mutate(
#       status = dplyr::case_when(
#         cd$low ~ "tail"
#   gf_histogram(formula, data = data, fill = ~ cut(cd[1:2]
#   
# }

#' @rdname pqrdata2
#' @export

cdata_f <- aggregatingFunction1( cdata_v, output.multiple = TRUE, na.rm = TRUE )

  
#' @rdname pqrdata
#' @return For `pdata`, a vector of probabilities
#' @examples
#' data(penguins, package = 'palmerpenguins')
#' pdata(penguins$flipper_length_mm, 3:6)
#' pdata( ~ flipper_length_mm, 3:6, data = penguins)
#' @export

pdata <- function (formula, q, data = NULL, ...) 
{
  vals_call <- substitute(formula)
  args <- eval(substitute(alist(vals_call, ...)))
  if (is.null(data)) { data <- parent.frame() }
  args[["q"]] <- q
  args[["data"]] <- data
  do.call("pdata_f", args)
}



#' @param lower.tail a logical indicating whether to use the lower or upper tail probability
#' @rdname pqrdata2
#' @export

pdata_v <- function(x, q, lower.tail = TRUE, ... ) {
  .check_for_quant_input(x)
  n <- sum( ! is.na(x) )
  probs <- sapply( q, function(q) { sum( x <= q , na.rm = TRUE ) } ) / n
  if (lower.tail) { 
  	return(probs)
  } else {
	return( 1 - probs )
  }
}

#' @rdname pqrdata2
#' @export

pdata_f <- aggregatingFunction1( pdata_v, output.multiple = TRUE, na.rm = TRUE )



#' @rdname pqrdata
#' @param n number of values to sample
#' @return For `rdata`, a vector of sampled values.
#' @examples
#' data(penguins, package = 'palmerpenguins')
#' rdata(penguins$species, 10)
#' rdata( ~ species, n = 10, data = penguins)
#' rdata(flipper_length_mm ~ species,  n = 5, data = penguins)
#' @export

rdata <- function (formula, n, data = NULL, ...) 
{
  vals_call <- substitute(formula)
  if ( ! is.numeric(n) || n <= 0 ) {
    stop("Invalid value of n.  Are our arguments in the correct order?")
  }
  if (is.null(data)) { data <- parent.frame() }
  args <- eval(substitute(alist(vals_call, ...)))
  args[["n"]] <- n
  args[["data"]] <- data
  do.call("rdata_f", args)
}
 
#' @rdname pqrdata2
#' @export

rdata_v <- function(x, n, replace = TRUE, ... ) {
  sample( x, n, replace = replace, ...)
}

#' @rdname pqrdata2
#' @export

rdata_f <- aggregatingFunction1( rdata_v, output.multiple = TRUE, na.rm = TRUE )

#' @rdname pqrdata
#' @return For `ddata`, a vector of probabilities (empirical densities)
#' @examples
#' data(penguins, package = 'palmerpenguins')
#' ddata(penguins$species, 'setosa')
#' ddata( ~ species, 'setosa', data = penguins)
#' @export

ddata <- function (formula, q, data = NULL, ...) 
{
  vals_call <- substitute(formula)
  args <- eval(substitute(alist(vals_call, ...)))
  if (is.null(data)) { data <- parent.frame() }
  args[["q"]] <- q
  args[["data"]] <- data
  do.call("ddata_f", args)
}

#' @rdname pqrdata2
#' @export

ddata_v <- function(x, q, ..., data = NULL, log = FALSE, na.rm = TRUE) {
  if( !is.null(data) ) {
    vals <- eval( substitute(x), data, enclos = parent.frame())
  } else {
    vals <- eval(substitute(x), parent.frame())
  }
  n <- sum( ! is.na(vals) )
  probs <- sapply(q, function(x) { sum( vals == x, na.rm = na.rm) / n } )
  if (log) { probs <- log(probs) }
  return (probs)
}

#' @rdname pqrdata2
#' @export

ddata_f <- aggregatingFunction1( ddata_v, output.multiple = TRUE, na.rm = TRUE )

.check_for_quant_input <- function(x) {
  if (is.data.frame(x) ) {
    stop("Give a variable, not a data frame.")
  }
  
  if (is.factor(x) | is.character(x) ) {
    stop("Input must be a numerical variable, not categorical.")
  }
  return(TRUE)
}

# context('pdata(), qdata(), and rdata()')


test_that("formula interface works.", {
	expect_equal(ignore_attr = TRUE,  length(rdata( ~age, 10, data = mosaicData::HELPrct)), 10 )
	expect_equal(ignore_attr = TRUE,  pdata( ~age, 30, data = mosaicData::HELPrct), prop( ~ mosaicData::HELPrct$age <= 30) )
	expect_equal(ignore_attr = TRUE,  qdata( ~age, .75, mosaicData::HELPrct), quantile(mosaicData::HELPrct$age, .75) )
	expect_equal(ignore_attr = TRUE,  qdata(~ age, c(.25,.75), mosaicData::HELPrct), quantile(mosaicData::HELPrct$age, c(.25,.75)) )
})

test_that("vector/data interface works.", {
	expect_equal(ignore_attr = TRUE,  length(rdata( mosaicData::HELPrct$age, 10, mosaicData::HELPrct)), 10 )
	expect_equal(ignore_attr = TRUE,  pdata(mosaicData::HELPrct$age, 30, data = mosaicData::HELPrct), prop(~ mosaicData::HELPrct$age <= 30) )
	expect_equal(ignore_attr = TRUE,  qdata(mosaicData::HELPrct$age, .75, data = mosaicData::HELPrct), quantile(mosaicData::HELPrct$age, .75) )
	expect_equal(ignore_attr = TRUE,  qdata(mosaicData::HELPrct$age, c(.25,.75), data = mosaicData::HELPrct), quantile(mosaicData::HELPrct$age, c(.25,.75)) )
})


test_that("vector (no data) interface works.", {
	expect_equal(ignore_attr = TRUE,  length(rdata( n = 10, mosaicData::HELPrct$age)), 10 )
	expect_equal(ignore_attr = TRUE,  pdata(mosaicData::HELPrct$age, 30), prop( ~ mosaicData::HELPrct$age <= 30) )
	expect_equal(ignore_attr = TRUE,  qdata(mosaicData::HELPrct$age, .75), quantile(mosaicData::HELPrct$age, .75) )
	expect_equal(ignore_attr = TRUE,  qdata( p = c(.25,.75), mosaicData::HELPrct$age), 
	                   quantile(mosaicData::HELPrct$age, c(.25,.75)) )
})

#test_that("error messages generated", {
#	expect_error( pdata( 30, ~age | sex, mosaicData::HELPrct) )
#	expect_error( qdata( .75, ~age | sex, mosaicData::HELPrct) )
#	expect_error( rdata( 30, ~age | sex, mosaicData::HELPrct) )
#	expect_error( qdata( 30, ~age, mosaicData::HELPrct) )
#})

#' Create bar graphs from raw data
#'
#' [lattice::barchart()] from the `lattice` package makes bar graphs from
#' pre-tabulated data.  Raw data can be tabulated using [xtabs()], but the syntax
#' is unusual compared to the other lattice plotting functions.  `bargraph` provides 
#' an interface that is consistent with the other `lattice` functions.
#'
#' @param x a formula describing the plot
#' @param data a data frame in which the formula `x` is evaluated
#' @param groups a variable or expression used for grouping.  See [lattice::barchart()].
#' @param horizontal a logical indicating whether bars should be horizontal
#' @param \dots additional arguments passed to [lattice::barchart()]
#' @param origin beginning point for bars.  For the default behavior used by 
#'        [lattice::barchart()] set `origin` to `NULL`, but
#'         0 is often a better default. If 0 is not good, perhaps you should use
#'        a different kind of plot as the results may be misleading.
# #' @param subset a vector used to subset \code{data}.  This may be an expression that
# #' will be evaluated within \code{data}.
#' @param ylab a character vector of length one used for the y-axis label
#' @param xlab a character vector of length one used for the x-axis label
#' @param type one of `"frequency"`, `"count"`, `"percent"`, or `"proportion"`
#'   indicating what type of scale to use.  Unique prefixes are sufficient.
#' @param auto.key a logical expression indicating whether a legend should be automatically produced
#' @param scales is a list determining how the x- and y-axes are drawn
#' @return a trellis object describing the plot
#' @seealso [lattice::barchart()]
#' @details `bargraph(formula, data=data, ...)` works by creating a new data frame from `xtabs(formula, data=data)`
#' and then calling [lattice::barchart()] using modified version of the formula and this
#' new data frame as inputs.  This has implications on, for example, conditional plots where 
#' one desires to condition on some expression that will be evaluated in `data`.  This typically
#' does not work because the required variables do not exist in the output of `xtabs`.  One solution
#' is to first add a new variable to `data` first and then to condition using this new variable.
#' See the examples.
#'
#' @examples
#' if (require(mosaicData)) {
#' data(HELPrct)
#' bargraph( ~ substance, data = HELPrct)
#' bargraph( ~ substance, data = HELPrct, horizontal = TRUE)
#' bargraph( ~ substance | sex, groups = homeless, auto.key = TRUE, data = HELPrct)
#' bargraph( ~ substance, groups = homeless, auto.key=TRUE, 
#'             data = HELPrct |> filter(sex == "male"))
#' HELPrct2 <- mutate(HELPrct, older = age > 40)
#' bargraph( ~ substance | older, data = HELPrct2)
#' }
#' @export

bargraph <- function(x, data = parent.frame(), groups = NULL, horizontal = FALSE, origin = 0, 
                     ylab = ifelse(horizontal, "", type), 
                     xlab = ifelse(horizontal, type, ""), 
                     type = c("count", "frequency", "proportion", "percent"),
                     auto.key = TRUE,
                     scales = list(),
                     ...) {
  type <- match.arg(type)
  if (type == "frequency") type <- "count"
  
  if (!inherits(x, "formula") || !is.null(lhs(x)))
    stop("first argument should be a formula with no lhs.")
 
  xtab0 <- 
    tally(x, data = data, groups = !!rlang::enexpr(groups), format = type, groups.first = TRUE)
    # tally(x, data = data, groups = groups, format = type, groups.first = TRUE)
    # if (is.null(groups)) {
    #   tally(x, data = data, format = type, groups.first = TRUE)
    # } else {
    #   tally(x, data = data, groups = !!rlang::enexpr(groups), format = type, groups.first = TRUE)
    # }
  
  xtab <- as.data.frame(xtab0) 
  # grab the last variable name, to handle the case that 
  # there is a variable called "Freq"
  lastvar <- names(xtab)[ncol(xtab)]
  # add dummy variable to handle case when 
  xtab <- xtab |> dplyr::mutate(..X.. = xtab[, 1])
  sgroups <- substitute(groups)
  
  
  
  if (horizontal) {
    if (! is.null(condition(x))){
      formula <- as.formula( paste("..X.. ~ ", lastvar, " | ", deparse(condition(x)) ) )
    } else {
      formula <- as.formula( paste("..X.. ~ ", lastvar) )
    }
  } else {
    if (! is.null(condition(x))){
      formula <- as.formula(paste(lastvar, " ~ ..X.. | ", deparse(condition(x))))
    } else {
      formula <- as.formula(paste(lastvar, " ~ ..X..")) 
    }
    if(length(scales) == 0 ){
      scales = list(x = list(rot = 30))
    }
  }
  if (xlab == "" && ! horizontal) { xlab <- names(dimnames(xtab0))[1] }
  if (ylab == "" &&   horizontal) { ylab <- names(dimnames(xtab0))[1] }
  
  barchart(formula, data = xtab, groups = eval(sgroups), 
           origin=origin, ylab = ylab, xlab = xlab, auto.key = auto.key, scales=scales, ...)
}

# context("bargraph()")
testthat::test_that("bargraph works", {
  
  require(mosaicData) 
  data(HELPrct)
  HELPrct2 <- mutate(HELPrct, older = age > 40)
  
  wrapped_expect_doppelganger("bargraph1", bargraph( ~ substance, data = HELPrct))
  wrapped_expect_doppelganger("bargraph2", bargraph( ~ substance, data = HELPrct, horizontal = TRUE))
  wrapped_expect_doppelganger("bargraph3", bargraph( ~ substance | sex, groups = homeless, auto.key = TRUE, data = HELPrct))
  wrapped_expect_doppelganger("bargraph4", bargraph( ~ substance, groups = homeless, auto.key=TRUE,
                                                     data = HELPrct |> filter(sex == "male")))
  wrapped_expect_doppelganger("bargraph5", bargraph( ~ substance | older, data = HELPrct2))
})

#' @rdname defunct-fetch
#' @export

fetchGapminder1 <- function(...) {
  .Defunct(msg="Use fetchGapminder1() from the fetch package instead.")
}

#' @rdname defunct-fetch
#' @export
fetchGapminder <- function(...) {
  .Defunct(msg="Use fetchGapminder() from the `fetch' package instead.")
}

utils::globalVariables(c("nlab", "cover"))

#' Compute confidence intervals from (multiple) simulated data sets
#' 
#' This function automates the calculation of coverage rates for exploring
#' 	the robustness of confidence interval methods.
#' 
#' @param n size of each sample
#' @param samples number of samples to simulate
#' @param rdist function used to draw random samples
#' @param args arguments required by `rdist`
#' @param plot one of `"print"`, `"return"`, `"horizontal"`, or `"none"` 
#'   describing whether a plot should be printed, returned, printed with horizontal intervals,
#'   or not generated at all.
#' @param estimand true value of the parameter being estimated
#' @param conf.level confidence level for intervals
#' @param method function used to compute intervals.  Standard functions that 
#' 	  produce an object of class `htest` can be used here.
#' @param method.args arguments required by `method`
#' @param interval a function that computes a confidence interval from data.  Function
#' 	  should return a vector of length 2.
#' @param estimate a function that computes an estimate from data
#' @param verbose print summary to screen?
#' 
#' 
#' @return A data frame with variables 
#' 	`lower`,
#' 	`upper`,
#' 	`estimate`,
#' 	`cover` ('Yes' or 'No'),
#' 	and 
#' 	`sample`
#' 	is returned invisibly.  See the examples for a way to use this to display the intervals
#' 	graphically.
#'
#' 
#' @examples
#' # 1000 95% intervals using t.test; population is N(0,1)
#' CIsim(n = 10, samples = 1000)    
#' # this time population is Exp(1); fewer samples, so we get a plot 
#' CIsim(n = 10, samples = 100, rdist = rexp, estimand = 1) 
#' # Binomial treats 1 like success, 0 like failure
#' CIsim(n = 30, samples = 100, rdist = rbinom, args = list(size = 1, prob = .7), 
#'        estimand = .7, method = binom.test, method.args = list(ci = "Plus4"))  
#' 
#' @keywords inference 
#' @keywords simulation 
#' 
#' @export


# this is borrowed from fastR.  If it stays in mosaic, it should be removed from fastR

CIsim <-
  function (n, samples = 100, rdist = rnorm, args = list(), 
            plot = if (samples <= 200) "draw" else "none",
            estimand = 0, 
            conf.level = 0.95, method = t.test, method.args = list(),
            interval = function(x) {
              do.call(method, c(list(x, conf.level = conf.level), method.args))$conf.int
            }, estimate = function(x) {
              do.call(method, c(list(x, conf.level = conf.level), method.args))$estimate
            }, verbose = TRUE) 
{
    plot <- match.arg(plot, c("draw", "horizontal", "return", "none"))
    # Grid will have a row for each simulated sample
    Grid <- expand.grid(n = n, sample = 1:samples) |> mutate(estimand = estimand)
    # a list of data sets, one for each row in Grid
    sampleData <- 
      lapply(1:nrow(Grid),
             function(r) do.call(rdist, c(list(n = Grid[r, "n"]),  args)))
    CIs <- Grid |> mutate(
      lower    = sapply(sampleData, function(x) { interval(x)[1] }),
      upper    = sapply(sampleData, function(x) { interval(x)[2] }),
      estimate =  sapply(sampleData, function(x) { estimate(x) }),
      cover    = (estimand <= lower) + (estimand < upper),
      cover    = factor(cover, levels = 0L:2L, labels = c("Low", "Yes", "High")),
      nlab     = reorder(factor(paste0("n = ", n)), n)
    )
                    
    if (verbose) {
      message("Interval coverage:")
      message(paste(
        capture.output(t(tally(~ cover | n, data = CIs, format = "prop"))), 
        collapse = "\n"))
    }
    
    plotG <- 
      ggplot(aes(x = sample, y = estimate, ymin = lower, ymax = upper), data = CIs) + 
        geom_errorbar(aes(color = cover)) +
        geom_point(aes(color = cover), size = 0.7) + 
        geom_abline(slope = 0, intercept = estimand, alpha = 0.4) +
        scale_colour_manual(drop  =  FALSE, breaks = c("High", "Yes", "Low"),
                              values = c("red", "navy", "red"), guide = 'none')
      
    switch(plot,
           return = return(plotG + facet_wrap(~ nlab)),
           horizontal = print(plotG + coord_flip() + facet_wrap(~ nlab)),
           draw = print(plotG + facet_wrap(~ nlab)),
           none = {}
           )
    return(invisible(CIs |> select(-nlab)))
}

# testthat::test_that("Clsim works", {
#   wrapped_expect_doppelganger("CIsim1", CIsim(n=10, samples=100, verbose = FALSE))
#   wrapped_expect_doppelganger("CIsim2", CIsim(n=10, samples=100, rdist=rexp, estimand=1, verbose = FALSE))
#   wrapped_expect_doppelganger("CIsim3", CIsim(n=30, samples=100, rdist=rbinom, args=list(size=1, prob=.7),
#                                              estimand = .7, method = binom.test, method.args=list(ci = "Plus4"), verbose = FALSE))
# })

.mosaicEnv <- new.env()
assign("mosaic.theme",   list(), envir = .mosaicEnv)
assign("mosaic.options", list(), envir = .mosaicEnv)
assign("mosaic.manipulate.initialized", FALSE, envir = .mosaicEnv)

.onLoad <- function(libname, pkgname) {
  ## library.dynam("mosaic", pkgname, libname )
 
  pks <- invisible(suppressPackageStartupMessages(
    sapply(pkgs_to_attach, requireNamespace, quietly = TRUE)
  ))
  for (p in pkgs_to_attach) {
    if (! is_attached(p)) suppressPackageStartupMessages(attachNamespace(p))
  }
  
  mosaic.par.set(.defaultMosaicOptions())
}

.onAttach <- function(libname, pkgname) {

  # have histogram use xhistogram stuff by default
  origLatticeOptions <- lattice::lattice.options(
    histogram.breaks = xhistogramBreaks,
    prepanel.default.histogram = prepanel.xhistogram,
    panel.histogram = panel.xhistogram)
  assign("original.lattice.options", origLatticeOptions, envir = .mosaicEnv)
  packageStartupMessage(
    paste0("\nThe 'mosaic' package masks several functions from core packages ",
           "in order to add \nadditional features.  ",
           "The original behavior of these functions should not be affected by this."
           # "\n\nNote: If you use the Matrix package, be sure to load it BEFORE loading mosaic.",
           # "\n\nHave you tried the ggformula package for your plots?"
           ),
    appendLF = TRUE)
}

is_attached <- function(x) {
  paste0("package:", x) %in% search()
}

.noGenerics <- FALSE

.onUnload <- function(libpath) {
    ## library.dynam.unload("mosaic", libpath)
}


pkgs_to_attach <-
  c("lattice", "ggplot2", "Matrix", "dplyr", "ggformula", "mosaicData")
  # c("ggstance", "ggridges")

#' Some favorite statistical summaries
#' 
#' Likely you mean to be using [favstats()].  Each of these computes the 
#' mean, standard deviation, quartiles, sample size and number of missing values for a numeric vector,
#' but [favstats()] can take a formula describing how these summary statistics 
#' should be aggregated across various subsets of the data.
#' @param x  numeric vector 
#' @param na.rm  boolean indicating whether missing data should be ignored 
#' @param type an integer between 1 and 9 selecting one of the nine quantile algorithms detailed
#'  in the documentation for [stats::quantile()]
#' @param ... additional arguments (currently ignored)
#' 
#' @return A vector of statistical summaries
#' @keywords stats
#' @examples
#' fav_stats(1:10)
#' fav_stats(faithful$eruptions)
#' data(penguins, package = "palmerpenguins")
#' 
#' # Note: this is favstats() rather than fav_stats()
#' favstats(bill_length_mm ~ species, data = penguins)  
#' @export

fav_stats <- function (x, ..., na.rm = TRUE, type = 7) 
{
  if (!is.null(dim(x)) && min(dim(x)) != 1) 
    warning("Not respecting matrix dimensions.  Hope that's OK.")
  # x <- as.vector(x)
  if (! is.numeric(x)) {
    warning("Auto-converting ", class(x), " to numeric.")
    x <- as.numeric(x)
    if (!is.numeric(x)) stop("Auto-conversion to numeric failed.")
  }
  
  qq <- if (na.rm) 
    stats::quantile(x, na.rm = na.rm, type = type)
  else 
    rep(NA, 5)
  val <- data.frame(
    min=qq[1],  
    Q1 = qq[2], 
    median = qq[3], 
    Q3 = qq[4], 
    max = qq[5],
  #  iqr = stats::IQR(x, na.rm = na.rm, type =type),
    mean = base::mean(x, na.rm = na.rm), 
    sd = stats::sd(x, na.rm = na.rm), 
    n = base::sum(! is.na(x)),
    missing = base::sum( is.na(x) )
  )
  rownames(val) <- ""
  return(val)
}

#' @import mosaicCore
#' @export 
mosaicCore::makeFun

#' @import ggformula
#' @import ggplot2

NA

#' mosaic
#'
#' Data sets and utilities from Project MOSAIC (mosaic-web.org) used to teach mathematics, 
#' statistics, computation and modeling.  Funded by the NSF, Project MOSAIC is a 
#' community of educators working to tie together aspects of quantitative work that students 
#' in science, technology, engineering and mathematics will need in their professional lives, 
#' but which are usually taught in isolation, if at all.
#'
#'
#' @name mosaic-package
#' @aliases mosaic mosaic-package
#' @title mosaic: the Project MOSAIC package
#' @author Randall Pruim (\email{rpruim@@calvin.edu}), Daniel Kaplan (\email{kaplan@@macalester.edu}), Nicholas Horton (\email{nhorton@@smith.edu})
#' @references
#' \url{http://www.mosaic-web.org}
#'
#' @keywords package
#' @importFrom Matrix mean
#' @importFrom utils head modifyList
# #' @importFrom gridExtra grid.arrange arrangeGrob
# #' @importFrom ggdendro segment label dendro_data
#' @import methods
#' @importFrom MASS fitdistr fractions
#' @importFrom dplyr bind_rows bind_cols 
#' @importFrom dplyr filter select mutate summarise group_by arrange ungroup 
#' @importFrom tibble tibble rownames_to_column
#' @importFrom dplyr do n first left_join 
#' @importFrom tidyr gather
#' @importFrom grid grid.polyline grid.text grid.lines grid.points grid.rect grid.layout gpar unit.c 
#' @import lattice
#' @import mosaicData
#' @importFrom grDevices col2rgb colorRampPalette gray heat.colors rgb topo.colors
#' @importFrom graphics hist plot stem
#' @importFrom stats TukeyHSD nls
#' @importFrom stats df
#' @importFrom stats addmargins aggregate anova aov approxfun as.formula chisq.test coef
#' @importFrom stats complete.cases confint cutree dbeta dcauchy dchisq dexp dgamma dist dlnorm dnorm
#' @importFrom stats dpois dt dweibull formula lm loess loess.control lsfit model.frame model.matrix
#' @importFrom stats  model.response na.omit pnorm ppoints predict printCoefmat qnorm qt rbinom resid
#' @importFrom stats rmultinom rnorm runif setNames splinefun terms uniroot update vcov wilcox.test xtabs
#' @importFrom utils browseURL capture.output menu read.csv read.table str tail head

"_PACKAGE"

utils::globalVariables(c('densy','densx','dots')) 
#' @importFrom mosaicCore named named_among unnamed
NA

#' Plots of Discrete and Continuous Distributions
#' 
#' Provides a simple way to generate plots of pdfs, probability mass functions,
#' cdfs, probability histograms, and normal-quantile plots for distributions
#' known to R.
#'
#' @param dist 
#' 	  A string identifying the distribution.  This should work
#' 	  with any distribution that has associated functions beginning
#' 	  with 'd', 'p', and 'q' (e.g, 
#' 	  [dnorm()],
#' 	  [pnorm()], and 
#' 	  [qnorm()]).  `dist` should match the name of the 
#' 	  distribution with the initial 'd', 'p', or 'q' removed.
#' @param xlim a numeric vector of length 2 or `NULL`, in which case
#'   the central 99.8 of the distribution is used.
#' @param ylim a numeric vector of length 2 or `NULL`, in which case
#'   a heuristic is used to avoid chasing asymptotes in distributions like
#'   the F distributions with 1 numerator degree of freedom.
#' @param add a logical indicating whether the plot should be added to the previous lattice plot.
#'   If missing, it will be set to match `under`.
#' @param under a logical indicating whether adding should be done in a layer under or over the existing 
#'   layers when `add = TRUE`.
#' @param packets,rows,columns specification of which panels will be added to when 
#'   `add` is `TRUE`.  See [latticeExtra::layer()].
#' @param params a list containing parameters for the distribution.  If `NULL` (the default), 
#'   this list is created from elements of `\dots` that are either unnamed or have names among
#'   the formals of the appropriate distribution function.  See the examples.
#' @param kind one of "density", "cdf", "qq", or "histogram" (or prefix 
#' 	 of any of these)
#' @param xlab,ylab as per other lattice functions
#' @param breaks  a vector of break points for bins of histograms,
#'   as in [histogram()] 
#' @param type  passed along to various lattice graphing functions 
#' @param resolution  number of points to sample when generating the plots
#' @param ...  other arguments passed along to lattice graphing routines
#' @seealso [ggformula::gf_dist()]
#' 
#' @details `plotDist()` determines whether the distribution 
#' 	is continuous or discrete by seeing if all the sampled quantiles are 
#' 	unique.  A discrete random variable with many possible values could 
#' 	fool this algorithm and be considered continuous.
#' 
#' The plots are done referencing a data frame with variables
#' `x` and `y` giving points on the graph of the 
#' pdf, pmf, or cdf for the distribution.  This can be useful in conjunction
#' with the `groups` argument.  See the examples.
#' 
#' @examples
#' plotDist('norm')
#' plotDist('norm', type='h')
#' plotDist('norm', kind='cdf')
#' plotDist('exp',  kind='histogram')
#' plotDist('binom', params=list( 25, .25))       # explicit params
#' plotDist('binom', 25, .25)                     # params inferred
#' plotDist('norm', mean=100, sd=10, kind='cdf')  # params inferred
#' plotDist('binom', 25, .25, xlim=c(-1,26) )     # params inferred
#' plotDist('binom', params=list( 25, .25), kind='cdf')
#' plotDist('beta', params=list( 3, 10), kind='density')
#' plotDist('beta', params=list( 3, 10), kind='cdf')
#' plotDist( "binom", params=list(35,.25), 
#'            groups= y < dbinom(qbinom(0.05, 35, .25), 35,.25) )
#' plotDist( "binom", params=list(35,.25), 
#'            groups= y < dbinom(qbinom(0.05, 35, .25), 35,.25), 
#'            kind='hist')
#' plotDist("norm", mean=10, sd=2, col="blue", type="h")
#' plotDist("norm", mean=12, sd=2, col="red", type="h", under=TRUE)
#' plotDist("binom", size=100, prob=.30) +
#'   plotDist("norm", mean=30, sd=sqrt(100 * .3 * .7))
#' plotDist("chisq", df=4, groups = x > 6, type="h")
#' plotDist("f", df1=1, df2 = 99)
#' if (require(mosaicData)) {
#' histogram( ~age|sex, data=HELPrct)
#' m <- mean( ~age|sex, data=HELPrct)
#' s <- sd(~age|sex, data=HELPrct)
#' plotDist( "norm", mean=m[1], sd=s[1], col="red", add=TRUE, packets=1)
#' plotDist( "norm", mean=m[2], sd=s[2], col="blue", under=TRUE, packets=2)
#' }
#' 
#' @keywords graphics 
#' @keywords stats 
#' @export

plotDist <- function( 
  dist, ...,
  xlim = NULL,
  ylim = NULL,
  add,
  under = FALSE,
  packets=NULL,
  rows=NULL,
  columns=NULL,
  kind = c('density','cdf','qq','histogram'), 
  xlab = "", ylab = "", breaks = NULL, type, 
  resolution = 5000L,  params = NULL ) {
  
  kind = match.arg(kind)
  if (missing(add)) add <- under
  ddist = paste('d', dist, sep='')
  qdist = paste('q', dist, sep='')
  pdist = paste('p', dist, sep='')
  
  original_call <- match.call()  
  dots <- original_call
  dots[[1]] <- NULL
  unnamed_dots <- original_call
  named_dots <- original_call
  unnamed_dots[[1]] <- NULL
  named_dots[[1]] <- NULL
  groupless_dots <- original_call
  groupless_dots[[1]] <- NULL
  for (i in length(unnamed_dots):1) {
    if (names(unnamed_dots)[i] != "") {
      unnamed_dots[i] <- NULL
    } else {
      named_dots[i] <- NULL
    }
  }
  if (is.null(params)) {
    params <- original_call
    params[[1]] <- NULL
    for (item in names(formals()) ) {
      if (item %in% names(params)) params[[item]] <- NULL
    }
    dparams <- c(unnamed(params) , named_among( params, names(formals(ddist))) )
    pparams <- c(unnamed(params) , named_among( params, names(formals(pdist))) )
    qparams <- c(unnamed(params) , named_among( params, names(formals(qdist))) )
  } else {
    dparams <- params
    pparams <- params
    qparams <- params
  }
  # attempting to make evaluation of these arguments more intuitive 
  env <- parent.frame()
  dparams <- lapply(dparams, function(x) eval(x, env))
  pparams <- lapply(pparams, function(x) eval(x, env))
  qparams <- lapply(qparams, function(x) eval(x, env))
  
  values = do.call(qdist, c(p=list(ppoints(resolution)), qparams)) 

  fewerValues <- unique(values)
  discrete = length(fewerValues) < length(values) 
  
  if (! discrete) {
    values = seq(
      do.call(qdist, c(p = list(0.001), qparams)),
      do.call(qdist, c(p = list(0.999), qparams)),
      length.out = resolution
    ) 
    fewerValues <- values
  }
  
  if ( is.null(breaks) && discrete ){
    step = min(diff(fewerValues))
    breaks = seq( min(fewerValues) -.5 * step , max(fewerValues) + .5*step, step)
  }
  
  if (kind=='cdf') {
    if (discrete) {
      step = min(diff(fewerValues))
      cdfx <- seq( min(fewerValues) -1.5 * step , max(fewerValues) + 1.5*step, length.out=resolution)
      cdfx <- sort(unique( c(fewerValues, cdfx) ) )
      cdfy <- approxfun( fewerValues, do.call(pdist, c(list(q=fewerValues),pparams)), method='constant', 
                         f=0, yleft=0, yright=1 ) (cdfx)
    } else {
      cdfx <- values
      cdfy <- do.call( pdist, c(list(q=values), pparams) ) 
    }
  }
  if (missing(type)) {
    if (discrete) {
      type = switch(kind,
                    density = c('p','h'),
                    cdf = 'p',
                    histogram = 'density',
                    qq = 'l')  
    } else {
      type = switch(kind,
                    density = 'l',
                    cdf = 'l',
                    histogram = 'density',
                    qq = 'l')
    }
  }
  
  if (add) {
    rlang::check_installed('latticeExtra')
    call_without_add <- original_call
    call_without_add["add"] <- FALSE
    return( 
      trellis.last.object() + 
        latticeExtra::as.layer(
          eval.parent(call_without_add),
          under=under,
          packets=packets,
          rows=rows,
          columns=columns) 
    )
  } else { # not adding
    if (is.null(xlim)) {
      xlim <- do.call(qdist, c(list(p=c(0.001, .999)), qparams))
    }
    ydata <- 
      switch(kind,
             density = do.call(ddist, c(list(x=fewerValues), dparams)),
             cdf = cdfy,
             qq = do.call( ddist, c(list(x=values), dparams)),
             histogram = do.call(ddist, c(list(x=values), dparams))
      )
    if (is.null(ylim)) {
      ymax <- 
        min(
          1.6 * quantile(ydata, 0.90, na.rm=TRUE),
          1.1 * max(ydata, na.rm=TRUE),
          na.rm = TRUE)
      ylim = c(0, ymax)  
    }
    
    switch(kind, 
           density = 
             lattice::xyplot( y ~ x, 
                              data=data.frame(y = ydata, x = fewerValues), 
                              xlim = xlim, ylim = ylim,
                              type=type, xlab=xlab, ylab=ylab, ...),
           cdf = 
             lattice::xyplot( y ~ x, 
                              data=data.frame(y = ydata, x = cdfx), 
                              xlim = xlim, ylim = ylim,
                              type=type, xlab = xlab, ylab = ylab, ...),
           qq = 
             lattice::qqmath( ~ x, 
                              data = data.frame(x = values, y = ydata),
                              xlim = xlim, ylim = ylim,
                              type = type, xlab = xlab, ylab = ylab, ...),
           histogram = 
             histogram( ~ x,
                        data = data.frame(x = values,  y = ydata),
                        xlim = xlim, ylim = ylim,
                        type = type, xlab = xlab, breaks = breaks, ...)
    )
  }
}

# context("plotDist()")
require(mosaicData)
testthat::test_that("plotDist works", {
  wrapped_expect_doppelganger("plotDist1", plotDist('norm'))
  wrapped_expect_doppelganger("plotDist2", plotDist('norm', type='h'))
  wrapped_expect_doppelganger("plotDist3", plotDist('norm', kind='cdf'))
  wrapped_expect_doppelganger("plotDist4", plotDist('exp',  kind='histogram'))
  wrapped_expect_doppelganger("plotDist5", plotDist('binom', params=list( 25, .25)))
  wrapped_expect_doppelganger("plotDist6", plotDist('binom', 25, .25))
  wrapped_expect_doppelganger("plotDist7", plotDist('norm', mean=100, sd=10, kind='cdf'))
  wrapped_expect_doppelganger("plotDist8", plotDist('binom', params=list( 25, .25), kind='cdf'))
  wrapped_expect_doppelganger("plotDist9", plotDist('beta', params=list( 3, 10), kind='density'))
  wrapped_expect_doppelganger("plotDist10", plotDist('beta', params=list( 3, 10), kind='cdf'))
  wrapped_expect_doppelganger("plotDist11", plotDist( "binom", params=list(35,.25),
                                                     groups= y < dbinom(qbinom(0.05, 35, .25), 35,.25) ))
  wrapped_expect_doppelganger("plotDist12", plotDist( "binom", params=list(35,.25),
                                                     groups= y < dbinom(qbinom(0.05, 35, .25), 35,.25),
                                                     kind='hist'))
  wrapped_expect_doppelganger("plotDist13", plotDist("norm", mean=10, sd=2, col="blue", type="h"))
  wrapped_expect_doppelganger("plotDist14", plotDist("norm", mean=12, sd=2, col="red", type="h", under=TRUE))
  wrapped_expect_doppelganger("plotDist15", plotDist("binom", size=100, prob=.30) +
                                plotDist("norm", mean=30, sd=sqrt(100 * .3 * .7)))
  
  
  
  wrapped_expect_doppelganger("plotDist16", plotDist("chisq", df=4, groups = x > 6, type="h"))
  wrapped_expect_doppelganger("plotDist17", plotDist("f", df1=1, df2 = 99))
  
  histogram( ~age|sex, data=HELPrct)
  m <- mean( ~age|sex, data=HELPrct)
  s <- sd(~age|sex, data=HELPrct)
  
  wrapped_expect_doppelganger("plotDist18", plotDist( "norm", mean=m[1], sd=s[1], col="red", add=TRUE, packets=1))
  wrapped_expect_doppelganger("plotDist19", plotDist( "norm", mean=m[2], sd=s[2], col="blue", under=TRUE, packets=2))
})

utils::globalVariables(c('pair','lwr','upr','fitted','.resid',
                         '.std.resid', '.cooksd', '.fitted', 
                         'lower', 'upper',
                         'fcoef', 'density', 'probability',
                         '.hat', 'grid.arrange',  'estimate','se')) 

#' @importFrom ggplot2 fortify
#' @importFrom stats qqnorm
# #' @importFrom broom augment
#' 
NA

#' Generic plotting
#' 
#' Generic function plotting for R objects.  Currently plots exist for 
#' `data.frame`s, `lm`s, (including `glm`s).
#' 
#' @rdname mplot
#' @param object an R object from which a plot will be constructed.
#' @param data_text text representation of the data set.  In typical use cases, the default value should suffice.
#' @export

mplot <- function(object, ...) {
  if (inherits(object, "data.frame")) {
    return(mPlot(object, ..., data_text = rlang::expr_deparse(substitute(object)))) # substitute(object))) 
  }
  
  UseMethod("mplot")
}

#' @rdname mplot
#' @export
mplot.default <- function(object, ...) {
  message("mplot() doesn't know how to handle this kind of input.")
  message('use methods("mplot") to see a list of available methods.')
}

#' @rdname mplot
#' @param data a data frame containing the variables that might be used in the plot.
# Note that for maps, the data frame must contain coordinates of the polygons 
# comprising the map and a variable for determining which coordinates are part
# of the same region.  See \code{\link{sp2df}} for one way to create such
# a data frame.  Typically \code{\link{merge}} will be used to combine the map
# data with some auxiliary data to be displayed as fill color on the map, although
# this is not necessary if all one wants is a map.
#' @param format,default default type of plot to create; one of 
#' `"scatter"`,
#' `"jitter"`,
#' `"boxplot"`,
#' `"violin"`,
#' `"histogram"`,
#' `"density"`,
#' `"frequency polygon"`,
#' or
# \code{"xyplot"}. 
# or
#' `"map"`.  
#' Unique prefixes suffice.
#' @param system which graphics system to use (initially) for plotting (\pkg{ggplot2} 
#'   or \pkg{lattice}).  A check box will allow on the fly change of plotting system.
#' @param show a logical, if `TRUE`, the code will be displayed each time the plot is 
#'   changed.
#' @return Nothing.  Just for side effects. 
#' @param which a numeric vector used to select from 7 potential plots
#' @param ask if TRUE, each plot will be displayed separately after the user 
#' responds to a prompt.
#' @param multiplot if TRUE and `ask == FALSE`, all plots will be 
#' displayed together.
#' @param title title for plot
#' @param ... additional arguments.  If `object` is an `lm`, subsets
#' of these arguments are passed to `gridExtra::grid.arrange` and to the 
#' \pkg{lattice} plotting routines; in particular,
#' `nrow` and `ncol` can be used to control the number of rows
#' and columns used.
#' @param id.nudge a numeric used to increase (>1) or decrease (<1) the amount that observation labels are 
#' nudged.  Use a negative value to nudge down instead of up.
#' @param id.n Number of id labels to display.
#' @param id.size Size of id labels.
#' @param id.color Color of id labels.
#' @param add.smooth A logicial indicating whether a LOESS smooth should be added 
#'   (where this makes sense to do).
#'   Currently ignored for lattice plots.
#' @param span A positive number indicating the amount of smoothing. 
#'   A larger number indicates more smoothing. See [`stats::loess()`] for details.
#'   Currently ignored for lattice plots.
#' @param smooth.color,smooth.size,smooth.alpha Color, size, and alpha used for 
#'   LOESS curve.  Currently ignored for lattice plots.
#' @details
#' The method for models (lm and glm) is still a work in progress, but should be usable for 
#' relatively simple models.  When the results for a logistic regression model created with
#' [glm()] are satisfactory will depend on the format and structure of the data
#' used to fit the model.
#' 
#' Due to a bug in RStudio 1.3, the method for data frames may not display the controls
#' consistently.  We have found that executing this code usually fixes the problem:
#' 
#' ```
#' library(manipulate) 
#' manipulate(plot(A), A = slider(1, 10))
#' ```
#' 
#' 
#' @examples
#' lm( width ~ length * sex, data = KidsFeet) |>
#'   mplot(which = 1:3, id.n = 5)
#' lm( width ~ length * sex, data = KidsFeet) |>
#'   mplot(smooth.color = "blue", smooth.size = 1.2, smooth.alpha = 0.3, id.size = 3)
#' lm(width ~ length * sex, data = KidsFeet) |>
#'   mplot(rows = 2:3, which = 7)
# #' @importFrom ggrepel geom_text_repel
#' @export

mplot.lm <- 
  function(
    object, 
    which = c(1:3, 7), 
    system = c("ggplot2", "lattice", "base"),
    ask = FALSE, 
    multiplot = "package:gridExtra" %in% search(),
    par.settings = theme.mosaic(),
    level = .95,
    title = paste("model: ", deparse(object$call), "\n"),
    rows = TRUE,
    id.n = 3L,
    id.size = 5,
    id.color = "red",
    id.nudge = 1,
    add.smooth = TRUE, 
    smooth.color = "red",
    smooth.alpha = 0.6,
    smooth.size = 0.7,
    span = 3/4, 
    ...) {
  
  system <- match.arg(system)
  check_installed('ggrepel')
  
  geom_smooth_or_not <-  
    if (add.smooth)
      geom_line(stat = "smooth", method = "loess", span = span, 
                alpha = smooth.alpha, color = smooth.color, linewidth = smooth.size) 
    else
      geom_blank() 
  
  dots <- list(...)
  if ("col" %in% names(dots)) {
    dots$col <- dots$col[1]
  }
  
  if (multiplot && ! "package:gridExtra" %in% search()) {
    message("multiplot = TRUE only works when 'gridExtra' is loaded.")
    message("    I'm setting multiplot = FALSE and continuing.")
    multiplot <- FALSE
  }
  
  if (system == "base") {
    return(plot( object, which = intersect(which, 1:6)))
  }
 
  rlang::check_installed('broom') 
  fdata <- broom::augment(object) 
  fdata <- 
    fdata |> 
    mutate(
      .row = 1L:nrow(fdata)
    )
 
  # broom::augment() does always supply .resid :-/
  
  if (is.null(fdata[[".resid"]])) {
    fdata <- fdata |> mutate(.resid = resid(object))
  }
  
  fdata_clean <- fdata |> filter(!is.na(.std.resid))
  
  removed_idx <- which(fdata$.hat >= 1)
  if (any(c(2, 3, 5, 6) %in% which) && length(removed_idx)) {
    warning("Observations with leverage 1 not plotted: ", 
            paste(removed_idx, collapse = ", "),
            call. = FALSE)
  }
    
#  fdata <- cbind(fdata, row = 1:nrow(fdata))
  
  if (!inherits(object, "lm")) 
    stop("use only with \"lm\" objects")
  if (!is.numeric(which) || any(which < 1) || any(which > 7)) 
    stop("'which' must be in 1:7")
  isGlm <- inherits(object, "glm")
  show <- rep(FALSE, 7)
  show[which] <- TRUE
  
  ylab23 <- if (isGlm) 
    "Std. deviance resid."
  else "Standardized residuals"
  
  
  # residuals vs fitted
  g1 <- ggplot(fdata, aes(.fitted, .resid)) +
    geom_point()  +
    geom_smooth_or_not +
    geom_hline(linetype = 2, linewidth = .2, yintercept = 0) +
    ggrepel::geom_text_repel(
      data = fdata |> arrange(-abs(.std.resid)) |> head(id.n),
      aes(label = .row), 
      color = id.color,
      segment.color = id.color,
      size = id.size) + 
    scale_x_continuous("Fitted Value") +
    scale_y_continuous("Residual") +
    labs(title = "Residuals vs Fitted")
  
  l1 <- do.call(xyplot, 
                c(list( .std.resid ~ .fitted, data = fdata,
                        type = c("p","smooth"),
                        panel = function(x,y,...) {
                          panel.abline(h = 0, linetype = 2, lwd = .5) 
                          panel.xyplot(x,y,...)
                        },
                        main = "Residuals vs Fitted",
                        xlab = "Fitted Value",
                        ylab = "Residual",
                        par.settings = par.settings),
                  dots)
  )
  
  # normal qq
  # remove NAs and NaNs before computing quantiles
  
  a <- quantile(fdata$.std.resid, c(0.25, 0.75), na.rm = TRUE)
  b <- qnorm(c(0.25, 0.75))
  slope <- diff(a)/diff(b)
  int <- a[1] - slope * b[1]
  QN <- 
    as.data.frame(qqnorm(fdata$.std.resid, plot.it = FALSE)) |> 
    mutate(.row = 1:nrow(fdata))
  g2 <- ggplot(fdata_clean, aes(sample = .std.resid)) +
    stat_qq() +
    geom_abline(slope = slope, intercept = int, linetype = "dashed") +
    ggrepel::geom_text_repel(
      inherit.aes = FALSE,
      data = QN |> arrange(-abs(y)) |> head(id.n),
      aes(y = y,  x = x, label = .row),
      color = id.color,
      segment.color = id.color,
      size = id.size) + 
    scale_x_continuous("Theoretical Quantiles") +
    scale_y_continuous("Standardized Residuals") +
    labs(title = "Normal Q-Q")
  
  l2 <- do.call(qqmath, 
                c(list( ~ .std.resid, data = fdata_clean, 
                        panel = function(x,...) {
                          panel.abline(a = int, b = slope)
                          panel.qqmath(x,...)
                        },
                        main = "Normal Q-Q",
                        xlab = "Theoretical Quantiles",
                        ylab = ylab23,
                        par.settings = par.settings),
                  dots)
  )
  
  # scale-location
  
  g3 <- ggplot(fdata_clean, aes(.fitted, sqrt(abs(.std.resid)))) +
    geom_point() +
    geom_smooth_or_not +
    ggrepel::geom_text_repel(
      data = fdata_clean |> arrange(-abs(.std.resid)) |> head(id.n),
      aes(label = .row), 
      color = id.color,
      segment.color = id.color,
      size = id.size) + 
    scale_x_continuous("Fitted Values") +
    scale_y_continuous(as.expression(
      substitute(sqrt(abs(YL)), list(YL = as.name(ylab23))) )) +
    labs(title = "Scale-Location")
  
  l3 <- do.call(xyplot, 
                c(list( sqrt(abs(.std.resid)) ~ .fitted, data = fdata_clean,
                        type = c("p","smooth"),
                        main = "Scale-Location",
                        xlab = "Fitted Value",
                        ylab = as.expression(
                          substitute(sqrt(abs(YL)), list(YL = as.name(ylab23)))
                        ),
                        par.settings = par.settings),
                  dots)  
  )
  
  # cook's distance
  g4 <-  
    ggplot(data = fdata, aes(.row, .cooksd, ymin = 0, ymax = .cooksd)) +
    geom_point() + 
    geom_linerange() +
    scale_x_continuous("Observation Number", limits = c(0, NA)) +
    scale_y_continuous("Cook's distance") +
    labs(title = "Cook's Distance") 
  if (id.n > 0L) {
    g4 <- g4 + 
      ggrepel::geom_text_repel(
        data = fdata |> arrange(-abs(.cooksd)) |> head(id.n),
        aes(x = .row, y = .cooksd, label = .row),
        color = id.color,
        segment.color = id.color,
        size = id.size) 
  }
  
  l4 <- do.call( xyplot,
                 c(list( .cooksd ~ .row, data = fdata, 
                         type = c("p","h"),
                         main = "Cook's Distance",
                         xlab = "Observation number",
                         ylab = "Cook's distance",
                         par.settings = par.settings),
                   dots)  
  )
  
  # residuals vs leverage
  g5 <- 
    ggplot(fdata_clean, aes(x = .hat, y = .std.resid)) +
    geom_point() +
    geom_smooth_or_not +
    ggrepel::geom_text_repel(
      data = fdata_clean |> arrange(-abs(.std.resid)) |> head(id.n),
      aes(label = .row), 
      color = id.color,
      segment.color = id.color,
      size = id.size) + 
    geom_hline(linetype = 2, linewidth = .2, yintercept = 0) +
    labs(title = "Residuals vs Leverage",
         x     = "Leverage",
         y     = "Standardized Residual") +
    lims(x = c(0, NA))
  
  l5 <- do.call( xyplot, 
                 c(list( .std.resid ~ .hat, data = fdata_clean,
                         type = c('p','smooth'),
                         panel = function(x,y,...) {
                           panel.abline( h = 0, lty = 2, lwd = .5)
                           panel.xyplot( x, y, ...)
                         },
                         main = "Residuals vs Leverage",
                         xlab = "Leverage",
                         ylab = "Standardized Residuals",
                         par.settings = par.settings),
                   dots)
  )
  
  # cooksd vs leverage
  g6 <- ggplot(fdata_clean, aes(.hat, .cooksd)) +
    geom_point() +
    geom_smooth_or_not +
    ggrepel::geom_text_repel(
      data = fdata_clean |> arrange(-abs(.std.resid)) |> head(id.n),
      aes(label = .row), 
      color = id.color,
      segment.color = id.color,
      size = id.size) + 
    scale_x_continuous("Leverage") +
    scale_y_continuous("Cook's distance") +
    labs(title = "Cook's dist vs Leverage")
  
  l6 <- do.call(xyplot,
                c(list( .cooksd ~ .hat, data = fdata_clean,
                        type = c("p", "smooth"),
                        main = "Cook's dist vs Leverage",
                        xlab = "Leverage",
                        ylab = "Cook's distance",
                        par.settings = par.settings),
                  dots)
  )

  g7 <- mplot(summary(object), level = level, rows = rows, ..., system = "ggplot2")
  
  l7 <- mplot(summary(object), level = level, rows = rows, ..., system = "lattice")
  
  plots <- if (system == "ggplot2") {
    list(g1, g2, g3, g4, g5, g6, g7)
  } else {
    lapply( list(l1, l2, l3, l4, l5, l6, l7), 
            function(x) update(x, par.settings = par.settings))
  }
  
  plots <- plots[which] 
  
  if (ask) {
    for (p in plots) {
      readline("Hit <RETURN> for next plot")
      print(p)
    }
  } 
  if (multiplot) {
    rlang::check_installed('gridExtra')
    dots <- list(...)
    nn <- intersect( 
      union(names(formals(gridExtra::arrangeGrob)), names(formals(grid.layout))),
      names(dots) 
    )
    dots <- dots[ nn ]
    return(do.call(gridExtra::grid.arrange, c(plots, dots)))
    result <-  do.call(
      gridExtra::arrangeGrob, 
      c(plots, dots) # , c(list(main = title), dots))
    )
    plot(result)
    return(result)
  }

# Question: should a single plot be returned as is or in a list of length 1?
if (length(plots) == 1) {
  return(plots[[1]])
}
  
  return(plots)
}

#' @rdname mplot
#' @examples
#' \dontrun{
#' mplot( HELPrct )
#' mplot( HELPrct, "histogram" )
#' }
#' @export

mplot.data.frame <- 
  function(
    object, format, default = format, 
    system = c("ggformula", "ggplot2", "lattice"),  
    show = FALSE, 
    data_text = rlang::expr_deparse(substitute(object)),
    # data_text = substitute(object),
    title = "", ...
  ) {
  print(data_text)
  return(
    mPlot(object, format = format, default = default, system = system, 
        show = show, title = title, data_text = data_text, ...)
  )
}  
#   plotTypes <- c('scatter', 'jitter', 'boxplot', 'violin', 'histogram', 
#                  'density', 'frequency polygon', 'xyplot')
#   if (missing(default) & missing(format)) {
#     choice <- 
#       menu(title = "Choose a plot type.",
#            choices = c(
#              "1-variable (histogram, density plot, etc.)",
#              "2-variable (scatter, boxplot, etc.)" 
#            )
#       )
#     default <- c("histogram", "scatter") [choice]
#   }
#   default <- match.arg(default, plotTypes)
#   system <- match.arg(system)
# 
#   dataName <- substitute(object)
#   if (default == "xyplot") 
#     default <- "scatter"
#   if (default %in% c("scatter", "jitter", "boxplot", "violin")) {
#     return(
#       mScatter(lazy_data, default = default, system = system, show = show, title = title)
#     )
#   }
# #   if (default == "map") {
# #     return(eval(parse(
# #       text = paste("mMap(", dataName, 
# #                    ", default = default, system = system, show = show, title = title)"))
# #     ))
# #   }
#   return(eval(parse(
#     text = paste("mUniplot(", dataName, 
#                  ", default = default, system = system, show = show, title = title)"))
#   ))
# }


#' Extract data from R objects
#' 
#' @rdname fortify
#' @param level confidence level
#' @param ... additional arguments
#' @export

fortify.summary.lm <- function(model, data = NULL, level = 0.95, ...) {
  E <- as.data.frame(coef(model, level = level))
  # grab only part of the third name that comes before space
  statName <- strsplit(names(E)[3], split = " ")[[1]][1]
  names(E) <- c("estimate", "se", "stat", "pval")
  # add coefficient names to data frame
  E$coef <- row.names(E)
  E$statName <- statName
  E$lower <- confint(model, level = level, ...)[,1]
  E$upper <- confint(model, level = level, ...)[,2]
  E$level <- level
  return(E)
}

#' @rdname fortify
#' @export

fortify.summary.glm <- function(model, data = NULL, level = 0.95, ...) {
  E <- as.data.frame(coef(model, level = level))
  # grab only part of the third name that comes before space
  statName <- strsplit(names(E)[3], split = " ")[[1]][1]
  names(E) <- c("estimate", "se", "stat", "pval")
  # add coefficient names to data frame
  E$coef <- row.names(E)
  E$statName <- statName
  E <- mutate(E, 
              lower = estimate + qnorm((1-level)/2) * se,
              upper = estimate + qnorm(1-(1-level)/2) * se,
              level = level)
  return(E)
}

#' @rdname confint
#' @param object and R object
#' @param parm a vector of parameters
#' @param level a confidence level
#' @examples
#' lm(width ~ length * sex, data = KidsFeet) |>
#'   summary() |>
#'   confint()
#' @export

confint.summary.lm <- function (object, parm, level = 0.95, ...)  {
  cf <- coef(object)[, 1]
  pnames <- names(cf)
  if (missing(parm)) 
    parm <- pnames
  else if (is.numeric(parm)) 
    parm <- pnames[parm]
  a <- (1 - level)/2
  a <- c(a, 1 - a)
  fac <- qt(a, object$df[2])
  pct <- paste( format(100*a, digits = 3, trim = TRUE, scientific = FALSE), "%" )
  ci <- array(NA, dim = c(length(parm), 2L), dimnames = list(parm, 
                                                             pct))
  ses <- sqrt(diag(vcov(object)))[parm]
  ci[] <- cf[parm] + ses %o% fac
  ci
}

#' @rdname mplot
#' @param level a confidence level
#' @param par.settings \pkg{lattice} theme settings 
#' @param rows rows to show.  This may be a numeric vector, 
#' `TRUE` (for all rows), or a character vector of row names.
#' @examples
#' lm(width ~ length * sex, data = KidsFeet) |>
#'   summary() |>
#'   mplot()
#'   
#' lm(width ~ length * sex, data = KidsFeet) |>
#'   summary() |>
#'   mplot(rows = c("sex", "length"))
#'   
#' lm(width ~ length * sex, data = KidsFeet) |>
#'   summary() |>
#'   mplot(rows = TRUE)
#' @export
 
mplot.summary.lm <- function(object, 
                             system = c("ggplot2", "lattice"),
                             level = 0.95,
                             par.settings = trellis.par.get(),
                             rows = TRUE,
                             ...){
  system <- match.arg(system)
  fdata <- fortify(object, level = level) |> 
    mutate(signif = pval < (1-level),
           fcoef = factor(coef, levels = coef)
           )
  row.names(fdata) <- fdata$coef
  fdata <- fdata[rows, ]
  fdata <- fdata[nrow(fdata):1, ]
  
  g <- ggplot(data = fdata,
              aes(x = fcoef, y = estimate, 
                  ymin = lower, ymax = upper, 
                  color = signif)) + # (pval < (1-level)/2))) + 
    geom_pointrange(size = 1.2) + 
    geom_hline(yintercept = 0, color = "red", alpha = .5, linetype = 2) + 
    labs(x = "coefficient", title = paste0(format(100*level), "% confidence intervals") ) +
    theme(legend.position = "none") +
    coord_flip()
  
  cols <- rep( par.settings$superpose.line$col, length.out = 2)
  cols <- cols[2 - fdata$signif]
  
  l <- xyplot( fcoef ~ estimate + lower + upper,
               data = fdata,
               fdata = fdata,
               xlab = "estimate",
               ylab = "coefficient",
               main = paste0(format(100 * level), "% confidence intervals"),
               ...,
               panel = function(x, y, fdata, ...) {
                 dots <- list(...)
                 if ("col" %in% names(dots)) {
                   dots$col <- rep(dots$col, length.out = 2) [2 - fdata$signif]
                 } 
                 dots <- .updateList(
                   list(lwd = 2, alpha = 0.6, cex = 1.4, col = cols),
                   dots
                 )
                 dots[["type"]] <- NULL
                                     
                 panel.abline(v = 0, col = "red", alpha = .5, lty = 2) 
                 do.call( panel.points,
                          c( list (x = fdata$estimate, y = y), 
                             dots )
                 )
                 do.call( panel.segments, 
                          c( list(y0 = y, y1 = y, x0 = fdata$lower, 
                                  x1 = fdata$upper),
                             dots )
                 )
               }
  )
  
  if (system == "ggplot2") {
    return(g)
  } else { 
    return(l)
  }
}


#' @export
 
mplot.summary.glm <- mplot.summary.lm

#' @rdname fortify
#' @param model an R object
#' @param data original data set, if needed
#' @param order one of `"pval"`, `"diff"`, or `"asis"` determining the 
#'   order of the `pair` factor, which determines the order in which the differences
#'   are displayed on the plot.
#' @export
 
# fortify.TukeyHSD <- function(model, data, ...) {
#   nms <- names(model)
#   l <- length(model)
#   plotData <- do.call( 
#     rbind, 
#     lapply(seq_len(l), function(i) {
#       res <- transform(as.data.frame(model[[i]]), 
#                        var = nms[[i]], 
#                        pair = row.names(model[[i]]) ) 
#     } ) 
#   )
#   names(plotData) <- c("diff", "lwr", "upr", "pval", "var", "pair")
#   return(plotData)
# }  

fortify.TukeyHSD <- function(model, data, order = c("asis", "pval", "difference"), ...) {
  order <- match.arg(order)
  nms <- names(model)
  l <- length(model)
  plotData <- do.call( 
    rbind, 
    lapply(seq_len(l), function(i) {
      res <- transform(as.data.frame(model[[i]]), 
                       var = nms[[i]], 
                       pair = row.names(model[[i]]) ) 
    } ) 
  )
  names(plotData) <- c("diff", "lwr", "upr", "pval", "var", "pair")
  plotData <-
    plotData |>
    mutate(pair = 
             switch(order,
                    "asis" = reorder(pair, 1:nrow(plotData)),
                    "pval" = reorder(pair, pval),
                    "difference" = reorder(pair, diff),
                    )
    )
  
  return(plotData)
}  

#' @rdname mplot
#' @param xlab label for x-axis
#' @param ylab label for y-axis
#' @param order one of `"pval"`, `"diff"`, or `"asis"` determining the 
#'   order of the `pair` factor, which determines the order in which the differences
#'   are displayed on the plot.
#' @examples
#' lm(age ~ substance, data = HELPrct) |>
#'   TukeyHSD() |>
#'   mplot()
#' lm(age ~ substance, data = HELPrct) |>
#'   TukeyHSD() |>
#'   mplot(system = "lattice")
#' @export

mplot.TukeyHSD <- function(object, system = c("ggplot2", "lattice"), 
                           ylab = "", xlab = "difference in means", 
                           title = paste0(attr(object, "conf.level") * 100, "% family-wise confidence level"),
                           par.settings = trellis.par.get(),
                           order = c("asis", "pval", "difference"),
                           # which = 1L:2L,
                           ...) {
  system <- match.arg(system)
  order <- match.arg(order) 
  fdata <- fortify(object, order = order)
 
  # res <- list()
  
  if (system == "ggplot2") {
    # if (1 %in% which) {
      p1 <-
        ggplot( data = fdata,
                aes(x = diff, color = log10(pval), y = factor(pair, levels = rev(levels(pair)))) ) +
        geom_point(size = 2) +
        geom_segment(aes(x = lwr, xend = upr, y = pair, yend = pair) ) +
        geom_vline( xintercept = 0, color = "red", linetype = 2, alpha = .5 ) + 
        facet_grid( var ~ ., scales = "free_y") +
        labs(x = xlab, y = ylab, title = title) 
    #   res <- c(res, list(p1))
    # }
    # if (2 %in% which) {
    #   p2 <- 
    #     ggplot( data = fdata,
    #             aes(x = diff, color = log10(pval), y = factor(pair, levels = rev(levels(pair)))) ) +
    #     geom_point(size = 2) +
    #     geom_segment(aes(x = lwr, xend = upr, y = pair, yend = pair) ) +
    #     geom_vline( xintercept = 0, color = "red", linetype = 2, alpha = .5 ) + 
    #     facet_grid( var ~ ., scales = "free_y") +
    #     labs(x = xlab, y = ylab, title = title) 
    #   res <- c(res, list(p2))
    # }
  return(p1)
  }

  cols <- par.settings$superpose.line$col[1 + 
            as.numeric( sign(fdata$lwr) * sign(fdata$upr) < 0)]
  
  xyplot( factor(pair, levels = rev(levels(pair))) ~ diff + lwr + upr | var, data = fdata, 
          panel = function(x,y,subscripts,...) {
            n <- length(x)
            m <- round(n/3)
            panel.abline(v = 0, col = "red", lty = 2, alpha = .5)
            panel.segments(x0 = x[(m+1):(2*m)], x1 = x[(2*m+1):(3*m)], y0 = y, y1 = y, col = cols[subscripts])
            panel.xyplot(x[1:m], y, cex = 1.4, pch = 16, col = cols[subscripts])
          },
          scales = list( y = list(relation = "free", rot = 30) ),
          xlab = xlab,
          ylab = ylab,
          main = title,
          ...
  )
}

# context("clustering")
testthat::test_that("Clustering works", {
  require(mosaicData)
  KidsFeet2 <- KidsFeet |> select(-name, -birthmonth) |> rescale() 
  M <- dist(KidsFeet2)
  Cl <- hclust(M)
  
  
  testcase1 <- structure(list(x = c(17.642578125, 6.03125, 17.642578125), 
                              y = c(2.15388023295916, 2.15388023295916, 2.15388023295916), 
                              xend = c(6.03125, 6.03125, 29.253906), 
                              yend = c(2.15388023295916, 1.83303857851649, 2.15388023295916), 
                              order = c(34L, 34L, 8L), 
                              group = c(4L, 4L, 1L)), 
                         row.names = c(NA,3L), 
                         class = "data.frame")
  
  testcase2 <- structure(list(idx = 1:3, 
                              position = c(15L, 37L, 35L), 
                              variable = c("birthyear","birthyear", "birthyear"), 
                              value = c(1, 0, 0), 
                              variable_num = c(1L, 1L, 1L)), 
                         row.names = c(NA, 3L), 
                         class = "data.frame")
  
  testcase3 <- structure(list(birthyear = c(1, 0, 0), 
                              length = c(0.47457627118644,0.644067796610169, 0.491525423728813), 
                              width = c(0.263157894736842,0.473684210526316, 0.947368421052631), 
                              sex = c(0, 0, 0), 
                              biggerfoot = c(0, 0, 1), 
                              domhand = c(1, 0, 1), 
                              idx = 1:3, 
                              position = c(15L, 37L, 35L)), 
                         row.names = c(NA, 3L), 
                         class = "data.frame")
  
  testcase4 <- structure(list(x = c(1, 2, 3), 
                              y = c(0, 0, 0), 
                              label = structure(1:3, 
                                                .Label = c("6","21", "35", "30", "10", "34", "17", 
                                                           "23", "20", "9", "18", "32","16", "38", 
                                                           "1", "22", "29", "5", "12", "4", "26", 
                                                           "7", "14", "19", "31", "27", "37", "25", 
                                                           "8", "36", "33", "24", "39", "11", "3", 
                                                           "28", "2", "13", "15"), class = "factor"), 
                              order = c(6L,21L, 35L)), 
                         row.names = c(NA, 3L), 
                         class = "data.frame")
  
  expect_equal(testcase1, fortify(Cl, k=5) |> head(3))
  expect_equal(testcase2, fortify(Cl, which="heatmap", data=KidsFeet2) |> head(3))
  expect_equal(testcase3, fortify(Cl, which="data", data=KidsFeet2) |> head(3))
  # seems like ggdendro migth be caught by the stringsAsFactors change, so this 
  # didn't succeed on debian when CRAN checked it.  Removing the check for now  3/5/2020
  # expect_equal(testcase4, fortify(Cl, which="labels") |> head(3))
  
  
  wrapped_expect_doppelganger("clustering1", mplot(Cl, data=KidsFeet2, k=4, heatmap=2))
  wrapped_expect_doppelganger("clustering2", mplot(Cl, data=KidsFeet2, k=4, heatmap=0.5, enumerate="transparent"))
  wrapped_expect_doppelganger("clustering3", mplot(Cl, data=KidsFeet2, k=4, heatmap=2, type="triangle"))
  wrapped_expect_doppelganger("clustering4", mplot(Cl, data=KidsFeet2, k=4, heatmap=0, type="triangle"))
})

# context("mplot")

testthat::test_that("lm mplot works", {
  
  
  testcases1 <- lm( width ~ length * sex, data = KidsFeet) |>
    mplot(multiplot=FALSE, which = 1:3, id.n = 5)
  
  testcases2 <- lm( width ~ length * sex, data = KidsFeet) |>
    mplot(smooth.color = "blue", smooth.size = 1.2, smooth.alpha = 0.3, id.size = 3)
  
  singletest <- lm(width ~ length * sex, data = KidsFeet) |>
    mplot(rows = 2:3, which = 7)
  
  
  num <- 1
  
  for(case in testcases1) {
    wrapped_expect_doppelganger(paste("mplot", as.character(num), sep = ""), case)
    num <- num + 1
  }
  
  for(case in testcases2) {
    wrapped_expect_doppelganger(paste("mplot", as.character(num), sep = ""), case)
    num <- num + 1
  }
  
  wrapped_expect_doppelganger(paste("mplot", as.character(num), sep = ""), singletest)
  num <- num + 1
  
  testthat::test_that("getVarFormula Works", {
    testcase <- structure(c(2.62, 2.875, 2.32, 3.215, 3.44, 3.46, 3.57, 3.19, 
                            3.15, 3.44, 3.44, 4.07, 3.73, 3.78, 5.25, 5.424, 5.345, 2.2, 
                            1.615, 1.835, 2.465, 3.52, 3.435, 3.84, 3.845, 1.935, 2.14, 1.513, 
                            3.17, 2.77, 3.57, 2.78, 21, 21, 22.8, 21.4, 18.7, 18.1, 14.3, 
                            24.4, 22.8, 19.2, 17.8, 16.4, 17.3, 15.2, 10.4, 10.4, 14.7, 32.4, 
                            30.4, 33.9, 21.5, 15.5, 15.2, 13.3, 19.2, 27.3, 26, 30.4, 15.8, 
                            19.7, 15, 21.4), .Dim = c(32L, 2L), 
                          .Dimnames = list(c("Mazda RX4", "Mazda RX4 Wag", "Datsun 710", 
                                             "Hornet 4 Drive", "Hornet Sportabout",  
                                             "Valiant", "Duster 360", "Merc 240D", 
                                             "Merc 230", "Merc 280", "Merc 280C", "Merc 450SE", 
                                             "Merc 450SL", "Merc 450SLC", "Cadillac Fleetwood",
                                             "Lincoln Continental", "Chrysler Imperial", 
                                             "Fiat 128", "Honda Civic", "Toyota Corolla", 
                                             "Toyota Corona", "Dodge Challenger", "AMC Javelin", 
                                             "Camaro Z28", "Pontiac Firebird", "Fiat X1-9", 
                                             "Porsche 914-2", "Lotus Europa", "Ford Pantera L", 
                                             "Ferrari Dino", "Maserati Bora", "Volvo 142E"), 
                                           c("wt", "mpg")))
    
    expect_equal(ignore_attr = TRUE, testcase, getVarFormula( ~ wt + mpg, data = mtcars))
  })
})

#'Expand the left-hand side of a formula
#'
#'Expands the contents of functions used in a formula.
#'
#'@rdname expandFun
#'@name expandFun
#'@aliases expandFun
#'
#'@param formula A mathematical expression (see examples and [plotFun()])
#'@param \ldots additional parameters
#'
#'@return A list with the new expanded formula and the combined formals
#'
#'
#'@examples
#'f=makeFun(x^2~x)
#'expandFun(f(z)~z) #Returns z^2~z
#' @export

expandFun <- function(formula,...){
  exp = lhs(formula)
  #See if first operator is a primitive
  #Check if first operation is an arithmetic operator
  if (is.primitive(eval(exp[[1]]))&&getGroup(toString(exp[[1]]))[[1]]=="Arith"){
    ## Recursively replace variables
    if (inherits(exp[[2]], 'call')) {
      form2 = formula
      form2[[2]] = exp[[2]]
      lside = Recall(form2)
      exp[[2]] = lside$formula[[2]]
    }
    if (inherits(exp[[3]] , "call")) {
      form3 = formula
      form3[[2]] = exp[[3]]
      rside = Recall(form3)
      exp[[3]] = rside$formula[[2]]
    }
    newcall = paste("(",deparse(exp[[2]]),")",deparse(exp[[1]]),"(",deparse(exp[[3]]),")")
    newcall = parse(text = newcall)[[1]]
    formula[[2]]=newcall
    return(list(formula=formula, formals=
      as.pairlist(c(lside$formals, rside$formals))))
  }
  ##Save function to somehwere
  func =eval(exp[[1]])  # get the function itself
  body=body(func)
  formals=formals(func)
  vars = all.vars(body)
  
  ##Replace body with match.call and evaluate to find out which params map to what
  fnew = func
  body(fnew)=parse(text="as.list(match.call())")
  exp[[1]] <- as.name("fnew")
  argmap = eval(exp)
  #Substitute new variables into the body.
  if (inherits(body, "{")){#If there is more than one call in the function, takes the last call.
    body=body[[length(body)]]
  }
  sformals = deparse(formals)
  sbody = deparse(body)
  for (pattern in vars) { # loop over the names of the formals
    if (!is.null(argmap[[pattern]])){
      replacement = deparse(argmap[[pattern]])
      sformals = gsub(pattern, replacement, sformals)
      replacement = paste("(",replacement,")") #To preserve order of operations
      newbody = gsub(pattern,replacement, sbody)
      sbody = newbody
    }
  }
  #Substitute lhs of formula for new expression
  
  # old version -- conflicts with changes to formals()
  # newbody=parse(text=newbody)
  # formula[[2]] = newbody[[1]]
  # #update formals
  # sformals = strsplit(sformals, "[[\\(\\)]]*")[[1]][2]
  # sformals = paste("as.pairlist(alist(", sformals, "))", sep="")
  # formals(func) = eval(parse(text=sformals))
  # formals=formals(func)
 
  # replacement code  
  newbody <- parse(text = newbody)
  formula[[2]] <- newbody[[1]]
  #update formals
  sformals <- gsub(pattern = "^as.pairlist\\(alist", replacement = "pairlist", sformals)
  sformals <- strsplit(sformals, "[[\\(\\)]]*")[[1]][2]
  sformals <- paste("as.pairlist(alist(", sformals, "))", sep="")
  formals(func) <- eval(parse(text = sformals))
  
  return(list(formula = formula, formals = formals))
}

# context("expandFun()")

testthat::test_that("expandFun works", {
  
  testcase <- list(formula = (z)^2 ~ z, formals = as.pairlist(alist(x = )))
  
  expect_equal(ignore_attr = TRUE, testcase, expandFun(makeFun(x^2~x)(z)~z))
})

#' Random Regressors
#' 
#' A utility function for producing random regressors with a specified
#' number of degrees of freedom.
#' @param df degrees of freedom, i.e., number of random regressors
#' @param rdist random distribution function for sampling
#' @param args arguments for `rdist` 
#' @param nrow number of rows in resulting matrix.  This can often be omitted in
#'   the context of functions like `lm` where it is inferred from the data frame, 
#'   if one is provided.
#' @param seed seed for random number generation 
#' 
#' 
#' @return A matrix of random variates with `df` columns.  
#' In its intended use, the number of rows will be selected to match the 
#' size of the data frame supplied to `lm`
#' 
#' @examples
#' rand(2,nrow=4)
#' rand(2,rdist=rpois, args=list(lambda=3), nrow=4)
#' summary(lm( waiting ~ eruptions + rand(1), faithful))
#' @keywords distribution 
#' @keywords regression 
#' @export

rand = function(df=1, rdist=rnorm, args=list(), nrow, seed=NULL ){
	if(missing(nrow)) {
		nrow <- length(get( ls( envir=parent.frame())[1], envir=parent.frame()))
	}
	if (!is.null(seed)){
		set.seed(seed)
	}

	result <-  matrix( do.call( rdist, args=c(list(n=df*nrow), args) ), nrow=nrow ) 
#	colnames(result) <- paste('rand',1:df,sep="")
	return(result)
}

#' Student's t-Test
#' 
#' Performs one and two sample t-tests.  
#' The mosaic `t.test` provides wrapper functions around the function 
#' of the same name in \pkg{stats}.
#' These wrappers provide an extended interface that allows for a more systematic
#' use of the formula interface.
#' 
#' @rdname ttest
#' 
#' @inheritParams stats::t.test
#' @param groups 
#'   When `x` is a formula, `groups` can be used to compare groups:  
#'   `x = ~ var, groups = g` is equivalent to ` x = var ~ g `.
#'   See the examples. 

#' 
#' @return an object of class `htest`
#' 
#' @details
#' This is a wrapper around [stats::t.test()] from the \pkg{stats} package
#' to extend the functionality of the formula interface.  In particular, one can 
#' now use the formula interface for a 1-sample t-test.  Before, the formula interface
#' was only permitted for a 2-sample test.  The type of formula that can be used
#' for the 2-sample test has also be broadened.  See the examples.
#'
#' @seealso [mosaic::prop.test()], [mosaic::binom.test()], 
#'   [stats::t.test()]
#' 
#' @examples
#'   t.test(HELPrct$age)
#'   # We can now do this with a formula
#'   t.test(~ age, data = HELPrct)
#'   # data = can be omitted, but it is better to use it
#'   t.test(~ age, HELPrct)
#'   # the original 2-sample formula
#'   t.test(age ~ sex, data = HELPrct)
#'   # alternative 2-sample formulas
#'   t.test(~ age | sex, data = HELPrct)
#'   t.test(~ age, groups = sex, data = HELPrct)
#'   # 2-sample t from vectors
#'   with(HELPrct, t.test(age[sex == "male"], age[sex == "female"]))
#'   # just the means
#'   mean(age ~ sex, data = HELPrct)

#' @export
t_test <- function(x, ...) {
  UseMethod("t_test")
}

#' @rdname ttest
#' @export
t.test <- function(x, ...) {
  UseMethod("t_test")
}

#' @rdname ttest
#' @method t_test formula
#' @export
t_test.formula <- 
  function (formula, data, ..., groups = NULL) {
    
    formula <- 
      mosaicCore::mosaic_formula_q(
        formula, groups = !!rlang::enexpr(groups), max.slots = 2, 
        envir = if (is.environment(data)) data else environment(formula))
    dots <- list(...)
   
    if (length(formula) == 3) {
      if (missing(data)) {
        return( stats::t.test(formula, ...) )
      } else {
        return( stats::t.test(formula, data = data, ...) )
      }
    }
    
    if (missing(data)) {
      data <- parent.frame()
    }
    
    evalF <- mosaicCore::evalFormula(formula, data)
    if (ncol(evalF$right) < 1L) 
      stop("No data specified in rhs of formula.") 
    
    vname <- names(evalF$right)[1L]
    if (ncol(evalF$right) > 1L) {  
      stop("Multiple variables specified in rhs of formula.")
    }
   
    mf <- model.frame(formula, data = data) 
    x <- evalF$right[, 1]
    res <- do.call( stats::t.test, c(list(x = x), dots) ) 
    # res$data.name <- names(mf[1])
    res$data.name <- names(evalF$right)[1]
    return(res)
}

#' @rdname ttest
#' @method t_test default
#' @export
t_test.default <-
  function (x, y = NULL, alternative = c("two.sided", "less", "greater"), 
            mu = 0, paired = FALSE, var.equal = FALSE, conf.level = 0.95, 
            ...) {
    xname <- deparse(substitute(x))
    yname <- deparse(substitute(y))
    res <- 
      stats::t.test( x, y = y, alternative = alternative,
                     mu = mu, paired = paired, var.equal = var.equal, 
                     conf.level = conf.level,  ...) 
    res$data.name <- 
      if (is.null(y)) {
        xname
      } else {
        paste(xname, "and", yname)
      }
    res
  }

# context("t.test()")

Boys <- filter(mosaicData::Galton, sex=="M")
y <- Galton$height
a <- Galton$sex

test_that("2-sample tests give same results as stats::t.test", {
  
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(y ~ a)),
    confint(t.test(y ~ a))
  )
  
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(height ~ sex, data=mosaicData::Galton)),
    confint(t.test(height ~ sex, data=mosaicData::Galton))
  )
  
  expect_equal(ignore_attr = TRUE,  
    with(mosaicData::Galton, confint(stats::t.test(height ~ sex))),
    with(mosaicData::Galton, confint(t.test(height ~ sex)))
  )
  
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(Boys$father, Boys$height)),
    confint(t.test(Boys$father, Boys$height))
  )
  
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(Boys$father, Boys$height)),
    confint(with(Boys, t.test(father, height)))
  )
  
  # expect_equal(ignore_attr = TRUE,  
  #   confint(stats::t.test(Boys$father, Boys$height)),
  #   confint(t.test(father, height, data=Boys))
  # )
})

test_that("paired tests give same results as stats::t.test", {
  
  # expect_equal(ignore_attr = TRUE,  
  #   confint(stats::t.test(Boys$father, Boys$height, paired=TRUE)),
  #   confint(t.test(father, height, data=Boys, paired=TRUE))
  # )
   
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(Boys$father, Boys$height, paired=TRUE)),
    confint(t.test(Boys$father, Boys$height, paired=TRUE))
  )
  
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(Boys$father, Boys$height, paired=TRUE)),
    confint(t.test(~ (father - height), data=Boys))
  )
  
  expect_equal(ignore_attr = TRUE,  
    confint(stats::t.test(Boys$father, Boys$height, paired=TRUE)),
    confint(with(Boys, t.test(father, height, paired=TRUE)))
  )
})

test_that("1-sample tests give same results as stats::t.test", {
  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::t.test(Boys$height)),
    confint(t.test(~ height, data=Boys))
  )
  
  # expect_equal(ignore_attr = TRUE, 
  #   confint(stats::t.test(Boys$height)),
  #   confint(t.test(height, data=Boys))
  # )
  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::t.test(Boys$height)),
    confint(t.test(Boys$height))
  )
  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::t.test(Boys$height)),
    confint(with(Boys, t.test(height)))
  )
})

#' @rdname defunct
#'
#' @export

gwm <- function(...) { # formula, data = parent.frame(), drop = FALSE, ...) {
  .Defunct(msg = "gwm() has been removed from `mosaic'.  \n    It will be replaced by better tools in `mosaicModel'.")
}

utils::globalVariables(c('.mosaicEnv'))

#' Setting options for mosaic package functions
#'
#' A mechanism for setting options in the mosaic package.
#'
#' @rdname mosaic.options
#' @name mosaic.options
#' @aliases mosaicGetOption mosaic.par.set 
#'
#' @param name the name of the option being set
#' @param value the value to which to set the option
#' @export

## copied from lattice

mosaic.options <- function (...) 
{
    new <- list(...)
    if (is.null(names(new)) && length(new) == 1 && is.list(new[[1]])) 
        new <- new[[1]]
    old <- .mosaicEnv$mosaic.options
    if (length(new) == 0) 
        return(old)
    nm <- names(new)
    if (is.null(nm)) 
        return(old[unlist(new)])
    isNamed <- nm != ""
    if (any(!isNamed)) 
        nm[!isNamed] <- unlist(new[!isNamed])
    retVal <- old[nm]
    names(retVal) <- nm
    nm <- nm[isNamed]
    .mosaicEnv$mosaic.options <- .updateList(old, new[nm])
    return(invisible(retVal))
}
#' @rdname mosaic.options
#' @export

mosaic.getOption <- function (name) 
{
    get("mosaic.options", envir = .mosaicEnv)[[name]]
}
#' @rdname mosaic.options
#' @param theme a list appropriate for a mosaic theme
#' @param warn a logical.  UNUSED at present.
#' @param strict a logical or numeric.
#' @param \dots additional arguments that are turned into a list if a list cannot be inferred from 
#' `theme`, `name`, and `value`.
#' @export

mosaic.par.set <- function (name, value, ..., theme, warn = TRUE, strict = FALSE) 
{
    mosaic.theme <- get("mosaic.theme", envir = .mosaicEnv)
	old.mosaic.theme <- mosaic.theme
    if (missing(theme)) {
        if (!missing(value)) {
            theme <- list(value)
            names(theme) <- name
        }
        else if (!missing(name) && is.list(name)) {
            theme <- name
        }
        else theme <- list(...)
    } else {
        if (is.character(theme)) 
            theme <- get(theme)
        if (is.function(theme)) 
            theme <- theme()
        if (!is.list(theme)) {
            warning("Invalid 'theme' specified")
            theme <- NULL
        }
    }
    if (strict) {
        if (strict > 1L) 
            mosaic.theme <- theme
        else mosaic.theme[names(theme)] <- theme
    }
    else mosaic.theme <- .updateList(mosaic.theme, theme)
    assign("mosaic.theme", mosaic.theme, envir = .mosaicEnv)
    return(invisible(old.mosaic.theme))
}
#' @rdname mosaic.options
#' @export

mosaic.par.get <- function (name = NULL) 
{
    mosaic.theme <- get("mosaic.theme", envir = .mosaicEnv)

    if (is.null(mosaic.theme)) {
        mosaic.theme <- get("mosaic.theme", envir = .mosaicEnv)
    }
    if (is.null(name)) 
        mosaic.theme
    else if (name %in% names(mosaic.theme)) 
        mosaic.theme[[name]]
    else NULL
}
#' @keywords ignore
.updateList <- function(x, val)
{
  if (is.null(x)) x <- list()
  modifyList(x, val)
}
#' @keywords ignore
.defaultMosaicOptions <- function()
  list(na.rm=TRUE,
       aggregate.overall=FALSE,
       graphics='lattice'
       )
#' @rdname mosaic.options
#' @details `restoreLatticeOptions` returns any lattice
#' options that were changed when the mosaic package was loaded
#' back to their pre-mosaic state.
#' @export

restoreLatticeOptions <- function() {
  do.call(lattice::lattice.options, .mosaicEnv$original.lattice.options)
}

#' @rdname mosaic.options
#' @details `mosaicLatticeOptions` sets a number 
#' of defaults for lattice graphics.
#' @export

mosaicLatticeOptions <- function() {
  lattice::lattice.options(
    histogram.breaks = xhistogramBreaks,
    prepanel.default.histogram = prepanel.xhistogram,
    panel.histogram = panel.xhistogram)
}

#' @importFrom utils read.delim
NA

#' Read data files
#' 
#' A wrapper around various file reading functions.
#' 
#' @param file character:
#' 	 The name of the file which the data are to be read from. 
#' 	 This may also be a complete URL or a path to a compressed file.
#' 	 If it does not contain an absolute path, the file name is 
#' 	 relative to the current working directory, 
#' 	 `getwd()`.  Tilde-expansion is performed where supported. 
#' 	 See [read.table()] for more details.
#' 
#' @param header logical;  
#'     For `.txt` and `.csv` files, this indicates whether the first line of the file includes variables names.
#' 
#' @param na.strings character: strings that indicate missing data.
#' 
#' @param comment.char 
#' character: a character vector of length one containing a single character or an empty string. Use "" to turn 
#' off the interpretation of comments altogether.
#' 
#' @param filetype one of `"default"`, `"csv"`, `"txt"`, or `"rdata"`
#' indicating the type of file being loaded.  The default is to use the filename
#' to guess the type of file.
#' 
#' @param readr a logical indicating whether functions from the `readr` package should be
#'   used, if available.
#' 
#' @param \dots  additional arguments passed on to 
#'   [read.table()], or [load()] or one of the functions
#'   in the `readr` package.  Note that a message will indicate which 
#'   underlying function is being used.
#'   
#' @details
#' Unless `filetype` is specified,
#' `read.file` uses the (case insensitive) file extension to determine how to read
#' data from the file.  If `file` ends in `.rda` or `.rdata`, then
#' [load()] is used to load the file.  If `file`
#' ends in `.csv`, then [readr::read_csv()] or [read.csv()] is used.  
#' Otherwise, [read.table()] is used.
#' @param package if specified, files will be searched for among the documentation
#' files provided by the package.
#' @param stringsAsFactors a logical indicating whether strings should be converted to factors.
#'   This has no affect when using `readr`.
#' 
#' @return A data frame, unless `file` unless `filetype` is `"rdata"`, 
#' in which  case arbitrary objects may be loaded and a character vector
#' holding the names of the loaded objects is returned invisibly.
#' @seealso [read.csv()], [read.table()], 
#' [readr::read_table()], [readr::read_csv()], 
#' [load()].
#' 
#' @keywords util 
#' @examples
#' \dontrun{
#' Dome <- read.file("http://www.mosaic-web.org/go/datasets/Dome.csv")
#' }
#' @export

read.file <-
  function (file, header = T, na.strings = "NA",
            comment.char = NULL, 
            filetype = c("default", "csv", "txt", "tsv", "fw", "rdata"), 
            stringsAsFactors = FALSE,
            readr = FALSE,
            package = NULL, ...) 
{
    readr_available <- readr && rlang::is_installed('readr')
    using_readr <- FALSE
    
    if (!is.null(package)) {
      file <- docFile(file, package=package, character.only=TRUE)
    }
    filetype <- match.arg(tolower(filetype), choices=filetype)
    if (filetype == "default") {
      filetype <- "txt"
      if (regexpr("\\.csv$", tolower(file)) > 0) {
        filetype <- "csv"
      } 
      
      if (regexpr("\\.tsv$", tolower(file)) > 0) {
        filetype <- "tsv"
      } 
      
      if (regexpr("\\.fw$", tolower(file)) > 0) {
        using_readr <- TRUE
      }
      
      if ( regexpr("\\.rdata$", tolower(file)) > 0 || 
           regexpr("\\.rda", tolower(file)) >0 ){
        filetype <- "rdata"
      } 
    }
    
    if (filetype %in% c("csv", "tsv", "fw")) {
      # try to use readr for these
      using_readr <- TRUE
    }
    
    if (!file.exists(file) && grepl("https://", file)) {  # assume we are reading a URL
      if (! requireNamespace("RCurl")) stop("Package `RCurl' must be installed.")
      file <- textConnection(RCurl::getURL(file))
      using_RCurl <- TRUE
      using_readr <- FALSE
    }
    
    using_readr <- using_readr && readr_available
    
    if (using_readr){ rlang::check_installed('readr') }
    
    if (using_readr) { 
      if (! is.null(comment.char)) message("comment.char is currently being ignored.")
      if (length(na.strings) > 1) {
        message("Currently, only the first item in na.strings is used.")
        message("Additional items will be ignored.")
        na.strings = na.strings[1]
      } 
    }
    if (filetype == "csv") {
      if (using_readr) {
        message("Reading data with readr::read_csv()")
        return(as.data.frame(readr::read_csv(file, col_names = header, na = na.strings, ...)))
      } else {
        message("Reading data with read.csv()")
        return(read.csv(file, header=header, na.strings = na.strings, 
                        stringsAsFactors = stringsAsFactors, ...))
      }
    }
    
    if (filetype == "fw" && readr_available) {
      rlang::check_installed('readr')
      message("Reading data with readr::read_table()")
      return(as.data.frame(readr::read_table(file, col_names = header, na = na.strings, ...)))
    }
    
    if (filetype == "tsv") {
      if (using_readr) {
        message("Reading data with readr::read_tsv()")
        return(as.data.frame(readr::read_tsv(file, col_names = header, na = na.strings, ...)))
      } else {
        message("Reading data with read.delim()")
        return(read.delim(file, header = header, na = na.strings, ...))
      }
    }
    
    if (filetype == "rdata") {
      message("Reading data with load(); ignoring extra arguments.")
      varNames <- load(file)  
      return(invisible(varNames))
    }
    
    # fall through to read.table() for any other file format.
    message("Reading data with read.table()")
    return(
      read.table(file, header = header, na.strings = na.strings, stringsAsFactors=stringsAsFactors,...)
    )
}

#' Convert between degrees and radians
#' 
#' Facilitates conversion between degrees and radians.
#' @rdname deg2rad
#' @return a numeric vector
#' @param x a numeric vector
#' @examples
#' deg2rad(180)
#' @export

deg2rad <- function(x) {
	x/180 * base::pi
}

#' @rdname deg2rad
#' @examples
#' rad2deg(2*pi)
#' @seealso [latlon2xyz()], [googleMap()], and [rgeo()].
#' @export

rad2deg <- function(x) {
	x / base::pi * 180
}

#' Convert back and forth between latitude/longitude and XYZ-space
#' 
#' @rdname latlon2xyz
#' @return a matrix each row of which describes the latitudes and longitudes
#' @param x,y,z numeric vectors
#' @examples
#' xyz2latlon(1, 1, 1)     # point may be on sphere of any radius
#' xyz2latlon(0, 0, 0)     # this produces a NaN for latitude
#' @export

xyz2latlon <- function(x,y,z) {
	# rescale to unit sphere
	R <- sqrt( x^2 + y^2 + z^2)
	x <- x/R; y <- y/R; z <- z/R;

	r <- sqrt( x^2 + y^2)
	lat <- rad2deg( asin(z) )

	long <- (r>0) * rad2deg( acos(x/r) ) 
	long <- ( 1 - 2 * (y < 0) ) * long
	long [ is.na(long) ] <- 0
	return( cbind(lat=lat, lon=long) )
}

#' @rdname latlon2xyz
#' @param latitude,longitude vectors of latitude and longitude values
#' @return a matrix each row of which contains the x, y, and z coordinates of a point on a unit sphere
#' 
#' @examples
#' latlon2xyz(30, 45)
#' lonlat2xyz(45, 30)
#' @seealso [deg2rad()], [googleMap()], and [rgeo()].
#' @export

latlon2xyz <- function(latitude,longitude) {
	z <- sin(deg2rad(latitude))
	r <- sqrt(1 - z^2)
	x <- rad2deg(cos( longitude ))
	y <- rad2deg(sin( longitude ))
	return(cbind( x=x, y=y, z=z ))
}

#' @rdname latlon2xyz
#' @export
lonlat2xyz <- function(longitude, latitude) {
  latlon2xyz(latitude, longitude)
}

#' @rdname rgeo
#' @export
rlatlon <- function(...) { 
  rgeo(...)
}


#' @rdname rgeo
#' @export
rlonlat <- function(...){
  rgeo(...)[,c(2,1)]
}

#' Sample longitude and latitude on a sphere
#' 
#' Randomly samples longitude and latitude on earth so that equal areas are
#' (approximately) equally likely to be sampled.  
#' (Approximation assumes earth as a perfect sphere.)
#' 
#' @rdname rgeo
#' @param n number of random locations
#' 
#' @param latlim,lonlim  
#'   range of latitudes and longitudes to sample within, only implemented for `rgeo`.
#' 
#' @param verbose 
#'   return verbose output that includes Euclidean coordinates on unit sphere as well as 
#' longitude and latitude.
#' 
#' @param ... arguments passed through to other functions
#'
#' @return a data frame with variables `long` and `lat`.  If `verbose` is
#' TRUE, then x, y, and z coordinates are also included in the data frame.
#' @examples
#' rgeo(4)
#' # sample from a region that contains the continental US
#' rgeo(4, latlim = c(25,50), lonlim = c(-65, -125))
#' @details
#' `rgeo` and `rgeo2` differ in the algorithms used to generate random positions.  
#' Each assumes a spherical globe.  `rgeo` uses that fact that each of the x, y and z
#' coordinates is uniformly distributed (but not independent of each other).  Furthermore, the 
#' angle about the z-axis is uniformly distributed and independent of z.  This provides 
#' a straightforward way to generate Euclidean coordinates using `runif`.  These are then
#' translated into latitude and longitude.
#' 
#' `rlatlon` is an alias for `rgeo` and 
#' `rlonlat` is too, expect that it reverses the 
#' order in which the latitude and longitude values are 
#' returned.
#' 
#' `rgeo2` samples points in a cube by independently sampling each coordinate.  It then
#' discards any point outside the sphere contained in the cube and projects the non-discarded points
#' to the sphere.  This method must oversample to allow for the discarded points.
#' 
#' @seealso [deg2rad()], [googleMap()] and [latlon2xyz()].
#' 
#' @keywords random 
#' @keywords geometry 
#' @keywords map 
#' @export

rgeo <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {

	zlim <- sin(sort(deg2rad(latlim)))
	z <- runif( n,  zlim[1], zlim[2] )

	#beta <- runif( n, min(deg2rad(lonlim)), max(deg2rad(lonlim)) )
	blim <- deg2rad(sort(lonlim))
	beta <- runif( n, blim[1], blim[2] )

	r <- sqrt(1-z^2)
	x <- r * cos(beta)
	y <- r * sin(beta)

	# now convert this to latitude and longitude 
	latlon <- xyz2latlon(x,y,z)
	if (verbose) {
		return(data.frame(lat=latlon[,1], lon=latlon[,2], x=x, y=y, z=z))
	}
	return(data.frame(lat=latlon[,1], lon=latlon[,2]))
}

#' @rdname rgeo
#' @examples
#' rgeo2(4)
#' @export

rgeo2 <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {

	# oversample pts in a cube

	m <- 10 + 3*n
	x <- runif(m,-1,1)
	y <- runif(m,-1,1)
	z <- runif(m,-1,1)

	# select pts inside unit sphere and project them onto surface

	r <- sqrt( x^2 + y^2 + z^2 )
	ids <- which( r < 1 ) [1:n]
	x <- x[ids]
	y <- y[ids]
	z <- z[ids]
	r <- r[ids]
	x <- x/r
	y <- y/r
	z <- z/r

	# now convert this to latitude and longitude 
	latlon <- xyz2latlon(x,y,z)
	if (verbose) {
		return(data.frame(lat=latlon[,1], lon=latlon[,2], x=x, y=y, z=z))
	}
	return(data.frame(lat=latlon[,1], lon=latlon[,2]))
}
#' Display a point on earth on a Google Map
#' 
#' Creates a URL for Google Maps for a particular latitude and
#' longitude position.  This function has been deprecated due to changes in
#' Google's access policies.  Give [leaflet_map()] a try as an alternative.

#' @rdname googleMap
#' @param latitude,longitude vectors of latitude and longitude values
#' @param position a data frame containing latitude and longitude positions
#' @param zoom zoom level for initial map (1-20)
#' @param maptype one of `'roadmap'`, `'satellite'`, `'terrain'`, and `'hybrid'`
#' @param mark a logical indicating whether the location should be marked with a pin
#' @param radius a vector of radii of circles centered at position that are displayed on the map
#' @param browse a logical indicating whether the URL should be browsed (else only returned as a string)
#' @param \dots additional arguments passed to `browseURL`
#' @return a string containing a URL.  Optionally, as a side-effect, the URL is visited in a browser
#' @examples
#' \dontrun{
#' googleMap(40.7566, -73.9863, radius=1)   # Times Square
#' googleMap(position=rgeo(2), radius=1)    # 2 random locations
#' }
#' @seealso [leaflet_map()], [deg2rad()], [latlon2xyz()] and [rgeo()].
#' @export

googleMap <- function(latitude, longitude, position=NULL,
	zoom=12, 
	maptype=c('roadmap','satellite','terrain','hybrid'),
	mark=FALSE,
	radius=0,
	browse=TRUE,
	...
	)
{
  .Deprecated(new = "leaflet_map")
  
	urls <- .googleMapURL( 
				latitude=latitude, longitude=longitude,
				position=position, zoom=zoom, 
				maptype = maptype, mark=mark, radius=radius
			)

	if (browse) {
		return(invisible( sapply( urls, function(x) { browseURL(x,...) } ) ))
	} else {
		return(invisible(urls))
	}
}
#' Simple Leaflet Maps
#' 
#' Primarily designed to work with [rgeo()] to display randomly sampled
#' points on the globe.
#' 
# @importFrom leaflet leaflet addTiles addCircles addMarkers
#' @param latitude,longitude vectors of latitude and longitude values.
#'   If `latitude` is a data frame, then it is treated as `position`.
#'   This facilitates "piping" from [rgeo()].  See examples.
#' @param position a data frame containing latitude and longitude positions
#' @param zoom zoom level for initial map (1-20)
# #' @param maptype one of `'roadmap'`, `'satellite'`, `'terrain'`, and `'hybrid'`
#' @param mark a logical indicating whether the location should be marked with a pin
#' @param radius a vector of radii of circles (in miles) centered at position that are displayed on the map
#' @param units units for radii of circles (km, miles, meters, or feet).
#' @param \dots additional arguments passed to [leaflet::addCircles()]
#' @return a leaflet map
#' @examples
#' # the leaflet package is required
#' if (require(leaflet)) {
#'   # Times Square
#'   leaflet_map(40.7566, -73.9863, radius = 1, units = "miles")  
#'   # 3 random locations; 5 km circles
#'   leaflet_map(position = rgeo(3), radius = 5, mark = TRUE, color = "red") 
#'   # using pipes
#'   rgeo(4, latlim = c(25,50), lonlim = c(-65, -125)) |>
#'     leaflet_map(radius = 5, mark = TRUE, color = "purple")
#' }
#' @seealso [deg2rad()], [latlon2xyz()] and [rgeo()].
#' @export
 
leaflet_map <- 
  function(latitude = NULL, longitude = NULL, position = NULL,
           zoom = 12,  
           mark = FALSE, radius = 0, 
           units = c("km", "miles", "meters", "feet"),
           ...
  ) {
    rlang::check_installed('leaflet')
    units <- match.arg(units)
    # how many meters per unit?
    if (inherits(latitude, "data.frame")) {
      position <- latitude
      latitude <- NULL
      longitude <- NULL
    }
    
    conversion <- switch(
      units,
      miles = 1609.34,
      km = 1000,
      meters = 1,
      feet = 1609.34 / 5280
    )
    if (! is.null(position)) {
      if (!is.null(latitude) || !is.null(longitude)) {
        stop("Specify either position or lat/lon, but not both.")
      }
      if ("latitude" %in% names(position)) {
        latitude  <- position[, "latitude"]
      } else {
        latitude  <- position[, 1]
      }
      if ("longitude" %in% names(position)) {
        longitude  <- position[, "longitude"]
      } else {
        longitude  <- position[, 2]
      }
    }
    
    m <-
      leaflet::leaflet() |>
      leaflet::addTiles() # Add default OpenStreetMap map tiles
    if (mark) {
      m <- m |> leaflet::addMarkers(
        lng = longitude, lat = latitude, 
        popup = paste("lat/lon: ", format(latitude, digits = 5), 
                      "/", format(longitude, digits = 5), sep = "")
      )
    }
    m <- m |> 
      leaflet::addCircles(
        lng = longitude,
        lat = latitude,
        radius = radius * conversion,  # convert from unit to meters
        popup = paste(
          "radius: ", radius, " ", units, "; 
          lat/lon: ",  format(latitude, digits = 5),  "/", format(longitude, digits = 5), 
          sep = ""),
        ...
      )
    
    return(m) 
    
  }


#' rgeo internal functions
#' 
#' These are not really intended for public consumption.
#'
#' @name rgeo-internals
#' @rdname rgeo-internals
#' @return a URL as a string
#' @inheritParams googleMap
#' @keywords internal
#' 
.googleMapURL <- function(latitude, longitude, position=NULL,
	zoom=11, 
	maptype=c('roadmap','satellite','terrain','hybrid'),
	mark=FALSE,
	radius=0
	) 
{
	filename <- "googlemap3.html"

	if (FALSE) { # can't get browseURL to accept a file with parameters added on
		package <- "mosaic"
		paths <- .find.package(package, verbose = TRUE)
		paths <- paths[file_test("-d", file.path(paths, "google"))]
		paths <- file.path(paths, "google")
		paths <- paths[file_test("-f", file.path(paths, filename))]
		url <- file.path(paths, filename)
		url <- paste("file://",url,sep="")
	}
	url <- paste('http://www.mosaic-web.org/go/',filename,sep="")
	if (is.null(position)) {
		position <- data.frame(lat=latitude,lon=longitude)
	}
	latitude  <- position[,1]
	longitude <- position[,2]
	maptype <- match.arg(maptype)
	center <- paste(latitude,",",longitude,sep="")
	markString <- ""
	if (mark == TRUE) { markString <- paste('&mlat=',round(latitude,6),'&mlon=',round(longitude,6) ,sep="") } 

	return(invisible(paste(
		url,
		'?lat=', round(latitude,6),
		'&lon=', round(longitude,6),
		markString,
		'&zoom=', zoom,
		'&radius=', paste(as.character(radius),collapse=","),
		sep="")))
}

#' .openStreetMapURL <- function(latitude, longitude, position=NULL,
#' 	zoom=11, 
#' 	maptype=c('roadmap','satellite','terrain','hybrid'),
#' 	mark=FALSE,
#' 	radius=0
#' 	) 
#' {
#' 	filename <- "openstreet.html"
#' 
#' 	if (FALSE) { # can't get browseURL to accept a file with parameters added on
#' 		package <- "mosaic"
#' 		paths <- .find.package(package, verbose = TRUE)
#' 		paths <- paths[file_test("-d", file.path(paths, "google"))]
#' 		paths <- file.path(paths, "google")
#' 		paths <- paths[file_test("-f", file.path(paths, filename))]
#' 		url <- file.path(paths, filename)
#' 		url <- paste("file://",url,sep="")
#' 	}
#' 	url <- paste('http://www.mosaic-web.org/go/',filename,sep="")
#' 	if (is.null(position)) {
#' 		position <- data.frame(lat=latitude,lon=longitude)
#' 	}
#' 	latitude  <- position[,1]
#' 	longitude <- position[,2]
#' 	maptype <- match.arg(maptype)
#' 	center <- paste(latitude,",",longitude,sep="")
#' 	markString <- ""
#' 	if (mark == TRUE) { markString <- paste('&mlat=',round(latitude,6),'&mlon=',round(longitude,6) ,sep="") } 
#' 
#' 	return(invisible(paste(
#' 		url,
#' 		'?lat=', round(latitude,6),
#' 		'&lon=', round(longitude,6),
#' 		markString,
#' 		'&zoom=', zoom,
#' 		'&radius=', paste(as.character(radius),collapse=","),
#' 		sep="")))
#' }
#' #' @rdname rgeo-internals
#' #' @param width,height width and height of window containing google map
#' #' @keywords internal
#' #'
#' .googleMapURL2 <- function(latitude, longitude, position=NULL,
#' 	zoom=12, 
#' 	width=600, 
#' 	height=400, 
#' 	maptype=c('roadmap','satellite','terrain','hybrid'),
#' 	mark=FALSE
#' 	) 
#' {
#' 	latitude  <- position[,1]
#' 	longitude <- position[,2]
#' 	url <- "http://maps.google.com/maps/api/staticmap?"
#' 	maptype <- match.arg(maptype)
#' 	center <- paste(latitude,",",longitude,sep="")
#' 	size <- paste(width,'x',height,sep="")
#' 	markString <- ""
#' 	if (mark == TRUE) { markString <- paste('&markers=size:tiny|', center,sep="") } 
#' 
#' 	return(invisible(paste(
#' 		url,
#' 		'center=', center,
#' 		markString,
#' 		'&zoom=', zoom,
#' 		'&size=', size,
#' 		'&sensor=false', 
#' 		'&maptype=', maptype,
#' 		sep="")))
#' }

#' Exact Tests for Proportions
#' 
#' The `binom.test()` function
#' performs an exact test of a simple null hypothesis about the probability of success in a 
#' Bernoulli experiment from summarized data or from raw data.
#' The mosaic `binom.test` provides wrapper functions around the function of the same name in \pkg{stats}.
#' These wrappers provide an extended interface (including formulas).  
#' 
# @usage binom.test( x, n, p = 0.5, alternative = c("two.sided", "less", "greater"), conf.level = 0.95,...) 
#'
#' @param x  count of successes, length 2 vector of success and failure counts, a formula,
#'   			or a character, numeric, or factor vector containing raw data.
#' 
#' @param n  sample size (successes + failures) or a data frame 
#'   (for the formula interface) 
#' @param p probability for null hypothesis 
#' @param alternative  type of alternative hypothesis 
#' @param conf.level  confidence level for confidence interval 
#' @param success  level of variable to be considered success.  All other levels are 
#'   	considered failure.
#' @param data a data frame (if missing, `n` may be a data frame)
#' @param ci.method a method to use for computing the confidence interval 
#'   (case insensitive and may be abbreviated).  See details below.
#' @param ... additional arguments (often ignored) 
#' 
#' @return an object of class `htest`
#' 
#' @note When `x` is a 0-1 vector, 0 is treated as failure and 1 as success. Similarly,
#' for a logical vector `TRUE` is treated as success and `FALSE` as failure.
#' 
#'
#' @seealso [mosaic::prop.test()], [stats::binom.test()]
#' 

#' @details 
#' `binom.test()` is a wrapper around [stats::binom.test()] from the stats 
#' package to simplify its use when the raw data are available, in which case 
#' an extended syntax for `binom.test()` is provided.  See the examples.
#' 
#' Also, five confidence interval methods are provided:
#' * 
#' `"Clopper-Pearson", "binom.test"`: This is the interval produced when using 
#'   [stats::binom.test()]
#'   from the stats package.  It guarantees a coverage rate at least as large as 
#'   the nominal coverage rate, but may produce wider intervals than some of the methods
#'   below, which may either under- or over-cover depending on the data.
#' * `"Score", "Wilson", "prop.test":  This is the usual method used by [stats::prop.test()]
#'   and is computed by inverting p-values from score tests. It is often attributed to 
#'   Edwin Wilson.  If specified with \code{"prop.test"}, the continuity correction is applied
#'   (as is the default in \code{\link{prop.test}()}), else the continuity correction is not 
#'   applied.
#'   * `"Wald"` This is the interval traditionally taught in entry level statistics courses.
#'   It uses the sample proportion to estimate the standard error and uses normal
#'   theory to determine how many standard deviations to add and/or subtract from
#'   the sample proportion to determine an interval.
#'   * \"Agresti-Coull"`
#'   This is the Wald method after setting \eqn{n' = n + z^2} and 
#'   \eqn{p'= (x + z^2/2) / n}' and using \eqn{x' = n' p'} and \eqn{n'}
#'   in place of \eqn{x} and \eqn{n}.
#'   
#'   * `"Plus4"`
#'   This is Wald after adding in two artificial success and two artificial failures.  It 
#'   is nearly the same as the Agresti-Coull method when the confidence level is 95%. since
#'   \eqn{z^2} is approximately 4 and \eqn{z^2/2} is approximately 2.
#'   
#' 
#' @examples
#' # Several ways to get a confidence interval for the proportion of Old Faithful
#' # eruptions lasting more than 3 minutes.
#' data(faithful)
#' binom.test(faithful$eruptions > 3)
#' binom.test(97, 272)
#' binom.test(c(97, 272-97))
#' faithful$long <- faithful$eruptions > 3
#' binom.test(faithful$long)
#' binom.test(resample(1:4, 400), p=.25)
#' binom.test(~ long, data = faithful)
#' binom.test(~ long, data = faithful, ci.method = "Wald")
#' binom.test(~ long, data = faithful, ci.method = "Plus4")
#' with(faithful, binom.test(~long))
#' with(faithful, binom.test(long))
#' 
#' @keywords stats
#' 
#' @rdname binom.test
#' @export
binom.test <- 
  function( 
    x, n = NULL, p = 0.5, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, 
    ci.method = c("Clopper-Pearson", "binom.test", "Score", "Wilson", "prop.test", "Wald", "Agresti-Coull", "Plus4"), 
    data = NULL,
    success = NULL,
    ...) 
  {
    missing_n <- !is.null(n)
    x_quosure <- rlang::enquo(x)
    x_eval <- 
      tryCatch(
        rlang::eval_tidy(x_quosure, data),
        error = function(e) {
          if (is.null(data) && ! missing_n) {
            stop("binom.test(): Improper `n'; did you forget `data =' perhaps?", call. = FALSE) 
          }
          rlang::f_rhs(x_quosure)
        }
      )
    
    # data.name should always hold a string or list of strings.
    # these will be combined as needed later to create a single string.
    
    data.name <- 
      list(x    = deparse(rlang::enexpr(x)), 
           n    = deparse(rlang::enexpr(n)), 
           data = deparse(substitute(data))
      )
    
    ci.method <- 
      match.arg(
        tolower(ci.method)[1], 
        choices = c("clopper-pearson", "binom.test", "prop.test", "score", "wilson", 
                    "wald", "agresti-coull", "plus4"))
    
    if (ci.method %in% c("wilson")) ci.method <- "score"
    if (ci.method %in% c("binom.test")) ci.method <- "clopper-pearson"
    
    res <- update_ci(
      binom_test(
        x = x_eval,
        n = n,
        p = p,
        alternative = alternative, 
        conf.level = conf.level, 
        data.name = data.name,    # ignored by some methods, used by others
        data = data,
        success = success,
        ...),
      method = ci.method
    )
    
    res
  }

setGeneric(
  "binom_test",
  function( x, n = NULL, p = 0.5, 
            alternative = c("two.sided", "less", "greater"), 
            conf.level = 0.95, ...) 
  {
    standardGeneric('binom_test')
  }
)

## @aliases binom_test,ANY-method
setMethod(
  'binom_test',
  'ANY',
  function(
    x, n, p = 0.5, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    stats::binom.test( x = x, n = n , p = p,
                       alternative = alternative,
                       conf.level = conf.level,...) 
  }
)

## @aliases binom_test,formula-method
setMethod(
  'binom_test',
  'formula',
  function(
    x, n = NULL, p = 0.5, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (is.null(data)) {
      if (! is.null(n)) stop("Improper `n'; did you forget `data = ' perhaps?", call. = FALSE)
      data <- environment(x) # rlang::fn_env(x)
    }
    
    formula <- mosaic_formula_q(x, groups = NULL, max.slots = 1)
    dots <- list(...)
    
    form <- lattice::latticeParseFormula(formula, data, 
                                         subscripts = TRUE, drop = TRUE)
    if (missing(data.name)) {
      data.name <- 
        paste0(deparse(substitute(data)), "$", form$right.name)
    } 
    if (is.list(data.name)) {
      data.name <- 
        paste0(data.name$data, "$", form$right.name)
    }
    # now data.name should be set and data should hold the data
    
    subscr <- form$subscr
    cond <- form$condition
    x <- form$right
    if (length(cond) == 0) {
      cond <- list(gl(1, length(x)))
    }
    
    binom_test(x, p = p, alternative = alternative, 
               conf.level = conf.level, success = success, 
               data.name = data.name, ...)
  }
)

##  @aliases binom_test,numeric-method
setMethod(
  'binom_test',
  'numeric',
  function( x,  n = NULL, p = 0.5, 
            alternative = c("two.sided", "less", "greater"), 
            conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) 
      stop( "binom.test: If data is not NULL, first argument should be a formula.")
    
    if ( length(x) == 1 ) {
      result <- 
        stats::binom.test(x = x, n = n, p = p, alternative = alternative,
                          conf.level = conf.level) 
      if (is.list(data.name)) {  ### check this VV
        result$data.name <- paste(data.name$x, "out of", data.name$n)
      } else {
        result$data.name <- paste(rlang::enexpr(x), "out of", rlang::enexpr(n))
      }
      return(result)
    }
    
    if ( length(x) == 2 ) {
      result <- 
        stats::binom.test(
          x = x[1], n = base::sum(x), p = p, 
          alternative = alternative, conf.level = conf.level) 
      if (is.list(data.name)) {
        result$data.name <- data.name$x 
      } else {
        result$data.name <- deparse(rlang::enexpr(x))
      }
      return(result)
    }
    
    if (missing(data.name)) { 
      data.name <- deparse(rlang::enexpr(x))
    }
    if (is.list(data.name)) {
      data.name <- data.name$x  # deparse(rlang::f_rhs(data.name$x))
    }
    
    # set a reasonable value for success if none given
    if (is.null(success)) {
      success <- 
        if (all(x %in% c(0, 1))) 1 else
          if (0 %in% x) 0 else 
            min(x, na.rm=TRUE)
    }
    
    binom_test(x = factor(x), p = p, alternative = alternative, 
               conf.level = conf.level, 
               success = success, 
               data.name = data.name, ...)
  }
)

## @aliases binom_test,character-method
setMethod(
  'binom_test',
  'character',
  function(
    x,  n, p = 0.5, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) 
      stop("binom.test: If data is not NULL, first argument should be a formula.")
    
    if (missing(data.name)) { 
      data.name <- deparse(rlang::enexpr(x))
    }
    if (is.list(data.name)) {
      data.name <- data.name$x  # deparse(rlang::f_rhs(data.name$x)) 
    }
    binom_test(x = factor(x), p = p, alternative = alternative, 
               conf.level = conf.level, 
               success = success, 
               data.name = data.name, ...)
  }
)

## @aliases binom_test,logical-method
setMethod(
  'binom_test',
  'logical',
  function(
    x,  n, p = 0.5, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) stop( "binom.test: If data is not NULL, first argument should be a formula.")
    
    if (missing(data.name)) { 
      data.name <- deparse(rlang::enexpr(x))
    }
    if (is.list(data.name)) {
      data.name <- data.name$x  # deparse(rlang::f_rhs(data.name$x)) 
    }
    binom_test(x = factor(x, levels = c('TRUE','FALSE')), 
               p = p, alternative=alternative, 
               conf.level = conf.level, 
               success    = success, 
               data.name  = data.name, ...)
  }
)

## @aliases binom_test,factor-method
setMethod(
  'binom_test',
  'factor',
  function(
    x,  n, p = 0.5, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success=NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) {
      stop( "binom.test: If data is not NULL, first argument should be a formula.")
    }
    
    if (missing(data.name)) { 
      data.name <- deparse(rlang::enexpr(x))
    }
    if (is.list(data.name)) { 
      data.name <- data.name$x  
    }
    if ( missing(success) || is.null(success) ) {
      success <- levels(x)[1]
    }
    x <- x [!is.na(x)]
    count <- base::sum(x==success)
    n <- length(x)
    result <- stats::binom.test( x=count, n=n , p = p,
                                 alternative = alternative,
                                 conf.level = conf.level, ...) 
    result$data.name <- data.name
    if (!is.null(success)) {
      result$data.name <- 
        paste0(data.name, "  [with success = ", success, "]")
    }
    return(result)
  }
)

# This allows us to use these functions from MASS without attaching the MASS package.

#' @importFrom MASS fitdistr
#' @export
MASS::fitdistr

#' @importFrom MASS fractions
#' @export
MASS::fractions

#' @importFrom mosaicCore lhs rhs condition makeFun 
#' @importFrom mosaicCore counts props prop prop1 perc percs count counts tally 
#' @importFrom mosaicCore dfapply ediff inspect msummary n_missing
#' @importFrom mosaicCore logit ilogit
#' @importFrom mosaicCore compare 
#' @export
mosaicCore::lhs
#' @export
mosaicCore::rhs
#' @export
mosaicCore::condition
#' @export
mosaicCore::makeFun
#' @export
mosaicCore::counts
#' @export
mosaicCore::props
#' @export
mosaicCore::prop
#' @export
mosaicCore::prop1
#' @export
mosaicCore::perc
#' @export
mosaicCore::percs
#' @export
mosaicCore::count
#' @export
mosaicCore::counts
#' @export
mosaicCore::tally
#' @export
mosaicCore::dfapply
#' @export
mosaicCore::ediff
#' @export
mosaicCore::inspect
#' @export
mosaicCore::msummary
#' @export
mosaicCore::n_missing
#' @export
mosaicCore::logit
#' @export
mosaicCore::ilogit
#' @export
mosaicCore::compare

#' Extract value from an object
#' 
#' Functions like [integrate()] and [nlm()] return objects that contain more 
#' information that simply the value of the integration or optimization.  `value()` extracts
#' the primary value from such objects.  Currently implemented situations include the output from
#' [integrate()],
#' [nlm()],
#' [cubature::adaptIntegrate()], and 
#' [uniroot()].
#' 
#' @rdname value
#' @param object an object from which a "value" is to be extracted.
#' @param ... additional arguments (currently ignored).
#' @export
#' @examples
#' integrate(sin, 0, 1) |> value()
#' nlm(cos, p = 0) |> value()
#' uniroot(cos, c(0, 2)) |> value()
 
value <- function(object, ...) {
  UseMethod("value")
}

#' @rdname value
#' @export
value.integrate <- function(object, ...) {
  object$value
}

#' @rdname value
#' @export
value.default <- function(object, ...) {
 
  # for anything with a value slot -- in particular integrate() 
  if ("value" %in% names(object))
    return(object$value)
  
  if ("estimate" %in% names(object))
    return(object$estimate)
  
  # # for nlm()
  # 
  # if (all(c("estimate", "minimum", "gradient", "code", "iterations") %in% names(object)))
  #   return(object$estimate)
 
  # for uniroot() 
  if (all(c("root", "f.root", "iter", "init.it", "estim.prec") %in% names(object)))
    return(object$root)
  
  # for cubature::adaptIntegrate()
  if (all(c("integral", "error", "functionEvaluations", "returnCode")  %in% names(object))){   
    if (! object$returnCode == 0L)
      warning("Non-zero return code from adaptIntegrate()", call. = FALSE)
    return(object$integral)
  }
 
  if (is.numeric(object))
    return(as.numeric(object))  # strip off attributes, etc.
  
  # if all else fails...
  return(object)
}

#' Format strings for pretty output
#' 
#' @param x a vector 
#' @param pre text to prepend onto string
#' @param post text to postpend onto string
#' @param width desired width of string
#' @param \dots additional arguments passed to [format()]
#' @return a vector of strings padded to the desired width
#' @examples
#' surround(rbinom(10,20,.5), " ", " ", width=4)
#' surround(rnorm(10), " ", " ", width=8, digits = 2, nsmall = 2)
#' @export

surround <- function (x, pre = " ", post = " ", width = 8, ...) 
{
    x <- format(as.vector(x), ...)
    l <- length(x)
    format(paste(rep(pre, l), x, rep(post, l), sep = ""), width = width, just = "centre")
}

#' @details
#' `.do.safe.call` avoids conflicts between named arguments and ... by taking named arguments
#' preferentially.
#'
#' @rdname mosaic-internal
#'
#' @param what either a function or a non-empty character string naming the function to be called.
#' @param args a list of arguments to the function call. The names attribute of args gives the argument names.
#' @param quote a logical value indicating whether to quote the arguments.
#' @param envir an environment within which to evaluate the call.
#'	This will be most useful if what is a character string and the arguments are symbols or quoted expressions.
#' @return The result of the (evaluated) function call.
#'
#' @seealso [do.call()]
#' @keywords internal

.do.safe.call <- function (what, args, quote = FALSE, envir = parent.frame(), ...) {
	dots <- list(...)
	args <- modifyList(dots, args)
	do.call(what, args, quote, envir)
}

# patterned after similar function in ggplot2
# a bit flakey at the moment, so I'm eliminating its use in favor of directly calling
# requireNamespace.  ---rjp


.try_requireNamespace <-function(package) {
  available <- suppressMessages(suppressWarnings(sapply(package,
                                                        requireNamespace, quietly = TRUE,
                                                        character.only = TRUE,
                                                        warn.conflicts = FALSE)))
  missing <- package[!available]
  if (length(missing) > 0)
    stop("Missing packages.  Please retry after installing the following: ",
         paste(missing, collapse = ", "),
         call. = FALSE)
}

.require_manipulate_namespace <-function() {
  if (! rstudio_is_available())
    stop("RStudio required for manipulate.")
  if (! requireNamespace("manipulate", quietly=TRUE))
    stop("Unable to locate the manipulate package.  Please install with `install.packages(\"manipulate\")'")
  
  if( ! get('mosaic.manipulate.initialized', .mosaicEnv) ) {
    manipulate::manipulate(plot(1:N), N = manipulate::slider(5, 10))
    assign("mosaic.manipulate.initialized", TRUE, envir = .mosaicEnv)
  }
  
}

#' Check whether RStudio is in use
#'
#' This functions checks that RStudio is in use.  It will likely be removed
#' from this package once the versions of RStudio in popular use rely on the
#' manipulate package on CRAN which will provide its own version.
#'
#' @return a logical
#'
#' @rdname rstudio
#'
#' @export
rstudio_is_available <- function() {
  identical(.Platform$GUI, "RStudio")
}

is_object_not_found_error <- function(e, name = "") {
  grepl("object", e$message) && grepl(name, e$message) && grepl("not found", e$message)
}

#' Exact and Approximate Tests for Proportions
#' 
#' The mosaic `prop.test` provides wrapper functions around the function of the same name in \pkg{stats}.
#' These wrappers provide an extended interface (including formulas).  
#' `prop.test` performs an approximate test of a simple null hypothesis about the 
#' probability of success in a Bernoulli or multinomial experiment
#' from summarized data or from raw data.
#' 
# @usage prop.test( x, n, p = NULL, alternative = c("two.sided", "less", "greater"), 
#' 					conf.level = 0.95, ...) 
#' 
#' @param x  count of successes, length 2 vector of success and failure counts, a formula,
#'   			or a character, numeric, or factor vector containing raw data.
#'     		
#'
#' 
#' @param n  sample size (successes + failures) or a data frame 
#'   (for the formula interface) 
#' 
#' @param p  a vector of probabilities of success. 
#' The length of p must be the same as the number of groups specified by x, 
#' and its elements must be greater than 0 and less than 1.
#' 
#' @param alternative   character string specifying the alternative hypothesis, must be one of 
#' `"two.sided"` (default), `"greater"` or `"less"`. You can specify just the initial letter. 
#' Only used for testing the null that a single proportion equals a given value, or that two proportions 
#' are equal; ignored otherwise.
#' 
#' @param conf.level confidence level of the returned confidence interval. Must be a single number 
#' between 0 and 1. Only used when testing the null that a single proportion equals a given value, 
#' or that two proportions are equal; ignored otherwise.
#'
#' @param success  level of variable to be considered success.  All other levels are 
#'   	considered failure.
#'
#' @param data a data frame (if missing, `n` may be a data frame)
#' 
#' @param ... additional arguments (often ignored).  
#'   When `x` is a formula, `groups` can be used to compare groups:  
#'   `x = ~ var, groups=g` is equivalent to ` x = var ~ g `. `na.rm` can be a logical
#'   or an integer vector of length 1 or 2 to indicate dimension along which NA's are 
#'   removed before coputing the test.
#'   See the examples. 
#' 
#' @note When `x` is a 0-1 vector, 0 is treated as failure and 1 as success. Similarly,
#' for a logical vector `TRUE` is treated as success and `FALSE` as failure.
#'
#' @return an `htest` object
#' 
#' @details
#' This is a wrapper around [prop.test()] to simplify its use
#' when the raw data are available, in which case 
#' an extended syntax for `prop.test` is provided.  
#' 
#' @seealso [mosaic::binom.test()], [stats::prop.test()]
#' 
#' @examples
#' # Several ways to get a confidence interval for the proportion of Old Faithful
#' # eruptions lasting more than 3 minutes.
#' prop.test( faithful$eruptions > 3 )
#' prop.test(97,272)
#' faithful$long <- faithful$eruptions > 3
#' prop.test( faithful$long )
#' prop.test( ~long , data = faithful )
#' prop.test( homeless ~ sex, data = HELPrct )
#' prop.test( ~ homeless | sex, data = HELPrct )
#' prop.test( ~ homeless, groups = sex, data = HELPrct )
#' prop.test(anysub ~ link, data = HELPrct, na.rm = TRUE)
#' prop.test(link ~ anysub, data = HELPrct, na.rm = 1)
#' prop.test(link ~ anysub, data = HELPrct, na.rm = TRUE)
#' 
#' @keywords stats

#'
#' @rdname prop.test
#' @export

prop.test <- function( x, n, p = NULL, 
                       alternative = c("two.sided", "less", "greater"), 
                       conf.level = 0.95, data = NULL, success=NULL, ...) 
{
  missing_n <- missing(n)
  x_lazy <- rlang::enquo(x)
  x_eval <- 
    tryCatch(
      rlang::eval_tidy(x_lazy, data),
      error = function(e) {
        if (is.null(data) && ! missing_n) {
          stop("prop.test(): Improper `n'; did you forget `data =' perhaps?", call. = FALSE) 
        }
        rlang::f_rhs(x_lazy)
      }
    )
  
  # this list will later be converted to a string using the appropriate information
  # dependent upon which of the prop_test methods is called.  
  
  data.name <- list(x = rlang::enexpr(x), 
                    n = rlang::enexpr(n), 
                    data = substitute(data)) 
  
  if (missing_n) {
    prop_test(x_eval, p = p, alternative = alternative, 
              conf.level = conf.level, data = data, data.name = data.name, 
              success = success, ...)
  } else {
    prop_test(x_eval, n, p = p, alternative = alternative, 
              conf.level = conf.level, data = data, data.name = data.name, 
              success=success, ...)
  }
}

#' Internal function for testing proportion
#' 
#' This function is wrapped by [`prop.test()`], which most users should use instead.
#' 
#' @param x a vector, count, or formula.
#' @param n a vector of counts of trials (not needed when `x` is a table or matrix).
#' @param p a vector of probabilities of success (for the null hypothesis).  
#'   The length must be the same as the number of groups specified by `x`.
#' @param ... additional arguments passed to methods.
# #' @param data a data frame
# #' @param data.name a character string used to label the data in output.
# #' @param groups an expression defining groups when `x` is a formula.
# #' @param success the level to be considered "success".
#' @inheritParams stats::prop.test
#' @export
prop_test <- 
  function(
    x, n, p = NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, ...)  
  {
    UseMethod("prop_test")
  }


#' @export
prop_test.default <-
  function(
    x, n, p = NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, ..., data, data.name) 
  {
    dots <- list(...)
    dots[['success']] <- NULL
    if (missing(n)) {
      res <- do.call(stats::prop.test, 
                     c(
                       list(x = x, p = p, alternative = alternative,
                            conf.level = conf.level), 
                       dots)
      )
    } else {
      res <- do.call(stats::prop.test, 
                     c(
                       list(x = x, n = n , p = p, alternative = alternative,
                            conf.level = conf.level), 
                       dots)
      )
    }
    res$data.name <- data.name$x
    res
  }

#' @export
prop_test.formula <-
  function(
    x, n, p=NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, data.name, data = NULL, groups = NULL, na.rm = FALSE, ...) 
  {
    missing_n <- missing(n)
    if (is.null(data)) {
      if (! missing_n) stop("Improper `n'; did you forget `data = ' perhaps?", call. = FALSE)
      data <- environment(x)
    }
    
    formula <- 
      mosaicCore::mosaic_formula_q(x, groups = !!rlang::enexpr(groups), max.slots = 2)
    
    missing_data.name <- missing(data.name)
    if (is.null(data)) {
      data <- environment(x)
    }
    
    dots <- list(...)
    #    groups <- eval(substitute(groups), data, environment(formula))
    #    subset <- eval(substitute(subset), data, environment(formula))
    if (missing_n) { #  && !missing.data) {
      form <- lattice::latticeParseFormula(formula, data, #subset = subset, #groups = groups,  
                                           subscripts = TRUE, drop = TRUE)
      if (missing_data.name) {
        data.name <- 
          paste(rlang::enexpr(data), "$", form$right.name, sep="")
      } 
      if (is.list(data.name)) {
        data.name <- 
          paste(data.name$data, "$", form$right.name, sep="")
      }
    } else {
      form <- lattice::latticeParseFormula(formula, n, #subset = subset, #groups = groups,  
                                           subscripts = TRUE, drop = TRUE)
      if (missing_data.name) {
        data.name <- 
          paste(rlang::enexpr(n), "$", form$right.name, sep="")
      }
      if (is.list(data.name)) {
        data.name <- 
          paste(data.name$n, "$", form$right.name, sep="")
      }
      data <- n
    }
    # now data.name should be set and data should hold the data
    
    #    groups <- eval(substitute(groups), data, environment(formula))
    #    subset <- eval(substitute(subset), data, environment(formula))
    groups <- form$groups
    subscr <- form$subscr
    cond <- form$condition
    x <- form$right
    
    if (! is.null(form$left) && !is.null(form$condition) )
      stop("Formulas may not have both lhs and condition for prop.test.")
    
    if (! is.null(form$left) || !is.null(form$condition) ) {
      table_from_formula <-  tally( formula, data=data, margin=FALSE, format="count" )
      if (! is.null(success)) {
        key <- names(dimnames(table_from_formula))[1]
        # if (! success %in% data[, key]) {
        #   stop("(in prop_test) `", success, "' is not a value of `", key, "'", 
        #        call. = FALSE)
        # }
        data[, key] <- factor(data[, key] == success, levels = c("TRUE", "FALSE"))
        table_from_formula <-  tally( formula, data=data, margin=FALSE, format="count" )
      }
      
      dims <- length(dim(table_from_formula))
      if (isTRUE(na.rm)) {
        na.rm <- 1:dims
      }
      if (1 %in% na.rm) {
        table_from_formula <-
          table_from_formula[ which(!is.na(dimnames(table_from_formula)[[1]])), ]
      }
      if (2 %in% na.rm) {
        table_from_formula <-
          table_from_formula[,  which(!is.na(dimnames(table_from_formula)[[2]]))]
      }
      
      if (dim(table_from_formula)[1] > 2) {
        if (any(is.na(dimnames(table_from_formula)[[1]]))) {
          stop(paste0(names(dimnames(table_from_formula))[1], ' has ', dim(table_from_formula)[1],
                      ' levels (including NA).  Only 2 are allowed.'), call. = FALSE)
        }
          stop(paste0(dimnames(table_from_formula)[1], ' has ', dim(table_from_formula)[1],
                      ' levels.  Only 2 are allowed.'))
      }
      for (i in 1:dims) {
        if (any(is.na(dimnames(table_from_formula)[[i]]))) {
          warning(
            call. = FALSE,
            paste0("NA is being treated as a category for ", names(dimnames(table_from_formula))[i]))
        }
      }
      res <- stats::prop.test( t(table_from_formula), 
                               p=p,
                               conf.level=conf.level, 
                               alternative=alternative, 
                               ...)  
      res$data.name <- paste0("tally(", deparse(formula), ")")
      return(res)
    }
    
    if (length(cond) == 0) {
      cond <- list(gl(1, length(x)))
    }
    
    prop_test(x, p=p, alternative=alternative, 
              conf.level=conf.level, success=success, data.name=data.name, ...)
  }

#' @export
prop_test.numeric <-
  function(
    x,  n, p=NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) 
      stop( "prop.test: If data is not NULL, first argument should be a formula.")
    
    # first handle case when n is provided
    if ( !missing(n) ) {  
      if (missing(data.name)) {
        data.name <- paste(rlang::enexpr(x), "out of", rlang::enexpr(n))
      }
      if (is.list(data.name)) {
        data.name <- paste(data.name$x, "out of", data.name$n)
      }
      result <-  stats::prop.test(x=x, n=n, p=p, alternative=alternative,
                                  conf.level=conf.level,...) 
      result$data.name <- data.name 
      if (!is.null(success)) 
        result$data.name <- 
        paste0(data.name, "  [with success = ", success, "]")
      return(result)
    }
    
    # when n is missing, treat the numbers as raw data rather than counts
    
    if (missing(data.name)) { 
      data.name <- rlang::enexpr(x)
    }
    if (is.list(data.name)) {
      data.name <- data.name$x 
    }
    # set a reasonable value for success if none given
    if (is.null(success)) {
      success <- 
        if (all(x %in% c(0, 1))) 1 else
          if (0 %in% x) 0 else 
            min(x, na.rm=TRUE)
    }
    
    prop_test(x=factor(x), p=p, alternative=alternative, 
              conf.level=conf.level, 
              success=success, 
              data.name=data.name, ...)
  }

#' @export
prop_test.character <-
  function(
    x,  n, p = NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) 
      stop( "binom.test: If data is not NULL, first argument should be a formula.")
    
    if (missing(data.name)) { 
      data.name <- rlang::enexpr(x)
    }
    if (is.list(data.name)) { 
      data.name <- data.name$x 
    }
    prop_test(x=factor(x), p=p, alternative=alternative, 
              conf.level=conf.level, 
              success=success, 
              data.name=data.name, ...)
  }

#' @export
prop_test.logical <-
  function(
    x,  n, p=NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success=NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) 
      stop( "binom.test: If data is not NULL, first argument should be a formula.")
    
    if (missing(data.name)) { 
      data.name <- rlang::enexpr(x)
    }
    if (is.list(data.name)) { 
      data.name <- data.name$x 
    }
    prop_test(x=factor(x, levels=c('TRUE','FALSE')), p=p, alternative=alternative, 
              conf.level=conf.level, 
              success=success, 
              data.name=data.name, ...)
  }

#' @export
prop_test.factor <-
  function(
    x,  n, p = NULL, 
    alternative = c("two.sided", "less", "greater"), 
    conf.level = 0.95, success = NULL, ..., data = NULL, data.name) 
  {
    if (! is.null(data)) 
      stop( "binom.test: If data is not NULL, first argument should be a formula.")
    
    if (missing(data.name)) { 
      data.name <- rlang::enexpr(x)
    }
    if (is.list(data.name)) { 
      data.name <- data.name$x 
    }
    if (is.null(success)) {
      success <- levels(x)[1]
    }
    x <- x [!is.na(x)]
    count <- sum(x==success)
    n <- length(x)
    result <- stats::prop.test( x=count, n=n , p = p,
                                alternative = alternative,
                                conf.level = conf.level, ...) 
    result$data.name <- data.name
    if (!is.null(success)) 
      result$data.name <- 
      paste0(data.name, "  [with success = ", success, "]")
    return(result)
  }

#' Alternative formula interface for cor.test
#' 
#' [stats::cor.test()] in \pkg{stats} accepts formulas of the 
#' shape `~ y + x`.  The \pkg{mosaic} package allows the use 
#' of `y ~ x` as an alternative formula shape.
#' 
#' @param formula a formula
#' @param x,y numeric vectors of data values. x and y must have the same length.
#' @param ... other arguments passed to [stats::cor.test()].
#' @seealso [stats::cor.test()] in the \pkg{stats} package.
#' @examples
#' # This is an example from example(stats::cor.test) done in old and new style
#' require(graphics)
#' cor.test(~ CONT + INTG, data = USJudgeRatings)
#' cor.test(CONT ~ INTG, data = USJudgeRatings)
#' @rdname cor.test
#' @export
cor_test.formula <- function(formula, ...) {
  dots <- list(...)
  if (! is.null(lhs(formula))) {
    f <- substitute(~ y + x, list(y = lhs(formula), x = rhs(formula)))
    environment(f) <- environment(formula)
    formula <- f
  }
  do.call(stats::cor.test, c(list(formula), dots))
}

#' @export
#' @rdname cor.test
cor.test <- function(x, ...) {
  UseMethod("cor_test")
}

#' @export
#' @rdname cor.test
cor_test <- function(x, ...) {
  UseMethod("cor_test")
}
    
#' @export
#' @rdname cor.test
cor_test.default <- function(x, y, ...) {
  stats::cor.test(x, y, ...)
}

# context("cor.test()")

testthat::test_that("Cor test works", {
  require(graphics)
  
  testcase1 <- structure(list(statistic = c(t = -0.860504567943113), parameter = c(df = 41L), 
                              p.value = 0.394515192722409, estimate = c(cor = -0.133190890463189), 
                              null.value = c(correlation = 0), alternative = "two.sided", 
                              method = "Pearson's product-moment correlation", data.name = "CONT and INTG", 
                              conf.int = structure(c(-0.41685910878005, 0.174118233649785
                              ), conf.level = 0.95)), class = "htest")
  
  testcase2 <- structure(list(statistic = c(t = -0.860504567943113), parameter = c(df = 41L), 
                              p.value = 0.394515192722409, estimate = c(cor = -0.133190890463189), 
                              null.value = c(correlation = 0), alternative = "two.sided", 
                              method = "Pearson's product-moment correlation", data.name = "CONT and INTG", 
                              conf.int = structure(c(-0.41685910878005, 0.174118233649785
                              ), conf.level = 0.95)), class = "htest")
  
  expect_equal(ignore_attr = TRUE, testcase1, cor.test(~ CONT + INTG, data = USJudgeRatings))
  expect_equal(ignore_attr = TRUE, testcase2, cor.test(CONT ~ INTG, data = USJudgeRatings))
})

#' Find zeros of functions
#' 
#' Compute numerically zeros of a function or simultaneous zeros of multiple functions.
#' @param expr A formula.  The right side names the variable with respect to which the zeros should be found.  
#' The left side is an expression, e.g. `sin(x) ~ x`.  
#' All free variables (all but the variable on the right side) named in the expression must be assigned 
#' a value via `\ldots`
#' @param \dots Formulas corresponding to additional functions to use in simultaneous zero finding
#' and/or specific numerical values for the free variables in the expression.
#' @param xlim The range of the dependent variable to search for zeros. `Inf` is a legitimate value, 
#' but is interpreted in the numerical sense as the non-Inf largest floating point number.  This can also
#' be specified replacing `x` with the name of the variable.  See the examples.
#' @param near a value near which zeros are desired
#' @param within only look for zeros at least this close to near.  `near` and `within` provide an
#' alternative to using `xlim` to specify the search space.
#' @param nearest the number of nearest zeros to return.  Fewer are returned if fewer are found.
#' @param iterate maximum number of times to iterate the search. Subsequent searches take place with the range
#'        of previously found zeros.  Choosing a large number here is likely to kill performance without 
#'        improving results, but a value of 1 (the default) or 2 works well when searching in `c(-Inf,Inf)` for
#'        a modest number of zeros near `near`.
#' @param npts How many sub-intervals to divide the `xlim` into when looking for candidates for zeros.  
#' The default is usually good enough.
#' If `Inf` is involved, the intervals are logarithmically spaced up to the largest finite floating point number.  
#' There is no guarantee that all the roots will be found.
#' @param sortBy specifies how the zeros found will be sorted. Options are 'byx', 'byy', or 'radial'.
#' 
#' @details
#' Searches numerically using `uniroot`.
#' 
#' @return A dataframe of zero or more numerical values.  Plugging these into the
#' expression on the left side of the formula should result in values near zero.
#'
#' @author Daniel Kaplan (\email{kaplan@@macalester.edu}) 
#' 
#' @examples
#' findZeros( sin(t) ~ t, xlim=c(-10,10) )
#' # Can use tlim or t.lim instead of xlim if we prefer
#' findZeros( sin(t) ~ t, tlim=c(-10,10) )
#' findZeros( sin(theta) ~ theta, near=0, nearest=20)
#' findZeros( A*sin(2*pi*t/P) ~ t, xlim=c(0,100), P=50, A=2)
#' # Interval of a normal at half its maximum height.
#' findZeros( dnorm(x,mean=0,sd=10) - 0.5*dnorm(0,mean=0,sd=10) ~ x )
#' # A pathological example
#' # There are no "neareset" zeros for this function.  Each iteration finds new zeros.
#' f <- function(x) { if (x==0) 0 else sin(1/x) }
#' findZeros( f(x) ~ x, near=0 )
#' # Better to look nearer to 0
#' findZeros( f(x) ~ x, near=0, within=100 )
#' findZeros( f(x) ~ x, near=0, within=100, iterate=0 )
#' findZeros( f(x) ~ x, near=0, within=100, iterate=3 )
#' # Zeros in multiple dimensions (not run: these take a long time)
#' # findZeros(x^2+y^2+z^2-5~x&y&z, nearest=3000, within = 5)
#' # findZeros(x*y+z^2~z&y&z, z+y~x&y&z, npts=10)
#' @keywords calculus 
#' @export

findZeros <- function(expr, ..., xlim=c(near-within, near+within), near=0, within=Inf, 
                      nearest=10, npts=1000, iterate=1, sortBy=c('byx', 'byy', 'radial')) {
  dots <- list(...)
  sortBy <- match.arg(sortBy)
  rhsVars <- all.vars(rhs(expr))
  if (is.list(iterate)) { # this is a recursive call
    ignore.limits <- iterate[['ignore.limits']]
    iterate <- iterate[['iterate']]
  } else { # this is the original call
    ignore.limits <- FALSE
  }
  
  if( length(rhsVars) != 1 ){
    if(any(within==Inf))
      within[within==Inf]=100
    return(unique(signif(findZerosMult(expr,..., npts=nearest, rad=within, near = near, sortBy = sortBy), 7)))
  }
  
  pfun <- function(x){  # removed . from name, was .x
    mydots <- dots
    mydots[[rhsVars]] <- x
    eval( lhs(expr), envir=mydots, enclos=parent.frame() )
  }
  
  if (! ignore.limits ) {
    xlim <- inferArgs( dots=dots, vars=rhsVars, defaults=list(xlim=xlim) )[['xlim']]
  }
  tryCatch( xlim <- range(xlim), error = function(e) stop(paste('Improper limits value --', e)))
  
  if( xlim[1] >= xlim[2] )  
    stop(paste("Left limit (", xlim[1], ") must be less than right limit (", xlim[2], ")."))
  internal.near <- near
  if ( internal.near  < xlim[1] || internal.near  > xlim[2]) { 
    internal.near  <- mean(xlim[1],xlim[2]) 
  }
  mx <- max(xlim - internal.near )  # max amount to add to internal.near  when searching
  mn <- min(xlim - internal.near )  # min amount to add to internal.near  when searching (will be negative)
  if (mx < 0) stop('Bug alert: near outside search interval.')
  if (mn > 0) stop('Bug alert: near outside search interval.')
  
  # Deal with very large numbers for the interval, e.g. Inf
  verybig <- .Machine$double.xmax
  plainbig <- npts^(.75) # linear spacing below this.
  mx <- max( min( verybig,mx), -verybig)
  mn <- min( max(-verybig,mn),  verybig)
  rightseq  <- NULL
  leftseq   <- NULL
  middleseq <- NULL
  if( mx > plainbig ) { 
    rightseq <- exp( seq( log(max(plainbig,mn)),log(mx),length=npts) )
  }
  middleseq <- seq( max(-plainbig,mn), min(plainbig,mx), length=npts)
  if( mn < -plainbig ){
    leftseq <- -exp( seq( log(-mn), log(-min(-plainbig,mx)), length=npts))
  }
  
  searchx <- sort(unique(internal.near   + c(0, leftseq, middleseq, rightseq)))
  
  y <- sapply( searchx, pfun )
  testinds <- which( diff(sign(y)) != 0 )
  if (length(testinds) < 1 ) {
    warning("No zeros found.  You might try modifying your search window or increasing npts.")
    return( numeric(0) )
  } else {
    testinds <- testinds[ order(abs(searchx[testinds] - near)) ]
    N <- min( length(testinds), 2*nearest )
    zeros <- rep(NA, N )
    for (k in 1:N) {  # look in subinterval k, i.e., between testinds[k] and  testinds[k+1]
      if ( searchx[testinds[k]] < searchx[testinds[k]+1] ) {
        ur <- uniroot(function(qqzz){ sapply( qqzz, pfun) }, lower=searchx[testinds[k]], upper=searchx[testinds[k]+1])
        zeros[k] <- round( ur$root, digits=trunc(-log10(ur$estim.prec)) )
      } else {
        warning("Potential bug alert: Attempting to search in region where signs of function at endpoints are equal.  Skipping this interval.")
      }
    }
  }
  
  o <- order( abs(zeros - near) )
  result <- sort(unique(zeros[o[1:min(nearest,length(zeros))]]))
  if ( iterate > 0 && length(result) > 1 )  {
    adjust <- min(diff(result))
    # Note: negative value of iterate to indicate that we will be in a secondary iteration
    return ( findZeros( expr, ..., xlim=range(c(result-adjust, result+adjust)), 
                        near=near, within=within, 
                        nearest=nearest, 
                        npts=npts, 
                        iterate= list(iterate=iterate - 1, ignore.limits = TRUE) ) )
  } else {
    result <- data.frame(result)
    colnames(result)<- rhsVars
    return(unique(signif(result, 7)))
  }
}
#' Solve an equation
#' @rdname findZeros
#'
#'
#'@param form Expression to be solved
#'
#'@details Uses findZerosMult of findZeros to solve the given expression
#'
#'@return a dataframe with solutions to the expression.
#'
#'@author Cecylia Bocovich
#'
#'@examples
#'solve(3*x==3~x)
#'
#'# plot out sphere (not run)
#'# sphere = solve(x^2+y^2+z^2==5~x&y&z, within=5, nearest=1000)
#'# cloud(z~x+y, data=sphere)
#'@export

solve.formula <- function(form, ..., near=0, 
                  within=Inf, nearest=10, npts=1000, iterate=1, sortBy=c('byx', 'byy', 'radial')){
  dots = list(...)
  system = list(form)
  sortBy <- match.arg(sortBy)
  freeVars = list()
  
  
  # Separate formulae and free vars
  if(length(dots)>0){
    for(i in (1:length(dots))){
      if (inherits( dots[[i]], "formula"))
        system = append(system, dots[[i]]) # system contains all equations
      else{
        if (inherits(dots[[i]], "numeric"))
          freeVars[[names(dots)[i]]] <- dots[[i]] # freeVars contains values we will sub in
        else stop(paste("Improper value: ", deparse(dots[[i]])))
      }
    }
  }
  
  # change expression into formula.
  for(i in (1:length(system))){
    formula = system[[i]]
    exp = lhs(formula)
    exp = parse(text=paste(deparse(exp[[2]], width.cutoff=500), "-",
                           deparse(exp[[3]], width.cutoff=500), sep=""))[[1]]
    formula[[2]] <- exp
    system[[i]] <- formula
  }
  
  return(do.call(findZeros, c(system, freeVars, near=near, 
                              within=within, nearest=nearest, npts=npts, iterate=iterate, sortBy=sortBy)))
}

# context('Finding Zeros')

test_that("zeros are found", {
  expect_equal(ignore_attr = TRUE,  round(findZeros(x^2 - 2 ~x)[,"x"],4), round(c(-sqrt(2), sqrt(2)),4) )
  expect_equal(ignore_attr = TRUE,  round(findZeros(x^2 - 3 ~x)[,"x"],4), round(c(-sqrt(3), sqrt(3)),4) )
})

test_that("zeros are within search interval", {
  expect_true( all( abs(findZeros(sin(1/x) ~ x, near=0, within=4)) < 4) )
})

test_that("Can find zeros in two variables",{
  Z = findZerosMult(a*x^2-v~a&x, v=8)
  expect_equal(Z[1,]$a*Z[1,]$x^2-8, 0, tolerance = 0.001)
  
  Z = findZerosMult(a^2+x^2-8~a&x, npts = 1000, sortBy='radial')
  expect_equal(Z[46,]$a^2+Z[46,]$x^2-8, 0, tolerance = 0.001)
})

test_that("Works with Broyden",{
  Z = findZeros(x*y+z^2~z&y&z, z+y~x&y&z, npts=10)
  x = Z[,"x"]
  y = Z[,"y"]
  z = Z[,"z"]
  expect_equal(x*y+z^2, rep(0, 10), tolerance = 0.001)
  expect_equal(z+y, rep(0, 10), tolerance = 0.001)
  
  Z = findZeros(x*y+z^2+z^2*w~w&x&y&z, w*z+y~w&x&y&z, npts=10)
  w = Z[,"w"]
  x = Z[,"x"]
  y = Z[,"y"]
  z = Z[,"z"]
  expect_equal(x*y+z^2+z^2*w, rep(0,10), tolerance = 0.001)
  expect_equal(w*z+y, rep(0,10), tolerance = 0.001)
  
  Z = findZeros(x*y^2-9+z~x&y&z, x*y*z+z*w~x&y&z, w=10)
  x = Z[,"x"]
  y = Z[,"y"]
  z = Z[,"z"]
  w = 10
  expect_equal(x*y^2-9+z, rep(0, 10), tolerance = 0.001)
  expect_equal(x*y*z+z*w, rep(0,10), tolerance = 10)
  
})

# These were taken out by DTK on Aug 10, since they are somewhat stochastic.
# test_that("Works properly with different centers", {
#   Z = findZeros((x-500)^2+ (y-200)^2 + (z-300)^2 - 25~x&y&z, near = c(x= 500, y=200, z=300), nearest = 1000, within = 25)
#   x = Z[, "x"]
#   y = Z[, "y"]
#   z = Z[, "z"]
#   expect_that((x-500)^2+ (y-200)^2 + (z-300)^2 - 25, equals(rep(0, 1000), tol=0.01))
# })
# 
# test_that("Solve function works properly", {
#   sphere = solve(x^2+y^2+z^2==5~x&y&z, within=5, nearest=1000)
#   x = sphere[,"x"]
#   y = sphere[,"y"]
#   z = sphere[,"z"]
#   expect_that(x^2+y^2+z^2, equals(rep(5, 1000), tol=.001))
# })

utils::globalVariables(
  c('xend','variable', 
    'Cl', 'xend', 'yend', 'variable', 'variable_num',
    'h', 'position', 'value', 'idx')) 

#' mosaic tools for clustering
#' 
#' @rdname clustering
#' @param model a model
#' @param data a data-like object
#' @param which which kind of fortification to compute
#' @param ... additional arguments passed on to \code{link{dendro_data}}
#' @export

fortify.hclust <- function(model, data, 
                           which=c("segments", "heatmap", "leaves", "labels", "data"), 
                           k=1, ...) {
  
  which <- match.arg(which)
  
  ord <- model$order
  
  if (which == "segments") {
    rlang::check_installed('ggdendro')
    grps <- cutree(model, k=k)
    return( ggdendro::segment(ggdendro::dendro_data(model, ...)) |>
              dplyr::mutate(order = ord[round(xend)], 
                     group = grps[ord[round(xend)]])
            
    )
  }
  
  if (which %in% c("leaves", "labels")) { 
    rlang::check_installed('ggdendro')
    ord <- model$order
    return( ggdendro::label(ggdendro::dendro_data(model, ...)) |> 
              dplyr::mutate( order = ord )
    )
  }
  
  # if (which == "heatmap0") {
  #   res <- fortify(model, data, which="data") |> 
  #     melt(id.vars = c("idx","position")) |> 
  #     mutate(variable = as.character(variable))
  #   uv <- unique(res$variable)
  #   res$variable_num <-  sapply( 
  #     res$variable, 
  #     function(x) which(x == uv)
  #   )
  #   return(res)
  # }
  
  if (which == "heatmap") {
    res <- fortify(model, data, which="data") |> 
      tidyr::gather(variable, value, -idx, -position) |>
      dplyr::mutate(variable = as.character(variable))
    uv <- unique(res$variable)
    res$variable_num <-  sapply( 
      res$variable, 
      function(x) which(x == uv)
    )
    return(res)
  }
  if (which == "data") {  
    if (missing(data)) { 
      stop('missing data')
    } else {
      return( 
        data |> 
          dplyr::mutate(idx = 1:nrow(data),
                 position = order(ord))
      )
    }
  }
  
}




#' @rdname clustering
#' @param object an object of class `"hclust"`
#' @param colorize whether to show clusters in different colors
#' @param k number of clusters
#' @param labels a logical indicating whether labels should be used
#' to identify leaves of the tree.
#' @param heatmap the ratio of size of heatmap to size of dendrogram.  
#'   Use `0` or `FALSE` to omit the heatmap.
#' @param enumerate a color used for numbers within heatmap.  Use 
#'   `"transparent"` to hide.
#' @examples
#' KidsFeet |> select(-name, -birthmonth) |> rescale() -> KidsFeet2
#'   M <- dist(KidsFeet2)
#'   Cl <- hclust(M)
#'   fortify(Cl, k=5) |> head(3)
#'   fortify(Cl, which="heatmap", data=KidsFeet2) |> head(3)
#'   fortify(Cl, which="data", data=KidsFeet2) |> head(3)
#'   fortify(Cl, which="labels") |> head(3)
#'   mplot(Cl, data=KidsFeet2, k=4, heatmap=2)
#'   mplot(Cl, data=KidsFeet2, k=4, heatmap=0.5, enumerate="transparent")
#'   mplot(Cl, data=KidsFeet2, k=4, heatmap=2, type="triangle")
#'   mplot(Cl, data=KidsFeet2, k=4, heatmap=0, type="triangle")
#' @export

mplot.hclust <- function(object, data, colorize = TRUE, k=1, 
                         labels = FALSE,
                         heatmap = 0, 
                         enumerate="white", ...) {
  ggenv <- parent.frame()
  ggenv$h=5
  p <- ggplot( environment = ggenv ) 
  if (colorize && k > 1) { 
    p <- p + 
      geom_segment(data=fortify(object, which="segments", k=k, ...),
                   aes(x=x, y=y, xend=xend, yend=yend, colour=factor(group))) +
      guides(colour=guide_legend(title="group"))
      
  } else { 
    p <- p + geom_segment(data=fortify(object, which="segments", ...),
                          aes(x=x, y=y, xend=xend, yend=yend ))
  }
  
  if (labels) {
    lobject <- fortify(object, which="leaves")
    p <- p + 
      scale_x_continuous(breaks = lobject$x,
                         labels = lobject$label) 
  }
  
  if (heatmap) {
    HeatMapData <- fortify(object, data, which="heatmap") |>
      mutate(h = rescale(variable_num, heatmap * max(object$height) * c(-1/12, -1)))
    TicksData <-  
      HeatMapData |>  
      group_by(variable) |> 
      summarise(pos=unique(variable_num), h=unique(h)) 
    p <- p + 
      geom_tile(data=HeatMapData,
                aes(x=position, 
                    y= h,
                    fill=value), 
                colour="white") +
      geom_text(data=HeatMapData,
                aes(x=position, y= h, label=idx), 
                colour=enumerate, size=3, angle=90) +
      scale_y_continuous(
        breaks = TicksData$h,
        labels = TicksData$variable
      )
#      geom_text(data = HeatMapData |> 
#                  group_by(variable) |> 
#                  summarise(pos=unique(variable_num), h=min(h)), 
#                aes(x = 0, y = h, label=variable),
#                hjust=1)  
  }
  p <- p + theme_minimal() + 
           labs(x="", y="") + 
           theme(axis.ticks.y=element_blank())
  if (labels) {
    p <- p + theme(axis.text.x = element_text(angle=90, hjust=1), 
        axis.ticks.x=element_blank())
  }
  p
}

utils::globalVariables(c('.'))
require(rlang)
#' @importFrom mosaicCore mosaic_formula_q mosaic_formula

# evaluate a lazy object and return the unevaluated expression if the expression
# doesn't evaluate to an existing object.

safe_eval <- function(x) {
  tryCatch(rlang::eval_tidy(x), 
           error = function(e) x$expr)
}


.fetchFromDots <- function( dots, name, class = 'data.frame', n = 1, default = NULL ) {
  result <- dots[[name]]
  if (is.null(result)) {
    if (length(result) < n) return(default)
    result <- dots[[n]]
    if (! inherits(result, 'class') ) result <- default
  }
  return(result)
}

#' Check if formula
#' 
#' @param x an object
#' @return TRUE for a formula, FALSE otherwise, even if evaluation throws an error
#'
#' @rdname mosaic-internal
#' @keywords internal

.is.formula <- function(x)  
  tryCatch( inherits(x, 'formula'), error = function(e) {FALSE} )

#' Check for simple formula
#'
#' @param x a formula
#'
#' @return TRUE if formula has no left-hand side or a simple right-hand side 
#' (e.g., `NULL`, ., 1,  or 0)
#'
#' @rdname mosaic-internal
#' @keywords internal
.is.simple.formula <-  function(x){
  inherits(x, "formula") &&
    (length(x) == 2 || is.null(x[[3]]) ||
       (length(x[[3]]) == 1 &&
          ((is.numeric(x[[3]]) && (x[[3]] == 0 || x[[3]] == 1)) ||  
             (all.names(x[[3]]) %in% c(".")))))
}

# This could use a better name and a better description



#' Extract simple part from formula
#'
#' @param x a formula
#'
#' @return simple part of formula or NULL if formula is not simple
#'
#' @rdname mosaic-internal
#' @keywords internal

.simple.part <- function(x) {
  if (! .is.simple.formula(x) ) {
    return(NULL) 
  } else {
    return(x[[2]])
  }
}

.flatten <- function(x) {
  result <- c()
  for (item in x) result <- c(result, item)
  return(result)
}


#' Aggregate for mosaic
#'
#' Compute function on subsets of a variable in a data frame.
#'
#' @rdname aggregatingAux
#' @return  a vector
#' @param formula a formula.  Left side provides variable to be summarized.  Right side and condition
#'                            describe subsets.  If the left side is empty, right side and condition are
#'                            shifted over as a convenience.
#' @param data a data frame.  
#' Note that the default is `data = parent.frame()`.  This makes it convenient to
#' use this function interactively by treating the working environment as if it were 
#' a data frame.  But this may not be appropriate for programming uses.  
#' When programming, it is best to use an explicit `data` argument
#' -- ideally supplying a data frame that contains the variables mentioned
#' in `formula`.
#' @param FUN a function to apply to each subset 
#' @param groups grouping variable that will be folded into the formula (if there is room for it).  
#' This offers some additional flexibility in how formulas can be specified.
#' @param subset a logical indicating a subset of `data` to be processed.
#' @param drop a logical indicating whether unused levels should be dropped.
#' @param \dots additional arguments passed to `FUN`
#' @param .format format used for aggregation. `"default"` and `"flat"` are equivalent.  
# #' @param format format used for aggregation. \code{"default"} and \code{"flat"} are equivalent.  
# #'   Ignored  if \code{.format} is not \code{NULL}.
#' @param .overall currently unused
#' @param .name a name used for the resulting object
# #' @param name a name used for the resulting object.  Ignored if \code{.format} is not \code{NULL}.
#' @param .envir an environment in which to evaluate expressions 
# #' @param envir an environment in which to evaluate expressions. 
# #'   Ignored if \code{.envir} is not \code{NULL}.
#' @param .multiple a logical indicating whether FUN returns multiple values
# #' @param multiple a logical indicating whether FUN returns multiple values
#'   Ignored if `.multiple` is not `NULL`.
#'
#' @examples
#' if (require(mosaicData)) {
#' maggregate( cesd ~ sex, HELPrct, FUN = mean )
#' # using groups instead
#' maggregate( ~ cesd, groups = sex, HELPrct, FUN = sd )
#' # the next four all do the same thing
#' maggregate( cesd ~ sex + homeless, HELPrct, FUN = mean )
#' maggregate( cesd ~ sex | homeless, HELPrct, FUN = sd )
#' maggregate( ~ cesd | sex , groups= homeless, HELPrct, FUN = sd )
#' maggregate( cesd ~ sex, groups = homeless, HELPrct, FUN = sd )
#' # this is unusual, but also works.
#' maggregate( cesd ~ NULL , groups = sex, HELPrct, FUN = sd )
#' }
#'
#' @export
maggregate <- 
  function(
    formula, 
    data = parent.frame(), 
    FUN, 
    groups = NULL, 
    subset, 
    drop = FALSE, 
    ...,
    .format = c('default', 'table', 'flat'), 
    .overall = mosaic.par.get("aggregate.overall"), 
    # .multiple = NULL,
    .multiple = FALSE, 
    # .name = NULL,
    .name = deparse(substitute(FUN)), 
    # .envir = NULL,
    .envir = parent.frame () 
    ) {
 
  # if (inherits(formula, c("environment", "data.frame")) && 
  #     inherits(data, "formula")) {
  #   temp <- formula
  #   formula <- data
  #   data <- temp
  # }
  #   
  if (! inherits(data, c("environment", "data.frame")) ) {
    if (inherits(data, c("tbl")))
      stop ("Your tbl is not a data.frame.  Perhaps you need dplyr functions here.",
            call. = FALSE)
    else 
      stop("data must be an environment or data.frame.", call. = FALSE)
  }
  formula <- 
    mosaicCore::mosaic_formula_q(formula, groups = !!rlang::enexpr(groups), envir = .envir)
  
  if (length(formula) == 2) { 
    return(FUN( eval(formula[[2]], data, .envir), ...))
  }
  
  dots <- list(...)
  groupName <- ".group"  # gets changed to something better later when possible.
  
  .format <- match.arg(.format, c('default', 'table', 'flat'))
  evalF <- mosaicCore::evalFormula(formula, data = data)
  
  if (!missing(subset)) {
    subset <- eval(substitute(subset), data, environment(formula))
    if (!is.null(evalF$left))           evalF$left <- evalF$left[subset, ]
    if (!is.null(evalF$right))         evalF$right <- evalF$right[subset, ]
    if (!is.null(evalF$condition)) evalF$condition <- evalF$condition[subset, ]
  }
  # this should now be standardized by the call to mosaic_formula_q() above.
  #  if ( is.null( evalF$left ) ) {
  #    evalF$left <- evalF$right
  #    evalF$right <- evalF$condition
  #    evalF$condition <- NULL
  #  }
  
  if ( is.null(evalF$left) || ncol(evalF$left) < 1 )  {
    if (ncol(evalF$right) > 1) warning("Too many variables in rhs; ignoring all but first.")
    if (.format == "table") {
      if (.multiple) stop ("table view unavailable for this function.")
      ldata <- evalF$right[ , 1, drop = FALSE]
      gdata <- group_by(data)
      res <- as.data.frame(
        dplyr::do(gdata, foo = FUN( as.data.frame(.)[, 1], ...) ) )
      names(res)[ncol(res)] <- gsub(".*::", "", .name)
      return(res)
      
      return(evalF$right[ , 1, drop = FALSE] |> 
               group_by() |>
               dplyr::do( do.call(FUN, list(evalF$right[, 1], ...)) ) |>
               as.data.frame()
      )
      #      return(plyr::ddply(evalF$right[ , 1, drop = FALSE], names(NULL),
      #                   function(x) do.call(FUN, list(evalF$right[, 1], ...)) 
      #      )[, -1])  # remove the .id column since it is uninteresting here.
    }
    return( do.call(FUN, alist(evalF$right[, 1], ...) ) )
  } else {
    if (ncol(evalF$left) > 1) warning("Too many variables in lhs; ignoring all but first.")
    if (.format == 'table') {
      if (.multiple) stop ("table view unavailable for this function.")
      ldata <- mosaicCore::joinFrames(evalF$left[ , 1, drop = FALSE], evalF$right, evalF$condition) 
      ldata$.var <- ldata[, 1]
      gdata <- do.call( group_by, c(list(ldata),  
                                    lapply(union(names(evalF$right), names(evalF$condition)),
                                           as.name )) )
      res <- as.data.frame(
        dplyr::do(gdata, foo = FUN( as.data.frame(.)[, 1], ...) ) )
      names(res)[ncol(res)] <- gsub(".*::", "", .name)
      #      res <-  plyr::ddply( 
      #        mosaicCore::joinFrames(evalF$left[ , 1, drop = FALSE], evalF$right, evalF$condition), 
      #        union(names(evalF$right), names(evalF$condition)),
      #        function(x) do.call(FUN, list(x[, 1], ...))
      #      )
      
    } else {
      res <- lapply( split( evalF$left[, 1], 
                            mosaicCore::joinFrames(evalF$right, evalF$condition), 
                            drop = drop),
                     function(x) { do.call(FUN, alist(x, ...) ) }
      )
      groupName <- paste(c(names(evalF$right), names(evalF$condition)), collapse = ".")
      
      if (! .multiple ) res <- unlist(res)
      
      if (! is.null(evalF$condition) ) {
        if (ncol(evalF$left) > 1) message("Too many variables in lhs; ignoring all but first.")
        res2 <- lapply( split( evalF$left[, 1], evalF$condition, drop = drop),
                        function(x) { do.call(FUN, alist(x, ...) ) }
        )
        groupName <- paste(names(evalF$condition), collapse = ".")
        if (!.multiple) {
          res <- c( res, unlist(res2) )
        } else {
          res <- c(res, res2)
        }
      }
      if (.multiple) {
        result <- res
        res <- result[[1]]
        for (item in result[-1]) {
          res <- as.data.frame(rbind(res, item))
        }
        if ( nrow(res) == length(names(result)) ) {
          res[groupName] <- names(result)
        } else {
          res[groupName] <- rep(names(result), each = nrow(res) / length(names(result)) )
        }
        res <- res[, c(ncol(res), 1:(ncol(res) -1))]
      }
    }
  }
  w <- grep("V[[:digit:]]+",  names(res))
  if (length(w) == 1) {
    names(res)[w] <- gsub(".*:{2,3}", "", .name)
  } else {
    names(res)[w] <- paste0( gsub(".*:{2,3}", "", .name), 1:length(w) )
  }
  row.names(res) <- NULL
  return( res )
}

# for handling functions of two inputs
# under construction still

maggregate2 <- function(formula, data = parent.frame(), FUN, subset, 
                        overall = mosaic.par.get("aggregate.overall"), 
                        .format = c('default', 'table', 'flat'), drop = FALSE, 
                        .multiple = FALSE, 
                        groups = NULL, 
                        .name = deparse(substitute(FUN)), 
                        ...) {
  dots <- list(...)
  formula <- 
    mosaicCore::mosaic_formula_q(formula, groups = !!rlang::enexpr(groups), as.environment(data))
  
  .format <- match.arg(.format)
  
  evalF <- mosaicCore::evalFormula(formula, data = data)
  
  if (!missing(subset)) {
    subset <- eval(substitute(subset), data, environment(formula))
    if (!is.null(evalF$left))           evalF$left <- evalF$left[subset, ]
    if (!is.null(evalF$right))         evalF$right <- evalF$right[subset, ]
    if (!is.null(evalF$condition)) evalF$condition <- evalF$condition[subset, ]
  }
  # this should now be standardized by the call to mosaic_formula_q() above.
  #  if ( is.null( evalF$left ) ) {
  #    evalF$left <- evalF$right
  #    evalF$right <- evalF$condition
  #    evalF$condition <- NULL
  #  }
  
  if ( is.null(evalF$left) || ncol(evalF$left) < 1 )  
    stop("formula must have lhs.")
  
  if (ncol(evalF$left) > 1) stop("Too many variables in lhs.")
  
  if (.format == 'table') {
    if (.multiple) stop ("table view unavailable.")
    ldata <- mosaicCore::joinFrames(evalF$left[ , 1, drop = FALSE], evalF$right, evalF$condition) 
    ldata$.var <- ldata[, 1]
    gdata <- do.call( group_by, c(list(ldata),  as.name(names(evalF$condition)) ) )
    res <- as.data.frame( dplyr::do(gdata, foo = FUN( as.data.frame(.)[, 1], as.data.frame(.)[, 2], ...) ) )
    names(res)[ncol(res)] <- gsub(".*::", "", .name)
  } else {
    res <- lapply( split( evalF$left[, 1], 
                          mosaicCore::joinFrames(evalF$right, evalF$condition), 
                          drop = drop),
                   function(x) { do.call(FUN, alist(x, ...) ) }
    )
    groupName <- paste(c(names(evalF$right), names(evalF$condition)), collapse = ".")
    
    if (! .multiple ) res <- unlist(res)
    
    if (! is.null(evalF$condition) ) {
      if (ncol(evalF$left) > 1) message("Too many variables in lhs; ignoring all but first.")
      res2 <- lapply( split( evalF$left[, 1], evalF$condition, drop = drop),
                      function(x) { do.call(FUN, alist(x, ...) ) }
      )
      groupName <- paste(names(evalF$condition), collapse = ".")
      if (!.multiple) {
        res <- c( res , unlist(res2) )
      } else {
        res <- c(res, res2)
      }
    }
    if (.multiple) {
      result <- res
      res <- result[[1]]
      for (item in result[-1]) {
        res <- as.data.frame(rbind(res, item))
      }
      if ( nrow(res) == length(names(result)) ) {
        res[groupName] <- names(result)
      } else {
        res[groupName] <- rep(names(result), each = nrow(res) / length(names(result)) )
      }
      res <- res[, c(ncol(res), 1:(ncol(res) -1))]
    }
  }
  
  w <- grep("V[[:digit:]]+",  names(res))
  if (length(w) == 1) {
    names(res)[w] <- gsub(".*:{2,3}", "", .name)
  } else {
    names(res)[w] <- paste0( gsub(".*:{2,3}", "", .name), 1:length(w) )
  }
  row.names(res) <- NULL
  return( res )
}

utils::globalVariables(c('panel.plotFun1')) 

#' show confidence and prediction bands on plots
#' 
#' @param x,y numeric vectors
#' @param interval a vector subset of `'confidence'` and `'prediction'`
#' @param level confidence level
#' @param model model to be used for generating bands
#' @param band.col a vector of length 1 or 2 giving the color of bands 
#' @param band.lty a vector of length 1 or 2 giving the line type for bands
#' @param band.show logical vector of length 1 or 2 indicating whether 
#' confidence and prediction bands should be shown
#' @param fit.show logical indicating whether the model fit should be shown
#' @param band.alpha a vector of length 1 or 2 alpha level for bands
#' @param band.lwd a vector of length 1 or 2 giving line width for bands
#' @param npts resolution parameter for bands (increase to get better resolution)
#' @param \dots additional arguments
#' 
#' @export

panel.lmbands <-
function (x, y, 
		  interval = "confidence", 
		  level = 0.95, 
		  model = lm(y ~ x), 
		  band.col = c(conf = slcol[3], pred=slcol[2]),
		  band.lty = c(conf = slty[3], pred=slty[2]),
		  band.show = TRUE,
		  fit.show = TRUE,
		  band.alpha = .6,
		  band.lwd = 1,
		  npts = 100,
		  ...) 
{
  slcol <- trellis.par.get("superpose.line")$col
  slty  <- trellis.par.get("superpose.line")$lty
  
  inflate <- function (x, size, default) {
    if (is.null(x)) return( rep(default,length.out=size) )
    x <- x[!is.na(x)]
    if (length(x) == 0) { return(rep(default, length.out=size)) }
    rep(x, length.out=size)
  }

	band.alpha <- inflate(band.alpha, 2, 0.6)
	band.lwd <- inflate(band.lwd, 2, 1)
	band.show <- inflate(band.show, 2, TRUE)
	band.col <- inflate(band.col, 2, "gray50")
  band.lty <- inflate(band.lty, 2, 1)

  fit <- makeFun(model)
	Clower <- function(x) {fit(x, interval="confidence", level=level)[,2]}
	Cupper <- function(x) {fit(x, interval="confidence", level=level)[,3]}
	Plower <- function(x) {fit(x, interval="prediction", level=level)[,2]}
	Pupper <- function(x) {fit(x, interval="prediction", level=level)[,3]}

	dots <- list(...)
  bandDots <- dots
	bandDots[['lty']] <- NULL
	bandDots[['col']] <- NULL
	bandDots[['alpha']] <- NULL
	bandDots[['lwd']] <- NULL

  if (band.show[2]) {
    do.call( panel.plotFun1, 
             c(list(makeFun(Plower(x)~x), 
                    col=band.col[2], 
                    lty=band.lty[2], 
                    npts=npts, 
                    alpha=band.alpha[2], 
                    lwd=band.lwd[2]),
               bandDots) )
    
    do.call( panel.plotFun1, 
             c(list(makeFun(Pupper(x)~x), 
                    col=band.col[2], 
                    lty=band.lty[2], 
                    alpha=band.alpha[2], 
                    lwd=band.lwd[2],
                    npts=npts), 
               bandDots) )
  }

  if (band.show[1]) {
    do.call( panel.plotFun1, 
             c(list(makeFun(Clower(x)~x), 
                    col=band.col[1], 
                    lty=band.lty[1], 
                    alpha=band.alpha[1], 
                    lwd=band.lwd[1],
                    npts=npts), 
               bandDots) )
    
    do.call( panel.plotFun1, 
             c(list(makeFun(Cupper(x)~x), 
                    col=band.col[1], 
                    lty=band.lty[1], 
                    alpha=band.alpha[1], 
                    lwd=band.lwd[1],
                    npts=npts), 
               bandDots) )
  }

	if (fit.show) {
	  do.call( panel.plotFun1, c(list(makeFun(fit(x) ~ x)), dots) )
	}

  do.call("panel.xyplot", c(list(x, y), dots))
}

utils::globalVariables(c(".group", ".color", ".cond", "mypanel"))

#' Plot a regression model
#' 
#' @description Visualize a regression model amid the data that generated it. 
#' 
#' @param mod A model of type [lm()] or [glm()]
#' @param \dots arguments passed to [lattice::xyplot()] or `rgl::plot3d`.
#'
#' @section Caution:
#' This is still underdevelopment.  The API is subject to change, and some
#' use cases may not work yet.  Watch for improvements in subsequent versions of the package.
#' 
#' @details The goal of this function is to assist with visualization
#' of statistical models. Namely, to plot the model on top of the data
#' from which the model was fit. 
#' 
#' The primary plot type is a scatter plot.  The x-axis can be assigned
#' to one of the predictors in the model.  Additional predictors are thought
#' of as co-variates.  The data and fitted curves are partitioned by
#' these covariates.  When the number of components to this partition is large,
#' a random subset of the fitted curves is displayed to avoid visual clutter.
#' 
#' If the model was fit on one quantitative variable (e.g. SLR), then
#' a scatter plot is drawn, and the model is realized as parallel or
#' non-parallel lines, depending on whether interaction terms are present.
#' 
#' Eventually we hope to support 3-d visualizations of models with 2 quantitative
#' predictors using the `rgl` package.
#' 
# If \code{key} indicates two quantitative variables, then
# if the \code{rgl} package is available, the data points are drawn in 3-space,
# and the model is realized as a series of planes.
#' 
#' Currently, only linear regression models and 
#' generalized linear regression models are supported. 
#' 
#' @return A lattice or ggplot2 graphics object.
#' 
# or in the case of a 3D model, nothing
# but an RGL window will pop up.
#' 
#' @seealso [`plotPoints()`], [`plotFun()`]
#' @author Ben Baumer, Galen Long, Randall Pruim
#' @export
#' @examples
#'
#' require(mosaic)
#' 
#' mod <- lm( mpg ~ factor(cyl), data = mtcars)
#' plotModel(mod)
#' 
#' # SLR
#' mod <- lm( mpg ~ wt, data = mtcars)
#' plotModel(mod, pch = 19)
#' 
#' # parallel slopes
#' mod <- lm( mpg ~ wt + factor(cyl), data=mtcars)
#' plotModel(mod)
#' 
#' \dontrun{ 
#' # multiple categorical vars
#' mod <- lm( mpg ~ wt + factor(cyl) + factor(vs) + factor(am), data = mtcars)
#' plotModel(mod)
#' plotModel(mod, mpg ~ am)
#' 
#' # interaction
#' mod <- lm( mpg ~ wt + factor(cyl) + wt:factor(cyl), data = mtcars)
#' plotModel(mod)
#' 
#' # polynomial terms
#' mod <- lm( mpg ~ wt + I(wt^2), data = mtcars)
#' plotModel(mod)
#' 
#' # GLM
#' mod <- glm(vs ~ wt, data=mtcars, family = 'binomial')
#' plotModel(mod)
#' 
#' # GLM with interaction
#' mod <- glm(vs ~ wt + factor(cyl), data=mtcars, family = 'binomial')
#' plotModel(mod)

#' # 3D model
#' mod <- lm( mpg ~ wt + hp, data = mtcars)
#' plotModel(mod)
#' 
#' # parallel planes
#' mod <- lm( mpg ~ wt + hp + factor(cyl) + factor(vs), data = mtcars)
#' plotModel(mod)
#' 
#' # interaction planes
#' mod <- lm( mpg ~ wt + hp + wt * factor(cyl), data = mtcars)
#' plotModel(mod)
#' plotModel(mod, system="g") + facet_wrap( ~ cyl )
#' }

# ------------------------------------------------------------------------------
# TODO
#
# detect and pass through 3-d models
# support for interaction in planes
# key = ~ x | z for faceting one covariate?
# 
# ------------------------------------------------------------------------------

plotModel <- function(mod, ...) { UseMethod("plotModel") }

#' @rdname plotModel
#' @export
plotModel.default <- function(mod, ...) {
  plotModel(parseModel(mod), ...)
}

#' @rdname plotModel
#' @param formula a formula indicating how the variables are to be displayed.  In the style of 
#'   `lattice` and `ggformula`. 
#' @param auto.key If TRUE, automatically generate a key.
#' @param drop If TRUE, unused factor levels are dropped from `interaction()`. 
#' @param max.levels currently unused
#' @param system which of `ggplot2` or `lattice` to use for plotting
#' @export
plotModel.parsedModel <- 
  function(mod, formula = NULL, ..., auto.key = NULL, drop = TRUE, 
           max.levels = 9L, system=c("ggplot2", "lattice")) {
    
    system <- match.arg(system)
  
    x <- mod # fixing variable name mis-match with generic method.
    
    if (length(x$varTypes) < 2L) 
      stop("Only models with explanatory variables can be plotted.")
    
    if (! inherits(formula, "formula")) { 
      key <- x$varTypes[-1][1]
      formula <- y ~ x
      formula[[2]] <- as.name(x$responseName)
      formula[[3]] <- as.name(names(key))
    }
   
    # variable groups
    #  * keyVar = x-axis variable
    #  * coVars = other rhs vars
    #  * condVars = variables in condition
    #  * restVars: other variables in the model
    
    key <- all.vars(rhs(formula))[1]
    coVars <- intersect( setdiff(all.vars(rhs(formula)), key), names(x$varTypes) )
    condVars <- intersect( all.vars(condition(formula)), names(x$varTypes) )
    other_data <- x$data[, -1, drop=FALSE]
    other_data[[key]] <- NULL
    restVars <- setdiff( names(other_data), union(coVars, condVars) )
    discreteVars <- names(x$varTypes[x$varTypes == "discrete"])
    
    nonEmpty <- function(data, varTypes = x$varTypes) {
        if (ncol(data) == 0L) data$blank <- ""
        data
      }
    
    discretePart <- function(data, varTypes = x$varTypes) {
      data[ , intersect(names(data), names(varTypes[varTypes == "discrete"])), drop=FALSE]
    }
    
    my_interaction <- function( data, drop = TRUE, sep = ":", lex.order = TRUE ) {
      if (ncol(data) == 0L) {
        factor(rep("", nrow(data)))
      } else {
        interaction( data, drop = drop, sep = sep, lex.order = TRUE )
      }
    }
      

# Some aux variables:
#   .color: one for each combination of discrete co-vars
#       this determines the color of the lines
#   .group: one for each combination of co-vars (selected levels of contintuous too)
#       this determins which points are connected to make curves
#   .cond:  one for each combination of discrete cond-vars
#       this determines which lines show up in which panels
    
    point_data <- 
      x$data |>
      mutate(
        .color = my_interaction(x$data[, intersect(discreteVars, restVars), 
                                       drop = FALSE]),
        .group = my_interaction(x$data[, c(coVars, restVars), drop = FALSE]),
        .cond = my_interaction(x$data[, condVars, drop = FALSE])
      )
   
    
# create data for lines
    # utility function 
    some_levels <- function(x) {
      if (is.numeric(x)) 
        return( sort(unique(ntiles(x, 3, format = "median"))) )
      return(sort(unique(x)))
    }
   
    # a few values of each explanatory variable
    
    levels_all <- expand.grid(lapply(other_data, some_levels))
    levels_shown <- levels_all
    
    # convert the model into a function
    modelFunc <- makeFun(x$model)
   
    # eplanatory variable
    
    if (x$varTypes[key] == "continuous") {
      p <- xyplot( formula,  data = x$data )
      x_points <- seq(p$x.limits[1], p$x.limits[2], length.out = 40)
    } else {
      x_points <- unique( x$data[[key]] )
    }
    
    if (length(restVars) > 0L && is.null(auto.key)) {
      auto.key <- 
        list(points = TRUE, lines=TRUE, 
             columns = base::min(length(unique(point_data$.color)), 3))
    } else { 
      if (is.null(auto.key)) auto.key <- FALSE
    }
    
    if (nrow(levels_shown) == 0L) {
      levels_shown <- data.frame( matrix(nrow = length(x_points), ncol=0) )
    }
      
    line_data <- bind_rows(lapply(1:length(x_points), function(x) levels_shown)) 
    
    line_data[["x"]] <-
      line_data[[key]] <- 
      rep(x_points, times = nrow(levels_shown))
    
    if (x$factorResponse) {
      line_data[["y"]] <- 
        line_data[[x$responseName]] <- 
        1 + predict(x$model, newdata=line_data, type="response")
    } else {
      line_data[["y"]] <- 
        line_data[[x$responseName]] <- 
        predict(x$model, newdata=line_data, type="response")
    }
    
    line_data <- 
      line_data |>
      mutate(
        .color = my_interaction(line_data[, restVars, drop = FALSE]),
        .group = my_interaction(line_data[, c(coVars, restVars, condVars), drop = FALSE]),
        .cond = my_interaction(line_data[, condVars, drop = FALSE])
      )
    
    
    mypanel <- 
      function(x, y, line_data, point_data, 
               group.number = NULL, group.value = NULL, type = "p",
               ...) {
        panel.xyplot(x, y, type = "p", ...)
        line_data <- 
          line_data |> 
          arrange(.color, .group, x) 
        if (! is.null(group.value) ) {
          line_data <- line_data |> filter(as.numeric(.cond) == packet.number())
        }
        ncolors <- length(unique(line_data$.color))
        ngroups <- length(unique(line_data$.group))
        grid::grid.polyline(
          x = line_data$x, 
          y = line_data$y, 
          default.units = "native",
          id = as.numeric(line_data$.group),
          gp = gpar(
            col = trellis.par.get("superpose.line")$col[1:base::max(as.numeric(line_data$.color))]
          )
        )
      }
    if (system == "ggplot2") {
      if (length(unique(point_data$.color)) < 2L) {
        ggplot() +
          geom_point(aes(y = .data[[x$responseName]], x = .data[[key]]), 
                     size = 1.2,
                     data = point_data |> droplevels()) +
          geom_line (aes(y = .data[[x$responseName]], x = .data[[key]]), 
                     linewidth = 0.5,
                     data = line_data |> droplevels())
      } else {
        ggplot() +
          geom_point(aes(y = .data[[x$responseName]], x = .data[[key]], 
                         colour = .data[[".color"]], group = .data[[".group"]]), 
                     size=1.2, data = point_data |> droplevels()) +
          geom_line (aes(y = .data[[x$responseName]], x = .data[[key]], 
                         colour = .data[[".color"]], group = .data[[".group"]]), 
                     linewidth = 0.5, data = line_data |> droplevels())
      }
    } else {
      xyplot(formula, 
             data = point_data |> droplevels(),
             line_data = line_data |> droplevels(),
             point_data = point_data |> droplevels(),
             groups = .color,
             auto.key = auto.key,
             ...,
             panel = panel.superpose,
             panel.groups = mypanel) 
#       rlang::check_installed('latticeExtra')
#       xyplot( formula, data = point_data,
#               groups = .group,
#               auto.key = TRUE,
#               ...) +
#         latticeExtra::as.layer(
#           xyplot(formula, data = line_data,
#                  type = 'l',
#                  groups = .group,
#                  auto.key = TRUE,
#                  ...
#           ))
#                  
      
    } 
  }

# revise this later.
plotModel_planes <- function(pmod, key, ...) {
  # do something with rgl
  if (!requireNamespace("rgl")) {
    stop("Please install rgl for 3D support")
  }
  xName = names(key)[1] # pmod$numericNames[1] 
  yName = names(key)[2] # pmod$numericNames[2] 
  zName = pmod$responseName 
  # get better default axis labels
  # if (is.null(match.arg("xlab"))) { xlab = "mod$numericNames[1]"; }
  rgl::plot3d(x = pmod$data[, xName], 
              y = pmod$data[, yName], 
              z = pmod$data[, zName], 
              ...)
  
  # Now need to add the planes...

}

# original version of above
plotModel0.mplanes <- function(mod, ...) {
  # do something with rgl
  if (!requireNamespace("rgl")) {
    stop("Please install rgl for 3D support")
  }
  xName = mod$numericNames[1] 
  yName = mod$numericNames[2] 
  zName = mod$responseName 
  # get better default axis labels
  # if (is.null(match.arg("xlab"))) { xlab = "mod$numericNames[1]"; }
  rgl::plot3d(x = mod$model[, xName], 
         y = mod$model[, yName], 
         z = mod$model[, zName], 
         ...)
  
  # all combinations of categorical levels
  categories = expand.grid(mod$xlevels) 
  
  coefs <- coef(mod)
  levels <- coefs[!names(coefs) %in% c(xName, yName)]
  
  # sort of a hack -- ideally there is a better solution
  varNames <- gsub("factor|\\(|\\)", "", attr(terms(mod), "term.labels"))
  # drop all interactions, since the linear terms should be there
  varNames <- varNames[!grepl(":", varNames)]
  
  categories <- cbind(x = rep(0, nrow(categories)), y = rep(0, nrow(categories)), categories)
  names(categories) <- varNames
  intercepts <- predict(mod, newdata = categories)
  
  if (nrow(categories) == 0) {
    rgl::planes3d(coefs[xName], coefs[yName], -1, coefs["(Intercept)"], alpha = 0.5, col = "lightgray")
  } else {
    for (intercept in intercepts) {
      rgl::planes3d(coefs[xName], coefs[yName], -1, intercept, alpha = 0.5, col = "lightgray")
    }
  }
}

parseModel <- function(x) {
  res <- list(model = x)
  res$data <- eval(x$call$data, environment(x$call$formula))[all.vars(x$call$formula)]
  res$fit <- makeFun(x)
  inputs <- head(formals(res$fit), -2)  # remove ... and transform
  
  discreteOrContinuous <- function(n, data, model) {
    if (is.factor(data[[n]])    || 
        is.character(data[[n]]) ||
        is.logical(data[[n]])   ||
        paste0("factor(", n, ")" ) %in% names(attr(terms(model), "dataClasses"))
    )
      "discrete" else "continuous"
  }
  
  # determine number of each variable type (categorical or quantitative)
  res$varTypes <- sapply(names(res$data), function(n) discreteOrContinuous(n, res$data, x)) 
  names(res$varTypes) <- names(res$data)
  
  # varTypes <- attr(terms(x), "dataClasses")[modelVars(x)]
  res$responseName <- as.character(lhs(terms(x)))
  res$numericNames <- names(which(x$varTypes == "continuous")) # , x$responseName)
  res$catNames <- names(which(x$varTypes == "discrete")) # , x$responseName)
  res$factorResponse <- is.factor(eval(lhs(terms(x)), res$data))
  class(res) <- c("parsedModel")
  return(res)
}

# context("plotModel")

testthat::test_that("plotModel works", {

  mod <- lm( mpg ~ factor(cyl), data = mtcars)
  mod2 <- lm( mpg ~ wt, data = mtcars)
  mod3 <- lm( mpg ~ wt + factor(cyl), data=mtcars)
  # Default
  wrapped_expect_doppelganger("plotModel1", plotModel(mod))
  # SLR
  wrapped_expect_doppelganger("plotModel2", plotModel(mod2, pch = 19))
  # parallel slopes
  wrapped_expect_doppelganger("plotModel3", plotModel(mod3))
})

#' Conditionally convert vectors to factors
#' 
#' A generic function and several instances for creating factors from
#' other sorts of data.  The primary use case is for vectors that contain
#' few unique values and might be better considered as factors.  When
#' applied to a data frame, this is applied to each variable in the data frame.
#' 
#' @param x an object
#' @param max.levels an integer.  Only convert if the number of unique values is no 
#' more than `max.levels`.
#' @param ... additional arguments (currently ignored)
#' 
#' @export
#' @examples
#' data(KidsFeet, package="mosaicData")
#' str(KidsFeet)
#' factorize(KidsFeet$birthyear)
#' str(factorize(KidsFeet))
#' # alternative spelling
#' str(factorise(KidsFeet))
 
factorize <- function(x,  ...) {
  UseMethod("factorize")
}

#' @rdname factorize
#' @export
factorize.default <- function(x, ...) {
  x
}

#' @rdname factorize
#' @export
factorize.numeric <- function(x, max.levels = 5L, ...){
  if (length(unique(x)) <=  max.levels) return ( factor(x, levels=sort(unique(x))) )  
  x
}

#' @rdname factorize
#' @export
factorize.character <- function(x, max.levels = 5L, ...){
  if (length(unique(x)) <=  max.levels) return ( factor(x, levels=sort(unique(x))) )  
  x
}

#' @rdname factorize
#' @export
factorize.data.frame <- function(x, max.levels=5L, ...) {
  as.data.frame( lapply(x, factorize, max.levels=max.levels) )
}

# an alias with alternative spelling
#' @rdname factorize
#' @export
factorise <- factorize

# context("factorize")

testthat::test_that("factorize works", {
  data(KidsFeet, package="mosaicData")
  testcase <- structure(c(2L, 1L, 1L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 1L, 2L, 1L, 
                          2L, 1L, 2L, 1L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 1L, 2L, 
                          2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L), 
                         .Label = c("87", "88"), 
                         class = "factor")
  
  expect_equal(ignore_attr = TRUE, testcase, factorize(KidsFeet$birthyear))
  # alternative spelling
  expect_equal(ignore_attr = TRUE, testcase, factorise(KidsFeet$birthyear))
})

#' Defunct functions now in the fetch package
#' 
#' These functions have been moved to the fetch package.
#'
#' @param ... arguments 
#' @rdname defunct-fetch
#' 
#' @export

fetchData <- function(...) {
  .Defunct(msg="Use fetchData() from the `fetch' package instead.")
}

#' Check for whole number values
#'
#' Unlike [is.integer()], which checks the type of argument is `integer`, 
#' this function checks whether the value of the argument is an integer
#' (within a specified tolerance).
#'
#' @param x a vector
#' @param tol a numeric tolerance
#' @return a logical vector indicating whether `x` has a whole number value
#' @details
#' This function is borrowed from the examples for [is.integer()]
#'
#' @examples
#' is.wholenumber(1)
#' all(is.wholenumber(rbinom(100,10,.5)))
#' is.wholenumber((1:10)/2)
#' @export

is.wholenumber <-
  function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol

# context("is.wholenumber")

testthat::test_that("is.wholenumber works", {
  expect_true(is.wholenumber(1))
  expect_true(all(is.wholenumber(rbinom(100,10,.5))))
  
  testcase <- c(FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, FALSE, 
                TRUE)
  expect_equal(testcase, is.wholenumber((1:10)/2))
})

#' Factor cross products
#' 
#' Construct a product of factors.
#'
#' @param \dots  factors to be crossed.
#' @param sep  separator between levels
#' @param drop.unused.levels should levels that do not appear in cross product be dropped?
#' 
#' @return a factor
#' 
#' @examples
#' x <- letters[1:3]
#' y <- c(1,2,1,1,3,1,3)
#' cross(x, y)
#' cross(x, y, drop.unused.levels=TRUE)
#' 
#' @keywords manipulate 
#' @export

cross <- function(..., sep=":", drop.unused.levels=FALSE) {
	factors <- list(...)
	factors <- lapply( factors, function(x) { as.factor(x) } )
	if ( length(factors) < 1 ) {
		stop('No factors specified.')
	}
	levelsList <- lapply(factors, levels)

	result <- factors[[1]]
	levels <- levels(result)
	factors[[1]] <- NULL
	while( length(factors) > 0 ) {
		levels <- as.vector( 
					outer (levels(factors[[1]]), levels, function(x,y) { paste(y,x,sep=sep) } ) 
					)
		if (drop.unused.levels ) {
			result <- factor( paste( result, factors[[1]], sep=sep))
		} else {
			result <- factor( paste( result, factors[[1]], sep=sep), levels=levels)
		}
		factors[[1]] <- NULL
	}
	return(result)
}

# context("cross")

testthat::test_that("Cross works", {
  x <- letters[1:3]
  y <- c(1,2,1,1,3,1,3)
  
  testcase1 <- structure(c(1L, 5L, 7L, 1L, 6L, 7L, 3L), 
                         .Label = c("a:1", "a:2","a:3", "b:1", 
                                    "b:2", "b:3", "c:1", "c:2", 
                                    "c:3"), 
                         class = "factor")
  
  testcase2 <- structure(c(1L, 3L, 5L, 1L, 4L, 5L, 2L), 
                         .Label = c("a:1", "a:3", "b:2", "b:3", "c:1"), 
                         class = "factor")
  
  expect_equal(ignore_attr = TRUE, testcase1, cross(x, y))
  expect_equal(ignore_attr = TRUE, testcase2, cross(x, y, drop.unused.levels=TRUE))
})

utils::globalVariables(c('.row'))
#' @importFrom mosaicCore vector2df nice_names
require(parallel)
require(profvis)
require(compiler)
require(rlang)
parallel::detectCores()
NA


#' Set seed in parallel compatible way
#'
#' When the parallel package is used, setting the RNG seed for reproducibility
#' involves more than simply calling [set.seed()]. `set.rseed` takes
#' care of the additional overhead.
#'
#' @param seed seed for the random number generator
#' @details
#' If the `parallel` package is not on the search path, then [set.seed()] is called.
#' If `parallel` is on the search path, then the RNG kind is set to `"L'Ecuyer-CMRG"`,
#' the seed is set and `mc.reset.stream` is called.
#'
#' @examples
#' # These should give identical results, even if the `parallel' package is loaded.
#' set.rseed(123); do(3) * resample(1:10, 2)
#' set.rseed(123); do(3) * resample(1:10, 2)
#' @export

set.rseed <- function(seed) {
  if ("package:parallel" %in% search()) {
    set.seed(seed, kind = "L'Ecuyer-CMRG")
    parallel::mc.reset.stream()
  } else {
    set.seed(seed)
  }
}


#' Do Things Repeatedly
#'
#' `do()` provides a natural syntax for repetition tuned to assist
#' with replication and resampling methods.
#'
#' @rdname do
#' @param n  number of times to repeat
#'
#' @param object an object
#'
#' @param cull  function for culling output of objects being repeated.  If NULL,
#'   a default culling function is used.  The default culling function is
#'   currently aware of objects of types
#'   `lme`,
#'   `lm`,
#'   `htest`,
#'   `table`,
#'   `cointoss`, and
#'   `matrix`.
#'
#' @param mode  target mode for value returned
#'
#' @param algorithm a number used to select the algorithm used.  Currently numbers below 1
#'   use an older algorithm and numbers >=1 use a newer algorithm which is faster in some
#'   situations.
#' @param parallel a logical indicating whether parallel computation should be attempted
#'   using the \pkg{parallel} package (if it is installed and loaded).
#'
#' @param e1 an object (in cases documented here, the result of running `do`)
#' @param e2 an object (in cases documented here, an expression to be repeated)
#' @param ... additional arguments
#'
#' @note `do` is a thin wrapper around `Do` to avoid collision with
#'   [dplyr::do()] from the \pkg{dplyr} package.
#' @return `do` returns an object of class `repeater` which is only useful in
#'   the context of the operator `*`.  See the examples.
#' @author Daniel Kaplan (\email{kaplan@@macalaster.edu})
#'   and Randall Pruim (\email{rpruim@@calvin.edu})
#'
#' @section Naming:
#' The names used in the object returned from `do()` are inferred from the
#' objects created in each replication.  Roughly, this the strategy employed.
#'
#' * If the objects have names, those names are inherited, if possible.
#' * If the objects do not have names, but `do()` is used with a simple
#' function call, the name of that function is used.
#' Example: `do(3) * mean(~height, data = Galton)` produces a data frame with
#' a variable named `mean`.
#' * In cases where names are not easily inferred and a single result is produced,
#' it is named `result`.
#'
#' To get different names, one can rename the objects as they are created, or
#' rename the result returned from `do()`.  Example of the former:
#' `do(3) * c(mean_height = mean(~height, data = resample(Galton)))`.
#' If you like the name `result`, try wrapping the expression in `{ }`:
#' `do(3) * { mean(~ height, data = resample(Galton)) }`.
#'
#' @seealso [replicate()], [set.rseed()]
#'
#' @examples
#' do(3) * rnorm(1)
#' do(3) * "hello"
#' do(3) * 1:4
#' do(3) * mean(rnorm(25))
#' do(3) * lm(shuffle(height) ~ sex + mother, Galton)
#' do(3) * anova(lm(shuffle(height) ~ sex + mother, Galton))
#' do(3) * c(sample.mean = mean(rnorm(25)))
#' # change the names on the fly
#' do(3) * mean(~height, data = resample(Galton))
#' do(3) * c(mean_height = mean(~height, data = resample(Galton)))
#' # this results in the name being `result`
#' do(3) * { mean(~height, data = resample(Galton)) }
#' set.rseed(1234)
#' do(3) * tally( ~sex|treat, data=resample(HELPrct))
#' set.rseed(1234)  # re-using seed gives same results again
#' do(3) * tally( ~sex|treat, data=resample(HELPrct))
#' @keywords iteration
#' @export

do <- function(object, ...) {
  UseMethod("do")
}

#' @rdname do
#' @export
do.numeric <- function(object, ...) {
  Do(n=object, ...)
}

#' @rdname do
#' @export
do.default <- function(object, ...) {
  dplyr::do(object, ...)
}

#' @rdname do
#' @export
Do <- function(n=1L, cull=NULL, mode='default', algorithm=1.0, parallel=TRUE) {
	new( 'repeater', n=n, cull=cull, mode=mode, algorithm=algorithm, parallel=parallel)
}

#' @rdname mosaic-internal
#' @keywords internal
#' @details
#' `.make.data.frame` converts things to a data frame
#' @param x object to be converted
#' @return a data frame

.make.data.frame <- function( x ) {
	if (is.data.frame(x)) return(x)
	if (is.vector(x)) {
		nn <- names(x)
		result <- as.data.frame( matrix(x, nrow=1) )
		if (! is.null(nn) ) names(result) <- nn
		return(result)
	}
	return(as.data.frame(x))
	}



null2na <- function(x) if (is.null(x)) NA else x

#' Repeater objects
#'
#' Repeater objects can be used with the `*` operator to repeat
#' things multiple time using a different syntax and different output
#' format from that used by, for example, [replicate()].
#'
#' @rdname repeater-class
#' @name repeater-class
#' @seealso [do()]
#' @section Slots:
#' \describe{
#'   \item{`n`:}{Object of class `"numeric"` indicating how many times to repeat something.}
#'   \item{`cull`:}{Object of class `"function"` that culls the output from each repetition.}
#'   \item{`mode`:}{Object of class `"character"` indicating the output mode
#'   ('default', 'data.frame', 'matrix', 'vector', or 'list').  For most purposes 'default' (the default)
#'   should suffice.}
#'   \item{`algorithm`:}{an algorithm number.}
#'   \item{`parallel`:}{a logical indicating whether to attempt parallel execution.}
#' }
#' @exportClass repeater

setClass('repeater',
	representation = representation(n='numeric', cull='ANY', mode='character',
                                  algorithm='numeric', parallel='logical'),
	prototype = prototype(n=1, cull=NULL, mode="default", algorithm=1, parallel=TRUE)
)


# old version
if(FALSE) {
.merge_data_frames <- function(a, b) {
  a <- .make.data.frame(a)
  b <- .make.data.frame(b)
  if (nrow(b) < 1) return (a)
  if (nrow(a) < 1) return (b)

	a$mosaic_merge_id <- paste('A',1:nrow(a))
	b$mosaic_merge_id <- paste('B',1:nrow(b))
	result <- merge(a,b,all=TRUE)
	w <- which(names(result) == 'mosaic_merge_id')
	result <- result[, -w]
	return(result)
}
}


#' @rdname mosaic-internal
#' @keywords internal
#' @details `.merge_data_frames` is a wrapper around merge
#'
#' @param a a data frame
#' @param b a data frame
#'
#' @return a data frame

.merge_data_frames = function(a,b) {
  a <- .make.data.frame(a)
  b <- .make.data.frame(b)
  if (nrow(b) < 1) return (a)
  if (nrow(a) < 1) return (b)
  missing.from.b = setdiff(names(a),names(b))
  missing.from.a = setdiff(names(b),names(a))
  for (var in missing.from.b) b[[var]] = NA
  for (var in missing.from.a) a[[var]] = NA
  dplyr::bind_rows(a,b)
}


#' @rdname mosaic-internal
#' @keywords internal
#' @details
#' `.squash_names` squashes names of a data frame into a single string
#'
#' @param object an object
#' @param sep a character
#'
#' @return a character vector

.squash_names <- function(object,sep=":") {
	if ( ncol(object) < 1 ) {return(rep("",nrow(object)))}

	result <- object[,1]
	if ( ncol(object) < 2 ) {return(as.character(result))}

	for (c in 2:ncol(object)) {
		result <- paste(result, as.character(object[,c]), sep=sep)
	}
	return(result)

}


#' @rdname do
#' @param x an object created by `do`.
#' @export
print.repeater <- function(x, ...)
    {
  		message(paste('This repeats a command',x@n,'times. Use with *.'))
  		return(invisible(x))
    }

.list2tidy.data.frame <- function (l) {

  # see if we really just have a vector
  ul <- unlist( l )
  if ( length(ul) == length(l) ) {
    result <- data.frame(..result.. = as.vector(ul))
    row.names(result) <- NULL
    if( !is.null(names(l[[1]])) ) names(result) <- names(l[[1]])
    return(result)
  }

  # if each element is a data frame, combine them with bind_rows
  if ( all( sapply( l, is.data.frame ) ) ) {
    return(
      lapply(l, function(x) {mutate(x, .row= 1:n())}) |>
        dplyr::bind_rows() |>
        mutate(.index = c(1, 1 + cumsum( diff(.row) != 1 )))
    )
  }

  # If rbind() works, do it
  tryCatch(
    return ( as.data.frame( do.call( rbind, l) ) ),
    error=function(e) {}
  )

  if (all (sapply(l, length) ) == length(l[[1]]) ) {
    result <-  as.data.frame( matrix( ul, nrow=length(l) ) )
    names(result) <- names(l[[1]])
    return(result)
  }

  # nothing worked.  Just return the list as is.
  return( l )
}

#' Cull objects used with do()
#'
#' The [do()] function facilitates easy replication for
#' randomization tests and bootstrapping (among other things).  Part of what
#' makes this particularly useful is the ability to cull from the objects
#' produced those elements that are useful for subsequent analysis.
#' `cull_for_do` does this culling.  It is generic, and users
#' can add new methods to either change behavior or to handle additional
#' classes of objects.
#'
#' @param object an object to be culled
#' @param ... additional arguments (currently ignored)
#'
#' @details When `do(n) * expression` is evaluated, `expression`
#' is evaluated `n` times to produce a list of `n` result objects.
#' `cull_for_do` is then applied to each element of this list to
#' extract from it the information that should be stored.  For example,
#' when applied to a object of class `"lm"`,
#' the default `cull_for_do` extracts the coefficients, coefficient
#' of determinism, an the estimate for the variance, etc.
#'
#' @export
#' @examples
#' cull_for_do(lm(length ~ width, data = KidsFeet))
#' do(1) * lm(length ~ width, data = KidsFeet)


cull_for_do <- function(object, ...) {
  UseMethod("cull_for_do")
}


#' @export
cull_for_do.default <- function(object, ...) {
  object
}

#' @export
cull_for_do.fitdistr <- function(object, ...) {
  est <- object$estimate
  names(est) <- paste0(names(est), ".est")
  se <- object$sd
  names(se) <- paste0(names(se), ".se")
  c(est, se)
}

#' @export
cull_for_do.aov <- function(object, ...) {
  cull_for_do(anova(object))
}

#' @export
cull_for_do.anova <- function(object, ...) {
    res <- as.data.frame(object)
    res <- cbind (data.frame(source=row.names(res)), res)
    names(res)[names(res) == "Df"] <- "df"
    names(res)[names(res) == "Sum Sq"] <- "SS"
    names(res)[names(res) == "Mean Sq"] <- "MS"
    names(res)[names(res) == "F value"] <- "F"
    names(res)[names(res) == "Pr(>F)"] <- "pval"
    names(res)[names(res) == "Sum of Sq"] <- "diff.SS"
    names(res)[names(res) == "Res.Df"] <- "res.df"
    return(res)
    return( data.frame(
      SSTotal= sum(object$`Sum Sq`),
      SSModel= object$`Sum Sq`[1],
      SSError= object$`Sum Sq`[2],
      MSTotal= sum(object$`Sum Sq`),
      MSModel= object$`Mean Sq`[1],
      MSError= object$`Mean Sq`[2],
      F=object$`F value`[1],
      dfModel=object$Df[1],
      dfError=object$Df[2],
      dfTotal=sum(object$Df)
    ) )
}

#' @export
cull_for_do.table <- function(object, ...) {
  result <- data.frame(object)
  res <- result[[ncol(result)]]
  nms <- as.character(result[[1]])
  if (ncol(result) > 2) {
    for (k in 2:(ncol(result)-1)) {
      nms <- paste(nms, result[[k]],sep=".")
    }
  }
  names(res) <- nms
  return(res)
}

#' @export
cull_for_do.aggregated.stat <- function(object, ...) {
  result <- object
  res <- as.vector(result[, "S"])  # ncol(result)]
  names(res) <-
    paste( attr(object, 'stat.name'),
           .squash_names(object[,1:(ncol(object)-3),drop=FALSE]), sep=".")
  return(res)
}

#' @export
cull_for_do.lme <- function(object, ...) {
  result <- object
  names(result) <- mosaicCore::nice_names(names(result))
  return( object$coef$fixed )
}

#' @export
cull_for_do.lm <- function(object, ...) {
  sobject <- summary(object)
  Fstat <- sobject$fstatistic[1]
  DFE <- sobject$fstatistic["dendf"]
  DFM <- sobject$fstatistic["numdf"]
  if (!is.null(Fstat)) {
    names(Fstat) <- "F"
    result <-  c(coef(object), sigma=sobject$sigma,
                 r.squared = sobject$r.squared,
                 Fstat,
                 DFM,
                 DFE)
  } else {
    result <-  c(coef(object), sigma=sobject$sigma,
                 r.squared = sobject$r.squared
    )
  }
  mosaicCore::vector2df(result, nice_names = TRUE)
}

#  @export
# cull_for_do.groupwiseModel <- function(object, ...) {
#   sobject <- summary(object)
#   Fstat <- sobject$fstatistic[1]
#   DFE <- sobject$fstatistic["dendf"]
#   DFM <- sobject$fstatistic["numdf"]
#   if (!is.null(Fstat)) {
#     names(Fstat) <- "F"
#     result <-  c(coef(object), sigma=sobject$sigma,
#                  r.squared = sobject$r.squared,
#                  Fstat,
#                  DFM,
#                  DFE)
#   } else {
#     result <-  c(coef(object), sigma=sobject$sigma,
#                  r.squared = sobject$r.squared
#     )
#   }
#   names(result) <- nice_names(names(result))
#   return(result)
# }
#

#' @export
cull_for_do.htest <- function(object, ...) {
  if (is.null(object$conf.int)) {
    result <-  data.frame(
      statistic = null2na(object$statistic),
      parameter = null2na(object$parameter),
      p.value = null2na(object$p.value),
      method = null2na(object$method),
      alternative = null2na(object$alternative),
      data = null2na(object$data.name)
    )
  } else {
    result <-  data.frame(
      statistic = null2na(object$statistic),
      parameter = null2na(object$parameter),
      p.value = null2na(object$p.value),
      conf.level = attr(object$conf.int,"conf.level"),
      lower = object$conf.int[1],
      upper = object$conf.int[2],
      method = null2na(object$method),
      alternative = null2na(object$alternative),
      data = null2na(object$data.name)
    )
  }
  if ( !is.null(names(object$statistic)) )
    names(result)[1] <-  names(object$statistic)
  if ( !is.null(names(object$parameter)) )
    names(result)[2] <- names(object$parameter)
  return(result)
}

#   if (inherits(object, 'table') ) {
#     nm <- names(object)
#     result <-  as.vector(object)
#     names(result) <- nm
#     return(result)
#   }

#' @export
cull_for_do.cointoss <- function(object, ...) {
  return( c(n=attr(object,'n'),
            heads=sum(attr(object,'sequence')=='H'),
            tails=sum(attr(object,'sequence')=='T'),
            prop=sum(attr(object,'sequence')=="H") / attr(object,'n')
  ) )
}

#' @export
cull_for_do.matrix <- function(object, ...) {
  if (ncol(object) == 1) {
    nn <- rownames(object)
    object <- as.vector(object)
    if (is.null(nn)) {
      names(object) <- paste('v',1:length(object),sep="")
    } else {
      names(object) <- nn
    }
    return(object)
  }
  if (nrow(object) > 1) {
    res <- as.data.frame(object)
    res[[".row"]] <- row.names(object)
    return(res)
  }
  # if we get here, we have a 1-row or empty matrix
  row.names(object) <- NULL
  object
}

#' @rdname do
#' @aliases *,repeater,ANY-method
#' @export

setMethod(
  "*",
  signature(e1 = "repeater", e2="ANY"),
  function (e1, e2)
  {
    # `enquo()` is:
    # - Not needed since `...` can't be passed to an operator.
    # - Unusable here, because dispatch forces arguments, which causes `enquo()`
    #   to return a forced promise.
    e2_expr <- substitute(e2)
    e2_env <- parent.frame()
    n = e1@n

    cull = e1@cull
    if (is.null(cull)) {
      cull <- cull_for_do
    }

    out.mode <- if (!is.null(e1@mode)) e1@mode else 'default'

    resultsList <- if( e1@parallel && "package:parallel" %in% search() ) {
      if (getOption("mosaic:parallelMessage", TRUE)) {
        message("Using parallel package.\n",
                "  * Set seed with set.rseed().\n",
                "  * Disable this message with options(`mosaic:parallelMessage` = FALSE)\n")
      }
      parallel::mclapply( integer(n), function(...) { cull(eval(e2_expr, e2_env)) } )
    } else {
      lapply( integer(n), function(...) { cull(eval(e2_expr, e2_env)) } )
    }

    if (out.mode=='default') {  # is there any reason to be fancier?
      out.mode = 'data.frame'
    }

    result <- switch(out.mode,
                     "list" = resultsList,
                     "data.frame" = .list2tidy.data.frame( resultsList ),
                     "matrix" = as.matrix( do.call( rbind, resultsList) ),
                     "vector" = unlist(resultsList)
    )
    class(result) <- c(paste('do', class(result)[1], sep="."), class(result))
    if (inherits( result, "data.frame")) {
      # we get mutliple parts here if expression involves, for example, ::
      # just grab last part. (paste()ing would be out of order
      alt_name <- tryCatch(
        tail(as.character(e2_expr[[1]]), 1),
        error = function(e) "result"
      )
      names(result) <- mosaicCore::nice_names(names(result))
      names(result)[names(result) == "..result.."] <-
        if(mosaicCore::nice_names(alt_name) == alt_name) alt_name else "result"
    }
    attr(result, "lazy") <- e2_expr
    attr(result, "lazy_env") <- e2_env
    if (out.mode == "data.frame") attr(result, "culler") <- cull
    return(result)
  })

utils::globalVariables(c('.latticeEnv'))

#' Create function from data
#'
#' These functions create mathematical functions from data, by smoothing, splining, or linear
#' combination (fitting).  Each of them takes a formula and a data frame as an argument
#' @rdname FunctionsFromData
#' @name FunctionsFromData
#' @aliases smoother linearModel spliner connector
#'
#' @param formula a formula.  Only one quantity is allowed on the left-hand side, the
#' output quantity
#' @param data a data frame
#' @param method a method for splining.  See [spline()].
#' @param monotonic a `TRUE/FALSE` flag specifying whether the spline should
#' respect monotonicity in the data
#' @param span parameter to smoother.  How smooth it should be.
#' @param degree parameter to smoother. 1 is locally linear, 2 is locally quadratic.
#'
#' @details
#' These functions use data to create a mathematical, single-valued function of the inputs.
#' All return a function whose arguments are the variables used on the right-hand side of the formula.
#' If the formula involves a transformation, e.g. `sqrt(age)` or `log(income)`,
#' only the variable itself, e.g. `age` or `income`, is an argument to the function.
#' 
#' `linearModel` takes a linear combination of the vectors specified on the right-hand side.
#' It differs from `project` in that `linearModel` returns a function
#' whereas `project` returns the coefficients.  NOTE: An intercept term is not included
#' unless that is explicitly part of the formula with `+1`.  This conflicts with the
#' standard usage of formulas as found in `lm`.  Another option for creating
#' such functions is to combine [lm()] and [makeFun()].
#' 
#' `spliner` and `connector` currently work for only one input variable.
#' 
#' 
#'
#' @seealso [project()] method for formulas
#'
#' @examples
#' if (require(mosaicData)) {
#' data(CPS85)
#' f <- smoother(wage ~ age, span=.9, data=CPS85)
#' f(40)
#' g <- linearModel(log(wage) ~ age + educ + 1, data=CPS85)
#' g(age=40, educ=12)
#' # an alternative way to define g (Note: + 1 is the default for lm().)
#' g2 <- makeFun(lm(log(wage) ~ age + educ, data=CPS85))
#' g2(age=40, educ=12)
#' x<-1:5; y=c(1, 2, 4, 8, 8.2)
#' f1 <- spliner(y ~ x)
#' f1(x=8:10)
#' f2 <- connector(x~y)
#' }
#' @export

spliner <- function(formula, data=NULL, method="fmm", monotonic=FALSE) {
  .interpolatingFunction(formula, data, method=method, monotonic=monotonic)
}

#' @rdname FunctionsFromData
#' @export

connector <- function(formula, data=NULL, method="linear") {
  .interpolatingFunction(formula, data, connect=TRUE)
}
#' @rdname FunctionsFromData
#' @param \dots additional arguments to [stats::loess()] or [stats::lm()]
#' @export

smoother <- function(formula, data, span=0.5, degree=2, ... ) {
  input.names <- all.vars(formula)[-1]
  L <- loess(formula, data, span=span, degree=degree, ..., control=loess.control(surface="direct"))
  makeDF <- paste( "data.frame( ", paste(input.names, collapse=",", sep=""), ")")
  F <- function() {
    D <- eval(parse(text=makeDF))
    predict(L, newdata=D)
  }
  tmp <- paste("alist( ", paste(input.names, "=", collapse = ",", sep = ""), ")")
  tmp <- eval(parse(text = tmp))
  formals(F) <- tmp
  return(F)
}

#' @rdname FunctionsFromData
#' @export

linearModel <- function(formula, data, ...) {
  input.names <- all.vars(formula)[-1]
  L <- lm(update(formula, ~-1+.), data, ...)
  makeDF <- paste( "data.frame( ", paste(input.names, collapse=",", sep=""), ")")
  F <- function(showcoefs) {
    if( showcoefs ) coef(L)
    else { # evaluate the function 
      D <- eval(parse(text=makeDF))
      predict(L, newdata=D)
    }
  }
  tmp <- paste("alist( ", paste(input.names, "=", collapse = ",", sep = ""), ", showcoefs=FALSE)")
  tmp <- eval(parse(text = tmp))
  formals(F) <- tmp
  attr(F, "mosaicType") <- "Fitted Linear Model"
  return(F)
}

.interpolatingFunction <- function(formula, data, method="fmm", monotonic=FALSE, connect=FALSE) {
  fnames <- all.vars(formula)
  if( length(fnames) > 2 )
    stop("Sorry: Doesn't yet handle multiple input variables.")
  
  values <- model.frame(formula, data=data )
  y <- model.response(values)
  x <- mat(formula, data=data)
  if( connect ) SF <- approxfun(x, y, rule=2)
  else {
    if( ! monotonic )  SF <- splinefun(x, y, method=method)
    else SF <- splinefun(x, y, method="monoH.FC")
  }
  F <- function(foobar, deriv=0 ){
    x <- get(fnames[2])
    if(connect) SF(x)
    else SF(x, deriv=deriv)
  }
  if (connect) tmp <- paste("alist( ", fnames[2], "=)", sep="")
  else tmp <- paste("alist( ", fnames[2], "=, deriv=0)", sep="")
  formals(F) <- eval(parse(text=tmp))
  return(F)
}

# context("smoothers")
require(mosaicData)
testthat::test_that("smoothers works", {
  
  data(CPS85)
  f <- smoother(wage ~ age, span=.9, data=CPS85)
  g <- linearModel(log(wage) ~ age + educ + 1, data=CPS85)
  # an alternative way to define g (Note: + 1 is the default for lm().)
  g2 <- makeFun(lm(log(wage) ~ age + educ, data=CPS85))
  
  x<-1:5; y=c(1, 2, 4, 8, 8.2)
  f1 <- spliner(y ~ x)
  f1(x=8:10)
  f2 <- connector(x~y)
  
  testcase1 <- function (age) 
  {
    D <- eval(parse(text = makeDF))
    predict(L, newdata = D)
  }
  testcase2 <- c(`1` = 10.2530736275772)
  
  testcase3 <- structure(function (age, educ, showcoefs = FALSE) 
  {
    if (showcoefs) 
      coef(L)
    else {
      D <- eval(parse(text = makeDF))
      predict(L, newdata = D)
    }
  }, mosaicType = "Fitted Linear Model")
  
  testcase4 <- c(`1` = 2.01015841331998)
  
  testcase5 <- function (x, deriv = 0) 
  {
    x <- get(fnames[2])
    if (connect) 
      SF(x)
    else SF(x, deriv = deriv)
  }
  
  testcase6 <- c(-74.2666666666667, -148.777777777778, -256.466666666667)
  
  
  testcase7 <- function (y) 
  {
    x <- get(fnames[2])
    if (connect) 
      SF(x)
    else SF(x, deriv = deriv)
  }
  
  expect_equal(ignore_attr = TRUE, testcase2, f(40))
  expect_equal(ignore_attr = TRUE, testcase4, g(age=40, educ=12))
  
  expect_equal(ignore_attr = TRUE, testcase6, f1(x=8:10))
  
  
})

#' Swap values among columns of a data frame
#' 
#' Swap values among columns of a data frame
#' 
#' @param data a data frame
#' @param which a formula or an integer or character vector specifying columns in 
#' `data`
#' @details `swap` is not a particularly speedy function.  It is intended primarily
#' as an aid for teaching randomization for paired designs.  Used this way, the number of
#' randomizations should be kept modest (approximately 1000) unless you are very patient.
#' 
#' @export
#' @examples
#' if (require(tidyr)) {
#'   Sleep2 <- sleep |> spread( key=group, val=extra )
#'   names(Sleep2) <- c("subject", "drug1", "drug2")
#'   swap(Sleep2, drug1 ~ drug2)
#'   mean( ~(drug1 - drug2), data=Sleep2)
#'   do(3) * mean( ~(drug1 - drug2), data=Sleep2 |> swap(drug1 ~ drug2) ) 
#' } 
 
swap <- function(data, which) {
 
  if (inherits(which, "formula")) {
    which <- all.vars(which)
    bad <- setdiff(which, names(data))
    if (length(bad) >= 1L) {
      stop( "formula contains variables not in data: ", bad[1] )
    }
  }
  
  if (is.character(which)) {
    which <- match(which, names(data))
  }
  

  if (! is.integer(which) ) stop("Value of `which' doesn't make sense to me.")
  
  if (length(which) <= 1) {
    message("Nothing to swap.  Returning data unchanged.")
    return(data)
  }
  
  subdata <- data[, which]
  n <- nrow(data)
  idx <- do(n) * shuffle(1:ncol(subdata))
  replacement <- data.frame(
    lapply( 1:ncol(subdata), function(i) subdata[cbind(1:n, idx[,i])] )
  )
  res <- data
  res[, which] <- replacement
  
  res
}

# probably don't need this anymore
fetchByCol <- function(data, cols) {
  res <- data[[ cols[1] ]]
  
  for(i in (1:length(res))) {
    res[i] <- data[ i, cols[i] ]
  }
  res
}

utils::globalVariables(c("xleft", "xright"))

#' Tally test statistics
#' 
#' Tally test statistics from data and from multiple draws
#' from a simulated null distribution
#' @param sample sample data
#' 
#' @param rdata a matrix of randomly generated data under 
#'   null hypothesis.  
#' 
#' @param FUN a function that computes the test statistic from
#'   a data set.  The default value does nothing, making it easy to 
#'   use this to tabulate precomputed statistics into a null distribution.
#'   See the examples.
#' 
#' @param direction 1 or 2 indicating whether samples in `rdata`
#'   are in rows (1) or columns (2).
#'  
#' @param system graphics system to use for the plot
#' 
#' @param shade a color to use for shading. 
#' 
#' @param alpha opacity of shading.
#' 
#' @param bins number of bins for histogram.
#' @param binwidth bin width for histogram.
#' 
#' @param fill fill color for histogram.
#' 
#' @param color border color for histogram.
#' 
#' @param stemplot 
#' indicates whether a stem plot should be displayed
#' @param q quantiles of sampling distribution to display
#' 
#' @param fun same as `FUN` so you don't have to remember if it
#'   should be capitalized
#'
#' @param xlim limits for the horizontal axis of the plot.
#'
#' @param center center of null distribution
#'
#' @param alternative one of `default`, `two.sided`, `less`, or `greater`
#' 
#' @param sig.level  significance threshold for `wilcox.test` used to detect lack of symmetry
#' 
#' @param quiet a logicial indicating whether the text output should be suppressed
#' 
#' @param \dots additional arguments passed to [lattice::histogram()] or [ggplot2::geom_histogram()]
#' 
#' @return A lattice or ggplot showing the sampling distribution. 
#' 
#' As side effects, information
#' about the empirical sampling distribution and (optionally) a stem plot are
#' printed to the screen.
#'
#' @examples
#' # is my spinner fair?
#' x <- c(10, 18, 9, 15)   # counts in four cells
#' rdata <- rmultinom(999, sum(x), prob = rep(.25, 4))
#' statTally(x, rdata, fun = max, binwidth = 1)  # unusual test statistic
#' statTally(x, rdata, fun = var, shade = "red", binwidth = 2)  # equivalent to chi-squared test
#' # Can also be used with test stats that are precomputed.
#' if (require(mosaicData)) {
#' D <- diffmean( age ~ sex, data = HELPrct); D
#' nullDist <- do(999) * diffmean( age ~ shuffle(sex), data = HELPrct)
#' statTally(D, nullDist)
#' statTally(D, nullDist, system = "lattice")
#' }
#' 
#' @keywords inference 
#' @export
 
statTally <-
function (sample, rdata, FUN, direction = NULL, 
          alternative=c('default', 'two.sided', 'less', 'greater'), 
          sig.level = 0.1, 
          system = c("gg", "lattice"),
          shade = "navy",
          alpha = 0.10,
          binwidth = NULL, bins = NULL, fill = "gray80", color = "black",
          center = NULL, stemplot = dim(rdata)[direction] < 201, 
          q = c(0.5, 0.9, 0.95, 0.99), fun = function(x) x, xlim, 
          quiet = FALSE,
          ...) 
{

  mymessage <- 
    if (quiet) {
      function(...) {}
    } else {
      function(...) {cat(..., "\n")}
    }
  
  system <- match.arg(system)
	alternative <- match.arg(alternative) 
	
	dots <- list(...)

	if (missing(FUN)) {
		FUN = fun
	}
	
	if ( is.null(direction) ) {
		size <- max(NROW(sample), NCOL(sample))
		if ( NROW (rdata) == size ) {
			direction <- 2
		} else if ( NCOL(rdata) == size ) {
			direction <- 1
		} else {
			stop(paste("sample and rdata have incompatible dimensions:", 
			           c(size, NROW(rdata), NCOL(rdata))))
		}
	}

    dstat <- FUN(sample)
    stats <- apply(rdata, direction, FUN)
	if (alternative == 'default') {
		if (is.null(center)) center <- median(stats, na.rm=TRUE)
		pv <- pval(wilcox.test(stats, mu=center, exact=FALSE)) 
		if (is.na(pv) || pv > sig.level) {
           alternative <- "two.sided"
		   mymessage(paste('\nNull distribution appears to be symmetric. (p = ', signif(pv,3),')'))
		} else {
           alternative <- if (dstat < center) 'less' else 'greater'
		   mymessage(paste('\nNull distribution appears to be asymmetric. (p = ', signif(pv,3),')', sep=""))
		}
	}
	if (is.null(center)) center <- 0

    mymessage(paste("\nTest statistic applied to sample data = ", signif(dstat, 4)))
    mymessage("\nQuantiles of test statistic applied to random data:")
    capture.output(quantile(stats, q, na.rm = TRUE)) |>
      paste(collapse = "\n") |> 
      mymessage()
    if (any( ! is.finite(stats))) {
      mymessage("** Note:  ", table(is.finite(stats))["FALSE"], " non-finite or missing values excluded.")
      stats <- stats[is.finite(stats)]
    }
    if (stemplot) {
        stem(stats)
    }
	results <- data.frame(stat=stats)
	if (missing(xlim)) xlim <- range(pretty(c(stats,dstat)))

	if (alternative == 'two.sided') {
	  hi <- center + abs(dstat - center)
	  lo <- center - abs(dstat - center)
	}
	if (alternative == 'greater') {
	  hi <- dstat
	  lo <- -Inf
	}
	if (alternative == 'less') {
	  hi <-  Inf
	  lo <- dstat
	}

	Rect_Data <- data.frame(
	  xleft = c(-Inf, hi),
	  xright = c(lo, Inf)
	)

	plot1 <- 
	  switch(
	    system,
	    gg = 
	      tryCatch( 
	        ggplot() +
	          geom_histogram(na.rm = TRUE,
	            aes(y = after_stat(density), x = stat),
	            data = results,
	            fill = fill, color = color, binwidth = binwidth, bins = bins,
	            ...) +
	          geom_rect(
	            aes(ymin = 0, ymax = Inf, xmin = xleft, xmax = xright),
	            data = Rect_Data,
	            fill = shade, color = "transparent", alpha = alpha, inherit.aes = FALSE) +
	          lims(x = xlim),
	        #             
	        #                  
	        # gf_dhistogram( ~ stat, data = results, fill = fill, color = color, binwidth = binwidth) |>
	        #   gf_rect(0 + Inf ~ xleft + xright, fill = shade, alpha = alpha, data = Rect_Data,
	        #           inherit = FALSE) |>
	        #   gf_lims(x = xlim), 
	        error = function(e) NULL
	      ),
	    lattice = 
	      tryCatch( histogram(~stat, data=results,  #groups=stat >= dstat, 
	                          xlim = xlim, width = binwidth, col = fill, border = color, 
	                          ...,
	                          panel = function(x,...){
	                            panel.xhistogram(x, ...)
	                            grid.rect( x=grid::unit(lo, "native"), y = grid::unit(0, "native"),
	                                       hjust = 1,
	                                       height = unit(1, "npc"), vjust = 0,
	                                       gp=gpar(fill = shade, col = shade, alpha = alpha))
	                            grid.rect( x=grid::unit(hi, "native"), y = grid::unit(0, "native"),
	                                       hjust = 0,
	                                       height = unit(1, "npc"), vjust = 0,
	                                       gp=gpar(fill = shade, col = shade, alpha = alpha))
	                          }
	      ) , error = function(e) NULL
	      )
	  )
	
	# for backward compatibility with a code chunk in fast 2e
	if (system == "gg" && !is.null(dots$xlab)) {
	  plot1 <- plot1 |> gf_labs(x = dots$xlab)
	}
	
	
	# add in observed statistic for the remaining summaries.
	stats <- c(dstat, stats)
	mymessage("\nOf the ", length(stats), " samples (1 original + ", length(stats) -1, " random),")
    mymessage("\t", paste(sum(stats == dstat), "(", round(100 * 
        sum(stats == dstat)/length(stats), 2), "% )", "had test stats =", 
        signif(dstat, 4)))
	if (alternative != 'greater') {
    	mymessage("\t", paste(sum(stats <= lo), "(", round(100 * sum(stats <= lo)/length(stats), 2), 
							  "% )", "had test stats <=", 
   	     signif(lo, 4)))
	}
	if (alternative != 'less') {
    mymessage("\t", paste(sum(stats >= hi), "(", round(100 * sum(stats >= hi)/length(stats), 2), 
						  "% )", "had test stats >=", signif(hi, 4)))
	}
	
    return(plot1)
}

# context('statTally()')

expect_output <- function(x, expectation) {
  expect_true(any(grepl(expectation, x)))
}

test_that("Two-sided test", {
  set.seed(1)
  rdata <- data.frame("null" = rnorm(1000))
  expect_output(capture.output(tmp <- statTally(1, rdata, alternative="two.sided")),
                 "168 \\( 16.78 % \\) had test stats <= -1")
  expect_output(capture.output(tmp <- statTally(1, rdata, alternative="two.sided")),
                 "159 \\( 15.88 % \\) had test stats >= 1")
})

test_that("One-sided tests", {
  set.seed(1)
  rdata <- data.frame("null" = rnorm(1000))
  expect_output(capture.output(tmp <- statTally(1, rdata, alternative="greater")),
                 "*159 \\( 15.88 % \\) had test stats >= 1*")
  expect_output(capture.output(tmp <- statTally(1, rdata, alternative="less")),
                 "*843 \\( 84.22 % \\) had test stats <= 1*")
  expect_output(capture.output(tmp <- statTally(-1, rdata, alternative="greater")),
                 "*833 \\( 83.22 % \\) had test stats >= -1*")
  expect_output(capture.output(tmp <- statTally(-1, rdata, alternative="less")),
                 "*169 \\( 16.88 % \\) had test stats <= -1*")

})

require(mosaicData)
x <- c(10, 18, 9, 15)   # counts in four cells
set.seed(16)
rdata <- rmultinom(999, sum(x), prob = rep(.25, 4))
set.seed(17)
D <- diffmean( age ~ sex, data = HELPrct)
set.seed(19)
nullDist <- do(999) * diffmean( age ~ shuffle(sex), data = HELPrct)

testthat::test_that("statTally works", {
  
  wrapped_expect_doppelganger("statTally1", statTally(x, rdata, fun = max, binwidth = 1))
  wrapped_expect_doppelganger("statTally2", statTally(x, rdata, fun = var, shade = "red", binwidth = 2))
  wrapped_expect_doppelganger("statTally3", statTally(x, rdata, fun = max, binwidth = 1))
  #wrapped_expect_doppelganger("statTally4", statTally(D, nullDist))
  #wrapped_expect_doppelganger("statTally5", statTally(D, nullDist, system = "lattice"))
})

#' Add to Lattice Plots
#' 
#' Simplified lattice plotting by adding additional elements to existing plots.
#'
#' @param x  callable graphical element to be added to a panel or panels in a lattice plot 
#' @param data a list containing objects that can be referred to in `x`.  Panel functions also 
#' have access to the data already used in the panel by the underlying lattice plot.  See 
#' [latticeExtra::layer()] for details.
#' @param ... additional arguments passed to [latticeExtra::layer()].
#' @param plot a lattice plot to add to.  Defaults to previous lattice plot.
#' @return a trellis object
#' @seealso [latticeExtra::layer()]
#' 
#' @details
#' `ladd` is a wrapper around [latticeExtra::layer()] that simplifies 
#' certain common plotting additions.  The same caveats that apply to that function
#' apply here as well.  In particular, `ladd` uses non-standard evaluation.  
#' For this reason care must be taken if trying to use `ladd` within other functions
#' and the use of `data` may be required to pass information into the environment
#' in which `x` will be evaluated.
#' 
#' @author Randall Pruim (\email{rpruim@@calvin.edu})
#' 
#' @examples
#' p <- xyplot(rnorm(100) ~rnorm(100))
#' print(p)
#' ladd(panel.abline(a=0,b=1))
#' ladd(panel.abline(h=0,col='blue'))
#' ladd(grid.text('Hello'))
#' ladd(grid.text(x=.95,y=.05,'text here',just=c('right','bottom')))
#' q <- xyplot(rnorm(100) ~rnorm(100)|factor(rbinom(100,4,.5)))
#' q <- update(q, layout=c(3,2))
#' ladd(panel.abline(a=0,b=1), plot=q)
#' ladd(panel.abline(h=0,col='blue'))
#' ladd( grid.text("(2,1)",gp=gpar(cex=3,alpha=.5)), columns=2, rows=1)
#' ladd( grid.text("p5",gp=gpar(cex=3,alpha=.5)), packets=5)
#' q
#' ladd( grid.text(paste(current.column(), current.row(),sep=','), gp=gpar(cex=3,alpha=.5)) )
#' histogram( ~eruptions, data=faithful )
#' # over would probably be better here, but the demonstrates what under=TRUE does.
#' ladd(panel.densityplot(faithful$eruptions, lwd=4), under=TRUE) 
#'
#' @keywords graphics 
#' @export

ladd <- function (x, data=NULL, ..., plot=trellis.last.object()) 
{
  rlang::check_installed('latticeExtra')
  return( plot + eval( substitute( latticeExtra::layer(foo, data=data, ...),
                                   list(foo=substitute(x) ) )
  ) )
}

# context("ladd()")

testthat::test_that("basic ladd works", {
  set.seed(5)
  p <- xyplot(rnorm(100) ~rnorm(100))
  p
  wrapped_expect_doppelganger("ladd1", ladd(panel.abline(a=0,b=1)))
  wrapped_expect_doppelganger("ladd2", ladd(panel.abline(h=0,col='blue')))
  wrapped_expect_doppelganger("ladd3", ladd(grid.text('Hello')))
  wrapped_expect_doppelganger("ladd4", ladd(grid.text(x=.95,y=.05,'text here',just=c('right','bottom'))))
})



testthat::test_that("ladd with factors works", {
  set.seed(6)
  q <- xyplot(rnorm(100) ~rnorm(100)|factor(rbinom(100,4,.5)))
  q <- update(q, layout=c(3,2))
  q
  wrapped_expect_doppelganger("ladd5", ladd(panel.abline(a=0,b=1), plot=q))
  wrapped_expect_doppelganger("ladd6", ladd(panel.abline(h=0,col='blue')))
  wrapped_expect_doppelganger("ladd7", ladd( grid.text("(2,1)",gp=gpar(cex=3,alpha=.5)), columns=2, rows=1))
  wrapped_expect_doppelganger("ladd8", ladd( grid.text("p5",gp=gpar(cex=3,alpha=.5)), packets=5))
})

# Refresh the plot
q
ladd( grid.text(paste(current.column(), current.row(),sep=','), gp=gpar(cex=3,alpha=.5)) )
histogram( ~eruptions, data=faithful )
testthat::test_that("ladd under works", {
  # over would probably be better here, but the demonstrates what under=TRUE does.
  wrapped_expect_doppelganger("ladd9", ladd(panel.densityplot(faithful$eruptions, lwd=4), under=TRUE))
})

#' Fit splines to data
#'
#' These functions create mathematical functions from data, using splines.
#'
#' @param formula a formula.  Only one quantity is allowed on the left-hand side, the
#' output quantity
#' @param data a data frame in which `formula` is evaluated.
#' @param type type of splines to use; one of 
#' `"linear"`, `"cubic"`, `"natural"` (cubic with linear tails, the default), 
#' or `"polynomial"`.
#' @param degree parameter for splines when `type` is `"polynomial"`.  
#' 1 is locally linear, 2 is locally quadratic, etc.
#' @param df degrees of freedom (used to determine how many knots should be used)
#' @param knots a vector of knots
#' @param \dots additional arguments passed to spline basis functions
#' ([splines::ns()] and [splines::bs()]).
#'
#' @return a function of the explanatory variable
#'
#' @seealso [splines::bs()]  and [splines::ns()] for the bases used to generate the splines.
#'
#' @examples
#' f <- fitSpline( weight ~ height, data=women, df=5 )
#' xyplot( weight ~ height, data=women )
#' plotFun(f(height) ~ height, add=TRUE)
#'
#' g <- fitSpline( length ~ width, data = KidsFeet, type='natural', df=5 )
#' h <- fitSpline( length ~ width, data = KidsFeet, type='linear', df=5 )
#' xyplot( length ~ width, data = KidsFeet, col='gray70', pch=16)
#' plotFun(g, add=TRUE, col='navy')
#' plotFun(h, add=TRUE, col='red')
#' @export

fitSpline <- function( formula, data=parent.frame(), 
			df = NULL,
			knots = NULL,
			degree = 3,
 			type=c('natural','linear','cubic','polynomial'),
			...) {
  
  rlang::check_installed('splines')

	type <- match.arg(type)

    xnames <- all.vars(rhs(formula))
    ynames <- all.vars(lhs(formula))
	if (length(xnames) != 1 || length(ynames) != 1) 
    	stop("Sorry: Doesn't handle multiple variables (yet).")
	x <- get( xnames, data )
	y <- get( ynames, data )

	method  <- switch(type,
					natural="natural",
					linear="linear",
					polynomial="polynomial",
					cubic='cubic'
					)
    if (method == 'natural') {
		if (is.null(knots)) {
			model <- lm( y ~ splines::ns(x, df=df, ...) )
		} else {
			model <- lm( y ~ splines::ns(x, df=df, knots=knots, ...) )
		}
    } else {
		if (method == 'linear') degree=1
		if (method == 'cubic') degree=3
		if (is.null(knots)) {
			model <- lm( y ~ splines::bs(x, df=df, degre=degree, ...) )
		} else {
			model <- lm( y ~ splines::bs(x, df=df, knots=knots, degre=degree, ...) )
		}
    }

	result <- function(x) {}
	    body(result) <- parse(text = paste("return(predict(model, newdata=data.frame(", 
            paste("x= ", xnames, collapse = ",", sep = ""), 
            "), ...))"))
        formals(result) <- eval(parse(text = paste("as.pairlist(alist(", 
            paste(xnames, "= ", collapse = ",", sep = ""), ", ...=))")))
		environment(result) <- list2env( c( as.list(environment(result)), 
								       list(knots=knots, df=df, degree=degree, model=model)) )
    	attr(result, "coefficients") <- coef(model)
    return(result)
}

#' Create new variables from logicals
#' 
#' Utility functions for creating new variables from logicals describing the levels
#' 
#' @param \dots named logical "rules" defining the levels.
#' @param .method one of `"unique"`, `"first"`, and `"last"`.  
#' If `"unique"`, exactly one rule must be `TRUE` for each position.
#' If `"first"`, the first `TRUE` rule defines the level.
#' If `"last"`, the last `TRUE` rule defines the level.
#' @param .debug one of `"default"`, `"always"`, and `"never"`, indicating
#' whether debugging information should be printed.  If `"default"`, debugging 
#' information is printed only when multiple rules give conflicting definitions 
#' for some positions.  
#' @param .ordered a logical indicating whether the resulting factored should be ordered
#' Ignored if `.asFactor` is `FALSE`.
#' @param .sort One of `"given"` (the default) or `"alpha"` or 
#' a vector of integers the same length as the number of levels indicating the 
#' order in which the levels should appear in the resulting factor. 
#' Ignored if `.asFactor` is `FALSE`.
#' @param .default character vector of length 1 giving name of default level or 
#' `NULL` for no default.
#' @param .asFactor A logical indicating whether the returned value should be a factor.
#' 
#' @details
#' Each logical "rule" corresponds to a level in the resulting variable.  
#' If `.default` is defined, an implicit rule is added that is `TRUE` 
#' whenever all other rules are `FALSE`.
#' When there are multiple `TRUE` rules for a slot, the first or last such is used
#' or an error is generated, depending on the value of `method`.   
#' 
#' `derivedVariable` is designed to be used with [transform()] or 
#' [dplyr::mutate()] to add new 
#' variables to a data frame.  `derivedFactor`() is the same but that the 
#' default value for `.asFactor` is `TRUE`.  See the examples.
#' 
#' @examples
#' Kf <- mutate(KidsFeet, biggerfoot2 = derivedFactor(
#'                    dom = biggerfoot == domhand,
#'                    nondom = biggerfoot != domhand)
#'                    )
#' tally( ~ biggerfoot + biggerfoot2, data = Kf)
#' tally( ~ biggerfoot + domhand, data = Kf)
#' 
#' # Three equivalent ways to define a new variable
#' # Method 1: explicitly define all levels
#' modHELP <- mutate(HELPrct, drink_status = derivedFactor( 
#'   abstinent = i1 == 0,
#'   moderate = (i1>0 & i1<=1 & i2<=3 & sex=='female') |
#'      (i1>0 & i1<=2 & i2<=4 & sex=='male'),
#'   highrisk = ((i1>1 | i2>3) & sex=='female') | 
#'       ((i1>2 | i2>4) & sex=='male'),
#'   .ordered = TRUE)
#' )
#' tally( ~ drink_status, data = modHELP)
#'
#' # Method 2: Use .default for last level
#' modHELP <- mutate(HELPrct, drink_status = derivedFactor( 
#'   abstinent = i1 == 0,
#'   moderate = (i1<=1 & i2<=3 & sex=='female') |
#'      (i1<=2 & i2<=4 & sex=='male'),
#'   .ordered = TRUE,
#'   .method = "first",
#'   .default = "highrisk")
#' )
#' tally( ~ drink_status, data = modHELP)
#' 
#' # Method 3: use TRUE to catch any fall through slots
#' modHELP <- mutate(HELPrct, drink_status = derivedFactor( 
#'   abstinent = i1 == 0,
#'   moderate = (i1<=1 & i2<=3 & sex=='female') |
#'      (i1<=2 & i2<=4 & sex=='male'),
#'   highrisk=TRUE,
#'   .ordered = TRUE,
#'   .method = "first"
#'   )
#' )
#' tally( ~ drink_status, data = modHELP)
#' is.factor(modHELP$drink_status)
#' 
#' modHELP <- mutate(HELPrct, drink_status = derivedVariable( 
#'   abstinent = i1 == 0,
#'   moderate = (i1<=1 & i2<=3 & sex=='female') |
#'      (i1<=2 & i2<=4 & sex=='male'),
#'   highrisk=TRUE,
#'   .ordered = TRUE,
#'   .method = "first"
#'   )
#' )
#' is.factor(modHELP$drink_status)
#' @export

derivedVariable <- function(..., 
                          .ordered=FALSE, 
                          .method = c("unique","first", "last"),
                          .debug = c("default", "always", "never"),
                          .sort = c("given","alpha"),
                          .default = NULL,
                          .asFactor = FALSE
) {
  .method <- match.arg( .method )
  .debug <- match.arg( .debug )
  
  if (! is.numeric(.sort) ) {
    .sort <- match.arg( .sort )
  }
  
  rules <- list(...)
  rules <- lapply( rules, as.logical )
  rulesM <- do.call(cbind, rules)
  
  noApplicableRule <- apply( rulesM, 1, function(x) all(!x) )
  undefined <- any( noApplicableRule, na.rm=TRUE )
  
  if (undefined) {
    defaultRule <- list( noApplicableRule )
    names(defaultRule) <- if (is.null(.default)) "other" else .default
    rules <- c(rules, defaultRule)
    rulesM <- do.call(cbind, rules)
  }
  
  lastApplicableRule  <- 
    apply( rulesM, 1, 
           function(x) if (all(is.na(x))) NA else max(which(x), na.rm=TRUE) )
  firstApplicableRule <- 
    apply( rulesM, 1, 
           function(x) if (all(is.na(x))) NA else min(which(x), na.rm=TRUE) )
  firstApplicableRule[firstApplicableRule == Inf] <- NA
  lastApplicableRule[lastApplicableRule == -Inf] <- NA
  
  multipleDefs <- any( lastApplicableRule != firstApplicableRule, na.rm=TRUE )
  troubles <- (multipleDefs && .method == "unique") || ( undefined && is.null(.default) )
  
  ruleApplied <- switch(.method,
                        "unique" = firstApplicableRule,
                        "first" = firstApplicableRule, 
                        "last" = lastApplicableRule
  )
  
  debug <- (.debug == "always") || (troubles && .debug=="default")
  
  if (debug) {
    cat("\n")
    if (multipleDefs) {
      print(tally(~ firstApplicableRule + lastApplicableRule, useNA="if") )
    } else {
      cat("Rules applied: ")
      print(tally(~ruleApplied, useNA="if"))
    }
    cat("\n")
  }
  
  if (.method == "unique" && any(firstApplicableRule != lastApplicableRule, na.rm=TRUE) ) {
    stop('Conflicting rules.  Do you want to use .method="first" or .method="last"?')
  }
  
  if (undefined && is.null(.default)) {
    stop("There are items with no TRUE rules and no default.")
  }
  
  levels <- names(rules)
  if (length(rules)  < 1) return(NULL)
  
  if (! is.numeric( .sort ) ) {
    order <- switch( .sort,
                     "alpha" = order(names(rules)),
                     "given" = 1:length(names(rules))
    ) 
  } else {
    order <- .sort
  }
 
  if (.asFactor) 
    factor( names(rules)[ ruleApplied ], 
            ordered=.ordered, 
            levels = names(rules)[order] )
  else 
    names(rules)[ruleApplied] 
}

#' @rdname derivedVariable
#' @export
#' 
derivedFactor <- function(..., .asFactor = TRUE) {
  derivedVariable(..., .asFactor = .asFactor)
}

# context("derivedFactor()")

testthat::test_that("derivedFactor Works", {
  Kf <- mutate(KidsFeet, biggerfoot2 = derivedFactor(
                     dom = biggerfoot == domhand,
                     nondom = biggerfoot != domhand)
                     )
  # Method 1: explicitly define all levels
  modHELP <- mutate(HELPrct, drink_status = derivedFactor(
    abstinent = i1 == 0,
    moderate = (i1>0 & i1<=1 & i2<=3 & sex=='female') |
      (i1>0 & i1<=2 & i2<=4 & sex=='male'),
    highrisk = ((i1>1 | i2>3) & sex=='female') |
      ((i1>2 | i2>4) & sex=='male'),
    .ordered = TRUE)
  )
  
  # Method 2: Use .default for last level
  modHELP2 <- mutate(HELPrct, drink_status = derivedFactor(
    abstinent = i1 == 0,
    moderate = (i1<=1 & i2<=3 & sex=='female') |
      (i1<=2 & i2<=4 & sex=='male'),
    .ordered = TRUE,
    .method = "first",
    .default = "highrisk")
  )
  
  # Method 3: use TRUE to catch any fall through slots
  modHELP3 <- mutate(HELPrct, drink_status = derivedFactor(
    abstinent = i1 == 0,
    moderate = (i1<=1 & i2<=3 & sex=='female') |
      (i1<=2 & i2<=4 & sex=='male'),
    highrisk=TRUE,
    .ordered = TRUE,
    .method = "first"
  )
  )
  regressionTest <- structure(list(name = structure(c(10L, 24L, 36L, 20L, 23L, 34L, 13L, 4L, 14L, 
                                                      8L, 29L, 33L, 5L, 6L, 21L, 22L, 7L, 28L, 26L, 
                                                      19L, 3L, 20L, 25L, 15L, 31L, 16L, 1L, 10L, 30L, 
                                                      11L, 9L, 4L, 27L, 12L, 32L, 17L, 35L, 18L, 2L), 
                                                    .Label = c("Abby", "Alisha", "Andy", "Caitlin", "Cal", 
                                                               "Cam", "Caroline", "Damon", "Danielle", "David",
                                                               "Dwayne", "Dylan", "Edward", "Eleanor", "Erica", "Glen", "Hannah",
                                                               "Hayley", "Heather", "Josh", "Julie", "Kate", "Lang", "Lars",
                                                               "Laura", "Lee", "Leigh", "Maggie", "Mark", "Mike", "Peggy", "Peter", 
                                                               "Ray", "Scotty", "Teshanna", "Zach"), class = "factor"), 
                                   birthmonth = c(5L,  10L, 12L, 1L, 2L, 3L, 2L, 6L, 5L, 9L, 9L, 3L, 8L, 3L, 11L, 4L,  12L, 3L, 6L, 
                                                  3L, 6L, 7L, 9L, 9L, 10L, 7L, 2L, 12L, 11L, 8L, 6L, 7L, 3L, 4L, 4L, 3L, 3L, 1L, 9L), 
                                   birthyear = c(88L, 87L, 87L, 88L, 88L, 88L, 88L, 88L, 88L, 88L, 87L, 88L, 87L, 88L, 87L, 88L, 87L, 
                                                 88L, 88L, 88L, 88L, 88L, 88L, 88L, 88L, 88L, 88L, 87L, 88L, 88L, 88L, 88L, 88L, 88L, 
                                                 88L, 88L, 88L, 88L, 88L), 
                                   length = c(24.4,  25.4, 24.5, 25.2, 25.1, 25.7, 26.1, 23, 23.6, 22.9, 27.5, 24.8,  26.1, 27, 26, 
                                              23.7, 24, 24.7, 26.7, 25.5, 24, 24.4, 24, 24.5,   24.2, 27.1, 26.1, 25.5, 24.2, 23.9, 
                                              24, 22.5, 24.5, 23.6, 24.7,   22.9, 26, 21.6, 24.6), 
                                   width = c(8.4, 8.8, 9.7, 9.8, 8.9, 9.7,  9.6, 8.8, 9.3, 8.8, 9.8, 8.9, 9.1, 9.8, 9.3, 7.9, 8.7, 8.8, 
                                             9,  9.5, 9.2, 8.6, 8.3, 9, 8.1, 9.4, 9.5, 9.5, 8.9, 9.3, 9.3, 8.6, 8.6, 9, 8.6, 8.5, 9, 7.9, 8.8), 
                                   sex = structure(c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 2L, 2L, 1L, 
                                                     1L, 2L, 2L, 2L, 1L, 2L, 1L, 1L, 1L, 2L, 2L, 2L, 1L, 1L,  2L, 2L, 2L, 2L), 
                                                   .Label = c("B", "G"), 
                                                   class = "factor"), 
                                   biggerfoot = structure(c(1L,    1L, 2L, 1L, 1L, 2L, 1L, 1L, 2L, 2L, 2L, 1L, 1L, 1L, 1L, 2L, 2L,2L, 
                                                            1L, 2L, 2L, 1L, 2L, 1L, 1L, 1L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 2L, 1L, 1L, 2L, 1L), 
                                                          .Label = c("L", "R"), 
                                                          class = "factor"), 
                                   domhand = structure(c(2L, 1L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 
                                                         1L, 2L, 2L, 2L, 2L, 2L, 2L, 1L, 2L, 1L, 2L, 2L, 2L, 1L, 2L, 
                                                         2L, 2L, 2L, 2L, 2L, 1L, 2L, 2L, 2L, 1L, 1L, 2L, 2L, 2L, 2L), 
                                                       .Label = c("L", "R"), class = "factor"), 
                                   biggerfoot2 = structure(c(2L, 1L, 1L, 2L, 2L, 1L, 2L, 2L, 1L, 2L, 1L, 2L, 2L, 2L, 2L, 1L,
                                                             2L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 2L, 2L, 1L, 2L, 2L, 2L, 1L, 
                                                             2L, 2L, 2L, 2L, 2L, 1L, 2L), .Label = c("dom", "nondom"), 
                                                           class = "factor")), 
                              class = "data.frame", 
                              row.names = c(NA, -39L))
  
  
  testcase1 <- structure(c(2L, 11L, 20L, 6L), 
                         .Dim = c(2L, 2L), 
                         .Dimnames = list(biggerfoot = c("L", "R"), 
                                          biggerfoot2 = c("dom", "nondom")), 
                         class = "table")
  testcase2 <- structure(c(2L, 6L, 20L, 11L), 
                         .Dim = c(2L, 2L), 
                         .Dimnames = list(biggerfoot = c("L", "R"), 
                                          domhand = c("L", "R")), 
                         class = "table")
  
  testcase3 <- structure(c(abstinent = 68L, 
                           moderate = 28L, 
                           highrisk = 357L), 
                         .Dim = 3L, 
                         .Dimnames = list(drink_status = c("abstinent", "moderate", "highrisk")), 
                         class = "table")
  
  testcase4 <- structure(c(abstinent = 68L, moderate = 28L, highrisk = 357L), .Dim = 3L, .Dimnames = list(
    drink_status = c("abstinent", "moderate", "highrisk")), class = "table")
  
  testcase5 <- structure(c(abstinent = 68L, moderate = 28L, highrisk = 357L), .Dim = 3L, .Dimnames = list(
    drink_status = c("abstinent", "moderate", "highrisk")), class = "table")
  
  expect_equal(ignore_attr = TRUE, regressionTest, Kf)
  expect_equal(ignore_attr = TRUE, testcase1, tally( ~ biggerfoot + biggerfoot2, data = Kf) )
  expect_equal(ignore_attr = TRUE, testcase2, tally( ~ biggerfoot + domhand, data = Kf) )
  expect_equal(ignore_attr = TRUE, testcase3, tally( ~ drink_status, data = modHELP) )
  expect_equal(ignore_attr = TRUE, testcase4, tally( ~ drink_status, data = modHELP2) )
  expect_equal(ignore_attr = TRUE, testcase5, tally( ~ drink_status, data = modHELP3) )
})

#'
#' Fit a nonlinear least squares model
#'
#' Allows you to specify a formula with parameters, along with starting
#' guesses for the parameters.  Refines those guesses to find the 
#' least-squares fit.
#'
#' @return a function
#' 
#' @param formula formula specifying the model
#' @param data dataframe containing the data to be used
#' @param start passed as `start` to [nls()].  If and empty list,
#' a simple starting point is used (thus avoiding the usual warning message).
#' @param \dots additional arguments passed to [nls()]
#' @param object an R object (typically a the result of fitModel)
#' 
#' @details
#' Fits a nonlinear least squares model to data.  In contrast
#' to linear models, all the parameters (including linear ones)
#' need to be named in the formula.  The function returned 
#' simply contains the formula together with pre-assigned 
#' arguments setting the parameter value.  Variables used in the 
#' fitting (as opposed to parameters) are unassigned arguments 
#' to the returned function.  
#' @note
#' This doesn't work with categorical explanatory variables.  Also, 
#' this does not work with synthetic data that fit the model perfectly.
#' See \code{link{nls}} for details.
#'
#' @seealso [linearModel()], [nls()]
#'
#' @examples
#' if (require(mosaicData)) {
#' f <- fitModel(temp ~ A+B*exp(-k*time), data=CoolingWater, start=list(A=50,B=50,k=1/20))
#' f(time=50)
#' coef(f)
#' summary(f)
#' model(f)
#' }
#' @export

fitModel <- function(formula, data=parent.frame(), start=list(), ...) {
  argsAndParams <- all.vars(rhs(formula))    # [-1]
  response <- eval(lhs(formula), data)
  n <- length(response)
  for ( nm in setdiff(argsAndParams, names(start)) ) {
  	  x <- tryCatch(get( nm, data), error=function(e) {list()} )  
	  if (length (x)  != n && nm != "pi") start[[nm]] <- 1
  }

  model <- nls(formula, data=data, start=start, ... )
  result <- makeFun(model)
  class(result) <- c("nlsfunction", class(result))
  return(result)
}

#' @rdname fitModel
#' @export
model <- function(object, ...) {
  UseMethod('model')
}

#' @rdname fitModel
#' @export
model.nlsfunction <- function(object, ...) {
  as.list(environment(object))$model
}

#' @rdname fitModel
#' @export
  summary.nlsfunction <-  function(object, ...) {
  summary( model( object), ... )
}

#' @rdname fitModel
#' @export
coef.nlsfunction <- function(object, ...) {
  coef( model(object), ... )
}

# context('fitModel()')


test_that("A function is created", {
  expect_true(is.function(fitModel( width ~ A * length + B, data=mosaicData::KidsFeet)))
  expect_equal(ignore_attr = TRUE, names(formals(fitModel( width ~ A * length + B, data=mosaicData::KidsFeet))), 
                    c('length','...','transformation'))
})

test_that("Function gives correct results", {
  formula <- width ~ A * length + B
  f <- fitModel( formula, data=mosaicData::KidsFeet )
  model <- nls(formula, data=mosaicData::KidsFeet, start=list(A=1, B=1)) 
  expect_equal(ignore_attr = TRUE,  f(mosaicData::KidsFeet$length), fitted(model) )
})

#' Frequency Polygons
#' 
#' Frequency polygons are an alternative to histograms that make it simpler to overlay multiple
#' distributions.
#'
#' @param x a formula or a numeric vector
#' @param \dots additional arguments passed on to [histogram()] 
#' and `panel`.
#' @param panel a panel function
#' @param prepanel a prepanel function
#'
#' @return a trellis object
#' @note This function make use of `histogram` to determine overall layout.  Often 
#' this works reasonably well but sometimes it does not. In particular, when `groups` is
#' used to overlay multiple frequency polygons, there is often too little head room.  
#' In the latter cases, it may be 
#' necessary to use `ylim` to determine an appropriate viewing rectangle for the 
#' plot.
#' 
#' @examples
#' freqpolygon(~age | substance, data=HELPrct, v=35)
#' freqpolygon(~age, data=HELPrct, labels=TRUE, type='count')
#' freqpolygon(~age | substance, data=HELPrct, groups=sex)
#' freqpolygon(~age | substance, data=HELPrct, groups=sex, ylim=c(0,0.11))
#' ## comparison of histogram and frequency polygon
#' histogram(~eruptions, faithful, type='density', width=.5)
#' ladd( panel.freqpolygon(faithful$eruptions, width=.5 ))
#' @export

freqpolygon <- function(x, 
                        ..., 
                        panel="panel.freqpolygon",
                        prepanel="prepanel.default.freqpolygon"
) {
  densityplot(x, ..., panel=panel, prepanel = prepanel)
}

#' @rdname freqpolygon

#' @export
#' 
prepanel.default.freqpolygon <- function(
  x, darg = list(), plot.points = FALSE, ref = FALSE,
  groups = NULL, subscripts = TRUE, 
  jitter.amount = 0.01 * diff(current.panel.limits()$ylim), 
  center = NULL, nint = NULL, breaks=NULL, width = darg$width, type = "density",
  ...) 
{
  if (!is.numeric(x)) 
    x <- as.numeric(x)
  if (missing(breaks) || is.null(breaks)) {
    breaks <- xhistogramBreaks(x, center=center, width=width, nint=nint)
  } 
  if (is.function(breaks) || is.character(breaks)) {
    breaks <- do.call(breaks, list(x=x, center=center, width=width, nint=nint, ...) )
  }
  
  if (sum(!is.na(x)) < 1) 
    prepanel.null()
  else if (sum(!is.na(x)) == 1) {
    list(xlim = rep(x, 2), ylim = rep(0, 2), dx = 1, dy = 1)
  }
  else if (is.null(groups)) {
    h <- hist(x, plot = FALSE, breaks=breaks, warn.unused=FALSE, ...)
    h$height <- 
      switch(
        type,
        'density' = h$density,
        'count' = h$count,
        'percent' = 100 * h$count / length(x),
        h$density
      )
    quants <- quantile(x, c(0.15, 0.85), names = FALSE, na.rm = TRUE)
    ok <- h$mids > quants[1] & h$mids < quants[2]
    list(xlim = range(h$mids), ylim = range(h$height), dx = diff(h$mids[ok]), 
         dy = diff(h$height[ok]))
  } else {
    vals <- sort(unique(groups))
    xl <- range(x, finite = TRUE)
    yl <- 0
    dxl <- numeric(0)
    dyl <- numeric(0)
    for (i in seq_along(vals)) {
      id <- (groups[subscripts] == vals[i])
      if (sum(id, na.rm = TRUE) > 1) {
        
        h <- do.call(hist, c(list(x = x[id], plot = FALSE, breaks = breaks, warn.unused = FALSE))) 
        h$height <- 
          switch(
            type,
            'density' = h$density,
            'count' = h$count,
            'percent' = 100 * h$count / length(x),
            h$density
          )
        xl <- c(xl, h$mids)
        yl <- c(yl, h$height)
        quants <- quantile(x[id], c(0.15, 0.85), names = FALSE, na.rm = TRUE)
        ok <- h$mids > quants[1] & h$mids < quants[2]
        dxl <- c(dxl, diff(h$mids[ok]))
        dyl <- c(dyl, diff(h$height[ok]))
      }
    }
    list(xlim = range(xl, finite = TRUE), 
         ylim = range(yl, finite = TRUE), dx = dxl, dy = dyl)
  }
}

#' Turn histograms into frequency polygons
#' 
#' Turn histograms into frequency polygons
#' 
#' @rdname freqpoly
#' @param x a vector of values for which a frequency polygon is desired.
#' @param plot a logical indicating if a plot should be generated.
#' @param ... additional arguments passed on to [hist()].
#' @return An object of class `"freqpoly"` (invisibly).  Additionally, if `plot` is 
#' `TRUE`, a plot is generated.
#' @export
#' @examples
#' freqpoly(faithful$eruptions)
#' bks <- c(0, 1, 1.5, 2, 3, 3.5, 4, 4.5, 5, 7)
#' hist(faithful$eruptions, breaks = bks)
#' freqpoly(faithful$eruptions, col = rgb(0,0,1,.5), lwd = 5, breaks = bks, add = TRUE)
#' 
freqpoly <- function(x, plot = TRUE, ...) {
  h <- hist(x, plot = FALSE, warn.unused = FALSE, ...)
  res <- hist2freqpolygon(h)
  res$xname = rlang::expr_deparse(substitute(x))
  dots <- list(...)
  # remove from ... things that plot() might not like.
  bad_args <- c("breaks", "include.lowest", "right", "density", "angle", 
                 "border", "warn.unused", "nclass")
  pm <- pmatch(names(dots), bad_args)
  dots[!is.na(pm)] <- NULL
  
  if (plot) do.call("plot", c(list(res), dots))
  invisible(res)
}

#' @export
#' @rdname freqpoly
#' @param hist a histogram object produced by \code{link{hist}()}.
#' 
hist2freqpolygon <- function(hist) {
  res <- hist
  if (hist$counts[1] > 0) {
    res$counts <- c(0, res$counts)
    res$density <- c(0, res$density)
    res$mids <- c(res$mids[1] - diff(res$mids[1:2]), res$mids)
    res$breaks <- c(res$breaks[1] - 2 * (res$breaks[1] - res$mids[1]), res$breaks)
  }
  
  if (tail(hist$counts, 1) > 0) {
    res$counts <- c(res$counts, 0)
    res$density <- c(res$density, 0)
    nbreaks <- length(res$breaks)
    res$mids <- c(res$mids, res$mids[nbreaks-1] + diff(res$mids[(nbreaks-2):(nbreaks-1)]))
    res$breaks <- c(res$breaks, res$breaks[nbreaks] + 2 * (res$mids[nbreaks] - res$breaks[nbreaks]))
  }
  
  width <- diff(res$breaks)  # one for each mid
  ratio <- tail(width, -1) / (tail(width, -1) + head(width, -1))
  brk_density <- head(res$density, -1) + ratio * diff(res$density)
  brk_counts <- head(res$counts, -1) + ratio * diff(res$counts)
  
  res <-
    tibble::tibble(
      x = c(res$mids, head(tail(res$breaks, -1), -1)), 
      density = c(res$density, brk_density),
      counts = c(res$counts, brk_counts)) |> 
    arrange(x) |> 
    as.list()
  res$equidist <- hist$equidist
  res$xname <- hist$xname
  class(res) <- "freqpolygon"
  res
}

#' @rdname freqpoly
#' @param freq A logical indicating whether the vertical scale should be frequency (count).
#' @param col A color for the frequency polygon.
#' @param lty An integer indicating the line type.
#' @param lwd An integer indicating the line width.
#' @param main A title for the plot.
#' @param sub A sub-title for the plot.
#' @param xlab Label for the horizontal axis.
#' @param ylab Label for the vertical axis.
#' @param xlim A numeric vector of length 2.
#' @param ylim A numeric vector of length 2.
#' @param axes A logical indicating whether axes should be drawn.
#' @param labels A logical indicating whether labels should be printed or a character
#' vector of labels to add.
#' @param add A logical indicating whether the plot should be added to the current plot
#' @param ann A logical indicating whether annotations (titles and axis titles) should be plotted.
#' @export
plot.freqpolygon <- 
  function (x, freq = equidist, col = graphics::par("fg"), lty = NULL, lwd = 1, 
            main = paste("Frequency polygon of",  paste(x$xname, collapse = "\n")), 
            sub = NULL, xlab = x$xname, ylab, xlim = range(x$x), 
            ylim = NULL, axes = TRUE, labels = FALSE, 
            add = FALSE, ann = TRUE, ...) 
  {
    equidist <- if (is.logical(x$equidist)) 
      x$equidist
    else {
      h <- diff(x$breaks)
      diff(range(h)) < 1e-07 * mean(h)
    }
    
    if (freq && !equidist) 
      warning("the AREAS in the plot are wrong -- rather use 'freq = FALSE'")
    y <- if (freq) x$counts else x$density
    nB <- length(x$x)
    if (is.null(y) || 0L == nB) 
      stop("'x' is wrongly structured")
    grDevices::dev.hold()
    on.exit(grDevices::dev.flush())
    if (!add) {
      if (is.null(ylim)) 
        ylim <- range(y, 0)
      if (missing(ylab)) 
        ylab <- if (!freq) 
          "Density"
      else "Frequency"
      graphics::plot.new()
      graphics::plot.window(xlim, ylim, "", ...)
      if (ann) 
        graphics::title(main = main, sub = sub, xlab = xlab, ylab = ylab, 
              ...)
      if (axes) {
        graphics::axis(1, ...)
        graphics::axis(2, ...)
      }
    }
    # rect(x$breaks[-nB], 0, x$breaks[-1L], y, col = col, border = border, 
    #      angle = angle, density = density, lty = lty)
    graphics::lines(x$x, y, col = col, lty = lty, lwd = lwd) 
    if ((logl <- is.logical(labels) && labels) || is.character(labels)) 
      graphics::text(x$x[c(TRUE, FALSE)], y[c(TRUE, FALSE)], labels = if (logl) {
        if (freq) 
          x$counts[c(TRUE, FALSE)]
        else round(x$density[c(TRUE, FALSE)], 3)
      }
      else labels, adj = c(0.5, -0.5))
    invisible()
  }

#' @rdname freqpolygon
#' @param plot.points one of `TRUE`, `FALSE`, `"jitter"`, or `"rug"` indicating
#' how points are to be displayed
#' @param gcol color of guidelines
#' @param glwd width of guidelines
#' @param groups,weights,jitter.amount,identifier as in [lattice::densityplot()] 
#' or [histogram()]
#' @param type one of `'density'`, `'percent'`, or `'count'`
#' @param breaks a vector of breaks for the frequency polygon bins
#' @param nint an approximate number of bins for the frequency polygon
#' @param center center of one of the bins
#' @param width width of the bins
#' @param h,v a vector of values for additional horizontal and vertical lines
#' @param ref a logical indicating whether a horizontal reference line should be 
#' added (roughly equivalent to `h=0`)
#' @param darg a list of arguments for the function computing the frequency polygon.
#'   This exists primarily for compatibility with `densityplot` and is unlikely
#'   to be needed by the end user.
#' @param subscripts as in other lattice prepanel functions
#' @export

panel.freqpolygon <- 
  function (
    x, darg=list(),
    plot.points = FALSE, ref = FALSE, 
    groups = NULL, weights = NULL, 
    jitter.amount = 0.01 * diff(current.panel.limits()$ylim), 
    type='density', 
    breaks=NULL, 
    nint= NULL,
    center=NULL, 
    width = darg$width,
    gcol=trellis.par.get('reference.line')$col,
    glwd=trellis.par.get('reference.line')$lwd,
    h, v, 
    ..., identifier = "freqpoly") 
  {
    if (missing(breaks) || is.null(breaks)) {
      breaks <- xhistogramBreaks(x, center=center, width=width, nint=nint)
    } 
    if (is.function(breaks) || is.character(breaks)) {
      breaks <- do.call(breaks, list(x=x, center=center, width=width, nint=nint, ...) )
    }
    
    if (ref) {
      reference.line <- trellis.par.get("reference.line")
      panel.abline(h = 0, col = reference.line$col, lty = reference.line$lty, 
                   lwd = reference.line$lwd, identifier = paste(identifier, "abline"))
    }
    if (!is.null(groups)) {
      return(panel.superpose(x, darg = darg, plot.points = plot.points, 
                             ref = FALSE, groups = groups, 
                             panel.groups = panel.freqpolygon, jitter.amount = jitter.amount, 
                             type = type, breaks=breaks, nint=nint, ...))
    }
    else {
      switch(as.character(plot.points), 
             `TRUE` = panel.xyplot(x = x, y = rep(0, length(x)), type = type, ..., identifier = identifier), 
             rug = panel.rug(x = x, start = 0, end = 0, 
                             x.units = c("npc", "native"), type = type, ..., 
                             identifier = paste(identifier, "rug")), 
             jitter = panel.xyplot(x = x, y = jitter(rep(0, length(x)), amount = jitter.amount), 
                                   type = 'p', ..., identifier = identifier))                                     
      
    }    
    hist.master <- 
      hist2freqpolygon(hist(as.numeric(x), plot = FALSE, breaks=breaks, warn.unused=FALSE, ...))
    
    hist.master$height <- switch(type,
                                 'density' = hist.master$density,
                                 'count' = hist.master$counts,
                                 'percent' = 100 * hist.master$counts / length(x),
                                 hist.master$density
    )
    
    if (!missing(v)) {
      for (x in v) {
        panel.abline(v = x, col = gcol, lwd = glwd)
      }
    }
    if (!missing(h)) {
      for (y in h) {
        panel.abline(h = y, col = gcol, lwd = glwd)
      }
    }  
    panel.xyplot(x=hist.master$x, 
                 y=hist.master$height,
                 default.units='native',
                 type='l', ...
    )
    
  }

# copied from lattice because it isn't exported there.
prepanel.null <- function () 
{
  list(xlim = rep(NA_real_, 2), ylim = rep(NA_real_, 2), dx = NA_real_, 
       dy = NA_real_)
}

# context("frequencypolygon")

testthat::test_that("freqpolygon() examples are working",{
 
  wrapped_expect_doppelganger(
    "FirstExample", 
    freqpolygon(~age | substance, data = HELPrct, v = 35))
  
  wrapped_expect_doppelganger(
    "SecondExample", 
    freqpolygon(~age, data = HELPrct, labels = TRUE, type = 'count'))
  
  wrapped_expect_doppelganger(
    "ThirdExample", 
    freqpolygon(~age | substance, data = HELPrct, groups = sex))
  
  wrapped_expect_doppelganger(
    "FourthExample", 
    freqpolygon(~age | substance, data = HELPrct, groups = sex, ylim = c(0,0.11)))
})

#' Lattice Theme
#' 
#' A theme for use with lattice graphics.
#'
#' @param bw whether color scheme should be "black and white"
#' @param lty vector of line type codes
#' @param lwd vector of line widths
#' @param ... additional named arguments passed to 
#'   [lattice::trellis.par.set()]
#' 
#' @return Returns a list that can be supplied as the `theme` to 
#' [lattice::trellis.par.set()].
#' @note
#'   These two functions are identical.  `col.mosaic` is named 
#' similarly to [lattice::col.whitebg()], but since more 
#' than just colors are set, `theme.mosaic` is a preferable name.
#'
#'
#' @seealso [lattice::trellis.par.set()], [lattice::show.settings()] 
#' 
#' @rdname themes
#'
#' @examples
#' trellis.par.set(theme=theme.mosaic())
#' show.settings()
#' trellis.par.set(theme=theme.mosaic(bw=TRUE))
#' show.settings()
#' 
#' @keywords graphics 
#' @export

theme.mosaic <-
  function (bw = FALSE, lty = if (bw) 1:7 else 1, lwd = 2.0, ...) 
  {
    aBlue <- colorRampPalette(c("white", "navy"))(10)[8]
    paleBlue <- colorRampPalette(c("white", "navy"))(10)[6]
    lightBlue <- colorRampPalette(c("white", "steelblue"))(10)[5]
    veryLightBlue <- colorRampPalette(c("white", "steelblue"))(12)[3]
    darkBlue <- colorRampPalette(c("white", "navy"))(10)[9]
    paleGreen <- colorRampPalette(c("white", "darkGreen"))(10)[8]
    if (bw) {
      res <- list(background = list(col = "transparent"), axis.line = list(col = "gray30"), 
                  axis.text = list(col = "gray30"), plot.polygon = list(col = "gray80"), 
                  box.rectangle = 
                    list(col = "gray10"), box.umbrella = list(col = "gray10", 
                                                              lty = 1), box.dot = list(col = "gray10"), dot.line = list(col = "gray50"), 
                  dot.symbol = 
                    list(col = "gray30", pch = 16), plot.line = list(col = "black", 
                                                                     lwd = lwd), plot.symbol = list(col = "black", fill = "gray80", 
                                                                                                  pch = 16), regions = list(col = gray((1:100)/100)), 
                  reference.line = 
                    list(col = "gray50"), add.line = list(lty = 1,  col = "gray80", lwd = lwd), 
                  superpose.polygon = 
                    list(col = c("gray30", "gray70", "black", "gray50", "gray20", "gray80", 
                                 "gray60", "gray40"), fill = c("gray80")), 
                  superpose.line = 
                    list(lty = lty, lwd = lwd, 
                         col = c("gray30", "gray70", "black", 
                                 "gray50", "gray20", "gray80", "gray60", "gray40")), 
                  superpose.symbol = 
                    list(pch = c(16, 15, 18, 1, 3, 6, 0, 5), 
                         cex = rep(0.7, 7), 
                         col = c("gray30","gray70", "black", "gray50", "gray20", "gray80", 
                                 "gray60", "gray40")),
                  par.strip.text = list(cex = 0.5, col = c("gray60", "gray30")) 
      )
    } else {
      res <- list(background = list(col = "transparent"), 
                  plot.polygon = list(col = paleBlue), 
                  superpose.polygon = 
                    list(
                    col = c(aBlue, 
                            "lightskyblue3", "darkgreen", "tan", "orange", 
                            "purple", "lightgreen")), 
                  box.rectangle = 
                    list(col = darkBlue), box.umbrella = list(col = darkBlue), 
                  dot.line = list(col = "#e8e8e8"), 
                  dot.symbol = 
                    list(col = darkBlue, pch = 16), 
                  plot.line = 
                    list(lwd = lwd, col = darkBlue), 
                  plot.symbol = 
                    list(col = darkBlue, pch = 16), 
                  regions = list(col = heat.colors(100)), 
                  reference.line =  list(col = "#e8e8e8"), 
                  add.line = list(lty = 1, col = "gray20", lwd = lwd), 
                  superpose.line = 
                    list(lty = lty, 
                         col = c(darkBlue, "lightskyblue3", "darkgreen",  
                                 "tan", "orange", "purple", "pink", "lightgreen")), 
                  superpose.symbol = 
                    list(pch = c(16, 15, 18, 1, 3,  6, 0, 5), 
                         cex = rep(0.7, 7), 
                         col = c(darkBlue, "lightskyblue3", "darkgreen", "tan", "orange", 
                                                  "purple", "pink", "lightgreen")), 
                  strip.background = 
                    list(alpha = 1, 
                         col = c("#ffe5cc", veryLightBlue, "#ccffff", 
                                 "#cce6ff", "#ffccff", "#ffcccc", "#ffffcc")), 
                  strip.shingle = 
                    list(alpha = 1, 
                         col = c("#ff7f00", 
                                 darkBlue, "#00ffff", "#0080ff", "#ff00ff", "#ff0000", 
                                 "#ffff00")), 
                  par.strip.text = list(cex = 0.5)
      )
    }
    res <- c(res, list(...))
    res
  }

#' @rdname themes
#' @export

col.mosaic <- theme.mosaic

#' Augmented version of `qqmath`
#'
#' 
#' @param x,data,panel,\dots  as in [lattice::qqmath()]
#'
#' @return a trellis object
#'
#' @examples
#' x <- rnorm(100)
#' xqqmath( ~ x)                  # with quartile line
#' xqqmath( ~ x, fitline = TRUE)  # with fitted line
#' xqqmath( ~ x, idline = TRUE)   # with y = x
#' x <- rexp(100, rate = 10)
#' xqqmath( ~ x, distribution = qexp)     # with quartile line
#' xqqmath( ~ x, distribution = qexp, slope = 1/10) 
#' xqqmath( ~ x, distribution = qexp, slope = mean(x)) 
#' @export

xqqmath <-
  function (x, data = NULL, panel = "panel.xqqmath", ...) 
  {
    qqmath(x, data = data, panel = panel, ...)
  }

#' @rdname xqqmath
#' @param qqmathline a logical: should line be displayed passing through first and third quartiles?
#' @param idline a logical; should the line y=x be added to the plot?
#' @param fitline a logical; should a fitted line be added to plot?  Such a line will use `slope`
#'        and `intercept` if provided, else the standard deviation and mean of the data.
#'        If `slope` is specified, the line will be added unless `fitline` is 
#'        `FALSE`.
#' @param slope slope for added line
#' @param intercept intercept for added line 
#' @param overlines a logical: should lines be on top of qq plot?
#' @param groups,pch,lwd,lty as in lattice plots
#' @param col.line color to use for added lines
#'
#' @export

panel.xqqmath <-
  function (x, qqmathline = !(fitline || idline), idline = FALSE, 
            fitline = NULL, slope = NULL, intercept = NULL, overlines = FALSE, 
            groups = NULL, ..., col.line = trellis.par.get("add.line")$col, 
            pch = 16, lwd = 2, lty = 2) 
  {
    if (!is.null(groups)) {
      panel.superpose(x, groups = groups, panel.groups = "panel.xqqmath", 
                      qqmathline = qqmathline, idline = idline, fitline = fitline, 
                      intercept = intercept, slope = slope, overlines = overlines, 
                      ..., col.line = col.line, pch = pch, lwd = lwd, lty = lty)
    }
    else {
      if( !is.null(slope) ) {
        if (is.null(fitline)) fitline <- TRUE
        if (is.null(intercept)) intercept <- 0
      }
      lty <- rep(lty, length = 3)
      if (overlines) {
        panel.qqmath(x, ..., pch = pch)
      }
      if (idline) {
        panel.abline(0, 1, col.line = col.line, lty = lty[3], lwd = lwd)
      }
      if (is.null(fitline)) fitline <- FALSE
      if (fitline) {
        if (is.null(slope)) {
          slope = sd(x, na.rm=TRUE)
        }
        if (is.null(intercept)) {
          intercept = mean(x, na.rm=TRUE)
        }
        panel.abline(intercept, slope, col.line = col.line, 
                     lty = lty[2], lwd = lwd)
      }
      if (qqmathline) {
        panel.qqmathline(x, col.line = col.line, lty = lty[1], 
                         lwd = lwd, ...)
      }
      if (!overlines) {
        panel.qqmath(x, ..., pch = pch)
      }
    }
  }

#' Defunct functions
#' 
#' The following functions were once a part of the mosaic pacakge but have
#' been removed.  In some cases, an alternative is available and is suggested
#' if you attempt to execute the function.
#' 
#' @rdname defunct
#' @param ... arguments, ignored since the function is defunct
#' @export

perctable <- function(...) 
{
  .Defunct("tally")
}

#' @rdname defunct
#' @export

proptable <- function(...) 
{
  .Defunct("tally")
}

#' Rescale 
#' 
#' Rescale vectors or variables within data frames.  This can be useful
#' for comparing vectors that are on different scales, for example in
#' parallel plots or heatmaps.
#' @rdname rescale
#' @param x an R object to rescale
#' @param domain a numeric vector of length 2 or `NULL`
#' @param range a numeric vector of length 2
#' @param ... additional arguments
#' @export 
rescale <- function(x, range, domain = NULL, ...) {
  UseMethod("rescale")
}

#' @rdname rescale
#' @export
rescale.data.frame <- function( x, 
                                range = c(0,1), 
                                domain = NULL,
                                ...) {
  dfapply( x, 
           function(x) { rescale(x, range = range, domain = domain, ...) },
           function(x) TRUE ) |>
    as.data.frame()
}

#' @rdname rescale
#' @export
rescale.factor <- function(x, range, domain = NULL, ...) {
  if( is.null(domain)) domain <- range(1:nlevels(x))
  rescale( as.numeric(x), domain = domain, range = range)
#  width <- diff(range)
#  n <- length(levels(x)) - 1
#  range[1]  - 1/n + width * as.numeric(x) / n
}

#' @rdname rescale
#' @export
rescale.numeric <- function(x, range = c(0,1), domain = NULL, ...) {
  if (is.null(domain)) domain <- range(x, na.rm = TRUE)
  range_width  <- diff(range)
  domain_width <- diff(domain)
  range[1] + range_width * (x - min(x)) / domain_width
}

#' @rdname rescale
#' @export
rescale.default <- function(x, range = c(0,1), domain = NULL, ...) {
  rescale( as.numeric(x, range = range, domain = domain, ...) )
}

#' @rdname rescale
#' @export
rescale.character <- function(x, range=c(0,1), domain = NULL, ...) {
  rescale( as.factor(x), range = range, domain = domain)
}

globalVariables(c("FUNCTION_TBD", "NA.RM", "OUTPUT.MULTIPLE"))
require(rlang)

#' 1-ary Aggregating functions
#' 
#' `aggregatinFuntion1` creates statistical summaries of one numerical vector that are formula aware.
#' 
#' @import rlang
#' @rdname aggregatingFunction1
#' @aliases aggregatingFunction1 
#' @param fun a function that takes a numeric vector and computes a summary statistic,
#' returning a numeric vector.
#' @param output.multiple a boolean indicating whether `fun` returns multiple values
#' @param envir an environment in which evaluation takes place.
#' @param na.rm the default value for na.rm in the resulting function.
#' @param style one of `"formula1st"`, `"formula2nd"` or `"flexible"`.  In the first
#' two cases, the first argument must be a formula or evaluate to an object.  In the latter case, 
#' bare names will be converted into formulas.
#' @return a function that generalizes `fun` to handle a formula/data frame interface.
#' 
#' @details The logic of the resulting function is this: 1) If the first argument is a formula,
#' use that formula and `data` to create the necessary call(s) to `fun`; (2) Else simply
#' pass everything to `fun` for evaluation.
#' 
#' @section Note: 
#' Earlier versions of this function supported a "bare name + data frame" interface.  This
#' functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary, and (c) 
#' difficult to maintain.
#' 
#' @examples
#' if (require(mosaicData)) {
#'   foo <- aggregatingFunction1(base::mean)
#'   foo( ~ length, data = KidsFeet)
#'   base::mean(KidsFeet$length)
#'   foo(length ~ sex, data = KidsFeet)
#' } 
#' @export
# 

# Testing a possible replacement for aggregatingFunction1
# Might be masked below by the old one -- be sure to check.

aggregatingFunction1 <- 
  function(fun, 
           output.multiple = FALSE, 
           envir = parent.frame(), 
           na.rm = getOption("na.rm", FALSE),
           style = c("formula1st", "formula", "flexible")
  ) {
    fun <- deparse(substitute(fun))
    style = match.arg(style)
    templates <- list(
      flexible = # this should probably be removed in the near future.
        function(
          x, ..., 
          data = NULL,
          groups = NULL, 
          na.rm = NA.RM)  ## getOption("na.rm", FALSE)) 
        {
          pframe <- parent.frame()
          subst_x <- substitute(x)
          lazy_formula <- 
            tryCatch(
              rlang::enquo(x),
              error = function(e) {
                if (grepl("Promise has already been forced", e$message) ||
                    # next line can be deleted when lazyeval updates on CRAN
                    grepl("e of NULL environment", e$message) 
                ) 
                  NULL
                else 
                  stop(e)
              }
            )
          ## See issue #531 for problem with using lazy_formula.
          if (is.null(lazy_formula) || ! inherits(lazy_formula$expr, "formula")) {
            lazy_formula <- structure(list(expr = subst_x, env = pframe), class = "lazy")
          }
          if (is.null(data)) {
            result <-
              tryCatch(FUNCTION_TBD(x, ..., na.rm = na.rm), 
                       error = function(e) {e} , 
                       warning = function(w) {w}) 
            if (! inherits(result, c("error", "warning")))  return(result) 
            data <- parent.frame()
          }
          formula <- formularise(lazy_formula, parent.frame(2)) 
          formula <- 
            mosaicCore::mosaic_formula_q(formula, groups = !!rlang::enexpr(groups), max.slots = 3) 
          maggregate(formula, data = data, FUN = FUNCTION_TBD, na.rm = na.rm, ..., 
                     .multiple = output.multiple)
        },
      formula = 
        function(
          x, ..., 
          data = NULL,
          groups = NULL, 
          na.rm = NA.RM)  ## getOption("na.rm", FALSE)) 
        {
          if (is.null(data)) {
            result <-
              tryCatch(FUNCTION_TBD(x, ..., na.rm = na.rm), error = function(e) {e} , warning = function(w) {w} ) 
            if (! inherits(result, c("error", "warning")))  return(result) 
            data <- parent.frame()
          }
          if (! inherits(x, "formula")) stop("`x' must be a formula")
          formula <- 
            mosaicCore::mosaic_formula_q(x, groups = !!rlang::enexpr(groups), max.slots = 3) 
          maggregate(formula, data = data, FUN = FUNCTION_TBD, ..., .multiple = output.multiple)
        },
      
      formula1st = 
        function(
          x, ..., 
          data = NULL,
          groups = NULL, 
          na.rm = NA.RM)  ## getOption("na.rm", FALSE)) 
        {
          if (rlang::is_formula(x)) {
            if (is.null(data)) data <- environment(x)
            formula <- 
              mosaicCore::mosaic_formula_q(x, groups = !!rlang::enexpr(groups), max.slots = 3) 
            
            return(maggregate(formula, data = data, FUN = FUNCTION_TBD, ..., 
                              na.rm = na.rm,
                              .multiple = OUTPUT.MULTIPLE))
          }
          FUNCTION_TBD(x, ..., na.rm = na.rm)
        }
    )
    
    fun_text <- deparse(templates[[style]])
    fun_text <- gsub("FUNCTION_TBD", fun, fun_text) 
    fun_text <- gsub("OUTPUT.MULTIPLE", output.multiple, fun_text)
    if (missing(na.rm)) {
      fun_text <- gsub("NA.RM", deparse(substitute(getOption("na.rm", FALSE))), fun_text)
    } else {
      fun_text <- gsub("NA.RM", deparse(substitute(na.rm)), fun_text)
    }
    res <- eval(parse(text = fun_text))
    environment(res) <- parent.frame()
    res
  } 


# #' @export
# aggregatingFunction1or2 <- function(fun, input.multiple = FALSE, output.multiple = FALSE, 
#                                      envir = parent.frame(), na.rm = getOption("na.rm",FALSE)) {
#   result <- function(x, ..., data, groups = NULL) {
#     orig.call <- match.call()
#     fun.call <- orig.call 
#     fun.call[[1]] <- substitute(..fun..)
#     fun.call[[2]] <- substitute(x)
#     
#     # if data is not given, we will try to evaluate fun()
#     missingData <- FALSE  
#     if (missing(data)) {
#       missingData <- TRUE
#       data = parent.frame()
#       
#       result <- tryCatch(eval(fun.call, envir = parent.frame()) , 
#                           error = function(e) {e} ,
#                           warning = function(w) {w}) 
#       if ( ! inherits(result, "warning") && ! inherits(result,"error")) {
#         return(result) 
#       }
#     }
#     # either data was specified or fun() generated an error.
#     # so we will generate a new call.
#     maggregate.call <- orig.call  
#     x_name <- substitute(x)
#     if (! .is.formula(x)) {
#       if ( !missingData) {
#         fun.call[['data']] <- NULL
#       }
#       if (input.multiple) {
#         result <- tryCatch(eval(fun.call, envir = data, enclos = parent.frame()),
#                             error = function(e) {e},
#                             warning = function(w) {w}) 
#         if ( ! inherits(result, "warning") && ! inherits(result,"error")) {
#           return(result) 
#         }
#       }
#       
#       x <- eval(substitute( 
#         mosaicCore::mosaic_formula_q(.x, groups = quote(groups)), 
#         list(.x = substitute(x) , .g = substitute(groups))
#       ) )
#       
#       if ("groups" %in% names(maggregate.call)) maggregate.call[['groups']] <- NULL
#       
#     }
#     # now x is a formula
#     
#     maggregate.call[[1]] <- quote(maggregate)
#     maggregate.call$formula <- x
#     maggregate.call$data <- data 
#     maggregate.call$x <- NULL
#     maggregate.call$FUN <- substitute(..fun..)  # keep substitute here or no?
#     maggregate.call$.multiple <- output.multiple
#     maggregate.call$na.rm <- substitute(na.rm)
#     #    print(maggregate.call)
#     return(eval(maggregate.call, envir = envir))
#   }
#   formals(result) <- c(formals(result), ..fun.. = substitute(fun), na.rm = substitute(na.rm))
#   return(result)
# }

#' 1- or 2-ary aggregating functions
#' 
#' `aggregatingFunction1or2()` creates statistical summaries for functions like
#' [var()] that can have either 1 or 2 numeric vector inputs.
#' 
#' @param fun a function that takes 1 or 2 numeric vectors and computes a summary statistic,
#' returning a numeric vector of length 1.
#' @param na.rm the default value for na.rm in the resulting function.
#' @param output.multiple a boolean indicating whether `fun` returns multiple values

#' 
#' @details This was designed primarily to support `var` which can be used to compute
#' either the variance of one variable or the covariance of two variables.
#' The logic of the resulting function is this: 1) If the first two arguments are both formulas,
#' then those formulas are evaluated (with `data`) to compute the covariance; 
#' (2) If the first argument is a formula, and the second is `NULL`, 
#' then the formula and `data` are used to create the necessary call(s) to `fun`; 
#' (3) Else everything is simply passed to `fun` for evaluation.
#' 
#' @section Note: 
#' Earlier versions of this function supported a "bare name + data frame" interface.  This
#' functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary, 
#' and (c) difficult to maintain.
#' @export
aggregatingFunction1or2 <- 
  function(fun, output.multiple = FALSE, 
           na.rm = getOption("na.rm", FALSE)) {
    template <- 
      function(x, y = NULL, na.rm = NA.RM, ..., data = NULL) { 
        if (rlang::is_formula(x)) {
          if (rlang::is_formula(y)) {
            x <- rlang::eval_tidy(rlang::f_rhs(x), data, environment(x))
            y <- rlang::eval_tidy(rlang::f_rhs(y), data, environment(y))
          } else {
            formula <- mosaicCore::mosaic_formula_q(x, max.slots = 2)
            if (is.null(data)) data <- environment(formula)
            return(maggregate(
              formula,
              data = data,
              FUN = FUNCTION_TBD,
              .multiple = OUTPUT.MULTIPLE,
              na.rm = na.rm,
              ...
            ))
          }
        }
        FUNCTION_TBD(x, y, na.rm = na.rm, ...)
      }
    
    fun_name <- deparse(substitute(fun))
    fun_text <- deparse(template)
    fun_text <- gsub("FUNCTION_TBD", fun_name, fun_text) 
    fun_text <- gsub("OUTPUT.MULTIPLE", deparse(substitute(output.multiple)), fun_text) 
    if (missing(na.rm)) {
      fun_text <- gsub("NA.RM", deparse(substitute(getOption("na.rm", FALSE))), fun_text)
    } else {
      fun_text <- gsub("NA.RM", deparse(substitute(na.rm)), fun_text)
    }
    res <- eval(parse(text = fun_text))
    environment(res) <- parent.frame()
    res   
  }


#' 2-ary aggregating functions
#' 
#' `aggregatinFuntion2` creates statistical summaries of two numerical vectors that 
#' are formula aware.

#' @rdname aggregatingFunction2
#' @aliases aggregatingFunction2 
#' @param fun a function that takes two numeric vectors and computes a summary statistic,
#' returning a numeric vector of length 1.
#' @return a function that generalizes `fun` to handle a formula/data frame interface.
#' @details 
#' This was designed to support functions like `cov()` which can be used to compute
#' numerical summaries from two numeric vectors.
#' The logic of the resulting function is this: 1) If the first two arguments are both formulas,
#' then those formulas are evaluated (with `data`) to compute the covariance; 
#' (2) If the first argument is a formula, and the second is `NULL`, 
#' then the left and ride sides of the formula and `data` are used to create the 
#' vectors passed to `fun`;
#' (3) Else everything is simply passed to `fun` for evaluation.
#' 
#' @section Note: 
#' Earlier versions of this function supported a "bare name + data frame" interface.  This
#' functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary, 
#' and (c) difficult to maintain.
#' 
#' @examples
#' if(require(mosaicData)) {
#'   foo <- aggregatingFunction2(stats::cor)
#'   foo(length ~ width, data = KidsFeet)
#'   stats::cor(KidsFeet$length, KidsFeet$width)
#' }
#' @export

aggregatingFunction2 <- function(fun) {
  
  template <- 
    function(x, y = NULL, ..., data = NULL) { 
      
      tryCatch(y, error = function(e) {
        if (!is.null(data)) {
          stop("When `data' is specified, `y' should not be used.",
               call. = FALSE)
        }
      })
      
      if (!(is.null(y) || inherits(y, "formula")) && !is.null(data)) {
        stop("When `data' is specified, `y' should be a formula or NULL.", 
             call. = FALSE)
      }
      if (rlang::is_formula(x)) {
        if (rlang::is_formula(y)) {
          x <- rlang::eval_tidy(rlang::f_rhs(x), data, environment(x))
          y <- rlang::eval_tidy(rlang::f_rhs(y), data, environment(y))
        } else {
          formula <- mosaicCore::mosaic_formula_q(x, max.slots = 3)
          x <- rlang::eval_tidy(rlang::f_rhs(formula), data, environment(formula))
          y <- rlang::eval_tidy(rlang::f_lhs(formula), data, environment(formula))
        }
        FUNCTION_TBD(x, y, ...)
      }
      FUNCTION_TBD(x, y, ...)
    }
  
  fun_name <- deparse(substitute(fun))
  fun_text <- deparse(template)
  fun_text <- gsub("FUNCTION_TBD", fun_name, fun_text) 
  
  res <- eval(parse(text = fun_text))
  environment(res) <- parent.frame()
  res
}

# old version -- keep until we establish that new version is working properly.

# aggregatingFunction2 <- function(fun) {
#   result <- function(x, y = NULL, ..., data = parent.frame()) { # , ..fun.. = fun) {
#     orig.call <- match.call()
#     mosaic.call <- orig.call 
#     mosaic.call[[1]] <- fun
#     
#     if ( #"data" %in% names(orig.call) && 
#       ! .is.formula(eval(orig.call$x, parent.frame())))  {  
#       if (!'data' %in% names(formals(fun)) && ! "..." %in% names(formals(fun))) {
#         if("data" %in% names(mosaic.call)) mosaic.call[["data"]] <- NULL  # in case original function didn't have ...
#       }
#       return (eval(mosaic.call , data, enclos = parent.frame()))
#     }
#     
#     # message("Using mosaic super powers!")
#     formula <- eval(orig.call$x, parent.frame())
#     mosaic.call[['data']] <- NULL
#     # if (is.null(mosaic.call[['data']])) mosaic.call[['data']] <- quote(parent.frame())
#     mosaic.call$x <- eval(lhs(formula), envir = data, enclos = parent.frame())
#     mosaic.call$y <- eval(rhs(formula), envir = data, enclos = parent.frame())
#     if (! "..." %in% names(formals(orig.call))) {
#       for (n in setdiff(names(mosaic.call), names(formals(fun)))) {
#         if (n != "") mosaic.call[[n]] <- NULL
#       }
#     }
#     if (! is.null(condition(formula))) {
#       stop("Formula must be of the form y ~ x.")
#     }
#     return(eval(mosaic.call))
#   }
#   assign("fun", fun, environment(result))
#   return(result)
# }


#' Aggregating functions
#' 
#' The `mosaic` package makes several summary statistic functions (like `mean` and `sd`)
#' formula aware.
#' 
#' @rdname aggregating
#' @aliases sum min max mean median sd var cov cor favstats
#' @param x a numeric vector or a formula
#' @param y a numeric vector or a formula
#' @param groups a grouping variable, typically a name of a variable in `data`
#' @param data a data frame in which to evaluate formulas (or bare names).
#' Note that the default is `data = parent.frame()`.  This makes it convenient to
#' use this function interactively by treating the working environment as if it were 
#' a data frame.  But this may not be appropriate for programming uses.  
#' When programming, it is best to use an explicit `data` argument
#' -- ideally supplying a data frame that contains the variables mentioned.
#' @param \dots additional arguments
### @param ..fun.. the underlying function.  It would be very unusual to change 
### this from its default value.
#' @param na.rm a logical indicating whether `NA`s should be removed before computing
#' @details Many of these functions mask core R functions to provide an additional formula 
#' interface.  Old behavior should be unchanged.  But if the first argument is a formula,
#' that formula, together with `data` are used to generate the numeric vector(s) 
#' to be summarized.  Formulas of the shape `x ~ a` or `~ x | a` can be used to
#' produce summaries of `x` for each subset defined by `a`.  Two-way aggregation
#' can be achieved using formulas of the form `x ~ a + b` or ` x ~ a | b`.  See
#' the examples.
#' 
# ' When an object exists both in \code{data} and in the environment of the formula, \code{data}
# ' takes precedence, but disambiguation is possible using \code{.data$x} or \code{.env$x}.  See
# ' the examples.
#' 
#' @section Note:
#' Earlier versions of these functions supported a "bare name + data frame" interface.  This
#' functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary,
#' and (c) difficult to maintain.
#'
#' @export
mean_ <- aggregatingFunction1(base::mean)

#' @rdname aggregating
#' @export
mean <- function(x, ...) {
  the_call <- match.call()
  the_call[[1]] <- as.name("mean_")
  
  if (inherits(x, c("Matrix", "sparseMatrix", "sparseVector")))
    return(Matrix::mean(x, ...))
  
  return(eval(the_call, parent.frame()))
}

#' @rdname aggregating
#' @export
median <- aggregatingFunction1(stats::median)
#' @rdname aggregating
#' @export

range <- aggregatingFunction1(base::range)
#' @rdname aggregating
#' @export
sd <- aggregatingFunction1(stats::sd)
#' @rdname aggregating
#' @export
max <- aggregatingFunction1(base::max)
#' @rdname aggregating
#' @export
min <- aggregatingFunction1(base::min)
#' @rdname aggregating
#' @export
sum <- aggregatingFunction1(base::sum)
#' @rdname aggregating
#' @export
IQR <- aggregatingFunction1(stats::IQR)
#' @rdname aggregating
#' @export
fivenum <- aggregatingFunction1(stats::fivenum)
#' @rdname aggregating
#' @export
iqr <- aggregatingFunction1(stats::IQR)
#' @rdname aggregating
#' @export
prod <- aggregatingFunction1(base::prod)
#' @rdname aggregating
#' @export
sum <- aggregatingFunction1(base::sum)
#' @rdname aggregating
#' @export
favstats <- aggregatingFunction1(fav_stats, output.multiple = TRUE, na.rm = TRUE)
#' @rdname aggregating
#' @export
quantile <- aggregatingFunction1(stats::quantile, output.multiple = TRUE, 
                                 na.rm = getOption("na.rm", FALSE))
#' @rdname aggregating
#' @export
var <- aggregatingFunction1or2(stats::var)
#' @rdname aggregating
#' @export
cor <- aggregatingFunction2(stats::cor)
#' @rdname aggregating
#' 
#' @examples
#' mean(HELPrct$age)
#' mean( ~ age, data = HELPrct)
#' mean( ~ drugrisk, na.rm = TRUE, data = HELPrct)
#' mean(age ~ shuffle(sex), data = HELPrct)
#' mean(age ~ shuffle(sex), data = HELPrct, .format = "table")
#' # wrap in data.frame() to auto-convert awkward variable names
#' data.frame(mean(age ~ shuffle(sex), data = HELPrct, .format = "table"))
#' mean(age ~ sex + substance, data = HELPrct)
#' mean( ~ age | sex + substance, data = HELPrct)
#' mean( ~ sqrt(age), data = HELPrct)
#' sum( ~ age, data = HELPrct)
#' sd(HELPrct$age)
#' sd( ~ age, data = HELPrct)
#' sd(age ~ sex + substance, data = HELPrct)
#' var(HELPrct$age)
#' var( ~ age, data = HELPrct)
#' var(age ~ sex + substance, data = HELPrct)
#' IQR(width ~ sex, data = KidsFeet)
#' iqr(width ~ sex, data = KidsFeet)
#' favstats(width ~ sex, data = KidsFeet)
#' 
#' cor(length ~ width, data = KidsFeet)
#' cov(length ~ width, data = KidsFeet)
#' tally(is.na(mcs) ~ is.na(pcs), data = HELPmiss)
#' cov(mcs ~ pcs, data = HELPmiss)             # NA because of missing data
#' cov(mcs ~ pcs, data = HELPmiss, use = "complete")  # ignore missing data
#' # alternative approach using filter explicitly
#' cov(mcs ~ pcs, data = HELPmiss |> filter(!is.na(mcs) & !is.na(pcs)))    
#' @export

cov <- aggregatingFunction2(stats::cov)

#' All pairs mean and sum of absolute differences
#' 
#' All pairs mean and sum of absolute differences
#' 
#' @inheritParams MAD

#' @return the mean or sum of the absolute differences between each pair
#' of values in `c(x,...)`.
#' @seealso [mad()]
#' @rdname MAD_
#' @export
MAD_ <- function(x, ..., na.rm = getOption("na.omit", FALSE)) {
  SAD_(x, ..., na.rm = na.rm) / (length(x) + length(list(...)))
}

#' @rdname MAD_
#' @param ... additional arguments appended to `x`
#' 
#' @export
SAD_ <- function(x, ..., na.rm = getOption("na.omit", FALSE)) {
  x <- c(x, unlist(list(...)))
  x <- na.omit(x)
  M <- outer(x, x, "-")
  base::sum(upper.tri(M) * abs(M))
}

#' All pairs mean and sum of absolute differences
#' 
#' The functions compute the sum or mean of all pairwise absolute 
#' differences.  This differs from [stats::mad()], which computes
#' the median absolute difference of each value from the median of 
#' all the values.  See the `ISIwithR` package (and the textbook it
#' accompanies) for examples using these functions in the context of 
#' simulation-based inference.
#' 
#' @param x a numeric vector or a formula.  
#' @param ... additional arguments passed through to `MAD_` 
#'   or `SAD_`.  If `x` is a formula, `...` should
#'   include an argument named `data` if the intent is to 
#'   interpret the formula in a data frame.
#' @param na.rm a logical indicating whether NAs should be removed before
#'   calculating.
#' @param groups a grouping variable, typically a name of a variable in `data`
#' @param data a data frame in which to evaluate formulas (or bare names).
#'   Note that the default is `data = parent.frame()`.  This makes it convenient to
#'   use this function interactively by treating the working environment as if it were 
#'   a data frame.  But this may not be appropriate for programming uses.  
#'   When programming, it is best to use an explicit `data` argument
#'   -- ideally supplying a data frame that contains the variables mentioned.
#' @return the mean or sum of the absolute differences between each pair
#'   of values in `c(x,...)`.
#' @seealso [mad()], [MAD_()]
#' @rdname MAD
#' @export
#' @examples
#' SAD(1:3)
#' MAD(1:3)
#' MAD(~eruptions, data = faithful)

MAD <- aggregatingFunction1(MAD_)

#' @rdname MAD
#' @export
SAD <- aggregatingFunction1(SAD_)

# Note: one of d, p, q, or n must be a **named** argument.  This allows other things to 
# be in ... without causing issues for the d/p/q/r function called.
#' Distribution wrapper
#' 
#' Utility function wrapping up the d/p/q/r distribution functions
#' 
#' @param dist a character description of a distribution, for example 
#'   `"norm"`, `"t"`, or `"chisq"`
#' @param type one of `"x"`, `"p"`, `"q"`, or `"r"`
#' @param ... additional arguments passed on to underlying distribution function.
#'   Note that one of `d`, `p`, `q`, or `n` must 
#'   be a named argument in ...
#' @export
#' @examples
#' # 3 random draws from N(1,2)
#' dpqrdist("norm", "r", n = 3, mean = 1, sd = 2)
#' # These should all be the same
#' dpqrdist("norm", "d", x = 0) == dnorm(x = 0)
#' dpqrdist("norm", "p", q = 0, mean = 1, sd = 2) == pnorm(q = 0, mean = 1, sd = 2)
#' dpqrdist("norm", "q", p = 0.5, mean = 1, sd = 2) == qnorm(p = 0.5, mean = 1, sd = 2)
#' @importFrom purrr possibly map_dbl
 
dpqrdist <- function( dist, type = c("d","p","q","r"), ...) {
  type <- match.arg(type)
  dots <- list(...)
  distFunName <- paste0(type, dist)
  distFun <- rlang::as_function(distFunName)
  dist_dots <- dots[names(dots) %in% names(formals(distFunName))]
  first_name <- as.name(names(formals(distFunName))[[1]])
  first_val <- dist_dots[[first_name]]
  dist_dots[[first_name]] <- NULL
  # for the sake of some distribution functions that throw errors, for example when p = 0 or 1
  # we convert errors into NaNs [Note: is.na(NaN) is TRUE]
  dpqrfun <- purrr::possibly(distFun, otherwise = NaN)
  # make sure an output value is computed for each input, even if some throw errors
  res <- do.call(sapply, c(list(first_val, dpqrfun), dist_dots))
  if (length(res) > 0 & all(is.na(res))) {
    stop("No values could be computed. Did you specify all the required parameters?")
  }
  res
}


#' Illustrated probability calculations from distributions
#' 
#' Illustrated probability calculations from distributions
#' 
#' @param dist a character description of a distribution, for example 
#'   `"norm"`, `"t"`, or `"chisq"`
#' @param q a vector of quantiles
#' @param plot a logical indicating whether a plot should be created
#' @param verbose a logical
#' @param invisible a logical
#' @param digits the number of digits desired
#' @param xlim x limits
#' @param ylim y limits
# #' @param vlwd width of vertical lines
# #' @param vcol color of vertical lines
# #' @param rot angle for rotating text indicating probability
#' @param resolution Number of points used for detecting discreteness and generating plots.  
#'        The default value of 5000 should work well except for discrete distributions
#'        that have many distinct values, especially if these values are not evenly spaced.
#' @param ... Additional arguments, typically for fine tuning the plot.
#' @param return If `"plot"`, return a plot.  If `"values"`, return a vector of numerical values.
#' @param refinements A list of refinements to the plot.  See [ggformula::gf_refine()].
#' @details The most general function is `pdist` which can work with 
#' any distribution for which a p-function exists.  As a convenience, wrappers are 
#' provided for several common distributions.
#' @return A vector of probabilities; a plot is printed as a side effect.
#' @examples
#' pdist("norm", -2:2)
#' pdist("norm", seq(80,120, by = 10), mean = 100, sd = 10)
#' pdist("chisq", 2:4, df = 3)
#' pdist("f", 1, df1 = 2, df2 = 10)
#' pdist("gamma", 2, shape = 3, rate = 4)
#' @export

pdist <- function (dist = "norm", q, plot = TRUE, verbose = FALSE, invisible = FALSE, 
                   digits = 3L, 
                   xlim, ylim,
                   resolution = 500L,
                   return = c("values", "plot"),
                   ...,
                   refinements = list())
{
  
  return <- match.arg(return)
  
  dots <- list(...)
  
  # if (is.null(vlwd)) {
  #   vlwd <- if (is_discrete_dist( dist, ... )) 0 else 2
  # }
  
  p <- dpqrdist(dist, type = "p", q = q, ...) 
  
  if (verbose) {
    cat("Verbose output not yet implemented.\n")
  }
  
  res_plot <-
    do.call(
      gf_refine,
      c(list(
        plot_multi_dist(
          dist = dist, p = p, 
          xlim = xlim, ylim = ylim, 
          digits = digits, 
          resolution = resolution,
          ...)),
        refinements)
    )
  
  if (return == "plot") {
    return(res_plot)
  }
  if (plot) {
    print(res_plot)
  }
  if (invisible) { 
    return(invisible(p))
  }
  return(p)
}

#' Illustrated quantile calculations from distributions
#' 
#' Illustrated quantile calculations from distributions
#' 
#' @param dist a character description of a distribution, for example 
#'   `"norm"`, `"t"`, or `"chisq"`
#' @param p a vector of probabilities
#' @param plot a logical indicating whether a plot should be created
#' @param verbose a logical
#' @param invisible a logical
#' @param digits the number of digits desired
#' @param xlim x limits.  By default, these are chosen to show the central 99.8\% 
#'   of the distribution.
#' @param ylim y limits
# #' @param vlwd width of vertical lines
# #' @param vcol color of vertical lines
# #' @param rot angle for rotating text indicating probability
#' @param resolution number of points used for detecting discreteness and generating plots.  
#'   The default value of 5000 should work well except for discrete distributions
#'   that have many distinct values, especially if these values are not evenly spaced.
#' @param refinements A list of refinements to the plot.  See [ggformula::gf_refine()].
#' @param ... additional arguments, including parameters of the distribution
#'   and additional options for the plot.  To help with name collisions (eg `size` for binomial 
#'   distributions and `shape` for gamma distributions), argument names beginning `plot_` will
#'   be renamed to remove `plot_` and passed only to the plot.  The unprefixed version will 
#'   used as a parameter for the the distribution.
#' @param return If `"plot"`, return a plot.  If `"values"`, return a vector of numerical values.
#' @details The most general function is `qdist` which can work with 
#' any distribution for which a q-function exists.  As a convenience, wrappers are 
#' provided for several common distributions.
#' 
#' @return a vector of quantiles; a plot is printed as a side effect
#' @examples
#' qdist("norm", seq(.1, .9, by = 0.10), 
#'       title = "Deciles of a normal distribution", show.legend = FALSE,
#'       pattern = "rings")
#' xqnorm(seq(.2, .8, by = 0.20), mean = 100, sd = 10)
#' qdist("unif", .5)
#' xqgamma(.5, shape = 3, scale = 4)
#' xqgamma(.5, shape = 3, scale = 4, color = "black")
#' xqbeta(.5, shape1 = .9, shape2 = 1.4, dlwd = 1)
#' xqchisq(c(.25,.5,.75), df = 3)
#' xcbinom(c(0.80, 0.90), size = 1000, prob = 0.40)
#' # displayed as if continuous
#' xcbinom(c(0.80, 0.90), size = 5000, prob = 0.40)
#' xpbinom(c(480, 500, 520), size = 1000, prob = 0.48)
#' xpbinom(c(40, 60), size = 100, prob = 0.5)
#' xqpois(c(0.25, 0.5, 0.75), lambda = 12)
#' xcpois(0.50, lambda = 12)
#' xcpois(0.50, lambda = 12, refinements = list(scale_color_brewer(type = "qual", palette = 5)))
#' @export 

qdist <- function (dist = "norm", p, plot = TRUE, verbose = FALSE, invisible = FALSE, 
                   resolution = 500L,
                   digits = 3L, 
                   xlim, ylim,
                   return = c("values", "plot"),
                   refinements = list(),
                   ...
)
{
  return = match.arg(return)
  dots <- list(...)
  
  # if (is.null(vlwd)) {
  #   vlwd <- if (is_discrete_dist( dist, ... )) 0 else 2
  # }
  # 
  q <- dpqrdist(dist, type = "q", p = p, ...) 
  # adjust probs for discrete dists where requested p's might not be (exactly) possible
  p <- dpqrdist(dist, type = "p", q = q, ...)  
  if (verbose) {
    cat("Verbose output not yet implemented.\n")
  }
  
  res_plot <-  
    do.call(
      ggformula::gf_refine,
      c(list(
        plot_multi_dist(
          dist = dist, p = p, 
          xlim = xlim, ylim = ylim, 
          digits = digits, 
          resolution = resolution,
          ...)
      ), refinements)
    )
  
  if (return == "plot") return (res_plot)
  
  if (plot) {
    print(res_plot)
  }
  if (invisible) { 
    return(invisible(q))
  }
  return(q)
}


utils::globalVariables(c("i", "name", "p_all"))

plot_multi_dist <- 
  function(dist, p, q, xlim, ylim, digits = 3L, resolution = 500L,
           lower.tail = TRUE,
           dlwd = 0, 
           pattern = c("stripes", "rings"),
           ...) 
  {
    dots <- list(...)
    pattern <- match.arg(pattern)
    
    plot_dots <- dots[ ! names(dots) %in% names(formals(paste0("p",dist))) ]
    dist_dots <- dots[   names(dots) %in% names(formals(paste0("p",dist))) ]
    
    discrete <- do.call(is_discrete_dist, c(list(dist), dist_dots))
    
    if (missing(p)) {
      if (missing(q)) { stop( "one of p or q must be specified") }
      q <- sort(q)
      p <- dpqrdist(dist, type = "p", q = q, lower.tail = lower.tail, ...)
      p_less <- dpqrdist(dist, type = "p", q = q, lower.tail = TRUE, ...)
    } else {
      if (!missing(q)) {
        warning("Both p and q were specified.  I'm using p")
      }
      p <- sort(p)
      q <- sort(dpqrdist(dist, type = "q", p = p, lower.tail = lower.tail, ...))
      p_less <- dpqrdist(dist, type = "p", q = q, ...)
    }
    
    names(plot_dots) <- gsub("plot_", "", names(plot_dots)) 
    if ('lty' %in% names(plot_dots)) { plot_dots$linetype <- plot_dots$lty; plot_dots$lty <- NULL }
    if ('pch' %in% names(plot_dots)) { plot_dots$shape <- plot_dots$pch;    plot_dots$pch <- NULL }
    if ('cex' %in% names(plot_dots)) { plot_dots$size <- plot_dots$cex;     plot_dots$cex <- NULL }
    
    if (! 'xlab' %in% names(plot_dots)) { plot_dots$xlab <- "" }
    
    if (missing(xlim) || is.null(xlim)) {
      xlim_opts <- dpqrdist(dist, type = "q", p = c(0, 0.0001, 0.001, 0.01, 0.99, 0.999, 0.9999, 1), ...)
      if (is.matrix(xlim_opts)) {
        xlim <- c(min(xlim_opts[, 4], na.rm = TRUE), max(xlim_opts[,5], na.rm = TRUE))
      } else {
        xlim <- xlim_opts[4:5]
      }
      
      # extend range if it doesn't extend too far
      mid_width <- diff(xlim)
      a <- xlim_opts[min(which(xlim[2] - xlim_opts < 1.8 * mid_width), na.rm = TRUE)]
      b <- xlim_opts[max(which(xlim_opts - xlim[1] < 1.8 * mid_width), na.rm = TRUE)]
      xlim = c(a,b)
    }
    
    if (discrete) {
      xdata <- unique(dpqrdist(dist, type = "q", p = ppoints(resolution), ...))
      step = min(diff(xdata))
      xlim = c(-step/2, step/2) + xlim  # widen by half step in each direction
      fill <- seq( min(xdata) -1.5 * step , max(xdata) + 1.5*step, length.out = resolution)
      xdata <- c(xdata, fill) 
      xdata <- dpqrdist(dist, type = "q", p = dpqrdist(dist, type = "p", q = xdata, ...), ...)
      xdata <- sort(unique(xdata))
    } else {
      xdata <- seq(xlim[1], xlim[2], length.out = resolution)
      xdata <- dpqrdist(dist, type = "q", p = dpqrdist(dist, type = "p", q = xdata, ...), ...)
      xdata <- sort(unique(xdata))
    }
    
    # not a fix: xdata <- sort(xdata) 
    ydata <- dpqrdist(dist, type = "d", x = xdata, ...)
    ymax <- 1.1 * quantile(ydata, 0.95, na.rm = TRUE)
    if (missing(ylim)) {
      ylim = c(0, 1.4 * ymax)  # quantile(ydata, 0.95, na.rm = TRUE))
      # print(ylim)
    }
    
    # this could be funny if limits don't span q
    p_less <- sort(unique(c(0, p_less, 1)))
    q <- dpqrdist(dist, type = "q", p = p_less, ...) #  c(xlim[1], q, xlim[2])
    
    # if (discrete) {
    #   if (is.null(plot_dots$pch)) plot_dots$pch = 16
    # }
    
    
    
    # args <- c(
    #   list(
    #     ydata ~ xdata, 
    #     xlim = xlim, ylim = ylim, 
    #     xlab = "", 
    #     ylab = if (discrete) "probability" else "density" 
    #   ),
    #   plot_dots
    # )
    # 
    # res_plot <- do.call("xyplot", args)
    
    Groups <- 
      tibble::tibble(p = diff(p_less)) |>
      mutate(
        i = 1 : n(),
        name = if (pattern == "stripes") LETTERS[i] else LETTERS[ceiling(1 + abs(i - mean(i)))]
      ) |> 
      group_by(name) |>
      mutate(
        p_all = sum(p),
        label = paste(first(name), ":", formatC(p_all, format = "f", digits = digits), sep = "")
      )
    
    if (length(xdata) != length(ydata)) {
      stop("It looks like you have multiple parameter settingss specified for your distribution. 
           Plotting only works for one parameter setting.")
    }
    
    if (discrete) { 
      # q <- tail(q, -1)
      Ddensity <- 
        tibble::tibble(
          x = xdata, density = ydata, 
          group = sapply(xdata, function(x) {sum(x > q, na.rm = TRUE)})
        ) |>
        mutate(
          group = pmax(1, group),
          probability = Groups$label[group] 
        ) 
      
      # print(Ddensity)
      
      plot <- 
        do.call(
          gf_point, 
          c(list(density ~ x, color = ~ probability, group = ~ group, fill = ~ probability,
                 data = Ddensity), plot_dots)) |>
        gf_segment(density + 0 ~ x + x) |>
        gf_labs(x = "", y = "probability") |>
        gf_refine(
        scale_fill_viridis_d(end = 0.9),
        scale_color_viridis_d(end = 0.9)
      )
    
  } else {
    Ddensity <- 
      tibble::tibble(
        x = xdata, density = ydata, 
        group = sapply(xdata, function(x) {sum(x > q, na.rm = TRUE)})
      ) |>
      mutate(
        group = pmax(1, group),
        probability = Groups$label[group] # factor(group, labels = labels[sort(unique(group))])
      ) |>
      dplyr::filter(!is.na(probability))  # avoids issues when xlim is wider than support
    
    plot <- 
      do.call(gf_area, 
              c(list(density ~ x, fill = ~ probability, 
                     group = ~ group, data = Ddensity), plot_dots)) |>
      gf_refine(
        scale_fill_viridis_d(end = 0.9),
        scale_color_viridis_d(end = 0.9)
      )
  }  
  return(plot)
}


#' @rdname pdist
#' @inheritParams stats::pgamma 
#' @param log.p A logical indicating whether probabilities should be 
#'   returned on the log scale.
#' @seealso [qdist()], [xpnorm()], [xqnorm()].
#' @export
xpgamma <- function(q, shape, rate = 1, scale = 1/rate, 
                    lower.tail = TRUE, log.p = FALSE, ...) {
  if (!missing(rate) && !missing(scale)) {
    if (abs(rate * scale - 1) < 1e-15) 
      warning("specify 'rate' or 'scale' but not both")
    else stop("specify 'rate' or 'scale' but not both")
  }
  pdist("gamma", q = q, shape = shape, scale = scale,
        lower.tail = lower.tail, log.p = log.p, ...)
}
#' @rdname qdist
#' @inheritParams stats::qgamma
#' @param log.p A logical indicating whether probabilities should be 
#'   returned on the log scale.
#' @export
xqgamma <- function(p, shape, rate = 1, scale = 1/rate, 
                    lower.tail = TRUE, log.p = FALSE, ...) {
  if (!missing(rate) && !missing(scale)) {
    if (abs(rate * scale - 1) < 1e-15) 
      warning("specify 'rate' or 'scale' but not both")
    else stop("specify 'rate' or 'scale' but not both")
  }
  qdist("gamma", p = p, shape = shape, scale = scale,
        lower.tail = lower.tail, log.p = log.p,
        ...)
}
#' @rdname cdist
#' @inheritParams stats::qgamma
#' @param log.p A logical indicating whether probabilities should be 
#'   returned on the log scale.
#' @export
xcgamma <- function(p, shape, rate = 1, scale = 1/rate, 
                    lower.tail = TRUE, log.p = FALSE, ...) {
  if (!missing(rate) && !missing(scale)) {
    if (abs(rate * scale - 1) < 1e-15) 
      warning("specify 'rate' or 'scale' but not both")
    else stop("specify 'rate' or 'scale' but not both")
  }
  cdist("gamma", q = q, shape = shape, scale = scale,
        lower.tail = lower.tail, log.p = log.p,
        ...)
}

#' @rdname pdist
#' @inheritParams stats::pt
#' @param log.p A logical indicating whether probabilities should be 
#'   returned on the log scale.
#' @export
xpt <- function(q, df , ncp, lower.tail = TRUE, log.p = FALSE, ...)  
  if (missing(ncp)) {
    pdist("t", q = q, df = df, lower.tail = lower.tail, 
          log.p = log.p, ...)
  } else {
    pdist("t", q = q, df = df, ncp = ncp, lower.tail = lower.tail, 
          log.p = log.p, ...)
  }
#' @rdname qdist
#' @inheritParams stats::qt
#' @param log.p A logical indicating whether probabilities should be 
#'   returned on the log scale.
#' @export
xqt <- function(p, df , ncp, lower.tail = TRUE, log.p = FALSE, ...)  
  if (missing(ncp)) {
    qdist("t", p = p, df = df, lower.tail = lower.tail, 
          log.p = log.p, ...)
  } else {
    qdist("t", p = p, df = df, ncp = ncp, lower.tail = lower.tail, 
          log.p = log.p, ...)
  }



#' @rdname cdist
#' @inheritParams stats::pt
#' @param log.p A logical indicating whether probabilities should be 
#'   returned on the log scale.
#' @export
xct <- function(p, df , ncp, lower.tail = TRUE, log.p = FALSE, ...)  
  if (missing(ncp)) {
    cdist("t", p = p, df = df, lower.tail = lower.tail, 
          log.p = log.p, ...)
  } else {
    cdist("t", p = p, df = df, ncp = ncp, lower.tail = lower.tail, 
          log.p = log.p, ...)
  }

#' @rdname pdist
#' @export
xpchisq <- function(q, df, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)   
  pdist("chisq", q = q, df = df, ncp = ncp, lower.tail = lower.tail, 
        log.p = log.p, ...)
#' @rdname qdist
#' @export
xqchisq <- function(p, df, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)   
  qdist("chisq", p = p, df = df, ncp = ncp, lower.tail = lower.tail, 
        log.p = log.p, ...)
#' @rdname cdist
#' @export
xcchisq <- function(p, df, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)   
  cdist("chisq", p = p, df = df, ncp = ncp, lower.tail = lower.tail, 
        log.p = log.p, ...)

#' @rdname pdist
#' @inheritParams stats::pf
#' @export
xpf <- function(q, df1, df2, lower.tail = TRUE, log.p = FALSE, ...)  
  pdist("f", q = q, df1 = df1, df2 =df2, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname qdist
#' @inheritParams stats::qf
#' @export
xqf <- function(p, df1, df2, lower.tail = TRUE, log.p = FALSE, ...)  
  qdist("f", p = p, df1 = df1, df2 =df2, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname cdist
#' @inheritParams stats::qf
#' @export
xcf <- function(p, df1, df2, lower.tail = TRUE, log.p = FALSE, ...)  
  cdist("f", p = p, df1 = df1, df2 =df2, 
        lower.tail = lower.tail, log.p = log.p, ...)


#' @rdname pdist
#' @inheritParams stats::pbinom
#' @export
xpbinom <- function(q, size, prob, lower.tail = TRUE, log.p = FALSE, ...)  
  pdist("binom", q = q, size = size, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname qdist
#' @inheritParams stats::qbinom
#' @export
xqbinom <- function(p, size, prob, lower.tail = TRUE, log.p = FALSE, ...)  
  qdist("binom", p = p, size = size, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname cdist
#' @inheritParams stats::qbinom
#' @export
xcbinom <- function(p, size, prob, lower.tail = TRUE, log.p = FALSE, ...)  
  cdist("binom", p = p, size = size, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname pdist
#' @inheritParams stats::ppois
#' @export
xppois <- function(q, lambda, lower.tail = TRUE, log.p = FALSE, ...)  
  pdist("pois", q = q, lambda = lambda, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname qdist
#' @inheritParams stats::qpois
#' @export
xqpois <- function(p, lambda, lower.tail = TRUE, log.p = FALSE, ...)  
  qdist("pois", p = p, lambda = lambda, 
        lower.tail = lower.tail, log.p = log.p, ...)


#' @rdname cdist
#' @inheritParams stats::qpois
#' @export
xcpois <- function(p, lambda, lower.tail = TRUE, log.p = FALSE, ...)  
  cdist("pois", p = p, lambda = lambda, 
        lower.tail = lower.tail, log.p = log.p, ...)


#' @rdname pdist
#' @inheritParams stats::pgeom
#' @export
xpgeom <- function(q, prob, lower.tail = TRUE, log.p = FALSE, ...)  
  pdist("geom", q = q, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname qdist
#' @inheritParams stats::qgeom
#' @export
xqgeom <- function(p, prob, lower.tail = TRUE, log.p = FALSE, ...)  
  qdist("geom", p = p, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)


#' @rdname cdist
#' @inheritParams stats::qgeom
#' @export
xcgeom <- function(p, prob, lower.tail = TRUE, log.p = FALSE, ...)  
  cdist("geom", p = p, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname pdist
#' @inheritParams stats::pnbinom
#' @export
xpnbinom <- function(q, size, prob, mu, lower.tail = TRUE, log.p = FALSE, ...) { 
  if (missing (mu)) {
    pdist("nbinom", q = q, size = size, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)
  } else {
    pdist("nbinom", q = q, size = size, mu = mu, 
        lower.tail = lower.tail, log.p = log.p, ...)
  }
}

#' @rdname qdist
#' @inheritParams stats::qnbinom
#' @export
#' 
xqnbinom <- function(p, size, prob, mu, lower.tail = TRUE, log.p = FALSE, ...) {
  if (missing (mu)) {
    qdist("nbinom", p = p, size = size, prob = prob, 
        lower.tail = lower.tail, log.p = log.p, ...)
  } else {
    qdist("nbinom", p = p, size = size, mu = mu, 
        lower.tail = lower.tail, log.p = log.p, ...)
  }
}

#' @rdname cdist
#' @inheritParams stats::qnbinom
#' @export
xcnbinom <- function(p, size, prob, mu, lower.tail = TRUE, log.p = FALSE, ...)  
  cdist("nbinom", p = p, size = size, prob =prob, mu = mu, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname pdist
#' @inheritParams stats::pbeta
#' @export
xpbeta <- function(q, shape1, shape2, ncp = 0, 
                   lower.tail = TRUE, log.p = FALSE, ...)  
  pdist("beta", q = q, shape1 = shape1, shape2 = shape2, ncp = 0, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname qdist
#' @inheritParams stats::qbeta
#' @export
xqbeta <- function(p, shape1, shape2, ncp = 0, 
                   lower.tail = TRUE, log.p = FALSE, ...)  
  qdist("beta", p = p, shape1 = shape1, shape2 = shape2, ncp = 0, 
        lower.tail = lower.tail, log.p = log.p, ...)

#' @rdname cdist
#' @inheritParams stats::qbeta
#' @export
xcbeta <- function(p, shape1, shape2, ncp = 0, 
                   lower.tail = TRUE, log.p = FALSE, ...)  
  cdist("beta", p = p, shape1 = shape1, shape2 = shape2, ncp = 0, 
        lower.tail = lower.tail, log.p = log.p, ...)


is_discrete_dist <- function(dist, resolution = 100L, ... ) {
  q <- dpqrdist(dist, type = "q", p = ppoints(resolution), ...) 
  # continuous dists should have all unique values
  # discrete dists will typically have many ties
  # discrete dists with many values may look continuous
  length(unique(q)) / length(q) <= 0.95
} 

# context('dpqrdist()')

test_that("dpqrdist works for normal dist", {
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("norm", "d", x = c(0,1,2)), 
    dnorm(c(0,1,2))
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("norm", "d", x = c(0,1,2), mean=10, sd=2),
    dnorm(c(0,1,2), mean=10, sd=2) 
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("norm", "p", q = c(0,1,2)), 
    pnorm(c(0,1,2))
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("norm", "p", q = c(0,1,2), mean=10, sd=2),
    pnorm(c(0,1,2), mean=10, sd=2) 
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("norm", "q", p = c(.1,.2,.3)),
    qnorm(c(.1,.2,.3))
  ) 
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("norm", "q", p = c(.1,.2,.3), mean=10, sd=2),
    qnorm(c(.1,.2,.3), mean=10, sd=2) 
  )    
})

test_that("dpqrdist works for t dist", {
   expect_equal(ignore_attr = TRUE, 
     dpqrdist("t", "d", x = c(0,1,2), df=10), 
     dt(c(0,1,2), df=10)
   )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("t", "p", q = c(0,1,2), df=10), 
    pt(c(0,1,2), df=10)
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("t", "q", p = c(.1,.2,.3), df=10), 
    qt(c(.1,.2,.3), df=10)
  )
})

test_that("dpqrdist works for binomial dist", {
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("binom", "d", x = c(0,1,2), size=10, prob=0.4), 
    dbinom(c(0,1,2), size=10, prob=0.4)
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("binom", "p", q = c(0,1,2), size=10, prob=0.4), 
    pbinom(c(0,1,2), size=10, prob=0.4)
  )
  expect_equal(ignore_attr = TRUE, 
    dpqrdist("binom", "q", p = c(.25, .5, .75), size=10, prob=0.4), 
    qbinom(c(.25, .5, .75), size=10, prob=0.4)
  )
})

# test_that("pdist works", {
#   #pdist("norm", -2:2) |> dput()
#   # testcase <- c(0.0227501319481792, 0.158655253931457, 0.5, 0.841344746068543, 
#   #               0.977249868051821)
#   # test <- pdist("norm", -2:2)
#   # expect_equal(ignore_attr = TRUE, test, testcase)
#   #wrapped_expect_doppelganger("pdist1", test <- pdist("norm", -2:2))
#   #pdist("norm", seq(80,120, by = 10), mean = 100, sd = 10)
#   # pdist("chisq", 2:4, df = 3)
#   # pdist("f", 1, df1 = 2, df2 = 10)
#   # pdist("gamma", 2, shape = 3, rate = 4)
# })

utils::globalVariables(c('coordinates',"Name","Code","long","lat"))

## Files needed for the country and state functions:
#### US_States : a SpatialPolygonsDataFrame object for US states
#### US_States_df : fortify(US_States)
#### stateAlternatives : a vector for standardizing state names
#### World_Countries : a SpatialPolygonsDataFrame object for world countries
#### World_Countries_df : fortify(World_Countries)
#### countryAlternatives : a vector for standardizing country names
#### CIA: a dataframe specifying the possible datasets available for
####      download through the CIAdata() function

#' Standardization of Geographic Names
#' 
#' Often different sources of geographical data will use different names for
#' the same region.  These utilities make it easier to merge data from different
#' sources by converting names to standardized forms.
#' 
#' @details
#' \describe{
#' 
#' \item{standardName}{This is the most general standardizing function.
#' In addition to `x`, this function requires another argument:
#' `standard` - a named vector in which each name is a particular
#' spelling of the region name in question and the corresponding value
#' is the standardized version of that region name}
#'  
#' \item{standardCountry}{This function will standardize the country
#' names in `x` to the standard ISO_a3 country code format. If 
#' `returnAlternatives` is set to `TRUE`, this function will also
#' return the the named vector used to standardize the country names}
#' 
#' \item{standardState}{This function will standardize the US state
#' names in `x` to the standard two-letter abbreviations. If 
#' `returnAlternatives` is set to `TRUE`, this function will also
#' return the the named vector used to standardize the state names}
#' }
#' In all three cases, any names not found in `standard`
#' will be left unaltered.  Unless supressed, a warning message will
#' indicate the number of such cases, if there are any.
#' 
#' @param x A vector with the region names to standardize
#' @param standard a named vector providing the map from 
#' non-standard names (names of vector) to standard names (values of vector)
#' @param ignore.case a logical indicating whether case should be ignored 
#' when matching.
#' @param quiet a logical indicating whether warnings should be suppressed
#' @export
standardName <- function(x, standard, ignore.case=TRUE, returnAlternatives=FALSE, quiet=FALSE) {
  nn <- names(x)  
  if (ignore.case) {
    x <- toupper(x)
    names(standard) <- toupper(names(standard))
  }
  res <- standard[x]
  numNAs <- sum(is.na(res))
  res[ is.na(res) ] <- x [ is.na(res) ]
  names(res) <- nn
  if (! quiet && (numNAs > 0)) {
    warning(paste(numNAs, "items were not translated"))
  }
  if (!returnAlternatives) return(res)
  list(standardized = res, alternatives = countryAlternatives)
}

#' @export
#' @rdname standardName
#' @param returnAlternatives a logical indicating whether all alternatives should
#' be returned in addition to the standard name.
standardCountry <- function(x, ignore.case=TRUE, returnAlternatives = FALSE, 
                            quiet=FALSE) {
  standardName(x, countryAlternatives, 
               ignore.case = ignore.case, 
               returnAlternatives=returnAlternatives,
               quiet=quiet)
}

#' @export
#' @rdname standardName
standardState <- function(x, ignore.case=TRUE, 
                          returnAlternatives = FALSE, 
                          quiet=FALSE) {
  standardName(x, stateAlternatives, 
               ignore.case = ignore.case, 
               returnAlternatives=returnAlternatives, 
               quiet=quiet)
}

#' @export
fortify.SpatialPolygonsDataFrame <- function(model, data, region=NULL, ...) {
  if (!is.null(region)) stop("`region' not supported")  
  model@data$id <- rownames(model@data) 
  region <- "id"
  fmodel <- do.call(fortify, list(model, region="id"), 
                    envir=environment(ggplot2::fortify))
  merge(fmodel, model@data, by="id")
}

#' Make a map with `ggplot2`
#'
#' `makeMap` takes in two sources of data that refer to geographical
#' regions and merges them together. Depending on the arguments passed,
#' it returns this merged data or a ggplot object constructed with the data.
#'
#' @param data A dataframe with regions as cases
#' @param map An object that can be fortified to a dataframe (ex: a dataframe itself,
#' or a SpatialPolygonsDataFrame) 
#' @param key.data The column name in the `data` that holds the
#' unique names of each region
#' @param key.map The column name in the `map` that holds the
#' unique names of each region
#' @param key The combination of `key.data` and `key.map` 
#' @param tr.data A function of the transformation to be performed to
#' the `key.data` column
#' @param tr.map A function of the transformation to be performed to
#' the `key.map` column
#' @param plot The plot desired for the output. `plot` = "none"
#' returns the merged data that is the result of merging the `data`
#' and `map` together; `plot`="frame" returns an empty
#' (unplottable) ggplot object; `plot` = "border" (the default)
#' returns a ggplot object with one geom_polygon layer that shows the
#' borders of the regions.
#' @export 
makeMap <- function (data = NULL, map=NULL, key=c(key.data, key.map), 
                  key.data, key.map, tr.data = identity, tr.map = identity,
                  plot=c("borders", "frame", "none")) {
  message("Mapping API still under development and may change in future releases.")
  plot <- match.arg(plot)
  if (is.null(map) && is.null(data)) {
    stop("At least one of `data' and `map' must be specified.")
  }
  
  # If map is given, make sure it looks like a map
  if (!is.null(map)) {
    map <- fortify(map)
    if (! "long" %in% names(map)) {
      if ("lon" %in% names(map)) map$long <- map$lon
    } 
    
    for (n in c("lat", "long", "order", "group")) {
      if (!(n %in% names(map))) {
        stop(paste0("`map' does not appear to be a map (missing ", n, ")"))
      }
    }
  }
 
  # If we have both data and map, merge them 
  if (!is.null(data) && !is.null(map)) {
    key <- rep(key, out.length=2)
    if (!(key[1] %in% names(data))) {
      stop(paste(key[1], "not in names of `data'"))
    }
    if (!(key[2] %in% names(map))) {
      stop(paste(key[2], "not in names of `map'"))
    }
    map[[key[2]]] <- tr.map(map[[key[2]]])
    data[[key[1]]] <- tr.data(data[[key[1]]])
    data <- merge(data, map, by.x=key[1], by.y=key[2])
  }
  
  # make sure data is ready to go
  if (is.null(data)) data <- map
  
  for (n in c("lat", "long", "group", "order")) {
    if (! n %in% names(data)) {
      stop(paste0("`data' does not appear to be properly formatted (missing ", n, ")"))
    }
  }
  
  switch(plot, 
         borders = 
           ggplot(data |> group_by(group) |> arrange(order) |> ungroup(), 
                  aes(x=long, y=lat, group=group)) +
           geom_polygon(color="darkgray", fill=NA) + theme_map() +
           labs(x="", y=""),
         frame = 
           ggplot(data |> group_by(group) |> arrange(order) |> ungroup(), 
                  aes(x=long, y=lat, group=group)),
         none = 
           data)
}


#' Make a world map with `ggplot2`
#'
#' `mWorldMap` takes in one dataframe that includes information
#' about different countries. It merges this dataframe with a dataframe
#' that includes geographical coordinate information. Depending on the
#' arguments passed, it returns this data or a ggplot object constructed
#' with the data.
#' 
#' @param data A dataframe with countries as cases
#' @param key The column name in the `data` that holds
#' the unique names of each country
#' @param fill A variable in the `data` used to specify the fill
#' color of countries in the map (note: if `fill` is not null, then
#' `plot` cannot be set to "none")
#' @param plot The plot desired for the output. `plot` = "none"
#' returns the merged data that is the result of merging the `data`
#' and the dataframe with the geographical coordinate information;
#' `plot` = "frame" returns an empty (unplottable) ggplot object;
#' `plot` = "border" (the default) returns a ggplot object with
#' one geom_polygon layer that shows the borders of the countries
#' 
#' @examples
#'\dontrun{
#' gdpData <- CIAdata("GDP")      # load some world data
#' 
#' mWorldMap(gdpData, key="country", fill="GDP")
#'
#' gdpData <- gdpData |> mutate(GDP5 = ntiles(-GDP, 5, format="rank")) 
#' mWorldMap(gdpData, key="country", fill="GDP5")
#'
#' mWorldMap(gdpData, key="country", plot="frame") +
#' geom_point()
#' 
#' mergedData <- mWorldMap(gdpData, key="country", plot="none")
#' 
#' ggplot(mergedData, aes(x=long, y=lat, group=group, order=order)) +
#' geom_polygon(aes(fill=GDP5), color="gray70", size=.5) + guides(fill=FALSE)  
#' }
#' @export 
mWorldMap <- function(data = NULL, key = NA, fill = NULL, plot = c("borders", "frame", "none")) {
  plot <- match.arg(plot)
  map <- makeMap(data = data, map = World_Countries_df, key = c(key, "iso_a3"), 
              tr.data = standardCountry, tr.map = toupper, plot = plot)
  if (plot != "none") { 
    # deleting coord_map() for now for sake of CRAN checks (not understood) 2014-08-20
    map <- map # + coord_map()   
  }
  if ( (!is.null(fill) && plot != "none") ) {
    map <- map + geom_polygon(aes(fill = {{ fill }}), color = "darkgray")
  }
  map
}


#' Make a US map with `ggplot2`
#'
#' `mUSMap` takes in one dataframe that includes information
#' about different US states. It merges this dataframe with a dataframe
#' that includes geographical coordinate information. Depending on the
#' arguments passed, it returns this data or a ggplot object constructed
#' with the data.
#'
#' @param data A dataframe with US states as cases
#' @param key The column name in the `data` that holds the unique
#' names of each state
#' @param fill A variable in the `data` used to specify the fill
#' color of states in the map (note: if `fill` is not null, then
#' `plot` cannot be set to "none")
#' @param plot The plot desired for the output. `plot` = "none"
#' returns the merged data that is the result of merging the `data`
#' and the dataframe with the geographical coordinate information;
#' `plot` = "frame" returns an empty (unplottable) ggplot object;
#' `plot` = "border" (the default) returns a ggplot object with
#' one geom_polygon layer that shows the borders of the states
#' @param style The style in which to display the map. `compact` gives 
#' a polyconic projection with Alaska and Hawaii on the lower left corner;
#' `real` gives the real size and position of all states without any
#' projection.
#'  
#' @examples
#' USArrests2 <- USArrests |> tibble::rownames_to_column("state")
#' mUSMap(USArrests2, key="state", fill = "UrbanPop") 
#' @export 
mUSMap <- function(data = NULL, key, fill = NULL, 
                   plot = c("borders", "frame", "none"),
                   style = c("compact","real")) {
  plot <- match.arg(plot)
  style <- match.arg(style)
  if (style == "compact") {US_States_df <- US_States_comp_df}
  map <- makeMap(data = data, map = US_States_df, key = c(key, "STATE_ABBR"), 
              tr.data = standardState, tr.map = toupper, plot = plot)
  if ( (!is.null(fill) && plot != "none") ) {
    map <- map + geom_polygon(aes(fill = .data[[{{fill}}]]), color = "darkgray")
  }
  map
}


#' Return a dataset based on the CIA World Factbook
#' 
#' This function can be used in two different ways. Without an argument, it returns a reference
#' table that includes information about all the CIA World Factbook tables that are available
#' through this function. Note the  `Name` column that indicates a unique name for each
#' available dataset. If this name is passed as an argument to the function, the function 
#' will return the corresponding dataset.
#' 
#' @param name An optional parameter specifying the name of the desired dataset.
#' If multiple names are given, a merge will be attempted on the individual data
#' sets.
#' 
#' @examples
#' \dontrun{
#' head(CIAdata())
#' Population <- CIAdata("pop")
#' nrow(Population)
#' head(Population)
#' 
#' PopArea <- 
#'   CIAdata(c("pop","area")) |> 
#'   mutate(density = pop / area)
#' nrow(PopArea)
#' head(PopArea)
#' PopArea |> 
#'   filter(!is.na(density)) |>
#'   arrange(density) |> 
#'   tail()
#' }
# #' @importFrom readr parse_number
#' @export
CIAdata <- function (name = NULL) {
 
  rlang::check_installed('readr') 
  if (is.null(name)) return(CIA)  
  
  if (length(name) > 1) {
    return(Reduce(function(A,B) merge(A,B, by="country", all=TRUE), 
                  Map(CIAdata, name=name)))
  }
  
  if (name %in% CIA$Name) {
    sub <- subset(CIA, Name == name)
  } else {
    if (name %in% CIA$Code) {
      sub <- subset(CIA, Code == name)
    } else {
      message("Unable to locate desired table.  See list of available tables.")
      return(CIA)
    }
  }
  code <- sub[["Code"]]
  url <- (paste0("https://www.cia.gov/library/publications/the-world-factbook/rankorder/rawdata_",
                 code, ".txt"))
  message(paste("Retrieving data from", url))
  
  if (! requireNamespace("RCurl")) stop("Package `RCurl' must be installed.")

# it appears that the file format has moved from tab delimited to fixed width
# parsing on two or more spaces seems to work, at least for some files.
  
#  table <- read.delim(textConnection(RCurl::getURL(url, ssl.verifypeer = FALSE)),
#                      header = FALSE, stringsAsFactors = FALSE)
  lines <- readLines(textConnection(RCurl::getURL(url, ssl.verifypeer = FALSE)))
  table <- as.data.frame(do.call(rbind, strsplit( lines, "  +")), stringsAsFactors=FALSE)
  table[, 1] <- NULL
  names(table) <- c("country", name)
  table[[2]] = readr::parse_number(table[[2]])
    # as.numeric(gsub("[^.+[:digit:] ]", "", table[[2]]))
  return(table)
}


#' Transforms a shapefile into a dataframe
#'
#' This function takes in a shapefile (formal class of
#' `SpatialPolygonsDataFrame`) and transforms it into a dataframe
#'
#' @param map A map object of class `SpatialPolygonsDataFrame`
#' @param ... Other arguments, currently ignored
#' @return A dataframe, in which the first 7 columns hold geographical
#' information (ex: `long` and `lat`)
#' @examples
#'
#' \dontrun{
#' if(require(maptools)) {
#'   data(wrld_simpl)
#'   worldmap <- sp2df(wrld_simpl)
#' }
#' 
#' if ( require(ggplot2) && require(maptools) ) {
#'   data(wrld_simpl)
#'   World <- sp2df(wrld_simpl)
#'   World2 <- merge(World, Countries, by.x="NAME", by.y="maptools", all.y=FALSE)
#'   Mdata <- merge(Alcohol, World2, by.x="country", by.y="gapminder", all.y=FALSE)
#'   Mdata <- Mdata[order(Mdata$order),]
#'   qplot( x=long, y=lat, fill=ntiles(alcohol,5),
#'          data=subset(Mdata, year==2008), group = group,
#'                      geom="polygon")
#' }
#' }
#' @export
sp2df <- function (map, ...) 
{
  if (! requireNamespace("sp")) stop( "Package `sp' must be installed.")

  map@data$id <- rownames(map@data)
  coords_matrix <- sp::coordinates(map)  # in sp, which maptools depends on
  map@data$clon = coords_matrix[, 1]
  map@data$clat = coords_matrix[, 2]
  map_points <- do.call(fortify, list(map, region="id"), 
                        envir=environment(ggplot2::fortify))
  result <- merge(map_points, map@data, by = "id")
  return(result)
}

#' ggplot2 theme for maps
#' 
#' A very plain \pkg{ggplot2} theme that is good for maps.
#' 
#' @details
#' This theme is largely based on an example posted by 
#' Winston Chang at the \pkg{ggplot2} Google group forum.
#' 
#' @param base_size the base font size for the theme.
#' @export
theme_map <- function (base_size=12) {
  theme_grey(base_size) %+replace%
    theme(
      axis.title = element_blank (),
      axis.text = element_blank (),
      # axis.text = element_text(margin = grid::unit(0, "cm")),
      panel.background = element_blank (),
      panel.grid = element_blank (),
      axis.ticks.length = grid::unit (0,"cm"),
      # axis.ticks.margin = grid::unit (0.01,"cm"),
      panel.spacing = grid::unit (0,"lines"),
      plot.margin = grid::unit(c(0,0,0,0),"lines"),
      complete = TRUE
    )    
}

# context("maps2")

# turning off because CIAdata load seems to be causing trouble for CRAN
# should replace this with some other data

if (FALSE) {
gdpData <- CIAdata("GDP")      # load some world data
gdpData <- gdpData |> mutate(GDP5 = ntiles(-GDP, 5, format="rank"))


testthat::test_that("World Maps work", {
  

  wrapped_expect_doppelganger("worldmap1", mWorldMap(gdpData, key="country", fill="GDP")) 
  wrapped_expect_doppelganger("worldmap2", mWorldMap(gdpData, key="country", fill="GDP5")) 
  wrapped_expect_doppelganger("worldmap3", mWorldMap(gdpData, key="country", plot="frame") +
                                geom_point()) 
  
  mergedData <- mWorldMap(gdpData, key="country", plot="none")
  wrapped_expect_doppelganger("worldmap4", ggplot(mergedData, aes(x=long, y=lat, group=group, order=order)) +
                                geom_polygon(aes(fill=GDP5), color="gray70", size=.5) + guides(fill=FALSE)) 
  
})

}  # end if(FALSE)


USArrests2 <- USArrests |> tibble::rownames_to_column("state")
testthat::test_that("US Maps work", {
  wrapped_expect_doppelganger("usmaps1", mUSMap(USArrests2, key = "state", fill = "UrbanPop"))
  
})

#' Scatter plot of points
#' 
#' Make or add a scatter plot in a manner coordinated with `plotFun`.
#' 
#' @param x A formula specifying y ~ x or z ~ x&y
#'
#' @param data Data frame containing the variables to be plotted.  If not specified, 
#' the variables will be looked up in the local environment
#'
#' @param add If `TRUE`, add points as a new layer to an existing plot.
#' If `NULL`, the value of `under` will be used.
#' 
#' @param under If `TRUE`, the new layer will be underneath existing layers.
#' 
#' @param panelfun Lattice panel function to be used for adding.  Set only if you want something other
#' than a scatter plot.  Mainly, this is intended to add new functionality through other functions.
#' 
#' @param plotfun Lattice function to be used for initial plot creation. Set only
#' if you want something other than a scatter plot. Mainly, this is intended to add new functionality through
#' other functions.
#' 
#' @param \dots additional arguments
#' 
#' @param plot a trellis plot, by default the most recently created one.  If `add` is `TRUE`,
#' new points will be added as a new layer to `plot`.
#' 
#' @return A trellis graphics object 
#' 
#' @seealso [plotFun()]
#' @examples
#' if (require(mosaicData)) {
#' plotPoints( width ~ length, data=KidsFeet, groups=sex, pch=20)
#' f <- makeFun( lm( width ~ length * sex, data=KidsFeet))
#' plotFun( f(length=length,sex="G")~length, add=TRUE, col="pink")
#' plotFun( f(length=length,sex="B")~length, add=TRUE)
#' }
#' @export

plotPoints <- function( x, data=parent.frame(), add=NULL, under=FALSE,
                        panelfun=panel.xyplot, plotfun=xyplot, ..., plot=trellis.last.object()
                        ) {
  if (is.null(add)) add <- under
  if (!add) return(plotfun(x, data=data, ...))
  # else we're adding on
    rlang::check_installed('latticeExtra')
    xpts <- evalSubFormula(rhs(x),data=data)
    ypts <- evalSubFormula(lhs(x),data=data)
    dots <- list(...)
    plot + latticeExtra::layer( 
      do.call( panelfun, c(list(xpts[[1]],ypts[[1]]),dots)),
               data=as.list(environment()),
               under=under)
}

#' Generate a natural-looking function
#'
#' Produce a random function that is the sum of Gaussian random variables
#'
#' @param vars a formula; the LHS is empty and the RHS indicates the variables used 
#' for input to the function (separated by &)
#'
#' @param seed seed for random number generator, passed to [set.seed()].
#' @param n the number of Gaussians.  By default, this will be selected randomly.
#' @return a function with the appropriate number of inputs
#'
#' @details
#' `rfun` is an easy way to generate a natural-looking but random function with ups and downs
#' much as you might draw on paper.  In two variables, it provides a good way to produce
#' a random landscape that is smooth.
#' Things happen in the domain -5 to 5. The function is pretty flat outside of that.
#' Use `seed` to create a fixed function that will be the same for everybody
#'
#' @rdname rfun
#' @name rfun
#' @keywords random
#' @aliases rfun rpoly2
#'
#' @examples
#' f <- rfun( ~ u & v)
#' plotFun(f(u,v)~u&v,u=range(-5,5),v=range(-5,5))
#' myfun <- rfun(~ u & v, seed=1959)
#' g <- rpoly2( ~ x&y&z, seed=1964)
#' plotFun(g(x,y,z=2)~x&y,xlim=range(-5,5),ylim=range(-5,5))
#' @export
 
rfun <- function(vars=~x&y, seed=NULL, n=0) {
  if (!is.null(seed) ) set.seed(seed)
  if (! inherits(vars, "formula"))
    stop("Must provide a formula, e.g. ~x&y, to identify the variables")
  nmaxes <- ifelse(n==0, ceiling( runif(1,min=4,max=10)), n)
  varnames <- all.vars(vars)
  nvars <- length(varnames)
  locs <- list()
  for (k in 1:nvars) locs[[k]] <- runif(nmaxes, min=-3,max=3)
  signsmax <- runif(nmaxes,min=3,max=10)*sign( runif(nmaxes,min=-1,max=1) )
  xscales <- runif( nmaxes, min=.1, max=5 )
  # The functions are being created by hand for n=1,2,3.
  # For larger n, they are computed
  if( nvars == 1 ) {
    f <- function() {
      x <- eval(parse(text=varnames[1]))
      res <- 0  
      for(k in 1:nmaxes){
        res <- res +  signsmax[k]*exp(-(xscales[k]*(x-locs[[1]][k])^2)/9) 
      }
      return(res)
    }
  }
  if( nvars == 2 ) {
    f <- function() {
      x <- eval(parse(text=varnames[1]))
      y <- eval(parse(text=varnames[2]))
      res <- 0  
      for(k in 1:nmaxes){
        res <- res +  signsmax[k]*exp(-(xscales[k]*(x-locs[[1]][k])^2 + (y-locs[[2]][k])^2)/9) 
      }
      return(res)
    }
  }
  if( nvars == 3 ) {
    f <- function() {
      x <- eval(parse(text=varnames[1]))
      y <- eval(parse(text=varnames[2]))
      z <- eval(parse(text=varnames[3]))
      res <- 0  
      for(k in 1:nmaxes){
        res <- res +  signsmax[k]*exp(-(xscales[k]*(x-locs[[1]][k])^2 + (y-locs[[2]][k])^2 + (z-locs[[3]][k])^2)/9) 
      }
      return(res)
    }
  }
  if( nvars > 3 ) {
    f <- function() {
      x <- eval(parse(text=varnames[1]))
      res <- 0  
      for(k in 1:nmaxes){
        foo <- xscales[k]*(x-locs[[1]][k])^2
        for(j in 2:nvars) {
          x <- eval(parse(text=varnames[j]))
          foo <- foo + (x-locs[[j]][k])^2
        }
        res <- res +  signsmax[k]*exp(-foo/9) 
      }
      return(res)
    }
  }
  tmp <- paste( "alist( ", paste(varnames, "=",collapse=",",sep=""),")")
  tmp <- eval(parse(text=tmp))
  formals(f) <- tmp
  return(f)
}
 
#' random 2nd degree polynomials
#'
#' `rpoly2` generates a random 2nd degree polynomial  (as a function)
#'
#' @rdname rfun
#'
#' @return a function defined by a 2nd degree polynomial
#' with coefficients selected randomly according to a Unif(-1,1) distribution.
#'
#' @keywords random
#' @details These functions are particularly useful for teaching calculus.
#' @export

rpoly2 <- function(vars=~x&y,seed=NULL){
  if (! is.null(seed)) set.seed(round(seed))
  if (! inherits(vars, "formula"))
    stop("Must provide a formula, e.g. ~x&y, to identify the variables")
  varnames <- all.vars(vars)
  if( length(vars)==2) {
    # no left-hand side of the formula (as is sensible)
    # So put a placeholder on the left-hand side
    vars[[3]] <- vars[[2]]
    vars[[2]] <- as.name(varnames[1])
  }
  fres <- makeFun(vars) # function to return
  
  # construct the body of a function
  nvars <- length(varnames)
  # How many coefficients in a 2nd-order polynomial (without intercept)?
  constantval <- runif(1,min=-1,max=1)
  coefslin <- runif(nvars,min=-1,max=1)
  coefsquad <- runif(nvars,min=-1,max=1)
  coefscross <- matrix(runif(nvars*nvars, min=-1,max=1),nrow=nvars)
  f <- function() {
    givenvars <- as.list(match.call())[-1] # arguments as a list
    if( nvars != length(givenvars)) 
      stop("Not all inputs specified.")
    for( k in 1:nvars )
      givenvars[[k]] <- eval.parent(givenvars[[k]],n=1)
    res <- constantval    
    for (k in 1:nvars) {
      x <- givenvars[[k]]
      res <-  res + coefslin[k]*x + coefsquad[k]*x^2
      if( k<(nvars-1)){
        for (j in (k+1):nvars) {
          res <- res + coefscross[k,j]*x*givenvars[[j]]
        }
      }
    }
    return(res)
    }
  
  body(fres) <- body(f)
  environment(fres) <- environment()
  return(fres)
}

# context("rfun()")

testthat::test_that("rfun works", {
  f <- rfun( ~ u & v)
  testcase <- function (u, v) 
  {
    x <- eval(parse(text = varnames[1]))
    y <- eval(parse(text = varnames[2]))
    res <- 0
    for (k in 1:nmaxes) {
      res <- res + signsmax[k] * exp(-(xscales[k] * (x - locs[[1]][k])^2 + 
                                         (y - locs[[2]][k])^2)/9)
    }
    return(res)
  }
  
  wrapped_expect_doppelganger("rfun1", plotFun(f(u,v)~u&v,u=range(-5,5),v=range(-5,5)))
  
  myfun <- rfun( ~ u & v, seed = 1959)
  g <- rpoly2( ~ x & y & z, seed = 1964)
  
  wrapped_expect_doppelganger("rfun2", plotFun(g(x,y,z=2)~x&y,xlim=range(-5,5),ylim=range(-5,5)))
})

#' Central portion of a distribution
#' 
#' This function determines the critical values for isolating 
#' a central portion of a distribution with a specified probability.
#' This is designed to work especially well for symmetric distributions,
#' but it can be used with any distribution.
#' 
#' @inheritParams qdist
#' @param dist a character string naming a distribution family (e.g., "norm").
#'   This will work for any family for which the usual d/p/q functions exist.
#' @param p the proportion to be in the central region, with equal proportions 
#'   in either "tail".
#' @param verbose a logical indicating whether a more verbose output value should be returned.
#' @param pattern One of `"stripes"` or `"rings"`. 
#'   In the latter case, pairs of regions (from the outside to the inside) 
#'   are grouped together for coloring and probability calculation.
#' 
#' @param ... additional arguments passed to the distribution functions.  Typically
#'   these specify the parameters of the particular distribution desired.  See the examples.
# @param tail one of \code{"upper"} or \code{"lower"} specifying whether 
#   the lower or upper critical value is returned.  
# @param warn a logical indicating whether a warning should be given when
#   using a distribution that is not symmetric.
#' @return a pair of numbers indicating the upper and lower bounds, unless `verbose` is 
#'   `TRUE`, in which case a 1-row data frame is returned containing these bounds, 
#'   the central probability, the tail probabilities, and the name of the distribution.
#' @note This function is still experimental and changes the input or output
#'   formats are possible in future versions of the package.
#' 
#' @examples
#' cdist( "norm", .95)
#' cdist( "t", c(.90, .95, .99), df=5)
#' cdist( "t", c(.90, .95, .99), df=50)
#' # plotting doesn't work well when the parameters are not constant
#' cdist( "t", .95, df=c(3,5,10,20), plot = FALSE)
#' cdist( "norm", .95, mean=500, sd=100 )
#' cdist( "chisq", c(.90, .95), df=3 )
#' # CI
#' x <- rnorm(23, mean = 10, sd = 2)
#' cdist("t", p = 0.95, df=22)
#' mean(x) + cdist("t", p = 0.95, df=22) * sd(x) / sqrt(23)
#' confint(t.test(x))
#' cdist("t", p = 0.95, df=22, verbose = TRUE)
#' @export
# # another possible implementation
# cdist <- 
#   function( dist, p, ..., verbose = FALSE ) {
#     alpha <- (1-p)/2
#     lo <- alpha
#     hi <- 1 - alpha
#     qdist <- paste0("q", dist)
#     loQ <- do.call( dpqrdist, c(list(dist = dist, type="q", p=lo), list(...) ) ) 
#     hiQ <- do.call( dpqrdist, c(list(dist = dist, type="q", p=hi), list(...) ) ) 
#     if (verbose) {
#       as.data.frame(
#         c( 
#           list(
#             lo = loQ,
#             hi = hiQ,
#             central_p = p, 
#             tail_p = alpha,
#             dist = dist
#             ),
#           list(...)
#         )
#       )
#     } else {
#        res <- cbind(loQ, hiQ)
#        if (nrow(res) == 1) {
#          res <- res[1, , drop=TRUE]
#          names(res) <- NULL
#        }
#        res
#     }
#   }

cdist <- function (
  dist = "norm", p, plot = TRUE, verbose = FALSE, invisible = FALSE, 
  digits = 3L, 
  xlim = NULL, ylim = NULL,
  resolution = 500L,
  return = c("values", "plot"),
  pattern = c("rings", "stripes"),
  ...,
  refinements = list())
{
  
  pattern <- match.arg(pattern)
  return <- match.arg(return)
  
  dots <- list(...)
  alpha <- (1-p)/2
  p <- sort(c(alpha, 1-alpha))
  q <- dpqrdist(dist, type = "q", p = p, ...) 
  
  if (verbose) {
    cat("Verbose output not yet implemented.\n")
  }
 
  if (return == "plot" || plot) { 
    res_plot <-
      do.call(
        gf_refine,
        c(list(
          plot_multi_dist(
            dist = dist, p = p, 
            xlim = xlim, ylim = ylim, 
            digits = digits, 
            resolution = resolution,
            pattern = pattern,
            ...)),
          refinements)
      )
  }
  
  if (return == "plot") {
    return(res_plot)
  }
  if (plot) {
    print(res_plot)
  }
  if (invisible) { 
    return(invisible(q))
  }
  return(q)
}

# 
# .cdist <- function( dist, p, ... , tail=c("upper","lower"), warn=TRUE) {
#   tail = match.arg(tail)
#   alpha <- (1-p)/2
#   lo <- alpha
#   hi <- 1 - alpha
#   qdist <- paste0("q", dist)
#   loT <- do.call( dpqrdist, c(list(dist = dist, type="q", lo), list(...) ) ) 
#   hiT <- do.call( dpqrdist, c(list(dist = dist, type="q", hi), list(...) ) ) 
#   # hiT <- do.call( qdist, c(list(hi), list(...) ) ) 
#   if ( any( abs(hiT) - abs(loT) > 1e-5 ) ) {
#     if (warn) warning(paste0("It looks like your distribution is not symmetric.  I'm providing ", tail, " tails.") )
#   }
#   
#   return(
#     switch( tail,
#             "lower" = loT,
#             "upper" = hiT
#     )
#   )
# }
# 

#' Return the path to a documentation file in a package
#' 
#' Return the path to a documentation file in a package
#' 
#' @param file the name of a file
#' @param package the name of a package
#' @param character.only a logical. If `TRUE` package names must be
#' specified as character, else names will be converted as a convenience
#' as is [library()] and [library()].
#' @return a character vector specifying the path to the file on the user's system.
#' @export

docFile <- function(file, package="mosaic", character.only=FALSE) {
  if (! character.only)  package <- as.character(substitute(package))
  path <- find.package(package, verbose=FALSE)
  return(file.path(path,"doc",file))
}

# Function to calculate odds ratios and confidence intervals
# on odds ratios.

# Written by Kevin Middleton; largely redone by R. Pruim

# Successes in Column 1
# Treatment of interest in Row 2



#' Odds Ratio and Relative Risk for 2 x 2 Contingency Tables
#' 
#' This function calculates the odds ratio and relative risk for a 2 x 2 
#' contingency table and a
#' confidence interval (default `conf.level` is 95 percent) for the
#' each estimate. `x` should be a matrix, data frame or table. "Successes"
#' should be located in column 1 of `x`, and the treatment of interest
#' should be located in row 2. The odds ratio is calculated as (Odds row 2) /
#' (Odds row 1). The confidence interval is calculated from the log(OR) and
#' backtransformed.
#' 
#' 
#' @rdname oddsRatio
#' @param x a 2 x 2 matrix, data frame, or table of counts
#' @param object an R object to print or summarise.  Here an object of class
#' `"oddsRatio"` or `"relrisk"`.
#' @param conf.level the confidence interval level
#' @param verbose a logical indicating whether verbose output should be displayed
#' @param quiet a logical indicating whether verbose output should be suppressed
#' @param relrisk a logical indicating whether the relative risk should be returned
#' instead of the odds ratio
#' @param digits number of digits to display

#' @param ... additional arguments
#' @return an odds ratio or relative risk.  If `verpose` is true,
#' more details and the confidence intervals are displayed.
#' @author Kevin Middleton (\email{kmm@@csusb.edu}); modified by 
#' R Pruim.
#' @seealso [chisq.test()], [fisher.test()]
#' @keywords stats
#' @examples
#' M1 <- matrix(c(14, 38, 51, 11), nrow = 2)
#' M1
#' oddsRatio(M1)
#' 
#' M2 <- matrix(c(18515, 18496, 1427, 1438), nrow = 2)
#' rownames(M2) <- c("Placebo", "Aspirin")
#' colnames(M2) <- c("No", "Yes")
#' M2
#' oddsRatio(M2)
#' oddsRatio(M2, verbose = TRUE)
#' relrisk(M2, verbose = TRUE)
#' if (require(mosaicData)) {
#' relrisk(tally(~ homeless + sex, data = HELPrct) )
#' do(3) * relrisk( tally( ~ homeless + shuffle(sex), data = HELPrct) )
#' }
#' @export

orrr <- function(x, conf.level = 0.95, verbose = !quiet, quiet = TRUE, digits = 3,
                      relrisk = FALSE){
  if (any(dim(x) != c(2,2))) {
    stop("expecting something 2 x 2")
  }
  x <- as.matrix(x)
  names(x) <- NULL
  row.names(x) <- NULL
  colnames(x) <- NULL
  rowsums <- rowSums(x)
  p1 <- x[1, 1] / rowsums[1]
  p2 <- x[2, 1] / rowsums[2]
  o1 <- p1 / (1 - p1)
  o2 <- p2 / (1 - p2)
  RR <- p2 / p1
  OR <- o2 / o1
  crit <- qnorm((1 - conf.level)/2, lower.tail = FALSE)
  
  names(RR) <- "RR"
  log.RR <- log(RR)
  SE.log.RR <- sqrt( sum( x[,2]/x[,1]/rowsums) )
  log.lower.RR <- log.RR - crit * SE.log.RR
  log.upper.RR <- log.RR + crit * SE.log.RR
  lower.RR <- exp(log.lower.RR)
  upper.RR <- exp(log.upper.RR)
  
  names(OR) <- "OR"
  log.OR <- log(OR)
  SE.log.OR <- sqrt(sum(1/x))
  log.lower.OR <- log.OR - crit * SE.log.OR
  log.upper.OR <- log.OR + crit * SE.log.OR
  lower.OR <- exp(log.lower.OR)
  upper.OR <- exp(log.upper.OR)

  res <- if (relrisk) {
    structure(RR,
              p1 = p1, 
              p2 = p2, 
              o1 = o1, 
              o2 = o2, 
              OR = OR, 
              lower.OR = lower.OR, 
              upper.OR = upper.OR, 
              RR = RR,
              lower.RR = lower.RR, 
              upper.RR = upper.RR, 
              conf.level = conf.level,
              class=c("relrisk", "numeric"))
  } else {  
    structure(OR,
              p1 = p1, 
              p2 = p2, 
              o1 = o1, 
              o2 = o2, 
              OR = OR, 
              lower.OR = lower.OR, 
              upper.OR = upper.OR, 
              RR = RR,
              lower.RR = lower.RR, 
              upper.RR = upper.RR, 
              conf.level = conf.level,
              class=c("oddsRatio", "numeric"))
  }
  if (verbose) print(summary(res))
  res
}

#' @rdname oddsRatio
#' @export
oddsRatio <- function(x, conf.level = 0.95, verbose = !quiet, quiet = TRUE, digits = 3) {
  orrr(x, conf.level = conf.level, verbose = verbose, digits = digits, relrisk = FALSE)
}

#' @rdname oddsRatio
#' @export
relrisk <- function(x, conf.level = 0.95, verbose = !quiet, quiet = TRUE, digits = 3) {
  orrr(x, conf.level = conf.level, verbose = verbose, digits = digits, relrisk = TRUE)
}

#' @rdname oddsRatio
#' @export
print.oddsRatio <- function(x, digits = 4, ...) {
  print(as.numeric(x))
}

#' @rdname oddsRatio
#' @export
print.relrisk <- function(x, digits = 4, ...) {
  print(as.numeric(x))
}

#' @rdname oddsRatio
#' @export
summary.oddsRatio <-
          function(object, digits = 4, ...){
            summary_relrisk_oddsratio(object, digits = digits, ...) 
          }

#' @rdname oddsRatio
#' @export
summary.relrisk <- 
  function(object, digits = 4, ...){
    summary_relrisk_oddsratio(object, digits = digits, ...) 
  }

summary_relrisk_oddsratio <- function(x, digits = 4, ...){
  cat("\n")
  cat("Odds Ratio\n")
  cat("\n")
  cat("Proportions\n")
  cat("\t   Prop. 1:\t", format(attr(x,"p1"), digits = digits), "\n")
  cat("\t   Prop. 2:\t", format(attr(x,"p2"), digits = digits), "\n")
  cat("\t Rel. Risk:\t", format(attr(x,"RR"), digits = digits), "\n\n")
  cat("Odds\n")
  cat("\t    Odds 1:\t", format(attr(x,"o1"), digits = digits), "\n")
  cat("\t    Odds 2:\t", format(attr(x,"o2"), digits = digits), "\n")
  cat("\tOdds Ratio:\t", format(attr(x,"OR"), digits = digits), "\n\n")
  cat(format(100 * attr(x,"conf.level")), "percent confidence interval:\n")
  cat("\t", format(attr(x,"lower.RR"), digits = digits), "< RR <", 
      format(attr(x,"upper.RR"), digits = digits), "\n")
  cat("\t", format(attr(x,"lower.OR"), digits = digits), "< OR <", 
      format(attr(x,"upper.OR"), digits = digits), "\n")
}

#'Find the zeros of a function of two or more variables
#'
#'Compute numerically zeros of a function of two or more variables.
#'All free variables (all but the variable on the right side) named in the expression must be assigned 
#' a value via `\ldots`
#'
#'@param ... arguments for values NOTE: if the system has more than one equation and the rhs
#'variables do not match up, there will be an error.
#'@param npts number of desired zeros to return
#'@param rad radius around near in which to look for zeros
#'@param near center of search for zeros
#'@param sortBy options for sorting zeros for plotting.  Options are 'byx', 'byy' and 'radial'.  The
#'default value is 'byx'.
#'
#'@details sorts points in the domain according to the sign of the function value at respective points.
#' Use continuity and uniroot to find zeros between points of opposite signs.  Returns any number of
#' points which may be sorted and plotted according to x, y, or radial values.
#' 
#'@return A data frame of numerical values which should all result in a value of zero when input into
#' original function
#' 
#'@author Cecylia Bocovich
#' 
#'@examples
#' findZerosMult(a*x^2-8~a&x, npts = 50)
#' findZerosMult(a^2+x^2-8~a&x, npts = 100, sortBy='radial')
#' \dontrun{findZerosMult(a^2+x^2-8~a&x, npts = 1000, sortBy='radial')}
#' @export

findZerosMult <- function(..., npts=10, rad = 5, near=0, sortBy='byx'){
  dots = list(...)
  system = list()
  freeVars = list()
  roots = data.frame()#where we will store the roots.
  
  # Separate formulae and free vars
  for(i in (1:length(dots))){
    if( inherits(dots[[i]], "formula"))
      system = append(system, dots[[i]]) # system contains all equations
    else{
      if (inherits(dots[[i]], "numeric"))
        freeVars[[names(dots)[i]]] <- dots[[i]] # freeVars contains values we will sub in
      else stop(paste("Improper value: ", deparse(dots[[i]])))
    }
  }
  numEq = length(system)
  
  rhsVars = all.vars(rhs(system[[1]]))
  for(i in (1:length(system))){
    rhsVars = union(rhsVars, all.vars(rhs(system[[i]])))
  }
  
  allvars = all.vars(system[[1]])
  for(i in (1:length(system))){
    allvars = union(allvars, all.vars(system[[i]]))
  }
  
  #Change all formula in the system are functions. Make sure rhs is consistent
  for(i in (1:numEq)){
    system[[i]][[3]]<- parse(text=paste(allvars, collapse="&"))[[1]]
    system[[i]] = do.call(makeFun, as.list(c(system[[i]], freeVars)))
  }
  
  if(length(near)==1)
    near = rep(0, length(rhsVars))
  
  #if there is only one equation, use uniroot
  if(length(system)==1){ 
    points = .findPoints(system, freeVars, rhsVars, rad, near, npts= npts*2)
    
    for(j in (1:length(rows(points[[1]])))){ #Loop over all positive points
      if(length(points[[1]])==0) break
      pt1= points[[1]][j,]
      
      for(k in (1:length(rows(points[[2]])))){#Loop over all negative points
        if(length(points[[2]])==0) break
        pt2 = points[[2]][k,]
        
        newf<-function(t){}
        newbody = "{"
        for(l in (1:length(rhsVars))){
          newbody = paste(newbody,rhsVars[l], "=t*",toString(pt1[l]),
                          "+(1-t)*",toString(pt2[l]),"\n")
        }
        newbody = parse(text = paste(newbody, 
                                     "do.call(system[[1]],as.list(c(parse(text=rhsVars), freeVars)))}"))
        body(newf)<- newbody
        
        troot = uniroot(newf, c(0,1))$root
        root=pt1
        root = troot*pt1+(1-troot)*pt2
        roots <- rbind(roots, root)
        if(length(rows(roots)) >= npts){
          colnames(roots) <- rhsVars
          roots = .sort(roots, near = near, type=sortBy)
          rownames(roots) <- (1:(length(rows(roots))))
          return(roots) #return ordered by the first variable.
        }
      }
    }
    if(length(roots)==0) stop("No roots found - try widening search")
    colnames(roots) = rhsVars
    roots = .sort(roots, near = near, type=sortBy)
    rownames(roots) <- (1:(length(rows(roots))))
    return(roots)
  }
  
  #Use Broyden when system has more than one equation.
  if(length(system)>1){
    set.seed(23)
    
    if(length(system) < length(rhsVars)){ #Need to add equations
      need = length(rhsVars) - length(system)
      junk = runif(length(rhsVars), min=-2, max=2)
      if(1< need)
        for(i in 1:need)
          junk = cbind(junk, runif(length(rhsVars), min=-2,max=2))
      
      points = .findPoints(system, freeVars, rhsVars, rad, near, npts=npts*2)
      
      for(i in (1:length(system))){
        for(j in (1:(length(rows(points[[i]]))))){#might be a more efficient way to do this
          if(length(points[[i]])==0) break
          for(k in (1:(length(rows(points[[length(points)-i+1]]))))){
            if(length(points[[length(points)-i+1]])==0) break
            for(l in (1:need)){
              pt1 = points[[i]][j,]
              pt2 = points[[length(points)-i+1]][k,]
              
              m = cbind(t(pt2-pt1), junk)
              norm = qr(m)$qr[,2]
              hyperplane = function(){}
              body(hyperplane) <- parse(text=paste("(c(",
                                                   toString(norm), ")%*%(c(", paste(rhsVars, collapse=","), ")-c(", toString(pt1), ")))[[1]]", sep=""
              ))
              formals(hyperplane) <- eval(parse( 
                text=paste( "as.pairlist(alist(", 
                            paste(rhsVars, "=", collapse=",", sep=""), "))"
                )
              ))
              environment(hyperplane) <- environment(system[[1]])
              if(l>1)
                newsystem = append(newsystem, hyperplane)
              else
                newsystem = append(system, hyperplane)
            }
            root = try(Broyden(newsystem, rhsVars, x=near+.001, maxiters=1e3), silent=TRUE)
            if(!inherits(root, "try-error")){
              roots = rbind(roots, root)
            }
            if(length(rows(roots)) >= npts){
              colnames(roots) = rhsVars
              roots = .sort(roots, near = near, type=sortBy)
              rownames(roots) <- (1:(length(rows(roots))))
              return(roots) #return ordered by the first variable.
            }
          }
        }
      }
      if(length(roots)==0) stop("No zeros found - try widening search")
      colnames(roots) = rhsVars
      roots = .sort(roots, near = near, type=sortBy)
      rownames(roots) <- (1:(length(rows(roots))))
      return(roots)
      
    }
    
    else{ #equal number of equations and variables
      root = try(Broyden(system, rhsVars, x=near+.001), silent=TRUE)
      if(!inherits(root, "try-error"))
        roots = rbind(roots, root)
      set.seed(35)
      for(i in (1:(npts-1))){
        startingVec <- runif(1, min=near[1]-rad, max=near[1]+rad)
        for(j in (1:(length(near)-1)))
          startingVec <- append(startingVec, runif(1, min=near[j+1]-rad, max=near[j+1]-rad))
        root = try(Broyden(system, rhsVars, x=startingVec), silent=TRUE)
        if(!inherits(root, "try-error")){
          roots = rbind(roots, root)
        }
      }
      if(length(roots)==0) stop("None found - try widening search")
      colnames(roots) = rhsVars
      return(roots)
    }
  }
}

#sort the list of zeros by x, y, or radially.
.sort <- function(data, near=c(0,0), type = 'byx'){
  #introduce some sort of infer args business...
  
  if(type == 'byx'){
    if (inherits(data, "data.frame")) {
      return(data[order(data[[1]]),])
    } else {
      return(data[order(data[1]),])
    }
  }
  
  if(type == 'byy'){
    if (inherits(data, "data.frame")) {
      return(data[order(data[[2]]),])
    } else {
      return(data[order(data[2]),])
    }
  }
  
  if(type == 'radial'){
    npts = length(rows(data))
    ref = near
    data$angle <- apply(data, 1, function(row){angle <- atan((ref[2]-row[2])/(ref[1]-row[1]))
                                               if(sign(ref[1]-row[1])==-1) angle = angle+pi
                                               if(sign(angle)==-1) angle = 2*pi+angle
                                               return(angle)})
    data <- data[order(data$angle),]
    
    #     for(i in (1:(npts-2))){
    #      if(sqrt((data[i,1]-data[i+1,1])^2+(data[i,2]-data[i+1,2])^2) >= 
    #        sqrt((data[i,1]-data[i+2,1])^2+(data[i,2]-data[i+2,2])^2)){
    #        ref.x = mean(data[i,1],data[i+1,1],data[i+2,1])
    #        ref.y = mean(data[i,2],data[i+1,2],data[i+2,2])
    #        ref = c(ref.x,ref.y)
    #         
    #         #find new angles with new reference point
    #        data$angle <- apply(data, 1, function(row){angle <- atan((ref[2]-row[2])/(ref[1]-row[1]))
    #                                                   if(sign(ref[1]-row[1])==-1) angle = angle+pi
    #                                                   if(sign(angle)==-1) angle = 2*pi+angle
    #                                                   return(angle)})
    #         
    #        data[i:npts,]=data[i-1+order(data$angle[i:npts]),]
    #      }
    #     }
    
    data$angle = NULL
    return(data)
  }
  stop("Incorrect entry for type")
}

#returns npts number of points sorted into bins.  the ith bin can be paired with the
#ith to last bin and used to find the equation of a line with a zero on it.
#
.findPoints<- function(system, freeVars, vars, rad, near, npts){
  numBins = 2^(length(system))
  points = list()
  for(i in (1:numBins)) points[[i]] = data.frame()
  set.seed(1) #set seed for random number generation
  for(p in (1:npts)){
    #Construct a point within search range
    pt =runif(length(vars), min=near-rad, max=near+rad)    
    binNum = 1
    for(i in (1:(length(system)))){
      if(!sign(do.call(system[[i]], as.list(c(pt, freeVars))) )==1) #Positive
        binNum = binNum + 2^(i-1)
    }
    points[[binNum]] = rbind(points[[binNum]], pt)
  }
  #   if(length(points[[1]])==0||length(points[[2]])==0){
  #     warning("No zeros found. Try choosing a different start value or widening your search.")
  #     return(numeric(0))
  #   }
  for(i in (1:length(points))) try(colnames(points[[i]]) <- vars, silent=TRUE)
  return(points)
}
#'Multi-Dimensional Root Finding
#'
#'Implementation of Broyden's root finding function to numerically compute the root of
#'a system of nonlinear equations
#'
#'@param system A list of functions
#'
#'@param vars A character string list of variables that appear in the functions
#'
#'@param x A starting vector
#'
#'@param tol The tolerance for the function specifying how precise it will be
#'
#'@param maxiters maximum number of iterations.
#'
#'@export

Broyden <- function(system, vars, x=0, tol = .Machine$double.eps^0.4, maxiters=1e4){
  n = length(system)
  if(is.null(x)) x = rep(1,length(system))#Add in something that makes sure this is valid.
  if(toString(names(x))=="") names(x) = vars
  
  A = diag(n) #Default derivative is the identity matrix
  
  #Evaluates a system of equations at a given point. FIX.
  .evalSys <- function(.x.,System){
    n=length(System)
    FF = rep(0,n)
    for( i in (1:n)){
      if(length(formals(System[[i]]))< length(.x.))
        FF[i]=do.call(System[[i]], as.list(.x.[-which(names(.x.)==
          setdiff(names(.x.), names(formals(System[[i]]))))]))
      else FF[i] = do.call(System[[i]], as.list(.x.))
    }
    return(FF)
  }
  
  FF=.evalSys(x,system)
  
  for(iter in (1:maxiters)){
    if(max(abs(FF))<tol) 
      return(x)
    xnew=as.vector(x-A%*%FF)
    names(xnew)=names(x)
    del = xnew-x
    FFnew = .evalSys(xnew,system)
    Del = FFnew-FF
    Anew = A+((del-A%*%Del)%*%t(del)%*%A)/(t(del)%*%A%*%Del)[[1]]
    x=xnew
    A=Anew
    FF=FFnew
  }
  
  stop("Failed to converge")
  
}

#' @rdname defunct
#' @export

xhistogram <- function (...) { # x, data=NULL, panel=panel.xhistogram, type='density', 
                               #       center=NULL, width=NULL, ...) {
   .Defunct(msg = "xhistogram() is no longer needed; simply use histogram() with the mosaic package attached.")
}

#' Augmented histograms
#' 
#' The \pkg{mosaic} package adds some additional functionality to 
#' [lattice::histogram()], making it simpler to obtain certain common 
#' histogram adornments.  This is done be resetting the default panel
#' and prepanel functions used by histogram.
#' 
#' @rdname xhistogram
#' @aliases histogram
#' @param x a formula or a numeric vector
#' @param panel a panel function
#' @param type one of `'density'`, `'count'`, or `'percent'`
#' @param nint approximate number of bins
#' @param breaks break points for histogram bins, a function for computing such,
#'        or a method [hist()] knows about given as a character string.
#'        When using the \pkg{mosaic} package defaults, 
#'        [mosaic::xhistogramBreaks()] is used.
#' @param \dots additional arguments passed from [lattice::histogram()] 
#' to the panel function; by default when the \pkg{mosaic} package has been loaded this 
#' will be [panel.xhistogram()].
#'
#' @seealso [lattice::histogram()], [mosaicLatticeOptions()],
#' and [restoreLatticeOptions()].
#' 
#' @note Versions of \pkg{lattice} since 0.20-21 support setting custom defaults 
#' for `breaks`, `panel`, and `prepanel` used by 
#' `histogram()`, so `xhistogram()` is no longer needed.  
#' As a result, `xhistogram()` 
#' (which was required in earlier versions of \pkg{mosaic}
#' is no longer needed and has been removed.
#' 
#' @details
#' The primary additional functionality added to [histogram()]
#' are the arguments `width` and `center` which provide a simple
#' way of describing equal-sized bins, and `fit` which can be used to
#' overlay the density curve for one of several distributions.  The
#' `groups` argument can be used to color the bins.  The primary use
#' for this is to shade tails of histograms, but there may be other uses
#' as well.
#'

#' @return `xhistogramBreaks` returns a vector of break points
#' @examples
#' if (require(mosaicData)) {
#' histogram(~age | substance, HELPrct, v=35, fit='normal')
#' histogram(~age, HELPrct, labels=TRUE, type='count')
#' histogram(~age, HELPrct, groups=cut(age, seq(10,80,by=10)))
#' histogram(~age, HELPrct, groups=sex, stripes='horizontal')
#' histogram(~racegrp, HELPrct, groups=substance,auto.key=TRUE)
#' xhistogramBreaks(1:10, center=5, width=1)
#' xhistogramBreaks(1:10, center=5, width=2)
#' xhistogramBreaks(0:10, center=15, width=3)
#' xhistogramBreaks(1:100, center=50, width=3)
#' xhistogramBreaks(0:10, center=5, nint=5)
#' }
#' @export

xhistogramBreaks <- function(x, center=NULL, width=NULL, nint, ...) {
  x <- x[!is.na(x)]
  if (is.factor(x)) return(seq_len(1 + nlevels(x)) - 0.5)
  if (length(x) < 2) return (x)
  
  if (is.null(center)) { center <- 0 }
  if (missing(nint) || is.null(nint)) { 
    nint <- round(1.5 *log2(length(x)) + 1) 
  }
  
  if (is.null(width)) { 
    nint <- max(nint - 1, 1)
    width <- diff(range(x)) / nint
  }
  if (width <= 0) { width <- diff(range(c(0,x))) / nint }
  
  if (width <= 0) {stop("`width' too small.")}

  shift <- ( (floor( (min(x) - center)/width) ):(1 + ceiling( (max(x) - center)/width)) )
  breaks <-  -.5 * width + center + shift * width
# was an attempt to pretty-up the breaks
#  digits <- 15
#  while ( digits > 2 && diff(range(diff(breaks))) > 0) {
#    digits <- digits - 1 
#    width <- round(width,digits)
#    breaks <-  -.5 * width + center + shift * width
#  }
  
  if (breaks[2] < min(x)) breaks <- tail(breaks,-1)
  if (breaks[length(breaks)-1] > max(x)) breaks <- head(breaks,-1)
  
  if (min(breaks) > min(x) || max(breaks) < max(x)) 
	  stop("Bug alert: break points don't cover data.")
  return(breaks)
}

#' @rdname xhistogram
#' @export

prepanel.xhistogram <- 
  function (x, breaks=xhistogramBreaks, ...) 
  {
    if (is.function(breaks))  {
      breaks <- breaks(x, ...)
    }
    lattice::prepanel.default.histogram(x, breaks = breaks, ...)
  }

#' @rdname xhistogram
#' @param dcol color of density curve
#' @param dalpha alpha for density curve
#' @param gcol color of guidelines
#' @param fcol fill colors for histogram rectangles when using `groups`.  
#' (Use `col`, which is passed through to the histogram panel function,
#' when not using `groups`.)
#' 
#' @param dmath density function for density curve overlay
#' @param verbose be verbose?
#' @param dn number of points to sample from density curve
#' @param dlwd,glwd like `lwd` but affecting the density line and guide lines, respectively
#' @param args a list of additional arguments for `dmath`
#' @param labels should counts/densities/percents be displayed or each bin?
#' @param density a logical indicating whether to overlay a density curve
#' @param under a logical indicating whether the density layers should be under or 
#' over other layers of the plot.
#' @param fit a character string describing the distribution to fit.  Known distributions include
#'      `"exponential"`, `"normal"`, `"lognormal" `, `"poisson"`, `"beta"`, `"geometric"`,
#'      `"t"`, `"weibull"`, `"cauchy"`, `"gamma"`, `"chisq"`, and `"chi-squared"`
#'        
#' @param start numeric value passed to [MASS::fitdistr()]
#' @param center center of one of the bins
#' @param width width of the bins
#' @param groups as per [lattice::histogram()]
#' @param stripes one of `"vertical"`, `"horizontal"`, or `"none"`, indicating
#'        how bins should be striped when `groups` is not `NULL`
#' @param h,v a vector of values for additional horizontal and vertical lines
#' @param alpha transparency level
#' @export
panel.xhistogram <-
function (x, 
	dcol = trellis.par.get("plot.line")$col, dalpha=1, dlwd = 2, 
  gcol = trellis.par.get("add.line")$col, glwd = 2, 
	fcol = trellis.par.get("superpose.polygon")$col,
	dmath = dnorm, 
	verbose = FALSE,
    dn = 100, args = NULL, labels = FALSE, density = NULL, under=FALSE, fit = NULL, 
    start = NULL, type = "density", v, h, groups=NULL, center=NULL, width=NULL, breaks,
    nint = round(1.5 * log2(length(x)) + 1),
	stripes=c('vertical','horizontal','none'), alpha=1, ...) 
{
  if (is.null(density)) density <- under
  if (missing(breaks) || is.null(breaks)) {
    breaks <- xhistogramBreaks(x, center=center, width=width, nint=nint)
  } 
  if (is.function(breaks))   {
    breaks <- breaks(x, center = center, width = width, nint = nint, ...)
  }
  stripes <- match.arg(stripes)
  
  if (!is.null(fit)) {
    dmath = switch( tolower(fit), 
                    "exponential" = dexp,
                    "normal"      = dnorm,
                    "lognormal"   = dlnorm,
                    "log-normal"  = dlnorm,
                    "poisson"     = dpois,
                    "beta"        = dbeta,
                    "t"           = dt3,
                    "weibull"     = dweibull,
                    "cauchy"      = dcauchy,
                    "gamma"       = dgamma,
                    "chisq"       = dchisq,
                    "chi-squared" = dchisq,
                    "chi.squared" = dchisq,
                    "kde"         = NULL  # just a place holder, not used
    )
    x = x[!is.na(x)]
    density <- TRUE
    if (is.null(args) && ! fit == "kde") {
      if (is.null(start)) {
        args = fitdistr(x, fit)$estimate
      }
      else {
        args = fitdistr(x, fit, start = start)$estimate
      }
    }
  } 
  if (is.null(args)) {
    args <- if (!is.null(fit) && fit == "kde") list() else 
        list(mean = base::mean(x, na.rm = TRUE), 
             sd = stats::sd(x, na.rm = TRUE))
  }
  
  ###  done cleaining up args; away we go
  if (density && type != 'density') {
    warning("Use type='density' when adding density overlays.")
  }
  if (density && verbose) {
    cat("args for density function:\n")
    print(args)
  }
  if ( density && under ) {  # else do this near the end
    if (fit == "kde") {
      if (is.null(args)) args <- list()
      panel.densityplot(x, darg = args, plot.points = FALSE) 
    } else {
      panel.mathdensity(dmath = dmath, args = args, n = dn, 
                      col = dcol, alpha=dalpha,lwd = dlwd)
    }
  }

  ## plotting main part of histogram 
  if (!is.null(groups)) {
    	hist.master <- hist(as.numeric(x), plot = FALSE, breaks=breaks, warn.unused=FALSE, ...)
		hist.master$height <- switch(type,
			'density' = hist.master$density,
			'count' = hist.master$count,
			'percent' = 100 * hist.master$count / length(x)
			)
		nbreaks <- length(hist.master$breaks)
		groups <- factor(groups)
		ngroups <- nlevels(groups)
		props <- (table(groups))/length(groups)
		fcol <- rep(fcol, length=length(props))
		cumdensity= rep(0, length(hist.master$mids))
		cumrdensity= rep(0, length(hist.master$mids))
		for (level in 1:ngroups) {
			hist.level <- hist(
				as.numeric(x)[groups==levels(groups)[level] ], 
				plot=FALSE,
				breaks=hist.master$breaks,
				warn.unused=FALSE,
				...
			)
			hist.level$density <- hist.level$density * props[level]
			hist.level$rdensity <- hist.level$density / hist.master$density 
			switch( stripes, 
				vertical = 
				grid.rect(
					x=hist.level$breaks[-nbreaks] + cumrdensity*diff(breaks),
					y=0,
					width=diff(hist.level$breaks) * hist.level$rdensity,
					height=hist.master$height,
					just=c('left','bottom'),
					default.units='native',
					gp=gpar(col=fcol[level], fill=fcol[level],alpha=alpha),
					),
				horizontal = 
				grid.rect(
					x=hist.level$breaks[-nbreaks],
					y=0 + cumrdensity* hist.master$density,
					width=diff(hist.level$breaks),
					height=hist.master$height * hist.level$rdensity,
					just=c('left','bottom'),
					default.units='native',
					gp=gpar(col=fcol[level], fill=fcol[level],alpha=alpha),
					),
				none=
				grid.rect(
					x=hist.level$breaks[-nbreaks],
					y=0,
					width=diff(hist.level$breaks),
					height=hist.level$height,
					just=c('left','bottom'),
					default.units='native',
					gp=gpar(col='black', fill=fcol[level],alpha=alpha),
					)
			)
			cumdensity <- cumdensity + hist.level$density
			cumrdensity <- cumrdensity + hist.level$rdensity
			if (stripes != 'none') {
			grid.rect(
				x=hist.master$breaks[-nbreaks],
				y=0,
				width=diff(hist.master$breaks),
				height=hist.master$height,
				just=c('left','bottom'),
				default.units='native',
				gp=gpar(col='black', fill='transparent'),
				)
			}
		}
		if (verbose) { print(hist.master) }
	} else {
    	panel.histogram(x, type = type, breaks=breaks, ...)
	}
    if (labels) {
        myhist <- hist(x, plot = FALSE, warn.unused=FALSE, breaks=breaks, ...)
        if (type == "count") {
            aa <- max(myhist$counts) * 0.02
            grid.text(label = as.character(round(myhist$counts, 3)), 
                x = myhist$mids, y = aa + myhist$counts, just = c("centre", 
                  "bottom"), default.units = "native")
        }
        else if (type == "percent") {
            sumCounts <- sum(myhist$counts)
            aa <- max(myhist$counts/sumCounts) * 0.04
            grid.text(label = as.character(round(100*myhist$counts/sumCounts, 
                1)), x = myhist$mids, y = aa + (100*myhist$counts/sumCounts), just = c("centre", 
                "bottom"), default.units = "native")
        }
        else {
            aa <- max(myhist$density) * 0.02
            grid.text(label = as.character(round(myhist$density, 3)), 
                x = myhist$mids, y = aa + myhist$density, just = c("centre", 
                  "bottom"), default.units = "native")
        }
    }
  ## additional adornments
  
  if ( density && !under ) {
    if (fit == "kde") {
      if (is.null(args)) args <- list()
      panel.densityplot(x, darg = args, plot.points = FALSE, 
                        lwd = dlwd, col = dcol, alpha = dalpha) 
    } else {
      panel.mathdensity(dmath = dmath, args = args, n = dn, 
                      col = dcol, alpha = dalpha, lwd = dlwd)
    }
  }
  if (!missing(v)) {
        for (x in v) {
            panel.abline(v = x, col = gcol, lwd = glwd)
        }
    }
    if (!missing(h)) {
        for (y in h) {
            panel.abline(h = y, col = gcol, lwd = glwd)
        }
    }
}

dt3 <- function (x, df, m=0, s=1, log = FALSE) {
  if (log) 
    return( dt( (x-m)/s, df=df, log=TRUE)  - log(s) )
  dt( (x-m)/s, df=df, log=FALSE) / s
}  

#' Extract Chi-squared statistic
#' 
#' Extract Chi-squared statistic
#' 
#' @param x An object of class `"htest"` a coming from a Chi-squared test,
#' an object of class `"table"`, or
#' the inputs to [tally()].
#' @param correct a logical indicating whether a continuity correction should be 
#' applied.
#' @param ... additional arguments passed on to `tally` or `chisq.test`.
#' @seealso [ggplot2::after_stat()]
#' @examples
#' 
#' if(require(mosaicData)) {
#'   Mites.table <- tally( ~ outcome + treatment, data=Mites )
#'   Mites.table 
#'   chisq.test(Mites.table)
#'   chisq(Mites.table)
#'   chisq(chisq.test(Mites.table))
#'   ## Randomization test.  Increase replications to decrease Monte Carlo error.
#'   do(3) * chisq( tally( ~ outcome + shuffle(treatment),  data=Mites ) )
#'   Mites.rand <- do(1000) * chisq( tally( ~ outcome + shuffle(treatment),  data=Mites ) )
#'   tally( ~(X.squared >= chisq(Mites.table)), data=Mites.rand, format="proportion")
#' }
#'   
#' @export
chisq <- function(x, ...) {
  UseMethod("chisq")
}

#' @rdname chisq
#' @export
chisq.htest <- function(x, ...) {
  if (! grepl ("Chi-squared", x$method) ) {
    stop("This doesn't look like a Chi-sqaured test to me.")
  }
  setNames(x$statistic, "X.squared")
}

#' @rdname chisq
#' @export
chisq.table <- function(x, correct = FALSE, ...) {
  setNames(
  suppressWarnings(chisq.test(x, correct = correct, ...)$statistic),
    "X.squared"
  )
}
  
#' @rdname chisq
#' @export
chisq.default <- function(x, correct = FALSE, ...) {
  dots <- list(...)
  chisq_dots <- dots[intersect(names(dots), names(formals(chisq.test)))]
  # remove chisq.test() ... from tally_dots
  tally_dots <- dots
  tally_dots[names(chisq_dots)] <- NULL
  
  tally_res <- do.call(tally, c(list(x), tally_dots)) 
  setNames(
    suppressWarnings(
    do.call(
      chisq.test, 
      c(list(tally_res, correct = correct), chisq_dots)
    )$statistic),
    "X.squared"
  )
}

testthat::test_that("chisq works", {
  require(mosaicData)
  Mites.table <- tally( ~ outcome + treatment, data=Mites )
  ## Randomization test.  Increase replications to decrease Monte Carlo error.
  set.seed(5)
  do(3) * chisq( tally( ~ outcome + shuffle(treatment),  data=Mites ) )
  Mites.rand <- do(1000) * chisq( tally( ~ outcome + shuffle(treatment),  data=Mites ) )
  
  testcase1 <- 
    structure(
      list(statistic = c(`X-squared` = 5.68847857969099), 
           parameter = c(df = 1L), p.value = 0.0170766534882757, method = "Pearson's Chi-squared test with Yates' continuity correction", 
           data.name = "Mites.table", observed = structure(c(15L, 11L, 4L, 17L), 
                                                           .Dim = c(2L, 2L), 
                                                           .Dimnames = list(
                                                             outcome = c("no wilt", "wilt"), 
                                                             treatment = c("mites", "no mites")), 
                                                           class = "table"), 
           expected = structure(c(10.5106382978723, 15.4893617021277, 
                                  8.48936170212766, 12.5106382978723), 
                                .Dim = c(2L, 2L), 
                                .Dimnames = list(
                                  outcome = c("no wilt", "wilt"), 
                                  treatment = c("mites",  "no mites"))), 
           residuals = structure(c(1.38474578496035,  -1.14069053162584, -1.54080286914851, 1.26924325246454), 
                                 .Dim = c(2L, 2L), class = "table", 
                                 .Dimnames = list(outcome = c("no wilt", "wilt"), 
                                                  treatment = c("mites", "no mites"))), 
           stdres = structure(c(2.68397982358462,  -2.68397982358462, -2.68397982358462, 2.68397982358462), 
                              .Dim = c(2L, 2L), class = "table", 
                              .Dimnames = list(outcome = c("no wilt",  "wilt"), 
                                               treatment = c("mites", "no mites")))), class = "htest")
  
  testcase2 <- c(X.squared = 7.20374769340935)
  testcase3 <- c(X.squared = 5.68847857969099)
  
  expect_equal(ignore_attr = TRUE, chisq.test(Mites.table), testcase1)
  expect_equal(ignore_attr = TRUE, chisq(Mites.table), testcase2)
  expect_equal(ignore_attr = TRUE, chisq(chisq.test(Mites.table)), testcase3)
})

#' Create vector based on roughly equally sized groups
#'
#' @param x a numeric vector
#' @param n (approximate) number of quantiles
#' @param format a specification of desired output format.
#' @param digits desired number of digits for labeling of factors.
#' @return a vector.  The type of vector will depend on `format`.
#' @examples
#' if (require(mosaicData)) {
#' tally( ~ ntiles(age, 4), data=HELPrct)
#' tally( ~ ntiles(age, 4, format="center"), data=HELPrct)
#' tally( ~ ntiles(age, 4, format="interval"), data=HELPrct)
#' tally( ~ ntiles(age, 4, format="left"), data=HELPrct)
#' tally( ~ ntiles(age, 4, format="right"), data=HELPrct)
#' tally( ~ ntiles(age, 4, format="mean"), data=HELPrct)
#' tally( ~ ntiles(age, 4, format="median"), data=HELPrct)
#' bwplot( i2 ~ ntiles(age, n=5, format="interval"), data=HELPrct)
#' }
#' @export

ntiles <-  function(x, n=3, 
                    format=c("rank", "interval", "mean", "median", "center", "left", "right"), 
                    digits=3){
  format <- match.arg(format)
  
  # Figure out names
  qnames <- c("1st","2nd","3rd","4th","5th","6th","7th","8th","9th","10th")
  if (n > 10) qnames <- c(qnames, paste(11:n,"th",sep=""))
  
  xrank <- rank(x, na.last=TRUE, ties.method='first')
  xrank[is.na(x)] <- NA
  size <- max(xrank, na.rm=TRUE)
  cts <- round( seq(1, size, length.out = (n+1) ) )
  bin <- as.numeric(cut( xrank, breaks = cts, include.lowest=TRUE ))
  
  left <- min( x ~ bin, na.rm=TRUE )
  right <- max( x ~ bin, na.rm=TRUE )
  md <- median( x ~ bin, na.rm=TRUE )
  mn <- mean( x ~ bin, na.rm=TRUE )
  center <- signif( ( left + right ) / 2, digits )
  
  res <- switch(format,
                "rank" =  factor(bin, labels=qnames[1:n], ordered=TRUE),
                "interval" =  factor(
                  bin,  
                  labels=paste0("[",signif(left,digits=digits),",", signif(right,digits=digits),"]"), 
                  ordered=TRUE),
                "center" = center[bin],
                "mean" = mn[bin],
                "median" = md[bin],
                "left" = left[bin],
                "right" = right[bin]
  )                
  return(res)
}

#' @import mosaicCore
NA

#' Average Shifted Histograms
#' 
#' An ASH plot is the average over all histograms of a fixed bin width. 
#' 
#' @param x A formula or numeric vector.
#' @param width The histogram bin width.
#' @param adjust A numeric adjustment to `width`.  Primarily useful when `width` is 
#'   not specified.  Increasing `adjust` makes the plot smoother.
#' @param panel A panel function.
#' @param prepanel A prepanel function.
#' @param ... Additional arguments passed to panel and prepanel functions or `data`, a 
#'   data frame in which to find the variables used for the plot.
#' @param data A data frame.
#' @export
#' @examples
#' ashplot( ~age | substance, groups = sex, data = HELPrct)
ashplot <- function(x, data = data, ..., width = NULL, adjust = NULL, 
                    panel = panel.ashplot, prepanel = prepanel.default.ashplot) 
{
  densityplot(x, data = data, panel = panel, width = width, ..., prepanel = prepanel)
}

#' @rdname ashplot
#' @param darg a list of arguments for the function computing the ASH.
#' @param groups as in other lattice plots
#' @param subscripts as in other lattice prepanel functions
#' 
#' @export
prepanel.default.ashplot <- function(
  x, darg, groups = NULL, subscripts = TRUE, 
  ...) 
{
  if (!is.numeric(x)) 
    x <- as.numeric(x)
  if (sum(!is.na(x)) < 1) 
    prepanel.null()
  else if (sum(!is.na(x)) == 1) {
    list(xlim = rep(x, 2), ylim = rep(0, 2), dx = 1, dy = 1)
  }
  else if (is.null(groups)) {
    h <- mosaicCore::ash_points(x, binwidth = darg$width, adjust = darg$adjust)
    quants <- quantile(x, c(0.15, 0.85), names = FALSE, na.rm = TRUE)
    ok <- h$x > quants[1] & h$x < quants[2]
    list(xlim = range(h$x), ylim = range(h$y), dx = diff(h$x[ok]), 
         dy = diff(h$y[ok]))
  }
  else {
    vals <- sort(unique(groups))
    xl <- range(x, finite = TRUE)
    yl <- 0
    dxl <- numeric(0)
    dyl <- numeric(0)
    for (i in seq_along(vals)) {
      id <- (groups[subscripts] == vals[i])
      if (sum(id, na.rm = TRUE) > 1) {
        h <- do.call(mosaicCore::ash_points, c(list(x = x[id], binwidth = darg$width, darg$adjust))) 
        xl <- c(xl, h$x)
        yl <- c(yl, h$y)
        quants <- quantile(x[id], c(0.15, 0.85), names = FALSE, na.rm = TRUE)
        ok <- h$x > quants[1] & h$x < quants[2]
        dxl <- c(dxl, diff(h$x[ok]))
        dyl <- c(dyl, diff(h$y[ok]))
      }
    }
    list(xlim = range(xl, finite = TRUE), 
         ylim = range(yl, finite = TRUE), dx = dxl, dy = dyl)
  }
}

#' @rdname ashplot
#' @param plot.points One of `TRUE`, `FALSE`, `"jitter"`, or `"rug"` 
#' @param ref a logical indicating whether a reference line should be displayed
#' @param jitter.amount when `plot.points="jitter"`, the value to use as the amount 
#' argument to [jitter()].
#' @param type type argument used to plot points, if requested. 
#'   This is not expected to be useful, it is available mostly to protect a `type` argument, 
#'   if specified, from affecting the display of the ASH.
#' @param identifier A character string that is prepended to the names of i
#'   grobs that are created by this panel function.
#' @export
panel.ashplot <- 
  function (
    x, 
    darg = list(),
    plot.points = FALSE,
    ref = FALSE, 
    groups = NULL, 
    jitter.amount = 0.01 * diff(current.panel.limits()$ylim), 
    type = "p", 
    ..., 
    identifier = "ash") 
  {
    if (ref) {
      reference.line <- trellis.par.get("reference.line")
      panel.abline(h = 0, col = reference.line$col, lty = reference.line$lty, 
                   lwd = reference.line$lwd, identifier = paste(identifier, 
                                                                "abline"))
    }
    if (!is.null(groups)) {
      panel.superpose(x, darg = darg, plot.points = plot.points, 
                      ref = FALSE, groups = groups, 
                      panel.groups = panel.ashplot, 
                      jitter.amount = jitter.amount, 
                      type = type, ...)
    }
    else {
      switch(
        as.character(plot.points), 
        `TRUE` = panel.xyplot(x = x, y = rep(0, length(x)), type = type, ..., identifier = identifier), 
        rug = panel.rug(x = x, start = 0, end = 0, 
                        x.units = c("npc", "native"), 
                        type = type, ..., 
                        identifier = paste(identifier, "rug")), 
        jitter = panel.xyplot(x = x, y = jitter(rep(0, length(x)), amount = jitter.amount), 
                              type = type, ..., identifier = identifier)
      )
      
      ash <- mosaicCore::ash_points(x, darg$width, adjust=darg$adjust)    
      if (sum(!is.na(x)) > 1) {
        panel.xyplot(ash$x, ash$y, type="l", ...)
      }
    }
  }

# context("ashplot()")
ashplot( ~age | substance, groups = sex, data = HELPrct)

testthat::test_that("Ashplots work", {
  wrapped_expect_doppelganger("ashplots1", ashplot( ~age | substance, groups = sex, data = HELPrct))
})

utils::globalVariables(c("SE.star", "estimate.star", ".index", "SE"))

#' Confidence interval methods for output of resampling
#'
#' Methods for `confint` to compute confidence intervals
#' on numerical vectors and numerical components of data frames.
#'
#' @rdname confint
#' @name confint
#'
#' @param method a character vector of methods to use for creating confidence
#' intervals.  Choices are "percentile" (or "quantile") which is the default,
#' "stderr" (or "se"), "bootstrap-t", and
#' "reverse" (or "basic"))
#'
#' @param margin.of.error if true, report intervals as a center and margin of error.
#' @param df degrees for freedom. This is required when `object` was produced using
#' \code{link{do}} when
#' using the standard error to compute the confidence interval since
#' typically this information is not recorded in these objects.  The default (`Inf`)
#' uses a normal critical value rather than a one derived from a t-distribution.
#'
#' @param ... additional arguments
#'
#' @return When applied to a data frame, returns a data frame giving the
#' confidence interval for each variable in the data frame using
#' `t.test` or `binom.test`, unless the data frame was produced using `do`, in which case
#' it is assumed that each variable contains resampled statistics that serve as an estimated sampling
#' distribution from which a confidence interval can be computed using either a central proportion
#' of this distribution or using the standard error as estimated by the standard deviation of the
#' estimated sampling distribution.  For the standard error method, the user must supply the correct
#' degrees of freedom for the t distribution since this information is typically not available in
#' the output of [do()].
#'
#' When applied to a numerical vector, returns a vector.
#'
#' @details
#' The methods of producing confidence intervals from bootstrap distributions are currently
#' quite naive.  In particular, when using the standard error, assistance may be required with the
#' degrees of freedom, and it may not be possible to provide a correct value in all situations.
#' None of the methods include explicit bias correction.
#' Let \eqn{q_a} be the \eqn{a} quantile of the bootstrap distribution,
#' let \eqn{t_a, df} be the \eqn{a} quantile of the t distribution with \eqn{df}
#' degrees of freedom,
#' let \eqn{SE_b} be the standard deviation of the bootstrap distribution,
#' and let \eqn{\hat{\theta}} be the estimate computed from the original data.
#' Then the confidence intervals with confidence level \eqn{1 - 2a} are
#' \describe{
#' \item{quantile}{\eqn{(q_a, q_{1-a}) } }
#' \item{reverse}{ \eqn{( 2 \hat{\theta} - q_{1-a}, 2\hat{\theta} - q_{a} )}}
#' \item{stderr}{\eqn{(\hat{\theta} - t_{1-a,df} SE_b, \hat{\theta} + t_{1-a,df} SE_b) }.
#'  When `df` is not provided,
#' at attempt is made to determine an appropriate value, but this should be double checked.
#' In particular, missing data an lead to unreliable results.
#' }
#' The bootstrap-t confidence interval is computed much like the reverse confidence interval
#' but the bootstrap t distribution is used in place of a theoretical t distribution.
#' This interval has much better properties than the reverse (or basic) method, which
#' is here for comparison purposes only and is not recommended.  The t-statistic
#' is computed from a mean, a standard deviation, a sample size which much be named
#' `"mean"`, `"sd"`, and `"n"` as they are when using [favstats()].
#' }
#' @references
#' Tim C. Hesterberg (2015): What Teachers Should Know about the Bootstrap:
#' Resampling in the Undergraduate Statistics Curriculum,
#' The American Statistician,
#' \url{https://www.tandfonline.com/doi/full/10.1080/00031305.2015.1089789}.
#'
#' @examples
#' if (require(mosaicData)) {
#'   bootstrap <- do(500) * diffmean( age ~ sex, data = resample(HELPrct) )
#'   confint(bootstrap)
#'   confint(bootstrap, method = "percentile")
#'   confint(bootstrap, method = "boot")
#'   confint(bootstrap, method = "se", df = nrow(HELPrct) - 1)
#'   confint(bootstrap, margin.of.error = FALSE)
#'   confint(bootstrap, margin.of.error = TRUE, level = 0.99,
#'     method = c("se", "perc") )
#'
#'   # bootstrap t method requires both mean and sd
#'   bootstrap2 <- do(500) * favstats(resample(1:10))
#'   confint(bootstrap2, method = "boot")
#' }
#' @export

confint.numeric <- function(object, parm, level = 0.95, ...,
                            method = "percentile",
                            margin.of.error = "stderr" %in% method == "stderr") {

  if ("conf.level" %in% names(list(...))) {
    stop("Use `level` to set the confidence level, not `conf.level`.")
  }
  method <- match.arg(method, c("stderr","percentile","quantile"), several.ok = TRUE)
  result <- list()
  for (m in method) {
    vals <- .mosaic.get.ci( object, level, m )
    result[[m]] <-  if( margin.of.error ) {
      c(center = mean(vals), margin.of.error = diff(vals) / 2,
        method = m, level = level)
    } else {
      vals
    }
  }
  if (length(result) > 1L) {
    result <- result[[1L]]
  }  else {
    result <- as.data.frame(do.call(rbind, result))
  }
  message(paste0(
    "Confidence Interval from Bootstrap Distribution (",
    length(object),
    " replicates)")
    )
  result
}

dont_randomize_data <- list(
  mean = function(x, ...) x,
  median = function(x, ...) x,
  sd = function(x, ...) x,
  max = function(x, ...) x,
  min = function(x, ...) x,
  sample = function(x, ...) x,
  resample = function(x, ...)
    if ( inherits(x, "lm") ) eval( x$call[["data"]], environment(formula(x)) ) else x,
  shuffle = function(x, ...) x,
  rflip = function(n, prob = 0.5, quiet, verbose, ...) {
    c( x = round(prob * n), n = n )
  }
)

dont_randomize_estimate <- list(
  sample = function(x, ...) x,
  resample = function(x, ...)
    if ( inherits(x, "lm") ) eval( x$call[["data"]], environment(formula(x)) ) else x,
  shuffle = function(x, ...) x,
  rflip = function(n, prob = 0.5, quiet, verbose, ...) {
    c(prop = prob)
  }
)

extract_data <- function(x) {
  x_lazy <- attr(x, "lazy")
  x_lazy_env <- attr(x, "lazy_env")
  if (is.null(x_lazy)) return(NULL)
  res <- eval(
    # rlang::f_rhs(x_lazy)[["data"]],
    x_lazy[["data"]],
    envir = dont_randomize_data,
    enclos = x_lazy_env
  )
  if (is.null(res)) {
    res <- eval(
      # rlang::f_rhs(x_lazy),
      x_lazy,
      envir = dont_randomize_data,
      enclos = x_lazy_env
    )
  }
  as.data.frame(res)
}

extract_estimate <- function(x) {
  x_lazy <- attr(x, "lazy")
  x_lazy_env <- attr(x, "lazy_env")
  if (is.null(x_lazy)) return(NA)
  if (rlang::is_formula(x_lazy)) {
    x_lazy <- rlang::f_rhs(x_lazy)
  }
  eval(
    # rlang::f_rhs(x_lazy),
    x_lazy,
    envir = dont_randomize_estimate,
    enclos = x_lazy_env)
}

#' @rdname confint
#' @export
confint.do.tbl_df <- function(object, parm, level = 0.95, ...,
                                 method = "percentile",
                                 margin.of.error = "stderr" %in% method,
                                 df = NULL) {

  if ("conf.level" %in% names(list(...))) {
    stop("Use `level` to set the confidence level, not `conf.level`.")
  }
  class(object) <- c("do.data.frame", class(object))
  confint(object, parm, level = level, ..., method = method,
          margin.of.error = margin.of.error, df = df)
}

#' @rdname confint
#' @export
confint.do.data.frame <- function(object, parm, level = 0.95, ...,
                                 method = "percentile",
                                 margin.of.error = "stderr" %in% method,
                                 df = NULL) {

  method <- tolower(method)
  method <-
    match.arg(
      method,
      c("percentile", "se", "stderr", "basic", "reverse",
        "quantile", "bootstrap-t"),
      several.ok = TRUE) # which method was selected
  method[method == "quantile"] <- "percentile"
  method[method == "basic"] <- "reverse"
  method[method == 'se'] <- 'stderr'
  method <- unique(method)

  bootT <- ("bootstrap-t" %in% method) &&
    all(c("mean", "sd", "n") %in% names(object))
  method <- setdiff(method, "bootstrap-t")

  compute_t_df <-
    grepl("^diffmean$|^mean$", as.character(attr(object, "lazy")[[1]]))
    # grepl("^diffmean$|^mean$", as.character(rlang::f_rhs(attr(object, "lazy"))[[1]]))

  if ("stderr" %in% method && is.null(df) && compute_t_df) {
    tryCatch({
      orig_data <- extract_data(object)
      orig_data <-
        orig_data |>
        select(
          .dots = intersect(names(orig_data),
          attr(object, "lazy") |> all.vars())
          #rlang::f_rhs(attr(object, "lazy")) |> all.vars())
        )
      df <- nrow(orig_data) - 1
      if ( ! all(complete.cases(orig_data)) ) {
        warning(
          "confint: Some missingness in the data. Check to make sure df is correct.",
          call. = FALSE)
      }
    },
      error = function(e) warning(
        "confint: I can't determine df from the original data.",
        call. = FALSE)
    )
  }

  if (is.null(df) || length(df) != 1) {
    df <- Inf
    if ("stderr" %in% method) warning("confint: Using df = Inf.", call. = FALSE)
  }

  if (missing(parm)) parm <- names(object)
  nms <- intersect(names(object),parm)
  res <- data.frame(
    name = character(),
    lower = double(),
    upper = double(),
    level = double(),
    method = character(),
    stringsAsFactors = FALSE
  )
  row <- 0
  culler <- attr(object, "culler")
  estimate <- culler(extract_estimate(object))
  if (is.null(names(estimate))) {
    names(estimate) <- names(object)
  }

  for (k in 1:length(nms) ) {
    for (m in method) {
      for (l in level) {
        if (is.numeric( object[[nms[k]]] )) {
          if (! nms[k] %in% names(estimate) ) next
          row <- row + 1
          # vals <- .mosaic.get.ci2( object[[nms[k]]], l, m, df = df)
          vals <-
            bootstrap_ci(
              object[[nms[k]]], level = l, method = m, df = df,
              estimate =  estimate[[ nms[k] ]]
            )
          res[row, "name"] <- nms[k]
          res[row, "estimate"] <- estimate[[ nms[k] ]]
          res[row, "lower"] <- vals[1]
          res[row, "upper"] <- vals[2]
          res[row, "level"] <- l
          res[row, "method"] <- m
        }
      }
    }
  }
  if (bootT) {
    res <- bind_rows(res, boott(object))
  }
  if (prod(dim(res)) == 0L) {
    warning("confint: Unable to compute any of the desired CIs", call. = FALSE)
    return(res)
  }
#  res <- subset(res, !is.na(res$name) ) # get rid of non-quantitative variables
  if( margin.of.error && prod(dim(res)) > 0 ) {
#     res[, "estimate"] <- with( res, (upper+lower)/2 )
    res[, "margin.of.error"] <- with( res,  (res$upper-res$lower)/2 )
#    res[ res$method != "stderr", "estimate"] <- NA
    res[ res$method != "stderr", "margin.of.error"] <- NA
  }
#  else {
#    res <- res[ , setdiff(names(res), "estimate") ]
#  }

  # Change the names to those given by confint.default
#  colnames(res) <-
#    if (method == "quantile")
#      c("name", paste(c((1-level)/2, 1-(1-level)/2)*100, "%" ))
#    else c("name", "lower", "upper")

#  if (margin.of.error)  # Report as a center and margin of error
#    res = .turn.to.margin(res)

  if (("stderr" %in% method) && (df < Inf)) {
    res$df = df
    res$df[res$method != "stderr"] <- NA
  }
  res <- res |> filter( !is.na(lower) & !is.na(upper) & !(lower == upper) )
  return( as.data.frame(res) )
}

.turn.to.margin <- function(res) {
  data.frame( name = res$name, center = (res[[2]]+res[[3]])/2,
              margin.of.error = (res[[3]]-res[[2]])/2)
}

.mosaic.get.ci <- function( vals, level, method, df = NULL) {
  alpha <- (1-level)/2
  if( method == "stderr" ) {
    if (is.null(df)) { df <- sum(!is.na(vals)) - 1 }
    res = mean(vals, na.rm = TRUE) +
      c(-1,1) * sd(vals, na.rm = TRUE) * qt(1-alpha, df)
  }
  # the sum(!is.na(vals)) above is to account for NAs in finding the degrees of freedom
  else res = stats::quantile(vals, c(alpha, 1-alpha) )
  return(res)
}

.mosaic.get.ci2 <- function( vals, level, method, df = Inf) {
  alpha <- (1 - level) / 2
  if( method == "stderr" ) {
    res = mean(vals, na.rm = TRUE) +
    c(-1,1) * sd(vals, na.rm = TRUE) * qt(1-alpha, df)
  }
  # the sum(!is.na(vals)) above is to account for NAs in finding the degrees of freedom
  else res = stats::quantile(vals, c(alpha, 1-alpha) )
  return(res)
}

bootstrap_ci <- function( x, level = 0.95, method,
                          df = Inf, t, ... ) {
  switch(
    method,
    `stderr` = tse_bootstrap_ci(x, level = level, df = df, ...),
    `percentile` = percentile_bootstrap_ci(x, level = level, ...),
    `reverse` = basic_bootstrap_ci(x, level = level, ...),
    `bootstrap-t` = boott_bootstrap_ci(x, level = level, estimate, ...)
  )
}

basic_bootstrap_ci <- function( x, estimate, ..., df = Inf, level = 0.95 ) {
  alpha <- (1 - level) / 2
  2 * estimate - stats::quantile(x, c(1-alpha, alpha))
}

percentile_bootstrap_ci <- function( x, ..., level = 0.95 ) {
  alpha <- (1 - level) / 2
  stats::quantile(x, c(alpha, 1-alpha) )
}

tse_bootstrap_ci <- function( x, ..., df = Inf, level = 0.95 ) {
  alpha <- (1 - level) / 2
  mean(x, na.rm = TRUE) +
    c(-1,1) * sd(x, na.rm = TRUE) * qt(1-alpha, df)
}

boott_bootstrap_ci <- function( x, se, ..., level = 0.95, estimate ) {
  alpha <- (1 - level) / 2
  t <- (x - mean(x)) / se
  q <- stats::quantile(t, c(upper = 1 - alpha, lower = alpha), na.rm = TRUE)
  as.vector(estimate - q * SE)
}



#' @rdname confint
#' @export

confint.data.frame <- function(object, parm, level = 0.95, ... )  {
  if ("conf.level" %in% names(list(...))) {
    stop("Use `level` to set the confidence level, not `conf.level`.")
  }
  results <- list()
  for (c in 1:ncol(object)) {
    x <- object[,c]
    if (is.numeric(x)) {
      newCI <- confint(t.test(x, ...))
      newRow <- data.frame( method = "t.test", estimate = newCI[1], lower = newCI[2],
                            upper = newCI[3], level = newCI[4])

    } else if ( (is.factor(x) && nlevels(x) <= 2) || (is.character(x) && length(unique(x)) <= 2) || is.logical(x)) {
      newCI <- confint(binom.test(x, ...))
      newRow <- data.frame( method = "binom.test", estimate = newCI[1], lower = newCI[2], upper = newCI[3], level = newCI[4])
    } else {
      newRow <- data.frame(method = "none", estimate = NA, lower = NA, upper = NA, level = NA)
    }
    results <- rbind(results,newRow)
  }

  row.names(results) <- names(object)

  return(results)
}

boott <- function(object, ...) {
  UseMethod("boott")
}

#' @export
boott.do.data.frame <- function( object, level = 0.95, ... ) {
  lz <- attr(object, "lazy")
  if (! all( c("mean", "sd", "n") %in% names(object))) {
    stop("Invalid object: Bootstrap-t requires columns named `mean', `sd', and `n'",
         call. = FALSE)
  }
  if (base::max(object$.row) > 2) stop("Too many groups.")

  estimate <- extract_estimate(object)$mean

  if (max(object$.row) == 1) {
    parm <- "mean"
    Boot <-
      object |>
      mutate(estimate.star = mean, SE.star = sd / sqrt(n),
             t = (estimate.star - mean(estimate.star)) / SE.star)
  }

  if (max(object$.row) == 2) {
    parm <- "diffmean"
    Boot <-
      object |>
      group_by(.index) |>
      summarise(estimate.star = diff(mean), SE.star = sqrt(sum(sd^2/ n))) |>
      mutate(t = (estimate.star - mean(estimate.star)) / SE.star)
    estimate <- diff(estimate)
  }

  q <- cdata( ~ t, level, data = Boot)
  res <-
    data.frame(
      name = parm,
      estimate = estimate,
      lower = estimate - q[, 2] * sd(Boot$estimate.star),
      upper = estimate - q[, 1] * sd(Boot$estimate.star),
      level = level,
      method = "bootstrap-t",
      stringsAsFactors = FALSE
    )
  row.names(res) <- NULL
  res
}

# context("binom.test()")

  TestData <- data.frame( a = factor(rep(letters[1:3], length.out = 100)),
                          b = rep(letters[1:3], length.out = 100), 
                          c = rep(c(TRUE, FALSE, FALSE), length.out = 100), 
                          d = rep(c(TRUE, FALSE, FALSE), length.out = 100), 
                          stringsAsFactors = FALSE
  )
  
test_that("formulas work", {
 
  X <- stats::binom.test(34, 100)
  A <- binom.test(~ a, data=TestData)
  B <- binom.test(~ b, data=TestData)
  C <- binom.test(~ c, data=TestData)
  
  expect_equal(ignore_attr = TRUE, confint(A), confint(X))
  expect_match(A$data.name, "TestData\\$a")
  
  expect_equal(ignore_attr = TRUE, confint(B), confint(X))
  expect_match(B$data.name, "TestData\\$b")
  
  expect_equal(ignore_attr = TRUE, confint(C), confint(X))
  expect_match(C$data.name, "TestData\\$c")
  
})

test_that("formula + unnamed second arg data frame throws an error", {
  expect_error(binom.test(~ a, TestData), "did you forget")
})

# No longer supporting this.
#
# test_that("formulas work with unnamed second arg", {
#  
#   X <- stats::binom.test(34, 100)
#   A <- binom.test(~ a, TestData)
#   B <- binom.test(~ b, TestData)
#   C <- binom.test(~ c, TestData)
#   
#   expect_equal(ignore_attr = TRUE, confint(A), confint(X))
#   expect_match(A$data.name, "TestData\\$a")
#   
#   expect_equal(ignore_attr = TRUE, confint(B), confint(X))
#   expect_match(B$data.name, "TestData\\$b")
#   
#   expect_equal(ignore_attr = TRUE, confint(C), confint(X))
#   expect_match(C$data.name, "TestData\\$c")
# })
  
test_that("success = works", {
  
  X <- stats::binom.test(33, 100)
  Y <- stats::binom.test(66, 100)
  A <- binom.test(~ a, data=TestData, success = "b")
  B <- binom.test(~ b, data=TestData, success = "b")
  C <- binom.test(~ c, data=TestData, success = FALSE)
  
  expect_equal(ignore_attr = TRUE, confint(A), confint(X))
  expect_match(A$data.name, "TestData\\$a")
  expect_match(A$data.name, "success = b")
  
  expect_equal(ignore_attr = TRUE, confint(B), confint(X))
  expect_match(B$data.name, "TestData\\$b")
  expect_match(B$data.name, "success = b")
  
  expect_equal(ignore_attr = TRUE, confint(C), confint(Y))
  expect_match(C$data.name, "TestData\\$c")
  expect_match(C$data.name, "success = FALSE")
})

test_that("bare vars throw error", {
  expect_error(binom.test(a, data = TestData), "first argument should be a formula")
  expect_error(binom.test(b, data = TestData), "first argument should be a formula")
  expect_error(binom.test(d, data = TestData), "first argument should be a formula")
})


# test_that("bare vars work", {
#   X <- stats::binom.test(34, 100)
#   A <- binom.test( a, data=TestData)
#   B <- binom.test( b, data=TestData)
#   C <- binom.test( c, data=TestData)
#   
#   expect_equal(ignore_attr = TRUE, confint(A), confint(X))
#   expect_match(A$data.name, "a")
#   expect_match(A$data.name, "success = a")
#   
#   expect_equal(ignore_attr = TRUE, confint(B), confint(X))
#   expect_match(B$data.name, "b")
#   expect_match(B$data.name, "success = a")
#   
#   expect_equal(ignore_attr = TRUE, confint(C), confint(X))
#   expect_match(C$data.name, "c")
#   expect_match(C$data.name, "success = TRUE")
# })

test_that("numbers work", {
  expect_equal(ignore_attr = TRUE,  
    confint(stats::binom.test(33, 100)),
    confint(binom.test(33, 100))
  )
  
})
  
  test_that("x treated as raw data when n is missing", {
    X <- resample(1:3, 100)
    x <- sum(X == min(X))
    expect_equal(ignore_attr = TRUE,   
      confint(binom.test(X)), 
      confint(binom.test(x, 100)) )
  })  
  
test_that("CI methods correct", {

  # Clopper-Pearson, the default but with 3 names
  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::binom.test(26,200)),
    confint(binom.test(26,200)))
  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::binom.test(26,200)),
    confint(binom.test(26,200, ci.method="clopper-pearson")))
  
  expect_equal(ignore_attr = TRUE, 
    confint(binom.test(26,200, ci.method="clopper-pearson")),
    confint(binom.test(26,200, ci.method="binom.test")))
 
  # Score/Wilson/prop.test  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::prop.test(26,200)),
    confint(binom.test(26,200, ci.method="prop.test", correct = TRUE)))
  
  expect_equal(ignore_attr = TRUE, 
    confint(stats::prop.test(26,200, correct = FALSE)),
    confint(binom.test(26,200, ci.method = "wilson")))
  
  expect_equal(ignore_attr = TRUE, 
    confint(binom.test(26,200, ci.method = "score")),
    confint(binom.test(26,200, ci.method = "wilson")))
  
  # Clopper vs external reference 
  # NIST example from http://www.itl.nist.gov/div898/handbook/prc/section2/prc241.htm
  ci <- confint(binom.test(4, 20, ci.method="clopper", conf.level=.9))
  expect_equal(ci[1,2], 0.071354, tolerance = 1e-5)
  expect_equal(ci[1,3], 0.401029, tolerance = 1e-5)

  # Wald vs external reference 
  # from http://www.stat.wmich.edu/s160/book/node47.html 
  ci <- confint(binom.test(15, 59, ci.method="Wald"))
  expect_equal(ci[1,2], 0.143, tolerance = 1e-3)
  expect_equal(ci[1,3], 0.365, tolerance = 1e-3)
 
  # Plus4 
  ci <- confint(binom.test(0, 100, ci.method="Plus4"))
  expect_equal(ci[1,2], 0.0,    tolerance = 1e-3)
  expect_equal(ci[1,3], 0.0456, tolerance = 1e-3)
  
})

test_that("binom.test compatibile with dplyr", {
  expect_equal(ignore_attr = TRUE, 
    data.frame(x = rep(c('a', 'b'), c(5, 10))) |>
      summarise(pval = pval(binom.test( ~ x))),
    data.frame(x = rep(c('a', 'b'), c(5, 10))) |>
      summarise(pval = pval(stats::binom.test(5, 15)))
  )
})

# context('do()')

test_that("naming as expected", {
  expect_equal(ignore_attr = TRUE,  names( do(2) * mean(~cesd, data=mosaicData::HELPrct)), "mean")
  expect_equal(ignore_attr = TRUE,  names( do(2) * var(~cesd, data=mosaicData::HELPrct)), "var")
  expect_equal(ignore_attr = TRUE,  names( do(2) * {var(~cesd, data=mosaicData::HELPrct)}), "result")
  expect_equal(ignore_attr = TRUE,  names( do(2) * 5 + 3), "result")
  expect_equal(ignore_attr = TRUE,  names( do(2) * lm(cesd ~ age + sex, data=mosaicData::HELPrct)), 
     c("Intercept", "age", "sexmale", "sigma", "r.squared", "F", "numdf", "dendf", ".row", ".index"))
})


test_that("expression is re-evaluated on each iteration", {
  set.seed(1)
  result <- do(5) * mean(~cesd, data = resample(mosaicData::HELPrct))
  expect_gt(length(unique(result[[1]])), 1)
})

test_that("dimension as expected", {
  expect_equal(ignore_attr = TRUE,  dim( do(2) * mean(~cesd, data=mosaicData::HELPrct)), c(2,1))
  expect_equal(ignore_attr = TRUE,  dim( do(2) * var(~cesd, data=mosaicData::HELPrct)), c(2,1))
  expect_equal(ignore_attr = TRUE,  dim( do(2) * {var(~cesd, data=mosaicData::HELPrct)}), c(2,1))
  expect_equal(ignore_attr = TRUE,  dim( do(2) * 5 + 3), c(2,1))
  expect_equal(ignore_attr = TRUE,  dim( do(2) * lm(cesd ~ age + sex, data=mosaicData::HELPrct)), 
                      c(2,10))
  expect_equal(ignore_attr = TRUE,  dim( do(2) * anova(lm(cesd ~ age + sex, data=mosaicData::HELPrct))), 
                      c(6,8))
})

# context('Aggregating Functions')

test_that("formula interface works", {
  expect_equal(ignore_attr = TRUE,  mean(~cesd, data=mosaicData::HELPrct), mean(mosaicData::HELPrct$cesd))
  expect_equal(ignore_attr = TRUE,  var(~cesd, data=mosaicData::HELPrct), var(mosaicData::HELPrct$cesd))
  expect_equal(ignore_attr = TRUE,  sd(~cesd, data=mosaicData::HELPrct), sd(mosaicData::HELPrct$cesd))
  expect_equal(ignore_attr = TRUE,  max(~cesd, data=mosaicData::HELPrct), max(mosaicData::HELPrct$cesd))
  expect_equal(ignore_attr = TRUE,  min(~cesd, data=mosaicData::HELPrct), min(mosaicData::HELPrct$cesd))
  expect_equal(ignore_attr = TRUE,  min(~cesd | sex, data=mosaicData::HELPrct), min(cesd ~ sex, data=mosaicData::HELPrct))
  expect_equal(ignore_attr = TRUE,  max(~cesd | sex, data=mosaicData::HELPrct), max(cesd ~ sex, data=mosaicData::HELPrct))
  expect_equal(ignore_attr = TRUE,  mean(~cesd | sex, data=mosaicData::HELPrct), mean(cesd ~ sex, data=mosaicData::HELPrct))
  expect_equal(ignore_attr = TRUE,  sd(~cesd | sex, data=mosaicData::HELPrct), sd(cesd ~ sex, data=mosaicData::HELPrct))
  expect_equal(ignore_attr = TRUE,  var(~cesd | sex, data=mosaicData::HELPrct), var(cesd ~ sex, data=mosaicData::HELPrct))
})

# no longer supporting this
# test_that("data frame interface works", {
#   expect_equal(ignore_attr = TRUE,  mean(cesd, data=mosaicData::HELPrct), mean(mosaicData::HELPrct$cesd))
#   expect_equal(ignore_attr = TRUE,  var(cesd, data=mosaicData::HELPrct), var(mosaicData::HELPrct$cesd))
#   expect_equal(ignore_attr = TRUE,  sd(cesd, data=mosaicData::HELPrct), sd(mosaicData::HELPrct$cesd))
#   expect_equal(ignore_attr = TRUE,  max(cesd, data=mosaicData::HELPrct), max(mosaicData::HELPrct$cesd))
#   expect_equal(ignore_attr = TRUE,  min(cesd, data=mosaicData::HELPrct), min(mosaicData::HELPrct$cesd))
# })


test_that("formulas work without data", {
  age <<- mosaicData::HELPrct$age
  sex <<- mosaicData::HELPrct$sex
  expect_equal(ignore_attr = TRUE,  min( age ), min( ~age ))
  expect_equal(ignore_attr = TRUE,  min( ~ age ), min( ~age, data=mosaicData::HELPrct ))
  expect_equal(ignore_attr = TRUE,  max( age ), max( ~age ))
  expect_equal(ignore_attr = TRUE,  max( ~ age ), max( ~age, data=mosaicData::HELPrct ))
  expect_equal(ignore_attr = TRUE,  sd( age ), sd( ~age ))
  expect_equal(ignore_attr = TRUE,  sd( ~ age ), sd( ~age, data=mosaicData::HELPrct ))
  expect_equal(ignore_attr = TRUE,  var( age ), var( ~age ))
  expect_equal(ignore_attr = TRUE,  var( ~ age ), var( ~age, data=mosaicData::HELPrct ))
  expect_equal(ignore_attr = TRUE,  median( ~ age ), median( ~age, data=mosaicData::HELPrct ))
  expect_equal(ignore_attr = TRUE,  median( age ~ sex ), median( age ~ sex, data=mosaicData::HELPrct ))
  expect_equal(ignore_attr = TRUE,  mean( age ), mean( ~age ))
  expect_equal(ignore_attr = TRUE,  mean( ~ age ), mean( ~age, data=mosaicData::HELPrct ))
})

test_that(
  "var grabs two vectors from data frame", {
    expect_equal(ignore_attr = TRUE, var( ~ age, ~ cesd, data = mosaicData::HELPrct), 
                      var(mosaicData::HELPrct$age, mosaicData::HELPrct$cesd))
  })

test_that("na.rm works", {
  x <<- 1:6; y <<- c(1,2,5,6)
  x[3] <- NA
  x[4] <- NA
  expect_equal(ignore_attr = TRUE,    mean(x, na.rm=TRUE), mean(y))
  expect_equal(ignore_attr = TRUE,      sd(x, na.rm=TRUE), sd(y))
  expect_equal(ignore_attr = TRUE,     var(x, na.rm=TRUE), var(y))
  expect_equal(ignore_attr = TRUE,  median(x, na.rm=TRUE), median(y))
  expect_equal(ignore_attr = TRUE,     max(x, na.rm=TRUE), max(y))
  expect_equal(ignore_attr = TRUE,     var(x, x, na.rm=TRUE), var(x, x, na.rm=TRUE))
  expect_equal(ignore_attr = TRUE,     var(x, 1:6, na.rm=TRUE), stats::var(x, 1:6, na.rm=TRUE))
})

test_that("sum( a + x ) works, etc.", {
  x <- 1:6; a <- 11:16
  expect_equal(ignore_attr = TRUE,  sum( a + x) , base::sum( a + x) )
  expect_equal(ignore_attr = TRUE,  mean( a + x) , base::mean( a + x) )
  expect_equal(ignore_attr = TRUE,  median( a + x) , stats::median( a + x) )
})

test_that("cov() and cor() work.", {
  expect_equal(ignore_attr = TRUE, cor( age ~ i1, data = HELPrct), cor(HELPrct$age, HELPrct$i1))
  expect_equal(ignore_attr = TRUE, cor( ~ age, ~ i1, data = HELPrct), cor(HELPrct$age, HELPrct$i1))
  expect_equal(ignore_attr = TRUE, cor( ~ age | i1, data = HELPrct), cor(HELPrct$age, HELPrct$i1))
  expect_equal(ignore_attr = TRUE, cov( age ~ i1, data = HELPrct), cov(HELPrct$age, HELPrct$i1))
  expect_equal(ignore_attr = TRUE, cov( ~ age, ~ i1, data = HELPrct), cov(HELPrct$age, HELPrct$i1))
  expect_equal(ignore_attr = TRUE, cov( ~ age | i1, data = HELPrct), cov(HELPrct$age, HELPrct$i1))
})

test_that("var() works.", {
  expect_equal(ignore_attr = TRUE, var( age ~ sex, data = HELPrct), 
                    aggregate(HELPrct$age, by = list(HELPrct$sex), FUN = var)$x)
  expect_equal(ignore_attr = TRUE, var( ~ age, ~ i1, data = HELPrct), var(HELPrct$age, HELPrct$i1))
  expect_equal(ignore_attr = TRUE, var( ~ age | sex, data = HELPrct), var(age ~ sex, data = HELPrct))
})

# no longer supporting this
# test_that("bare names work", {
#   expect_equal(ignore_attr = TRUE,  mean(   age, data = HELPrct), 
#                      mean( ~ age, data = HELPrct) )
#   expect_equal(ignore_attr = TRUE,  cor( age,  cesd, data = HELPrct), 
#                      cor( age ~ cesd, data = HELPrct) )
#   expect_equal(ignore_attr = TRUE,  var(  age, data = HELPrct), 
#                      var( ~ age, data = HELPrct) )
#   expect_equal(ignore_attr = TRUE,  favstats(   age, data = HELPrct), 
#                      favstats( ~ age, data = HELPrct) )
# })

test_that("aggregatingFunctions work", {
  require(mosaicData)
  foo <- aggregatingFunction1(base::mean)
  expect_equal(ignore_attr = TRUE, 24.7230769230769, foo( ~ length, data = KidsFeet))
  
 
  foo <- aggregatingFunction2(stats::cor)
  expect_equal(ignore_attr = TRUE, 0.641096051081777, foo(length ~ width, data = KidsFeet))
})


test_that("aggregating functions are compatible with dplyr", {
  require(dplyr)
  expect_equal(ignore_attr = TRUE, 
    data.frame(x = 1:10, y = 1:10) |> 
      summarise(mean = mean(~x), cor = cor(y ~ x)),
    data.frame(x = 1:10, y = 1:10) |> 
      summarise(mean = base::mean(x), cor = stats::cor(y, x))
  )
})

# context("resample")

testthat::test_that("resample works", {
  require(mosaicData)
  
  testcase1 <- c(1L, 0L, 1L, 0L, 1L, 0L, 1L, 1L, 1L, 1L, 1L, 0L, 1L, 1L, 0L, 
                 0L, 1L, 1L, 0L, 0L, 0L, 0L, 1L, 0L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 
                 1L, 0L, 0L, 1L, 1L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 0L, 1L, 1L, 1L, 
                 1L, 0L, 1L, 0L, 1L, 0L, 0L, 1L, 1L, 0L, 1L, 1L, 1L, 1L, 1L, 1L, 
                 0L, 0L, 0L, 0L, 1L, 0L, 1L, 0L, 0L, 1L, 0L, 1L, 0L, 0L, 0L, 1L, 
                 1L, 1L, 0L, 1L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 1L, 
                 1L, 0L, 0L, 1L, 1L)
  
  testcase2 <- structure(list(name = structure(c(7L, 10L, 15L, 7L, 11L, 15L, 20L, 18L, 8L, 20L), 
                                               .Label = c("Abby", "Alisha", "Andy", "Caitlin", "Cal", "Cam", "Caroline", "Damon", "Danielle", "David", "Dwayne", 
                                                                                "Dylan", "Edward", "Eleanor", "Erica", "Glen", "Hannah", "Hayley", 
                                                                                "Heather", "Josh", "Julie", "Kate", "Lang", "Lars", "Laura", 
                                                                                "Lee", "Leigh", "Maggie", "Mark", "Mike", "Peggy", "Peter", "Ray", 
                                                                                "Scotty", "Teshanna", "Zach"), class = "factor"), 
                              birthmonth = c(12L,5L, 9L, 12L, 8L, 9L, 7L, 1L, 9L, 7L), 
                              birthyear = c(87L, 88L, 88L, 87L, 88L, 88L, 88L, 88L, 88L, 88L), 
                              length = c(24, 24.4,   24.5, 24, 23.9, 24.5, 24.4, 21.6, 22.9, 24.4), 
                              width = c(8.7,  8.4, 9, 8.7, 9.3, 9, 8.6, 7.9, 8.8, 8.6), sex = structure(c(2L, 1L, 2L, 2L, 1L, 2L, 1L, 2L, 1L, 1L), 
                                                                                                        .Label = c("B", "G"), 
                                                                                                        class = "factor"), 
                              biggerfoot = structure(c(2L, 1L, 1L, 2L, 2L, 1L, 1L, 2L, 2L, 1L), 
                                                     .Label = c("L", "R"), 
                                                     class = "factor"), 
                              domhand = structure(c(1L,  2L, 2L, 1L, 1L, 2L, 2L, 2L, 1L, 2L), 
                                                  .Label = c("L", "R"), 
                                                  class = "factor"), 
                              orig.id = c("5", "4", "9", "5", "2", "9", "3", "8", "6", "3")), 
                         row.names = c("17", "1", "24", "17.1", "30", "24.1", "22", "38", "10", "22.1"),
                         class = "data.frame")
  
  testcase3 <- structure(c(B = 6L, G = 4L), 
                         .Dim = 2L, 
                         .Dimnames = list(sex = c("B", "G")), 
                         class = "table")
  
  
  testcase4 <- structure(list(name = structure(c(33L, 3L, 33L, 30L, 24L, 24L, 
                                                 4L, 27L, 22L, 22L), 
                                               .Label = c("Abby", "Alisha", "Andy", "Caitlin", 
                                                                                "Cal", "Cam", "Caroline", "Damon", "Danielle", "David", "Dwayne", 
                                                                                "Dylan", "Edward", "Eleanor", "Erica", "Glen", "Hannah", "Hayley", 
                                                                                "Heather", "Josh", "Julie", "Kate", "Lang", "Lars", "Laura", 
                                                                                "Lee", "Leigh", "Maggie", "Mark", "Mike", "Peggy", "Peter", "Ray", 
                                                                                "Scotty", "Teshanna", "Zach"), class = "factor"), 
                              birthmonth = c(3L,   6L, 3L, 11L, 10L, 10L, 6L, 3L, 4L, 4L), 
                              birthyear = c(88L, 88L,     88L, 88L, 87L, 87L, 88L, 88L, 88L, 88L), 
                              length = c(24.8, 24,  24.8, 24.2, 25.4, 25.4, 23, 24.5, 23.7, 23.7), 
                              width = c(8.9,  9.2, 8.9, 8.9, 8.8, 8.8, 8.8, 8.6, 7.9, 7.9), 
                              sex = structure(c(1L,  1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L), 
                                              .Label = c("B", "G"), 
                                              class = "factor"), 
                              biggerfoot = structure(c(1L, 2L, 1L, 1L, 1L, 1L, 1L, 1L, 
                                                       2L, 2L), 
                                                     .Label = c("L", "R"), 
                                                     class = "factor"), 
                              domhand = structure(c(2L,  2L, 2L, 2L, 1L, 1L, 2L, 2L, 2L, 2L), 
                                                  .Label = c("L", "R"), 
                                                  class = "factor"), 
                              orig.id = c("9.4.9", "4.7.4", "9.7.9", "10.9.7", "7.9.7", 
                                          "7.10.10", "6.3.3", "1.6.3", "3.1.6", "3.3.1"), 
                              id1 = c("B:4", "B:7", "B:7", "B:9", "B:9", "B:10", "G:3", "G:6", "G:1",  "G:3"), 
                              id2 = c("B:9", "B:4", "B:9", "B:7", "B:7", "B:10",  "G:3", "G:3", "G:6", "G:1")), 
                         row.names = c("9", "4", "9.1",   "10", "7", "7.1", "6", "1", "3", "3.1"), 
                         class = "data.frame")
  
  
  set.seed(13)
  expect_equal(ignore_attr = TRUE, testcase1, resample(0:1, 100))
  tally(resample(0:1, 100))
  set.seed(17) 
  Small <- sample(KidsFeet, 10)
  expect_equal(ignore_attr = TRUE, testcase2, resample(Small))
  set.seed(21) 
  Small <- sample(KidsFeet, 10)
  expect_equal(ignore_attr = TRUE, testcase3, tally(~ sex, data=resample(Small, groups=sex)))
  # shuffled can be used to reshuffle some variables within groups
  # orig.id shows where the values were in original data frame.
  set.seed(6)
  Small <- mutate(Small,
     id1 = paste(sex,1:10, sep=":"),
     id2 = paste(sex,1:10, sep=":"))
  expect_equal(ignore_attr = TRUE, testcase4, resample(Small, groups=sex, shuffled=c("id1","id2")))
  
})


testthat::test_that("rflip works", {
  
  
  testcase1 <- structure(5L, 
                         n = 10, 
                         prob = 0.5, 
                         sequence = c("H", "H", "H",  "T", "T", 
                                      "H", "T", "H", "T", "T"), 
                         verbose = TRUE, 
                         class = "cointoss")
  testcase2 <- structure(3L, 
                         n = 10, 
                         prob = 0.166666666666667, 
                         sequence = c("T", "T", "T", "H", "T", 
                                      "T", "T", "H", "T", "H"), 
                         verbose = FALSE, 
                         class = "cointoss")
  testcase3 <- structure(list(n = 10, 
                              heads = 1L, 
                              tails = 9, 
                              prob = 0.166666666666667), 
                         class = "data.frame", 
                         row.names = c(NA, -1L))
  
  set.seed(18)
  expect_equal(ignore_attr = TRUE, testcase1, rflip(10))
  set.seed(28)
  expect_equal(ignore_attr = TRUE, testcase2, rflip(10, prob = 1/6, quiet = TRUE))
  set.seed(5)
  expect_equal(ignore_attr = TRUE, testcase3, rflip(10, prob = 1/6, summarize = TRUE))
})

# context('favstats()')

test_that("favstats works for vectors ", {
  expect_equal(ignore_attr = TRUE,  favstats(1:10)$mean,  mean(1:10) )
  expect_equal(ignore_attr = TRUE,  favstats(1:10)$sd,  sd(1:10) )
  expect_equal(ignore_attr = TRUE,  favstats(1:10)$min,  min(1:10) )
  expect_equal(ignore_attr = TRUE,  favstats(1:10)$max,  max(1:10) )
  expect_equal(ignore_attr = TRUE,  favstats(1:10)$Q1,  quantile(1:10)[2] )
  expect_equal(ignore_attr = TRUE,  favstats(1:10)$Q3,  quantile(1:10)[4] )
})

test_that("data interface works", {
  expect_equal(ignore_attr = TRUE,  favstats(mosaicData::HELPrct$age), favstats(age, data=mosaicData::HELPrct) )
})

test_that("formula interface works", {
  expect_equal(ignore_attr = TRUE,  favstats(mosaicData::HELPrct$age), favstats(~age, data=mosaicData::HELPrct) )
})

test_that("formulas work without data", {
  expect_equal(ignore_attr = TRUE,  favstats(1:10), favstats(~1:10) )
})


test_that("missing data handled correctly", {
  myHELP <- mosaicData::HELPrct
  myHELP$age[1] <- NA
  expect_equal(ignore_attr = TRUE,  favstats(myHELP$age)$missing, 1 ) 
  expect_equal(ignore_attr = TRUE,  favstats(myHELP$age)$mean, mean(mosaicData::HELPrct$age[-1]) )
  expect_equal(ignore_attr = TRUE,  favstats(myHELP$age)$sd, sd(mosaicData::HELPrct$age[-1]) )
})

# context("prop.test()")

TestData <- data.frame( a = factor(rep(letters[1:3], length.out=100)),
                        b = rep(letters[1:3], length.out=100), 
                        c = rep(c(TRUE, FALSE, FALSE), length.out=100), 
                        d = rep(c(TRUE, FALSE, FALSE), length.out=100), 
                        stringsAsFactors = FALSE
)

test_that("formulas work", {
  
  X <- stats::prop.test(34, 100)
  A <- prop.test(~ a, data=TestData)
  B <- prop.test(~ b, data=TestData)
  C <- prop.test(~ c, data=TestData)
  
  expect_equal(ignore_attr = TRUE, confint(A), confint(X))
  expect_match(A$data.name, "TestData\\$a")
  
  expect_equal(ignore_attr = TRUE, confint(B), confint(X))
  expect_match(B$data.name, "TestData\\$b")
  
  expect_equal(ignore_attr = TRUE, confint(C), confint(X))
  expect_match(C$data.name, "TestData\\$c")
  
})

test_that("formulas + unnamed second arg throws error", {
   expect_error( prop.test(~ a, TestData), "did you forget")
})

# test_that("formulas work with unnamed second arg", {
#   
#   X <- stats::prop.test(34, 100)
#   A <- prop.test(~ a, TestData)
#   B <- prop.test(~ b, TestData)
#   C <- prop.test(~ c, TestData)
#   
#   expect_equal(ignore_attr = TRUE, confint(A), confint(X))
#   expect_match(A$data.name, "TestData\\$a")
#   
#   expect_equal(ignore_attr = TRUE, confint(B), confint(X))
#   expect_match(B$data.name, "TestData\\$b")
#   
#   expect_equal(ignore_attr = TRUE, confint(C), confint(X))
#   expect_match(C$data.name, "TestData\\$c")
# })

test_that("success = works", {
  
  X <- stats::prop.test(33, 100)
  Y <- stats::prop.test(66, 100)
  A <- prop.test(~ a, data=TestData, success = "b")
  B <- prop.test(~ b, data=TestData, success = "b")
  C <- prop.test(~ c, data=TestData, success = FALSE)
  
  expect_equal(ignore_attr = TRUE, confint(A), confint(X))
  expect_match(A$data.name, "TestData\\$a")
  expect_match(A$data.name, "success = b")
  
  expect_equal(ignore_attr = TRUE, confint(B), confint(X))
  expect_match(B$data.name, "TestData\\$b")
  expect_match(B$data.name, "success = b")
  
  expect_equal(ignore_attr = TRUE, confint(C), confint(Y))
  expect_match(C$data.name, "TestData\\$c")
  expect_match(C$data.name, "success = FALSE")
})


test_that("bare vars throw error", {
  expect_error(prop.test(a, data = TestData), "first argument should be a formula")
  expect_error(prop.test(b, data = TestData), "first argument should be a formula")
  expect_error(prop.test(d, data = TestData), "first argument should be a formula")
})

# test_that("bare vars work", {
#   X <- stats::prop.test(34, 100)
#   A <- prop.test( a, data=TestData)
#   B <- prop.test( b, data=TestData)
#   C <- prop.test( c, data=TestData)
#   
#   expect_equal(ignore_attr = TRUE, confint(A), confint(X))
#   expect_match(A$data.name, "a")
#   expect_match(A$data.name, "success = a")
#   
#   expect_equal(ignore_attr = TRUE, confint(B), confint(X))
#   expect_match(B$data.name, "b")
#   expect_match(B$data.name, "success = a")
#   
#   expect_equal(ignore_attr = TRUE, confint(C), confint(X))
#   expect_match(C$data.name, "c")
#   expect_match(C$data.name, "success = TRUE")
# })

test_that("numbers work", {
  expect_equal(ignore_attr = TRUE,  
    confint(stats::prop.test(33, 100)),
    confint(prop.test(33, 100))
  )
  
})

test_that("x treated as raw data when n is missing", {
  X <- resample(1:3, 100)
  x <- sum(X == min(X))
  expect_equal(ignore_attr = TRUE,   
    confint(prop.test(X)), 
    confint(prop.test(x, 100)) )
})  


test_that("tests for multiple proportions", {
  smokers  <- c( 83, 90, 129, 70 )
  patients <- c( 86, 93, 136, 82 )
  expect_equal(ignore_attr = TRUE,  
    stats::prop.test(smokers, patients),
    prop.test(smokers, patients)
  )
})
  
test_that("x treated as raw data", {
  X <- resample(1:3, 100)
  x <- sum(X == min(X))
  expect_equal(ignore_attr = TRUE,   
    confint(prop.test(X)), 
    confint(prop.test(x, 100)) )
})

# based on similar function in fastR
wald_ci <- function (x, n = 100, conf.level = 0.95, 
                     alternative = c("two.sided", "less", "greater") ) 
{
  alternative <- match.arg(alternative)
  one_or_two <- if (alternative == "two.sided") 2 else 1
  
  alpha <- 1 - conf.level
  p <- x/n
  zstar <- -qnorm(alpha/one_or_two)
  interval <- p + c(-1, 1) * zstar * sqrt(p * (1 - p)/n)
  if (alternative == "less") interval[1] <- 0
  if (alternative == "greater") interval[2] <- 1
  attr(interval, "conf.level") <- conf.level
  attr(interval, "method") <- "Wald"
  return(interval)
}

# based on similar function in fastR
plus4_ci <- function(x, n, conf.level = 0.95, 
                     alternative = c("two.sided", "less", "greater") ) 
{
  alternative <- match.arg(alternative)
  one_or_two <- if (alternative == "two.sided") 2 else 1
  
  alpha <- (1 - conf.level) 
  p <- (x + 2)/(n + 4)
  zstar <- -qnorm(alpha/one_or_two)
  interval <- p + c(-1, 1) * zstar * sqrt(p * (1 - p)/(n + 4))
  if (alternative == "less" || interval[1] < 0) interval[1] <- 0
  if (alternative == "greater" || interval[2] > 1) interval[2] <- 1
  
  attr(interval, "conf.level") <- conf.level
  attr(interval, "method") <- "plus4"
  return(interval)
}

agresti_ci <- function (x, n, conf.level = 0.95,
                        alternative = c("two.sided", "less", "greater") )
  
{
  alternative <- match.arg(alternative)
  one_or_two <- if (alternative == "two.sided") 2 else 1
  
  alpha <- 1 - conf.level
  p <- (x + 2)/(n + 4)
  zstar <- -qnorm(alpha/one_or_two)
  z2 <- zstar^2
  n_ <- n + z2
  p_ <- (x + z2/2) / n_
  se_ <- sqrt( p_ * (1-p_) / n_ )
  interval <- p_ + c(-1,1) * zstar * se_
  if (alternative == "less" || interval[1] < 0) interval[1] <- 0
  if (alternative == "greater" || interval[2] > 1) interval[2] <- 1
  
  attr(interval, "conf.level") <- conf.level
  attr(interval, "method") <- "Agresti-Coull"
  return(interval)
}

clopper_ci <- function(x, n, conf.level = 0.95, 
                       alternative = c("two.sided", "less", "greater") ) 
{
  interval <- stats::binom.test(x, n, conf.level = conf.level, alternative = alternative)$conf.int
  attr(interval, "method") <- "Score"
  interval
}

score_ci <- function(x, n, conf.level = 0.95, 
                     alternative = c("two.sided", "less", "greater"),
                     correct = TRUE) 
{
  interval <- stats::prop.test(x, n, conf.level = conf.level, alternative = alternative,
                               correct = correct)$conf.int
  attr(interval, "method") <- 
    if (correct) "Score (with continuity correction)" else "Score (without continuity correction)"
  interval
}

#' Update confidence interval
#' 
#' Update the confidence interval portion of an object returned from
#' `binom.test` using one of several alternative methods.
#' 
#' @param object An `"htest"` object produced by [binom.test()]
#' @param method a method for computing a confidence interval for a proportion.
#' @return an `"htest"` object with an updated confidence interval
#' 
#' @export
#' @seealso [mosaic::binom.test()]
#' 
update_ci <- 
  function(
    object, 
    method = c("clopper-pearson", "wald", "agresti-coull", "plus4", "score", "prop.test")
  ) {
    if (! inherits(object, "htest") && !grepl("^Exact binomial test", object$method) )
      stop( "I don't know how to handle that type of object.")
    
    method <- match.arg(method)
    level <- attr(object$conf.int, "conf.level") 
    if ((level < 0.90 || level > .98) && method == "plus4")
      warning(
        paste("Plus 4 confidence intervals are designed for 95% confidence intervals.",
              "Consider using another method when `conf.level` is not close to 0.95.",
              collapse="\n")
      )
    
    
    object$conf.int <- 
      switch(method,
             `clopper-pearson` =  
               clopper_ci(x = object$statistic, 
                          n = object$parameter, 
                          conf.level = level,
                          alternative = object$alternative),
             wald =  
               wald_ci(x = object$statistic, 
                       n = object$parameter, 
                       conf.level = level,
                       alternative = object$alternative),
             plus4 = 
               plus4_ci(x = object$statistic, 
                        n = object$parameter, 
                        conf.level = level,
                        alternative = object$alternative),
             `agresti-coull` = 
               agresti_ci(x = object$statistic, 
                          n = object$parameter, 
                          conf.level = level,
                          alternative = object$alternative),
             score = 
               score_ci(x = object$statistic, 
                        n = object$parameter, 
                        conf.level = level, alternative = object$alternative,
                        correct = FALSE),
             wilson =   # same as score
               score_ci(x = object$statistic, 
                        n = object$parameter, 
                        conf.level = level, alternative = object$alternative,
                        correct = FALSE),
             prop.test = # continuity correction + score
               score_ci(x = object$statistic, 
                        n = object$parameter, 
                        conf.level = level, alternative = object$alternative,
                        correct = TRUE),
      )
    
    object$method <- 
      switch(method,
             wald = "Exact binomial test (Wald CI)",
             plus4 = "Exact binomial test (Plus 4 CI)",
             `agresti-coull` = "Exact binomial test (Agresti-Coull CI)",
             score = "Exact binomial test (Score CI without continuity correction)",
             prop.test = "Exact binomial test (Score CI with continuity correction)"
      )
    
    object
  }

#' Cumulative frequency plots
#' 
#' A high-level function for producing a cumulative frequency plot using
#' `lattice` graphics.
#' 
#' 
#' @rdname plotCumfreq
#' @param x a formula or numeric vector 
#' @param data a data frame in which `x` is evaluated if `x` is a
#' formula.
#' @param \dots other lattice arguments
#' @seealso [lattice::histogram()], [lattice::densityplot()]
#' @return A plot of the empirical cumulative distribution function for sample values specified in `x`.
#' @keywords graphics
#' @examples
#' plotCumfreq(~eruptions, faithful, xlab = 'duration of eruptions')
#' @export

plotCumfreq <- function(x, data, ...) { UseMethod('plotCumfreq') }

#' @rdname plotCumfreq
#' @param subscripts as in lattice plots
#' @export

plotCumfreq.formula <- function(x, data=NULL, subscripts,
	...) {
	densityplot( x, data=data,
		ylab='cumulative frequency',
		panel=panel.cumfreq,
		prepanel=prepanel.cumfreq,
		...)
}

#' @rdname plotCumfreq
#' @export

plotCumfreq.default <- function(x, ...) {
	plotCumfreq.formula( ~ x, ...)
}

#' @rdname plotCumfreq
#' @aliases prepanel.cumfreq
#' @export

prepanel.cumfreq <- function(x, ...) {
	list( xlim=range(x), ylim=c(0,1), dx=1, dy=1 )
}

#' @rdname plotCumfreq
#' @aliases panel.cumfreq
#' @param type smooth or step-function?
#' @param groups grouping variable 
#' @export

panel.cumfreq <- function(x, type=c('smooth','step'), groups=NULL, ...) {
    if (!is.null(groups)) {
        panel.superpose(x, 
            ref = FALSE, groups = groups, 
            panel.groups = panel.cumfreq, 
            type = type, ...)
    }
    else {
		type=match.arg(type)
		if (type == 'step') {
			n <- length(x)
			xs <- rep(sort(x),each=2)
			p <- rep(ppoints(n-1),each=2)
			xs <- c(-Inf, xs, Inf)
			p <- c(0,0,p,1,1)
		} else {
			n <- length(x)
			xs <- sort(x)
			p <- ppoints(n)
			p <- aggregate(p, by=list(xs), mean)$x
			xs <- unique(xs)
		}
		panel.lines(x=xs, y=p, ...)
    }
}

utils::globalVariables( 
  c('button','picker','slider','checkbox','SD','Q','MEAN','slider','manipulate',
    '.plot_one_norm', '.plot_multi_norm', 'y1', 'y2', 'density', 'm'
  )
)

#' Augmented versions of pnorm and qnorm
#' 
#' These functions behave similarly to the functions with the initial `x`
#' removed from their names but add more verbose output and graphics.
#'
#' @param p probability
#' @param q quantile
#' @param mean,sd parameters of normal distribution.
#' @param plot logical.  If TRUE, show an illustrative plot.
#' @param verbose logical.  If TRUE, display verbose output.
#' @param invisible logical.  If TRUE, return value invisibly.
#' @param digits number of digits to display in output.
#' @param lower.tail logical.  If FALSE, use upper tail probabilities.
#' @param log.p logical.  If TRUE, uses the log of probabilities.
#' @param xlim,ylim limits for plotting.
# #' @param vlwd,vcol line width and color for vertical lines.
# #' @param rot angle of rotation for text labels.
#' @param manipulate logical.  If TRUE and in RStudio,
#'  	then sliders are added for interactivity.
#' @param return If `"plot"`, return a plot.  If `"values"`, return a vector of numerical values.
#' @param pattern One of `"stripes"` or `"rings"`.  In the latter case, pairs of regions 
#' (from inside to outside) are grouped together for coloring and probability calculation.
#' @param \dots additional arguments.
#' 
#' 
#' @seealso [lattice::histogram()], 
#' [chisq.test()], 
#' [pnorm()], 
#' [qnorm()], 
#' [lattice::qqmath()], and
#' [plot()]. 
#' 
#' 
#' @examples
#' xpnorm(650, 500, 100)
#' xqnorm(.75, 500, 100)
#' xpnorm(-3:3, return = "plot", system = "gg") |> 
#'   gf_labs(title = "My Plot", x = "") |> 
#'   gf_theme(theme_bw())
#' 
#' \dontrun{
#' if (rstudio_is_available() & require(manipulate)) {
#'   manipulate(xpnorm(score, 500, 100, verbose = verbose),
#'     score = slider(200, 800),
#' 	   verbose = checkbox(TRUE, label = "Verbose Output")
#'   )
#' }
#' }
#' @export

xpnorm <-
function (q, mean = 0, sd = 1, plot = TRUE, verbose = TRUE, invisible = FALSE, digits = 4, 
    lower.tail = TRUE, log.p = FALSE, xlim = mean + c(-4,4) * sd, ylim = c(0, 1.4 * dnorm(mean,mean,sd)), 
    # vlwd = 2, vcol = trellis.par.get('add.line')$col, rot = 45, 
    manipulate = FALSE, ..., return = c("value", "plot")) 
{
  return <- match.arg(return)
  
	if (manipulate && rstudio_is_available() && requireNamespace("manipulate")) {
		return(manipulate::manipulate( 
			xpnorm(q = Q, mean = MEAN, sd = SD, plot = TRUE, verbose = FALSE, invisible = invisible,
						   digits = digits, lower.tail = lower.tail, log.p = log.p, 
						   xlim = xlim, ylim = ylim, 
						   # vlwd = vlwd, vcol = vcol, rot = rot, 
						   manipulate = FALSE,
						   ...),
				   Q = manipulate::slider(mean-4*sd, mean+4*sd, initial = q[1], step = 8*sd/200, label = 'q'),
				   MEAN = manipulate::slider(mean-4*sd, mean+4*sd, initial = mean, step = 8*sd/200, label = 'mean'),
				   SD = manipulate::slider(0, 4*sd, initial = sd, step = 4*sd/100, label = 'st dev')
				   )
		)
	}
    p <- pnorm(q, mean = mean, sd = sd) 
    z <- (q - mean)/sd
    if (verbose) {
		message("\n")
		message(paste("If X ~ N(",format(mean, digits = digits),", ", 
		          format(sd, digits = digits),"), then \n",sep = ""))
        message(paste("\tP(X <= ", format(q, digits = digits), ") = P(Z <= ", 
                  format(z, digits = digits), 
            ") = ", format(p, digits = digits), "", sep = ""))
        message(paste("\tP(X >  ", format(q, digits = digits), ") = P(Z >  ", 
                  format(z, digits = digits), 
                  ") = ", format(1 - p, digits = digits), "", sep = ""))
        message("\n")
    }
    if (length(q) == 1) {
		  res_plot <- .plot_one_norm(p = p, q = q, mean = mean, sd = sd, xlim = xlim, ylim = ylim, 
		                             digits = digits, ...)
    }
    if (length(q) > 1) {
      res_plot <- 
        plot_multi_dist("norm", q = q, mean = mean, sd = sd, xlim = xlim, ylim = ylim, 
                        digits = digits, ...)
    }
    if (return == "plot") {
      if (invisible) {
        return(invisible(res_plot))
      } else {
        return(res_plot)
      }
    } else {
      if(plot) print(res_plot)
      if (invisible) { 
        return(invisible(pnorm(q, mean = mean, sd = sd, lower.tail = lower.tail, log.p = log.p)))
      }
      return(pnorm(q, mean = mean, sd = sd, lower.tail = lower.tail, log.p = log.p))
    }
}

#' @rdname xpnorm
#' @export

xqnorm <-
  function (p, mean = 0, sd = 1, plot = TRUE, verbose = TRUE, digits = getOption("digits"), 
            lower.tail = TRUE, log.p = FALSE, xlim, ylim, invisible = FALSE, 
            # vlwd = 2, vcol = trellis.par.get('add.line')$col, rot = 45, 
            ..., return = c("value", "plot"), pattern = c("stripes", "rings")) 
{
    return <- match.arg(return)
    pattern <- match.arg(pattern)
    q = qnorm(p, mean = mean, sd = sd, lower.tail = lower.tail, 
        log.p = log.p)
    z = (q - mean)/sd
    p <- pnorm(z)
    if (verbose) {
      message("\n")
      message(paste("If X ~ N(",format(mean, digits = digits),", ", 
                format(sd, digits = digits),"), then \n",sep = ""))
      message(paste("\tP(X", " <= ", format(q, digits = digits), ") = ", 
                    format(p, digits = digits), "", sep = ""))
      message(paste("\tP(X", " >  ", format(q, digits = digits), ") = ", 
                    format(1 - p, digits = digits), "", sep = ""))
      message("\n")
    }
    if (! lower.tail) { # make sure we have lower tail probs from here on
      p <- 1 - p
    }
    
    if (length(p) == 1) {
		  res_plot <- .plot_one_norm(p = p, q = q, mean, sd, xlim = xlim, ylim = ylim, digits = digits, 
		      # vlwd = vlwd, vcol = vcol, rot = rot, 
		      ...)
    }
    if (length(p) > 1) {
		  res_plot <- 
		    plot_multi_dist("norm", p = sort(p), # q = sort(q), 
		                    mean = mean, sd = sd, 
		                    xlim = xlim, ylim = ylim, digits = digits, pattern = pattern,
		      ...)
    }
    
    if (return == "plot") {
      if (invisible) {
        return(invisible(res_plot))
      } else {
        return(res_plot)
      }
    } else {
      if (plot) print(res_plot)
      if (invisible) { 
        return(invisible(q))
      }
      return(q)
    }
}

#' @rdname xpnorm
#' @export
#' 
xcnorm <-
  function (
    p, mean = 0, sd = 1, plot = TRUE, verbose = TRUE, digits = getOption("digits"), 
    lower.tail = TRUE, log.p = FALSE, xlim, ylim, invisible = FALSE, 
    ..., return = c("value", "plot"), pattern = "rings") {
    pattern <- match.arg(pattern)
    if (length(mean) > 1 || length(sd) > 1) stop("Only 1 mean and 1 sd allowed.")
    p <- c((1-p)/2, p + (1-p)/2)
    xqnorm(
      p, mean = mean, sd = sd, verbose = verbose, digits = digits, 
      lower.tail = lower.tail, log.p = log.p, xlim = xlim, ylim = ylim, 
      invisible = invisible, ..., return = return, pattern = pattern)
  }
    

#' Central Probability in a Normal or T Distribution
#' 
#' These versions of the quantile functions take a vector of 
#' *central* probabilities as its first argument.
#' 
#' @seealso [stats::qnorm()], [cdist()]
#' 
#' @inheritParams stats::qnorm
#' 
#' @param side One of "upper", "lower", or "both" indicating
#' whether a vector of upper or lower quantiles or a matrix of 
#' both should be returned.
#' @param log.p logical.  If TRUE, uses the log of probabilities.
#' 
#' @export
#' 
#' @examples
#' qnorm(.975)
#' cnorm(.95)
#' xcnorm(.95)
#' xcnorm(.95, verbose = FALSE, return = "plot") |>
#'   gf_refine(
#'   scale_fill_manual( values = c("navy", "limegreen")),
#'   scale_color_manual(values = c("black", "black")))
#' cnorm(.95, mean = 100, sd = 10)
#' xcnorm(.95, mean = 100, sd = 10)
#' 
cnorm <- 
  function(p, mean = 0, sd = 1, log.p = FALSE,
           side = c("both", "upper", "lower")
  ) {
    side <- match.arg(side)
    upper <- qnorm(p + (1-p)/2, mean = mean, sd = sd, log.p = log.p) 
    lower <- qnorm((1-p)/2,     mean = mean, sd = sd, log.p = log.p) 
    switch(
      side,
      upper = upper,
      lower = lower,
      both  = cbind(lower, upper)
    )
}

#' @rdname cnorm
#' @inheritParams stats::qt
#' 
#' @export
ct <- 
  function(p, df, ncp, log.p = FALSE,
           side = c("upper", "lower", "both")
  ) {
    side <- match.arg(side)
    upper <- qt(p + (1-p)/2, df = df, ncp = ncp, log.p = log.p) 
    lower <- qt((1-p)/2,     df = df, ncp = ncp, log.p = log.p) 
    switch(
      side,
      upper = upper,
      lower = lower,
      both  = cbind(lower, upper)
    )
}

#' midpoints along a sequence
#' 
#' Compute a vector of midpoints between values in a numeric vector

#' @param x a numeric vector
#' @return a vector of length 1 less than `x`
#' @export
#' @examples
#' mid(1:5)
#' mid((1:5)^2)

mid <- function(x) { 
	if (!is.numeric(x) || length(x) < 2) {
		stop( "`x' must be a numeric vector of length at least 2")
	}
	x[-1] - .5 * diff(x)
}


.plot_multi_norm <- function(p, q, mean, sd, xlim, ylim, digits = 4, dlwd = 2, 
    # vlwd = 2, vcol = trellis.par.get('add.line')$col, rot = 0, 
    ...) 
{
	dots <- list(...)

	if (! 'lty' %in% names(dots)) { dots$lty <- 1 }

	z <- (q - mean) / sd
			z <- (q - mean) / sd 
	zmax = max(4, abs(z) * 1.6)
	if (missing(xlim)) {
		xlim = mean + c(-1, 1) * abs(zmax) * sd
	}
	ymax = dnorm(mean, mean = mean, sd = sd)
	if (missing(ylim)) {
		ylim = c(0, 1.4 * ymax)
	}
	xdata = seq(xlim[1], xlim[2], length.out = 800)
	ydata = dnorm(xdata, mean = mean, sd = sd)
	groups = apply(sapply(xdata, function(x) {x < q}), 2, sum)

	p <- c(0, p, 1)
	q <- c(xlim[1], q, xlim[2])

	D <- 
	  data.frame(x = xdata, density = ydata) |>
	  mutate(group = apply(sapply(x, function(x) {x >= q}), 2, sum)) |>
	  head(-1) |> tail(-1)
	
	Q <- 
	  data.frame(q = mid(q), m = 1.1 * max(D$density)) |>
	  mutate(group = apply(sapply(q, function(x) {x >= q}), 2, sum))
	
	# plot <- 
	#   ggplot() +
	#   geom_area(aes(y = density, x = x, fill = factor(group)), data = D, show.legend = FALSE) +
	#   geom_text(aes(y = m,  x = q, color = factor(group)), data = Q, 
	#             label = paste("", round(diff(p), 3), sep = ""),
	#             angle = 45, show.legend = FALSE) +
	#   geom_point(aes(y = 1.1 * m, x = q), color = "transparent", data = Q)
	  
	plot <- 
	  gf_area(density ~ x, fill = ~as.character(group %% 2), data = D, 
	          group = ~group,
	          show.legend = FALSE, ...) |>
	  gf_text(m ~ q, color = ~ as.character(group %% 2), data = Q, 
	            label = paste("", round(diff(p), 3), sep = ""),
	            angle = 45, show.legend = FALSE) |>
	  gf_point((1.1 * m) ~ q, color = "transparent", data = Q, 
	           inherit = FALSE, show.legend = FALSE) |>
	  gf_refine(
	    scale_fill_viridis_d(end = 0.9),
	    scale_color_viridis_d(end = 0.9)
	  )
	
	return(plot)
}


.plot_one_norm <- function(p, q, mean, sd, xlim, ylim, digits = 4, 
    ...) 
{
	z <- (q - mean) / sd
	zmax = max(4, abs(z) * 1.6)
	if (missing(xlim)) {
		xlim = mean + c(-1, 1) * zmax * sd
	}
	
	Ddensity <-
	   tibble::tibble(
	     x = seq(xlim[1], xlim[2], length.out = 500),
	     density = dnorm(x, mean, sd),
	     tail = ifelse(x <= q, 
	                   paste0("P(X <= ", format(q, digits = 2), ") = ", round(pnorm(z), 3)),
	                   paste0("P(X >  ", format(q, digits = 2), ") = ", round(1 - pnorm(z), 3))
	     )
	   )
	
	Dtext <- 
	  tibble::tibble(
	    x = q,
	    z = (q - mean)/sd,
	    y1 = stats::dnorm(mean, mean, sd),
	    y2 = 1.1 * stats::dnorm(mean, mean, sd),
	    label = paste("z = ", round(z, 2))
	  )
	  

	# res_plot <-
	#   ggplot(data = Ddensity, aes(x = x, y = density, group = tail, fill = tail)) +
	#     geom_area() + 
	#     geom_text(aes(x = x, y = y2, label = label), data = Dtext, 
	#               vjust = 1, hjust = 0.5, inherit.aes = FALSE) +
	#     geom_segment(aes(x = x, xend = x, y = 0, yend = y1), data = Dtext, inherit.aes = FALSE) +
	#     guides(fill = guide_legend(title = "")) +
	#     theme(legend.position = "top")

	res_plot <-
	  gf_area(density ~ x, data = Ddensity, group = ~ tail, fill = ~ tail, 
	          show.legend = FALSE, ...) |>
	  gf_text(y2 ~ x, label = ~label, data = Dtext, 
	          vjust = 1, hjust = 0.5, inherit = FALSE,
	          show.legend = FALSE) |>
	  gf_segment(0 + y1 ~ x + x, data = Dtext, inherit = FALSE)  |>
	  gf_refine(
	    scale_fill_viridis_d(end = 0.9),
	    scale_color_viridis_d(end = 0.9)
	  )
	  
	    # gf_refine(guides(fill = guide_legend(title = ""))) |>
	
	return(res_plot)
	
}

# context("xpnorm")
require(manipulate)
testthat::test_that("xpnorm works", {
  wrapped_expect_doppelganger("xpnorm1", xpnorm(650, 500, 100)) 
  wrapped_expect_doppelganger("xpnorm2", xqnorm(.75, 500, 100))
  #wrapped_expect_doppelganger("xpnorm3", xpnorm(-3:3, return = "plot", system = "gg") |>
  #                              gf_labs(title = "My Plot", x = "") |>
  #                              gf_theme(theme_bw()))



  #   manipulate(xpnorm(score, 500, 100, verbose = verbose),
  #     score = slider(200, 800),
  # 	   verbose = checkbox(TRUE, label = "Verbose Output")
  #   )
})

#' Difference in means and proportions
#' 
#' Wrappers around `diff(mean(...))` and `diff(prop(...))` that 
#' facilitate better naming of the result
#' 
#' @param x,data,... as in [mosaic::mean()] or [mosaic::prop()]
#' @param only.2 a logical indicating whether differences should only be computed
#'   between two groups.
#' @examples
#' if (require(mosaicData)) {
#' diffprop( homeless ~ sex , data=HELPrct)
#' do(3) * diffprop( homeless ~ shuffle(sex) , data=HELPrct)
#' diffmean( age ~ substance, data=HELPrct, only.2=FALSE)
#' do(3) * diffmean(age ~ shuffle(substance), data=HELPrct, only.2=FALSE)
#' diffmean( age ~ sex, data=HELPrct)
#' do(3) * diffmean(age ~ shuffle(sex), data=HELPrct)
#' }
#' @export
diffmean <- function( x, ..., data=parent.frame(), only.2=TRUE ) {
  m <- mean_(x, ..., data=data)
  nms <- names(m)
  res <- diff(m)
  names(res) <- 
    if (length(nms) < 3) "diffmean" else paste(tail(nms,-1), head(nms, -1), sep="-")
  if (length(nms) > 2 && only.2) {
    stop("To compare more than two means, set only.2=FALSE")
  }
  res
}

#' @rdname diffmean
#' @export
diffprop <- function( x, ..., data = parent.frame(), only.2 = TRUE ) {
  p <- prop(x, ..., data = data)
  nms <- names(p)
  res <- diff(p)
  names(res) <- if (length(nms) < 3) "diffprop" else paste(tail(nms,-1), head(nms, -1), sep="-")
  if (length(nms) > 2 && only.2) {
    stop("To compare more than two proportions, set only.2=FALSE")
  }
  res
}

# context('Environment Checks (with(), etc.)')

test_that("with() works in place of data = ",{
  expect_equal(ignore_attr = TRUE,  mean(~length, data=mosaicData::KidsFeet), with(mosaicData::KidsFeet, mean(length)) )
  expect_equal(ignore_attr = TRUE,  sd(~length, data=mosaicData::KidsFeet), with(mosaicData::KidsFeet, sd(length)) )
  expect_equal(ignore_attr = TRUE,  diffmean(length ~ sex, data=mosaicData::KidsFeet), with(mosaicData::KidsFeet, diffmean(length ~ sex)) )
  expect_equal(ignore_attr = TRUE,  diffprop(domhand ~ sex, data=mosaicData::KidsFeet), with(mosaicData::KidsFeet, diffprop(domhand ~ sex)) )
  expect_equal(ignore_attr = TRUE,  t.test(length ~ sex, data=mosaicData::KidsFeet), with(mosaicData::KidsFeet, t.test(length ~ sex)) )
})

#' Compute z-scores
#' 
#' Compute z-scores
#' @param x a numeric vector
#' @param na.rm a logical indicating whether missing values should be removed
#' @export
#' @examples
#' data(penguins, package = "palmerpenguins")
#' penguins |> 
#'   group_by(species) |> 
#'   mutate(zbill_length_mm = zscore(bill_length_mm, na.rm = TRUE)) |> 
#'   head()
zscore <- function( x, na.rm=getOption("na.rm", FALSE) ) {
  ( x - mean(x, na.rm=na.rm)) / sd(x, na.rm=na.rm)
}

# to satisfy CRAN checks about conditional use of suggested packages
# code suggested by Lionel Henry

wrapped_expect_doppelganger <- function(title, fig, path = NULL, ...) {
  testthat::skip_if_not_installed("vdiffr")
  
  # path arg has been deprecated, so don't pass it along
  vdiffr::expect_doppelganger(title, fig, ...)
}

#' Sleep and Memory
#' 
#' Sleep and Memory
#' 
#' @docType data
#' @name Sleep
#' @usage data(Sleep)
#' @format  A data.frame with 24 observations on the following 2 variables.
#' \describe{
#'    \item{\code{Group }}{treatment group of the subject}
#'    \item{\code{Words }}{number of words recalled}
#' }
#' @source These data were used in a "resampling bake-off" hosted by Robin Lock.
#'
#' @details
#' In an experiment on memory (Mednicj et al, 2008), students were given lists
#' of 24 words to memorize.  After hearing the words they were assigned at
#' random to different groups. One group of 12 students took a nap for 1.5 hours
#' while a second group of 12 students stayed awake and was given a caffeine
#' pill.  The data set records the number of words each participant was
#' able to recall after the break.  

"Sleep"

##############################################
# coin toss
#

#' Tossing Coins
#'
#' These functions simplify simulating coin tosses for those (students primarily)
#' who are not yet familiar with the binomial distributions or just like this syntax
#' and verbosity better.
#'
#' @rdname rflip
#' @return  for `rflip`, a cointoss object 

#' @param n the number of coins to toss
#' @param prob probability of heads on each toss
#' @param quiet a logical.  If `TRUE`, less verbose output is used.
#' @param verbose  a logical.  If `TRUE`, more verbose output is used.
#' @param summarize if `TRUE`, return a summary (as a data frame).
#' @param summarise alternative spelling for `summarize`.
#' 
#' @examples
#' rflip(10)
#' rflip(10, prob = 1/6, quiet = TRUE)
#' rflip(10, prob = 1/6, summarize = TRUE)
#' do(5) * rflip(10)
#' @export

rflip <- function(n=1, prob=.5, quiet=FALSE, verbose = !quiet, summarize = FALSE, 
                  summarise = summarize) {
	if ( ( prob > 1 && is.integer(prob) ) ) {  
		# swap n and prob
		temp <- prob
		prob <- n
		n <- temp
	}
	if (summarise) {
	  heads <- rbinom(1, n, prob)
	  return(data.frame(n = n, heads = heads, tails = n - heads, prob = prob))
	} else {
	  r <- rbinom(n,1,prob)
	  result <- c('T','H')[ 1 + r ]
	  heads <- sum(r)
	  attr(heads,"n") <- n
	  attr(heads,"prob") <- prob 
	  attr(heads,"sequence") <- result
	  attr(heads,"verbose") <- verbose
	  class(heads) <- 'cointoss'
	  return(heads)
	}
}


#' @rdname rflip
#' @param x an object 
#' @param \dots additional arguments
#' @export

print.cointoss <- function(x, ...) {
	heads <- as.numeric(x)
	other <- attributes(x)
	if (other$verbose) {
		cat(paste('\nFlipping ', 
              other$n, 
              ' coin', ifelse( other$n > 1, "s", ""), 
              ' [ Prob(Heads) = ', other$prob, ' ] ...\n', sep=""))
	}

	if (attributes(x)$verbose) {
			cat('\n')
			#print(other$sequence)
			cat(paste(
			  strwrap( paste(other$sequence, collapse=" ") ), 
			  collapse = "\n"))
			cat('\n')
			cat(paste('\nNumber of Heads: ', heads, ' [Proportion Heads: ', heads/other$n, ']\n\n', sep=""))
	}
}

#' @rdname rflip
#' @return  for `nflip`, a numeric vector
#' @examples
#' as.numeric(rflip(10))
#' nflip(10)
#' @export

nflip <- function(n=1, prob=.5, ...) {
	as.numeric( rflip(n=n, prob=prob, ...) )
}


#' More Random Samples
#'
#' These functions simplify and unify sampling in various ways.
#'
#' @rdname resample
#'
#' @param x	Either a vector of one or more elements from which to choose, or a positive integer. 
#' @param size	a non-negative integer giving the number of items to choose.
#' @param replace	Should sampling be with replacement?
#' @param prob	A vector of probability weights for obtaining the elements of the vector being sampled.
#'
#' @details These functions are wrappers around [sample()] providing different defaults and 
#' natural names.
#' @examples
#' # 100 Bernoulli trials -- no need for replace=TRUE
#' resample(0:1, 100)
#' tally(resample(0:1, 100))
#' if (require(mosaicData)) {
#' Small <- sample(KidsFeet, 10)
#' resample(Small)
#' tally(~ sex, data=resample(Small))
#' tally(~ sex, data=resample(Small))
#' # fixed marginals for sex
#' tally(~ sex, data=Small)
#' tally(~ sex, data=resample(Small, groups=sex)) 
#' # shuffled can be used to reshuffle some variables within groups
#' # orig.id shows where the values were in original data frame.
#' Small <- mutate(Small, 
#'    id1 = paste(sex,1:10, sep=":"),  
#'    id2 = paste(sex,1:10, sep=":"))
#' resample(Small, groups=sex, shuffled=c("id1","id2"))
#' }
#' @export

resample <- function(..., replace=TRUE) {
  sample(..., replace=replace)
}

#' @rdname resample
#' @export
#' @examples
#' deal(Cards, 13)    # A Bridge hand
 
deal <- function(...) {
  sample(...)
}

#' @rdname resample
#' @export
#' @examples
#' shuffle(Cards)

shuffle <- function(x, replace=FALSE, prob=NULL, groups=NULL, orig.ids=FALSE) 
{
	if (!is.null(groups)){
		return( .shuffle_within(x, groups=groups, replace=replace) )
	}
	return( sample(x, replace=replace, prob=prob, groups=groups) )
}



##############################################
# override base::sample with something fancier
#

#' @rdname resample
#' @export
sample <- function (x, size, replace=FALSE, ...) {
	UseMethod('sample') 
}

.shuffle_within = function( x, groups=NULL, replace=FALSE, prob=NULL, orig.ids=FALSE, ... ){
	if (is.null(groups)) {
		stop("Must specify groups to resample within.")
	}
	# force groups to have the right size, recycling as needed.
	if (is.null(dim(x))) {
		groups <- rep(groups, length.out=length(x))
	} else {
		groups <- rep(groups, length.out=nrow(x))
	}
	groups = as.factor(groups)
	flag = c()
	levs = levels(groups);
	for (lev in levs) { # k in 1:length(levs) ) {
		ids = which( groups==lev )
		if (length(ids)==1 ) { flag = c(lev) }
		rids = sample(ids, replace=replace, orig.ids=orig.ids) 
		if( is.null(dim(x))) {
			x[ ids] = x[rids]}
		else {
			if( is.data.frame(x) | is.matrix(x) ) {
				x[ids,] = x[rids,]
			} else {
				x[ids] = x[rids]
			}
		}
	}
	if (length(flag) > 0) {
		message <- paste ("The following groups had only 1 member and can't be shuffled: ", flag)
		warning(message)
	}
	return(x)
}


#' @rdname resample
#' @export

sample.default <- function(x, size, replace=FALSE, prob=NULL, 
                           groups=NULL, orig.ids=FALSE, ...) { 
  missingSize <- missing(size)
  haveGroups <- ! is.null(groups)
  if (length(x) == 1L && is.numeric(x) && x >= 1) {
    n <- x
    x <- 1:n
    if (missingSize)  size <- n
  } else {
    n <- length(x)
    if (missingSize) size <- length(x)
  }
  if (haveGroups && size != n) {
    warning("'size' is ignored when using groups.")
    size <- n
  } 
  ids <- 1:n
  
  if (haveGroups) {
    groups <- rep( groups, length.out=size)  # recycle as needed
    result <- aggregate( ids, by=list(groups), FUN=base::sample, 
                         simplify=FALSE,
                         replace=replace, prob=prob)
    result <- unlist(result$x)
    if (orig.ids) { nms <- ids[result] }
    result <- x[result]
    if (orig.ids) { names(result) <- nms }
    return(result)
	}
	result <- base::sample(x, size, replace=replace, prob=prob) 
	return(result)
}

#' @rdname resample
#' @param groups a vector (or variable in a data frame) specifying
#' groups to sample within. This will be recycled if necessary.
#' @param orig.ids  a logical; should original ids be included in returned data frame?
#' @param \dots additional arguments passed to 
#' [base::sample()]
#' or [mosaic::sample()].
#' @param shuffled a vector of column names.  
#' these variables are reshuffled individually (within groups if `groups` is
#' specified), breaking associations among these columns.
#' examples.
#' @param fixed a vector of column names.  These variables are shuffled en masse,
#' preserving associations among these columns.
#' @param invisibly.return a logical, should return be invisible?
#' @param drop.unused.levels a logical, should unused levels be dropped?
#' @export

sample.data.frame <- function(x, size, replace = FALSE, prob = NULL, groups=NULL, 
                              orig.ids = TRUE, fixed = names(x), shuffled = c(),
                              invisibly.return = NULL, ...) {
  if( missing(size) ) size = nrow(x)
  if( is.null(invisibly.return) ) invisibly.return = size>50 
  shuffled <- intersect(shuffled, names(x))
  fixed <- setdiff(intersect(fixed, names(x)), shuffled)
  n <- nrow(x)
  ids <- 1:n
  groups <- eval( substitute(groups), x )
  newids <- sample(n, size, replace=replace, prob=prob, groups=groups, ...)
  origids <- ids[newids]
  result <- x[newids, , drop=FALSE]
  
  idsString <- as.character(origids)
  
  for (column in shuffled) {
    cids <- sample(newids, groups=groups[newids])
    result[,column] <- x[cids,column]
    idsString <- paste(idsString, ".", cids, sep="")
  }
  
  result <-  result[ , union(fixed,shuffled), drop=FALSE]
  if (orig.ids) result$orig.id <- idsString
  
  
  if (invisibly.return) { return(invisible(result)) } else {return(result)}
}

#' @rdname resample
#' @export

sample.matrix <- function(x, size, replace = FALSE, prob = NULL, groups=NULL, orig.ids=FALSE, ...) {
	if (! is.null(groups) ) {
		return(
			.shuffle_within(x, replace=replace, prob=prob, groups=groups, orig.ids=orig.ids)
		)
	}
	n <- nrow(x)
		ids <- base::sample(n, size, replace=replace, prob=prob)
		data <-  x [ ids, , drop=FALSE] 
		names(data) <- names(x)
		if (orig.ids) {
			attr(data,'orig.row') <- ids
		}
		if (length(ids) < 50) { return(data) } else {return(invisible(data))}
}

#' @rdname resample
#' @export
sample.factor <- function(x, size, replace = FALSE, prob = NULL, groups=NULL, orig.ids=FALSE, 
					drop.unused.levels = FALSE, ...) {
	if (! is.null(groups) ) {
		return(
			.shuffle_within(x, replace=replace, prob=prob, groups=groups, orig.ids=orig.ids)
		)
	}
	n <- length(x)
	ids <- base::sample(n, size, replace=replace, prob=prob)
	if (drop.unused.levels) {
		data <-  factor( x [ ids ] )
	} else {
		data <-  factor( x [ ids ], levels=levels(x) )
	}
	return(data)
}

#' @rdname resample
#' @param parametric A logical indicating whether the resampling should be done parametrically.
#' @param transformation NULL or a function providing a transformation to be applied to the 
#'   synthetic responses.  If NULL, an attempt it made to infer the appropriate transformation
#'   from the original call as recorded in `x`.
#' 
#' @export
#' @examples 
#' model <- lm(width ~length * sex, data = KidsFeet)
#' KidsFeet |> head()
#' resample(model) |> head()
#' Boot <- do(500) * lm(width ~ length * sex, data = resample(KidsFeet))
#' df_stats(~ Intercept + length + sexG + length.sexG, data = Boot, sd)
#' head(Boot)
#' summary(coef(model))
#' 
#' 
sample.lm <- 
  function(
    x, size, replace = FALSE, prob = NULL, groups=NULL, 
    orig.ids=FALSE, drop.unused.levels = FALSE, 
    parametric = FALSE, 
    transformation = NULL,
    ...) {
   
    if (!is.null(prob)) {
      warning("Unused argument: prob")
    }
    if (!is.null(groups)) {
      warning("Unused argument: groups")
    }
    
    if (! replace) {
      stop("Only resampling supported for linear model objects.")
    }
    # replace == TRUE
   
    orig_data <- eval( x$call[["data"]], environment(formula(x)) )
    dfx <- orig_data[, all.vars(formula(x))]
    complete_idx <- which(complete.cases(dfx))
    res <- dfx[complete_idx, ]
   
    if (! missing(size)) {
      if (size != nrow(res)) {
        stop ("Invalid value for `size'.")
      }
      
      warning("`size' is ignored when resampling an `lm' object.")
    }
    size <- nrow(res)
    
    res$resid <- resid(x)
    if (parametric) {
      res$new_resid <- rnorm(size, mean = 0, sd = summary(x)$sigma)
    } else {
      res$new_resid <- 
        (1 - 2 * rbinom(size, 1, 0.5)) * resample(resid(x))
    }
    res$new_response <- fitted(x) + res$new_resid
   
     
    if (is.null(transformation)) {
      transformation <- mosaicCore::infer_transformation(formula(x))
    }
    res[[1]] <- do.call(transformation, list(res$new_response))
    # remove "scratch columns"
    res <- res |> 
      #select_(.dots = setdiff(names(res), c("resid", "new_resid", "new_response")))
      select(any_of(setdiff(names(res), c("resid", "new_resid", "new_response"))))
    res
  }


#' Resample a Linear Model
#' 
#' Fit a new model to data created using `resample(model)`.
#' 
#' 
#' @param model a linear model object produced using [lm()].
#' @param ...  additional arguments passed through to [resample()].
#' @param envir an environment in which to (re)evaluate the linear model.
#' @seealso `resample()`
#' 
#' @examples
#' mod <- lm(length ~ width, data = KidsFeet)
#' do(1) * mod 
#' do(3) * relm(mod) 
#' # use residual resampling to estimate standard error (very crude because so few replications)
#' Boot <- do(100) * relm(mod)
#' sd(~ width, data = Boot)
#' # standard error as produced by summary() for comparison
#' mod |> summary() |> coef() 
#' 
#' @export
relm <- function(model, ..., envir = environment(formula(model))) {
  mcall <- model$call
  mcall[["data"]] <- resample(model, ...)
  eval(mcall, envir)
}

#' 
#' Simulate spinning a spinner
#' 
#' This is essentially `rmultinom` with a different interface.
#' 
#' @param n number of spins of spinner
#' @param probs a vector of probabilities.  If the sum is not 1, the 
#' probabilities will be rescaled.
#' @param labels a character vector of labels for the categories
#' @export
#' @examples
#' rspin(20, prob=c(1,2,3), labels=c("Red", "Blue", "Green"))
#' do(2) * rspin(20, prob=c(1,2,3), labels=c("Red", "Blue", "Green"))
 
rspin <- function(n, probs, labels=1:length(probs)) {
  if (any(probs < 0))
    stop("All probs must be non-negative.")
  
  probs <- probs/sum(probs)
  res <- t(rmultinom(1, n, probs)) |> as.data.frame()
  names(res) <- labels
  res
}

# #' @importFrom glue glue

utils::globalVariables( 
  c('picker', 'button', 'slider', 'checkbox', 'x', 'y', 'color', 'size', 
    'logScales', 'key', 'nbins',  'plotType', '.scatterString',
    'flipCoords', 'group', 'projection', 'facet', 'N'))
NA

system_choices <- function() {
  choices <- c("ggformula", "ggplot2", "lattice")
  available <- sapply(choices, isNamespaceLoaded)
  choices[available]
}

ggdir2pos <- function(x) {
  switch(EXPR = x,
    'N'  = 'c(0.5, 0.9)', 
    'NE' = 'c(0.9, 0.9)', 
    'E'  = 'c(0.9, 0.5)', 
    'SE' = 'c(0.9, 0.1)', 
    'S'  = 'c(0.5, 0.1)', 
    'SW' = 'c(0.1, 0.1)', 
    'W'  = 'c(0.1, 0.5)', 
    'NW' = 'c(0.1, 0.9)',
    "none" = '"none"',
    "top" = '"top"',
    "bottom" = '"bottom"',
    "left" = '"left"',
    "right" = '"right"'
  )
}

#' Extract data from a data frame using a formula interface
#' 
#' Uses the full model syntax.
#' 
#' @param data a data frame
#' @param formula a formula. 
#'   The right-hand side selects variables;  
#'   the left-hand side, if present, is used to set row names. 
#'   A `.` on the right-hand side
#'   indicates to use all variables not in the LHS.
#' @param intercept a logical indicating whether to include the intercept in the
#'   model default: FALSE (no intercept)
#' @examples
#' getVarFormula( ~ wt + mpg, data = mtcars)
#' @export

getVarFormula <- function(formula, data = parent.frame(), intercept = FALSE){
  x <- model.matrix(formula, data = data)
  attr(x, "assign") <- NULL
  attr(x, "contrasts") <- NULL
  kill.ind <- which( colnames(x)=="(Intercept)")
  if ( !intercept & length(kill.ind) > 0 ) x <- x[, -kill.ind]
  if ( length(formula) > 2 ) {
    lhsvar <- all.vars( formula[[2]] )
    rnames <- as.character( data[[lhsvar]] )
    rownames(x) <- rnames
  }
  return(x)
}


# Utilities
# Pull out the names of the quantitative and categorical variables in a data frame

.varsByType = function(data) {
  vnames = names(data)
  type = vnames
  numberNames <- .v2list(vnames[sapply(data, function(x) inherits(x, c("integer", "numeric", "POSIXct")))])
  factorNames <- .v2list(vnames[sapply(data, function(x) inherits(x, c("factor", "character", "logical", "ordered")))])
  return( list( c = factorNames, q = numberNames, all = .v2list(vnames) ) )
}
# Prepend a list with NA for optional items
.NAprepend <- function(L) {
  c(list(`none ` = NA), L)
}

#
# Converting a vector of names into a list.
.v2list <- function(names) {
  res = list()
  res[names] <- names
  return(res)
}

#' Interactive plotting 
#'
#' These functions provide a menu selection system (via \pkg{manipulate}) so that 
#' different aspects of a plot can be selected interactively.  
#' The \pkg{ggplot2} or \pkg{lattice}
#' command for generating the plot currently being displayed can be copied to the 
#' console, whence it can be copied to a document for later direct, non-interactive 
#' use.
#' 
#' Only `mPlot` is required by end users.  The other plotting functions 
#' are dispatched based on the value of `default`.  Furthermore, [mplot()] 
#' will dispatch `mPlot` when provided a data frame.
#' 
#' Currently maps are only supported in \pkg{ggplot2} and not in \pkg{lattice}.
#' 
#' Due to an unresolved issue with RStudio, the first time this function is called, and additional
#' plot is created to correctily initialize the mainipulate frameowrk.
#' 
#' @rdname mPlotting
#' @aliases mPlot
#' @param data a data frame containing the variables that might be used in the plot.
#' Note that for maps, the data frame must contain coordinates of the polygons 
#' comprising the map and a variable for determining which coordinates are part
#' of the same region.  See [sp2df()] for one way to create such
#' a data frame.  Typically [merge()] will be used to combine the map
#' data with some auxiliary data to be displayed as fill color on the map, although
#' this is not necessary if all one wants is a map.
#' @param format a synonym for `default`.
#' @param default default type of plot to create; one of 
#' `"scatter"`,
#' `"jitter"`,
#' `"boxplot"`,
#' `"violin"`,
#' `"sina"`,
#' `"histogram"`,
#' `"density"`,
#' `"density (contours)"`,
#' `"density (filled)"`,
#' `"frequency polygon"`,
#' `"xyplot"`, 
#' or
#' `"map"`.  Unique prefixes suffice.
#' @param system which graphics system to use (initially) for plotting (\pkg{ggplot2} 
#' or \pkg{lattice}).  A check box will allow on the fly change of plotting system.
#' @param show a logical, if `TRUE`, the code will be displayed each time the plot is 
#' changed.
#' @param title a title for the plot
#' @param \dots additional arguments 
#' @param data_text A text string describing the data.  It must be possible to recover the data
#' from this string using [eval()].  Typically users will not need to
#' modify this from the default value.
#' @section Note:
#' 
#' Due to an unresolved issue with RStudio, the first time this function is called, and additional
#' plot is created to correctily initialize the mainipulate frameowrk.
#' @return Nothing.  Just for side effects.  
#' @examples
#' \dontrun{
#' mPlot(HELPrct, format = "scatter")
#' mPlot(HELPrct, format = "density")
#' }
#' @export

mPlot <- function(data, 
                  format,
                  default = format,
                  system = system_choices()[1],
                  show = FALSE, 
                  title = "",
                  data_text = rlang::expr_deparse(substitute(data)),
                  # data_text = substitute(data), 
                  ...)
{

  plotTypes <- 
    c('scatter', 'jitter', 'boxplot', 'violin', 'sina', 'density (contours)', 'density (filled)',
      'histogram', 'density', 'frequency polygon', 'ASH plot', 'xyplot', 'map')
  
  if (missing(default) & missing(format)) {
    choice <- 
      menu(title = "Choose a plot type.",
           choices = c(
             "1-variable (histogram, density plot, etc.)",
             "2-variable (scatter, boxplot, etc.)", 
             "map")
      )
    if (choice == 0L) return(invisible(NULL))
    default <- c("histogram", "scatter", "map") [choice]
  }
  default <- match.arg(default, plotTypes)
  system <- match.arg(system, system_choices())
  if (default == 'xyplot') default <- 'scatter'
  if (default %in% c('scatter','jitter','boxplot','violin', 'sina', 'density (contours)', 'density (filled)')) {
    return( 
      mScatter(data, default = default, system = system, show = show, 
               title = title, data_text = data_text)
    )
  }
  if (default == "map") {
    system <- "ggplot2"
    if (! requireNamespace("ggplot2")) stop("ggplot2 is required.  Please install.")
    return(
      mMap(data, default = default, system = system, show = show, title = title,
           data_text = data_text)
    )
  }
  return( 
    mUniplot(data, default = default, system = system, show = show, title = title, 
             data_text = data_text)
  )
}

#' Interactively design plots
#' 
#' Proves a simple interface to let users interactively design plots in \pkg{ggformula}, \pkg{lattice}, or \pkg{ggplot2}.
#' An option is available to show the code used to create the plot. 
#' This can be copied and pasted elsewhere to (into an RMarkdown document, for example) to recreate the plot.
#' Only works in RStudio. Requires the \pkg{manipulate} package.
#' 
#' Currently maps are only supported in \pkg{ggplot2} and not in \pkg{lattice}.
#' 
#' Due to an unresolved issue with RStudio, the first time this function is called, and additional
#' plot is created to correctily initialize the mainipulate frameowrk.
#' 
#' @inheritParams mPlot
#' @return Nothing.  Used for side effects.
#' @examples 
#' \dontrun{
#'   mtcars2 <- 
#'     mtcars |> 
#'       mutate(
#'         cyl2 = factor(cyl), 
#'         carb2 = factor(carb),
#'         shape = c("V-shaped", "straight")[1 + vs], 
#'         gear2 = factor(gear), 
#'         transmission = c("automatic", "manual")[1 + am])
#'   design_plot(mtcars2)
#' }
#' @export
design_plot <- mPlot

#' @rdname mPlotting
#' @export
mMap <- function(data, default = 'map',
                 system = "ggplot2",
                 show = FALSE, title = title, data_text = rlang::expr_deparse(substitute(data)), ...) {
  
  .require_manipulate_namespace()
  # system <- "ggplot2" # only handling ggplot2 for now.
  system <- match.arg(system, "ggplot2")
  keyDefault <- ifelse ( system == "lattice", "none", "right" )
  variables <- .varsByType(head(data))
  latid <- min(c(grep("lat", names(variables$q)), length(variables$q)))
  longid <- min(c(grep("lon", names(variables$q)), length(variables$q)))
  groupid <- min(c(grep("group", names(variables$all)), length(variables$all)))
  
  snames <- .NAprepend(variables$all)
  cnames <- .NAprepend(variables$c)
  pnames <- list("mercator", 
                 "sinusoidal", 
                 "cylequalarea, lat0=0", 
                 "cylindrical", 
                 "rectangular, lat0=0", 
                 "gall, lat0=0", 
                 "mollweide", 
                 "gilbert", 
                 "azequidistant", 
                 "azequalarea", 
                 "gnomonic", 
                 "perspective, dist=5", 
                 "orthographic", 
                 "stereographic", 
                 "laue", 
                 "fisheye, n=5", 
                 "newyorker, r=5", 
                 "conic, lat0=0", 
                 "simpleconic, lat0=-20, lat1=20", 
                 "lambert, lat=-20, lat1=20",
                 "albers, lat=-20, lat1=20", 
                 "bonne, lat0=0", 
                 "polyconic", 
                 "aitoff", 
                 "lagrange", 
                 "bicentric, lon0=30",
                 "elliptic, lon0=-90", 
                 "globular", 
                 "vandergrinten", 
                 # "eisenlohr", 
                 "guyou", 
                 "square",
                 "tetra", 
                 "hex", 
                 "harrison, dist=5, angle=0", 
                 "trapezoidal, lat0=-50, lat1=50", 
                 "lune, lat=60, angle=30", 
                 "mecca, lat0=0",
                 "homing, lat0=0", 
                 "sp_mercator", 
                 "sp_albers, lat0=-50, lat1=50")
  lnames <- list("none","top","right","left",
                 "N" = "N", "NE" = "NE", 
                 "E" = "E", "SE" = "SE", 
                 "S" = "S", "SW" = "SW", 
                 "W" = "W", "NW" = "NW")
  manipulate::manipulate( { .doMap(data, variables, show = show, system = system, 
                                   x = x, y = y, 
                                   color = color, 
                                   group = group,
                                   projection = projection,
                                   facet = facet, 
                                   key = key,
                                   title = title, 
                                   data_text = data_text) },
                          show = manipulate::button("Show Expression"),
                          # system = manipulate::picker(as.list(system_choices()), initial = system, label = "Graphics System"),
                          x = manipulate::picker(variables$q, initial = variables$q[[longid]], label = "longitude (x)"),
                          y = manipulate::picker(variables$q, initial = variables$q[[latid]], label = "latitude (y)"),
                          group = manipulate::picker(variables$all, initial = variables$all[[groupid]], label = "region"),
                          color = manipulate::picker(snames, initial = "none ", label = "Color"),
                          facet = manipulate::picker(cnames, initial = "none ", label = "Facets"),
                          projection = manipulate::picker(pnames, initial = "mercator", label = "Projection"),
                          key = manipulate::picker(lnames, label = "Key", initial = keyDefault)
  )
}


.doMap<- function(data, variables, show = FALSE, 
                  system = "ggplot2", 
                  x = NA, y = NA, 
                  color = NA, 
                  group = group,
                  projection = projection,
                  facet = NA,
                  key = "right",
                  title = "",
                  data_text = rlang::expr_deparse(substitute(data)))
{
  system <- match.arg(system, "ggplot2")
  if (regexpr(",", projection) > 0) {
    projection <- sub(",", '", ', projection)
  } else {
    projection <- paste(projection, '"', sep = "")
  }
  projection <- paste('"',projection, sep = "")
  vals <- list(dataName = rlang::expr_deparse(substitute(data)), 
               x = x, y = y, 
               color = color, 
               group = group,
               facet = facet, 
               projection = projection,
               key = key,
               title = title)
  
  s <- .mapString(vals, system, variables = variables)
  if (show) cat(paste0("\n", s, "\n"))
  p <- eval(parse(text = s))
  print(p)
  return(invisible(p))
}

# maps 
.mapString <- function(s, system = system_choices()[1], variables)
{
  # s$dataName <- name2string(s$dataName)
  #  res <- paste("ggplot(data = ", s$data, ")", sep = "")
  #    res<-paste(res, "+geom_point(aes(x = ", s$x, ", y = ", s$y, "))", sep = "")
  geom <- "geom_polygon()"
  system <- match.arg(system, system_choices())
  if (system == "ggplot2" || TRUE) {
    res <- paste("ggplot(data = ", s$dataName, 
                 ",  aes(x = ", s$x, ", y = ", s$y, 
                 ", group = ",  s$group, "))", sep = "")
    res <- paste(res, " + ", geom, " ", sep = "")
    if (!is.null(s$color) && !is.na(s$color))
      res<-paste(res, " + aes(fill = ", s$color, ")", sep = "")
    if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
      res<-paste(res, " + facet_wrap(~", s$facet, ", ncol = 4)", sep = "")
    res <- paste( res, '+ coord_map(', s$projection, ')', sep = "" )
    if (!is.null(s$color) && !is.na(s$color)) {
      res <- paste(res, ' + theme(legend.position = ', ggdir2pos(s$key), ')', sep = "")
    }  
    res <- paste( res, '+ labs(title = "', s$title, '")', sep = "")
    res <- paste( res, '+ theme(',
                  'panel.background = element_rect("transparent"),',
                  'plot.background = element_rect("transparent"),',
                  'panel.grid.major = element_line(size = 0),',
                  'panel.grid.minor = element_line(size = 0),',
                  'axis.title = element_blank(),',
                  'axis.line = element_blank(),', 
                  'axis.text = element_blank(),',
                  'axis.ticks = element_blank() )'
    )
  } 
  return(res)
}

#' @rdname mPlotting
#' @export

mScatter <- 
  function(
    data, 
    default = c('scatter','jitter','boxplot','violin','line', 'sina', 
                'density (contours)', 'density (filled)'),
    system = "ggformula", show = FALSE, title = "",
    data_text = rlang::expr_deparse(substitute(data))) {
  
  .require_manipulate_namespace()
  system <- match.arg(system, system_choices())
  default <- match.arg(default)
  keyDefault <- ifelse ( system == "lattice", "none", "right" )
  variables <- .varsByType(head(data))
  # variables$q is the quantitative variables.
  plotnames <- 
    list("scatter", "jitter","boxplot", "violin", "line", "sina", "density (contours)", "density (filled)")
  snames <- .NAprepend(variables$all)
  if (length(variables$all) < 2 || length(variables$q) < 1) {
    stop("data must have at least 2 variables, at least one of which is quantitative")
  }
  cnames <- .NAprepend(variables$c)
  mnames <- list("none", linear = "linear", "smooth", "spline")
  lnames <- list("none","top","right","left",
                 "N" = "N", "NE" = "NE", 
                 "E" = "E", "SE" = "SE", 
                 "S" = "S", "SW" = "SW", 
                 "W" = "W", "NW" = "NW")
  # print(list(system, system_choices()))
  manipulate::manipulate( { 
    .doScatter(data, data_text = data_text, variables, show = show, system = system, x = x, y = y, plotType = plotType, 
               flipCoords = flipCoords, color = color, size = size, facet = facet, 
               logScales = logScales, model = model, key = key, title = title) },
    show = manipulate::button("Show Expression"),
    system = manipulate::picker(as.list(system_choices()), initial = system, label = "Graphics System"),
    plotType = manipulate::picker(plotnames, initial = default, label = "Type of plot      "),
    x = if (length(variables$q) >= 2) 
      manipulate::picker(variables$all, initial = variables$q[[2]], label = "   Any variable (x)   ")
    else 
      manipulate::picker(variables$all, initial = variables$c[[1]], label = "   Any variable (x)   ")
    ,
    y = manipulate::picker(variables$q, initial = variables$q[[1]],   label = "   Quant. variable (y)"),
    color = manipulate::picker(snames, initial = "none ", label = "Color"),
    facet = manipulate::picker(cnames, initial = "none ", label = "Facets"),
    model = manipulate::picker(mnames, initial = "none", label = "Model"),
    key = manipulate::picker(lnames, label = "Key", initial = keyDefault),
    size = manipulate::picker(snames, initial = "none ", label = "Size (gg only)"),
    logScales = manipulate::picker(list("none","x","y","both"), initial = "none", label = "log scales"),
    flipCoords = manipulate::checkbox(label = "Flip coordinates")
  )
  }

.doScatter <- function(data, variables, show = FALSE, 
                       system = system_choices()[1],
                       plotType = 
                         c("scatter", "jitter", "boxplot", "violin", "line", "sina", "density (contours)", "density (filled)"),
                       x = NA, y = NA, color = NA, 
                       size = NA, facet = NA, logScales = "none", flipCoords = FALSE,
                       model = "", key = "right", title = title, data_text  =  rlang::expr_deparse(substitute(data)))
{
  system <- match.arg(system, system_choices())
  plotType <- match.arg(plotType)
  
  if (system == "lattice" && plotType %in% c("sina", "density (contours)", "density (filled)")) {
    message('plot type ', plotType, ' unavailable in lattice.')
    return(gf_text(0 ~ 0, label = paste0("plot type ", plotType, " unavailable in lattice.")) |>
             gf_theme(theme_void()))
  }
  
  vals <- list(dataName = data_text, x = x, y = y, color = color, size = size, 
               plotType = plotType, flipCoords = flipCoords, facet = facet, 
               logScales = logScales , model = model, key = key, title = title)
  
  s <- .scatterString(vals, system, variables = variables)
  if (show) cat(paste0("\n", s, "\n"))
  p <- eval(parse(text = s))
  print(p)
  return(invisible(p))
}

# This is not currently in use -- was used as POC for a brief period

name2string <- function(x) {
  if (is.call(x)) {
    x <- deparse(x)
  }
  if (is.character(x)) {
    return(x)
  }
  message('returning `x` as is (', class(x), ')')
}

.scatterString <- 
  function(s, system = system_choices()[1], variables) {
    check_installed('glue')
    
    # s$dataName <- name2string(s$dataName)
    
    gf_fun <- c(scatter = "gf_point", jitter = "gf_jitter", boxplot = "gf_boxplot", 
                violin = "gf_violin", line = "gf_line", sina = "gf_sina", 
                'density (contours)' = "gf_density_2d", 
                'density (filled)' = 'gf_density_2d_filled')
    geom <- c(scatter = "geom_point()", jitter = "geom_jitter()", boxplot = "geom_boxplot()", 
              violin = "geom_violin()", line = "geom_line()",
                'density (contours)' = "geom_density_2d()", 
                'density (filled)' = 'geom_density_2d_filled()'
              )
    system <- match.arg(system, system_choices())
    s$logx <- s$logScales %in% c("both","x")
    s$logy <- s$logScales %in% c("both","y")
    if (s$plotType %in% c("boxplot", "violin", "sina") &&  (s$x %in% variables$q) ) {
      s$x <- glue::glue(.trim = FALSE, "ntiles({s$x})")
    }
    
    if (system == "ggformula") {
      if (!is.null(s$color) && !is.na(s$color)) {
        color_chunk <- glue::glue(.trim = FALSE, ", color = ~ {s$color}")
      } else {
        color_chunk <- ""
      }
      
      if (!is.null(s$size) && !is.na(s$size)) {
        size_chunk <-  glue::glue(.trim = FALSE, ", size = ~ {s$size}") 
      } else {
        size_chunk <- ""
      }
      
      res <- glue::glue(.trim = FALSE, 
        "{gf_fun[s$plotType]}({s$y} ~ {s$x}, data = {s$dataName}{color_chunk}{size_chunk})")
      
      if (s$model == "spline") res <- glue::glue(.trim = FALSE, "{res} |>\n  gf_spline()")
      if (s$model == "linear") res <- glue::glue(.trim = FALSE, "{res} |>\n  gf_lm()")
      if (s$model == "smooth") res <- glue::glue(.trim = FALSE, "{res} |>\n  gf_smooth()")
      
      if (s$logx) res <- glue::glue(.trim = FALSE, "{res} |>\n   gf_refine(scale_x_log10())")
      if (s$logy) res <- glue::glue(.trim = FALSE, "{res} |>\n   gf_refine(scale_y_log10())")
      if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
        res <- glue::glue(.trim = FALSE, "{res} |>\n  gf_facet_wrap(~ {s$facet}, ncol = 4)")
      
      if ((!is.null(s$color) && !is.na(s$color)) || 
          (!is.null(s$size)  && !is.na(s$size))) {
        res <- glue::glue(.trim = FALSE, '{res} |> \n  gf_theme(legend.position = {ggdir2pos(s$key)})')
      } 
      res <- glue::glue(.trim = FALSE, '{res} |> \n  gf_labs(title = "{s$title}", caption = "")')
      if ( s$flipCoords) {
        res <- glue::glue(.trim = FALSE, "{res} |> \n  gf_refine(coord_flip())")
      }
    } else if (system == "ggplot2") {
      res <- paste("ggplot(data = ", s$dataName, ", aes(x = ", s$x, ", y = ", s$y, "))", sep = "")
      res <- paste(res, " + ", geom[s$plotType], " ", sep = "")
      if (!is.null(s$color) && !is.na(s$color))
        res<-paste(res, " + aes(colour = ", s$color, ")", sep = "")
      if (!is.null(s$size) && !is.na(s$size))
        res<-paste(res, " + aes(size = ", s$size, ")", sep = "")
      if (s$logx) res <- paste(res, " + scale_x_log10()", sep = "")
      if (s$logy) res <- paste(res, " + scale_y_log10()", sep = "")
      if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
        res<-paste(res, " + facet_wrap(~", s$facet, ", ncol = 4)", sep = "")
      if (s$model=="spline") res <- paste(res, " + stat_spline()")
      if (s$model=="linear") res <- paste(res, " + stat_smooth(method = lm)")
      if (s$model=="smooth") res <- paste(res, " + stat_smooth(method = loess)") 
      if ((!is.null(s$color) && !is.na(s$color)) || (!is.null(s$size) && !is.na(s$size))){
        res <- paste(res, ' + theme(legend.position = ', ggdir2pos(s$key), ')', sep = "")
      }  
      res <- paste(res, ' + labs(title = "', s$title, '")', sep = "")
      if ( s$flipCoords) {
        res <- paste(res, ' + coord_flip()', sep = "")
      }
    } else {
      plotname <- 
        c(scatter = 'xyplot', jitter = 'xyplot', boxplot = 'bwplot', violin = 'bwplot', line = "xyplot")
      if (s$flipCoords) {
        res <- paste( plotname[s$plotType], "( ", s$x , " ~ ", s$y, sep = "")
      } else {
        res <- paste( plotname[s$plotType], "( ", s$y , " ~ ", s$x, sep = "")
      }
      if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
        res <- paste(res, " | ", s$facet)
      res <- paste(res, ", data = ", s$dataName, sep = "")
      if (s$plotType == 'violin')
        res <- paste(res, ", panel = panel.violin", sep = "")
      
      if (!is.null(s$color) && !is.na(s$color))
        res<-paste(res, ", groups = ", s$color, sep = "")
      res <- paste(res, ', main = "', s$title, '"', sep = "")
      if (s$plotType == "jitter") {
        res <- paste(res, ', jitter.x = TRUE, jitter.y = TRUE', sep = "")
      }
      if (s$plotType == "line" && ! s$model %in% c("linear", "smooth", "spline")) {
        res <- paste(res, ', type = "l"', sep = "")
      }
      scales <- character(0)
      if (s$logx)
        scales <- "x = list(log = TRUE)"
      if (s$logy)
        scales <- c(scales, "y = list(log = TRUE)")
      scales <- paste(scales, collapse = ", ")
      if (nchar(scales) > 0) {
        scales <- paste(", scales = list(", scales, ")", sep = "")
        res <- paste( res, scales, sep = "" )
      }
      if (s$model=="linear")
        res <- paste(res, if (s$plotType == "line") ', type = c("l","r")' else ', type = c("p","r")', sep  = "")
      if (s$model %in% c("smooth", "spline")) {
        pt <- if (s$plotType == "line") "l" else "p"
        res <- paste0(res, ', type = c("', pt, '", "', s$model, '")')
      }
      if (s$key %in% c('top','bottom','left','right')) {
        res <- paste(res, 
                     ', auto.key = list(space = "', s$key, '"',
                     if (s$plotType == "line") ", lines = TRUE, points = FALSE" else "",
                     if (s$key %in% c("top")) ", columns = 3" else "",
                     ')', sep = "")
      }
      if (s$key %in% c('N','NE','E','SE','S','SW','W','NW')) {
        dir2pos <- list(
          'N'  = 'c(.5,1)', 
          'NE' = 'c(1,1)', 
          'E'  = 'c(1,.5)', 
          'SE' = 'c(1,0)', 
          'S'  = 'c(.5,0)', 
          'SW' = 'c(0,0)', 
          'W'  = 'c(0,.5)', 
          'NW' = 'c(0,1)'
        ) 
        res <- paste(res, ', auto.key = list(',
                     'corner = ', dir2pos[[s$key]], 
                     if (s$plotType == "line") ", lines = TRUE, points = FALSE" else "",
                     if (s$key %in% c("N", "S")) ", columns = 3" else "",
                     ")",
                     sep = "")
      }
      
      res <- paste(res, ")", sep = "")
    }
    return(res)
  }
#' @rdname mPlotting  
#' @export

mUniplot <- function(data, default = c("histogram","density", "frequency polygon", "ASH plot"),
                     system = system_choices()[1], show = FALSE, title = "", 
                     data_text = rlang::expr_deparse(substitute(data))) {
  .require_manipulate_namespace()
  system <- match.arg(system, system_choices())
  default <- match.arg(default)
  keyDefault <- ifelse ( system == "lattice", "none", "right" )
  plotnames <- list("histogram", "density", "frequency polygon", "ASH plot")
  
  variables <- .varsByType(head(data))
  # variables$q is the quantitative variables.
  snames <- .NAprepend(variables$all)
  cnames <- .NAprepend(variables$c)
  if (length(variables$q) < 1) stop("data must have at least 1 quantitative variable")
  lnames <- list("none","top","right","left",
                 "N" = "N", "NE" = "NE", 
                 "E" = "E", "SE" = "SE", 
                 "S" = "S", "SW" = "SW", 
                 "W" = "W", "NW" = "NW")
  manipulate::manipulate( 
    { .doUniplot(data, variables = variables, show = show, system = system, plotType = plotType, x = x, 
                 nbins = nbins, color = color, 
                 facet = facet, 
                 model = model, key = key, title = title, data_text = data_text) },
    show = manipulate::button("Show Expression"),
    system = manipulate::picker(as.list(system_choices()), initial = system, label = "Graphics system"),
    plotType = manipulate::picker(plotnames, initial = default, label = "Plot type"),
    x = manipulate::picker(variables$q, initial = variables$q[[1]], label = "x axis"),
    # y = manipulate::picker(variables$q, initial = variables$q[[2]], label = "y axis"),
    nbins = manipulate::slider(2, 100, initial = 25, label = "Number of bins"),
    color = manipulate::picker(snames, initial = "none ", label = "Color"),
    #              size = manipulate::picker(snames, initial = "none ", label = "Size (gg only)"),
    facet = manipulate::picker(cnames, initial = "none ", label = "Facets"),
    #              logx = manipulate::checkbox(label = "log x-axis"),
    #              logy = manipulate::checkbox(label = "log y-axis"),
    key = manipulate::picker(lnames, label = "Key", initial = keyDefault)
  )
}


.doUniplot <- function(data, variables = variables, show = FALSE, 
                       system = system_choices()[1],
                       plotType = c("histogram", "densityplot", "frequency polygon", "ASH plot"),
                       x = NA, 
                       nbins = nbins, color = NA, 
                       # size = NA, 
                       facet = NA, 
                       # logx = FALSE, logy = FALSE, 
                       model = "", key = "right",
                       title = "", data_text = rlang::expr_deparse(substitute(data)))
{
  system <- match.arg(system, system_choices())
  plotType <- match.arg(plotType)
  vals <- list(dataName = data_text, plotType = plotType, x = x, nbins = nbins, 
               color = color, 
               # size = size, 
               facet = facet, 
               # logx = logx, logy = logy, 
               key = key,
               title = title)
  
  s <- .uniplotString(vals, system, variables)
  if (show) cat(paste("\n", s, "\n"))
  p <- eval(parse(text = s))
  print(p)
  return(invisible(p))
}

# 1-variable plots
.uniplotString <- function(s, system = system_choices()[1], variables)
{
  
  check_installed('glue')
  # s$dataName <- name2string(s$dataName)
  geom <- c(`histogram` = '', `densityplot` = ', geom = "line"',    
            `frequency polygon` = ', geom = "line"', `ASH plot` = ', geom = "blank"')
  stat <- c(`histogram` = '', `densityplot` = ', stat = "density"', 
            `frequency polygon` = ', stat = "bin"', `ASH plot` = ', stat = "bin"')
  gggeoms <- c(`histogram` = 'geom_histogram', 
               `densityplot` = 'geom_density',    
               `frequency polygon` = 'geom_freqpoly',
               `ASH plot` = 'geom_ash')
  
  system = match.arg(system, system_choices())
  adjust <- 10 / s$nbins
  binwidth <- eval( parse( text = paste("diff(range( ~", s$x, ", data = ", s$dataName,", na.rm = TRUE))"))) / s$nbins
  if ( any(is.na(binwidth)) || any(is.nan(binwidth)) ) binwidth <- NULL
  if (system == "ggformula") {
    
    gf_fun <- c(`histogram` = 'gf_histogram', 
                `densityplot` = 'gf_dens',    
                `frequency polygon` = 'gf_freqpoly',
                `ASH plot` = 'gf_ash')
    
    if (!is.null(s$color) && !is.na(s$color)) {
      color_chunk <- glue::glue(.trim = FALSE, ", color = ~ {s$color}")
    } else {
      color_chunk <- ""
    }
    
    params <- if (s$plotType %in% c('histogram', 'frequency polygon', 'ASH plot')) {
      paste(", binwidth = ", signif(binwidth,2), sep = "")
    } else {
      paste(", adjust = ", signif(adjust,2), sep = "")
    }
    
    res <- glue::glue(.trim = FALSE, '{gf_fun[s$plotType]}(~ {s$x}, data = {s$dataName}{color_chunk}{params})')
    
    if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
      res <- glue::glue(.trim = FALSE, "{res} |>\n   gf_facet_wrap(~ {s$facet})")
    res <- glue::glue(.trim = FALSE, '{res} |>\n   gf_labs(title = "{s$title}")')
    
    if (!is.null(s$color) && !is.na(s$color)){
      res <- glue::glue(.trim = FALSE, '{res} |> \n  gf_theme(legend.position = {ggdir2pos(s$key)})')
    }  
  } else if (system == "ggplot2") {
    res <- paste0("ggplot( data = ", s$dataName, ", aes(x = ", s$x, "))", sep = "")
    
    params <- if (s$plotType %in% c('histogram', 'frequency polygon', 'ASH plot')) {
      paste("binwidth = ", signif(binwidth,2), sep = "")
    } else {
      paste("adjust = ", signif(adjust,2), sep = "")
    }
    res <- paste0(res, " + " , gggeoms[s$plotType], "(", params, ")")
    
    if (!is.null(s$color) && !is.na(s$color))
      res<-paste(res, " + aes(colour = ", s$color, ")", sep = "")
    #    if (!is.null(s$size) && !is.na(s$size))
    #      res<-paste(res, " + aes(size = ", s$size, ")", sep = "")
    #    if (s$logx)
    #      res <- paste(res, " + scale_x_log10()", sep = "")
    #    if (s$logy)
    #      res <- paste(res, " + scale_y_log10()", sep = "")
    if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
      res<-paste(res, " + facet_wrap(~", s$facet, ", ncol = 4)", sep = "")
    res <- paste(res, ' + labs(title = "', s$title, '")', sep = "")
    if (!is.null(s$color) && !is.na(s$color)){
      res <- paste(res, ' + theme(legend.position = ', ggdir2pos(s$key), ')', sep = "")
    }  
    
  } else {
    plotName <- c(`histogram` = 'histogram', `densityplot` = 'densityplot', 
                  `frequency polygon` = 'freqpolygon', `ASH plot` = 'ashplot')
    res <- paste( plotName[s$plotType], "( ", " ~ ", s$x, sep = "")
    if (!is.null(s$facet) && !is.na(s$facet)) # why do I need both?
      res <- paste(res, " | ", s$facet)
    res <- paste(res, ", data = ", s$dataName, sep = "")
    if (!is.null(s$color) && !is.na(s$color))
      res<-paste(res, ", groups = ", s$color, sep = "")
    res <- paste(res, ', main = "', s$title, '"', sep = "")
    if (s$plotType %in% c('histogram', 'frequency polygon', 'ASH plot')) {
      res <- paste(res, ", width  = ", signif(binwidth,2), sep = "")
    } else {
      res <- paste(res, ", adjust = ", signif(adjust,2), sep = "")
    }
    scales <- character(0)
    #    if (s$logx)
    #      scales <- "x = list(log = TRUE)"
    #    if (s$logy)
    #      scales <- c(scales, "y = list(log = TRUE)")
    #    scales <- paste(scales, collapse = ", ")
    #    if (nchar(scales) > 0) {
    #      scales <- paste(", scales = list(", scales, ")", sep = "")
    #      res <- paste( res, scales, sep = "" )
    #    }
    if (s$key %in% c('top','bottom','left','right')) {
      res <- paste(res, ', auto.key = list(space = "', s$key, '")', sep = "")
    }
    if (s$key %in% c('N','NE','E','SE','S','SW','W','NW')) {
      dir2pos <- list(
        'N'  = 'c(.5,1)', 
        'NE' = 'c(1,1)', 
        'E'  = 'c(1,.5)', 
        'SE' = 'c(1,0)', 
        'S'  = 'c(.5,0)', 
        'SW' = 'c(0,0)', 
        'W'  = 'c(0,.5)', 
        'NW' = 'c(0,1)'
      ) 
      res <- paste(res, ', auto.key = list(corner = ', dir2pos[[s$key]], ')', sep = "")
    }
    
    res <- paste(res, ")", sep = "")
  }
  
  return(res)
}

#' Functions for teaching linear algebra.
#'
#' These functions provide a formula based interface to the construction
#' of matrices from data and for fitting.  You can use them both for numerical vectors
#' and for functions of variables in data frames.
#' These functions are intended to support teaching basic linear algebra
#' with a particular connection to statistics.
#'
#' @rdname linear.algebra
#' @name linear.algebra
# @aliases mat singvals dot
#'
#' @param formula a formula.  In `mat` and `singvals`,
#' only the right-hand side is used.
#' 
#' @param data a data frame from which to pull out numerical values
#' for the variables in the formula
#' 
#' @param A an alias for `formula` for backward compatibility.  
#'
# @param \dots additional arguments (currently ignored)
#'
#' 
#' `mat` returns a model matrix
#' 
#' To demonstrate singularity, use `singvals`.
#' 
#' @return `mat` returns a matrix 
#' 
#' @seealso [project()]
#'
# @usage mat(A,data=NULL)
#' @seealso [linearModel()], which returns a function.
#' @examples
#' a <- c(1,0,0); b <- c(1,2,3); c <- c(4,5,6); x <- rnorm(3)
#' # Formula interface
#' mat(~a+b)
#' mat(~a+b+1)
#' if (require(mosaicData)) {
#' mat(~length+sex, data=KidsFeet)
#' singvals(~length*sex*width, data=KidsFeet)
#' }
#' @export

mat <- function(formula, data=parent.frame(), A = formula) {
  if (! inherits(A, "formula")) stop("Must provide a formula, e.g., ~ a or ~ a + b ")
  A <- update(A, ~-1+.) # kill off automatic Intercept term
  if (is.null(data))
    M <- model.matrix( A )
  else
    M <- model.matrix( A, data=data ) 

  attr(M, "assign") <- NULL
  rownames(M) <- NULL

  return(M)
}

#
##################
#' @rdname linear.algebra
#' @return `singvals` gives singular values for each column in the model matrix
#' @export

singvals <- function(formula, data=parent.frame(), A = formula) {
  M <- mat(A, data=data)
  # formulated to give one singular value for each column in A
	svs <- La.svd(M, nv=ncol(M), nu=ncol(M))$d;
  c( svs, rep(0, ncol(M) - length(svs)));
}
##################

#' Augmented Chi-squared test
#' 
#' This augmented version of [chisq.test()] provides more verbose
#' output.
#'
#' @param x,y,correct,p,rescale.p,simulate.p.value,B as in [chisq.test()], but 
#' `x` may also be a formula, in which case `x` is replaced by `tally(x, data)`
#' prior to the call to [chisq.test()].
#' @param data a data frame for use when `x` is a formula.
#' 
#' @seealso [chisq.test()] 
#' 
#' @examples
#' # Physicians' Health Study data
#' phs <- cbind(c(104,189),c(10933,10845)) 
#' rownames(phs) <- c("aspirin","placebo") 
#' colnames(phs) <- c("heart attack","no heart attack") 
#' phs 
#' xchisq.test(phs) 
#' xchisq.test(sex ~ substance, data = HELPrct)
#' @export

xchisq.test <-
function (x, y=NULL, correct=TRUE, p = rep(1/length(x), length(x)),
          rescale.p=FALSE, simulate.p.value=FALSE, B=2000, data = environment(x)) 
{
  if (inherits(x, "formula")) {
    x <- tally(x, data = data) 
    orig <- chisq.test(x, correct = correct, p = p, 
                       rescale.p = rescale.p, 
                       simulate.p.value = simulate.p.value, B = B)
  } else {
    orig <- chisq.test(x = x, y = y, correct = correct, p = p, 
                       rescale.p = rescale.p, 
                       simulate.p.value=simulate.p.value, B=B)
  }
    if (is.matrix(orig$observed)) {
        dd <- dim(orig$observed)
    }
    else {
        dd <- c(1, length(orig$observed))
    }
    obs <- surround(orig$observed, " ", " ", digits = 2, nsmall = 2)
    exp <- surround(orig$expected, "(", ")", digits = 2, nsmall = 2)
    contribution <- (orig$observed - orig$expected) ^ 2 / orig$expected
	if (correct && all(dd == c(2,2)) && ! simulate.p.value) {
		# use continuity correction
		contrib2 <- (abs(orig$observed - orig$expected) - .5)^2 / orig$expected
		contribution <- pmin(contribution, contrib2)
	}
    contrib <- surround(contribution, "[", "]", digits = 2, nsmall = 2)
    resid <- surround(orig$residuals, "<", ">", digits = 2, nsmall = 2)
    blank <- rep(" ", prod(dd))
    result <- c(obs, exp, contrib, resid, blank)
    dim(result) <- c(dd, 5)
    print(orig)
    for (i in 1:dd[1]) {
        for (j in 1:5) {
            cat(result[i, , j])
            cat("\n")
        }
    }
    cat("key:\n")
    cat("\tobserved\n")
    cat("\t(expected)\n")
    cat("\t[contribution to X-squared]\n")
    cat("\t<Pearson residual>\n")
    orig$contribution <- contribution
    return(invisible(orig))
}

---
title: "Resources Related to the mosaic Package"
author: "Randall Pruim, Daniel T. Kaplan, and Nicholas J. Horton"
date: "`r Sys.Date()`"

  
vignette: >
  %\VignetteIndexEntry{Resources Related to the mosaic Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



This vignette describes related resources and materials useful for teaching
statistics with a focus on modeling and computation.

## Package Vignettes

The `mosaic` package includes a number of vignettes. These are available 
from within R, from 
[cran.r-project.org/package=mosaic](https://cran.r-project.org/package=mosaic),
or from [www.mosaic-web.org/mosaic/](https://www.mosaic-web.org/mosaic/).

 * *Minimal R* describes a minimal set of R commands for use in 
	Introductory Statistics and discusses why it is important
		to keep the set of commands small;
		
 * *Resampling methods in R* demonstrates how to use the `mosaic`
	package to compute p-values for randomization tests and bootstrap confidence 
	intervals in a number of common situations. The examples are based on the 
	``resampling bake off'' at USCOTS 2011.

 * *ggformula/lattice conversion examples* compares the lattice and ggformula formula interfaces for creating graphics. 

 * *Less Volume, More Creativity*,
based on slides from an ICOTS 2014 workshop, introduces the `mosaic` package and
related tools and describes some of the philosophy behind the design choices
made in the `mosaic` package.
	
 * *Graphics with the mosaic package*
 is gallery of plots made using tools from the `mosaic` package.

## Auxiliary packages

Some features of the mosaic package are provided through auxiliary packages. These include:

* mosaicModel -- implements high-level systems for working with statistical models: effect-size calculation, bootstrapped confidence intervals, prediction error, graphics for models with multiple inputs. The package contains an introductory vignette.
* mosaicCalc -- provides the calculus components of mosaic, including integration, differentiation, and differential equation solving. 

<!-- See *[Modeling-based calculus with R/mosaic](http://www.mosaic-web.org/wp-content/uploads/2019/01/UMAP-mosaic-calculus.pdf)* for an instructor-oriented introduction and *[Start R in Calculus](http://project-mosaic-books.com/?page_id=22)* for a student-facing guide. -->

Install these packages using `install.packages(c("mosaicCalc", "mosaicModel"))`.

## Mosaic paper

Pruim R, Kaplan DT and Horton NJ (2017). The mosaic Package: Helping
Students to 'Think with Data' Using R. *The R Journal*, 9(1), pp.
77-102.  https://journal.r-project.org/articles/RJ-2017-024/index.html.

Abstract: The mosaic package provides a simplified and systematic introduction to the core functionality related to descriptive statistics, visualization, modeling, and simulation-based inference required in first and second courses in statistics. This introduction to the package describes some of the guiding principles behind the design of the package and provides illustrative examples of several of the most important functions it implements. These can be combined to help students 'think with data' using R in their early course work, starting with simple, yet powerful, declarative commands.

## Project MOSAIC Little Books

The following longer documents are available at 
[github.com/ProjectMOSAIC/LittleBooks](https://github.com/ProjectMOSAIC/LittleBooks/blob/master/README.md).

 * *Start Teaching Statistics Using R* includes 
some strategies for teaching beginners, and introduction to the `mosaic` package,
and some additional things that instructors should know about using R.
(A spanish language translation can be found at
https://github.com/fjaraavilaa/MOSAIC-LittleBooks-Spanish.)

 * *[A Student's Guide to R](https://github.com/ProjectMOSAIC/LittleBooks/raw/master/StudentGuide/MOSAIC-StudentGuide.pdf)* provides a brief introduction to 
the R commands needed for all the basic statistical procedures in 
an Intro Stats course.  
(A spanish language translation can be found at
https://github.com/fjaraavilaa/MOSAIC-LittleBooks-Spanish.)


<!--  * *[Start R in Calculus](http://project-mosaic-books.com/?page_id=22)* highlights features  -->
<!-- of R and the `mosaic` package that can be used to teach  -->
<!-- calculus with R. -->

 
## Textbook Related

 * *Statistical Modeling: A Fresh Approach* (DT Kaplan, second edition)]
is an introduction to statistics embracing a modeling approach and employing 
resampling methods. 
The `mosaic` package is used throughout.

    * [www.mosaic-web.org/StatisticalModeling](https://dtkaplan.github.io/SM2-bookdown/)

 * *Foundations and Applications of Statistics: An Introduction Using R* (R Pruim, second edition) 
is an R-infused probability and mathematical statistics text that emphasizes
connections between probability and statistics. The first edition of the book 
predates the `mosaic` package, but much of the code originally in the `fastR` 
package has been moved into the `mosaic` package. 
The second edition is supported by the `fastR2` package and uses `ggformula` for
plotting.
    * [https://bookstore.ams.org/amstext-28](https://bookstore.ams.org/amstext-28)
    * [www.ams.org/publications/authors/books/postpub/amstext-28](http://www.ams.org/publications/authors/books/postpub/amstext-28)

 * *Modern Data Science with R* (BS Baumer, DT Kaplan, and NJ Horton)
		is an R-infused data science text that emphasizes conceptual understanding and computation.

    * [http://mdsr-book.github.io/](https://mdsr-book.github.io/)
    
 * *The Statistical Sleuth in R* (NJ Horton)
		describes how to undertake analyses in R for the
		examples in the Third Edition of the
		*Statistical Sleuth: A Course in Methods of Data Analysis* (2013),
		by Fred Ramsey and Dan Schafer.

    * [nhorton.people.amherst.edu/sleuth](https://nhorton.people.amherst.edu/sleuth/)

 * *Introduction to the Practice of Statistics in R* (NJ Horton and BS Baumer)
		describes how to undertake analyses in R that are introduced as examples in 
		*Introduction to the Practice of
		Statistics*, by David Moore, George McCabe,
		and Bruce Craig. 
		
    * [9th edition: nhorton.people.amherst.edu/ips9](https://nhorton.people.amherst.edu/ips9/) 
    * [6th edition: nhorton.people.amherst.edu/ips6e](https://nhorton.people.amherst.edu/ips6e/) 

 * *Statistics: Unlocking the Power of Data* (Lock, Lock, Lock, Lock, and Lock) 
		is an introductory statistics textbook that embraces a resampling approach. 
		
    An annotated companion to the examples in the book implemented using R can be found at 
		
    * [github.com/rpruim/Lock5withR/](https://github.com/rpruim/Lock5withR/blob/master/README.md)
    
    and the `Lock5withR` R package provides all the data sets used in the text.
		
    * [lock5stat.com](https://www.lock5stat.com/) 
		

 * *Stats: Data and Models* (NJ Horton)
		describes how to undertake analyses in R for the
		examples in the Fourth Edition of 
		*Stats: Data and Models* (2015),
		by Dick de Veaux, Paul Velleman, and Dave Bock.

    * [nhorton.people.amherst.edu/sdm4](https://nhorton.people.amherst.edu/sdm4/)

 * *Intro Stats* (P Frenett and NJ Horton)
		describes how to undertake analyses in R for the
		examples in the Fourth and Fifth Editions of 
		*Intro Stats* (2013),
		by Dick de Veaux, Paul Velleman, and Dave Bock.

    * [5th edition: nhorton.people.amherst.edu/is5](https://nhorton.people.amherst.edu/is5/)
    * [4th edition: nhorton.people.amherst.edu/is4](https://nhorton.people.amherst.edu/is4/)


 * *Introduction to Statistical Investigations* (Tintle *et al*) 
		is another introductory statistics textbook that embraces a resampling approach. 
		
    An annotated companion to the examples in the book implemented using R can be found at 
		
    * [github.com/rpruim/ISIwithR/](https://github.com/rpruim/ISIwithR/blob/master/README.md)
    
    The `ISIwithR` R package provides all the data sets used in the text.
    Additional information about the book and the approach used there can be found at
		
    * [www.isi-sats.com/isi/](http://www.isi-stats.com/isi/)
		
  <!-- * [Open Intro Stats](https://www.openintro.org/) -->

  <!--   OpenIntro Stats now has versions of their labs designed for  -->
  <!--   use with the `mosaic` package.   -->
  <!--   The `mosaic` labs were adapted by  -->
  <!--   Ben Baumer and Galen Long of Smith College and updated to `ggformula` by Bonnie Lin and Nicholas Horton of Amherst College. -->

## Articles
 * GW Cobb, "The introductory statistics course: a Ptolemaic curriculum?", 
   *Technology Innovations in Statistics Education*, 2007, 1(1), 
   [escholarship.org/uc/item/6hb3k0nz](https://escholarship.org/uc/item/6hb3k0nz).

 * Fieberg JR, Vitense K, Johnson DH. 2020. Resampling-based methods for biologists. 
 PeerJ 8:e9089 <https://doi.org/10.7717/peerj.9089>
 
 * NJ Horton, BS Baumer, and H Wickham, "Teaching precursors to data science in introductory and second courses in statistics," *CHANCE*, 2015, 28(2):40-50, 
 [nhorton.people.amherst.edu/precursors](https://nhorton.people.amherst.edu/precursors/)

 * NJ Horton, and J Hardin, "Teaching the next generation of statistics students to "Think With Data": special issue on statistics and the undergraduate curriculum," *TAS*, 2015, 69(4):259-265, 
 [https://amstat.tandfonline.com/doi/full/10.1080/00031305.2015.1094283](https://amstat.tandfonline.com/doi/full/10.1080/00031305.2015.1094283)
 
 * D Nolan and D Temple Lang, "Computing in the statistics curricula", 
*The American Statistician*, 2010, 64(2), 
[www.stat.berkeley.edu/~statcur/Preprints/ComputingCurric3.pdf](https://www.stat.berkeley.edu/~statcur/Preprints/ComputingCurric3.pdf).

## Other Resources

 * *Example R Code/Graphs Using NHANES* by Amy Wagaman,
 <https://awagaman.people.amherst.edu/stat230/Stat230CodeCompilationExampleCodeUsingNHANES.pdf>

---
title: "Untitled"
author: ""
date: ""
output: 
  html_document:
    fig_height: 3
    fig_width: 5
  pdf_document:
    fig_height: 3
    fig_width: 5
  word_document:
    fig_height: 3
    fig_width: 5
---

```{r, setup, include=FALSE}
require(mosaic)   # Load additional packages here 
knitr::opts_chunk$set(
  tidy=FALSE,     # display code as typed
  size="small")   # slightly smaller font for code
```


Delete this sentence and enter your text here.

```{r}

```

---
title: ""
author: ""
date: ""
output: html_document
runtime: shiny
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, size = "small")
require(mosaic)
theme_set(theme_bw())
```

This R Markdown document is made interactive using Shiny. Unlike the more traditional workflow of creating static reports, you can now create documents that allow your readers to change the assumptions underlying your analysis and see the results immediately. 

To learn more, see [Interactive Documents](https://bookdown.org/yihui/rmarkdown/shiny-documents.html)

## Inputs and Outputs

You can embed Shiny inputs and outputs in your document. Outputs are 
automatically updated whenever inputs change.  This demonstrates how a 
plot can be made interactive by wrapping it in the Shiny `renderPlot` 
function. The `selectInput()`, `sliderInput()`, and `checkboxGroupInput()`
functions create the input widgets used to drive the plot. 

The example below illustrates the relationships among histograms, frequency
polygons, ASH plots (Average Shifted Histogram), and density plots.

```{r eruptions, echo = FALSE, eval = TRUE}
inputPanel(
  selectInput("binwidth", label = "bin width:",
              choices = c(0.1, 0.2, 0.25, 0.5, 1.0, 1.5), selected = 0.5),
  sliderInput("center", label = "center of bin:",
              min = -1.0, max = 1.0, value = 0.0, step = 0.02),
  checkboxGroupInput("show", "Show the following",
                     choices = c(`histogram` = "hist",
                                 `frequency polygon` = "freq",
                                 `ASH plot` = "ash",
                                 `density plot` = "dens"),
                     selected = "hist")
)

renderPlot(
  height = 400,
  {
    # bin_width <- as.numeric(input$binwidth)
    # center <- as.numeric(input$center)
    
    
    the_plot <- gf_empty()
    
    if ("hist" %in% input$show)
      the_plot <- the_plot %>% 
        gf_dhistogram( ~ eruptions, data = faithful, 
                       binwidth = as.numeric(input$binwidth), 
                       center = input$center,
                       fill = "skyblue",
                       color = "gray50",
                       alpha = 0.2, size = 1)
    if ("ash" %in% input$show)
      the_plot <- the_plot %>%
        gf_ash( ~ eruptions, data = faithful,
                binwidth= as.numeric(input$binwidth),
                color = ~"ASH", size = 1.2)
    if ("freq" %in% input$show)
      the_plot <- the_plot %>%
        gf_freqpoly( after_stat(density) ~ eruptions, data = faithful, 
                    binwidth = as.numeric(input$binwidth), 
                    size = 1.2, color = ~"frequency polygon",
                    alpha = 0.7,
                    center = input$center)
    if ("dens" %in% input$show)
      the_plot <- the_plot %>%
        gf_dens( ~ eruptions, data = faithful, adjust = as.numeric(input$binwidth), 
                 color = ~"density", size = 1.2, alpha = 0.7)

    the_plot <- the_plot %>%
      gf_labs(x = "Duration of eruption (minutes)",
              y = "",
              title = "Old Faithful Eruptions",
              color = "Type of plot              ") %>%
      gf_lims(x = c(0.5, 6.5), y = c(0, 1.2)) %>%
      gf_refine(
        scale_color_manual(
          values = c("ASH" = "red", "frequency polygon" = "navy", "density" = "forestgreen",
                     "histogram" = "gray50"))
        )
    the_plot
  })
```

%\VignetteIndexEntry{Minimal R (ggformula version)}
%\VignettePackage{mosaic}
%\VignetteKeywords{mosaic, vignettes, minimal}
%\VignetteEngine{knitr::knitr}

\documentclass[10pt]{report}

\usepackage[landscape,margin=.40in,top=.30in,bottom=.30in,includehead,includefoot]{geometry}

\usepackage{multicol}
\usepackage{xcolor}
\usepackage{hyperref}
\usepackage{longtable}

\usepackage[utf8]{inputenc}

%%%%  fancy family
\usepackage{fancyvrb}
\usepackage{fancyhdr}
\pagestyle{fancy}
\fancyhf{}

\renewcommand{\chaptermark}[1]{\thispagestyle{fancy}\markboth{{#1}}{}}
\renewcommand{\sectionmark}[1]{\markright{{#1}}{}}

\chead{}
\lhead[\sf \thepage]{\sf \leftmark}
\rhead[\sf \leftmark]{\sf \thepage}

\pagestyle{fancy}


%%% some local defs

\newcounter{myenumi}
\newcommand{\saveenumi}{\setcounter{myenumi}{\value{enumi}}}
\newcommand{\reuseenumi}{\setcounter{enumi}{\value{myenumi}}}

\def\R{{\sf R}}
\def\Rstudio{{\sf RStudio}}
\def\RStudio{{\sf RStudio}}
\def\term#1{\textbf{#1}}
\def\tab#1{{\sf #1}}

\providecommand{\variable}[1]{}
\renewcommand{\variable}[1]{{\color{green!50!black}\texttt{#1}}}
\providecommand{\dataframe}[1]{}
\renewcommand{\dataframe}[1]{{\color{blue!80!black}\texttt{#1}}}
\providecommand{\function}[1]{}
\renewcommand{\function}[1]{{\color{purple!75!blue}\texttt{\StrSubstitute{#1}{()}{}()}}}
\providecommand{\option}[1]{}
\renewcommand{\option}[1]{{\color{brown!80!black}\texttt{#1}}}
\providecommand{\pkg}[1]{}
\renewcommand{\pkg}[1]{{\color{red!80!black}\texttt{#1}}}
\providecommand{\code}[1]{}
\renewcommand{\code}[1]{{\color{blue!80!black}\texttt{#1}}}

\newcommand{\cran}{\href{https://www.R-project.org/}{CRAN}}
\newcommand{\rterm}[1]{\textbf{#1}}

\usepackage{textcomp}  % for \texttildelow
\newcommand{\twiddle}{\raisebox{0.5ex}{\texttildelow}}


\title{Minimal R for Intro Stats}

\author{
Randall Pruim, Project MOSAIC
}

\date{\today}


\begin{document}
\parindent=0pt

\chead{\sf \bfseries \Large Enough R for Intro Stats (ggformula version)}
\rhead{July, 2017}
\lhead{R. Pruim}

<<setup,echo=FALSE,message=FALSE,include=FALSE>>=
#source('setup.R')
require(mosaic)
require(parallel)
require(ggformula)
options(digits=4)
theme_set(theme_bw())
trellis.par.set(theme=col.mosaic())
set.seed(123)
#knit_hooks$set(inline = function(x) {
#	if (is.numeric(x)) return(knitr:::format_sci(x, 'latex'))
#	x = as.character(x)
#	h = knitr:::hilight_source(x, 'latex', list(prompt=FALSE, size='normalsize'))
#	h = gsub("([_#$%&])", "\\\\\\1", h)
#	h = gsub('(["\'])', '\\1{}', h)
#	gsub('^\\\\begin\\{alltt\\}\\s*|\\\\end\\{alltt\\}\\s*$', '', h)
#})
knitr::opts_chunk$set(
	dev="pdf",
	eval=FALSE,
	tidy=FALSE,
	fig.align='center',
	fig.show='hold',
	message=FALSE
	)
@


\let\oldchapter=\chapter
\def\chapter{\setcounter{page}{1}\oldchapter}


%\begin{center}
%\section*{Enough R for Intro Stats}
%\end{center}

\def\opt#1{#1}
\def\squeeze{\vspace*{-4ex}}

\maketitle

\newpage

\vspace{1in}

\begin{center}
	\Large
	``Less volume, more creativity."

	\medskip

	\normalsize
	Mike McCarthy, Head Coach, Green Bay Packers
\end{center}

\bigskip

\begin{multicols}{2}
	\parindent=.5cm
	\parskip=4mm

Mike McCarthy had signs proclaiming his ``Less volume, more creativity" mantra
hung on the office walls of all of his coordinators during one off-season.  When asked about it, he
said,  ``A lot of times you end up putting in a lot more volume, because you are teaching fundamentals
and you are teaching concepts that you need to put in, but you may not necessarily use
because they are building blocks for other concepts and variations that will come off of that \dots
In the offseason you have a chance to take a step back and tailor it more specifically towards your
team and towards your players." % I think we've been able to accomplish that in Green Bay."


Statistics instructors using \R\ face a similar dilemma.  \R\ is capable of so much that it is tempting
to include this, and then that, and then the other, and then one more thing.  Vectors and lists and
recycling and coercion and functions and \dots  It all seems so fundamental to the way \R\ works.  And
when mastered, these concepts do become building blocks for other concepts and variations.


But when looking back at the end of a term, we have to admit that some of these things really aren't
necessary to get the job done, and may do more harm than good for beginners.  We too need to take a step
back and tailor things toward our students and their abilities and needs.
The colored commands on the next page are sufficient for an Introductory Statistics course that includes
ANOVA, regression, and resampling techniques.  The others are optional extras.
This is followed by a 1-page sampler showing usage examples for some of the functions.


Note: These pages are intended as a guide for instructors, not as a reference card for students.  Although they
may also be useful for students, they would need supplementing with additional details.

\columnbreak

The list of functions we present are not the only sufficient set of functions, but they were carefully chosen to fit as much
as possible into a small number of paradigms.  In particular,
\begin{enumerate}
	\item
		We make use of the ``formula interface" whenever possible.

		Students will need the formula interface to do regression an ANOVA.  Since we are going
		to teach it anyway, we use formulas as consistently and as often as we can.
		In some cases, my colleagues and I have written new functions or expanded the use of existing functions
		to serve this end.  These functions are available in the \pkg{mosaic} package and are
		indicated in the comments in our palette.  Some of the data sets are from the
		\pkg{mosaicData} package.

	\item
		We use \pkg{ggformula} for graphics.

    The \pkg{ggformula} package provides an interface to \pkg{ggplot2} graphics
		that uses the same formula interface used elsewhere.
		It encourages students to think about disaggregating data according to
		the values of covariates by making this very easy to do.  In addition,
		complex plots can be created by layering simpler plots.

		Previously we used \pkg{lattice} for graphics; \pkg{lattice} also uses
		a formula interface, but layering in \pkg{lattice} is more challenging
		and the \pkg{ggformula} interface is a bit cleaner.
\end{enumerate}


Whether you use this list or some other list, we encourage you to make a complete list of the commands you
want your students to learn over the course of a semester.  Organize them by topic.  Organize them again
by syntactic structure.  Ask yourself how they look as a whole.  Have you chosen a set of functions that
fit well together?
And most importantly: What is your creativity to volume quotient?


\end{multicols}


\newpage

\lhead{\href{https://cran.r-project.org/package=mosaic/}{cran.r-project.org/package=mosaic/}}
\rhead{\href{http://www.mosaic-web.org}{http://www.mosaic-web.org} (\Sexpr{Sys.Date()})}

\begin{multicols}{4}

\iftrue
\subsection*{Help}
<<>>=
apropos()
?
??
example()
@
\fi

\subsection*{Basic Calculations}
Basic calculation works like a calculator.
<<>>=
# basic ops: + - * / ^ ( )
log(); exp(); sqrt()
@
\squeeze
<<highlight=FALSE>>=
log10(); abs(); choose()
@
% uniroot()   # root finder





\subsection*{Formula Interface}
The following syntax (often with
some parts omitted) is used for
graphical summaries, numerical summaries,
and inference procedures.

<<>>=
goal(y ~ x | z, data = mydata, ...)
@

For plots:
\begin{itemize}
	\item
		\texttt{y}: is y-axis variable
	\item
		\texttt{x}: is x-axis variable
	\item
		\texttt{z}: conditioning variable

		(separate panels)
\end{itemize}
For other things:
\medskip

`\code{y \twiddle{} x | z}' can usually be
read `\code{y} is modeled by (or depends on) \code{x}
differently for each \code{z}'.
\medskip

See the sampler for examples.



\subsection*{Numerical Summaries}
These functions have
a formula interface to match plotting.
%
<<>>=
favstats()   # mosaic
tally()      # mosaic
mean()       # mosaic augmented
median()     # mosaic augmented
sd()         # mosaic augmented
var()        # mosaic augmented
diffmean()   # mosaic
@
\iftrue
\squeeze
<<highlight=FALSE>>=
quantile()   # mosaic augmented
prop()       # mosaic
perc()       # mosaic
rank()
IQR()        # mosaic augmented
min(); max() # mosaic augmented
@
\fi

\subsection*{Graphics (with ggformula)}

%\medskip

% \texttt{lattice} is not the only option,
% but it works well because (a) it allows
% for easy multi-variable plots with good default settings,
% and (b) \texttt{lattice} uses the formula interface.
%
<<>>=
gf_boxplot()      # ggformula
gf_point()        # ggformula
gf_histogram()    # ggformula
gf_density()      # ggformula
gf_dens()         # ggformula
gf_freqpol()      # ggformula
gf_qq()           # ggformula
gf_fun()          # ggformula
makeFun()         # mosaic
@
\squeeze
<<highlight=FALSE>>=
gf_dotplot()      # ggformula
gf_bar()          # ggformula
gf_col()          # ggformula
 @
\squeeze
<<eval=FALSE>>=
mplot(HELPrct)
@

\columnbreak

\subsection*{Randomization/Simulation}
%
<<>>=
rflip()     # mosaic
do()        # mosaic
sample()    # mosaic augmented
resample()  # with replacement
shuffle()   # mosaic
@
\squeeze
<<highlight=FALSE>>=
rbinom()
rnorm()     # etc, if needed
@


\subsection*{Distributions}
%
<<>>=
gf_dist()    # ggformula
# plain
pbinom(); pnorm();
# mosaic augmented
xpnorm(); xpchisq(); xpt()
xqbinom(); xqnorm();
xqchisq(); xqt()
@

\subsection*{Inference}
%
<<>>=
t.test()       # mosaic augmented
binom.test()   # mosaic augmented
prop.test()    # mosaic augmented
xchisq.test()            # mosaic
fisher.test()
pval()                   # mosaic
model <- lm()     # linear models
summary(model)
coef(model)
confint(model) # mosaic augmented
anova(model)
makeFun(model)           # mosaic
resid(model); fitted(model)
gf_model(model)       # ggformula
@
%\squeeze
<<highlight=FALSE>>=
mplot(TukeyHSD(model))
model <- glm() # logistic reg.
@


\subsection*{Data}
<<>>=
nrow(); ncol(); dim()
inspect()            # mosaic
names()
head(); tail()
factor()
@
\squeeze
<<highlight=FALSE>>=
read.file()          # mosaic
with()
summary()
glimpse()            # dplyr
ntiles()             # mosaic
cut()
c()
cbind(); rbind()
colnames()
rownames()
relevel()
reorder()
@
\squeeze
<<highlight=FALSE>>=
rep()
seq()
sort()
rank()
@
\subsection*{Data Transformation}
Even if students don't use these in
a first course, instructors may use them
to prepare data for student use.

<<highlight=FALSE>>=
select()             # dplyr
mutate()             # dplyr
filter()             # dplyr
arrange()            # dplyr
summarise()          # dplyr
group_by()           # dplyr
left_join()          # dplyr
inner_join()         # dplyr
@

\vfill

\end{multicols}

\newpage

\chead{\sf \bfseries \Large R Sampler for Intro Stats}

\def\opt#1{#1}
\def\squeeze{\vspace*{-4ex}}
% \noindent One key to success using \R\ in Intro Stats is keeping the volume low:
% \hfill
% ``Less volume, more creativity" (Mike McCarthy, head coach, Green Bay Packers)

<<more-hooks,eval=TRUE,echo=FALSE>>=
knitr::opts_chunk$set(
	eval=TRUE,
  size='small',
	fig.width=4,
	fig.height=1.9,
	fig.align="center",
	out.width=".25\\textwidth",
	out.height=".125\\textwidth",
	tidy=TRUE,
	comment=NA
)
@

\begin{multicols}{3}
<<echo=FALSE>>=
options(width=40)
options(show.signif.stars=FALSE)
@
<<coins,fig.keep="last">>=
rflip(6)
do(2) * rflip(6)
coins <- do(1000)* rflip(6)
tally(~ heads, data=coins)
@
\vspace*{-.25in}
<<>>=
tally(~ heads, data=coins, format="perc")
tally(~ (heads>=5 | heads<=1) , data=coins)
@
\vspace*{-.20in}
<<coins-hist,fig.keep="last">>=
gf_histogram(~ heads, data = coins, binwidth = 1,
            fill = ~ (heads >=5 | heads <= 1))
@
%\columnbreak
<<tally>>=
tally(sex ~ substance, data=HELPrct)
mean(age ~ sex, data = HELPrct)
diffmean(age ~ sex, data = HELPrct)
favstats(age ~ sex, data = HELPrct)
@
\vspace*{-.15in}
<<densityplot,fig.height=2.4, tidy = FALSE>>=
gf_dens(~ age | sex, data = HELPrct,
                color = ~ substance)
@
\vspace*{-.25in}
<<bwplot>>=
gf_boxplot(age ~ substance | sex, data = HELPrct)
@
\columnbreak
<<message=FALSE>>=
pval(binom.test(~ sex, data = HELPrct))
confint(t.test(~ age, data = HELPrct))
@
\iffalse
<<tidy=FALSE>>=
model <-
  lm(age ~ sex + substance, data = HELPrct)
anova(model)
@
\fi
\iffalse
<<tidy=FALSE>>=
gf_point(Sepal.Length ~ Sepal.Width,
        color = ~Species, data = iris)
@
\fi
\vspace{-.2in}
<<fig.keep="last", tidy=FALSE, fig.height=2.3>>=
model <-
  lm(length ~ width + sex, data = KidsFeet)
l.hat <- makeFun(model)
l.hat(width = 8.25, sex = "B")
gf_point(length ~ width, data = KidsFeet,
         color = ~ sex) %>%
  gf_fun(l.hat(w, sex = "B") ~ w, color = ~"B") %>%
  gf_fun(l.hat(w, sex = "G") ~ w, color = ~"G")
@
\vspace{-.2in}
<<fig.height=1.75>>=
gf_dist("chisq", df=4)
@

<<include=FALSE>>=
tally(homeless ~ sex, data = HELPrct)
@
<<include=FALSE>>=
chisq.test(tally(homeless ~ sex, data = HELPrct))
prop.test(homeless ~ sex, data = HELPrct)
@

\iffalse
Important things that I (mostly) avoid in Intro Stats:
\begin{itemize}
	\item missing data
	\item reshaping data
\end{itemize}
\R\ has functions for these things as well.
\fi

\vfill
\end{multicols}

\end{document}

---
title: "Simulation-based inference with mosaic"
author: Daniel Kaplan, Nicholas J. Horton, and  Randall Pruim
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Simulation-based inference with mosaic}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteKeywords{mosaic, resampling, simulation, bootstrapping, permutation}
---

```{r setup, include = FALSE}
library(mosaic)
require(grDevices) 
require(datasets) 
require(stats) 
require(lattice)
require(grid) 
require(mosaic) 
require(mosaicData) 
trellis.par.set(theme=col.mosaic(bw=FALSE))
trellis.par.set(fontsize=list(text=9))
options(keep.blank.line=FALSE) 
options(width=70)
require(vcd)
require(knitr)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align="center",
  fig.show="hold"
)
options('mosaic:parallelMessage'=FALSE)
```

\def\pkg#1{#1}

\bibliographystyle{abbrvnat}

## Introduction

The mosaic package is intended to support teaching statistics and modeling (as well as calculus) in a way that embraces the possibilities offered by modern computational techniques.
Our goal is to make effective computation accessible to university-level students at an introductory level.  With the broad availability of inexpensive, fast computing and powerful, free software such as R  the rate-limiting factor in accessibility is intellectual: providing a notation in which ideas can be expressed and understood clearly.  

This document describes how to use the mosaic package to carry out randomization-based statistical inference.  The mosaic package supports an approach that is intended to be easy for students to learn and to generalize.  To this end, we adopt the following tenets:
<!-- begin itemize -->

#. Students should be able to carry out useful statistics with only a very few lines of commands.
#. Basic commands should relate to statistical operations rather than programming operations.  We seek to avoid programming overhead such as loops, counters, and accumulators.
#. Black boxes should perform a conceptually straightforward operations.  Their output should be verifiable.
#. Statements should demonstrate the logical structure of statistics, rather than merely naming high-level procedures.
#. There should be a simple path to generalizing from simple descriptions --- means, counts, proportions --- to more complicated ones such as generalized linear models.
<!-- end itemize -->

The mosaic operations allow students to implement each of the
operations in what George Cobb calls the ``3 Rs'' of statistical
inference: Randomization, Replication, and Rejection (Cobb, 2007). 
By putting the 3 Rs together in various ways, students learn to
generalize and internalize the logic of inference, rather than just
to blindly follow formulaic methods.
Terry Speed (2011) notes the changing role of simulation in statistics.

<!-- begin quote -->
[Simulation used to be] something that people did when they can't do the math...
It now seems that we are heading into an era when all statistical analysis can be 
done by simulation.
<!-- end quote -->

Arguably, the most important operation in statistics is sampling:
ideally, selecting a random subset from a population.  Regrettably,
sampling takes work and time, so instructors tend to de-emphasize the
actual practice of sampling in favor of theoretical descriptions.
What's more, the algebraic notation in which much of conventional
textbook statistics is written does not offer an obvious notation for sampling.
With the computer, however, these efficiency and notation obstacles
can be overcome.  Sampling can be placed in its rightfully central
place among the statistical concepts in our courses.

Randomization-based inference using permutation testing and bootstrapping are an
increasingly important set of techniques for introductory statistics and beyond.
To help educators usefully compare different software approaches, 
Five members of the Lock family  posed a series of problems at USCOTS 2011 
(United States Conference on Teaching Statistics), 
as described at https://www.causeweb.org/cause/uscots/uscots11/breakout,
relating to bootstrapping and permutation tests. 

Bootstrapping and permutation testing are powerful and elegant approaches to
estimation and testing that can be implemented even in many
situations where asymptotic results are difficult to find or otherwise
unsatisfactory (Efron and Tibshirani, 1993; Hesterberg et al 2005).
Bootstrapping involves sampling _with_ replacement from a population,
repeatedly calculating a sample statistic of interest to empirically construct
the sampling distribution. Permutation testing for a two group comparison is
done by _permuting_ the labels for the grouping variable, then calculating
the sample statistic (e.g. difference between two groups using these new
labels) to empirically construct the null distribution. 

We will illustrate the use of the mosaic package using the Lock randomization 
problems.  Then we will move beyond the simple settings of the Lock problems to 
show how the mosaic operations makes it straightforward to generalize the 
logic of randomization to more complicated models.

## Background and setup

### R and RStudio

R is an open-source statistical environment that has been used at a number
of institutions to teach introductory statistics.  Among other advantages, R
makes it easy to demonstrate the concepts of statistical inference through
randomization while providing a sensible path for beginners to progress to
advanced and professional statistics. RStudio (https://posit.co/)
is an open-source integrated development environment for R which facilitates
use of the system.

### Setup

The mosaic package can be installed
using the standard features of the system (this needs only be done once).  
```{r eval=FALSE, echo=TRUE}
install.packages("mosaic")
```
 

Once installed, the package must be loaded so that it is available (this must
be done within each R session).  In addition to loading the mosaic package, 
the following commands also 
set the number of digits to display by default.^[If the parallel package is installed, loading it will allow the `do()` function
in the mosaic package to take advantage of multiple processors to speed up
simulations.]

```{r moreSetup}
require(mosaic)
options(digits = 3)
```
 

Commands such as the above can be provided for students in a set-up file to be
executed automatically each time an R session is started.  

The mosaic package works within R, 
so any of the data facilities within R can be used to
provide data sets to students.  In particular, R functions 
such as `read.csv()` and `read.table()` support 
accessing data files via a website URL, avoiding the need to download data 
files to individual student machines.  
 
Here, we load one of the Lock data sets:
<!-- Mustangs <- read.csv("http://www.mosaic-web.org/go/datasets/MustangPrice.csv") -->
```{r }
library(mosaic)
data(Mustangs)
```


## The Lock problems

The Lock problems are a series of short problems in statistical inference provided in 2011 by Robin Lock *et al.* to enable instructors to compare randomization software usability.

### Lock problem 1. Bootstrapping a mean (used Mustangs) {-}

<!-- begin quotation -->

 *A student collected data on the selling prices for a sample of used
  Mustang cars being offered for sale at an internet website. The
  price (in \$1,000's), age (in years) and miles driven (in 1,000's)
  for the 25 cars in the sample are given in the [file `MustangPrice.csv`].  
  Use these data to construct a 90% confidence interval for the mean
  price (in \$1,000's) of used Mustangs.*
<!-- end quotation -->

Students should learn to examine data before undertaking formal inference.  Any of the 
standard R tools can be used for this.  We advocate the use of ggformula-style graphics functions for two reasons:
<!-- begin enumerate -->

#. They use a syntax that is consistent with the syntax used in mosaic.  
#. They extend naturally to more complicated settings.
<!-- end enumerate -->

```{r dot, fig.width=4, fig.height=2}
gf_histogram( ~ Price, data = Mustangs)
```


The basic ggformula syntax here --- operator, "formula" involving `~`, and the use of `data =` to specify the relevant data set --- will be used throughout mosaic.  For example, here is the sample mean 
```{r meanmust}
mean( ~ Price, data = Mustangs)
```
 

Those familiar with R may wonder why the statement has not been written as 
```{r eval=FALSE}
mean(Mustangs$Price)
```

You can, of course, carry out exactly the same calculation this way.  By using 
```{r eval=FALSE}
mean( ~ Price, data = Mustangs)
```

we are making graphical and numerical summaries fit into a common template and 
anticipating the next steps, including the introduction of additional operations such as 
randomization and modeling.

Resampling, also known as random selection with replacement, is an important operation in randomization.  The `resample()` function performs this.  It can be illustrated in a very basic way on a small set of numbers:
```{r }
simple = c(1, 2, 3, 4, 5)
resample(simple)
resample(simple)
resample(simple)
```


When applied to a dataframe, `resample()` selects random rows.  It's easy to demonstrate this with a statement such as:

```{r results="hide"}
resample(Mustangs)
```

```{r echo=FALSE}
set.seed(104)
head(resample(Mustangs))
cat("... and so on")
```


By reference to the case numbers in the left column, you can see that case 19 has been selected twice.

One resampling trial of the mean can be carried out with

```{r }
mean( ~ Price, data = resample(Mustangs))
```

Even though a single trial is of little use, it's a nice idea to have
students do the calculation to show that they are 
(usually) getting a different results with each resample.

Another trial can be carried out by repeating the command:

```{r }
mean( ~ Price, data = resample(Mustangs))
```
 
Let's generate five more, with a single command: 

```{r }
do(5) * mean( ~ Price, data = resample(Mustangs))
```

Now conduct 2000 resampling trials^[2000 is a reasonable number of trials 
for illustration purposes, but Monte Carlo variability can be considerably reduced
by using 10,000 or more.
], 
saving the results in an object
called <span style="color:blue">Mustangs.Price.boot</span>:^[It is a good idea to adopt a 
naming convention for your bootstrap and randomization distributions.  Repeatedly using
the same name (e.g., `Trials`) can become very confusing and lead to 
errors from misidentifying which distribution one has.  The name we have chosen here
makes it clear which data set an variable are being considered and that we are 
creating a bootstrap distribution (as opposed to simulating a null distribution).]

```{r }
mustangs_price_boot <- do(2000) * mean( ~ Price, data = resample(Mustangs))
```
 


Plotting this resampling distribution is straightforward, e.g.:
```{r hist, fig.width=4, fig.height=2}
gf_histogram( ~ mean, data = mustangs_price_boot, 
  xlab="Mean Mustang Price (in thousand dollars)")
```
 

The simplest way to translate this distribution into a confidence interval is to apply the operator for that purpose:
```{r confint}
confint(mustangs_price_boot, level = 0.90, method = "quantile")
confint(mustangs_price_boot, level = 0.90, method = "stderr")
```

The `confint()` function is a black box.  In introducing it, an instructor might well want to show the calculations behind the confidence interval in some detail using general-purpose operations.  `confint()` implements two basic approaches, one based on quantiles of the distribution and the other using normal theory.
<!-- begin itemize -->


#. Calculation of the 90% confidence interval using quantiles
```{r qdata}
qdata( ~ mean, c(.05, .95), data = mustangs_price_boot)
# alternative
cdata(~ mean, 0.90, data = mustangs_price_boot)
```
 

#. Using normal theory and the standard error.
First calculate the critical value $t_\star$ for
the appropriate degrees of freedom.  Or use $z_\star$, either for simplicity or to demonstrate how $t_\star$ differs.  In either case, the confidence level needs to be converted to the appropriate tail probability.

```{r tstar}
tstar <- qt(.95, df = 24)
zstar <- qnorm(.95)
```
 
The resulting margin of error will be
```{r margin}
tstar * sd( ~ mean, data = mustangs_price_boot)
zstar * sd( ~ mean, data = mustangs_price_boot)
```
 

<!-- end itemize -->

There's little reason to repeat this set of commands every time one needs a
confidence interval.  That's where `confint()` comes in, abstracting the
operation, allowing the user to specify a `level` and a `method` (`"quantile"`
or `"stderr"`) and avoiding the need to convert the level into a tail
probability.  The extent to which one should repeat the detailed steps of the
margin-of-error calculation is a decision the instructor can make depending on
local circumstances and the instructor's goals.  R and mosaic supports both
approaches.



### Lock problem 2: Testing a proportion (NFL Overtimes) {-}

<!-- begin quotation -->

*The National Football League (NFL) uses an overtime
period to determine a winner for games that are tied at the end of
regulation time.  The first team to score in the overtime wins the
game. A coin flip is used to determine which team gets the ball
first.  Is there an advantage to winning the coin flip?  Data from the
1974 through 2009 seasons show that the coin flip winner won 240 of
the 428 games where a winner was determined in overtime.  Treat these
as a sample of NFL games to test whether there is sufficient evidence
to show that the proportion of overtime games won by the coin flip
winner is more than one half.*
<!-- end quotation -->

By and large, introductory students understand that, if first possession of the ball is unrelated to the game outcome, one would expect to see about half of the 428 games won by the team that won the coin flip.  The question is whether the observed 240 wins is "about" half of the 428 games.  Statistical inference compares the 240 to the $428/2 = 214$ in the context of sampling variation.


 Using the built-in binomial distribution operators.

Generate a simulation where each trial is a random sample of 428
games from a world in which the null hypothesis holds true and see what proportion of 
trials give a result as or more extreme as the observed 240.  

```{r proptable}
prop( ~ rbinom(1000, prob = 0.5, size = 428) >= 240)
```


The result indicates that it's very unlikely, if the null were true, that the coin flip winner would win 240 or more times.

The exact result differs slightly from one simulation to the next, but that variation can be made small by making the number of trials larger.

```{r proptable2}
prop( ~ rbinom(1000, prob = 0.5, size = 428) >= 240)
```


In this case, it is 
tempting entirely to avoid the variation due to simulation by doing a deterministic
probability calculation rather than the simulation.  This exact reproducibility comes at
a cost, though, the need to customize the cut-off point to reflect which tail of the distribution corresponds to "as or more extreme than 240."  On the right side, this means subtracting 1 from the observed number.
```{r pbinom}
xpbinom(239, prob = 0.5, size = 428)
```
 

Of course, R can handle all of these little details and do the entire 
calculation for us if we use the `binom.test()` function.
```{r }
binom.test(x = 240, n = 428)
```


Even if you want to use the deterministic probability calculation (using either
`pbinom()` or `binom.test()`), you might want to illustrate the logic with the random-number generator.


 Explicitly simulating a coin flip.

Recognizing that coin flips are a staple of statistics courses, the
mosaic package offers a function that simulates random coin tosses
without the need to refer to the binomial distribution.
Here is one trial involving flipping 428 coins:
```{r coinflip}
do(1) * rflip(428)
```
 

We'll do 2000 trials, and count what fraction of the trials the
coin toss winner (say, "heads") wins 240 or more of the 428 attempts:
```{r flips1, fig.width=4, fig.height=3}
nfl_null <- do(2000) * rflip(428)
prop( ~ heads >= 240, data = nfl_null)
```

```{r flips, fig.width=4, fig.height=2}
gf_histogram( ~ heads, fill = ~ (heads >= 240), data = nfl_null)
```
 

The observed pattern of 240 wins is not a likely outcome under the null hypothesis.
The shading added through the <span style="color:brown">groups =</span> option helps to visually reinforce the result.



### Lock problem 3: Permutation test of means from two groups (sleep and memory) {-} 

<!-- begin quotation -->

 *In an experiment on memory (Mednicj et al, 2008), students were given lists of 24 words
  to memorize.  After hearing the words they were assigned at random
  to different groups. One group of 12 students took a nap for 1.5
  hours while a second group of 12 students stayed awake and was given
  a caffeine pill.  The results below display the number of words each
  participant was able to recall after the break.  Test whether the
  data indicate a difference in mean number of words recalled between
  the two treatments.*
<!-- end quotation -->
<!-- <- read.csv("http://www.mosaic-web.org/go/datasets/SleepCaffeine.csv") -->
```{r fetchsleep}
data(Sleep) 
```


The Sleep group tends to have remembered somewhat more words on average:
```{r obsmean}
mean(Words ~ Group, data = Sleep)
obs <- diff(mean(Words ~ Group, data = Sleep))
obs
```
 

```{r fig.width=4, fig.height=2}
gf_boxplot(Words ~ Group, data = Sleep)
```


To implement the null hypothesis, scramble the Group with respect to the
outcome, Words:
```{r }
diff(mean(Words ~ shuffle(Group), data = Sleep))
```
 

That's just one trial.  Let's try again:
```{r }
diff(mean(Words ~ shuffle(Group), data = Sleep))
```

To get the distribution under the null
hypothesis, we carry out many trials:

```{r setseed134, echo=FALSE}
set.seed(134) # make sure the result below is "typical"
```


```{r sleep, tidy=FALSE, fig.width=4, fig.height=2}
sleep_null <- do(2000) * diff(mean(Words ~ shuffle(Group), data = Sleep))
gf_histogram( ~ Sleep, fill = ~ (Sleep >= obs), data = sleep_null, 
  binwidth = 0.4,
  xlab = "Distribution of difference in means under the null hypothesis")
```
 

The one-sided p-value for this test is the proportion of  
trials which yielded a result as extreme or more extreme as the observed difference.
We can calculate the one-sided p-value for this test by summing up the number of 
trials which yielded a result as extreme or more extreme as the observed difference.
Here only `r sum(sleep_null$Sleep >= obs)` of the 2000 trials gave a result that large, so the p-value is equal to `r prop(~ sleep_null$Sleep >= obs)`.  We conclude that
it's unlikely that the two groups have the same mean word recall back in their respective populations.


### Lock problem 4: Bootstrapping a correlation {-}

<!-- begin quotation -->
*The data on Mustang prices in Problem \#1 also contains the number
 of miles each car had been driven (in thousands).  Find a
 95% confidence interval for the correlation between price and mileage.*
<!-- end quotation -->

We can follow essentially the same workflow to obtain a confidence interval
for other statistics, including the correlation coefficient.^[Naive 
confidence intervals based on percentiles or bootstrap standard errors from 
small samples may not be
as good as other methods (for example, the bootstrap t is preferred when
generating a confidence interval for a mean), and the ability to easily 
create simulated bootstrap distributions does not guarantee that the resulting
intervals will have the desired properties.  Alternative methods and diagnostics
are important topics not covered here.]
```{r }
cor(Price ~ Miles, data = Mustangs)
```


```{r }
mustangs_cor_boot <- do(2000) * cor(Price ~ Miles, data = resample(Mustangs))
quantiles <- qdata( ~ cor, c(.025, .975), data = mustangs_cor_boot)
quantiles
```


```{r cor, fig.width=4, fig.height=2, tidy=FALSE}
mustangs_hist <- mutate(mustangs_cor_boot, 
  colorval = cut(cor, c(-Inf, quantiles, Inf),
    labels = c("Lower", "Middle", "Upper")))
gf_histogram( ~ cor, data = mustangs_hist, fill = ~ colorval, n = 50) 
confint(mustangs_cor_boot)
```
 


## Moving beyond means and proportions

The Lock problems refer to the descriptive statistics encountered in many 
introductory applied statistics courses: 
means, proportions, differences in means and proportions. There are advantages, however, 
in introducing such basic statistics in the context of a more general framework: 
linear models.  

Usually, people think about linear models in the context of regression.  For instance, using the Lock Mustang price dataset, 
there is presumably a relationship between the price of a car and its mileage.  Here's a graph:

```{r price-mileage-graph, fig.width=4, fig.height=3}
gf_point(Price ~ Miles, data = Mustangs) %>%
  gf_lm()
```

There's a pretty clear relationship here, which in the Lock problems was quantified with the correlation coefficient.

In regression, one finds a straight-line functional relationship between the two variables.
The built-in R `lm()` function does the calculations:
```{r mustangregression}
lm(Price ~ Miles, data = Mustangs)
```

The notation used is the same as used to produce a scatter plot.
The result indicates that for every additional 1000 miles driven, the price of a car 
typically decreases by \$219 dollars (or, given the units of the data, 0.2188 thousand 
dollars).  In other words, the value of a Mustang goes down by about 22 cents per mile 
driven.

This number, 22 cents decrease per mile, can be thought of as a slope.  It's likely much more meaningful to students (and car buyers!) than the correlation coefficient of $-0.82$.  The ability to give a description in terms of predicted change is an advantage of using regression as a description rather than correlation.  There are more such advantages to regression, which we will describe later.  But for now, our emphasis is on the use of regression as a framework for the traditional sorts of calculations of means, proportions, and differences of means and proportions.

The mean price of the Mustangs is, calculated in the ordinary way:
```{r }
mean( ~ Price, data = Mustangs)
```


The same result can be generate via a linear model:
```{r }
lm(Price ~ 1, data = Mustangs)
```


This may seem obscure.  In `lm()`, the `~` notation is used to identify the
response variable and the explanatory variables.  In `Price ~ Miles`, the response variable is `Price` and `Miles` is being used as the explanatory variable.  
The mileage varies from car to car; this variation in mileage is used to explain the variation in price.

In `Price ~ 1`, the `1` can be thought of as signifying that there are no explanatory variables, or, from a slightly different perspective, that the cars are all the same. The `mean()` function is set up to accept this same notation:
```{r }
mean(Price ~ 1, data = Mustangs)
```

Why?  So that the notation can be extended to include calculating the mean for different groups.  To illustrate, consider the Lock `Sleep` data which compares the ability to memorize words after a nap versus after taking caffeine.

The mean number of words memorized, ignoring the explanatory group, is:
```{r }
mean(Words ~ 1, data = Sleep)
```

To break this down by groups, use an explanatory variable:
```{r }
mean(Words ~ Group, data = Sleep)
```


Conventionally, regression is introduced as a technique for working with quantitative explanatory variables, e.g. the `Miles` in the Mustang data, while groupwise means are used for categorical variables such as `Group` in the sleep data.  But the regression technique applies to categorical data as well:
```{r }
lm(Words ~ Group, data = Sleep)
```

What's different between the information reported from `mean()` and the result of `lm()` is the format of the results.  The `GroupSleep` coefficient from `lm()` gives the *difference* between the two groups.
```{r }
diffmean(Words ~ Group, data = Sleep) 
```


The `lm()` function also can be used to calculate proportions and differences in proportions.  We'll illustrate with the `HELPrct` data which contains information about patients in the Health Evaluation and Linkage to Primary Care randomized clinical trial.  Consider the proportion of people who are reported as homeless:
```{r }
prop(homeless ~ 1, data = HELPrct)
```


The proportion of patients who are homeless differs somewhat between the sexes; males are somewhat more likely to be homeless.
```{r }
prop(homeless ~ sex, data = HELPrct)
```


The difference between these two proportions is:
```{r }
diffprop(homeless ~ sex, data = HELPrct)
```


The `lm` function gives the same results. (You have to specify which level you want the proportion of, "homeless" or "housed".)
```{r }
lm(homeless=="homeless" ~ 1, data = HELPrct)
```


Why use `lm()` when `mean()` and `prop()` and 
(and `diffmean()` and `diffprop()`) will give the same result?  
Because `lm()` can be generalized.  It works for both categorical and 
quantitative explanatory variables and, very importantly, `lm()` 
allows multiple explanatory variables to be used.

Even when using `mean()` and `prop()`, there's a reason to prefer the `~1` notation.  It emphasizes that the calculation is not trying to explain the variability in the response variable; there are no explanatory variables being used.  


```{r }
lm(homeless=="homeless" ~ sex, data = HELPrct)
```


### Randomization with linear models

Randomization can be performed with `lm()` in the same style as with `mean()` and `prop()`.  Here, for instance, is a resampling confidence interval on the 
price change with mileage in the Mustang data:
```{r }
mustangs_lm_boot <- do(2000) * lm(Price ~ Miles, data = resample(Mustangs))
confint(mustangs_lm_boot)
# compare to large sample estimate
confint(lm(Price ~ Miles, data = Mustangs))
```



Or, we can calculate a permutation test on the difference between age among men and women using the `HELPrct` data:

```{r }
# large sample estimate
msummary(lm(age ~ sex, data = HELPrct))
# compare to permutation test
HELPrct_null <- do(2000) * lm(age ~ shuffle(sex), data = HELPrct)
prop(~ (abs(sexmale) > 0.7841), data = HELPrct_null) 
```


### Multiple explanatory variables

The regression framework allows more than one explanatory to be used.  For instance, one can examine how each of `Age` and `Miles` is related to the `Price` of the Mustangs.

```{r }
mustangs_boot1 <- do(2000) * lm(Price ~ Age, data = resample(Mustangs))
mustangs_boot2 <- do(2000) * lm(Price ~ Miles, data = resample(Mustangs))
mustangs_boot3 <- do(2000) * lm(Price ~ Miles + Age, data = resample(Mustangs))

```

The first model suggests that `Price` goes down by about one to two thousand dollars per year of `Age`. 

```{r }
confint(mustangs_boot1)
```

The second model indicates that `Price` goes down by about 16 to 28 cents per mile driven.
```{r }
confint(mustangs_boot2)
```


The third model puts each explanatory variable in the context of the other, and suggests that `Age` may just be a proxy for `Miles`.
```{r }
confint(mustangs_boot3)
```

One way to interpret this result is that, adjusting for `Miles`, 
there is little 
evidence for an `Age` effect.  The inflation of the confidence intervals for 
`Miles` and `Age` is a result of the collinearity 
between those explanatory variables. 

We can compare the last model to the large sample result.

```{r}
confint(lm(Price ~ Miles + Age, data = Mustangs))
```

### More on Multiple Regression 

The previous example examining mustang prices using both age and miles demonstrates one method of bootstrapping commonly known as case resampling. Observations from the original data are randomly sampled to create a new bootstrap sample where further analysis (in this case multiple regression) are conducted.

Another method, residual resampling, involves adding randomly sampled residuals to the fitted y values to generate a new response vector. For example, if we had a data table of 3 observations along with their residuals after fitting on a linear model, we can use `resample()` to randomly select residuals for each observation, and generate a new predicted value by adding the original fitted y value to the resampled residual:


```{r results = "asis", echo = FALSE}
ds <- data.frame(
  y = c(234,324,231), residual = c(25,-23,-22))
set.seed(104)

ds <- ds %>%
  mutate(
    predicted = y - residual,
    resamp_residual = resample(residual),
    resamp_observed = predicted + resamp_residual) %>%
  select(y, predicted, residual, resamp_residual, resamp_observed)
knitr::kable(ds)
```

Here we see that the results are unchanged for the first car but the second and third cars have swapped residuals.


The `relm()` function does the aforementioned process in a convenient way. We can use `do()` to repeat this process and create a bootstrap distribution to generate a range of possible slope coefficient values.

```{r}
set.seed(104)

mustang_mod <- lm(Price ~ Miles + Age, data = Mustangs)
mustang_relm_boot <- do(2000) * relm(mustang_mod)
```
A 95% confidence interval can be constructed, and a histogram for the slope coefficent `Miles` variable is shown below.
```{r}
confint(mustang_relm_boot)
gf_histogram( ~ Miles, data = mustang_relm_boot)
```


#### Simulation and ANOVA

One way to think about ANOVA is a way of quantifying the extent to which a model term, in the context of other model terms, is better than random "junk." Consider, for example, the role of `Age` in a model of the Mustang prices that includes `Miles`.  Here's one ANOVA report:
```{r }
anova(lm(Price ~ Miles + Age, data = Mustangs))
```

The p-value on `Age` suggests that `Age` is not contributing to the model.  But there's a different ANOVA report available that suggests a different conclusion.
```{r }
anova(lm(Price ~ Age + Miles, data = Mustangs))
```

The difference between the two ANOVA reports can be explained in 
several ways, for example by regarding ANOVA as a sequential report 
on a nested sequence of models, so that the order of model terms makes 
a difference.  For instance, the $R^2$ from this sequence of models suggests 
that `Age` doesn't add much to `Miles`.

```{r }
do(1) * lm(Price ~ Miles, data = Mustangs)
do(1) * lm(Price ~ Miles + Age, data = Mustangs)
```


Here, `do()` has been used without any randomization, merely to format the results in a way that allows them be readily compared.  Note that the addition of `Age` as an explanatory variable has increased $R^2$ by a little bit, from $0.680$ to $0.681$.  

Shuffling `Age` allows the actual change in $R^2$ to be compared to what would be expected under the null hypothesis:
```{r }
mustangs_age_boot <- do(2000) * lm(Price ~ Miles + shuffle(Age), data = Mustangs )
favstats(~ r.squared, data = mustangs_age_boot)
cdata(~ r.squared, .95, data = mustangs_age_boot)
```


The observed $R^2$ from Price ~ Miles + Age- falls in the middle portion of the 
observed range when `Age` is shuffled.  We can conclude that `Age` is not an important predictor.

The result is very different when `Miles` is shuffled instead.
```{r }
mustangs_miles_boot <- do(2000) * lm(Price ~ shuffle(Miles) + Age, data = Mustangs)
favstats(~ r.squared, data = mustangs_miles_boot)
cdata(~ r.squared, .95, data = mustangs_miles_boot)
```

The observed value of $R^2 = 0.681$ falls well above the central 95% of resampled values;
we conclude that  `Miles` is an important predictor.

## Using simulations in other ways

The basic technology of resampling and shuffling can be used to
demonstrate many other concepts in statistics than the generation of
confidence intervals and p-values.  For example, it is very useful for
showing the origins of distributions such as t and F.  

For the sake of illustration, consider a $\chi^2$-test of the independence of `homeless` and `sex` in the `HELPrct` data.
To illustrate, here is a simulation to construct the distribution of p-values from a simple test under the null hypothesis.  The 

<!-- begin enumerate -->

#. Carry out the test, e.g.
```{r }
chisq.test(tally( ~ homeless + sex, 
                   data = HELPrct, margins = FALSE))
```


#. Modify the statement to extract the portion of interest.  
In this case, the p-value:
```{r }
pval(chisq.test(tally( ~ homeless + sex, 
                   data = HELPrct, margins = FALSE)) )
```


#. Insert randomization to implement the null hypothesis:
```{r }
pval(chisq.test(tally( ~ shuffle(homeless) + sex, 
                         data = HELPrct, margins = FALSE)))
```


#. Iterate to get the distribution under the null:
```{r }
chisq_null <- do(2000)* pval(chisq.test(tally( ~ shuffle(homeless) + sex, 
                         data = HELPrct, margins = FALSE)))
```

<!-- end enumerate -->
Strictly speaking, only the last step is needed.  The others are merely to 
illustrate construction of the statement and how each individual component
fits into the whole.

Students often think that when the null hypothesis is true, p-values should be large.
They can be surprised to see that the distribution of p-values under the null hypothesis is uniform from 0 to 1, or that there is a roughly 5% chance of rejecting the null 
(at $p < 0.05$) even when the null hypothesis is true.^[We can also observe in 
this data the challenges of approximating a discrete 
distribution that takes on relatively few values with a continuous distribution
like the Chi-squared distribution.]

```{r fig.width=4, fig.height=2}
prop( ~ (p.value < 0.05), data = chisq_null)
gf_histogram( ~ p.value, data = chisq_null, binwidth = 0.1, center = 0.05)
```

```{r fig.width=4, fig.height=2}
qqmath( ~ p.value, data = chisq_null, dist = qunif)
```


Perhaps more important, with the simulation approach it's straightforward to segue into more advanced and appropriate models and tests.  So, rather than the esteemed $\chi^2$ test, perhaps something a bit more modern and flexible such as logistic regression with `age` as a covariate:
```{r }
HELP_logistic_boot <- do(2000) * 
   glm(homeless=="homeless" ~ age + sex, 
     data = resample(HELPrct), family = "binomial")
confint(HELP_logistic_boot)
```


## Acknowledgments

Thanks to Sarah Anoke for useful comments on a draft and Jessica Yu for editorial assistance.
We also thank Robin Lock of St. Lawrence University for organizing the resampling demonstration
session at USCOTS 2011.

Project MOSAIC was supported by the US National Science Foundation (DUE-0920350).
More information about the package and this initiative can be 
found at the Project MOSAIC website: www.mosaic-web.org.

## Technical information

This document was produced using `r sessionInfo()$R.version$version.string` and version
`r sessionInfo()$otherPkgs$mosaic$Version` of the mosaic package.

## References
<!-- begin itemize -->

#. G. W. Cobb, The introductory statistics course: a Ptolemaic curriculum?, 
   _Technology Innovations in Statistics Education_, 2007, 1(1).
#. B. Efron \& R. J. Tibshirani, *An Introduction to the Bootstrap*, 1993, Chapman \& Hall, New York.
#. T. Hesterberg, D. S. Moore, S. Monaghan, A. Clipson \& R. Epstein.  *Bootstrap Methods and Permutation Tests (2nd edition)*, (2005), W.H. Freeman, New York.
#. D.T. Kaplan, *Statistical Modeling: A Fresh Approach*, 2nd edition, 
https://dtkaplan.github.io/SM2-bookdown/.
#. S.C. Mednicj, D. J. Cai, J. Kanady, S. P. Drummond.  "Comparing the benefits of caffeine, naps and placebo on verbal, motor and perceptual memory", *Behavioural Brain Research*, 2008, 193(1):79-86.
#. T. Speed, "Simulation", _IMS Bulletin_, 2011, 40(3):18.
<!-- end itemize -->

<!-- end document -->

---
title: "Less Volume, More Creativity -- Getting Started with the mosaic Package"
author: "Randall Pruim (updated by Nicholas Horton)"
date: '`r Sys.Date()`'
---

<style type="text/css">
span.boxed {
  border:5px solid gray;
  border-radius:10px;
  padding: 5px;
}
span.invboxed {
  border:5px solid gray;
  padding: 5px;
  border-radius:10px;
  color: white;
}
table, td, th
{
border:0px;
}
</style>

```{r, include = FALSE}
require(lubridate)
require(dplyr)
require(mosaic)
require(mosaicData)
theme_set(theme_bw())
trellis.par.set(theme = col.mosaic())
require(knitr)
opts_chunk$set(
  size = 'tiny', 
  tidy = FALSE,
  fig.width = 6, 
  fig.height = 3,
  fig.align = "center",
  out.width = "70%"
)
options(width = 90)
```

## Project MOSAIC and the `mosaic` package

NSF-funded project to develop a new way to introduce mathematics, statistics, computation 
and modeling to students in colleges and universities.

 * more information at [mosaic-web.org](http://www.mosaic-web.org)
 
 * the `mosaic` package is available via 
     * [CRAN](https://cran.r-project.org/package=mosaic)
     * [github](https://github.com/ProjectMOSAIC/mosaic) 
         * updates more frequently than CRAN
         * use `devtools::install_github(ProjectMOSAIC/mosaic)` to install 
         directly from github
         * add the optional `ref = "beta"` to install from the beta (developmental) 
         branch
         * add `build_vignettes = TRUE` if your system is equipped to build vignettes.
     
### A note about this document  

This document was originally created as an R presentation to be used as slides
accompanying various presentations.  It has been converted into a more traditional
document for use as a vignette in the `mosaic` package.

The examples below use the `mosaic` and `mosaicData` packages.  An earlier version
of this document used `lattice` graphics, but it has been updated to use `ggformula`

```{r}
library(mosaic)  # loads mosaicData and ggformula as well
```

## Less Volume, More Creativity

Many of the guiding principles of the `mosaic` package reflect the 
"Less Volume, More Creativity" mantra of Mike McCarthy who had a large
poster with those words placed in the "war room" (where assistant coaches
decide on the game plan for the upcoming opponent) as a constant reminder
not to add too much complexity to the game plan.


<table><tr align="top"><td width="20%" align="top">
<img src="images/MikeMcCarthy.jpg" align="top" width="200"></td>
<td align="top"> A lot of times you end up putting in a lot more volume, because you are teaching fundamentals  and you are teaching concepts that you need to put in, but you may not necessarily use because they are building blocks for other concepts and variations that will come off of that ... In the offseason you have a chance to take a step back and tailor it more specifically towards your  team and towards your players." 
<br><br>
Mike McCarthy, former Head Coach, Green Bay Packers 
</td>
</tr>
</table>

                                          
### SIBKIS: See It Big, Keep It Simple

Here is another elegant phrasing of a similar principle.
<table>
<tr>
<td>
Perfection is achieved, not when there is nothing more to add, but when there is nothing left to take away. 
<br><br>
--- Antoine de Saint-Exupery (writer, poet, pioneering aviator)
</td>
<td width="20%"> 
<img src="images/SaintExupery.jpg">
</tr>
</table>

## Less Volume, More Creativity in R

One key to successfully introducing R is finding a set of commands that is

 * **small**: fewer is better
 * **coherent**:  commands should be as similar as possible
 * **powerful**: can do what needs doing 
 
It is not enough to use R, it must be used elegantly.

Two examples of this principle:

* the **mosaic** package
* the **dplyr** package (Hadley Wickham)

### Minimal R

**Goal:** a minimal set of R commands for Intro Stats

**Result:** Minimal R Vignette (`vignette("MinimalR")`)

Much of the work on the `mosaic` package has been motivated
by 

* The Less Volume, More Creativity approach
* The Minimal R goal

### A few little details

If you (or your students) are just getting started with R, it is good to 
keep the following in mind:

#### R is case sensitive
 * many students are *not* case sensitive

#### Arrows and Tab
 * up/down arrows scroll through history
 * TAB completion can simplify typing

#### If all else fails, try ESC
 * If you see a + prompt, it means R is waiting for more input
 * If this is unintentional, you probably have a typo
 * ESC will get you pack to the command prompt

## The Most Important Template
&nbsp;

The following template is important because we can do so much with it.

&nbsp;

<center>
<h2><strong><span class="invboxed">goal</span> ( <span class="invboxed">yyy</span> ~ <span class="invboxed">xxx</span> , data = <span class="invboxed">mydata</span> )</strong> 
</h2></center>

&nbsp;

It is useful to name the components of the template:

&nbsp;

<center>
<h2><strong><span class="boxed">goal</span> ( <span class="boxed">&nbsp;y&nbsp;</span> ~ <span class="boxed">&nbsp;x&nbsp;</span> , data = <span class="boxed">mydata</span> )</strong> 
</h2></center>

 
&nbsp;
We're hiding a bit of complexity in the template, and there will be times 
that we will want to gussy things up a bit.  We'll indicate that by adding
`...` to the end of the template.  Just don't let `...` become a distractor
early on.

&nbsp;

<center>
<h2><strong><span class="boxed">goal</span> ( <span class="boxed">&nbsp;y&nbsp;</span> ~ <span class="boxed">&nbsp;x&nbsp;</span> , data = <span class="boxed">mydata</span> , ...)</strong> 
</h2></center>
&nbsp;

### Other versions
Here are some variations on the template.
```{r, eval = FALSE}
# simpler version
goal(~ x, data = mydata)          
# fancier version
goal(y ~ x | z , data = mydata)   
# unified version
goal(formula, data = mydata)     
```

## 2 Questions

Using the template generally requires answering two questions.  (These questions
are useful in the context of nearly all computer tools, just substitute "the computer" in for R in the questions.)

&nbsp;
<center>
<h2><strong><span class="boxed">goal</span> ( <span class="boxed">&nbsp;y&nbsp;</span> ~ <span class="boxed">&nbsp;x&nbsp;</span> , data = <span class="boxed">mydata</span> )</strong> 
</h2></center>
 
&nbsp;


###  What do you want R to do?  (goal)
 * This determines the R function to use 


###  What must R know to do that?  
 * This determines the inputs to the function
 * Must identify the variables and data frame

## How do we make this plot? (Questions)
```{r, echo=FALSE, out.width="60%", out.height="35%"}
gf_point(births ~ date, data = Births78) 
```

### What is the Goal?

 * 

### What does R need to know?

 * 
 * 

### How do we make this plot? (Answers)

#### What is the Goal?

* a scatter plot (`gf_point()`)

#### What does R need to know?

 * which variable goes where (`births ~ date`)
 * which data set (`data = Births78`)
    * use `?Births78` for documentation

#### Putting it all together

```{r, echo=TRUE, out.width="60%"}
gf_point(births ~ date, data = Births78) 
```

### Your turn: How do you make this plot?


```{r, echo=FALSE}
gf_boxplot(age ~ substance, data = HELPrct, xlab = "substance")
```

Some things you will need to know:

1. Command: <code>gf_boxplot()</code>

2. The data: <code>HELPrct</code>
 * Variables: <code>age</code>, <code>substance</code>
 * use `?HELPrct` for info about data

#### Answer

```{r, echo=TRUE}
gf_boxplot(age ~ substance, data=HELPrct)
```

### Your turn: How about this one?

```{r, echo=FALSE}
gf_boxplot(substance ~ age, data = HELPrct)
```

Some things you will need to know:

1. `gf_boxplot()` chooses between horizontal and 
vertical orientation based on the types of variables
assigned to each axis.

2. The data: <code>HELPrct</code
>
   * Variables: <code>age</code>, <code>substance</code>
   * use `?HELPrct` for info about data

#### Answer

```{r, echo=TRUE}
gf_boxplot(substance ~ age, data = HELPrct)
```

Note that we have reversed which variable is mapped to 
the x-axis and which to the y-axis by reversing their order 
in the formula. 

### Graphical Summaries: One Variable
```{r, }
gf_histogram(~ age, data = HELPrct) 
```

Note: When there is **one variable** it is on 
the **right side** of the formula.

### Graphical Summaries: Overview

#### One Variable
```{r, eval=FALSE, tidy=FALSE}
  gf_histogram( ~ age, data = HELPrct) 
    gf_density( ~ age, data = HELPrct) 
    gf_boxplot( ~ age, data = HELPrct) 
         gf_qq( ~ age, data = HELPrct) 
   gf_freqpoly( ~ age, data = HELPrct) 
```

#### Two Variables

```{r, eval=FALSE, tidy=FALSE}
gf_point(i1 ~ age,          data = HELPrct) 
gf_boxplot(age ~ substance, data = HELPrct) 
```

**Note:** `i1` is the average number of drinks (standard units) 
consumed per day in the past 30 days (measured at baseline)


### The Graphics Template

#### One variable
<center>
<h3><strong><span class="boxed">plotname</span> ( 
~ <span class="boxed">&nbsp;x&nbsp;</span> , data = <span class="boxed">mydata</span> , ...)</strong> 
</h3></center>

 * `gf_histogram()`,  `gf_qq()`,  `gf_density()`, `gf_freqpoly()`

&nbsp;


#### Two Variables
<center>
<h3><strong><span class="boxed">plotname</span> ( <span class="boxed">&nbsp;y&nbsp;</span> ~ <span class="boxed">&nbsp;x&nbsp;</span> , data = <span class="boxed">mydata</span> , ...)</strong> 
</h3></center>


 * `gf_point()`, `gf_line()`, `gf_boxplot()`

### Your turn
Create a plot of your own choosing with one of these data sets
```{r, eval=FALSE}
names(KidsFeet)    # 4th graders' feet
?KidsFeet
```
```{r, eval=FALSE}
names(Utilities)   # utility bill data
?Utilities
```
```{r, eval=FALSE}
require(NHANES)    # load package
names(NHANES)      # body shape, etc. 
?NHANES
```


### groups and panels

* Add `color = ~ `<span class="invboxed">group</span> 
 or `fill = ~ `<span class="invboxed">group</span> 
to overlay with different colors.
 
* Use `y ~ x | z` to create multipanel plots.

Here is an example.

```{r, tidy=FALSE}
gf_density( ~ age | sex, data = HELPrct, fill = ~ substance)
```

Beginners can create plots with 3 or 4 variables easily and quickly using
this template.

Bells & Whistles
================

The `ggformula` graphics system includes lots of bells and whistles including

 * titles
 * axis labels
 * colors
 * sizes
 * transparency
 * etc, etc.
 
I used to introduce these too early.  My current approach: 

 * Let the students ask or 
 * Let the data analysis drive 

#### An example with some bells and whistles

```{r}
library(lubridate)
Births78 <- Births78 %>%
  mutate(weekday = wday(date, label = TRUE, abbr = TRUE))
gf_line(births ~ date, color = ~ weekday, data = Births78)
```

**Notes** 

 * `wday()` is in the `lubridate` package
 * This version of the plot reveals a clear weekend (and holiday) pattern.  Typically, I like to have students conjecture about the "double wave" pattern and see if we can build plots to test their conjectures.

## Numerical Summaries

The `mosaic` package provides functions that make it simple to create
numerical summaries using the same template used for graphing (and later for 
describing linear models).

### Numerical Summaries: One Variable

Big idea: 

 * Replace plot name with summary name
 * Nothing else changes
 
```{r, fig.show='hold'}
gf_histogram( ~ age, data = HELPrct)  # binwidth = 5 (or 10) might be good here
        mean( ~ age, data = HELPrct)
```

#### Other summaries
The mosaic package includes formula aware versions of
`mean()`,
`sd()`,
`var()`,
`min()`,
`max()`,
`sum()`,
`IQR()`, ...

Also provides `favstats()` to compute our favorites.
```{r}
favstats( ~ age, data = HELPrct)
```

`favstats()` quickly becomes a go-to function in our courses.

`df_stats()` is similar, but

* stores the results in a data frame
* can be used to make custom summary tables

```{r, df-stats}
df_stats( ~ age, data = HELPrct)
df_stats( ~ age, data = HELPrct, mean, sd, median, iqr)
```

### Tallying 

```{r}
tally(~ sex, data = HELPrct)
tally(~ substance, data = HELPrct)
df_stats(~ substance, data = HELPrct, counts, props)
```

### Numerical Summaries: Two Variables

There are three ways to think about this.  All do the same thing.
```{r, eval = FALSE}
sd(age ~ substance, data = HELPrct)
sd(~ age | substance, data = HELPrct)
sd(~ age, groups = substance, data = HELPrct)  
# note option color = ~ substance is used for graphics
```
```{r, echo=FALSE}
sd(~ age, groups = substance, data = HELPrct)
```

This makes it possible to easily convert three different types of plots
into the (same) corresponding numerical summary.

`df_stats()` can also be used with multiple variables
and provides a different output format.

```{r, df-stats2}
df_stats(age ~ substance, data = HELPrct, sd)  
```

### Numerical Summaries: Tables

2-way tables are just tallies of 2 variables.

```{r}
tally(sex ~ substance, data = HELPrct)
tally( ~ sex + substance, data = HELPrct)
df_stats(sex ~ substance, data = HELPrct, counts)

```

Other output formats are available
```{r}
tally(sex ~ substance,   data = HELPrct, format = "proportion")
tally(substance ~ sex,   data = HELPrct, format = "proportion", margins = TRUE)
tally(~ sex + substance, data = HELPrct, format = "proportion", margins = TRUE)
tally(sex ~ substance,   data = HELPrct, format = "percent")
df_stats(sex ~ substance,   data = HELPrct, props, percs)
```

#### More examples
```{r, echo=FALSE}
HELPrct <- mutate(HELPrct, sex = factor(sex, labels = c('F','M')),
                     substance = factor(substance, labels = c('A', 'C', 'H')))
```
```{r, size='small'}
mean(age ~ substance | sex, data = HELPrct)
mean(age ~ substance | sex, data = HELPrct, .format = "table")
```

```{r, echo=FALSE}
rm(HELPrct)
data(HELPrct)
```
 * I've abbreviated some labels to make things fit better.  You can do this using `mutate()` (in the `dplyr` package) or `transform()`.
 * This also works for `median()`, `min()`, `max()`, `sd()`, `var()`, `favstats()`, etc.

### One Template to Rule a Lot

This master template can be used to do a large portion of what needs 
doing in an Intro Stats course.

 * single and multiple variable graphical summaries
 * single and multiple variable numerical summaries
 * linear models
 
```{r, eval=FALSE}
      mean(age ~ sex, data = HELPrct)
gf_boxplot(age ~ sex, data = HELPrct) 
        lm(age ~ sex, data = HELPrct)
```

```{r, echo=FALSE}
  mean(age ~ sex, data = HELPrct)
    coef(lm(age ~ sex, data = HELPrct))
```

It can be learned early and practiced often so that students become 
secure in their ability to use these functions.

## Some other things
The `mosaic` package includes some other things, too

 * data sets (they have now been moved to separate `mosaicData` and `NHANES` packages)
 * xtras: `xchisq.test()`, `xpnorm()`, `xqqmath()`
   * these functions add a bit of extra output to the similarly named functions that don't have a leading `x`
 * `mplot()` 
   * `mplot(HELPrct)` interactive plot creation
   * replacements for `plot()` in some situations
 * simplified `gf_histogram()` controls (e.g., `binwidth`)
 * simplified ways to add onto lattice plots (`gf_refine()`)

### Examples

```{r}
xpnorm(700, mean = 500, sd = 100)
```

```{r}
xpnorm(c(300, 700), mean = 500, sd = 100)
```

```{r, echo=FALSE}
phs <- cbind(c(104,189),c(10933,10845))
colnames(phs) <- c("heart attack","no heart attack")
rownames(phs) <- c("aspirin","placebo")
```

```{r}
xchisq.test(phs)
```

## Modeling 
Modeling is really the starting point for the `mosaic` 
design.

 * linear models (`lm()` and `glm()`) defined the template
 * `lattice` graphics use the template (so we chose `lattice`)
 * we added functionality so numerical summaries can be done with the same template
 * additional things added to make modeling easier for beginners

### Models as Functions
```{r}
model <- lm(width ~ length * sex, 
            data = KidsFeet)
Width <- makeFun(model)
Width(length = 25, sex = "B")
Width(length = 25, sex = "G")
```

Once models have been converted into functions, we can easily add them
to our plots using `plotFun()`.

                                                       
```{r, fig.keep='last'}
gf_point(width ~ length, data = KidsFeet, 
        color = ~ sex) %>%
  gf_fun(Width(length, sex = "B") ~ length, color = ~"B") %>%
  gf_fun(Width(length, sex = "G") ~ length, color = ~"G")
```

```{r, include=FALSE}
theme_set(theme_bw())
```

## Resampling -- You can do() it!

If you want to teach using randomization tests and bootstrap intervals,
the `mosaic` package provides some functions to simplify creating the
random distirubtions involved.

### An example: The Lady Tasting Tea

 * Often used on first day of class
 
 * Story
 
   * woman claims she can tell whether milk
   has been poured into tea or vice versa.
   
   * Question: How do we test this claim?
   
```{r, echo = FALSE }
require(mosaic)
trellis.par.set(theme = col.mosaic())
theme_set(theme_bw())
require(knitr)
opts_chunk$set(size = 'small', cache = TRUE)
options(width = 90)
set.seed(12345)
```


We use `rflip()` to simulate flipping coins
```{r}
rflip()
```

**Note:** We do this with students who do not (yet) know what a binomial
distribution is, so we want to avoid using `rbinom()` at this point.

Rather than flip each coin separately, we can flip multiple coins at once.

```{r} 
rflip(10)
```

 * easier to consider `heads` = correct; `tails` = incorrect than to compare with a given pattern
   * this switch bothers me more than it bothers my students

### Now let's do that a lot of times 

`rflip(10)` simulates 1 lady tasting 10 cups 1 time.

We can do that many times to see how guessing ladies do:
```{r}
do(2) * rflip(10)
```

 * `do()` is clever about what it remembers (in many common situations)
 * 2 isn't many -- we'll do many next -- but it is a good idea to take a look at a small example before generating a lot of random data.

Now let's simulate 5000 guessing ladies

```{r, ladies5000}
Ladies <- do(5000) * rflip(10)
head(Ladies, 2)
gf_histogram(~ heads, data = Ladies, binwidth = 1)
```

**Q.** How often does guessing score 9 or 10?

Here are 3 ways to find out

```{r}
tally( ~ (heads >= 9), data = Ladies)
tally( ~ (heads >= 9), data = Ladies, format = "prop")
 prop( ~ (heads >= 9), data = Ladies)
```

### A general approach to randomization

The Lady Tasting Tea illustrates a 3-step process that can be reused in many situations:

 1. Do it for your data
 2. Do it for "random" data
 3. Do it lots of times for "random" data
 
 * definition of "random" is important, but can often be handled by the `mosaic` functions `shuffle()` or `resample()`

### Example: Do mean ages differ by sex?
```{r}
diffmean(age ~ sex, data = HELPrct)
do(1) * 
  diffmean(age ~ shuffle(sex), data = HELPrct)
Null <- do(5000) * 
  diffmean(age ~ shuffle(sex), data = HELPrct)
```

```{r}
prop( ~ (abs(diffmean) > 0.7841), data = Null) 
gf_histogram( ~ diffmean, data = Null) %>%
  gf_vline(xintercept = -0.7841) 
```

### Example:  Bootstrap CI for difference in means

```{r}
Bootstrap <- do(5000) * 
  diffmean(age ~ sex, data = resample(HELPrct))

gf_histogram( ~ diffmean, data = Bootstrap) %>%
  gf_vline(xintercept = -0.7841)
```

```{r}
cdata( ~ diffmean, data = Bootstrap, p = 0.95)
confint(Bootstrap, method = "quantile")
confint(Bootstrap)  # default uses bootstrap st. err.
```

### Randomization and linear models

```{r, size="tiny"}
do(1) * lm(width ~ length, data = KidsFeet)
do(3) * lm(width ~ shuffle(length), data = KidsFeet)
```


```{r, size="tiny"}
do(1) * 
  lm(width ~ length + sex, data = KidsFeet)
do(3) * 
  lm(width ~ length + shuffle(sex), data = KidsFeet)
```


```{r}
Null <- do(5000) * 
  lm(width ~ length + shuffle(sex), 
                       data = KidsFeet)
gf_histogram( ~ sexG, data = Null, boundary = -0.2325) %>%
  gf_vline(xintercept = -0.2325)
```

```{r}
gf_histogram(~ sexG, data = Null, boundary = -0.2325) %>%
  gf_vline(xintercept = -0.2325)
prop(~ (sexG <= -0.2325), data = Null)
```

### Want to learn more?


More mosaic resources can be found at
https://www.mosaic-web.org/mosaic/articles/mosaic-resources.html.

The *RJournal* paper entitled "mosaic Package: Helping Students to `Think with Data' Using R (https://journal.r-project.org/articles/RJ-2017-024/index.html) provides further discussion of the mosaic modeling language and approach to teaching.

---
title: "Untitled"
author: ""
date: ""
output: 
  html_document:
    fig_height: 3
    fig_width: 5
  pdf_document:
    fig_height: 3
    fig_width: 5
  word_document:
    fig_height: 3
    fig_width: 5
---

```{r, setup, include=FALSE}
require(mosaic)   # Load additional packages here 
knitr::opts_chunk$set(
  tidy=FALSE,     # display code as typed
  size="small")   # slightly smaller font for code
```

## Using RMarkdown


### Text

Text can be decorated with **bold** or *italics*.  It is also possible to 

* create [links](https://rmarkdown.rstudio.com/) 
* include mathematics like $e=mc^2$ or 
$$y = \beta_0 + \beta_1 x_1 + \beta_2 x_2$$

Be sure to put a space after the * when you are creating bullets and a space after # when 
creating section headers, but **not** between $ and the mathematical formulas.

### Graphics

If the code of an R chunk produces a plot, this plot can be displayed in the resulting file.
```{r}
gf_point(births ~ date, data = Births78)
```

### R output
Other forms of R output are also displayed as they are produced.
```{r}
favstats(~ births, data = Births78)
```

### Destination formats

This file can be knit to HTML, PDF, or Word.  In RStudio, just select the desired output file type
and click on `Knit HTML`, `Knit PDF`, or `Knit Word`.  Use the dropdown menu next to that to 
change the desired file type.

### Documenting file creation 

It's useful to record some information about how your file was created.

  * File creation date: `r Sys.Date()`
  * `r R.version.string`
  * R version (short form): `r getRversion()`
  * `mosaic` package version: `r packageVersion("mosaic")`
  * Additional session information
  
```{r echo=FALSE}
sessionInfo()  # could use devtools::session_info() if you prefer that
```
  

---
title: Graphics with the mosaic and lattice packages
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Graphics with the mosaic package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r include=FALSE}
library(mosaic)
library(mosaicData)
library(NHANES)
library(gridExtra)
```

```{r include=FALSE}
# Some customization.  You can alter or delete as desired (if you know what you are doing).

# This changes the default colors in lattice plots.
trellis.par.set(theme=theme.mosaic())  

# knitr settings to control how R chunks work.
require(knitr)
set.seed(1)
opts_chunk$set(
  tidy=FALSE,     # display code as typed
  size="small",   # slightly smaller font for code
  fig.width=5,
  fig.height=3
)
```

This vignette is simply a suite of plots that exist primarily as part of our quality
control for the package. But since the examples might be useful to others as well, 
we've added this as a vignette in the package.

**This way of doing this is largely superceded by our `ggformula` package which provides a 
formula interface to `ggplot2`.**  You might also like to see the vignette that compares 
using `lattice` to using `ggformula`.

### lattice extras

The `mosaic` package resets the default panel function for histograms.  This changes
the default for bin selection and provides some additional arguments to histogram.
```{r}
histogram(~ rbinom( 500, 20, .3), width=1, fit="normal", v=c(6,10), h=0.1 )
```

### ladd()

`ladd()` provides a relatively easy way to add additional things to a lattice graphic.

```{r, fig.keep="last"}
xyplot( rnorm(100) ~ rnorm(100) )
ladd( grid.text("Here is some text", x=0, y=0, default.units="native") )
ladd( panel.abline( a=0, b=1, col="red", lwd=3, alpha=.4 ) )
ladd( panel.rect(x=-1, y=-1, width=1, height=1, col="gray80", fill="lightsalmon"))
ladd( panel.rect(x=0, y=0, width=2, height=2, col="gray80", fill="lightskyblue"), 
      under=TRUE)
```

### mplot()

In addition to the interactive uses of `mplot()`, it can be used in place of `plot()` in
several settings.

```{r, fig.height=8}
require(gridExtra)
mod <- lm(width ~ length * sex, data = KidsFeet)
mplot(mod, which = 1:7, multiplot = TRUE, ncol = 2)
```

```{r, fig.height=8}
mplot(mod, which=1:7, system="ggplot", ncol=2)
```

```{r}
mplot(mod, which=7)
mplot(mod, which=7, rows=-1)
mplot(mod, which=7, rows=c("sexG", "length", "length:sexG"), 
      title="Custom titles are supported")
```
```{r}
mod <- lm(age ~ substance, data=HELPrct)
mplot(TukeyHSD(mod))
mplot(TukeyHSD(mod), system="ggplot")
```

### plotFun() and makeFun()


```{r, fig.keep="last"}
mod <- lm(width ~ length* sex, data = KidsFeet)
L <- makeFun(mod)
L( length=15, sex="B")
L( length=15, sex="G")
xyplot(width ~ length, groups = sex, data = KidsFeet, auto.key=TRUE)
plotFun( L(length, sex="B") ~ length, add=TRUE, col=1 )
plotFun( L(length, sex="G") ~ length, add=TRUE, col=2 )
```

For logistic regression, `makeFun()` handles the conversion back to
probabilities by default.

```{r, label="logistic", fig.keep="last", fig.height=5}
mod <- glm( SmokeNow =="Yes" ~ Age + Race3, data=NHANES, family=binomial())
SmokerProb <- makeFun(mod)
xyplot( SmokeNow=="Yes" ~ Age, groups=Race3, data=NHANES, alpha=.01, xlim=c(20,90) )
plotFun(SmokerProb(Age, Race3="Black") ~ Age, col="black", add=TRUE)
plotFun(SmokerProb(Age, Race3="White") ~ Age, col="red", add=TRUE) 
ladd(grid.text("Black", x=25, y=SmokerProb(25, Race="Black"),hjust = 0, vjust=-0.2,
               gp=gpar(col="black"),
               default.units="native"))
ladd(grid.text("White", x=25, y=SmokerProb(25, Race="White"),hjust = 0, vjust=-0.2,
               gp=gpar(col="red"),
               default.units="native"))
```

```{r}
f <- makeFun(sin(x) ~ x)
plotFun( f(x) ~ x, xlim = c( -2 * pi, 2 * pi) )
```

```{r}
plotFun( x * sin(1/x) ~ x, xlim=c(-1,1) )
plotFun( x * sin(1/x) ~ x, xlim=c(-1,1), npts=10000 )
```

### Visualizing distributions

```{r}
plotDist("chisq", df=3)
plotDist("chisq", df=3, kind="cdf")
```

```{r}
xpnorm(80, mean=100, sd=15)
xpnorm(c(80,120), mean=100, sd=15)
```

```{r}
pdist("chisq", 4, df=3)
pdist("f", 3, df1=5, df2=20)
qdist("t", c(.025, .975) , df=5)
```


```{r}
histogram( ~ rbinom(1000, 20, .4), width=1, v=20 * .4 )
SD <- sqrt(20 * .4 * .6)
plotDist("norm", mean=.4*20, sd=SD, add=TRUE, alpha=.7)
```

```{r, fig.keep="last"}
plotDist("norm", col="blue", mean=2, xlim=c(-4,8))
plotDist("norm", mean=5, col="green", kind='histogram', add=TRUE)  # add, overtop
plotDist("norm", mean=0, col="red", kind='histogram', under=TRUE)  # add, but underneath!
```


### Maps

The `mosaic` package now provides facilities for producing choropleth maps.  The API is still under developement and may change in future releases.

```{r}
mUSMap(USArrests %>% mutate(state = row.names(.)), key="state", fill = "UrbanPop") 
```

Looks like it is safer to live in the North:

```{r}
mUSMap(USArrests %>% mutate(state = row.names(.)), key="state", fill = "Murder") 
```

Here is a sillier example

```{r}
Countries %>% mutate(nletters = nchar(gapminder)) %>%
  mWorldMap(key="gapminder", fill="nletters") 
```

---
title: "ggformula/lattice Comparison"
author: "Nicholas Horton (nhorton@amherst.edu)"
date: '`r Sys.Date()`'
output: 
  html_document:
    fig_height: 2.6
    fig_width: 5
  pdf_document:
    fig_height: 2.4
    fig_width: 5
    pandoc_args: ["-V", "classoption=twocolumn"]
  word_document:
    fig_height: 2.6
    fig_width: 5
---

```{r, setup, include = FALSE}
require(mosaic)   # Load additional packages here 
require(ggformula)

# Some customization.  You can alter or delete as desired (if you know what you are doing).
trellis.par.set(theme = theme.mosaic()) # change default color scheme for lattice
theme_set(theme_bw())                   # change default theme for ggformula
knitr::opts_chunk$set(
  tidy = FALSE,     # display code as typed
  size = "small")   # slightly smaller font for code
```

## Introduction

This document is intended to help users of the `mosaic` package migrate their `lattice` package graphics to `ggformula`.  The mosaic package provides a simplified and systematic introduction to the core functionality related to descriptive statistics, visualization, modeling, and simulation-based inference required in first and second courses in statistics.  

\newpage

\vspace*{.10in}
Originally, the `mosaic` package used `lattice` graphics but now support is also available for the improved `ggformula` system.  Going forward, `ggformula` will be the preferred graphics 
package for Project MOSAIC.

\vfill

\newpage

## Histograms 

<div class = "row">
<div class = "col-md-6">

### Histograms (ggformula)

```{r, hist-gg}
library(mosaic)   # also loads ggformula 
gf_histogram(~ age, data = HELPrct)
```

### Histogram options (ggformula)

```{r, hist-opt-gg}
gf_histogram(~ age, data = HELPrct,
             binwidth = 5) 
```


</div>
\newpage
\vspace*{.03in}

<div class = "col-md-6">
### Histograms (lattice)

```{r, hist-lattice}
library(mosaic)     # also loads lattice
histogram(~ age, data = HELPrct)
```

### Histogram options (lattice)

```{r, hist-opt-lattice}
histogram(~ age, width = 5, data = HELPrct)
```


</div>
</div>

\newpage

## Density Plots

<div class = "row">
<div class = "col-md-6">

### Density plots (ggformula)

```{r, dens-gg, message = FALSE}
gf_dens(~ age, data = HELPrct)
```

### Overlaid density plots (ggformula)

```{r, dens2-gg, message = FALSE}
gf_dens(~ age, data = HELPrct,
        color = ~ sex)
```

### Density over histograms (ggformula)

We can use stacked layers to add a density curve based on a 
maximum likelihood fit or a kernel density estimate (see also `gf_dist()`)

```{r, dens-hist-gg, message = FALSE}
gf_dhistogram( ~ age, data = HELPrct, 
               alpha = 0.5) %>%
  gf_fitdistr(color = ~"MLE", dist = "dnorm") %>% 
  gf_dens(color = ~"KDE")   
```


</div>
\newpage
\vspace*{.03in}

<div class = "col-md-6">
### Density plots (lattice)

```{r, dens-lattice, message = FALSE}
densityplot(~ age, data = HELPrct)
```

### Overlaid density plots (lattice)

```{r, dens2-lattice}
densityplot(~ age, data = HELPrct,
            groups = sex,  auto.key = TRUE)
```
### Density over histograms (lattice)

`mosaic` makes it easy to add a fitted distribution to a histogram.

```{r, dens-hist-lattice, message = FALSE}
histogram(~ age, data = HELPrct, 
          fit = "normal", dcol = "red")
```


</div>
</div>

\newpage



## Side by side boxplots 

<div class = "row">
<div class = "col-md-6">

### Side by side boxplots (ggformula)

```{r, boxplot-gg, message = FALSE}
gf_boxplot(age ~ sex, data = HELPrct)
```

### Faceted side by side boxplots (ggformula)

```{r, boxplot-facets-gg, message = FALSE}
gf_boxplot(age ~ sex | homeless, 
  data = HELPrct)
```

### Horizontal boxplots (ggformula)

```{r, boxploth-gg, message = FALSE}
gf_boxplot(sex ~ age, data = HELPrct)
```


</div>
\newpage
\vspace*{.03in}

<div class = "col-md-6">


### Side by side boxplots (lattice)

```{r, boxplot-lattice, message = FALSE}
bwplot(age ~ sex, data = HELPrct)
```

### Faceted side by side boxplots (lattice)

```{r, boxplot-facet-lattice, message = FALSE}
bwplot(age ~ sex | homeless, 
       data = HELPrct)
```

### Horizontal boxplots (lattice)

```{r, boxploth-lattice, message = FALSE}
bwplot(sex ~ age, data = HELPrct)
```

</div>
</div>

\newpage

## Scatterplots

<div class = "row">
<div class = "col-md-6">

### Basic scatterplot (ggformula)

```{r, scatter-gg, message = FALSE}
gf_point(cesd ~ age, data = HELPrct)
```



### Overlaid scatterplot with linear fit (ggformula)

```{r, scatter2-gg, message = FALSE}
gf_point(cesd ~ age, data = HELPrct,
         color = ~ sex) %>%
  gf_lm()
```

</div>
\newpage
\vspace*{.03in}

<div class = "col-md-6">

### Basic Scatterplot (lattice)

```{r, scatter-lattice, message = FALSE}
xyplot(cesd ~ age, data = HELPrct)
```

### Overlaid scatterplot with linear fit (lattice)

```{r, scatter2-lattice, message = FALSE}
xyplot(cesd ~ age,  data = HELPrct,
       groups = sex, 
       type = c("p", "r"), 
       auto.key = TRUE) 
```


</div>
</div>

\newpage


<div class = "row">
<div class = "col-md-6">

### Faceted scatterplot with smooth fit (ggformula)

```{r, scatter3-gg, message = FALSE}
gf_point(cesd ~ age | sex, 
         data = HELPrct) %>%
  gf_smooth(se = FALSE)
```



## More options for scatterplot with linear fit (ggformula)

```{r, scatter4-gg}
gf_point(cesd ~ age, data = HELPrct,
         color = ~ sex) %>%
  gf_lm() %>% 
  gf_theme(legend.position = "top") %>% 
  gf_labs(
    title = "This is my ggformula plot", 
    x     = "age (in years)", 
    y     = "CES-D measure of
depressive symptoms")
```


</div>
\newpage

<div class = "col-md-6">


## Faceted scatterplot with smooth fit (lattice)

```{r, scatter3-lattice, message = FALSE}
xyplot(cesd ~ age | sex,  data = HELPrct,
       type = c("p", "smooth"), 
       auto.key = TRUE) 
```




## More options for scatterplot with linear fit (lattice)

```{r, scatter4-lattice, message = FALSE}
xyplot(cesd ~ age, groups = sex, 
       type = c("p", "r"), 
       auto.key = TRUE, 
       main = "This is my lattice plot", 
       xlab = "age (in years)", 
       ylab = "CES-D measure of
depressive symptoms",
       data = HELPrct)
```

</div>
</div>

\newpage

## Refining graphs

<div class = "row">
<div class = "col-md-6">

### Log scales (ggformula)

```{r log-scales-gg}
gf_point(cesd ~ age, data = HELPrct) %>%
  gf_refine(scale_y_log10()) 
```

### Custom Colors (ggformula)

```{r refine-gg}
gf_dens(
  ~ cesd, data = HELPrct, 
  color = ~ sex) %>%
  gf_rug(
    0 ~ cesd, 
    position = position_jitter(height = 0)
  ) %>%
  gf_refine(
    scale_color_manual(
      values = c("navy", "red"))) 
```

</div>
\newpage
\vspace*{.03in}

<div class = "col-md-6">

### Log scales (lattice)

```{r, log-scales-lattice}
xyplot(
  cesd ~ age, data = HELPrct,
  scales = list(y = list(log = TRUE)))
```

### Custom Colors (lattice)

```{r, refine-lattice}
densityplot( 
  ~ cesd, data = HELPrct, groups = sex,
  rug = FALSE,
  par.settings = 
    list(
      superpose.line = 
        list(col = c("navy", "red")),
      superpose.symbol = 
        list(col = c("navy", "red"))
    )) 
```
</div>
</div>

\newpage

## Want to explore more?

Within RStudio, after loading the `mosaic` package, try running the command `mplot(ds)` where `ds` is a dataframe.  This will open up an interactive visualizer that will output the code to generate the figure (using `lattice`, `ggplot2`, or `ggformula`) when you click on `Show Expression`. 


\newpage

## References 

More information about `ggformula` can be found at https://www.mosaic-web.org/ggformula.

More information regarding Project MOSAIC (Kaplan, Pruim, and Horton) can be found at http://www.mosaic-web.org/.  Further information regarding the `mosaic` package can be found at https://www.mosaic-web.org/mosaic/ and https://journal.r-project.org/articles/RJ-2017-024/index.html.

Examples of how to bring multidimensional graphics into day one of an introductory statistics course can be found at https://escholarship.org/uc/item/84v3774z. 

\vfill

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Sleep.R
\docType{data}
\name{Sleep}
\alias{Sleep}
\title{Sleep and Memory}
\format{
A data.frame with 24 observations on the following 2 variables.
\describe{
\item{\code{Group }}{treatment group of the subject}
\item{\code{Words }}{number of words recalled}
}
}
\source{
These data were used in a "resampling bake-off" hosted by Robin Lock.
}
\usage{
data(Sleep)
}
\description{
Sleep and Memory
}
\details{
In an experiment on memory (Mednicj et al, 2008), students were given lists
of 24 words to memorize.  After hearing the words they were assigned at
random to different groups. One group of 12 students took a nap for 1.5 hours
while a second group of 12 students stayed awake and was given a caffeine
pill.  The data set records the number of words each participant was
able to recall after the break.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xpnorm.R
\name{xpnorm}
\alias{xpnorm}
\alias{xqnorm}
\alias{xcnorm}
\title{Augmented versions of pnorm and qnorm}
\usage{
xpnorm(
  q,
  mean = 0,
  sd = 1,
  plot = TRUE,
  verbose = TRUE,
  invisible = FALSE,
  digits = 4,
  lower.tail = TRUE,
  log.p = FALSE,
  xlim = mean + c(-4, 4) * sd,
  ylim = c(0, 1.4 * dnorm(mean, mean, sd)),
  manipulate = FALSE,
  ...,
  return = c("value", "plot")
)

xqnorm(
  p,
  mean = 0,
  sd = 1,
  plot = TRUE,
  verbose = TRUE,
  digits = getOption("digits"),
  lower.tail = TRUE,
  log.p = FALSE,
  xlim,
  ylim,
  invisible = FALSE,
  ...,
  return = c("value", "plot"),
  pattern = c("stripes", "rings")
)

xcnorm(
  p,
  mean = 0,
  sd = 1,
  plot = TRUE,
  verbose = TRUE,
  digits = getOption("digits"),
  lower.tail = TRUE,
  log.p = FALSE,
  xlim,
  ylim,
  invisible = FALSE,
  ...,
  return = c("value", "plot"),
  pattern = "rings"
)
}
\arguments{
\item{q}{quantile}

\item{mean, sd}{parameters of normal distribution.}

\item{plot}{logical.  If TRUE, show an illustrative plot.}

\item{verbose}{logical.  If TRUE, display verbose output.}

\item{invisible}{logical.  If TRUE, return value invisibly.}

\item{digits}{number of digits to display in output.}

\item{lower.tail}{logical.  If FALSE, use upper tail probabilities.}

\item{log.p}{logical.  If TRUE, uses the log of probabilities.}

\item{xlim, ylim}{limits for plotting.}

\item{manipulate}{logical.  If TRUE and in RStudio,
then sliders are added for interactivity.}

\item{\dots}{additional arguments.}

\item{return}{If \code{"plot"}, return a plot.  If \code{"values"}, return a vector of numerical values.}

\item{p}{probability}

\item{pattern}{One of \code{"stripes"} or \code{"rings"}.  In the latter case, pairs of regions
(from inside to outside) are grouped together for coloring and probability calculation.}
}
\description{
These functions behave similarly to the functions with the initial \code{x}
removed from their names but add more verbose output and graphics.
}
\examples{
xpnorm(650, 500, 100)
xqnorm(.75, 500, 100)
xpnorm(-3:3, return = "plot", system = "gg") |> 
  gf_labs(title = "My Plot", x = "") |> 
  gf_theme(theme_bw())

\dontrun{
if (rstudio_is_available() & require(manipulate)) {
  manipulate(xpnorm(score, 500, 100, verbose = verbose),
    score = slider(200, 800),
	   verbose = checkbox(TRUE, label = "Verbose Output")
  )
}
}
}
\seealso{
\code{\link[lattice:histogram]{lattice::histogram()}},
\code{\link[=chisq.test]{chisq.test()}},
\code{\link[=pnorm]{pnorm()}},
\code{\link[=qnorm]{qnorm()}},
\code{\link[lattice:qqmath]{lattice::qqmath()}}, and
\code{\link[=plot]{plot()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/value.R
\name{value}
\alias{value}
\alias{value.integrate}
\alias{value.default}
\title{Extract value from an object}
\usage{
value(object, ...)

\method{value}{integrate}(object, ...)

\method{value}{default}(object, ...)
}
\arguments{
\item{object}{an object from which a "value" is to be extracted.}

\item{...}{additional arguments (currently ignored).}
}
\description{
Functions like \code{\link[=integrate]{integrate()}} and \code{\link[=nlm]{nlm()}} return objects that contain more
information that simply the value of the integration or optimization.  \code{value()} extracts
the primary value from such objects.  Currently implemented situations include the output from
\code{\link[=integrate]{integrate()}},
\code{\link[=nlm]{nlm()}},
\code{\link[cubature:adaptIntegrate]{cubature::adaptIntegrate()}}, and
\code{\link[=uniroot]{uniroot()}}.
}
\examples{
integrate(sin, 0, 1) |> value()
nlm(cos, p = 0) |> value()
uniroot(cos, c(0, 2)) |> value()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xpnorm.R
\name{cnorm}
\alias{cnorm}
\alias{ct}
\title{Central Probability in a Normal or T Distribution}
\usage{
cnorm(p, mean = 0, sd = 1, log.p = FALSE, side = c("both", "upper", "lower"))

ct(p, df, ncp, log.p = FALSE, side = c("upper", "lower", "both"))
}
\arguments{
\item{p}{vector of probabilities.}

\item{mean}{vector of means.}

\item{sd}{vector of standard deviations.}

\item{log.p}{logical.  If TRUE, uses the log of probabilities.}

\item{side}{One of "upper", "lower", or "both" indicating
whether a vector of upper or lower quantiles or a matrix of
both should be returned.}

\item{df}{degrees of freedom (\eqn{> 0}, maybe non-integer).  \code{df
      = Inf} is allowed.}

\item{ncp}{non-centrality parameter \eqn{\delta}{delta};
    currently except for \code{rt()}, accurate only for \code{abs(ncp) <= 37.62}.
    If omitted, use the central \eqn{t} distribution.}
}
\description{
These versions of the quantile functions take a vector of
\emph{central} probabilities as its first argument.
}
\examples{
qnorm(.975)
cnorm(.95)
xcnorm(.95)
xcnorm(.95, verbose = FALSE, return = "plot") |>
  gf_refine(
  scale_fill_manual( values = c("navy", "limegreen")),
  scale_color_manual(values = c("black", "black")))
cnorm(.95, mean = 100, sd = 10)
xcnorm(.95, mean = 100, sd = 10)

}
\seealso{
\code{\link[stats:qnorm]{stats::qnorm()}}, \code{\link[=cdist]{cdist()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpqrdist.R
\name{qdist}
\alias{qdist}
\alias{xqgamma}
\alias{xqt}
\alias{xqchisq}
\alias{xqf}
\alias{xqbinom}
\alias{xqpois}
\alias{xqgeom}
\alias{xqnbinom}
\alias{xqbeta}
\title{Illustrated quantile calculations from distributions}
\usage{
qdist(
  dist = "norm",
  p,
  plot = TRUE,
  verbose = FALSE,
  invisible = FALSE,
  resolution = 500L,
  digits = 3L,
  xlim,
  ylim,
  return = c("values", "plot"),
  refinements = list(),
  ...
)

xqgamma(
  p,
  shape,
  rate = 1,
  scale = 1/rate,
  lower.tail = TRUE,
  log.p = FALSE,
  ...
)

xqt(p, df, ncp, lower.tail = TRUE, log.p = FALSE, ...)

xqchisq(p, df, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)

xqf(p, df1, df2, lower.tail = TRUE, log.p = FALSE, ...)

xqbinom(p, size, prob, lower.tail = TRUE, log.p = FALSE, ...)

xqpois(p, lambda, lower.tail = TRUE, log.p = FALSE, ...)

xqgeom(p, prob, lower.tail = TRUE, log.p = FALSE, ...)

xqnbinom(p, size, prob, mu, lower.tail = TRUE, log.p = FALSE, ...)

xqbeta(p, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)
}
\arguments{
\item{dist}{a character description of a distribution, for example
\code{"norm"}, \code{"t"}, or \code{"chisq"}}

\item{p}{a vector of probabilities}

\item{plot}{a logical indicating whether a plot should be created}

\item{verbose}{a logical}

\item{invisible}{a logical}

\item{resolution}{number of points used for detecting discreteness and generating plots.
The default value of 5000 should work well except for discrete distributions
that have many distinct values, especially if these values are not evenly spaced.}

\item{digits}{the number of digits desired}

\item{xlim}{x limits.  By default, these are chosen to show the central 99.8\\%
of the distribution.}

\item{ylim}{y limits}

\item{return}{If \code{"plot"}, return a plot.  If \code{"values"}, return a vector of numerical values.}

\item{refinements}{A list of refinements to the plot.  See \code{\link[ggformula:gf_refine]{ggformula::gf_refine()}}.}

\item{...}{additional arguments, including parameters of the distribution
and additional options for the plot.  To help with name collisions (eg \code{size} for binomial
distributions and \code{shape} for gamma distributions), argument names beginning \code{plot_} will
be renamed to remove \code{plot_} and passed only to the plot.  The unprefixed version will
used as a parameter for the the distribution.}

\item{shape, scale}{shape and scale parameters.  Must be positive,
    \code{scale} strictly.}

\item{rate}{an alternative way to specify the scale.}

\item{lower.tail}{logical; if \code{TRUE} (default), probabilities are
    \eqn{P[X < x]}, otherwise, \eqn{P[X \ge x]} (exceptionally not
    \emph{cadlag} here, to ensure, e.g., \code{pgamma(0, shape=0)} remains zero).}

\item{log.p}{A logical indicating whether probabilities should be
returned on the log scale.}

\item{df}{degrees of freedom (\eqn{> 0}, maybe non-integer).  \code{df
      = Inf} is allowed.}

\item{ncp}{non-centrality parameter \eqn{\delta}{delta};
    currently except for \code{rt()}, accurate only for \code{abs(ncp) <= 37.62}.
    If omitted, use the central \eqn{t} distribution.}

\item{df1, df2}{degrees of freedom.  \code{Inf} is allowed.}

\item{size}{number of trials (zero or more).}

\item{prob}{probability of success on each trial.}

\item{lambda}{vector of (non-negative) means.}

\item{mu}{alternative parametrization via mean: see \sQuote{Details}.}

\item{shape1, shape2}{non-negative parameters of the Beta distribution.}
}
\value{
a vector of quantiles; a plot is printed as a side effect
}
\description{
Illustrated quantile calculations from distributions
}
\details{
The most general function is \code{qdist} which can work with
any distribution for which a q-function exists.  As a convenience, wrappers are
provided for several common distributions.
}
\examples{
qdist("norm", seq(.1, .9, by = 0.10), 
      title = "Deciles of a normal distribution", show.legend = FALSE,
      pattern = "rings")
xqnorm(seq(.2, .8, by = 0.20), mean = 100, sd = 10)
qdist("unif", .5)
xqgamma(.5, shape = 3, scale = 4)
xqgamma(.5, shape = 3, scale = 4, color = "black")
xqbeta(.5, shape1 = .9, shape2 = 1.4, dlwd = 1)
xqchisq(c(.25,.5,.75), df = 3)
xcbinom(c(0.80, 0.90), size = 1000, prob = 0.40)
# displayed as if continuous
xcbinom(c(0.80, 0.90), size = 5000, prob = 0.40)
xpbinom(c(480, 500, 520), size = 1000, prob = 0.48)
xpbinom(c(40, 60), size = 100, prob = 0.5)
xqpois(c(0.25, 0.5, 0.75), lambda = 12)
xcpois(0.50, lambda = 12)
xcpois(0.50, lambda = 12, refinements = list(scale_color_brewer(type = "qual", palette = 5)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareMean.R, R/compareProportion.R,
%   R/deltaMethod.R, R/groupwiseModels.R, R/interval.R, R/mm.R, R/perctable.R,
%   R/xhistogram.R
\name{compareMean}
\alias{compareMean}
\alias{compareProportion}
\alias{deltaMethod}
\alias{gwm}
\alias{r.squared}
\alias{mm}
\alias{perctable}
\alias{proptable}
\alias{xhistogram}
\title{Defunct functions}
\usage{
compareMean(...)

compareProportion(...)

deltaMethod(...)

gwm(...)

r.squared(...)

mm(...)

perctable(...)

proptable(...)

xhistogram(...)
}
\arguments{
\item{...}{arguments, ignored since the function is defunct}
}
\description{
The following functions were once a part of the mosaic pacakge but have
been removed.  In some cases, an alternative is available and is suggested
if you attempt to execute the function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pqrdata.R
\name{qdata}
\alias{qdata}
\alias{cdata}
\alias{pdata}
\alias{rdata}
\alias{ddata}
\title{The Data Distribution}
\usage{
qdata(formula, p = seq(0, 1, 0.25), data = NULL, ...)

cdata(formula, p = 0.95, data = NULL, ...)

pdata(formula, q, data = NULL, ...)

rdata(formula, n, data = NULL, ...)

ddata(formula, q, data = NULL, ...)
}
\arguments{
\item{formula}{a formula or a vector}

\item{p}{a vector of probabilities}

\item{data}{a data frame in which to evaluate \code{formula}}

\item{\dots}{additional arguments passed to \code{quantile} or \code{sample}}

\item{q}{a vector of quantiles}

\item{n}{number of values to sample}
}
\value{
For \code{qdata}, a vector of quantiles

for \code{cdata}, a data frame giving
upper and lower limits and the central proportion requested

For \code{pdata}, a vector of probabilities

For \code{rdata}, a vector of sampled values.

For \code{ddata}, a vector of probabilities (empirical densities)
}
\description{
Density, distribution function, quantile function, and random generation
from data.
}
\examples{
data(penguins, package = "palmerpenguins")
qdata(flipper_length_mm ~ species, 0.5, data = penguins)
qdata( ~ flipper_length_mm, p = 0.5, groups = species, data = penguins)
qdata(penguins$flipper_length_mm, p = 0.5)
qdata( ~ flipper_length_mm, p = 0.5, data = penguins)
qdata( ~ flipper_length_mm, p = 0.5, groups = species, data = penguins)
data(penguins, package = 'palmerpenguins')
cdata(penguins$flipper_length_mm, 0.5)
cdata( ~ flipper_length_mm, 0.5, data = penguins)
cdata( ~ flipper_length_mm, 0.5, data = penguins)
cdata( ~ flipper_length_mm | species, data = penguins, p = .5)
data(penguins, package = 'palmerpenguins')
pdata(penguins$flipper_length_mm, 3:6)
pdata( ~ flipper_length_mm, 3:6, data = penguins)
data(penguins, package = 'palmerpenguins')
rdata(penguins$species, 10)
rdata( ~ species, n = 10, data = penguins)
rdata(flipper_length_mm ~ species,  n = 5, data = penguins)
data(penguins, package = 'palmerpenguins')
ddata(penguins$species, 'setosa')
ddata( ~ species, 'setosa', data = penguins)
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mPlots.R
\name{mPlot}
\alias{mPlot}
\alias{mMap}
\alias{mScatter}
\alias{mUniplot}
\title{Interactive plotting}
\usage{
mPlot(
  data,
  format,
  default = format,
  system = system_choices()[1],
  show = FALSE,
  title = "",
  data_text = rlang::expr_deparse(substitute(data)),
  ...
)

mMap(
  data,
  default = "map",
  system = "ggplot2",
  show = FALSE,
  title = title,
  data_text = rlang::expr_deparse(substitute(data)),
  ...
)

mScatter(
  data,
  default = c("scatter", "jitter", "boxplot", "violin", "line", "sina",
    "density (contours)", "density (filled)"),
  system = "ggformula",
  show = FALSE,
  title = "",
  data_text = rlang::expr_deparse(substitute(data))
)

mUniplot(
  data,
  default = c("histogram", "density", "frequency polygon", "ASH plot"),
  system = system_choices()[1],
  show = FALSE,
  title = "",
  data_text = rlang::expr_deparse(substitute(data))
)
}
\arguments{
\item{data}{a data frame containing the variables that might be used in the plot.
Note that for maps, the data frame must contain coordinates of the polygons
comprising the map and a variable for determining which coordinates are part
of the same region.  See \code{\link[=sp2df]{sp2df()}} for one way to create such
a data frame.  Typically \code{\link[=merge]{merge()}} will be used to combine the map
data with some auxiliary data to be displayed as fill color on the map, although
this is not necessary if all one wants is a map.}

\item{format}{a synonym for \code{default}.}

\item{default}{default type of plot to create; one of
\code{"scatter"},
\code{"jitter"},
\code{"boxplot"},
\code{"violin"},
\code{"sina"},
\code{"histogram"},
\code{"density"},
\code{"density (contours)"},
\code{"density (filled)"},
\code{"frequency polygon"},
\code{"xyplot"},
or
\code{"map"}.  Unique prefixes suffice.}

\item{system}{which graphics system to use (initially) for plotting (\pkg{ggplot2}
or \pkg{lattice}).  A check box will allow on the fly change of plotting system.}

\item{show}{a logical, if \code{TRUE}, the code will be displayed each time the plot is
changed.}

\item{title}{a title for the plot}

\item{data_text}{A text string describing the data.  It must be possible to recover the data
from this string using \code{\link[=eval]{eval()}}.  Typically users will not need to
modify this from the default value.}

\item{\dots}{additional arguments}
}
\value{
Nothing.  Just for side effects.
}
\description{
These functions provide a menu selection system (via \pkg{manipulate}) so that
different aspects of a plot can be selected interactively.
The \pkg{ggplot2} or \pkg{lattice}
command for generating the plot currently being displayed can be copied to the
console, whence it can be copied to a document for later direct, non-interactive
use.
}
\details{
Only \code{mPlot} is required by end users.  The other plotting functions
are dispatched based on the value of \code{default}.  Furthermore, \code{\link[=mplot]{mplot()}}
will dispatch \code{mPlot} when provided a data frame.

Currently maps are only supported in \pkg{ggplot2} and not in \pkg{lattice}.

Due to an unresolved issue with RStudio, the first time this function is called, and additional
plot is created to correctily initialize the mainipulate frameowrk.
}
\section{Note}{


Due to an unresolved issue with RStudio, the first time this function is called, and additional
plot is created to correctily initialize the mainipulate frameowrk.
}

\examples{
\dontrun{
mPlot(HELPrct, format = "scatter")
mPlot(HELPrct, format = "density")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confint.R, R/mplot.R
\name{confint}
\alias{confint}
\alias{confint.numeric}
\alias{confint.do.tbl_df}
\alias{confint.do.data.frame}
\alias{confint.data.frame}
\alias{confint.summary.lm}
\title{Confidence interval methods for output of resampling}
\usage{
\method{confint}{numeric}(
  object,
  parm,
  level = 0.95,
  ...,
  method = "percentile",
  margin.of.error = "stderr" \%in\% method == "stderr"
)

\method{confint}{do.tbl_df}(
  object,
  parm,
  level = 0.95,
  ...,
  method = "percentile",
  margin.of.error = "stderr" \%in\% method,
  df = NULL
)

\method{confint}{do.data.frame}(
  object,
  parm,
  level = 0.95,
  ...,
  method = "percentile",
  margin.of.error = "stderr" \%in\% method,
  df = NULL
)

\method{confint}{data.frame}(object, parm, level = 0.95, ...)

\method{confint}{summary.lm}(object, parm, level = 0.95, ...)
}
\arguments{
\item{object}{and R object}

\item{parm}{a vector of parameters}

\item{level}{a confidence level}

\item{...}{additional arguments}

\item{method}{a character vector of methods to use for creating confidence
intervals.  Choices are "percentile" (or "quantile") which is the default,
"stderr" (or "se"), "bootstrap-t", and
"reverse" (or "basic"))}

\item{margin.of.error}{if true, report intervals as a center and margin of error.}

\item{df}{degrees for freedom. This is required when \code{object} was produced using
\code{link{do}} when
using the standard error to compute the confidence interval since
typically this information is not recorded in these objects.  The default (\code{Inf})
uses a normal critical value rather than a one derived from a t-distribution.}
}
\value{
When applied to a data frame, returns a data frame giving the
confidence interval for each variable in the data frame using
\code{t.test} or \code{binom.test}, unless the data frame was produced using \code{do}, in which case
it is assumed that each variable contains resampled statistics that serve as an estimated sampling
distribution from which a confidence interval can be computed using either a central proportion
of this distribution or using the standard error as estimated by the standard deviation of the
estimated sampling distribution.  For the standard error method, the user must supply the correct
degrees of freedom for the t distribution since this information is typically not available in
the output of \code{\link[=do]{do()}}.

When applied to a numerical vector, returns a vector.
}
\description{
Methods for \code{confint} to compute confidence intervals
on numerical vectors and numerical components of data frames.
}
\details{
The methods of producing confidence intervals from bootstrap distributions are currently
quite naive.  In particular, when using the standard error, assistance may be required with the
degrees of freedom, and it may not be possible to provide a correct value in all situations.
None of the methods include explicit bias correction.
Let \eqn{q_a} be the \eqn{a} quantile of the bootstrap distribution,
let \eqn{t_a, df} be the \eqn{a} quantile of the t distribution with \eqn{df}
degrees of freedom,
let \eqn{SE_b} be the standard deviation of the bootstrap distribution,
and let \eqn{\hat{\theta}} be the estimate computed from the original data.
Then the confidence intervals with confidence level \eqn{1 - 2a} are
\describe{
\item{quantile}{\eqn{(q_a, q_{1-a}) } }
\item{reverse}{ \eqn{( 2 \hat{\theta} - q_{1-a}, 2\hat{\theta} - q_{a} )}}
\item{stderr}{\eqn{(\hat{\theta} - t_{1-a,df} SE_b, \hat{\theta} + t_{1-a,df} SE_b) }.
When \code{df} is not provided,
at attempt is made to determine an appropriate value, but this should be double checked.
In particular, missing data an lead to unreliable results.
}
The bootstrap-t confidence interval is computed much like the reverse confidence interval
but the bootstrap t distribution is used in place of a theoretical t distribution.
This interval has much better properties than the reverse (or basic) method, which
is here for comparison purposes only and is not recommended.  The t-statistic
is computed from a mean, a standard deviation, a sample size which much be named
\code{"mean"}, \code{"sd"}, and \code{"n"} as they are when using \code{\link[=favstats]{favstats()}}.
}
}
\examples{
if (require(mosaicData)) {
  bootstrap <- do(500) * diffmean( age ~ sex, data = resample(HELPrct) )
  confint(bootstrap)
  confint(bootstrap, method = "percentile")
  confint(bootstrap, method = "boot")
  confint(bootstrap, method = "se", df = nrow(HELPrct) - 1)
  confint(bootstrap, margin.of.error = FALSE)
  confint(bootstrap, margin.of.error = TRUE, level = 0.99,
    method = c("se", "perc") )

  # bootstrap t method requires both mean and sd
  bootstrap2 <- do(500) * favstats(resample(1:10))
  confint(bootstrap2, method = "boot")
}
lm(width ~ length * sex, data = KidsFeet) |>
  summary() |>
  confint()
}
\references{
Tim C. Hesterberg (2015): What Teachers Should Know about the Bootstrap:
Resampling in the Undergraduate Statistics Curriculum,
The American Statistician,
\url{https://www.tandfonline.com/doi/full/10.1080/00031305.2015.1089789}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oddsRatio.R
\name{orrr}
\alias{orrr}
\alias{oddsRatio}
\alias{relrisk}
\alias{print.oddsRatio}
\alias{print.relrisk}
\alias{summary.oddsRatio}
\alias{summary.relrisk}
\title{Odds Ratio and Relative Risk for 2 x 2 Contingency Tables}
\usage{
orrr(
  x,
  conf.level = 0.95,
  verbose = !quiet,
  quiet = TRUE,
  digits = 3,
  relrisk = FALSE
)

oddsRatio(x, conf.level = 0.95, verbose = !quiet, quiet = TRUE, digits = 3)

relrisk(x, conf.level = 0.95, verbose = !quiet, quiet = TRUE, digits = 3)

\method{print}{oddsRatio}(x, digits = 4, ...)

\method{print}{relrisk}(x, digits = 4, ...)

\method{summary}{oddsRatio}(object, digits = 4, ...)

\method{summary}{relrisk}(object, digits = 4, ...)
}
\arguments{
\item{x}{a 2 x 2 matrix, data frame, or table of counts}

\item{conf.level}{the confidence interval level}

\item{verbose}{a logical indicating whether verbose output should be displayed}

\item{quiet}{a logical indicating whether verbose output should be suppressed}

\item{digits}{number of digits to display}

\item{relrisk}{a logical indicating whether the relative risk should be returned
instead of the odds ratio}

\item{...}{additional arguments}

\item{object}{an R object to print or summarise.  Here an object of class
\code{"oddsRatio"} or \code{"relrisk"}.}
}
\value{
an odds ratio or relative risk.  If \code{verpose} is true,
more details and the confidence intervals are displayed.
}
\description{
This function calculates the odds ratio and relative risk for a 2 x 2
contingency table and a
confidence interval (default \code{conf.level} is 95 percent) for the
each estimate. \code{x} should be a matrix, data frame or table. "Successes"
should be located in column 1 of \code{x}, and the treatment of interest
should be located in row 2. The odds ratio is calculated as (Odds row 2) /
(Odds row 1). The confidence interval is calculated from the log(OR) and
backtransformed.
}
\examples{
M1 <- matrix(c(14, 38, 51, 11), nrow = 2)
M1
oddsRatio(M1)

M2 <- matrix(c(18515, 18496, 1427, 1438), nrow = 2)
rownames(M2) <- c("Placebo", "Aspirin")
colnames(M2) <- c("No", "Yes")
M2
oddsRatio(M2)
oddsRatio(M2, verbose = TRUE)
relrisk(M2, verbose = TRUE)
if (require(mosaicData)) {
relrisk(tally(~ homeless + sex, data = HELPrct) )
do(3) * relrisk( tally( ~ homeless + shuffle(sex), data = HELPrct) )
}
}
\seealso{
\code{\link[=chisq.test]{chisq.test()}}, \code{\link[=fisher.test]{fisher.test()}}
}
\author{
Kevin Middleton (\email{kmm@csusb.edu}); modified by
R Pruim.
}
\keyword{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{sp2df}
\alias{sp2df}
\title{Transforms a shapefile into a dataframe}
\usage{
sp2df(map, ...)
}
\arguments{
\item{map}{A map object of class \code{SpatialPolygonsDataFrame}}

\item{...}{Other arguments, currently ignored}
}
\value{
A dataframe, in which the first 7 columns hold geographical
information (ex: \code{long} and \code{lat})
}
\description{
This function takes in a shapefile (formal class of
\code{SpatialPolygonsDataFrame}) and transforms it into a dataframe
}
\examples{

\dontrun{
if(require(maptools)) {
  data(wrld_simpl)
  worldmap <- sp2df(wrld_simpl)
}

if ( require(ggplot2) && require(maptools) ) {
  data(wrld_simpl)
  World <- sp2df(wrld_simpl)
  World2 <- merge(World, Countries, by.x="NAME", by.y="maptools", all.y=FALSE)
  Mdata <- merge(Alcohol, World2, by.x="country", by.y="gapminder", all.y=FALSE)
  Mdata <- Mdata[order(Mdata$order),]
  qplot( x=long, y=lat, fill=ntiles(alcohol,5),
         data=subset(Mdata, year==2008), group = group,
                     geom="polygon")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rescale.R
\name{rescale}
\alias{rescale}
\alias{rescale.data.frame}
\alias{rescale.factor}
\alias{rescale.numeric}
\alias{rescale.default}
\alias{rescale.character}
\title{Rescale}
\usage{
rescale(x, range, domain = NULL, ...)

\method{rescale}{data.frame}(x, range = c(0, 1), domain = NULL, ...)

\method{rescale}{factor}(x, range, domain = NULL, ...)

\method{rescale}{numeric}(x, range = c(0, 1), domain = NULL, ...)

\method{rescale}{default}(x, range = c(0, 1), domain = NULL, ...)

\method{rescale}{character}(x, range = c(0, 1), domain = NULL, ...)
}
\arguments{
\item{x}{an R object to rescale}

\item{range}{a numeric vector of length 2}

\item{domain}{a numeric vector of length 2 or \code{NULL}}

\item{...}{additional arguments}
}
\description{
Rescale vectors or variables within data frames.  This can be useful
for comparing vectors that are on different scales, for example in
parallel plots or heatmaps.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linear-algebra.R
\name{linear.algebra}
\alias{linear.algebra}
\alias{mat}
\alias{singvals}
\title{Functions for teaching linear algebra.}
\usage{
mat(formula, data = parent.frame(), A = formula)

singvals(formula, data = parent.frame(), A = formula)
}
\arguments{
\item{formula}{a formula.  In \code{mat} and \code{singvals},
only the right-hand side is used.}

\item{data}{a data frame from which to pull out numerical values
for the variables in the formula}

\item{A}{an alias for \code{formula} for backward compatibility.

\code{mat} returns a model matrix

To demonstrate singularity, use \code{singvals}.}
}
\value{
\code{mat} returns a matrix

\code{singvals} gives singular values for each column in the model matrix
}
\description{
These functions provide a formula based interface to the construction
of matrices from data and for fitting.  You can use them both for numerical vectors
and for functions of variables in data frames.
These functions are intended to support teaching basic linear algebra
with a particular connection to statistics.
}
\examples{
a <- c(1,0,0); b <- c(1,2,3); c <- c(4,5,6); x <- rnorm(3)
# Formula interface
mat(~a+b)
mat(~a+b+1)
if (require(mosaicData)) {
mat(~length+sex, data=KidsFeet)
singvals(~length*sex*width, data=KidsFeet)
}
}
\seealso{
\code{\link[=project]{project()}}

\code{\link[=linearModel]{linearModel()}}, which returns a function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeo.R
\name{googleMap}
\alias{googleMap}
\title{Display a point on earth on a Google Map}
\usage{
googleMap(
  latitude,
  longitude,
  position = NULL,
  zoom = 12,
  maptype = c("roadmap", "satellite", "terrain", "hybrid"),
  mark = FALSE,
  radius = 0,
  browse = TRUE,
  ...
)
}
\arguments{
\item{latitude, longitude}{vectors of latitude and longitude values}

\item{position}{a data frame containing latitude and longitude positions}

\item{zoom}{zoom level for initial map (1-20)}

\item{maptype}{one of \code{'roadmap'}, \code{'satellite'}, \code{'terrain'}, and \code{'hybrid'}}

\item{mark}{a logical indicating whether the location should be marked with a pin}

\item{radius}{a vector of radii of circles centered at position that are displayed on the map}

\item{browse}{a logical indicating whether the URL should be browsed (else only returned as a string)}

\item{\dots}{additional arguments passed to \code{browseURL}}
}
\value{
a string containing a URL.  Optionally, as a side-effect, the URL is visited in a browser
}
\description{
Creates a URL for Google Maps for a particular latitude and
longitude position.  This function has been deprecated due to changes in
Google's access policies.  Give \code{\link[=leaflet_map]{leaflet_map()}} a try as an alternative.
}
\examples{
\dontrun{
googleMap(40.7566, -73.9863, radius=1)   # Times Square
googleMap(position=rgeo(2), radius=1)    # 2 random locations
}
}
\seealso{
\code{\link[=leaflet_map]{leaflet_map()}}, \code{\link[=deg2rad]{deg2rad()}}, \code{\link[=latlon2xyz]{latlon2xyz()}} and \code{\link[=rgeo]{rgeo()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingMeta.R
\name{MAD}
\alias{MAD}
\alias{SAD}
\title{All pairs mean and sum of absolute differences}
\usage{
MAD(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

SAD(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))
}
\arguments{
\item{x}{a numeric vector or a formula.}

\item{...}{additional arguments passed through to \code{MAD_}
or \code{SAD_}.  If \code{x} is a formula, \code{...} should
include an argument named \code{data} if the intent is to
interpret the formula in a data frame.}

\item{data}{a data frame in which to evaluate formulas (or bare names).
Note that the default is \code{data = parent.frame()}.  This makes it convenient to
use this function interactively by treating the working environment as if it were
a data frame.  But this may not be appropriate for programming uses.
When programming, it is best to use an explicit \code{data} argument
-- ideally supplying a data frame that contains the variables mentioned.}

\item{groups}{a grouping variable, typically a name of a variable in \code{data}}

\item{na.rm}{a logical indicating whether NAs should be removed before
calculating.}
}
\value{
the mean or sum of the absolute differences between each pair
of values in \code{c(x,...)}.
}
\description{
The functions compute the sum or mean of all pairwise absolute
differences.  This differs from \code{\link[stats:mad]{stats::mad()}}, which computes
the median absolute difference of each value from the median of
all the values.  See the \code{ISIwithR} package (and the textbook it
accompanies) for examples using these functions in the context of
simulation-based inference.
}
\examples{
SAD(1:3)
MAD(1:3)
MAD(~eruptions, data = faithful)
}
\seealso{
\code{\link[=mad]{mad()}}, \code{\link[=MAD_]{MAD_()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{CIAdata}
\alias{CIAdata}
\title{Return a dataset based on the CIA World Factbook}
\usage{
CIAdata(name = NULL)
}
\arguments{
\item{name}{An optional parameter specifying the name of the desired dataset.
If multiple names are given, a merge will be attempted on the individual data
sets.}
}
\description{
This function can be used in two different ways. Without an argument, it returns a reference
table that includes information about all the CIA World Factbook tables that are available
through this function. Note the  \code{Name} column that indicates a unique name for each
available dataset. If this name is passed as an argument to the function, the function
will return the corresponding dataset.
}
\examples{
\dontrun{
head(CIAdata())
Population <- CIAdata("pop")
nrow(Population)
head(Population)

PopArea <- 
  CIAdata(c("pop","area")) |> 
  mutate(density = pop / area)
nrow(PopArea)
head(PopArea)
PopArea |> 
  filter(!is.na(density)) |>
  arrange(density) |> 
  tail()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/do.R
\docType{class}
\name{repeater-class}
\alias{repeater-class}
\title{Repeater objects}
\description{
Repeater objects can be used with the \code{*} operator to repeat
things multiple time using a different syntax and different output
format from that used by, for example, \code{\link[=replicate]{replicate()}}.
}
\section{Slots}{

\describe{
\item{\code{n}:}{Object of class \code{"numeric"} indicating how many times to repeat something.}
\item{\code{cull}:}{Object of class \code{"function"} that culls the output from each repetition.}
\item{\code{mode}:}{Object of class \code{"character"} indicating the output mode
('default', 'data.frame', 'matrix', 'vector', or 'list').  For most purposes 'default' (the default)
should suffice.}
\item{\code{algorithm}:}{an algorithm number.}
\item{\code{parallel}:}{a logical indicating whether to attempt parallel execution.}
}
}

\seealso{
\code{\link[=do]{do()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ttest.R
\name{t_test}
\alias{t_test}
\alias{t.test}
\alias{t_test.formula}
\alias{t_test.default}
\title{Student's t-Test}
\usage{
t_test(x, ...)

t.test(x, ...)

\method{t_test}{formula}(formula, data, ..., groups = NULL)

\method{t_test}{default}(
  x,
  y = NULL,
  alternative = c("two.sided", "less", "greater"),
  mu = 0,
  paired = FALSE,
  var.equal = FALSE,
  conf.level = 0.95,
  ...
)
}
\arguments{
\item{x}{a (non-empty) numeric vector of data values.}

\item{...}{further arguments to be passed to or from methods.
    For the \code{formula} method, this includes arguments of the
    default method, but not \code{paired}.}

\item{formula}{a formula of the form \code{lhs ~ rhs} where \code{lhs}
    is a numeric variable giving the data values and \code{rhs} either 
    \code{1} for a one-sample or paired test or a factor
    with two levels giving the corresponding groups. If \code{lhs} is of 
    class \code{"\link[stats]{Pair}"} and \code{rhs} is \code{1}, a paired test
    is done, see Examples.}

\item{data}{an optional matrix or data frame (or similar: see
    \code{\link[stats]{model.frame}}) containing the variables in the
    formula \code{formula}.  By default the variables are taken from
    \code{environment(formula)}.}

\item{groups}{When \code{x} is a formula, \code{groups} can be used to compare groups:
\verb{x = ~ var, groups = g} is equivalent to \code{x = var ~ g}.
See the examples.}

\item{y}{an optional (non-empty) numeric vector of data values.}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.}

\item{mu}{a number indicating the true value of the mean (or
    difference in means if you are performing a two sample test).}

\item{paired}{a logical indicating whether you want a paired
    t-test.}

\item{var.equal}{a logical variable indicating whether to treat the
    two variances as being equal. If \code{TRUE} then the pooled
    variance is used to estimate the variance otherwise the Welch
    (or Satterthwaite) approximation to the degrees of freedom is used.}

\item{conf.level}{confidence level of the interval.}
}
\value{
an object of class \code{htest}
}
\description{
Performs one and two sample t-tests.
The mosaic \code{t.test} provides wrapper functions around the function
of the same name in \pkg{stats}.
These wrappers provide an extended interface that allows for a more systematic
use of the formula interface.
}
\details{
This is a wrapper around \code{\link[stats:t.test]{stats::t.test()}} from the \pkg{stats} package
to extend the functionality of the formula interface.  In particular, one can
now use the formula interface for a 1-sample t-test.  Before, the formula interface
was only permitted for a 2-sample test.  The type of formula that can be used
for the 2-sample test has also be broadened.  See the examples.
}
\examples{
  t.test(HELPrct$age)
  # We can now do this with a formula
  t.test(~ age, data = HELPrct)
  # data = can be omitted, but it is better to use it
  t.test(~ age, HELPrct)
  # the original 2-sample formula
  t.test(age ~ sex, data = HELPrct)
  # alternative 2-sample formulas
  t.test(~ age | sex, data = HELPrct)
  t.test(~ age, groups = sex, data = HELPrct)
  # 2-sample t from vectors
  with(HELPrct, t.test(age[sex == "male"], age[sex == "female"]))
  # just the means
  mean(age ~ sex, data = HELPrct)
}
\seealso{
\code{\link[=prop.test]{prop.test()}}, \code{\link[=binom.test]{binom.test()}},
\code{\link[stats:t.test]{stats::t.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFun.R
\name{inferArgs}
\alias{inferArgs}
\title{Infer arguments}
\usage{
inferArgs(
  vars,
  dots,
  defaults = alist(xlim = , ylim = , zlim = ),
  variants = c(".lim", "lim")
)
}
\arguments{
\item{vars}{a vector of variable names to look for}

\item{dots}{a named list of argument values}

\item{defaults}{named list or alist of default values for limits}

\item{variants}{a vector of optional postfixes for limit-specifying variable names}
}
\value{
a named list or alist of limits.  The names are determined by the names in \code{defaults}.

If multiple \code{variants} are matched, the first is used.
}
\description{
The primary purpose is for inferring argument settings from names derived from variables
occurring in a formula.  For example, the default use is to infer limits for variables
without having to call them \code{xlim} and \code{ylim} when the variables in the formula
have other names.  Other uses could easily be devised by specifying different \code{variants}.
}
\examples{
inferArgs(c('x','u','t'), list(t=c(1,3), x.lim=c(1,10), u=c(1,3), u.lim=c(2,4)))
inferArgs(c('x','u'), list(u=c(1,3)), defaults=list(xlim=c(0,1), ylim=NULL)) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingAux.R
\name{maggregate}
\alias{maggregate}
\title{Aggregate for mosaic}
\usage{
maggregate(
  formula,
  data = parent.frame(),
  FUN,
  groups = NULL,
  subset,
  drop = FALSE,
  ...,
  .format = c("default", "table", "flat"),
  .overall = mosaic.par.get("aggregate.overall"),
  .multiple = FALSE,
  .name = deparse(substitute(FUN)),
  .envir = parent.frame()
)
}
\arguments{
\item{formula}{a formula.  Left side provides variable to be summarized.  Right side and condition
describe subsets.  If the left side is empty, right side and condition are
shifted over as a convenience.}

\item{data}{a data frame.
Note that the default is \code{data = parent.frame()}.  This makes it convenient to
use this function interactively by treating the working environment as if it were
a data frame.  But this may not be appropriate for programming uses.
When programming, it is best to use an explicit \code{data} argument
-- ideally supplying a data frame that contains the variables mentioned
in \code{formula}.}

\item{FUN}{a function to apply to each subset}

\item{groups}{grouping variable that will be folded into the formula (if there is room for it).
This offers some additional flexibility in how formulas can be specified.}

\item{subset}{a logical indicating a subset of \code{data} to be processed.}

\item{drop}{a logical indicating whether unused levels should be dropped.}

\item{\dots}{additional arguments passed to \code{FUN}}

\item{.format}{format used for aggregation. \code{"default"} and \code{"flat"} are equivalent.}

\item{.overall}{currently unused}

\item{.multiple}{a logical indicating whether FUN returns multiple values
Ignored if \code{.multiple} is not \code{NULL}.}

\item{.name}{a name used for the resulting object}

\item{.envir}{an environment in which to evaluate expressions}
}
\value{
a vector
}
\description{
Compute function on subsets of a variable in a data frame.
}
\examples{
if (require(mosaicData)) {
maggregate( cesd ~ sex, HELPrct, FUN = mean )
# using groups instead
maggregate( ~ cesd, groups = sex, HELPrct, FUN = sd )
# the next four all do the same thing
maggregate( cesd ~ sex + homeless, HELPrct, FUN = mean )
maggregate( cesd ~ sex | homeless, HELPrct, FUN = sd )
maggregate( ~ cesd | sex , groups= homeless, HELPrct, FUN = sd )
maggregate( cesd ~ sex, groups = homeless, HELPrct, FUN = sd )
# this is unusual, but also works.
maggregate( cesd ~ NULL , groups = sex, HELPrct, FUN = sd )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpqrdist.R
\name{dpqrdist}
\alias{dpqrdist}
\title{Distribution wrapper}
\usage{
dpqrdist(dist, type = c("d", "p", "q", "r"), ...)
}
\arguments{
\item{dist}{a character description of a distribution, for example
\code{"norm"}, \code{"t"}, or \code{"chisq"}}

\item{type}{one of \code{"x"}, \code{"p"}, \code{"q"}, or \code{"r"}}

\item{...}{additional arguments passed on to underlying distribution function.
Note that one of \code{d}, \code{p}, \code{q}, or \code{n} must
be a named argument in ...}
}
\description{
Utility function wrapping up the d/p/q/r distribution functions
}
\examples{
# 3 random draws from N(1,2)
dpqrdist("norm", "r", n = 3, mean = 1, sd = 2)
# These should all be the same
dpqrdist("norm", "d", x = 0) == dnorm(x = 0)
dpqrdist("norm", "p", q = 0, mean = 1, sd = 2) == pnorm(q = 0, mean = 1, sd = 2)
dpqrdist("norm", "q", p = 0.5, mean = 1, sd = 2) == qnorm(p = 0.5, mean = 1, sd = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval.R
\name{confint.htest}
\alias{confint.htest}
\alias{pval}
\alias{pval.htest}
\alias{stat}
\alias{stat.htest}
\alias{stat.uneval}
\title{Extract summary statistics}
\usage{
\method{confint}{htest}(object, parm, level, ...)

pval(x, ...)

\method{pval}{htest}(x, digits = 4, verbose = FALSE, ...)

stat(x, ...)

\method{stat}{htest}(x, ...)

\method{stat}{uneval}(x, ...)
}
\arguments{
\item{object}{a fitted model object or an htest object.}

\item{parm}{a specification of which parameters are to be given confidence intervals,
either a vector of numbers or a vector of names. If missing, all parameters are considered.}

\item{level}{the confidence level required.}

\item{\dots}{Additional arguments.}

\item{x}{An object of class \code{htest}.}

\item{digits}{number of digits to display in verbose output}

\item{verbose}{a logical}
}
\value{
the extracted p-value, confidence interval, or test statistic
}
\description{
Extract confidence intervals, test statistics or p-values from an
\code{htest} object.
}
\examples{
confint(t.test(rnorm(100)))
pval(t.test(rnorm(100)))
stat(t.test(rnorm(100)))
confint(var.test(rnorm(10,sd=1), rnorm(20, sd=2)))
pval(var.test(rnorm(10,sd=1), rnorm(20, sd=2)))
if (require(mosaicData)) {
data(HELPrct)
stat(t.test (age ~ shuffle(sex), data=HELPrct))
# Compare to test statistic computed with permuted values of sex.
do(10) * stat(t.test (age ~ shuffle(sex), data=HELPrct))
}
}
\keyword{inference}
\keyword{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{standardName}
\alias{standardName}
\alias{standardCountry}
\alias{standardState}
\title{Standardization of Geographic Names}
\usage{
standardName(
  x,
  standard,
  ignore.case = TRUE,
  returnAlternatives = FALSE,
  quiet = FALSE
)

standardCountry(
  x,
  ignore.case = TRUE,
  returnAlternatives = FALSE,
  quiet = FALSE
)

standardState(x, ignore.case = TRUE, returnAlternatives = FALSE, quiet = FALSE)
}
\arguments{
\item{x}{A vector with the region names to standardize}

\item{standard}{a named vector providing the map from
non-standard names (names of vector) to standard names (values of vector)}

\item{ignore.case}{a logical indicating whether case should be ignored
when matching.}

\item{returnAlternatives}{a logical indicating whether all alternatives should
be returned in addition to the standard name.}

\item{quiet}{a logical indicating whether warnings should be suppressed}
}
\description{
Often different sources of geographical data will use different names for
the same region.  These utilities make it easier to merge data from different
sources by converting names to standardized forms.
}
\details{
\describe{

\item{standardName}{This is the most general standardizing function.
In addition to \code{x}, this function requires another argument:
\code{standard} - a named vector in which each name is a particular
spelling of the region name in question and the corresponding value
is the standardized version of that region name}

\item{standardCountry}{This function will standardize the country
names in \code{x} to the standard ISO_a3 country code format. If
\code{returnAlternatives} is set to \code{TRUE}, this function will also
return the the named vector used to standardize the country names}

\item{standardState}{This function will standardize the US state
names in \code{x} to the standard two-letter abbreviations. If
\code{returnAlternatives} is set to \code{TRUE}, this function will also
return the the named vector used to standardize the state names}
}
In all three cases, any names not found in \code{standard}
will be left unaltered.  Unless supressed, a warning message will
indicate the number of such cases, if there are any.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFun.R
\name{makeColorscheme}
\alias{makeColorscheme}
\title{Create a color generating function from a vector of colors}
\usage{
makeColorscheme(col)
}
\arguments{
\item{col}{a vector of colors}
}
\value{
a function that generates a vector of colors interpolated among the colors in \code{col}
}
\description{
Create a color generating function from a vector of colors
}
\examples{
cs <- makeColorscheme( c('red','white','blue') )
cs(10)
cs(10, alpha=.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeo.R
\name{rlatlon}
\alias{rlatlon}
\alias{rlonlat}
\alias{rgeo}
\alias{rgeo2}
\title{Sample longitude and latitude on a sphere}
\usage{
rlatlon(...)

rlonlat(...)

rgeo(n = 1, latlim = c(-90, 90), lonlim = c(-180, 180), verbose = FALSE)

rgeo2(n = 1, latlim = c(-90, 90), lonlim = c(-180, 180), verbose = FALSE)
}
\arguments{
\item{...}{arguments passed through to other functions}

\item{n}{number of random locations}

\item{latlim, lonlim}{range of latitudes and longitudes to sample within, only implemented for \code{rgeo}.}

\item{verbose}{return verbose output that includes Euclidean coordinates on unit sphere as well as
longitude and latitude.}
}
\value{
a data frame with variables \code{long} and \code{lat}.  If \code{verbose} is
TRUE, then x, y, and z coordinates are also included in the data frame.
}
\description{
Randomly samples longitude and latitude on earth so that equal areas are
(approximately) equally likely to be sampled.
(Approximation assumes earth as a perfect sphere.)
}
\details{
\code{rgeo} and \code{rgeo2} differ in the algorithms used to generate random positions.
Each assumes a spherical globe.  \code{rgeo} uses that fact that each of the x, y and z
coordinates is uniformly distributed (but not independent of each other).  Furthermore, the
angle about the z-axis is uniformly distributed and independent of z.  This provides
a straightforward way to generate Euclidean coordinates using \code{runif}.  These are then
translated into latitude and longitude.

\code{rlatlon} is an alias for \code{rgeo} and
\code{rlonlat} is too, expect that it reverses the
order in which the latitude and longitude values are
returned.

\code{rgeo2} samples points in a cube by independently sampling each coordinate.  It then
discards any point outside the sphere contained in the cube and projects the non-discarded points
to the sphere.  This method must oversample to allow for the discarded points.
}
\examples{
rgeo(4)
# sample from a region that contains the continental US
rgeo(4, latlim = c(25,50), lonlim = c(-65, -125))
rgeo2(4)
}
\seealso{
\code{\link[=deg2rad]{deg2rad()}}, \code{\link[=googleMap]{googleMap()}} and \code{\link[=latlon2xyz]{latlon2xyz()}}.
}
\keyword{geometry}
\keyword{map}
\keyword{random}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitModel.R
\name{fitModel}
\alias{fitModel}
\alias{model}
\alias{model.nlsfunction}
\alias{summary.nlsfunction}
\alias{coef.nlsfunction}
\title{Fit a nonlinear least squares model}
\usage{
fitModel(formula, data = parent.frame(), start = list(), ...)

model(object, ...)

\method{model}{nlsfunction}(object, ...)

\method{summary}{nlsfunction}(object, ...)

\method{coef}{nlsfunction}(object, ...)
}
\arguments{
\item{formula}{formula specifying the model}

\item{data}{dataframe containing the data to be used}

\item{start}{passed as \code{start} to \code{\link[=nls]{nls()}}.  If and empty list,
a simple starting point is used (thus avoiding the usual warning message).}

\item{\dots}{additional arguments passed to \code{\link[=nls]{nls()}}}

\item{object}{an R object (typically a the result of fitModel)}
}
\value{
a function
}
\description{
Allows you to specify a formula with parameters, along with starting
guesses for the parameters.  Refines those guesses to find the
least-squares fit.
}
\details{
Fits a nonlinear least squares model to data.  In contrast
to linear models, all the parameters (including linear ones)
need to be named in the formula.  The function returned
simply contains the formula together with pre-assigned
arguments setting the parameter value.  Variables used in the
fitting (as opposed to parameters) are unassigned arguments
to the returned function.
}
\note{
This doesn't work with categorical explanatory variables.  Also,
this does not work with synthetic data that fit the model perfectly.
See \code{link{nls}} for details.
}
\examples{
if (require(mosaicData)) {
f <- fitModel(temp ~ A+B*exp(-k*time), data=CoolingWater, start=list(A=50,B=50,k=1/20))
f(time=50)
coef(f)
summary(f)
model(f)
}
}
\seealso{
\code{\link[=linearModel]{linearModel()}}, \code{\link[=nls]{nls()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frequencyPolygon.R
\name{freqpolygon}
\alias{freqpolygon}
\alias{prepanel.default.freqpolygon}
\alias{panel.freqpolygon}
\title{Frequency Polygons}
\usage{
freqpolygon(
  x,
  ...,
  panel = "panel.freqpolygon",
  prepanel = "prepanel.default.freqpolygon"
)

prepanel.default.freqpolygon(
  x,
  darg = list(),
  plot.points = FALSE,
  ref = FALSE,
  groups = NULL,
  subscripts = TRUE,
  jitter.amount = 0.01 * diff(current.panel.limits()$ylim),
  center = NULL,
  nint = NULL,
  breaks = NULL,
  width = darg$width,
  type = "density",
  ...
)

panel.freqpolygon(
  x,
  darg = list(),
  plot.points = FALSE,
  ref = FALSE,
  groups = NULL,
  weights = NULL,
  jitter.amount = 0.01 * diff(current.panel.limits()$ylim),
  type = "density",
  breaks = NULL,
  nint = NULL,
  center = NULL,
  width = darg$width,
  gcol = trellis.par.get("reference.line")$col,
  glwd = trellis.par.get("reference.line")$lwd,
  h,
  v,
  ...,
  identifier = "freqpoly"
)
}
\arguments{
\item{x}{a formula or a numeric vector}

\item{\dots}{additional arguments passed on to \code{\link[=histogram]{histogram()}}
and \code{panel}.}

\item{panel}{a panel function}

\item{prepanel}{a prepanel function}

\item{darg}{a list of arguments for the function computing the frequency polygon.
This exists primarily for compatibility with \code{densityplot} and is unlikely
to be needed by the end user.}

\item{plot.points}{one of \code{TRUE}, \code{FALSE}, \code{"jitter"}, or \code{"rug"} indicating
how points are to be displayed}

\item{ref}{a logical indicating whether a horizontal reference line should be
added (roughly equivalent to \code{h=0})}

\item{groups, weights, jitter.amount, identifier}{as in \code{\link[lattice:densityplot]{lattice::densityplot()}}
or \code{\link[=histogram]{histogram()}}}

\item{subscripts}{as in other lattice prepanel functions}

\item{center}{center of one of the bins}

\item{nint}{an approximate number of bins for the frequency polygon}

\item{breaks}{a vector of breaks for the frequency polygon bins}

\item{width}{width of the bins}

\item{type}{one of \code{'density'}, \code{'percent'}, or \code{'count'}}

\item{gcol}{color of guidelines}

\item{glwd}{width of guidelines}

\item{h, v}{a vector of values for additional horizontal and vertical lines}
}
\value{
a trellis object
}
\description{
Frequency polygons are an alternative to histograms that make it simpler to overlay multiple
distributions.
}
\note{
This function make use of \code{histogram} to determine overall layout.  Often
this works reasonably well but sometimes it does not. In particular, when \code{groups} is
used to overlay multiple frequency polygons, there is often too little head room.
In the latter cases, it may be
necessary to use \code{ylim} to determine an appropriate viewing rectangle for the
plot.
}
\examples{
freqpolygon(~age | substance, data=HELPrct, v=35)
freqpolygon(~age, data=HELPrct, labels=TRUE, type='count')
freqpolygon(~age | substance, data=HELPrct, groups=sex)
freqpolygon(~age | substance, data=HELPrct, groups=sex, ylim=c(0,0.11))
## comparison of histogram and frequency polygon
histogram(~eruptions, faithful, type='density', width=.5)
ladd( panel.freqpolygon(faithful$eruptions, width=.5 ))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ashplots.R
\name{ashplot}
\alias{ashplot}
\alias{prepanel.default.ashplot}
\alias{panel.ashplot}
\title{Average Shifted Histograms}
\usage{
ashplot(
  x,
  data = data,
  ...,
  width = NULL,
  adjust = NULL,
  panel = panel.ashplot,
  prepanel = prepanel.default.ashplot
)

prepanel.default.ashplot(x, darg, groups = NULL, subscripts = TRUE, ...)

panel.ashplot(
  x,
  darg = list(),
  plot.points = FALSE,
  ref = FALSE,
  groups = NULL,
  jitter.amount = 0.01 * diff(current.panel.limits()$ylim),
  type = "p",
  ...,
  identifier = "ash"
)
}
\arguments{
\item{x}{A formula or numeric vector.}

\item{data}{A data frame.}

\item{...}{Additional arguments passed to panel and prepanel functions or \code{data}, a
data frame in which to find the variables used for the plot.}

\item{width}{The histogram bin width.}

\item{adjust}{A numeric adjustment to \code{width}.  Primarily useful when \code{width} is
not specified.  Increasing \code{adjust} makes the plot smoother.}

\item{panel}{A panel function.}

\item{prepanel}{A prepanel function.}

\item{darg}{a list of arguments for the function computing the ASH.}

\item{groups}{as in other lattice plots}

\item{subscripts}{as in other lattice prepanel functions}

\item{plot.points}{One of \code{TRUE}, \code{FALSE}, \code{"jitter"}, or \code{"rug"}}

\item{ref}{a logical indicating whether a reference line should be displayed}

\item{jitter.amount}{when \code{plot.points="jitter"}, the value to use as the amount
argument to \code{\link[=jitter]{jitter()}}.}

\item{type}{type argument used to plot points, if requested.
This is not expected to be useful, it is available mostly to protect a \code{type} argument,
if specified, from affecting the display of the ASH.}

\item{identifier}{A character string that is prepended to the names of i
grobs that are created by this panel function.}
}
\description{
An ASH plot is the average over all histograms of a fixed bin width.
}
\examples{
ashplot( ~age | substance, groups = sex, data = HELPrct)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFun.R
\name{plotFun}
\alias{plotFun}
\title{Plotting mathematical expressions}
\usage{
plotFun(
  object,
  ...,
  plot = trellis.last.object(),
  add = NULL,
  under = FALSE,
  xlim = NULL,
  ylim = NULL,
  npts = NULL,
  ylab = NULL,
  xlab = NULL,
  zlab = NULL,
  filled = TRUE,
  levels = NULL,
  nlevels = 10,
  labels = TRUE,
  surface = FALSE,
  groups = NULL,
  col = trellis.par.get("superpose.line")$col,
  col.regions = topo.colors,
  type = "l",
  lwd = trellis.par.get("superpose.line")$lwd,
  lty = trellis.par.get("superpose.line")$lty,
  alpha = NULL,
  discontinuities = NULL,
  discontinuity = 1,
  interactive = rstudio_is_available()
)
}
\arguments{
\item{object}{a mathematical expression or a function "of one variable" which will
converted to something intuitively equivalent to \code{object(x) ~ x}. (See examples)}

\item{...}{additional parameters, typically processed by \code{lattice} functions such as
\code{\link[lattice:xyplot]{lattice::xyplot()}}, \code{\link[lattice:levelplot]{lattice::levelplot()}} or their panel functions.
Frequently used parameters include
\describe{
\item{\code{main}}{main title for plot }
\item{\code{sub}}{subtitle for plot }
\item{\code{lwd}}{line width }
\item{\code{lty}}{line type }
\item{\code{col}}{a color or a (small) integer indicating which color in the current
color scheme is desired.}
}
Additionally, these arguments can be used to specify parameters for the function being
plotted and to specify the plotting window with natural names.  See the examples for such usage.}

\item{plot}{a trellis object; by default, the most recently created trellis plot.
When \code{add} is \code{TRUE}, the new function will
be plotted into a layer added to this object.}

\item{add}{if \code{TRUE}, then add a layer to an existing plot rather than creating a new plot.
If \code{NULL}, this will be determined by the value of \code{under}.}

\item{under}{if \code{TRUE}, then new layer is added beneath existing layers}

\item{xlim}{limits for x axis (or use variable names, see examples)}

\item{ylim}{limits for y axis (or use variable names, see examples)}

\item{npts}{number of points for plotting.}

\item{ylab}{label for y axis}

\item{xlab}{label for x axis}

\item{zlab}{label for z axis (when in surface-plot mode)}

\item{filled}{fill with color between the contours (\code{TRUE} by default)}

\item{levels}{levels at which to draw contours}

\item{nlevels}{number of contours to draw (if \code{levels} not specified)}

\item{labels}{if \code{FALSE}, don't label contours}

\item{surface}{draw a surface plot rather than a contour plot}

\item{groups}{grouping argument ala lattice graphics}

\item{col}{vector of colors for line graphs and contours}

\item{col.regions}{a vector of colors or a function (\code{topo.colors} by default) for generating such}

\item{type}{type of plot (\code{"l"} by default)}

\item{lwd}{vector of line widths for line graphs}

\item{lty}{vector of line types for line graphs}

\item{alpha}{number from 0 (transparent) to 1 (opaque) for the fill colors}

\item{discontinuities}{a vector of input values at which a function is
discontinuous or \code{NULL} to use
a heuristic to auto-detect.}

\item{discontinuity}{a positive number determining how sensitive the plot is to
potential discontinuity.  Larger values result in less sensitivity.  The default is 1.
Use \code{discontinuity = Inf} to disable discontinuity detection.  Discontinuity detection
uses a crude numerical heuristic and may not give the desired results in all cases.}

\item{interactive}{a logical indicating whether the surface plot should be
interactive.}
}
\value{
a \code{trellis} object
}
\description{
Plots mathematical expressions in one and two variables.
}
\details{
makes plots of mathematical expressions using the formula syntax.  Will
draw both line plots and contour/surface plots (for functions of two variables).
In RStudio, the surface plot comes with sliders to set orientation.
If the colors in filled surface plots are too blocky, increase \code{npts}
beyond the default of 50, though \code{npts=300} is as much as you're likely to ever need.
See examples for overplotting a constraint function on an objective function.
}
\examples{
plotFun( a*sin(x^2)~x, xlim=range(-5,5), a=2 )  # setting parameter value
plotFun( u^2 ~ u, ulim=c(-4,4) )                # limits in terms of u
# Note roles of ylim and y.lim in this example
plotFun( y^2 ~ y, ylim=c(-2,20), y.lim=c(-4,4) )    
# Combining plot elements to show the solution to an inequality
plotFun( x^2 -3 ~ x, xlim=c(-4,4), grid=TRUE )
ladd( panel.abline(h=0,v=0,col='gray50') )
plotFun( (x^2 -3) * (x^2 > 3) ~ x, type='h', alpha=.1, lwd=4, col='lightblue', add=TRUE )
plotFun( sin(x) ~ x, 
   groups=cut(x, findZeros(sin(x) ~ x, within=10)$x), 
   col=c('blue','green'), lty=2, lwd=3, xlim=c(-10,10) )
plotFun( sin(x) ~ x, 
   groups=cut(x, findZeros(sin(x) ~ x, within=10)$x), 
   col=c(1,2), lty=2, lwd=3, xlim=c(-10,10) )
## plotFun( sin(2*pi*x/P)*exp(-k*t)~x+t, k=2, P=.3)
f <- rfun( ~ u & v )
plotFun( f(u=u,v=v) ~ u & v, u.lim=range(-3,3), v.lim=range(-3,3) )
plotFun( u^2 + v < 3 ~ u & v, add=TRUE, npts=200 )
if (require(mosaicData)) {
# display a linear model using a formula interface
model <- lm(wage ~ poly(exper,degree=2), data=CPS85)
fit <- makeFun(model)
xyplot(wage ~ exper, data=CPS85)
plotFun(fit(exper) ~ exper, add=TRUE, lwd=3, col="red")
# Can also just give fit since it is a "function of one variable"
plotFun(fit, add=TRUE, lwd=2, col='white')
}
# Attempts to find sensible axis limits by default
plotFun( sin(k*x)~x, k=0.01 )
# Plotting a linear model with multiple predictors.
mod <- lm(length ~ width * sex, data=KidsFeet)
fitted.length <- makeFun(mod)
xyplot(length ~ width, groups=sex, data=KidsFeet, auto.key=TRUE)
plotFun(fitted.length(width, sex="B") ~ width, add=TRUE, col=1)
plotFun(fitted.length(width, sex="G") ~ width, add=TRUE, col=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingAux.R, R/do.R, R/utils.R
\name{.is.formula}
\alias{.is.formula}
\alias{.is.simple.formula}
\alias{.simple.part}
\alias{.make.data.frame}
\alias{.merge_data_frames}
\alias{.squash_names}
\alias{.do.safe.call}
\title{Check if formula}
\usage{
.is.formula(x)

.is.simple.formula(x)

.simple.part(x)

.make.data.frame(x)

.merge_data_frames(a, b)

.squash_names(object, sep = ":")

.do.safe.call(what, args, quote = FALSE, envir = parent.frame(), ...)
}
\arguments{
\item{x}{object to be converted}

\item{a}{a data frame}

\item{b}{a data frame}

\item{object}{an object}

\item{sep}{a character}

\item{what}{either a function or a non-empty character string naming the function to be called.}

\item{args}{a list of arguments to the function call. The names attribute of args gives the argument names.}

\item{quote}{a logical value indicating whether to quote the arguments.}

\item{envir}{an environment within which to evaluate the call.
This will be most useful if what is a character string and the arguments are symbols or quoted expressions.}
}
\value{
TRUE for a formula, FALSE otherwise, even if evaluation throws an error

TRUE if formula has no left-hand side or a simple right-hand side
(e.g., \code{NULL}, ., 1,  or 0)

simple part of formula or NULL if formula is not simple

a data frame

a data frame

a character vector

The result of the (evaluated) function call.
}
\description{
Check if formula

Check for simple formula

Extract simple part from formula
}
\details{
\code{.make.data.frame} converts things to a data frame

\code{.merge_data_frames} is a wrapper around merge

\code{.squash_names} squashes names of a data frame into a single string

\code{.do.safe.call} avoids conflicts between named arguments and ... by taking named arguments
preferentially.
}
\seealso{
\code{\link[=do.call]{do.call()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/do.R
\name{do}
\alias{do}
\alias{do.numeric}
\alias{do.default}
\alias{Do}
\alias{print.repeater}
\alias{*,repeater,ANY-method}
\title{Do Things Repeatedly}
\usage{
do(object, ...)

\method{do}{numeric}(object, ...)

\method{do}{default}(object, ...)

Do(n = 1L, cull = NULL, mode = "default", algorithm = 1, parallel = TRUE)

\method{print}{repeater}(x, ...)

\S4method{*}{repeater,ANY}(e1, e2)
}
\arguments{
\item{object}{an object}

\item{...}{additional arguments}

\item{n}{number of times to repeat}

\item{cull}{function for culling output of objects being repeated.  If NULL,
a default culling function is used.  The default culling function is
currently aware of objects of types
\code{lme},
\code{lm},
\code{htest},
\code{table},
\code{cointoss}, and
\code{matrix}.}

\item{mode}{target mode for value returned}

\item{algorithm}{a number used to select the algorithm used.  Currently numbers below 1
use an older algorithm and numbers >=1 use a newer algorithm which is faster in some
situations.}

\item{parallel}{a logical indicating whether parallel computation should be attempted
using the \pkg{parallel} package (if it is installed and loaded).}

\item{x}{an object created by \code{do}.}

\item{e1}{an object (in cases documented here, the result of running \code{do})}

\item{e2}{an object (in cases documented here, an expression to be repeated)}
}
\value{
\code{do} returns an object of class \code{repeater} which is only useful in
the context of the operator \code{*}.  See the examples.
}
\description{
\code{do()} provides a natural syntax for repetition tuned to assist
with replication and resampling methods.
}
\note{
\code{do} is a thin wrapper around \code{Do} to avoid collision with
\code{\link[dplyr:do]{dplyr::do()}} from the \pkg{dplyr} package.
}
\section{Naming}{

The names used in the object returned from \code{do()} are inferred from the
objects created in each replication.  Roughly, this the strategy employed.
\itemize{
\item If the objects have names, those names are inherited, if possible.
\item If the objects do not have names, but \code{do()} is used with a simple
function call, the name of that function is used.
Example: \code{do(3) * mean(~height, data = Galton)} produces a data frame with
a variable named \code{mean}.
\item In cases where names are not easily inferred and a single result is produced,
it is named \code{result}.
}

To get different names, one can rename the objects as they are created, or
rename the result returned from \code{do()}.  Example of the former:
\code{do(3) * c(mean_height = mean(~height, data = resample(Galton)))}.
If you like the name \code{result}, try wrapping the expression in \code{{ }}:
\code{do(3) * { mean(~ height, data = resample(Galton)) }}.
}

\examples{
do(3) * rnorm(1)
do(3) * "hello"
do(3) * 1:4
do(3) * mean(rnorm(25))
do(3) * lm(shuffle(height) ~ sex + mother, Galton)
do(3) * anova(lm(shuffle(height) ~ sex + mother, Galton))
do(3) * c(sample.mean = mean(rnorm(25)))
# change the names on the fly
do(3) * mean(~height, data = resample(Galton))
do(3) * c(mean_height = mean(~height, data = resample(Galton)))
# this results in the name being `result`
do(3) * { mean(~height, data = resample(Galton)) }
set.rseed(1234)
do(3) * tally( ~sex|treat, data=resample(HELPrct))
set.rseed(1234)  # re-using seed gives same results again
do(3) * tally( ~sex|treat, data=resample(HELPrct))
}
\seealso{
\code{\link[=replicate]{replicate()}}, \code{\link[=set.rseed]{set.rseed()}}
}
\author{
Daniel Kaplan (\email{kaplan@macalaster.edu})
and Randall Pruim (\email{rpruim@calvin.edu})
}
\keyword{iteration}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findZerosMult.R
\name{findZerosMult}
\alias{findZerosMult}
\title{Find the zeros of a function of two or more variables}
\usage{
findZerosMult(..., npts = 10, rad = 5, near = 0, sortBy = "byx")
}
\arguments{
\item{...}{arguments for values NOTE: if the system has more than one equation and the rhs
variables do not match up, there will be an error.}

\item{npts}{number of desired zeros to return}

\item{rad}{radius around near in which to look for zeros}

\item{near}{center of search for zeros}

\item{sortBy}{options for sorting zeros for plotting.  Options are 'byx', 'byy' and 'radial'.  The
default value is 'byx'.}
}
\value{
A data frame of numerical values which should all result in a value of zero when input into
original function
}
\description{
Compute numerically zeros of a function of two or more variables.
All free variables (all but the variable on the right side) named in the expression must be assigned
a value via \verb{\\ldots}
}
\details{
sorts points in the domain according to the sign of the function value at respective points.
Use continuity and uniroot to find zeros between points of opposite signs.  Returns any number of
points which may be sorted and plotted according to x, y, or radial values.
}
\examples{
findZerosMult(a*x^2-8~a&x, npts = 50)
findZerosMult(a^2+x^2-8~a&x, npts = 100, sortBy='radial')
\dontrun{findZerosMult(a^2+x^2-8~a&x, npts = 1000, sortBy='radial')}
}
\author{
Cecylia Bocovich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resample.R
\name{resample}
\alias{resample}
\alias{deal}
\alias{shuffle}
\alias{sample}
\alias{sample.default}
\alias{sample.data.frame}
\alias{sample.matrix}
\alias{sample.factor}
\alias{sample.lm}
\title{More Random Samples}
\usage{
resample(..., replace = TRUE)

deal(...)

shuffle(x, replace = FALSE, prob = NULL, groups = NULL, orig.ids = FALSE)

sample(x, size, replace = FALSE, ...)

\method{sample}{default}(
  x,
  size,
  replace = FALSE,
  prob = NULL,
  groups = NULL,
  orig.ids = FALSE,
  ...
)

\method{sample}{data.frame}(
  x,
  size,
  replace = FALSE,
  prob = NULL,
  groups = NULL,
  orig.ids = TRUE,
  fixed = names(x),
  shuffled = c(),
  invisibly.return = NULL,
  ...
)

\method{sample}{matrix}(
  x,
  size,
  replace = FALSE,
  prob = NULL,
  groups = NULL,
  orig.ids = FALSE,
  ...
)

\method{sample}{factor}(
  x,
  size,
  replace = FALSE,
  prob = NULL,
  groups = NULL,
  orig.ids = FALSE,
  drop.unused.levels = FALSE,
  ...
)

\method{sample}{lm}(
  x,
  size,
  replace = FALSE,
  prob = NULL,
  groups = NULL,
  orig.ids = FALSE,
  drop.unused.levels = FALSE,
  parametric = FALSE,
  transformation = NULL,
  ...
)
}
\arguments{
\item{\dots}{additional arguments passed to
\code{\link[base:sample]{base::sample()}}
or \code{\link[=sample]{sample()}}.}

\item{replace}{Should sampling be with replacement?}

\item{x}{Either a vector of one or more elements from which to choose, or a positive integer.}

\item{prob}{A vector of probability weights for obtaining the elements of the vector being sampled.}

\item{groups}{a vector (or variable in a data frame) specifying
groups to sample within. This will be recycled if necessary.}

\item{orig.ids}{a logical; should original ids be included in returned data frame?}

\item{size}{a non-negative integer giving the number of items to choose.}

\item{fixed}{a vector of column names.  These variables are shuffled en masse,
preserving associations among these columns.}

\item{shuffled}{a vector of column names.
these variables are reshuffled individually (within groups if \code{groups} is
specified), breaking associations among these columns.
examples.}

\item{invisibly.return}{a logical, should return be invisible?}

\item{drop.unused.levels}{a logical, should unused levels be dropped?}

\item{parametric}{A logical indicating whether the resampling should be done parametrically.}

\item{transformation}{NULL or a function providing a transformation to be applied to the
synthetic responses.  If NULL, an attempt it made to infer the appropriate transformation
from the original call as recorded in \code{x}.}
}
\description{
These functions simplify and unify sampling in various ways.
}
\details{
These functions are wrappers around \code{\link[=sample]{sample()}} providing different defaults and
natural names.
}
\examples{
# 100 Bernoulli trials -- no need for replace=TRUE
resample(0:1, 100)
tally(resample(0:1, 100))
if (require(mosaicData)) {
Small <- sample(KidsFeet, 10)
resample(Small)
tally(~ sex, data=resample(Small))
tally(~ sex, data=resample(Small))
# fixed marginals for sex
tally(~ sex, data=Small)
tally(~ sex, data=resample(Small, groups=sex)) 
# shuffled can be used to reshuffle some variables within groups
# orig.id shows where the values were in original data frame.
Small <- mutate(Small, 
   id1 = paste(sex,1:10, sep=":"),  
   id2 = paste(sex,1:10, sep=":"))
resample(Small, groups=sex, shuffled=c("id1","id2"))
}
deal(Cards, 13)    # A Bridge hand
shuffle(Cards)
model <- lm(width ~length * sex, data = KidsFeet)
KidsFeet |> head()
resample(model) |> head()
Boot <- do(500) * lm(width ~ length * sex, data = resample(KidsFeet))
df_stats(~ Intercept + length + sexG + length.sexG, data = Boot, sd)
head(Boot)
summary(coef(model))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval.R
\name{rsquared}
\alias{rsquared}
\title{Extract r-squared value}
\usage{
rsquared(x, ...)
}
\arguments{
\item{x}{an object}

\item{\dots}{additional arguments}
}
\description{
Attempts to extract an r-squared value from a model or model-like object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smoothers.R
\name{FunctionsFromData}
\alias{FunctionsFromData}
\alias{spliner}
\alias{smoother}
\alias{linearModel}
\alias{connector}
\title{Create function from data}
\usage{
spliner(formula, data = NULL, method = "fmm", monotonic = FALSE)

connector(formula, data = NULL, method = "linear")

smoother(formula, data, span = 0.5, degree = 2, ...)

linearModel(formula, data, ...)
}
\arguments{
\item{formula}{a formula.  Only one quantity is allowed on the left-hand side, the
output quantity}

\item{data}{a data frame}

\item{method}{a method for splining.  See \code{\link[=spline]{spline()}}.}

\item{monotonic}{a \code{TRUE/FALSE} flag specifying whether the spline should
respect monotonicity in the data}

\item{span}{parameter to smoother.  How smooth it should be.}

\item{degree}{parameter to smoother. 1 is locally linear, 2 is locally quadratic.}

\item{\dots}{additional arguments to \code{\link[stats:loess]{stats::loess()}} or \code{\link[stats:lm]{stats::lm()}}}
}
\description{
These functions create mathematical functions from data, by smoothing, splining, or linear
combination (fitting).  Each of them takes a formula and a data frame as an argument
}
\details{
These functions use data to create a mathematical, single-valued function of the inputs.
All return a function whose arguments are the variables used on the right-hand side of the formula.
If the formula involves a transformation, e.g. \code{sqrt(age)} or \code{log(income)},
only the variable itself, e.g. \code{age} or \code{income}, is an argument to the function.

\code{linearModel} takes a linear combination of the vectors specified on the right-hand side.
It differs from \code{project} in that \code{linearModel} returns a function
whereas \code{project} returns the coefficients.  NOTE: An intercept term is not included
unless that is explicitly part of the formula with \code{+1}.  This conflicts with the
standard usage of formulas as found in \code{lm}.  Another option for creating
such functions is to combine \code{\link[=lm]{lm()}} and \code{\link[=makeFun]{makeFun()}}.

\code{spliner} and \code{connector} currently work for only one input variable.
}
\examples{
if (require(mosaicData)) {
data(CPS85)
f <- smoother(wage ~ age, span=.9, data=CPS85)
f(40)
g <- linearModel(log(wage) ~ age + educ + 1, data=CPS85)
g(age=40, educ=12)
# an alternative way to define g (Note: + 1 is the default for lm().)
g2 <- makeFun(lm(log(wage) ~ age + educ, data=CPS85))
g2(age=40, educ=12)
x<-1:5; y=c(1, 2, 4, 8, 8.2)
f1 <- spliner(y ~ x)
f1(x=8:10)
f2 <- connector(x~y)
}
}
\seealso{
\code{\link[=project]{project()}} method for formulas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mosaic-package.R
\docType{package}
\name{mosaic-package}
\alias{mosaic-package}
\alias{mosaic}
\title{mosaic: the Project MOSAIC package}
\description{
mosaic
}
\details{
Data sets and utilities from Project MOSAIC (mosaic-web.org) used to teach mathematics,
statistics, computation and modeling.  Funded by the NSF, Project MOSAIC is a
community of educators working to tie together aspects of quantitative work that students
in science, technology, engineering and mathematics will need in their professional lives,
but which are usually taught in isolation, if at all.
}
\references{
\url{http://www.mosaic-web.org}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ProjectMOSAIC/mosaic}
  \item \url{https://www.mosaic-web.org/mosaic/}
  \item Report bugs at \url{https://github.com/ProjectMOSAIC/mosaic/issues}
}

}
\author{
Randall Pruim (\email{rpruim@calvin.edu}), Daniel Kaplan (\email{kaplan@macalester.edu}), Nicholas Horton (\email{nhorton@smith.edu})
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpqrdist.R
\name{pdist}
\alias{pdist}
\alias{xpgamma}
\alias{xpt}
\alias{xpchisq}
\alias{xpf}
\alias{xpbinom}
\alias{xppois}
\alias{xpgeom}
\alias{xpnbinom}
\alias{xpbeta}
\title{Illustrated probability calculations from distributions}
\usage{
pdist(
  dist = "norm",
  q,
  plot = TRUE,
  verbose = FALSE,
  invisible = FALSE,
  digits = 3L,
  xlim,
  ylim,
  resolution = 500L,
  return = c("values", "plot"),
  ...,
  refinements = list()
)

xpgamma(
  q,
  shape,
  rate = 1,
  scale = 1/rate,
  lower.tail = TRUE,
  log.p = FALSE,
  ...
)

xpt(q, df, ncp, lower.tail = TRUE, log.p = FALSE, ...)

xpchisq(q, df, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)

xpf(q, df1, df2, lower.tail = TRUE, log.p = FALSE, ...)

xpbinom(q, size, prob, lower.tail = TRUE, log.p = FALSE, ...)

xppois(q, lambda, lower.tail = TRUE, log.p = FALSE, ...)

xpgeom(q, prob, lower.tail = TRUE, log.p = FALSE, ...)

xpnbinom(q, size, prob, mu, lower.tail = TRUE, log.p = FALSE, ...)

xpbeta(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)
}
\arguments{
\item{dist}{a character description of a distribution, for example
\code{"norm"}, \code{"t"}, or \code{"chisq"}}

\item{q}{a vector of quantiles}

\item{plot}{a logical indicating whether a plot should be created}

\item{verbose}{a logical}

\item{invisible}{a logical}

\item{digits}{the number of digits desired}

\item{xlim}{x limits}

\item{ylim}{y limits}

\item{resolution}{Number of points used for detecting discreteness and generating plots.
The default value of 5000 should work well except for discrete distributions
that have many distinct values, especially if these values are not evenly spaced.}

\item{return}{If \code{"plot"}, return a plot.  If \code{"values"}, return a vector of numerical values.}

\item{...}{Additional arguments, typically for fine tuning the plot.}

\item{refinements}{A list of refinements to the plot.  See \code{\link[ggformula:gf_refine]{ggformula::gf_refine()}}.}

\item{shape, scale}{shape and scale parameters.  Must be positive,
    \code{scale} strictly.}

\item{rate}{an alternative way to specify the scale.}

\item{lower.tail}{logical; if \code{TRUE} (default), probabilities are
    \eqn{P[X < x]}, otherwise, \eqn{P[X \ge x]} (exceptionally not
    \emph{cadlag} here, to ensure, e.g., \code{pgamma(0, shape=0)} remains zero).}

\item{log.p}{A logical indicating whether probabilities should be
returned on the log scale.}

\item{df}{degrees of freedom (\eqn{> 0}, maybe non-integer).  \code{df
      = Inf} is allowed.}

\item{ncp}{non-centrality parameter \eqn{\delta}{delta};
    currently except for \code{rt()}, accurate only for \code{abs(ncp) <= 37.62}.
    If omitted, use the central \eqn{t} distribution.}

\item{df1, df2}{degrees of freedom.  \code{Inf} is allowed.}

\item{size}{number of trials (zero or more).}

\item{prob}{probability of success on each trial.}

\item{lambda}{vector of (non-negative) means.}

\item{mu}{alternative parametrization via mean: see \sQuote{Details}.}

\item{shape1, shape2}{non-negative parameters of the Beta distribution.}
}
\value{
A vector of probabilities; a plot is printed as a side effect.
}
\description{
Illustrated probability calculations from distributions
}
\details{
The most general function is \code{pdist} which can work with
any distribution for which a p-function exists.  As a convenience, wrappers are
provided for several common distributions.
}
\examples{
pdist("norm", -2:2)
pdist("norm", seq(80,120, by = 10), mean = 100, sd = 10)
pdist("chisq", 2:4, df = 3)
pdist("f", 1, df1 = 2, df2 = 10)
pdist("gamma", 2, shape = 3, rate = 4)
}
\seealso{
\code{\link[=qdist]{qdist()}}, \code{\link[=xpnorm]{xpnorm()}}, \code{\link[=xqnorm]{xqnorm()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ladd.R
\name{ladd}
\alias{ladd}
\title{Add to Lattice Plots}
\usage{
ladd(x, data = NULL, ..., plot = trellis.last.object())
}
\arguments{
\item{x}{callable graphical element to be added to a panel or panels in a lattice plot}

\item{data}{a list containing objects that can be referred to in \code{x}.  Panel functions also
have access to the data already used in the panel by the underlying lattice plot.  See
\code{\link[latticeExtra:layer]{latticeExtra::layer()}} for details.}

\item{...}{additional arguments passed to \code{\link[latticeExtra:layer]{latticeExtra::layer()}}.}

\item{plot}{a lattice plot to add to.  Defaults to previous lattice plot.}
}
\value{
a trellis object
}
\description{
Simplified lattice plotting by adding additional elements to existing plots.
}
\details{
\code{ladd} is a wrapper around \code{\link[latticeExtra:layer]{latticeExtra::layer()}} that simplifies
certain common plotting additions.  The same caveats that apply to that function
apply here as well.  In particular, \code{ladd} uses non-standard evaluation.
For this reason care must be taken if trying to use \code{ladd} within other functions
and the use of \code{data} may be required to pass information into the environment
in which \code{x} will be evaluated.
}
\examples{
p <- xyplot(rnorm(100) ~rnorm(100))
print(p)
ladd(panel.abline(a=0,b=1))
ladd(panel.abline(h=0,col='blue'))
ladd(grid.text('Hello'))
ladd(grid.text(x=.95,y=.05,'text here',just=c('right','bottom')))
q <- xyplot(rnorm(100) ~rnorm(100)|factor(rbinom(100,4,.5)))
q <- update(q, layout=c(3,2))
ladd(panel.abline(a=0,b=1), plot=q)
ladd(panel.abline(h=0,col='blue'))
ladd( grid.text("(2,1)",gp=gpar(cex=3,alpha=.5)), columns=2, rows=1)
ladd( grid.text("p5",gp=gpar(cex=3,alpha=.5)), packets=5)
q
ladd( grid.text(paste(current.column(), current.row(),sep=','), gp=gpar(cex=3,alpha=.5)) )
histogram( ~eruptions, data=faithful )
# over would probably be better here, but the demonstrates what under=TRUE does.
ladd(panel.densityplot(faithful$eruptions, lwd=4), under=TRUE) 

}
\seealso{
\code{\link[latticeExtra:layer]{latticeExtra::layer()}}
}
\author{
Randall Pruim (\email{rpruim@calvin.edu})
}
\keyword{graphics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDist.R
\name{plotDist}
\alias{plotDist}
\title{Plots of Discrete and Continuous Distributions}
\usage{
plotDist(
  dist,
  ...,
  xlim = NULL,
  ylim = NULL,
  add,
  under = FALSE,
  packets = NULL,
  rows = NULL,
  columns = NULL,
  kind = c("density", "cdf", "qq", "histogram"),
  xlab = "",
  ylab = "",
  breaks = NULL,
  type,
  resolution = 5000L,
  params = NULL
)
}
\arguments{
\item{dist}{A string identifying the distribution.  This should work
with any distribution that has associated functions beginning
with 'd', 'p', and 'q' (e.g,
\code{\link[=dnorm]{dnorm()}},
\code{\link[=pnorm]{pnorm()}}, and
\code{\link[=qnorm]{qnorm()}}).  \code{dist} should match the name of the
distribution with the initial 'd', 'p', or 'q' removed.}

\item{...}{other arguments passed along to lattice graphing routines}

\item{xlim}{a numeric vector of length 2 or \code{NULL}, in which case
the central 99.8 of the distribution is used.}

\item{ylim}{a numeric vector of length 2 or \code{NULL}, in which case
a heuristic is used to avoid chasing asymptotes in distributions like
the F distributions with 1 numerator degree of freedom.}

\item{add}{a logical indicating whether the plot should be added to the previous lattice plot.
If missing, it will be set to match \code{under}.}

\item{under}{a logical indicating whether adding should be done in a layer under or over the existing
layers when \code{add = TRUE}.}

\item{packets, rows, columns}{specification of which panels will be added to when
\code{add} is \code{TRUE}.  See \code{\link[latticeExtra:layer]{latticeExtra::layer()}}.}

\item{kind}{one of "density", "cdf", "qq", or "histogram" (or prefix
of any of these)}

\item{xlab, ylab}{as per other lattice functions}

\item{breaks}{a vector of break points for bins of histograms,
as in \code{\link[=histogram]{histogram()}}}

\item{type}{passed along to various lattice graphing functions}

\item{resolution}{number of points to sample when generating the plots}

\item{params}{a list containing parameters for the distribution.  If \code{NULL} (the default),
this list is created from elements of \verb{\\dots} that are either unnamed or have names among
the formals of the appropriate distribution function.  See the examples.}
}
\description{
Provides a simple way to generate plots of pdfs, probability mass functions,
cdfs, probability histograms, and normal-quantile plots for distributions
known to R.
}
\details{
\code{plotDist()} determines whether the distribution
is continuous or discrete by seeing if all the sampled quantiles are
unique.  A discrete random variable with many possible values could
fool this algorithm and be considered continuous.

The plots are done referencing a data frame with variables
\code{x} and \code{y} giving points on the graph of the
pdf, pmf, or cdf for the distribution.  This can be useful in conjunction
with the \code{groups} argument.  See the examples.
}
\examples{
plotDist('norm')
plotDist('norm', type='h')
plotDist('norm', kind='cdf')
plotDist('exp',  kind='histogram')
plotDist('binom', params=list( 25, .25))       # explicit params
plotDist('binom', 25, .25)                     # params inferred
plotDist('norm', mean=100, sd=10, kind='cdf')  # params inferred
plotDist('binom', 25, .25, xlim=c(-1,26) )     # params inferred
plotDist('binom', params=list( 25, .25), kind='cdf')
plotDist('beta', params=list( 3, 10), kind='density')
plotDist('beta', params=list( 3, 10), kind='cdf')
plotDist( "binom", params=list(35,.25), 
           groups= y < dbinom(qbinom(0.05, 35, .25), 35,.25) )
plotDist( "binom", params=list(35,.25), 
           groups= y < dbinom(qbinom(0.05, 35, .25), 35,.25), 
           kind='hist')
plotDist("norm", mean=10, sd=2, col="blue", type="h")
plotDist("norm", mean=12, sd=2, col="red", type="h", under=TRUE)
plotDist("binom", size=100, prob=.30) +
  plotDist("norm", mean=30, sd=sqrt(100 * .3 * .7))
plotDist("chisq", df=4, groups = x > 6, type="h")
plotDist("f", df1=1, df2 = 99)
if (require(mosaicData)) {
histogram( ~age|sex, data=HELPrct)
m <- mean( ~age|sex, data=HELPrct)
s <- sd(~age|sex, data=HELPrct)
plotDist( "norm", mean=m[1], sd=s[1], col="red", add=TRUE, packets=1)
plotDist( "norm", mean=m[2], sd=s[2], col="blue", under=TRUE, packets=2)
}

}
\seealso{
\code{\link[ggformula:gf_dist]{ggformula::gf_dist()}}
}
\keyword{graphics}
\keyword{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binom.test.R
\name{binom.test}
\alias{binom.test}
\title{Exact Tests for Proportions}
\usage{
binom.test(
  x,
  n = NULL,
  p = 0.5,
  alternative = c("two.sided", "less", "greater"),
  conf.level = 0.95,
  ci.method = c("Clopper-Pearson", "binom.test", "Score", "Wilson", "prop.test", "Wald",
    "Agresti-Coull", "Plus4"),
  data = NULL,
  success = NULL,
  ...
)
}
\arguments{
\item{x}{count of successes, length 2 vector of success and failure counts, a formula,
or a character, numeric, or factor vector containing raw data.}

\item{n}{sample size (successes + failures) or a data frame
(for the formula interface)}

\item{p}{probability for null hypothesis}

\item{alternative}{type of alternative hypothesis}

\item{conf.level}{confidence level for confidence interval}

\item{ci.method}{a method to use for computing the confidence interval
(case insensitive and may be abbreviated).  See details below.}

\item{data}{a data frame (if missing, \code{n} may be a data frame)}

\item{success}{level of variable to be considered success.  All other levels are
considered failure.}

\item{...}{additional arguments (often ignored)}
}
\value{
an object of class \code{htest}
}
\description{
The \code{binom.test()} function
performs an exact test of a simple null hypothesis about the probability of success in a
Bernoulli experiment from summarized data or from raw data.
The mosaic \code{binom.test} provides wrapper functions around the function of the same name in \pkg{stats}.
These wrappers provide an extended interface (including formulas).
}
\details{
\code{binom.test()} is a wrapper around \code{\link[stats:binom.test]{stats::binom.test()}} from the stats
package to simplify its use when the raw data are available, in which case
an extended syntax for \code{binom.test()} is provided.  See the examples.

Also, five confidence interval methods are provided:
*
\verb{"Clopper-Pearson", "binom.test"}: This is the interval produced when using
\code{\link[stats:binom.test]{stats::binom.test()}}
from the stats package.  It guarantees a coverage rate at least as large as
the nominal coverage rate, but may produce wider intervals than some of the methods
below, which may either under- or over-cover depending on the data.
\itemize{
\item `"Score", "Wilson", "prop.test":  This is the usual method used by \code{\link[stats:prop.test]{stats::prop.test()}}
and is computed by inverting p-values from score tests. It is often attributed to
Edwin Wilson.  If specified with \code{"prop.test"}, the continuity correction is applied
(as is the default in \code{\link{prop.test}()}), else the continuity correction is not
applied.
\itemize{
\item \code{"Wald"} This is the interval traditionally taught in entry level statistics courses.
It uses the sample proportion to estimate the standard error and uses normal
theory to determine how many standard deviations to add and/or subtract from
the sample proportion to determine an interval.
\item \"Agresti-Coull"`
This is the Wald method after setting \eqn{n' = n + z^2} and
\eqn{p'= (x + z^2/2) / n}' and using \eqn{x' = n' p'} and \eqn{n'}
in place of \eqn{x} and \eqn{n}.
\item \code{"Plus4"}
This is Wald after adding in two artificial success and two artificial failures.  It
is nearly the same as the Agresti-Coull method when the confidence level is 95\%. since
\eqn{z^2} is approximately 4 and \eqn{z^2/2} is approximately 2.
}
}
}
\note{
When \code{x} is a 0-1 vector, 0 is treated as failure and 1 as success. Similarly,
for a logical vector \code{TRUE} is treated as success and \code{FALSE} as failure.
}
\examples{
# Several ways to get a confidence interval for the proportion of Old Faithful
# eruptions lasting more than 3 minutes.
data(faithful)
binom.test(faithful$eruptions > 3)
binom.test(97, 272)
binom.test(c(97, 272-97))
faithful$long <- faithful$eruptions > 3
binom.test(faithful$long)
binom.test(resample(1:4, 400), p=.25)
binom.test(~ long, data = faithful)
binom.test(~ long, data = faithful, ci.method = "Wald")
binom.test(~ long, data = faithful, ci.method = "Plus4")
with(faithful, binom.test(~long))
with(faithful, binom.test(long))

}
\seealso{
\code{\link[=prop.test]{prop.test()}}, \code{\link[stats:binom.test]{stats::binom.test()}}
}
\keyword{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/panel.lmbands.R
\name{panel.lmbands}
\alias{panel.lmbands}
\title{show confidence and prediction bands on plots}
\usage{
panel.lmbands(
  x,
  y,
  interval = "confidence",
  level = 0.95,
  model = lm(y ~ x),
  band.col = c(conf = slcol[3], pred = slcol[2]),
  band.lty = c(conf = slty[3], pred = slty[2]),
  band.show = TRUE,
  fit.show = TRUE,
  band.alpha = 0.6,
  band.lwd = 1,
  npts = 100,
  ...
)
}
\arguments{
\item{x, y}{numeric vectors}

\item{interval}{a vector subset of \code{'confidence'} and \code{'prediction'}}

\item{level}{confidence level}

\item{model}{model to be used for generating bands}

\item{band.col}{a vector of length 1 or 2 giving the color of bands}

\item{band.lty}{a vector of length 1 or 2 giving the line type for bands}

\item{band.show}{logical vector of length 1 or 2 indicating whether
confidence and prediction bands should be shown}

\item{fit.show}{logical indicating whether the model fit should be shown}

\item{band.alpha}{a vector of length 1 or 2 alpha level for bands}

\item{band.lwd}{a vector of length 1 or 2 giving line width for bands}

\item{npts}{resolution parameter for bands (increase to get better resolution)}

\item{\dots}{additional arguments}
}
\description{
show confidence and prediction bands on plots
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Tukey.R
\name{TukeyHSD.lm}
\alias{TukeyHSD.lm}
\alias{TukeyHSD.formula}
\title{Additional interfaces to TukeyHSD}
\usage{
\method{TukeyHSD}{lm}(x, which, ordered = FALSE, conf.level = 0.95, ...)

\method{TukeyHSD}{formula}(
  x,
  which,
  ordered = FALSE,
  conf.level = 0.95,
  data = parent.frame(),
  ...
)
}
\arguments{
\item{x}{an object, for example of class \code{lm} or \code{formula}}

\item{which, ordered, conf.level, \dots}{just as in \code{\link[=TukeyHSD]{TukeyHSD()}} from the \code{base} package}

\item{data}{a data frame.  NB: This does not come second in the argument list.}
}
\description{
\code{\link[=TukeyHSD]{TukeyHSD()}} requires use of \code{\link[=aov]{aov()}}.
Since this is a hindrance for beginners, wrappers
have been provided to remove this need.
}
\examples{
## These should all give the same results
if (require(mosaicData)) {
  model <- lm(age ~ substance, data=HELPrct)
  TukeyHSD(model)
  TukeyHSD( age ~ substance, data=HELPrct)
  TukeyHSD(aov(age ~ substance, data=HELPrct))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{theme_map}
\alias{theme_map}
\title{ggplot2 theme for maps}
\usage{
theme_map(base_size = 12)
}
\arguments{
\item{base_size}{the base font size for the theme.}
}
\description{
A very plain \pkg{ggplot2} theme that is good for maps.
}
\details{
This theme is largely based on an example posted by
Winston Chang at the \pkg{ggplot2} Google group forum.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rand.R
\name{rand}
\alias{rand}
\title{Random Regressors}
\usage{
rand(df = 1, rdist = rnorm, args = list(), nrow, seed = NULL)
}
\arguments{
\item{df}{degrees of freedom, i.e., number of random regressors}

\item{rdist}{random distribution function for sampling}

\item{args}{arguments for \code{rdist}}

\item{nrow}{number of rows in resulting matrix.  This can often be omitted in
the context of functions like \code{lm} where it is inferred from the data frame,
if one is provided.}

\item{seed}{seed for random number generation}
}
\value{
A matrix of random variates with \code{df} columns.
In its intended use, the number of rows will be selected to match the
size of the data frame supplied to \code{lm}
}
\description{
A utility function for producing random regressors with a specified
number of degrees of freedom.
}
\examples{
rand(2,nrow=4)
rand(2,rdist=rpois, args=list(lambda=3), nrow=4)
summary(lm( waiting ~ eruptions + rand(1), faithful))
}
\keyword{distribution}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/do.R
\name{cull_for_do}
\alias{cull_for_do}
\title{Cull objects used with do()}
\usage{
cull_for_do(object, ...)
}
\arguments{
\item{object}{an object to be culled}

\item{...}{additional arguments (currently ignored)}
}
\description{
The \code{\link[=do]{do()}} function facilitates easy replication for
randomization tests and bootstrapping (among other things).  Part of what
makes this particularly useful is the ability to cull from the objects
produced those elements that are useful for subsequent analysis.
\code{cull_for_do} does this culling.  It is generic, and users
can add new methods to either change behavior or to handle additional
classes of objects.
}
\details{
When \code{do(n) * expression} is evaluated, \code{expression}
is evaluated \code{n} times to produce a list of \code{n} result objects.
\code{cull_for_do} is then applied to each element of this list to
extract from it the information that should be stored.  For example,
when applied to a object of class \code{"lm"},
the default \code{cull_for_do} extracts the coefficients, coefficient
of determinism, an the estimate for the variance, etc.
}
\examples{
cull_for_do(lm(length ~ width, data = KidsFeet))
do(1) * lm(length ~ width, data = KidsFeet)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop.test.R
\name{prop_test}
\alias{prop_test}
\title{Internal function for testing proportion}
\usage{
prop_test(
  x,
  n,
  p = NULL,
  alternative = c("two.sided", "less", "greater"),
  conf.level = 0.95,
  ...
)
}
\arguments{
\item{x}{a vector, count, or formula.}

\item{n}{a vector of counts of trials (not needed when \code{x} is a table or matrix).}

\item{p}{a vector of probabilities of success (for the null hypothesis).
The length must be the same as the number of groups specified by \code{x}.}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.  Only used for testing the null that a single proportion
    equals a given value, or that two proportions are equal; ignored
    otherwise.}

\item{conf.level}{confidence level of the returned confidence
    interval.  Must be a single number between 0 and 1.  Only used
    when testing the null that a single proportion equals a given
    value, or that two proportions are equal; ignored otherwise.}

\item{...}{additional arguments passed to methods.}
}
\description{
This function is wrapped by \code{\link[=prop.test]{prop.test()}}, which most users should use instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCumfreq.R
\name{plotCumfreq}
\alias{plotCumfreq}
\alias{plotCumfreq.formula}
\alias{plotCumfreq.default}
\alias{prepanel.cumfreq}
\alias{panel.cumfreq}
\title{Cumulative frequency plots}
\usage{
plotCumfreq(x, data, ...)

\method{plotCumfreq}{formula}(x, data = NULL, subscripts, ...)

\method{plotCumfreq}{default}(x, ...)

prepanel.cumfreq(x, ...)

panel.cumfreq(x, type = c("smooth", "step"), groups = NULL, ...)
}
\arguments{
\item{x}{a formula or numeric vector}

\item{data}{a data frame in which \code{x} is evaluated if \code{x} is a
formula.}

\item{\dots}{other lattice arguments}

\item{subscripts}{as in lattice plots}

\item{type}{smooth or step-function?}

\item{groups}{grouping variable}
}
\value{
A plot of the empirical cumulative distribution function for sample values specified in \code{x}.
}
\description{
A high-level function for producing a cumulative frequency plot using
\code{lattice} graphics.
}
\examples{
plotCumfreq(~eruptions, faithful, xlab = 'duration of eruptions')
}
\seealso{
\code{\link[lattice:histogram]{lattice::histogram()}}, \code{\link[lattice:densityplot]{lattice::densityplot()}}
}
\keyword{graphics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetchData.R, R/fetchGapminder.R,
%   R/fetchGoogle.R
\name{fetchData}
\alias{fetchData}
\alias{fetchGapminder1}
\alias{fetchGapminder}
\alias{fetchGoogle}
\title{Defunct functions now in the fetch package}
\usage{
fetchData(...)

fetchGapminder1(...)

fetchGapminder(...)

fetchGoogle(...)
}
\arguments{
\item{...}{arguments}
}
\description{
These functions have been moved to the fetch package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross.R
\name{cross}
\alias{cross}
\title{Factor cross products}
\usage{
cross(..., sep = ":", drop.unused.levels = FALSE)
}
\arguments{
\item{\dots}{factors to be crossed.}

\item{sep}{separator between levels}

\item{drop.unused.levels}{should levels that do not appear in cross product be dropped?}
}
\value{
a factor
}
\description{
Construct a product of factors.
}
\examples{
x <- letters[1:3]
y <- c(1,2,1,1,3,1,3)
cross(x, y)
cross(x, y, drop.unused.levels=TRUE)

}
\keyword{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPoints.R
\name{plotPoints}
\alias{plotPoints}
\title{Scatter plot of points}
\usage{
plotPoints(
  x,
  data = parent.frame(),
  add = NULL,
  under = FALSE,
  panelfun = panel.xyplot,
  plotfun = xyplot,
  ...,
  plot = trellis.last.object()
)
}
\arguments{
\item{x}{A formula specifying y ~ x or z ~ x&y}

\item{data}{Data frame containing the variables to be plotted.  If not specified,
the variables will be looked up in the local environment}

\item{add}{If \code{TRUE}, add points as a new layer to an existing plot.
If \code{NULL}, the value of \code{under} will be used.}

\item{under}{If \code{TRUE}, the new layer will be underneath existing layers.}

\item{panelfun}{Lattice panel function to be used for adding.  Set only if you want something other
than a scatter plot.  Mainly, this is intended to add new functionality through other functions.}

\item{plotfun}{Lattice function to be used for initial plot creation. Set only
if you want something other than a scatter plot. Mainly, this is intended to add new functionality through
other functions.}

\item{\dots}{additional arguments}

\item{plot}{a trellis plot, by default the most recently created one.  If \code{add} is \code{TRUE},
new points will be added as a new layer to \code{plot}.}
}
\value{
A trellis graphics object
}
\description{
Make or add a scatter plot in a manner coordinated with \code{plotFun}.
}
\examples{
if (require(mosaicData)) {
plotPoints( width ~ length, data=KidsFeet, groups=sex, pch=20)
f <- makeFun( lm( width ~ length * sex, data=KidsFeet))
plotFun( f(length=length,sex="G")~length, add=TRUE, col="pink")
plotFun( f(length=length,sex="B")~length, add=TRUE)
}
}
\seealso{
\code{\link[=plotFun]{plotFun()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.file.R
\name{read.file}
\alias{read.file}
\title{Read data files}
\usage{
read.file(
  file,
  header = T,
  na.strings = "NA",
  comment.char = NULL,
  filetype = c("default", "csv", "txt", "tsv", "fw", "rdata"),
  stringsAsFactors = FALSE,
  readr = FALSE,
  package = NULL,
  ...
)
}
\arguments{
\item{file}{character:
The name of the file which the data are to be read from.
This may also be a complete URL or a path to a compressed file.
If it does not contain an absolute path, the file name is
relative to the current working directory,
\code{getwd()}.  Tilde-expansion is performed where supported.
See \code{\link[=read.table]{read.table()}} for more details.}

\item{header}{logical;
For \code{.txt} and \code{.csv} files, this indicates whether the first line of the file includes variables names.}

\item{na.strings}{character: strings that indicate missing data.}

\item{comment.char}{character: a character vector of length one containing a single character or an empty string. Use "" to turn
off the interpretation of comments altogether.}

\item{filetype}{one of \code{"default"}, \code{"csv"}, \code{"txt"}, or \code{"rdata"}
indicating the type of file being loaded.  The default is to use the filename
to guess the type of file.}

\item{stringsAsFactors}{a logical indicating whether strings should be converted to factors.
This has no affect when using \code{readr}.}

\item{readr}{a logical indicating whether functions from the \code{readr} package should be
used, if available.}

\item{package}{if specified, files will be searched for among the documentation
files provided by the package.}

\item{\dots}{additional arguments passed on to
\code{\link[=read.table]{read.table()}}, or \code{\link[=load]{load()}} or one of the functions
in the \code{readr} package.  Note that a message will indicate which
underlying function is being used.}
}
\value{
A data frame, unless \code{file} unless \code{filetype} is \code{"rdata"},
in which  case arbitrary objects may be loaded and a character vector
holding the names of the loaded objects is returned invisibly.
}
\description{
A wrapper around various file reading functions.
}
\details{
Unless \code{filetype} is specified,
\code{read.file} uses the (case insensitive) file extension to determine how to read
data from the file.  If \code{file} ends in \code{.rda} or \code{.rdata}, then
\code{\link[=load]{load()}} is used to load the file.  If \code{file}
ends in \code{.csv}, then \code{\link[readr:read_csv]{readr::read_csv()}} or \code{\link[=read.csv]{read.csv()}} is used.
Otherwise, \code{\link[=read.table]{read.table()}} is used.
}
\examples{
\dontrun{
Dome <- read.file("http://www.mosaic-web.org/go/datasets/Dome.csv")
}
}
\seealso{
\code{\link[=read.csv]{read.csv()}}, \code{\link[=read.table]{read.table()}},
\code{\link[readr:read_table]{readr::read_table()}}, \code{\link[readr:read_csv]{readr::read_csv()}},
\code{\link[=load]{load()}}.
}
\keyword{util}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xhistogram.R
\name{xhistogramBreaks}
\alias{xhistogramBreaks}
\alias{histogram}
\alias{prepanel.xhistogram}
\alias{panel.xhistogram}
\title{Augmented histograms}
\usage{
xhistogramBreaks(x, center = NULL, width = NULL, nint, ...)

prepanel.xhistogram(x, breaks = xhistogramBreaks, ...)

panel.xhistogram(
  x,
  dcol = trellis.par.get("plot.line")$col,
  dalpha = 1,
  dlwd = 2,
  gcol = trellis.par.get("add.line")$col,
  glwd = 2,
  fcol = trellis.par.get("superpose.polygon")$col,
  dmath = dnorm,
  verbose = FALSE,
  dn = 100,
  args = NULL,
  labels = FALSE,
  density = NULL,
  under = FALSE,
  fit = NULL,
  start = NULL,
  type = "density",
  v,
  h,
  groups = NULL,
  center = NULL,
  width = NULL,
  breaks,
  nint = round(1.5 * log2(length(x)) + 1),
  stripes = c("vertical", "horizontal", "none"),
  alpha = 1,
  ...
)
}
\arguments{
\item{x}{a formula or a numeric vector}

\item{center}{center of one of the bins}

\item{width}{width of the bins}

\item{nint}{approximate number of bins}

\item{\dots}{additional arguments passed from \code{\link[lattice:histogram]{lattice::histogram()}}
to the panel function; by default when the \pkg{mosaic} package has been loaded this
will be \code{\link[=panel.xhistogram]{panel.xhistogram()}}.}

\item{breaks}{break points for histogram bins, a function for computing such,
or a method \code{\link[=hist]{hist()}} knows about given as a character string.
When using the \pkg{mosaic} package defaults,
\code{\link[=xhistogramBreaks]{xhistogramBreaks()}} is used.}

\item{dcol}{color of density curve}

\item{dalpha}{alpha for density curve}

\item{dlwd, glwd}{like \code{lwd} but affecting the density line and guide lines, respectively}

\item{gcol}{color of guidelines}

\item{fcol}{fill colors for histogram rectangles when using \code{groups}.
(Use \code{col}, which is passed through to the histogram panel function,
when not using \code{groups}.)}

\item{dmath}{density function for density curve overlay}

\item{verbose}{be verbose?}

\item{dn}{number of points to sample from density curve}

\item{args}{a list of additional arguments for \code{dmath}}

\item{labels}{should counts/densities/percents be displayed or each bin?}

\item{density}{a logical indicating whether to overlay a density curve}

\item{under}{a logical indicating whether the density layers should be under or
over other layers of the plot.}

\item{fit}{a character string describing the distribution to fit.  Known distributions include
\code{"exponential"}, \code{"normal"}, \code{"lognormal" }, \code{"poisson"}, \code{"beta"}, \code{"geometric"},
\code{"t"}, \code{"weibull"}, \code{"cauchy"}, \code{"gamma"}, \code{"chisq"}, and \code{"chi-squared"}}

\item{start}{numeric value passed to \code{\link[MASS:fitdistr]{MASS::fitdistr()}}}

\item{type}{one of \code{'density'}, \code{'count'}, or \code{'percent'}}

\item{h, v}{a vector of values for additional horizontal and vertical lines}

\item{groups}{as per \code{\link[lattice:histogram]{lattice::histogram()}}}

\item{stripes}{one of \code{"vertical"}, \code{"horizontal"}, or \code{"none"}, indicating
how bins should be striped when \code{groups} is not \code{NULL}}

\item{alpha}{transparency level}

\item{panel}{a panel function}
}
\value{
\code{xhistogramBreaks} returns a vector of break points
}
\description{
The \pkg{mosaic} package adds some additional functionality to
\code{\link[lattice:histogram]{lattice::histogram()}}, making it simpler to obtain certain common
histogram adornments.  This is done be resetting the default panel
and prepanel functions used by histogram.
}
\details{
The primary additional functionality added to \code{\link[=histogram]{histogram()}}
are the arguments \code{width} and \code{center} which provide a simple
way of describing equal-sized bins, and \code{fit} which can be used to
overlay the density curve for one of several distributions.  The
\code{groups} argument can be used to color the bins.  The primary use
for this is to shade tails of histograms, but there may be other uses
as well.
}
\note{
Versions of \pkg{lattice} since 0.20-21 support setting custom defaults
for \code{breaks}, \code{panel}, and \code{prepanel} used by
\code{histogram()}, so \code{xhistogram()} is no longer needed.
As a result, \code{xhistogram()}
(which was required in earlier versions of \pkg{mosaic}
is no longer needed and has been removed.
}
\examples{
if (require(mosaicData)) {
histogram(~age | substance, HELPrct, v=35, fit='normal')
histogram(~age, HELPrct, labels=TRUE, type='count')
histogram(~age, HELPrct, groups=cut(age, seq(10,80,by=10)))
histogram(~age, HELPrct, groups=sex, stripes='horizontal')
histogram(~racegrp, HELPrct, groups=substance,auto.key=TRUE)
xhistogramBreaks(1:10, center=5, width=1)
xhistogramBreaks(1:10, center=5, width=2)
xhistogramBreaks(0:10, center=15, width=3)
xhistogramBreaks(1:100, center=50, width=3)
xhistogramBreaks(0:10, center=5, nint=5)
}
}
\seealso{
\code{\link[lattice:histogram]{lattice::histogram()}}, \code{\link[=mosaicLatticeOptions]{mosaicLatticeOptions()}},
and \code{\link[=restoreLatticeOptions]{restoreLatticeOptions()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFun.R
\name{panel.levelcontourplot}
\alias{panel.levelcontourplot}
\title{Lattice plot that draws a filled contour plot}
\usage{
panel.levelcontourplot(
  x,
  y,
  z,
  subscripts = 1,
  at,
  shrink,
  labels = TRUE,
  label.style = c("mixed", "flat", "align"),
  contour = FALSE,
  region = TRUE,
  col = add.line$col,
  lty = add.line$lty,
  lwd = add.line$lwd,
  border = "transparent",
  ...,
  col.regions = regions$col,
  filled = TRUE,
  alpha.regions = regions$alpha
)
}
\arguments{
\item{x}{x on a grid}

\item{y}{y on a grid}

\item{z}{zvalues for the x and y}

\item{subscripts}{which points to plot}

\item{at}{cuts for the contours}

\item{shrink}{what does this do?}

\item{labels}{draw the contour labels}

\item{label.style}{where to put the labels}

\item{contour}{logical draw the contours}

\item{region}{logical color the regions}

\item{col}{color for contours}

\item{lty}{type for contours}

\item{lwd}{width for contour}

\item{border}{type of border}

\item{...}{dots additional arguments}

\item{col.regions}{a vector of colors or a function (\code{topo.colors} by default) for generating such}

\item{filled}{whether to fill the contours with color}

\item{alpha.regions}{transparency of regions}
}
\description{
Used within plotFun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/do.R
\name{set.rseed}
\alias{set.rseed}
\title{Set seed in parallel compatible way}
\usage{
set.rseed(seed)
}
\arguments{
\item{seed}{seed for the random number generator}
}
\description{
When the parallel package is used, setting the RNG seed for reproducibility
involves more than simply calling \code{\link[=set.seed]{set.seed()}}. \code{set.rseed} takes
care of the additional overhead.
}
\details{
If the \code{parallel} package is not on the search path, then \code{\link[=set.seed]{set.seed()}} is called.
If \code{parallel} is on the search path, then the RNG kind is set to \code{"L'Ecuyer-CMRG"},
the seed is set and \code{mc.reset.stream} is called.
}
\examples{
# These should give identical results, even if the `parallel' package is loaded.
set.rseed(123); do(3) * resample(1:10, 2)
set.rseed(123); do(3) * resample(1:10, 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xqqmath.R
\name{xqqmath}
\alias{xqqmath}
\alias{panel.xqqmath}
\title{Augmented version of \code{qqmath}}
\usage{
xqqmath(x, data = NULL, panel = "panel.xqqmath", ...)

panel.xqqmath(
  x,
  qqmathline = !(fitline || idline),
  idline = FALSE,
  fitline = NULL,
  slope = NULL,
  intercept = NULL,
  overlines = FALSE,
  groups = NULL,
  ...,
  col.line = trellis.par.get("add.line")$col,
  pch = 16,
  lwd = 2,
  lty = 2
)
}
\arguments{
\item{x, data, panel, \dots}{as in \code{\link[lattice:qqmath]{lattice::qqmath()}}}

\item{qqmathline}{a logical: should line be displayed passing through first and third quartiles?}

\item{idline}{a logical; should the line y=x be added to the plot?}

\item{fitline}{a logical; should a fitted line be added to plot?  Such a line will use \code{slope}
and \code{intercept} if provided, else the standard deviation and mean of the data.
If \code{slope} is specified, the line will be added unless \code{fitline} is
\code{FALSE}.}

\item{slope}{slope for added line}

\item{intercept}{intercept for added line}

\item{overlines}{a logical: should lines be on top of qq plot?}

\item{groups, pch, lwd, lty}{as in lattice plots}

\item{col.line}{color to use for added lines}
}
\value{
a trellis object
}
\description{
Augmented version of \code{qqmath}
}
\examples{
x <- rnorm(100)
xqqmath( ~ x)                  # with quartile line
xqqmath( ~ x, fitline = TRUE)  # with fitted line
xqqmath( ~ x, idline = TRUE)   # with y = x
x <- rexp(100, rate = 10)
xqqmath( ~ x, distribution = qexp)     # with quartile line
xqqmath( ~ x, distribution = qexp, slope = 1/10) 
xqqmath( ~ x, distribution = qexp, slope = mean(x)) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeo.R
\name{xyz2latlon}
\alias{xyz2latlon}
\alias{latlon2xyz}
\alias{lonlat2xyz}
\title{Convert back and forth between latitude/longitude and XYZ-space}
\usage{
xyz2latlon(x, y, z)

latlon2xyz(latitude, longitude)

lonlat2xyz(longitude, latitude)
}
\arguments{
\item{x, y, z}{numeric vectors}

\item{latitude, longitude}{vectors of latitude and longitude values}
}
\value{
a matrix each row of which describes the latitudes and longitudes

a matrix each row of which contains the x, y, and z coordinates of a point on a unit sphere
}
\description{
Convert back and forth between latitude/longitude and XYZ-space
}
\examples{
xyz2latlon(1, 1, 1)     # point may be on sphere of any radius
xyz2latlon(0, 0, 0)     # this produces a NaN for latitude
latlon2xyz(30, 45)
lonlat2xyz(45, 30)
}
\seealso{
\code{\link[=deg2rad]{deg2rad()}}, \code{\link[=googleMap]{googleMap()}}, and \code{\link[=rgeo]{rgeo()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.xtabs.R
\name{as.xtabs}
\alias{as.xtabs}
\alias{as.xtabs.data.frame}
\alias{as.xtabs.matrix}
\title{Convert objects to xtabs format}
\usage{
as.xtabs(x, ...)

\method{as.xtabs}{data.frame}(x, rowvar = NULL, colvar = NULL, labels = 1, ...)

\method{as.xtabs}{matrix}(x, rowvar = NULL, colvar = NULL, ...)
}
\arguments{
\item{x}{object (typically a data frame) to be converted to \code{xtabs}
format}

\item{...}{additional arguments to be passed to or from methods.}

\item{rowvar}{name of the row variable as character string}

\item{colvar}{name of the column variable as character string}

\item{labels}{column of data frame that contains the labels of the row
variable.}
}
\value{
An \code{xtabs} object.
}
\description{
Convert a data frame or a matrix into an \code{xtabs} object.
}
\details{
The intended use is to convert a two-way contingency table stored in a data
frame or a matrix into an \code{xtabs} object.
}
\examples{
# example from example(fisher.test)
df <- data.frame( X=c('Tea','Milk'), Tea=c(3,1), Milk=c(1,3) )
xt <- as.xtabs(df, rowvar="Guess", colvar="Truth"); xt
if (require(vcd)) { mosaic(xt) }
}
\keyword{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFun.R
\name{panel.plotFun}
\alias{panel.plotFun}
\title{Panel function for plotting functions}
\usage{
panel.plotFun(
  object,
  ...,
  type = "l",
  npts = NULL,
  zlab = NULL,
  filled = TRUE,
  levels = NULL,
  nlevels = 10,
  surface = FALSE,
  col.regions = topo.colors,
  lwd = trellis.par.get("superpose.line")$lwd,
  lty = trellis.par.get("superpose.line")$lty,
  alpha = NULL,
  discontinuity = NULL,
  discontinuities = NULL
)
}
\arguments{
\item{object}{an object (e.g., a formula) describing a function}

\item{...}{additional arguments, typically processed by \code{lattice} panel functions
such as \code{\link[lattice:panel.xyplot]{lattice::panel.xyplot()}} or \code{\link[lattice:panel.levelplot]{lattice::panel.levelplot()}}.
Frequently used arguments include
\describe{
\item{\code{lwd}}{line width}
\item{\code{lty}}{line type}
\item{\code{col}}{a color}
}}

\item{type}{type of plot (\code{"l"} by default)}

\item{npts}{an integer giving the number of points (in each dimension) to sample the function}

\item{zlab}{label for z axis (when in surface-plot mode)}

\item{filled}{fill with color between the contours (\code{TRUE} by default)}

\item{levels}{levels at which to draw contours}

\item{nlevels}{number of contours to draw (if \code{levels} not specified)}

\item{surface}{a logical indicating whether to draw a surface plot rather than a contour plot}

\item{col.regions}{a vector of colors or a function (\code{topo.colors} by default) for generating such}

\item{lwd}{width of the line}

\item{lty}{line type}

\item{alpha}{number from 0 (transparent) to 1 (opaque) for the fill colors}

\item{discontinuity}{a positive number determining how sensitive the plot is to
potential discontinuity.  Larger values result in less sensitivity.  The default is 1.
Use \code{discontinuity = Inf} to disable discontinuity detection.  Discontinuity detection
uses a crude numerical heuristic and may not give the desired results in all cases.}

\item{discontinuities}{a vector of input values at which a function is
discontinuous or \code{NULL} to use
a heuristic to auto-detect.}
}
\description{
Panel function for plotting functions
}
\examples{
x <- runif(30,0,2*pi) 
d <- data.frame( x = x,  y = sin(x) + rnorm(30,sd=.2) )
xyplot( y ~ x, data=d )
ladd(panel.plotFun( sin(x) ~ x, col='red' ) )
xyplot( y ~ x | rbinom(30,1,.5), data=d )
ladd(panel.plotFun( sin(x) ~ x, col='red', lty=2 ) )    # plots sin(x) in each panel
}
\seealso{
plotFun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mPlots.R
\name{getVarFormula}
\alias{getVarFormula}
\title{Extract data from a data frame using a formula interface}
\usage{
getVarFormula(formula, data = parent.frame(), intercept = FALSE)
}
\arguments{
\item{formula}{a formula.
The right-hand side selects variables;
the left-hand side, if present, is used to set row names.
A \code{.} on the right-hand side
indicates to use all variables not in the LHS.}

\item{data}{a data frame}

\item{intercept}{a logical indicating whether to include the intercept in the
model default: FALSE (no intercept)}
}
\description{
Uses the full model syntax.
}
\examples{
getVarFormula( ~ wt + mpg, data = mtcars)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resample.R
\name{rspin}
\alias{rspin}
\title{Simulate spinning a spinner}
\usage{
rspin(n, probs, labels = 1:length(probs))
}
\arguments{
\item{n}{number of spins of spinner}

\item{probs}{a vector of probabilities.  If the sum is not 1, the
probabilities will be rescaled.}

\item{labels}{a character vector of labels for the categories}
}
\description{
This is essentially \code{rmultinom} with a different interface.
}
\examples{
rspin(20, prob=c(1,2,3), labels=c("Red", "Blue", "Green"))
do(2) * rspin(20, prob=c(1,2,3), labels=c("Red", "Blue", "Green"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingMeta.R
\name{aggregatingFunction1or2}
\alias{aggregatingFunction1or2}
\title{1- or 2-ary aggregating functions}
\usage{
aggregatingFunction1or2(
  fun,
  output.multiple = FALSE,
  na.rm = getOption("na.rm", FALSE)
)
}
\arguments{
\item{fun}{a function that takes 1 or 2 numeric vectors and computes a summary statistic,
returning a numeric vector of length 1.}

\item{output.multiple}{a boolean indicating whether \code{fun} returns multiple values}

\item{na.rm}{the default value for na.rm in the resulting function.}
}
\description{
\code{aggregatingFunction1or2()} creates statistical summaries for functions like
\code{\link[=var]{var()}} that can have either 1 or 2 numeric vector inputs.
}
\details{
This was designed primarily to support \code{var} which can be used to compute
either the variance of one variable or the covariance of two variables.
The logic of the resulting function is this: 1) If the first two arguments are both formulas,
then those formulas are evaluated (with \code{data}) to compute the covariance;
(2) If the first argument is a formula, and the second is \code{NULL},
then the formula and \code{data} are used to create the necessary call(s) to \code{fun};
(3) Else everything is simply passed to \code{fun} for evaluation.
}
\section{Note}{

Earlier versions of this function supported a "bare name + data frame" interface.  This
functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary,
and (c) difficult to maintain.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findZeros.R
\name{findZeros}
\alias{findZeros}
\alias{solve.formula}
\title{Find zeros of functions}
\usage{
findZeros(
  expr,
  ...,
  xlim = c(near - within, near + within),
  near = 0,
  within = Inf,
  nearest = 10,
  npts = 1000,
  iterate = 1,
  sortBy = c("byx", "byy", "radial")
)

\method{solve}{formula}(
  form,
  ...,
  near = 0,
  within = Inf,
  nearest = 10,
  npts = 1000,
  iterate = 1,
  sortBy = c("byx", "byy", "radial")
)
}
\arguments{
\item{expr}{A formula.  The right side names the variable with respect to which the zeros should be found.
The left side is an expression, e.g. \code{sin(x) ~ x}.
All free variables (all but the variable on the right side) named in the expression must be assigned
a value via \verb{\\ldots}}

\item{\dots}{Formulas corresponding to additional functions to use in simultaneous zero finding
and/or specific numerical values for the free variables in the expression.}

\item{xlim}{The range of the dependent variable to search for zeros. \code{Inf} is a legitimate value,
but is interpreted in the numerical sense as the non-Inf largest floating point number.  This can also
be specified replacing \code{x} with the name of the variable.  See the examples.}

\item{near}{a value near which zeros are desired}

\item{within}{only look for zeros at least this close to near.  \code{near} and \code{within} provide an
alternative to using \code{xlim} to specify the search space.}

\item{nearest}{the number of nearest zeros to return.  Fewer are returned if fewer are found.}

\item{npts}{How many sub-intervals to divide the \code{xlim} into when looking for candidates for zeros.
The default is usually good enough.
If \code{Inf} is involved, the intervals are logarithmically spaced up to the largest finite floating point number.
There is no guarantee that all the roots will be found.}

\item{iterate}{maximum number of times to iterate the search. Subsequent searches take place with the range
of previously found zeros.  Choosing a large number here is likely to kill performance without
improving results, but a value of 1 (the default) or 2 works well when searching in \code{c(-Inf,Inf)} for
a modest number of zeros near \code{near}.}

\item{sortBy}{specifies how the zeros found will be sorted. Options are 'byx', 'byy', or 'radial'.}

\item{form}{Expression to be solved}
}
\value{
A dataframe of zero or more numerical values.  Plugging these into the
expression on the left side of the formula should result in values near zero.

a dataframe with solutions to the expression.
}
\description{
Compute numerically zeros of a function or simultaneous zeros of multiple functions.
}
\details{
Searches numerically using \code{uniroot}.

Uses findZerosMult of findZeros to solve the given expression
}
\examples{
findZeros( sin(t) ~ t, xlim=c(-10,10) )
# Can use tlim or t.lim instead of xlim if we prefer
findZeros( sin(t) ~ t, tlim=c(-10,10) )
findZeros( sin(theta) ~ theta, near=0, nearest=20)
findZeros( A*sin(2*pi*t/P) ~ t, xlim=c(0,100), P=50, A=2)
# Interval of a normal at half its maximum height.
findZeros( dnorm(x,mean=0,sd=10) - 0.5*dnorm(0,mean=0,sd=10) ~ x )
# A pathological example
# There are no "neareset" zeros for this function.  Each iteration finds new zeros.
f <- function(x) { if (x==0) 0 else sin(1/x) }
findZeros( f(x) ~ x, near=0 )
# Better to look nearer to 0
findZeros( f(x) ~ x, near=0, within=100 )
findZeros( f(x) ~ x, near=0, within=100, iterate=0 )
findZeros( f(x) ~ x, near=0, within=100, iterate=3 )
# Zeros in multiple dimensions (not run: these take a long time)
# findZeros(x^2+y^2+z^2-5~x&y&z, nearest=3000, within = 5)
# findZeros(x*y+z^2~z&y&z, z+y~x&y&z, npts=10)
solve(3*x==3~x)

# plot out sphere (not run)
# sphere = solve(x^2+y^2+z^2==5~x&y&z, within=5, nearest=1000)
# cloud(z~x+y, data=sphere)
}
\author{
Daniel Kaplan (\email{kaplan@macalester.edu})

Cecylia Bocovich
}
\keyword{calculus}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Clustering.R
\name{fortify.hclust}
\alias{fortify.hclust}
\alias{mplot.hclust}
\title{mosaic tools for clustering}
\usage{
\method{fortify}{hclust}(
  model,
  data,
  which = c("segments", "heatmap", "leaves", "labels", "data"),
  k = 1,
  ...
)

\method{mplot}{hclust}(
  object,
  data,
  colorize = TRUE,
  k = 1,
  labels = FALSE,
  heatmap = 0,
  enumerate = "white",
  ...
)
}
\arguments{
\item{model}{a model}

\item{data}{a data-like object}

\item{which}{which kind of fortification to compute}

\item{k}{number of clusters}

\item{...}{additional arguments passed on to \code{link{dendro_data}}}

\item{object}{an object of class \code{"hclust"}}

\item{colorize}{whether to show clusters in different colors}

\item{labels}{a logical indicating whether labels should be used
to identify leaves of the tree.}

\item{heatmap}{the ratio of size of heatmap to size of dendrogram.
Use \code{0} or \code{FALSE} to omit the heatmap.}

\item{enumerate}{a color used for numbers within heatmap.  Use
\code{"transparent"} to hide.}
}
\description{
mosaic tools for clustering
}
\examples{
KidsFeet |> select(-name, -birthmonth) |> rescale() -> KidsFeet2
  M <- dist(KidsFeet2)
  Cl <- hclust(M)
  fortify(Cl, k=5) |> head(3)
  fortify(Cl, which="heatmap", data=KidsFeet2) |> head(3)
  fortify(Cl, which="data", data=KidsFeet2) |> head(3)
  fortify(Cl, which="labels") |> head(3)
  mplot(Cl, data=KidsFeet2, k=4, heatmap=2)
  mplot(Cl, data=KidsFeet2, k=4, heatmap=0.5, enumerate="transparent")
  mplot(Cl, data=KidsFeet2, k=4, heatmap=2, type="triangle")
  mplot(Cl, data=KidsFeet2, k=4, heatmap=0, type="triangle")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xchisq.test.R
\name{xchisq.test}
\alias{xchisq.test}
\title{Augmented Chi-squared test}
\usage{
xchisq.test(
  x,
  y = NULL,
  correct = TRUE,
  p = rep(1/length(x), length(x)),
  rescale.p = FALSE,
  simulate.p.value = FALSE,
  B = 2000,
  data = environment(x)
)
}
\arguments{
\item{x, y, correct, p, rescale.p, simulate.p.value, B}{as in \code{\link[=chisq.test]{chisq.test()}}, but
\code{x} may also be a formula, in which case \code{x} is replaced by \code{tally(x, data)}
prior to the call to \code{\link[=chisq.test]{chisq.test()}}.}

\item{data}{a data frame for use when \code{x} is a formula.}
}
\description{
This augmented version of \code{\link[=chisq.test]{chisq.test()}} provides more verbose
output.
}
\examples{
# Physicians' Health Study data
phs <- cbind(c(104,189),c(10933,10845)) 
rownames(phs) <- c("aspirin","placebo") 
colnames(phs) <- c("heart attack","no heart attack") 
phs 
xchisq.test(phs) 
xchisq.test(sex ~ substance, data = HELPrct)
}
\seealso{
\code{\link[=chisq.test]{chisq.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop.test.R
\name{prop.test}
\alias{prop.test}
\title{Exact and Approximate Tests for Proportions}
\usage{
prop.test(
  x,
  n,
  p = NULL,
  alternative = c("two.sided", "less", "greater"),
  conf.level = 0.95,
  data = NULL,
  success = NULL,
  ...
)
}
\arguments{
\item{x}{count of successes, length 2 vector of success and failure counts, a formula,
or a character, numeric, or factor vector containing raw data.}

\item{n}{sample size (successes + failures) or a data frame
(for the formula interface)}

\item{p}{a vector of probabilities of success.
The length of p must be the same as the number of groups specified by x,
and its elements must be greater than 0 and less than 1.}

\item{alternative}{character string specifying the alternative hypothesis, must be one of
\code{"two.sided"} (default), \code{"greater"} or \code{"less"}. You can specify just the initial letter.
Only used for testing the null that a single proportion equals a given value, or that two proportions
are equal; ignored otherwise.}

\item{conf.level}{confidence level of the returned confidence interval. Must be a single number
between 0 and 1. Only used when testing the null that a single proportion equals a given value,
or that two proportions are equal; ignored otherwise.}

\item{data}{a data frame (if missing, \code{n} may be a data frame)}

\item{success}{level of variable to be considered success.  All other levels are
considered failure.}

\item{...}{additional arguments (often ignored).
When \code{x} is a formula, \code{groups} can be used to compare groups:
\verb{x = ~ var, groups=g} is equivalent to \code{x = var ~ g}. \code{na.rm} can be a logical
or an integer vector of length 1 or 2 to indicate dimension along which NA's are
removed before coputing the test.
See the examples.}
}
\value{
an \code{htest} object
}
\description{
The mosaic \code{prop.test} provides wrapper functions around the function of the same name in \pkg{stats}.
These wrappers provide an extended interface (including formulas).
\code{prop.test} performs an approximate test of a simple null hypothesis about the
probability of success in a Bernoulli or multinomial experiment
from summarized data or from raw data.
}
\details{
\if{html}{\out{<div class="sourceCode">}}\preformatted{				conf.level = 0.95, ...)
}\if{html}{\out{</div>}}

This is a wrapper around \code{\link[=prop.test]{prop.test()}} to simplify its use
when the raw data are available, in which case
an extended syntax for \code{prop.test} is provided.
}
\note{
When \code{x} is a 0-1 vector, 0 is treated as failure and 1 as success. Similarly,
for a logical vector \code{TRUE} is treated as success and \code{FALSE} as failure.
}
\examples{
# Several ways to get a confidence interval for the proportion of Old Faithful
# eruptions lasting more than 3 minutes.
prop.test( faithful$eruptions > 3 )
prop.test(97,272)
faithful$long <- faithful$eruptions > 3
prop.test( faithful$long )
prop.test( ~long , data = faithful )
prop.test( homeless ~ sex, data = HELPrct )
prop.test( ~ homeless | sex, data = HELPrct )
prop.test( ~ homeless, groups = sex, data = HELPrct )
prop.test(anysub ~ link, data = HELPrct, na.rm = TRUE)
prop.test(link ~ anysub, data = HELPrct, na.rm = 1)
prop.test(link ~ anysub, data = HELPrct, na.rm = TRUE)

}
\seealso{
\code{\link[=binom.test]{binom.test()}}, \code{\link[stats:prop.test]{stats::prop.test()}}
}
\keyword{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingMeta.R
\name{aggregatingFunction1}
\alias{aggregatingFunction1}
\title{1-ary Aggregating functions}
\usage{
aggregatingFunction1(
  fun,
  output.multiple = FALSE,
  envir = parent.frame(),
  na.rm = getOption("na.rm", FALSE),
  style = c("formula1st", "formula", "flexible")
)
}
\arguments{
\item{fun}{a function that takes a numeric vector and computes a summary statistic,
returning a numeric vector.}

\item{output.multiple}{a boolean indicating whether \code{fun} returns multiple values}

\item{envir}{an environment in which evaluation takes place.}

\item{na.rm}{the default value for na.rm in the resulting function.}

\item{style}{one of \code{"formula1st"}, \code{"formula2nd"} or \code{"flexible"}.  In the first
two cases, the first argument must be a formula or evaluate to an object.  In the latter case,
bare names will be converted into formulas.}
}
\value{
a function that generalizes \code{fun} to handle a formula/data frame interface.
}
\description{
\code{aggregatinFuntion1} creates statistical summaries of one numerical vector that are formula aware.
}
\details{
The logic of the resulting function is this: 1) If the first argument is a formula,
use that formula and \code{data} to create the necessary call(s) to \code{fun}; (2) Else simply
pass everything to \code{fun} for evaluation.
}
\section{Note}{

Earlier versions of this function supported a "bare name + data frame" interface.  This
functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary, and (c)
difficult to maintain.
}

\examples{
if (require(mosaicData)) {
  foo <- aggregatingFunction1(base::mean)
  foo( ~ length, data = KidsFeet)
  base::mean(KidsFeet$length)
  foo(length ~ sex, data = KidsFeet)
} 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{makeMap}
\alias{makeMap}
\title{Make a map with \code{ggplot2}}
\usage{
makeMap(
  data = NULL,
  map = NULL,
  key = c(key.data, key.map),
  key.data,
  key.map,
  tr.data = identity,
  tr.map = identity,
  plot = c("borders", "frame", "none")
)
}
\arguments{
\item{data}{A dataframe with regions as cases}

\item{map}{An object that can be fortified to a dataframe (ex: a dataframe itself,
or a SpatialPolygonsDataFrame)}

\item{key}{The combination of \code{key.data} and \code{key.map}}

\item{key.data}{The column name in the \code{data} that holds the
unique names of each region}

\item{key.map}{The column name in the \code{map} that holds the
unique names of each region}

\item{tr.data}{A function of the transformation to be performed to
the \code{key.data} column}

\item{tr.map}{A function of the transformation to be performed to
the \code{key.map} column}

\item{plot}{The plot desired for the output. \code{plot} = "none"
returns the merged data that is the result of merging the \code{data}
and \code{map} together; \code{plot}="frame" returns an empty
(unplottable) ggplot object; \code{plot} = "border" (the default)
returns a ggplot object with one geom_polygon layer that shows the
borders of the regions.}
}
\description{
\code{makeMap} takes in two sources of data that refer to geographical
regions and merges them together. Depending on the arguments passed,
it returns this merged data or a ggplot object constructed with the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeo.R
\name{rgeo-internals}
\alias{rgeo-internals}
\alias{.googleMapURL}
\title{rgeo internal functions}
\usage{
.googleMapURL(
  latitude,
  longitude,
  position = NULL,
  zoom = 11,
  maptype = c("roadmap", "satellite", "terrain", "hybrid"),
  mark = FALSE,
  radius = 0
)
}
\arguments{
\item{latitude, longitude}{vectors of latitude and longitude values}

\item{position}{a data frame containing latitude and longitude positions}

\item{zoom}{zoom level for initial map (1-20)}

\item{maptype}{one of \code{'roadmap'}, \code{'satellite'}, \code{'terrain'}, and \code{'hybrid'}}

\item{mark}{a logical indicating whether the location should be marked with a pin}

\item{radius}{a vector of radii of circles centered at position that are displayed on the map}
}
\value{
a URL as a string
}
\description{
These are not really intended for public consumption.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expandFun.R
\name{expandFun}
\alias{expandFun}
\title{Expand the left-hand side of a formula}
\usage{
expandFun(formula, ...)
}
\arguments{
\item{formula}{A mathematical expression (see examples and \code{\link[=plotFun]{plotFun()}})}

\item{\ldots}{additional parameters}
}
\value{
A list with the new expanded formula and the combined formals
}
\description{
Expands the contents of functions used in a formula.
}
\examples{
f=makeFun(x^2~x)
expandFun(f(z)~z) #Returns z^2~z
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.wholenumber.R
\name{is.wholenumber}
\alias{is.wholenumber}
\title{Check for whole number values}
\usage{
is.wholenumber(x, tol = .Machine$double.eps^0.5)
}
\arguments{
\item{x}{a vector}

\item{tol}{a numeric tolerance}
}
\value{
a logical vector indicating whether \code{x} has a whole number value
}
\description{
Unlike \code{\link[=is.integer]{is.integer()}}, which checks the type of argument is \code{integer},
this function checks whether the value of the argument is an integer
(within a specified tolerance).
}
\details{
This function is borrowed from the examples for \code{\link[=is.integer]{is.integer()}}
}
\examples{
is.wholenumber(1)
all(is.wholenumber(rbinom(100,10,.5)))
is.wholenumber((1:10)/2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingMeta.R
\name{aggregatingFunction2}
\alias{aggregatingFunction2}
\title{2-ary aggregating functions}
\usage{
aggregatingFunction2(fun)
}
\arguments{
\item{fun}{a function that takes two numeric vectors and computes a summary statistic,
returning a numeric vector of length 1.}
}
\value{
a function that generalizes \code{fun} to handle a formula/data frame interface.
}
\description{
\code{aggregatinFuntion2} creates statistical summaries of two numerical vectors that
are formula aware.
}
\details{
This was designed to support functions like \code{cov()} which can be used to compute
numerical summaries from two numeric vectors.
The logic of the resulting function is this: 1) If the first two arguments are both formulas,
then those formulas are evaluated (with \code{data}) to compute the covariance;
(2) If the first argument is a formula, and the second is \code{NULL},
then the left and ride sides of the formula and \code{data} are used to create the
vectors passed to \code{fun};
(3) Else everything is simply passed to \code{fun} for evaluation.
}
\section{Note}{

Earlier versions of this function supported a "bare name + data frame" interface.  This
functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary,
and (c) difficult to maintain.
}

\examples{
if(require(mosaicData)) {
  foo <- aggregatingFunction2(stats::cor)
  foo(length ~ width, data = KidsFeet)
  stats::cor(KidsFeet$length, KidsFeet$width)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statTally.R
\name{statTally}
\alias{statTally}
\title{Tally test statistics}
\usage{
statTally(
  sample,
  rdata,
  FUN,
  direction = NULL,
  alternative = c("default", "two.sided", "less", "greater"),
  sig.level = 0.1,
  system = c("gg", "lattice"),
  shade = "navy",
  alpha = 0.1,
  binwidth = NULL,
  bins = NULL,
  fill = "gray80",
  color = "black",
  center = NULL,
  stemplot = dim(rdata)[direction] < 201,
  q = c(0.5, 0.9, 0.95, 0.99),
  fun = function(x) x,
  xlim,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{sample}{sample data}

\item{rdata}{a matrix of randomly generated data under
null hypothesis.}

\item{FUN}{a function that computes the test statistic from
a data set.  The default value does nothing, making it easy to
use this to tabulate precomputed statistics into a null distribution.
See the examples.}

\item{direction}{1 or 2 indicating whether samples in \code{rdata}
are in rows (1) or columns (2).}

\item{alternative}{one of \code{default}, \code{two.sided}, \code{less}, or \code{greater}}

\item{sig.level}{significance threshold for \code{wilcox.test} used to detect lack of symmetry}

\item{system}{graphics system to use for the plot}

\item{shade}{a color to use for shading.}

\item{alpha}{opacity of shading.}

\item{binwidth}{bin width for histogram.}

\item{bins}{number of bins for histogram.}

\item{fill}{fill color for histogram.}

\item{color}{border color for histogram.}

\item{center}{center of null distribution}

\item{stemplot}{indicates whether a stem plot should be displayed}

\item{q}{quantiles of sampling distribution to display}

\item{fun}{same as \code{FUN} so you don't have to remember if it
should be capitalized}

\item{xlim}{limits for the horizontal axis of the plot.}

\item{quiet}{a logicial indicating whether the text output should be suppressed}

\item{\dots}{additional arguments passed to \code{\link[lattice:histogram]{lattice::histogram()}} or \code{\link[ggplot2:geom_histogram]{ggplot2::geom_histogram()}}}
}
\value{
A lattice or ggplot showing the sampling distribution.

As side effects, information
about the empirical sampling distribution and (optionally) a stem plot are
printed to the screen.
}
\description{
Tally test statistics from data and from multiple draws
from a simulated null distribution
}
\examples{
# is my spinner fair?
x <- c(10, 18, 9, 15)   # counts in four cells
rdata <- rmultinom(999, sum(x), prob = rep(.25, 4))
statTally(x, rdata, fun = max, binwidth = 1)  # unusual test statistic
statTally(x, rdata, fun = var, shade = "red", binwidth = 2)  # equivalent to chi-squared test
# Can also be used with test stats that are precomputed.
if (require(mosaicData)) {
D <- diffmean( age ~ sex, data = HELPrct); D
nullDist <- do(999) * diffmean( age ~ shuffle(sex), data = HELPrct)
statTally(D, nullDist)
statTally(D, nullDist, system = "lattice")
}

}
\keyword{inference}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resample.R
\name{relm}
\alias{relm}
\title{Resample a Linear Model}
\usage{
relm(model, ..., envir = environment(formula(model)))
}
\arguments{
\item{model}{a linear model object produced using \code{\link[=lm]{lm()}}.}

\item{...}{additional arguments passed through to \code{\link[=resample]{resample()}}.}

\item{envir}{an environment in which to (re)evaluate the linear model.}
}
\description{
Fit a new model to data created using \code{resample(model)}.
}
\examples{
mod <- lm(length ~ width, data = KidsFeet)
do(1) * mod 
do(3) * relm(mod) 
# use residual resampling to estimate standard error (very crude because so few replications)
Boot <- do(100) * relm(mod)
sd(~ width, data = Boot)
# standard error as produced by summary() for comparison
mod |> summary() |> coef() 

}
\seealso{
\code{resample()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingMeta.R
\name{mean_}
\alias{mean_}
\alias{sum}
\alias{min}
\alias{max}
\alias{mean}
\alias{median}
\alias{sd}
\alias{var}
\alias{cov}
\alias{cor}
\alias{favstats}
\alias{range}
\alias{IQR}
\alias{fivenum}
\alias{iqr}
\alias{prod}
\alias{quantile}
\title{Aggregating functions}
\usage{
mean_(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

mean(x, ...)

median(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

range(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

sd(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

max(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

min(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

sum(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

IQR(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

fivenum(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

iqr(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

prod(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

sum(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

favstats(x, ..., data = NULL, groups = NULL, na.rm = TRUE)

quantile(x, ..., data = NULL, groups = NULL, na.rm = getOption("na.rm", FALSE))

var(x, y = NULL, na.rm = getOption("na.rm", FALSE), ..., data = NULL)

cor(x, y = NULL, ..., data = NULL)

cov(x, y = NULL, ..., data = NULL)
}
\arguments{
\item{x}{a numeric vector or a formula}

\item{\dots}{additional arguments}

\item{data}{a data frame in which to evaluate formulas (or bare names).
Note that the default is \code{data = parent.frame()}.  This makes it convenient to
use this function interactively by treating the working environment as if it were
a data frame.  But this may not be appropriate for programming uses.
When programming, it is best to use an explicit \code{data} argument
-- ideally supplying a data frame that contains the variables mentioned.}

\item{groups}{a grouping variable, typically a name of a variable in \code{data}}

\item{na.rm}{a logical indicating whether \code{NA}s should be removed before computing}

\item{y}{a numeric vector or a formula}
}
\description{
The \code{mosaic} package makes several summary statistic functions (like \code{mean} and \code{sd})
formula aware.
}
\details{
Many of these functions mask core R functions to provide an additional formula
interface.  Old behavior should be unchanged.  But if the first argument is a formula,
that formula, together with \code{data} are used to generate the numeric vector(s)
to be summarized.  Formulas of the shape \code{x ~ a} or \code{~ x | a} can be used to
produce summaries of \code{x} for each subset defined by \code{a}.  Two-way aggregation
can be achieved using formulas of the form \code{x ~ a + b} or \code{ x ~ a | b}.  See
the examples.
}
\section{Note}{

Earlier versions of these functions supported a "bare name + data frame" interface.  This
functionality has been removed since it was (a) ambiguous in some cases, (b) unnecessary,
and (c) difficult to maintain.
}

\examples{
mean(HELPrct$age)
mean( ~ age, data = HELPrct)
mean( ~ drugrisk, na.rm = TRUE, data = HELPrct)
mean(age ~ shuffle(sex), data = HELPrct)
mean(age ~ shuffle(sex), data = HELPrct, .format = "table")
# wrap in data.frame() to auto-convert awkward variable names
data.frame(mean(age ~ shuffle(sex), data = HELPrct, .format = "table"))
mean(age ~ sex + substance, data = HELPrct)
mean( ~ age | sex + substance, data = HELPrct)
mean( ~ sqrt(age), data = HELPrct)
sum( ~ age, data = HELPrct)
sd(HELPrct$age)
sd( ~ age, data = HELPrct)
sd(age ~ sex + substance, data = HELPrct)
var(HELPrct$age)
var( ~ age, data = HELPrct)
var(age ~ sex + substance, data = HELPrct)
IQR(width ~ sex, data = KidsFeet)
iqr(width ~ sex, data = KidsFeet)
favstats(width ~ sex, data = KidsFeet)

cor(length ~ width, data = KidsFeet)
cov(length ~ width, data = KidsFeet)
tally(is.na(mcs) ~ is.na(pcs), data = HELPmiss)
cov(mcs ~ pcs, data = HELPmiss)             # NA because of missing data
cov(mcs ~ pcs, data = HELPmiss, use = "complete")  # ignore missing data
# alternative approach using filter explicitly
cov(mcs ~ pcs, data = HELPmiss |> filter(!is.na(mcs) & !is.na(pcs)))    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ntiles.R
\name{ntiles}
\alias{ntiles}
\title{Create vector based on roughly equally sized groups}
\usage{
ntiles(
  x,
  n = 3,
  format = c("rank", "interval", "mean", "median", "center", "left", "right"),
  digits = 3
)
}
\arguments{
\item{x}{a numeric vector}

\item{n}{(approximate) number of quantiles}

\item{format}{a specification of desired output format.}

\item{digits}{desired number of digits for labeling of factors.}
}
\value{
a vector.  The type of vector will depend on \code{format}.
}
\description{
Create vector based on roughly equally sized groups
}
\examples{
if (require(mosaicData)) {
tally( ~ ntiles(age, 4), data=HELPrct)
tally( ~ ntiles(age, 4, format="center"), data=HELPrct)
tally( ~ ntiles(age, 4, format="interval"), data=HELPrct)
tally( ~ ntiles(age, 4, format="left"), data=HELPrct)
tally( ~ ntiles(age, 4, format="right"), data=HELPrct)
tally( ~ ntiles(age, 4, format="mean"), data=HELPrct)
tally( ~ ntiles(age, 4, format="median"), data=HELPrct)
bwplot( i2 ~ ntiles(age, n=5, format="interval"), data=HELPrct)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Mustangs.R
\docType{data}
\name{Mustangs}
\alias{Mustangs}
\title{Mustang Prices}
\format{
A data frame with 25 observations on the following 3 variables.
\describe{
\item{\code{Age }}{age of vehicle in years}
\item{\code{Miles }}{1000s of miles driven}
\item{\code{Price }}{selling price in 1000s USD}
}
}
\source{
These data were used in a "resampling bake-off" hosted by Robin Lock.
}
\usage{
data(Mustangs)
}
\description{
Mustang Prices
}
\details{
#' @docType data

A student collected data on the selling prices for a sample of used Mustang
cars being offered for sale at an internet website.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rstudio_is_available}
\alias{rstudio_is_available}
\title{Check whether RStudio is in use}
\usage{
rstudio_is_available()
}
\value{
a logical
}
\description{
This functions checks that RStudio is in use.  It will likely be removed
from this package once the versions of RStudio in popular use rely on the
manipulate package on CRAN which will provide its own version.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dotPlot.R
\name{dotPlot}
\alias{dotPlot}
\alias{panel.dotPlot}
\title{Dotplots}
\usage{
dotPlot(x, breaks, ..., panel = panel.dotPlot)

panel.dotPlot(
  x,
  breaks,
  equal.widths = TRUE,
  groups = NULL,
  nint = if (is.factor(x)) nlevels(x) else round(1.3 * log2(length(x)) + 4),
  pch,
  col,
  lty = trellis.par.get("dot.line")$lty,
  lwd = trellis.par.get("dot.line")$lwd,
  col.line = trellis.par.get("dot.line")$col,
  alpha = trellis.par.get("dot.symbol")$alpha,
  cex = 1,
  type = "count",
  ...
)
}
\arguments{
\item{x}{a vector of values or a formula}

\item{breaks, equal.widths, groups, pch, col, lty, lwd, col.line, type, alpha}{as in \code{\link[=histogram]{histogram()}}}

\item{\dots}{additional arguments}

\item{panel}{a panel function}

\item{nint}{the number of intervals to use}

\item{cex}{a ratio by which to increase or decrease the dot size}
}
\value{
a trellis object
}
\description{
A high level function and panel function for producing
a variant of a histogram called a dotplot.
}
\examples{
if (require(mosaicData)) {
dotPlot( ~ age, data = HELPrct)
dotPlot( ~ age, nint=42, data = HELPrct)
dotPlot( ~ height | voice.part, data = singer, nint = 17,
          endpoints = c(59.5, 76.5), layout = c(4,2), aspect = 1,
          xlab = "Height (inches)")
}
}
\seealso{
\code{\link[=histogram]{histogram()}}
}
\keyword{graphics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{mWorldMap}
\alias{mWorldMap}
\title{Make a world map with \code{ggplot2}}
\usage{
mWorldMap(
  data = NULL,
  key = NA,
  fill = NULL,
  plot = c("borders", "frame", "none")
)
}
\arguments{
\item{data}{A dataframe with countries as cases}

\item{key}{The column name in the \code{data} that holds
the unique names of each country}

\item{fill}{A variable in the \code{data} used to specify the fill
color of countries in the map (note: if \code{fill} is not null, then
\code{plot} cannot be set to "none")}

\item{plot}{The plot desired for the output. \code{plot} = "none"
returns the merged data that is the result of merging the \code{data}
and the dataframe with the geographical coordinate information;
\code{plot} = "frame" returns an empty (unplottable) ggplot object;
\code{plot} = "border" (the default) returns a ggplot object with
one geom_polygon layer that shows the borders of the countries}
}
\description{
\code{mWorldMap} takes in one dataframe that includes information
about different countries. It merges this dataframe with a dataframe
that includes geographical coordinate information. Depending on the
arguments passed, it returns this data or a ggplot object constructed
with the data.
}
\examples{
\dontrun{
gdpData <- CIAdata("GDP")      # load some world data

mWorldMap(gdpData, key="country", fill="GDP")

gdpData <- gdpData |> mutate(GDP5 = ntiles(-GDP, 5, format="rank")) 
mWorldMap(gdpData, key="country", fill="GDP5")

mWorldMap(gdpData, key="country", plot="frame") +
geom_point()

mergedData <- mWorldMap(gdpData, key="country", plot="none")

ggplot(mergedData, aes(x=long, y=lat, group=group, order=order)) +
geom_polygon(aes(fill=GDP5), color="gray70", size=.5) + guides(fill=FALSE)  
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zscore.R
\name{zscore}
\alias{zscore}
\title{Compute z-scores}
\usage{
zscore(x, na.rm = getOption("na.rm", FALSE))
}
\arguments{
\item{x}{a numeric vector}

\item{na.rm}{a logical indicating whether missing values should be removed}
}
\description{
Compute z-scores
}
\examples{
data(penguins, package = "palmerpenguins")
penguins |> 
  group_by(species) |> 
  mutate(zbill_length_mm = zscore(bill_length_mm, na.rm = TRUE)) |> 
  head()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/favstats.R
\name{fav_stats}
\alias{fav_stats}
\title{Some favorite statistical summaries}
\usage{
fav_stats(x, ..., na.rm = TRUE, type = 7)
}
\arguments{
\item{x}{numeric vector}

\item{...}{additional arguments (currently ignored)}

\item{na.rm}{boolean indicating whether missing data should be ignored}

\item{type}{an integer between 1 and 9 selecting one of the nine quantile algorithms detailed
in the documentation for \code{\link[stats:quantile]{stats::quantile()}}}
}
\value{
A vector of statistical summaries
}
\description{
Likely you mean to be using \code{\link[=favstats]{favstats()}}.  Each of these computes the
mean, standard deviation, quartiles, sample size and number of missing values for a numeric vector,
but \code{\link[=favstats]{favstats()}} can take a formula describing how these summary statistics
should be aggregated across various subsets of the data.
}
\examples{
fav_stats(1:10)
fav_stats(faithful$eruptions)
data(penguins, package = "palmerpenguins")

# Note: this is favstats() rather than fav_stats()
favstats(bill_length_mm ~ species, data = penguins)  
}
\keyword{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CIsim.R
\name{CIsim}
\alias{CIsim}
\title{Compute confidence intervals from (multiple) simulated data sets}
\usage{
CIsim(
  n,
  samples = 100,
  rdist = rnorm,
  args = list(),
  plot = if (samples <= 200) "draw" else "none",
  estimand = 0,
  conf.level = 0.95,
  method = t.test,
  method.args = list(),
  interval = function(x) {
     do.call(method, c(list(x, conf.level = conf.level),
    method.args))$conf.int
 },
  estimate = function(x) {
     do.call(method, c(list(x, conf.level = conf.level),
    method.args))$estimate
 },
  verbose = TRUE
)
}
\arguments{
\item{n}{size of each sample}

\item{samples}{number of samples to simulate}

\item{rdist}{function used to draw random samples}

\item{args}{arguments required by \code{rdist}}

\item{plot}{one of \code{"print"}, \code{"return"}, \code{"horizontal"}, or \code{"none"}
describing whether a plot should be printed, returned, printed with horizontal intervals,
or not generated at all.}

\item{estimand}{true value of the parameter being estimated}

\item{conf.level}{confidence level for intervals}

\item{method}{function used to compute intervals.  Standard functions that
produce an object of class \code{htest} can be used here.}

\item{method.args}{arguments required by \code{method}}

\item{interval}{a function that computes a confidence interval from data.  Function
should return a vector of length 2.}

\item{estimate}{a function that computes an estimate from data}

\item{verbose}{print summary to screen?}
}
\value{
A data frame with variables
\code{lower},
\code{upper},
\code{estimate},
\code{cover} ('Yes' or 'No'),
and
\code{sample}
is returned invisibly.  See the examples for a way to use this to display the intervals
graphically.
}
\description{
This function automates the calculation of coverage rates for exploring
the robustness of confidence interval methods.
}
\examples{
# 1000 95\% intervals using t.test; population is N(0,1)
CIsim(n = 10, samples = 1000)    
# this time population is Exp(1); fewer samples, so we get a plot 
CIsim(n = 10, samples = 100, rdist = rexp, estimand = 1) 
# Binomial treats 1 like success, 0 like failure
CIsim(n = 30, samples = 100, rdist = rbinom, args = list(size = 1, prob = .7), 
       estimand = .7, method = binom.test, method.args = list(ci = "Plus4"))  

}
\keyword{inference}
\keyword{simulation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffmean.R
\name{diffmean}
\alias{diffmean}
\alias{diffprop}
\title{Difference in means and proportions}
\usage{
diffmean(x, ..., data = parent.frame(), only.2 = TRUE)

diffprop(x, ..., data = parent.frame(), only.2 = TRUE)
}
\arguments{
\item{x, data, ...}{as in \code{\link[=mean]{mean()}} or \code{\link[=prop]{prop()}}}

\item{only.2}{a logical indicating whether differences should only be computed
between two groups.}
}
\description{
Wrappers around \code{diff(mean(...))} and \code{diff(prop(...))} that
facilitate better naming of the result
}
\examples{
if (require(mosaicData)) {
diffprop( homeless ~ sex , data=HELPrct)
do(3) * diffprop( homeless ~ shuffle(sex) , data=HELPrct)
diffmean( age ~ substance, data=HELPrct, only.2=FALSE)
do(3) * diffmean(age ~ shuffle(substance), data=HELPrct, only.2=FALSE)
diffmean( age ~ sex, data=HELPrct)
do(3) * diffmean(age ~ shuffle(sex), data=HELPrct)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maps2.R
\name{mUSMap}
\alias{mUSMap}
\title{Make a US map with \code{ggplot2}}
\usage{
mUSMap(
  data = NULL,
  key,
  fill = NULL,
  plot = c("borders", "frame", "none"),
  style = c("compact", "real")
)
}
\arguments{
\item{data}{A dataframe with US states as cases}

\item{key}{The column name in the \code{data} that holds the unique
names of each state}

\item{fill}{A variable in the \code{data} used to specify the fill
color of states in the map (note: if \code{fill} is not null, then
\code{plot} cannot be set to "none")}

\item{plot}{The plot desired for the output. \code{plot} = "none"
returns the merged data that is the result of merging the \code{data}
and the dataframe with the geographical coordinate information;
\code{plot} = "frame" returns an empty (unplottable) ggplot object;
\code{plot} = "border" (the default) returns a ggplot object with
one geom_polygon layer that shows the borders of the states}

\item{style}{The style in which to display the map. \code{compact} gives
a polyconic projection with Alaska and Hawaii on the lower left corner;
\code{real} gives the real size and position of all states without any
projection.}
}
\description{
\code{mUSMap} takes in one dataframe that includes information
about different US states. It merges this dataframe with a dataframe
that includes geographical coordinate information. Depending on the
arguments passed, it returns this data or a ggplot object constructed
with the data.
}
\examples{
USArrests2 <- USArrests |> tibble::rownames_to_column("state")
mUSMap(USArrests2, key="state", fill = "UrbanPop") 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adapt_seq.R
\name{adapt_seq}
\alias{adapt_seq}
\title{Adaptively generate sequences in an interval}
\usage{
adapt_seq(
  from,
  to,
  length.out = 200,
  f = function(x, ...) {
     1
 },
  args = list(),
  quiet = FALSE
)
}
\arguments{
\item{from}{start of interval}

\item{to}{end of interval}

\item{length.out}{desired length of sequence}

\item{f}{a function}

\item{args}{arguments passed to \code{f}}

\item{quiet}{suppress warnings about NaNs, etc.}
}
\value{
a numerical vector
}
\description{
\code{adapt_seq} is similar to \code{seq} except that instead of
selecting points equally spaced along an interval, it selects points
such that the values of a function applied at those points are
(very) roughly equally spaced.  This can be useful for sampling
a function in such a way that it can be plotted more smoothly,
for example.
}
\examples{
adapt_seq(0, pi, 25, sin)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\name{theme.mosaic}
\alias{theme.mosaic}
\alias{col.mosaic}
\title{Lattice Theme}
\usage{
theme.mosaic(bw = FALSE, lty = if (bw) 1:7 else 1, lwd = 2, ...)

col.mosaic(bw = FALSE, lty = if (bw) 1:7 else 1, lwd = 2, ...)
}
\arguments{
\item{bw}{whether color scheme should be "black and white"}

\item{lty}{vector of line type codes}

\item{lwd}{vector of line widths}

\item{...}{additional named arguments passed to
\code{\link[lattice:trellis.par.set]{lattice::trellis.par.set()}}}
}
\value{
Returns a list that can be supplied as the \code{theme} to
\code{\link[lattice:trellis.par.set]{lattice::trellis.par.set()}}.
}
\description{
A theme for use with lattice graphics.
}
\note{
These two functions are identical.  \code{col.mosaic} is named
similarly to \code{\link[lattice:col.whitebg]{lattice::col.whitebg()}}, but since more
than just colors are set, \code{theme.mosaic} is a preferable name.
}
\examples{
trellis.par.set(theme=theme.mosaic())
show.settings()
trellis.par.set(theme=theme.mosaic(bw=TRUE))
show.settings()

}
\seealso{
\code{\link[lattice:trellis.par.set]{lattice::trellis.par.set()}}, \code{\link[lattice:show.settings]{lattice::show.settings()}}
}
\keyword{graphics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregatingMeta.R
\name{MAD_}
\alias{MAD_}
\alias{SAD_}
\title{All pairs mean and sum of absolute differences}
\usage{
MAD_(x, ..., na.rm = getOption("na.omit", FALSE))

SAD_(x, ..., na.rm = getOption("na.omit", FALSE))
}
\arguments{
\item{x}{a numeric vector or a formula.}

\item{...}{additional arguments appended to \code{x}}

\item{na.rm}{a logical indicating whether NAs should be removed before
calculating.}
}
\value{
the mean or sum of the absolute differences between each pair
of values in \code{c(x,...)}.
}
\description{
All pairs mean and sum of absolute differences
}
\seealso{
\code{\link[=mad]{mad()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeo.R
\name{leaflet_map}
\alias{leaflet_map}
\title{Simple Leaflet Maps}
\usage{
leaflet_map(
  latitude = NULL,
  longitude = NULL,
  position = NULL,
  zoom = 12,
  mark = FALSE,
  radius = 0,
  units = c("km", "miles", "meters", "feet"),
  ...
)
}
\arguments{
\item{latitude, longitude}{vectors of latitude and longitude values.
If \code{latitude} is a data frame, then it is treated as \code{position}.
This facilitates "piping" from \code{\link[=rgeo]{rgeo()}}.  See examples.}

\item{position}{a data frame containing latitude and longitude positions}

\item{zoom}{zoom level for initial map (1-20)}

\item{mark}{a logical indicating whether the location should be marked with a pin}

\item{radius}{a vector of radii of circles (in miles) centered at position that are displayed on the map}

\item{units}{units for radii of circles (km, miles, meters, or feet).}

\item{\dots}{additional arguments passed to \code{\link[leaflet:addCircles]{leaflet::addCircles()}}}
}
\value{
a leaflet map
}
\description{
Primarily designed to work with \code{\link[=rgeo]{rgeo()}} to display randomly sampled
points on the globe.
}
\examples{
# the leaflet package is required
if (require(leaflet)) {
  # Times Square
  leaflet_map(40.7566, -73.9863, radius = 1, units = "miles")  
  # 3 random locations; 5 km circles
  leaflet_map(position = rgeo(3), radius = 5, mark = TRUE, color = "red") 
  # using pipes
  rgeo(4, latlim = c(25,50), lonlim = c(-65, -125)) |>
    leaflet_map(radius = 5, mark = TRUE, color = "purple")
}
}
\seealso{
\code{\link[=deg2rad]{deg2rad()}}, \code{\link[=latlon2xyz]{latlon2xyz()}} and \code{\link[=rgeo]{rgeo()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resample.R
\name{rflip}
\alias{rflip}
\alias{print.cointoss}
\alias{nflip}
\title{Tossing Coins}
\usage{
rflip(
  n = 1,
  prob = 0.5,
  quiet = FALSE,
  verbose = !quiet,
  summarize = FALSE,
  summarise = summarize
)

\method{print}{cointoss}(x, ...)

nflip(n = 1, prob = 0.5, ...)
}
\arguments{
\item{n}{the number of coins to toss}

\item{prob}{probability of heads on each toss}

\item{quiet}{a logical.  If \code{TRUE}, less verbose output is used.}

\item{verbose}{a logical.  If \code{TRUE}, more verbose output is used.}

\item{summarize}{if \code{TRUE}, return a summary (as a data frame).}

\item{summarise}{alternative spelling for \code{summarize}.}

\item{x}{an object}

\item{\dots}{additional arguments}
}
\value{
for \code{rflip}, a cointoss object

for \code{nflip}, a numeric vector
}
\description{
These functions simplify simulating coin tosses for those (students primarily)
who are not yet familiar with the binomial distributions or just like this syntax
and verbosity better.
}
\examples{
rflip(10)
rflip(10, prob = 1/6, quiet = TRUE)
rflip(10, prob = 1/6, summarize = TRUE)
do(5) * rflip(10)
as.numeric(rflip(10))
nflip(10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomialCIs.R
\name{update_ci}
\alias{update_ci}
\title{Update confidence interval}
\usage{
update_ci(
  object,
  method = c("clopper-pearson", "wald", "agresti-coull", "plus4", "score", "prop.test")
)
}
\arguments{
\item{object}{An \code{"htest"} object produced by \code{\link[=binom.test]{binom.test()}}}

\item{method}{a method for computing a confidence interval for a proportion.}
}
\value{
an \code{"htest"} object with an updated confidence interval
}
\description{
Update the confidence interval portion of an object returned from
\code{binom.test} using one of several alternative methods.
}
\seealso{
\code{\link[=binom.test]{binom.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor-test.R
\name{cor_test.formula}
\alias{cor_test.formula}
\alias{cor.test}
\alias{cor_test}
\alias{cor_test.default}
\title{Alternative formula interface for cor.test}
\usage{
\method{cor_test}{formula}(formula, ...)

cor.test(x, ...)

cor_test(x, ...)

\method{cor_test}{default}(x, y, ...)
}
\arguments{
\item{formula}{a formula}

\item{...}{other arguments passed to \code{\link[stats:cor.test]{stats::cor.test()}}.}

\item{x, y}{numeric vectors of data values. x and y must have the same length.}
}
\description{
\code{\link[stats:cor.test]{stats::cor.test()}} in \pkg{stats} accepts formulas of the
shape \code{~ y + x}.  The \pkg{mosaic} package allows the use
of \code{y ~ x} as an alternative formula shape.
}
\examples{
# This is an example from example(stats::cor.test) done in old and new style
require(graphics)
cor.test(~ CONT + INTG, data = USJudgeRatings)
cor.test(CONT ~ INTG, data = USJudgeRatings)
}
\seealso{
\code{\link[stats:cor.test]{stats::cor.test()}} in the \pkg{stats} package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfun.R
\name{rfun}
\alias{rfun}
\alias{rpoly2}
\title{Generate a natural-looking function}
\usage{
rfun(vars = ~x & y, seed = NULL, n = 0)

rpoly2(vars = ~x & y, seed = NULL)
}
\arguments{
\item{vars}{a formula; the LHS is empty and the RHS indicates the variables used
for input to the function (separated by &)}

\item{seed}{seed for random number generator, passed to \code{\link[=set.seed]{set.seed()}}.}

\item{n}{the number of Gaussians.  By default, this will be selected randomly.}
}
\value{
a function with the appropriate number of inputs

a function defined by a 2nd degree polynomial
with coefficients selected randomly according to a Unif(-1,1) distribution.
}
\description{
Produce a random function that is the sum of Gaussian random variables

\code{rpoly2} generates a random 2nd degree polynomial  (as a function)
}
\details{
\code{rfun} is an easy way to generate a natural-looking but random function with ups and downs
much as you might draw on paper.  In two variables, it provides a good way to produce
a random landscape that is smooth.
Things happen in the domain -5 to 5. The function is pretty flat outside of that.
Use \code{seed} to create a fixed function that will be the same for everybody

These functions are particularly useful for teaching calculus.
}
\examples{
f <- rfun( ~ u & v)
plotFun(f(u,v)~u&v,u=range(-5,5),v=range(-5,5))
myfun <- rfun(~ u & v, seed=1959)
g <- rpoly2( ~ x&y&z, seed=1964)
plotFun(g(x,y,z=2)~x&y,xlim=range(-5,5),ylim=range(-5,5))
}
\keyword{random}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mplot.R
\name{fortify.summary.lm}
\alias{fortify.summary.lm}
\alias{fortify.summary.glm}
\alias{fortify.TukeyHSD}
\title{Extract data from R objects}
\usage{
\method{fortify}{summary.lm}(model, data = NULL, level = 0.95, ...)

\method{fortify}{summary.glm}(model, data = NULL, level = 0.95, ...)

\method{fortify}{TukeyHSD}(model, data, order = c("asis", "pval", "difference"), ...)
}
\arguments{
\item{model}{an R object}

\item{data}{original data set, if needed}

\item{level}{confidence level}

\item{...}{additional arguments}

\item{order}{one of \code{"pval"}, \code{"diff"}, or \code{"asis"} determining the
order of the \code{pair} factor, which determines the order in which the differences
are displayed on the plot.}
}
\description{
Extract data from R objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdist.R, R/dpqrdist.R
\name{cdist}
\alias{cdist}
\alias{xcgamma}
\alias{xct}
\alias{xcchisq}
\alias{xcf}
\alias{xcbinom}
\alias{xcpois}
\alias{xcgeom}
\alias{xcnbinom}
\alias{xcbeta}
\title{Central portion of a distribution}
\usage{
cdist(
  dist = "norm",
  p,
  plot = TRUE,
  verbose = FALSE,
  invisible = FALSE,
  digits = 3L,
  xlim = NULL,
  ylim = NULL,
  resolution = 500L,
  return = c("values", "plot"),
  pattern = c("rings", "stripes"),
  ...,
  refinements = list()
)

xcgamma(
  p,
  shape,
  rate = 1,
  scale = 1/rate,
  lower.tail = TRUE,
  log.p = FALSE,
  ...
)

xct(p, df, ncp, lower.tail = TRUE, log.p = FALSE, ...)

xcchisq(p, df, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)

xcf(p, df1, df2, lower.tail = TRUE, log.p = FALSE, ...)

xcbinom(p, size, prob, lower.tail = TRUE, log.p = FALSE, ...)

xcpois(p, lambda, lower.tail = TRUE, log.p = FALSE, ...)

xcgeom(p, prob, lower.tail = TRUE, log.p = FALSE, ...)

xcnbinom(p, size, prob, mu, lower.tail = TRUE, log.p = FALSE, ...)

xcbeta(p, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE, ...)
}
\arguments{
\item{dist}{a character string naming a distribution family (e.g., "norm").
This will work for any family for which the usual d/p/q functions exist.}

\item{p}{the proportion to be in the central region, with equal proportions
in either "tail".}

\item{plot}{a logical indicating whether a plot should be created}

\item{verbose}{a logical indicating whether a more verbose output value should be returned.}

\item{invisible}{a logical}

\item{digits}{the number of digits desired}

\item{xlim}{x limits.  By default, these are chosen to show the central 99.8\\%
of the distribution.}

\item{ylim}{y limits}

\item{resolution}{number of points used for detecting discreteness and generating plots.
The default value of 5000 should work well except for discrete distributions
that have many distinct values, especially if these values are not evenly spaced.}

\item{return}{If \code{"plot"}, return a plot.  If \code{"values"}, return a vector of numerical values.}

\item{pattern}{One of \code{"stripes"} or \code{"rings"}.
In the latter case, pairs of regions (from the outside to the inside)
are grouped together for coloring and probability calculation.}

\item{...}{additional arguments passed to the distribution functions.  Typically
these specify the parameters of the particular distribution desired.  See the examples.}

\item{refinements}{A list of refinements to the plot.  See \code{\link[ggformula:gf_refine]{ggformula::gf_refine()}}.}

\item{shape, scale}{shape and scale parameters.  Must be positive,
    \code{scale} strictly.}

\item{rate}{an alternative way to specify the scale.}

\item{lower.tail}{logical; if \code{TRUE} (default), probabilities are
    \eqn{P[X < x]}, otherwise, \eqn{P[X \ge x]} (exceptionally not
    \emph{cadlag} here, to ensure, e.g., \code{pgamma(0, shape=0)} remains zero).}

\item{log.p}{A logical indicating whether probabilities should be
returned on the log scale.}

\item{df}{degrees of freedom (\eqn{> 0}, maybe non-integer).  \code{df
      = Inf} is allowed.}

\item{ncp}{non-centrality parameter \eqn{\delta}{delta};
    currently except for \code{rt()}, accurate only for \code{abs(ncp) <= 37.62}.
    If omitted, use the central \eqn{t} distribution.}

\item{df1, df2}{degrees of freedom.  \code{Inf} is allowed.}

\item{size}{number of trials (zero or more).}

\item{prob}{probability of success on each trial.}

\item{lambda}{vector of (non-negative) means.}

\item{mu}{alternative parametrization via mean: see \sQuote{Details}.}

\item{shape1, shape2}{non-negative parameters of the Beta distribution.}
}
\value{
a pair of numbers indicating the upper and lower bounds, unless \code{verbose} is
\code{TRUE}, in which case a 1-row data frame is returned containing these bounds,
the central probability, the tail probabilities, and the name of the distribution.
}
\description{
This function determines the critical values for isolating
a central portion of a distribution with a specified probability.
This is designed to work especially well for symmetric distributions,
but it can be used with any distribution.
}
\note{
This function is still experimental and changes the input or output
formats are possible in future versions of the package.
}
\examples{
cdist( "norm", .95)
cdist( "t", c(.90, .95, .99), df=5)
cdist( "t", c(.90, .95, .99), df=50)
# plotting doesn't work well when the parameters are not constant
cdist( "t", .95, df=c(3,5,10,20), plot = FALSE)
cdist( "norm", .95, mean=500, sd=100 )
cdist( "chisq", c(.90, .95), df=3 )
# CI
x <- rnorm(23, mean = 10, sd = 2)
cdist("t", p = 0.95, df=22)
mean(x) + cdist("t", p = 0.95, df=22) * sd(x) / sqrt(23)
confint(t.test(x))
cdist("t", p = 0.95, df=22, verbose = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings.R
\name{mosaic.options}
\alias{mosaic.options}
\alias{mosaicGetOption}
\alias{mosaic.par.set}
\alias{mosaic.getOption}
\alias{mosaic.par.get}
\alias{restoreLatticeOptions}
\alias{mosaicLatticeOptions}
\title{Setting options for mosaic package functions}
\usage{
mosaic.options(...)

mosaic.getOption(name)

mosaic.par.set(name, value, ..., theme, warn = TRUE, strict = FALSE)

mosaic.par.get(name = NULL)

restoreLatticeOptions()

mosaicLatticeOptions()
}
\arguments{
\item{\dots}{additional arguments that are turned into a list if a list cannot be inferred from
\code{theme}, \code{name}, and \code{value}.}

\item{name}{the name of the option being set}

\item{value}{the value to which to set the option}

\item{theme}{a list appropriate for a mosaic theme}

\item{warn}{a logical.  UNUSED at present.}

\item{strict}{a logical or numeric.}
}
\description{
A mechanism for setting options in the mosaic package.
}
\details{
\code{restoreLatticeOptions} returns any lattice
options that were changed when the mosaic package was loaded
back to their pre-mosaic state.

\code{mosaicLatticeOptions} sets a number
of defaults for lattice graphics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frequencyPolygon.R
\name{freqpoly}
\alias{freqpoly}
\alias{hist2freqpolygon}
\alias{plot.freqpolygon}
\title{Turn histograms into frequency polygons}
\usage{
freqpoly(x, plot = TRUE, ...)

hist2freqpolygon(hist)

\method{plot}{freqpolygon}(
  x,
  freq = equidist,
  col = graphics::par("fg"),
  lty = NULL,
  lwd = 1,
  main = paste("Frequency polygon of", paste(x$xname, collapse = "\\n")),
  sub = NULL,
  xlab = x$xname,
  ylab,
  xlim = range(x$x),
  ylim = NULL,
  axes = TRUE,
  labels = FALSE,
  add = FALSE,
  ann = TRUE,
  ...
)
}
\arguments{
\item{x}{a vector of values for which a frequency polygon is desired.}

\item{plot}{a logical indicating if a plot should be generated.}

\item{...}{additional arguments passed on to \code{\link[=hist]{hist()}}.}

\item{hist}{a histogram object produced by \code{link{hist}()}.}

\item{freq}{A logical indicating whether the vertical scale should be frequency (count).}

\item{col}{A color for the frequency polygon.}

\item{lty}{An integer indicating the line type.}

\item{lwd}{An integer indicating the line width.}

\item{main}{A title for the plot.}

\item{sub}{A sub-title for the plot.}

\item{xlab}{Label for the horizontal axis.}

\item{ylab}{Label for the vertical axis.}

\item{xlim}{A numeric vector of length 2.}

\item{ylim}{A numeric vector of length 2.}

\item{axes}{A logical indicating whether axes should be drawn.}

\item{labels}{A logical indicating whether labels should be printed or a character
vector of labels to add.}

\item{add}{A logical indicating whether the plot should be added to the current plot}

\item{ann}{A logical indicating whether annotations (titles and axis titles) should be plotted.}
}
\value{
An object of class \code{"freqpoly"} (invisibly).  Additionally, if \code{plot} is
\code{TRUE}, a plot is generated.
}
\description{
Turn histograms into frequency polygons
}
\examples{
freqpoly(faithful$eruptions)
bks <- c(0, 1, 1.5, 2, 3, 3.5, 4, 4.5, 5, 7)
hist(faithful$eruptions, breaks = bks)
freqpoly(faithful$eruptions, col = rgb(0,0,1,.5), lwd = 5, breaks = bks, add = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgeo.R
\name{deg2rad}
\alias{deg2rad}
\alias{rad2deg}
\title{Convert between degrees and radians}
\usage{
deg2rad(x)

rad2deg(x)
}
\arguments{
\item{x}{a numeric vector}
}
\value{
a numeric vector
}
\description{
Facilitates conversion between degrees and radians.
}
\examples{
deg2rad(180)
rad2deg(2*pi)
}
\seealso{
\code{\link[=latlon2xyz]{latlon2xyz()}}, \code{\link[=googleMap]{googleMap()}}, and \code{\link[=rgeo]{rgeo()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xpnorm.R
\name{mid}
\alias{mid}
\title{midpoints along a sequence}
\usage{
mid(x)
}
\arguments{
\item{x}{a numeric vector}
}
\value{
a vector of length 1 less than \code{x}
}
\description{
Compute a vector of midpoints between values in a numeric vector
}
\examples{
mid(1:5)
mid((1:5)^2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factorize.R
\name{factorize}
\alias{factorize}
\alias{factorize.default}
\alias{factorize.numeric}
\alias{factorize.character}
\alias{factorize.data.frame}
\alias{factorise}
\title{Conditionally convert vectors to factors}
\usage{
factorize(x, ...)

\method{factorize}{default}(x, ...)

\method{factorize}{numeric}(x, max.levels = 5L, ...)

\method{factorize}{character}(x, max.levels = 5L, ...)

\method{factorize}{data.frame}(x, max.levels = 5L, ...)

factorise(x, ...)
}
\arguments{
\item{x}{an object}

\item{...}{additional arguments (currently ignored)}

\item{max.levels}{an integer.  Only convert if the number of unique values is no
more than \code{max.levels}.}
}
\description{
A generic function and several instances for creating factors from
other sorts of data.  The primary use case is for vectors that contain
few unique values and might be better considered as factors.  When
applied to a data frame, this is applied to each variable in the data frame.
}
\examples{
data(KidsFeet, package="mosaicData")
str(KidsFeet)
factorize(KidsFeet$birthyear)
str(factorize(KidsFeet))
# alternative spelling
str(factorise(KidsFeet))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project.R
\docType{methods}
\name{project}
\alias{project}
\alias{project,formula-method}
\alias{project,numeric-method}
\alias{project,matrix-method}
\alias{vlength}
\alias{dot}
\title{Projections}
\usage{
project(x, ...)

\S4method{project}{formula}(x, u = NULL, data = parent.frame(2), coefficients = TRUE, ...)

\S4method{project}{numeric}(x, u = rep(1, length(x)), type = c("vector", "length", "coef"), ...)

\S4method{project}{matrix}(x, u, data = parent.frame())

vlength(x, ...)

dot(u, v)
}
\arguments{
\item{x}{a numeric vector (all functions) or a formula (only for \code{project}).
Left-hand sides of formulas should be a single quantity}

\item{...}{additional arguments}

\item{u}{a numeric vector}

\item{data}{a data frame.}

\item{coefficients}{For \code{project(y ~ x)} indicates whether the projection
coefficents should be returned or the projection vector.}

\item{type}{one of \code{"length"} or \code{"vector"} determining the type of the
returned value}

\item{v}{a numeric vector}
}
\value{
\code{project} returns the projection of \code{x} onto \code{u}
(or its length if \code{u} and \code{v} are numeric vectors and \code{type == "length"})

\code{vlength} returns the length of the vector
(i.e., the square root of the sum of the squares of the components)

\code{dot} returns the dot product of \code{u} and \code{v}
}
\description{
Compute projections onto the span of a vector or a model space, dot products, and
vector lengths in Euclidean space.
}
\details{
\code{project} (preferably pronounced "pro-JECT" as in "projection")
does either of two related things:
(1) Given two vectors as arguments, it will project the first onto the
second, returning the point in the subspace of the second that is as
close as possible to the first vector.  (2) Given a formula as an argument,
will work very much like \code{lm()}, constructing a model matrix from
the right-hand side of the formula and projecting the vector on the
left-hand side onto the subspace of that model matrix.

In (2), rather than
returning the projected vector, \code{project()} returns the coefficients
on each of the vectors in the model matrix.
UNLIKE \code{lm()}, the intercept vector is NOT included by default.  If
you want an intercept vector, include \code{+1} in your formula.
}
\examples{
x1 <- c(1,0,0); x2 <- c(1,2,3); y1 <- c(3,4,5); y2 <- rnorm(3)
# projection onto the 1 vector gives the mean vector
mean(y2)            
project(y2, 1)
# return the length of the vector, rather than the vector itself
project(y2, 1, type='length')
project(y1 ~ x1 + x2) -> pr; pr
# recover the projected vector 
cbind(x1,x2) \%*\% pr -> v; v
project( y1 ~ x1 + x2, coefficients=FALSE )
dot( y1 - v, v ) # left over should be orthogonal to projection, so this should be ~ 0
if (require(mosaicData)) {
project(width~length+sex, data=KidsFeet)
}
vlength(rep(1,4))
if (require(mosaicData)) {
m <- lm( length ~ width, data=KidsFeet )
# These should be the same
vlength( m$effects )  
vlength( KidsFeet$length)
# So should these
vlength( tail(m$effects, -2) )
sqrt(sum(resid(m)^2))
}
v <- c(1,1,1); w <- c(1,2,3)
u <- v / vlength(v)  # make a unit vector
# The following should be the same:
project(w,v, type="coef") * v 
project(w,v)
# The following are equivalent
abs(dot( w, u ))
vlength( project( w, u) )
vlength( project( w, v) )
project( w, v, type='length' )
}
\seealso{
\code{link{project}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swap.R
\name{swap}
\alias{swap}
\title{Swap values among columns of a data frame}
\usage{
swap(data, which)
}
\arguments{
\item{data}{a data frame}

\item{which}{a formula or an integer or character vector specifying columns in
\code{data}}
}
\description{
Swap values among columns of a data frame
}
\details{
\code{swap} is not a particularly speedy function.  It is intended primarily
as an aid for teaching randomization for paired designs.  Used this way, the number of
randomizations should be kept modest (approximately 1000) unless you are very patient.
}
\examples{
if (require(tidyr)) {
  Sleep2 <- sleep |> spread( key=group, val=extra )
  names(Sleep2) <- c("subject", "drug1", "drug2")
  swap(Sleep2, drug1 ~ drug2)
  mean( ~(drug1 - drug2), data=Sleep2)
  do(3) * mean( ~(drug1 - drug2), data=Sleep2 |> swap(drug1 ~ drug2) ) 
} 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/derivedFactor.R
\name{derivedVariable}
\alias{derivedVariable}
\alias{derivedFactor}
\title{Create new variables from logicals}
\usage{
derivedVariable(
  ...,
  .ordered = FALSE,
  .method = c("unique", "first", "last"),
  .debug = c("default", "always", "never"),
  .sort = c("given", "alpha"),
  .default = NULL,
  .asFactor = FALSE
)

derivedFactor(..., .asFactor = TRUE)
}
\arguments{
\item{\dots}{named logical "rules" defining the levels.}

\item{.ordered}{a logical indicating whether the resulting factored should be ordered
Ignored if \code{.asFactor} is \code{FALSE}.}

\item{.method}{one of \code{"unique"}, \code{"first"}, and \code{"last"}.
If \code{"unique"}, exactly one rule must be \code{TRUE} for each position.
If \code{"first"}, the first \code{TRUE} rule defines the level.
If \code{"last"}, the last \code{TRUE} rule defines the level.}

\item{.debug}{one of \code{"default"}, \code{"always"}, and \code{"never"}, indicating
whether debugging information should be printed.  If \code{"default"}, debugging
information is printed only when multiple rules give conflicting definitions
for some positions.}

\item{.sort}{One of \code{"given"} (the default) or \code{"alpha"} or
a vector of integers the same length as the number of levels indicating the
order in which the levels should appear in the resulting factor.
Ignored if \code{.asFactor} is \code{FALSE}.}

\item{.default}{character vector of length 1 giving name of default level or
\code{NULL} for no default.}

\item{.asFactor}{A logical indicating whether the returned value should be a factor.}
}
\description{
Utility functions for creating new variables from logicals describing the levels
}
\details{
Each logical "rule" corresponds to a level in the resulting variable.
If \code{.default} is defined, an implicit rule is added that is \code{TRUE}
whenever all other rules are \code{FALSE}.
When there are multiple \code{TRUE} rules for a slot, the first or last such is used
or an error is generated, depending on the value of \code{method}.

\code{derivedVariable} is designed to be used with \code{\link[=transform]{transform()}} or
\code{\link[dplyr:mutate]{dplyr::mutate()}} to add new
variables to a data frame.  \code{derivedFactor}() is the same but that the
default value for \code{.asFactor} is \code{TRUE}.  See the examples.
}
\examples{
Kf <- mutate(KidsFeet, biggerfoot2 = derivedFactor(
                   dom = biggerfoot == domhand,
                   nondom = biggerfoot != domhand)
                   )
tally( ~ biggerfoot + biggerfoot2, data = Kf)
tally( ~ biggerfoot + domhand, data = Kf)

# Three equivalent ways to define a new variable
# Method 1: explicitly define all levels
modHELP <- mutate(HELPrct, drink_status = derivedFactor( 
  abstinent = i1 == 0,
  moderate = (i1>0 & i1<=1 & i2<=3 & sex=='female') |
     (i1>0 & i1<=2 & i2<=4 & sex=='male'),
  highrisk = ((i1>1 | i2>3) & sex=='female') | 
      ((i1>2 | i2>4) & sex=='male'),
  .ordered = TRUE)
)
tally( ~ drink_status, data = modHELP)

# Method 2: Use .default for last level
modHELP <- mutate(HELPrct, drink_status = derivedFactor( 
  abstinent = i1 == 0,
  moderate = (i1<=1 & i2<=3 & sex=='female') |
     (i1<=2 & i2<=4 & sex=='male'),
  .ordered = TRUE,
  .method = "first",
  .default = "highrisk")
)
tally( ~ drink_status, data = modHELP)

# Method 3: use TRUE to catch any fall through slots
modHELP <- mutate(HELPrct, drink_status = derivedFactor( 
  abstinent = i1 == 0,
  moderate = (i1<=1 & i2<=3 & sex=='female') |
     (i1<=2 & i2<=4 & sex=='male'),
  highrisk=TRUE,
  .ordered = TRUE,
  .method = "first"
  )
)
tally( ~ drink_status, data = modHELP)
is.factor(modHELP$drink_status)

modHELP <- mutate(HELPrct, drink_status = derivedVariable( 
  abstinent = i1 == 0,
  moderate = (i1<=1 & i2<=3 & sex=='female') |
     (i1<=2 & i2<=4 & sex=='male'),
  highrisk=TRUE,
  .ordered = TRUE,
  .method = "first"
  )
)
is.factor(modHELP$drink_status)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pqrdata.R
\name{qdata_v}
\alias{qdata_v}
\alias{qdata_f}
\alias{cdata_v}
\alias{cdata_f}
\alias{pdata_v}
\alias{pdata_f}
\alias{rdata_v}
\alias{rdata_f}
\alias{ddata_v}
\alias{ddata_f}
\title{The Data Distribution}
\usage{
qdata_v(x, p = seq(0, 1, 0.25), na.rm = TRUE, ...)

qdata_f(x, ..., data = NULL, groups = NULL, na.rm = TRUE)

cdata_v(x, p = 0.95, na.rm = TRUE, ...)

cdata_f(x, ..., data = NULL, groups = NULL, na.rm = TRUE)

pdata_v(x, q, lower.tail = TRUE, ...)

pdata_f(x, ..., data = NULL, groups = NULL, na.rm = TRUE)

rdata_v(x, n, replace = TRUE, ...)

rdata_f(x, ..., data = NULL, groups = NULL, na.rm = TRUE)

ddata_v(x, q, ..., data = NULL, log = FALSE, na.rm = TRUE)

ddata_f(x, ..., data = NULL, groups = NULL, na.rm = TRUE)
}
\arguments{
\item{x}{a vector containing the data}

\item{p}{a vector of probabilities}

\item{na.rm}{a logical indicating whether \code{NA}s should be removed before computing.}

\item{\dots}{additional arguments passed to \code{quantile} or \code{sample}}

\item{data}{a data frame in which to evaluate \code{formula}}

\item{groups}{a grouping variable, typically the name of a variable in \code{data}}

\item{q}{a vector of quantiles}

\item{lower.tail}{a logical indicating whether to use the lower or upper tail probability}

\item{n}{number of values to sample}

\item{replace}{a logical indicating whether to sample with replacement}

\item{log}{a logical indicating whether the result should be log transformed}
}
\description{
Utility functions for density, distribution function, quantile function,
and random generation from data.
}
\seealso{
\code{\link[=ddata]{ddata()}}, \code{\link[=pdata]{pdata()}}, \code{\link[=qdata]{qdata()}},
\code{\link[=rdata]{rdata()}}, \code{\link[=cdata]{cdata()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surround.R
\name{surround}
\alias{surround}
\title{Format strings for pretty output}
\usage{
surround(x, pre = " ", post = " ", width = 8, ...)
}
\arguments{
\item{x}{a vector}

\item{pre}{text to prepend onto string}

\item{post}{text to postpend onto string}

\item{width}{desired width of string}

\item{\dots}{additional arguments passed to \code{\link[=format]{format()}}}
}
\value{
a vector of strings padded to the desired width
}
\description{
Format strings for pretty output
}
\examples{
surround(rbinom(10,20,.5), " ", " ", width=4)
surround(rnorm(10), " ", " ", width=8, digits = 2, nsmall = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docFile.R
\name{docFile}
\alias{docFile}
\title{Return the path to a documentation file in a package}
\usage{
docFile(file, package = "mosaic", character.only = FALSE)
}
\arguments{
\item{file}{the name of a file}

\item{package}{the name of a package}

\item{character.only}{a logical. If \code{TRUE} package names must be
specified as character, else names will be converted as a convenience
as is \code{\link[=library]{library()}} and \code{\link[=library]{library()}}.}
}
\value{
a character vector specifying the path to the file on the user's system.
}
\description{
Return the path to a documentation file in a package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotModel.R
\name{plotModel}
\alias{plotModel}
\alias{plotModel.default}
\alias{plotModel.parsedModel}
\title{Plot a regression model}
\usage{
plotModel(mod, ...)

\method{plotModel}{default}(mod, ...)

\method{plotModel}{parsedModel}(
  mod,
  formula = NULL,
  ...,
  auto.key = NULL,
  drop = TRUE,
  max.levels = 9L,
  system = c("ggplot2", "lattice")
)
}
\arguments{
\item{mod}{A model of type \code{\link[=lm]{lm()}} or \code{\link[=glm]{glm()}}}

\item{\dots}{arguments passed to \code{\link[lattice:xyplot]{lattice::xyplot()}} or \code{rgl::plot3d}.}

\item{formula}{a formula indicating how the variables are to be displayed.  In the style of
\code{lattice} and \code{ggformula}.}

\item{auto.key}{If TRUE, automatically generate a key.}

\item{drop}{If TRUE, unused factor levels are dropped from \code{interaction()}.}

\item{max.levels}{currently unused}

\item{system}{which of \code{ggplot2} or \code{lattice} to use for plotting}
}
\value{
A lattice or ggplot2 graphics object.
}
\description{
Visualize a regression model amid the data that generated it.
}
\details{
The goal of this function is to assist with visualization
of statistical models. Namely, to plot the model on top of the data
from which the model was fit.

The primary plot type is a scatter plot.  The x-axis can be assigned
to one of the predictors in the model.  Additional predictors are thought
of as co-variates.  The data and fitted curves are partitioned by
these covariates.  When the number of components to this partition is large,
a random subset of the fitted curves is displayed to avoid visual clutter.

If the model was fit on one quantitative variable (e.g. SLR), then
a scatter plot is drawn, and the model is realized as parallel or
non-parallel lines, depending on whether interaction terms are present.

Eventually we hope to support 3-d visualizations of models with 2 quantitative
predictors using the \code{rgl} package.

Currently, only linear regression models and
generalized linear regression models are supported.
}
\section{Caution}{

This is still underdevelopment.  The API is subject to change, and some
use cases may not work yet.  Watch for improvements in subsequent versions of the package.
}

\examples{

require(mosaic)

mod <- lm( mpg ~ factor(cyl), data = mtcars)
plotModel(mod)

# SLR
mod <- lm( mpg ~ wt, data = mtcars)
plotModel(mod, pch = 19)

# parallel slopes
mod <- lm( mpg ~ wt + factor(cyl), data=mtcars)
plotModel(mod)

\dontrun{ 
# multiple categorical vars
mod <- lm( mpg ~ wt + factor(cyl) + factor(vs) + factor(am), data = mtcars)
plotModel(mod)
plotModel(mod, mpg ~ am)

# interaction
mod <- lm( mpg ~ wt + factor(cyl) + wt:factor(cyl), data = mtcars)
plotModel(mod)

# polynomial terms
mod <- lm( mpg ~ wt + I(wt^2), data = mtcars)
plotModel(mod)

# GLM
mod <- glm(vs ~ wt, data=mtcars, family = 'binomial')
plotModel(mod)

# GLM with interaction
mod <- glm(vs ~ wt + factor(cyl), data=mtcars, family = 'binomial')
plotModel(mod)
# 3D model
mod <- lm( mpg ~ wt + hp, data = mtcars)
plotModel(mod)

# parallel planes
mod <- lm( mpg ~ wt + hp + factor(cyl) + factor(vs), data = mtcars)
plotModel(mod)

# interaction planes
mod <- lm( mpg ~ wt + hp + wt * factor(cyl), data = mtcars)
plotModel(mod)
plotModel(mod, system="g") + facet_wrap( ~ cyl )
}
}
\seealso{
\code{\link[=plotPoints]{plotPoints()}}, \code{\link[=plotFun]{plotFun()}}
}
\author{
Ben Baumer, Galen Long, Randall Pruim
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mPlots.R
\name{design_plot}
\alias{design_plot}
\title{Interactively design plots}
\usage{
design_plot(
  data,
  format,
  default = format,
  system = system_choices()[1],
  show = FALSE,
  title = "",
  data_text = rlang::expr_deparse(substitute(data)),
  ...
)
}
\arguments{
\item{data}{a data frame containing the variables that might be used in the plot.
Note that for maps, the data frame must contain coordinates of the polygons
comprising the map and a variable for determining which coordinates are part
of the same region.  See \code{\link[=sp2df]{sp2df()}} for one way to create such
a data frame.  Typically \code{\link[=merge]{merge()}} will be used to combine the map
data with some auxiliary data to be displayed as fill color on the map, although
this is not necessary if all one wants is a map.}

\item{format}{a synonym for \code{default}.}

\item{default}{default type of plot to create; one of
\code{"scatter"},
\code{"jitter"},
\code{"boxplot"},
\code{"violin"},
\code{"sina"},
\code{"histogram"},
\code{"density"},
\code{"density (contours)"},
\code{"density (filled)"},
\code{"frequency polygon"},
\code{"xyplot"},
or
\code{"map"}.  Unique prefixes suffice.}

\item{system}{which graphics system to use (initially) for plotting (\pkg{ggplot2}
or \pkg{lattice}).  A check box will allow on the fly change of plotting system.}

\item{show}{a logical, if \code{TRUE}, the code will be displayed each time the plot is
changed.}

\item{title}{a title for the plot}

\item{data_text}{A text string describing the data.  It must be possible to recover the data
from this string using \code{\link[=eval]{eval()}}.  Typically users will not need to
modify this from the default value.}

\item{...}{additional arguments}
}
\value{
Nothing.  Used for side effects.
}
\description{
Proves a simple interface to let users interactively design plots in \pkg{ggformula}, \pkg{lattice}, or \pkg{ggplot2}.
An option is available to show the code used to create the plot.
This can be copied and pasted elsewhere to (into an RMarkdown document, for example) to recreate the plot.
Only works in RStudio. Requires the \pkg{manipulate} package.
}
\details{
Currently maps are only supported in \pkg{ggplot2} and not in \pkg{lattice}.

Due to an unresolved issue with RStudio, the first time this function is called, and additional
plot is created to correctily initialize the mainipulate frameowrk.
}
\examples{
\dontrun{
  mtcars2 <- 
    mtcars |> 
      mutate(
        cyl2 = factor(cyl), 
        carb2 = factor(carb),
        shape = c("V-shaped", "straight")[1 + vs], 
        gear2 = factor(gear), 
        transmission = c("automatic", "manual")[1 + am])
  design_plot(mtcars2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mosaic-package.R, R/reexports.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{makeFun}
\alias{fitdistr}
\alias{fractions}
\alias{lhs}
\alias{rhs}
\alias{condition}
\alias{counts}
\alias{props}
\alias{prop}
\alias{prop1}
\alias{perc}
\alias{percs}
\alias{count}
\alias{tally}
\alias{dfapply}
\alias{ediff}
\alias{inspect}
\alias{msummary}
\alias{n_missing}
\alias{logit}
\alias{ilogit}
\alias{compare}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{MASS}{\code{\link[MASS:fitdistr]{fitdistr()}}, \code{\link[MASS:fractions]{fractions()}}}

  \item{mosaicCore}{\code{\link[mosaicCore:compare]{compare()}}, \code{\link[mosaicCore:condition]{condition()}}, \code{\link[mosaicCore:count]{count()}}, \code{\link[mosaicCore:counts]{counts()}}, \code{\link[mosaicCore:counts]{counts()}}, \code{\link[mosaicCore:dfapply]{dfapply()}}, \code{\link[mosaicCore:ediff]{ediff()}}, \code{\link[mosaicCore:ilogit]{ilogit()}}, \code{\link[mosaicCore:inspect]{inspect()}}, \code{\link[mosaicCore:lhs]{lhs()}}, \code{\link[mosaicCore:logit]{logit()}}, \code{\link[mosaicCore:makeFun]{makeFun()}}, \code{\link[mosaicCore:makeFun]{makeFun()}}, \code{\link[mosaicCore:msummary]{msummary()}}, \code{\link[mosaicCore:n_missing]{n_missing()}}, \code{\link[mosaicCore:perc]{perc()}}, \code{\link[mosaicCore:percs]{percs()}}, \code{\link[mosaicCore:prop]{prop()}}, \code{\link[mosaicCore:prop1]{prop1()}}, \code{\link[mosaicCore:props]{props()}}, \code{\link[mosaicCore:rhs]{rhs()}}, \code{\link[mosaicCore:tally]{tally()}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mplot.R
\name{mplot}
\alias{mplot}
\alias{mplot.default}
\alias{mplot.lm}
\alias{mplot.data.frame}
\alias{mplot.summary.lm}
\alias{mplot.TukeyHSD}
\title{Generic plotting}
\usage{
mplot(object, ...)

\method{mplot}{default}(object, ...)

\method{mplot}{lm}(
  object,
  which = c(1:3, 7),
  system = c("ggplot2", "lattice", "base"),
  ask = FALSE,
  multiplot = "package:gridExtra" \%in\% search(),
  par.settings = theme.mosaic(),
  level = 0.95,
  title = paste("model: ", deparse(object$call), "\\n"),
  rows = TRUE,
  id.n = 3L,
  id.size = 5,
  id.color = "red",
  id.nudge = 1,
  add.smooth = TRUE,
  smooth.color = "red",
  smooth.alpha = 0.6,
  smooth.size = 0.7,
  span = 3/4,
  ...
)

\method{mplot}{data.frame}(
  object,
  format,
  default = format,
  system = c("ggformula", "ggplot2", "lattice"),
  show = FALSE,
  data_text = rlang::expr_deparse(substitute(object)),
  title = "",
  ...
)

\method{mplot}{summary.lm}(
  object,
  system = c("ggplot2", "lattice"),
  level = 0.95,
  par.settings = trellis.par.get(),
  rows = TRUE,
  ...
)

\method{mplot}{TukeyHSD}(
  object,
  system = c("ggplot2", "lattice"),
  ylab = "",
  xlab = "difference in means",
  title = paste0(attr(object, "conf.level") * 100, "\% family-wise confidence level"),
  par.settings = trellis.par.get(),
  order = c("asis", "pval", "difference"),
  ...
)
}
\arguments{
\item{object}{an R object from which a plot will be constructed.}

\item{...}{additional arguments.  If \code{object} is an \code{lm}, subsets
of these arguments are passed to \code{gridExtra::grid.arrange} and to the
\pkg{lattice} plotting routines; in particular,
\code{nrow} and \code{ncol} can be used to control the number of rows
and columns used.}

\item{which}{a numeric vector used to select from 7 potential plots}

\item{system}{which graphics system to use (initially) for plotting (\pkg{ggplot2}
or \pkg{lattice}).  A check box will allow on the fly change of plotting system.}

\item{ask}{if TRUE, each plot will be displayed separately after the user
responds to a prompt.}

\item{multiplot}{if TRUE and \code{ask == FALSE}, all plots will be
displayed together.}

\item{par.settings}{\pkg{lattice} theme settings}

\item{level}{a confidence level}

\item{title}{title for plot}

\item{rows}{rows to show.  This may be a numeric vector,
\code{TRUE} (for all rows), or a character vector of row names.}

\item{id.n}{Number of id labels to display.}

\item{id.size}{Size of id labels.}

\item{id.color}{Color of id labels.}

\item{id.nudge}{a numeric used to increase (>1) or decrease (<1) the amount that observation labels are
nudged.  Use a negative value to nudge down instead of up.}

\item{add.smooth}{A logicial indicating whether a LOESS smooth should be added
(where this makes sense to do).
Currently ignored for lattice plots.}

\item{smooth.color, smooth.size, smooth.alpha}{Color, size, and alpha used for
LOESS curve.  Currently ignored for lattice plots.}

\item{span}{A positive number indicating the amount of smoothing.
A larger number indicates more smoothing. See \code{\link[stats:loess]{stats::loess()}} for details.
Currently ignored for lattice plots.}

\item{format, default}{default type of plot to create; one of
\code{"scatter"},
\code{"jitter"},
\code{"boxplot"},
\code{"violin"},
\code{"histogram"},
\code{"density"},
\code{"frequency polygon"},
or
\code{"map"}.
Unique prefixes suffice.}

\item{show}{a logical, if \code{TRUE}, the code will be displayed each time the plot is
changed.}

\item{data_text}{text representation of the data set.  In typical use cases, the default value should suffice.}

\item{ylab}{label for y-axis}

\item{xlab}{label for x-axis}

\item{order}{one of \code{"pval"}, \code{"diff"}, or \code{"asis"} determining the
order of the \code{pair} factor, which determines the order in which the differences
are displayed on the plot.}

\item{data}{a data frame containing the variables that might be used in the plot.}
}
\value{
Nothing.  Just for side effects.
}
\description{
Generic function plotting for R objects.  Currently plots exist for
\code{data.frame}s, \code{lm}s, (including \code{glm}s).
}
\details{
The method for models (lm and glm) is still a work in progress, but should be usable for
relatively simple models.  When the results for a logistic regression model created with
\code{\link[=glm]{glm()}} are satisfactory will depend on the format and structure of the data
used to fit the model.

Due to a bug in RStudio 1.3, the method for data frames may not display the controls
consistently.  We have found that executing this code usually fixes the problem:

\if{html}{\out{<div class="sourceCode">}}\preformatted{library(manipulate) 
manipulate(plot(A), A = slider(1, 10))
}\if{html}{\out{</div>}}
}
\examples{
lm( width ~ length * sex, data = KidsFeet) |>
  mplot(which = 1:3, id.n = 5)
lm( width ~ length * sex, data = KidsFeet) |>
  mplot(smooth.color = "blue", smooth.size = 1.2, smooth.alpha = 0.3, id.size = 3)
lm(width ~ length * sex, data = KidsFeet) |>
  mplot(rows = 2:3, which = 7)
\dontrun{
mplot( HELPrct )
mplot( HELPrct, "histogram" )
}
lm(width ~ length * sex, data = KidsFeet) |>
  summary() |>
  mplot()
  
lm(width ~ length * sex, data = KidsFeet) |>
  summary() |>
  mplot(rows = c("sex", "length"))
  
lm(width ~ length * sex, data = KidsFeet) |>
  summary() |>
  mplot(rows = TRUE)
lm(age ~ substance, data = HELPrct) |>
  TukeyHSD() |>
  mplot()
lm(age ~ substance, data = HELPrct) |>
  TukeyHSD() |>
  mplot(system = "lattice")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bargraph.R
\name{bargraph}
\alias{bargraph}
\title{Create bar graphs from raw data}
\usage{
bargraph(
  x,
  data = parent.frame(),
  groups = NULL,
  horizontal = FALSE,
  origin = 0,
  ylab = ifelse(horizontal, "", type),
  xlab = ifelse(horizontal, type, ""),
  type = c("count", "frequency", "proportion", "percent"),
  auto.key = TRUE,
  scales = list(),
  ...
)
}
\arguments{
\item{x}{a formula describing the plot}

\item{data}{a data frame in which the formula \code{x} is evaluated}

\item{groups}{a variable or expression used for grouping.  See \code{\link[lattice:barchart]{lattice::barchart()}}.}

\item{horizontal}{a logical indicating whether bars should be horizontal}

\item{origin}{beginning point for bars.  For the default behavior used by
\code{\link[lattice:barchart]{lattice::barchart()}} set \code{origin} to \code{NULL}, but
0 is often a better default. If 0 is not good, perhaps you should use
a different kind of plot as the results may be misleading.}

\item{ylab}{a character vector of length one used for the y-axis label}

\item{xlab}{a character vector of length one used for the x-axis label}

\item{type}{one of \code{"frequency"}, \code{"count"}, \code{"percent"}, or \code{"proportion"}
indicating what type of scale to use.  Unique prefixes are sufficient.}

\item{auto.key}{a logical expression indicating whether a legend should be automatically produced}

\item{scales}{is a list determining how the x- and y-axes are drawn}

\item{\dots}{additional arguments passed to \code{\link[lattice:barchart]{lattice::barchart()}}}
}
\value{
a trellis object describing the plot
}
\description{
\code{\link[lattice:barchart]{lattice::barchart()}} from the \code{lattice} package makes bar graphs from
pre-tabulated data.  Raw data can be tabulated using \code{\link[=xtabs]{xtabs()}}, but the syntax
is unusual compared to the other lattice plotting functions.  \code{bargraph} provides
an interface that is consistent with the other \code{lattice} functions.
}
\details{
\code{bargraph(formula, data=data, ...)} works by creating a new data frame from \code{xtabs(formula, data=data)}
and then calling \code{\link[lattice:barchart]{lattice::barchart()}} using modified version of the formula and this
new data frame as inputs.  This has implications on, for example, conditional plots where
one desires to condition on some expression that will be evaluated in \code{data}.  This typically
does not work because the required variables do not exist in the output of \code{xtabs}.  One solution
is to first add a new variable to \code{data} first and then to condition using this new variable.
See the examples.
}
\examples{
if (require(mosaicData)) {
data(HELPrct)
bargraph( ~ substance, data = HELPrct)
bargraph( ~ substance, data = HELPrct, horizontal = TRUE)
bargraph( ~ substance | sex, groups = homeless, auto.key = TRUE, data = HELPrct)
bargraph( ~ substance, groups = homeless, auto.key=TRUE, 
            data = HELPrct |> filter(sex == "male"))
HELPrct2 <- mutate(HELPrct, older = age > 40)
bargraph( ~ substance | older, data = HELPrct2)
}
}
\seealso{
\code{\link[lattice:barchart]{lattice::barchart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findZerosMult.R
\name{Broyden}
\alias{Broyden}
\title{Multi-Dimensional Root Finding}
\usage{
Broyden(system, vars, x = 0, tol = .Machine$double.eps^0.4, maxiters = 10000)
}
\arguments{
\item{system}{A list of functions}

\item{vars}{A character string list of variables that appear in the functions}

\item{x}{A starting vector}

\item{tol}{The tolerance for the function specifying how precise it will be}

\item{maxiters}{maximum number of iterations.}
}
\description{
Implementation of Broyden's root finding function to numerically compute the root of
a system of nonlinear equations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisq.R
\name{chisq}
\alias{chisq}
\alias{chisq.htest}
\alias{chisq.table}
\alias{chisq.default}
\title{Extract Chi-squared statistic}
\usage{
chisq(x, ...)

\method{chisq}{htest}(x, ...)

\method{chisq}{table}(x, correct = FALSE, ...)

\method{chisq}{default}(x, correct = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{"htest"} a coming from a Chi-squared test,
an object of class \code{"table"}, or
the inputs to \code{\link[=tally]{tally()}}.}

\item{...}{additional arguments passed on to \code{tally} or \code{chisq.test}.}

\item{correct}{a logical indicating whether a continuity correction should be
applied.}
}
\description{
Extract Chi-squared statistic
}
\examples{

if(require(mosaicData)) {
  Mites.table <- tally( ~ outcome + treatment, data=Mites )
  Mites.table 
  chisq.test(Mites.table)
  chisq(Mites.table)
  chisq(chisq.test(Mites.table))
  ## Randomization test.  Increase replications to decrease Monte Carlo error.
  do(3) * chisq( tally( ~ outcome + shuffle(treatment),  data=Mites ) )
  Mites.rand <- do(1000) * chisq( tally( ~ outcome + shuffle(treatment),  data=Mites ) )
  tally( ~(X.squared >= chisq(Mites.table)), data=Mites.rand, format="proportion")
}
  
}
\seealso{
\code{\link[ggplot2:after_stat]{ggplot2::after_stat()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitSpline.R
\name{fitSpline}
\alias{fitSpline}
\title{Fit splines to data}
\usage{
fitSpline(
  formula,
  data = parent.frame(),
  df = NULL,
  knots = NULL,
  degree = 3,
  type = c("natural", "linear", "cubic", "polynomial"),
  ...
)
}
\arguments{
\item{formula}{a formula.  Only one quantity is allowed on the left-hand side, the
output quantity}

\item{data}{a data frame in which \code{formula} is evaluated.}

\item{df}{degrees of freedom (used to determine how many knots should be used)}

\item{knots}{a vector of knots}

\item{degree}{parameter for splines when \code{type} is \code{"polynomial"}.
1 is locally linear, 2 is locally quadratic, etc.}

\item{type}{type of splines to use; one of
\code{"linear"}, \code{"cubic"}, \code{"natural"} (cubic with linear tails, the default),
or \code{"polynomial"}.}

\item{\dots}{additional arguments passed to spline basis functions
(\code{\link[splines:ns]{splines::ns()}} and \code{\link[splines:bs]{splines::bs()}}).}
}
\value{
a function of the explanatory variable
}
\description{
These functions create mathematical functions from data, using splines.
}
\examples{
f <- fitSpline( weight ~ height, data=women, df=5 )
xyplot( weight ~ height, data=women )
plotFun(f(height) ~ height, add=TRUE)

g <- fitSpline( length ~ width, data = KidsFeet, type='natural', df=5 )
h <- fitSpline( length ~ width, data = KidsFeet, type='linear', df=5 )
xyplot( length ~ width, data = KidsFeet, col='gray70', pch=16)
plotFun(g, add=TRUE, col='navy')
plotFun(h, add=TRUE, col='red')
}
\seealso{
\code{\link[splines:bs]{splines::bs()}}  and \code{\link[splines:ns]{splines::ns()}} for the bases used to generate the splines.
}
